
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!========================================================================
! This subroutine receive data from all slaves of a coupler processor and
! resort them from 'per slaves' to 'per data' representation.
! As a result we have 4D, 3D, 2D stripes (il, jl/run_np(COUPLER)) 
! which we can write to files or exchange for mapping procedure.
!========================================================================
	
	module c_gd_module
	
	use shared_module
	use shared_const_module
	use c_module
	use shared_utils_module
	
	implicit none

	PRIVATE
	PUBLIC :: stripe_4D, stripe_3D, stripe_2D, stripe_2D_kind8, c_pack_kind4, c_pack_kind8
	PUBLIC :: c_gd_ini, c_distribute_comp, c_gather_comp
		
	! Work arrays for distribution procedures
	real (kind=4), allocatable :: c_pack_kind4(:,:)
	real (kind=8), allocatable :: c_pack_kind8(:,:)
	
	real (kind=4), allocatable :: stripe_4D(:,:)
	real (kind=4), allocatable :: stripe_3D(:,:)
	real (kind=4), allocatable :: stripe_2D(:,:)
	real (kind=8), allocatable :: stripe_2D_kind8(:,:)
	
	integer, dimension(:), allocatable :: i_s_coord, j_s_coord, il_belt, jl_belt, length_2D
	integer, allocatable :: c_status_distr(:,:)
	integer, allocatable :: c_request_gather(:), c_request_distr(:)
	
	
	! Work variables
	integer comp
	integer s, i, j, k, m, func, slave, sr_counter, counter, a
	integer :: comp_il, comp_jl, comp_kl, comp_ml
	

	CONTAINS 

!====================================================================================================================================================
	subroutine c_gd_ini

	integer :: max_slaves
	integer stripe_4D_size, stripe_3D_size, stripe_2D_size
	integer :: max_2D, max_3D, max_4D, max_i_loc, max_j_loc
	integer pack_kind4_size, pack_kind8_size, max_pack4, max_pack8
	integer max_rw_4D, max_rw_3D, max_rw_2D, max_rw_2D_8
	
	max_slaves = 0
	do comp = 1, NUM_OF_COMP
		if(max_slaves < c_inf(comp) % wk(me) % slave_count) then
			max_slaves = c_inf(comp) % wk(me) % slave_count
		end if
	end do
	
	max_2D = 0
	max_3D = 0
	max_4D = 0
	max_pack4 = 0
	max_pack8 = 0
	max_rw_2D = 0
	max_rw_3D = 0
	max_rw_4D = 0
	max_rw_2D_8 = 0
	
	do comp = 1, NUM_OF_COMP
			
!		if(c_inf(comp) % proc_count == 0) CYCLE
		
		max_i_loc = MAXVAL(c_inf(comp) % wk(me) % sl(:) % il_belt)
		max_j_loc = MAXVAL(c_inf(comp) % wk(me) % sl(:) % jl_belt)

		stripe_2D_size = MAXVAL(c_inf(comp) % wk(:) % stripe_count)*c_inf(comp) % il		
		if(max_2D < stripe_2D_size) max_2D = stripe_2D_size

	  	stripe_3D_size = stripe_2D_size*c_inf(comp) % kl
		if(max_3D < stripe_3D_size) max_3D = stripe_3D_size

	  	stripe_4D_size = stripe_3D_size*c_inf(comp) % ml
		if(max_4D < stripe_4D_size) max_4D = stripe_4D_size
			
		do a = 1, ACTION_NUM_FIELDS
			if(max_rw_4D < c_inf(comp) % rw(a, ARRAY_TYPE_4D) % ar_size) max_rw_4D = c_inf(comp) % rw(a, ARRAY_TYPE_4D) % ar_size
			if(max_rw_3D < c_inf(comp) % rw(a, ARRAY_TYPE_3D) % ar_size) max_rw_3D = c_inf(comp) % rw(a, ARRAY_TYPE_3D) % ar_size
			if(max_rw_2D < c_inf(comp) % rw(a, ARRAY_TYPE_2D) % ar_size) max_rw_2D = c_inf(comp) % rw(a, ARRAY_TYPE_2D) % ar_size 
			if(max_rw_2D_8 < c_inf(comp) % rw(a, ARRAY_TYPE_2D_KIND8) % ar_size) max_rw_2D_8 = c_inf(comp) % rw(a, ARRAY_TYPE_2D_KIND8) % ar_size 
		end do
	
		pack_kind4_size = MAX(	MAX(max_rw_4D*max_i_loc*max_j_loc*c_inf(comp) % kl*c_inf(comp) % ml, max_rw_3D*max_i_loc*max_j_loc*c_inf(comp) % kl, max_rw_2D*max_i_loc*max_j_loc),	&
					MAXVAL(c_map_info(comp, :, 1, MAP_NUM_FUNC))*max_i_loc*max_j_loc,					&
					MAXVAL(c_map_info(:, comp, 1, MAP_NUM_FUNC))*max_i_loc*max_j_loc)
		if(max_pack4 < pack_kind4_size) max_pack4 = pack_kind4_size
		
		pack_kind8_size = max_rw_2D_8*max_i_loc*max_j_loc
		if(max_pack8 < pack_kind8_size) max_pack8 = pack_kind8_size
	end do
	
	allocate(c_pack_kind4(max_pack4, max_slaves)); c_pack_kind4 = 0
	allocate(c_pack_kind8(max_pack8, max_slaves)); c_pack_kind8 = 0
	allocate (stripe_4D (max_4D, max_rw_4D)); stripe_4D = 0
	allocate (stripe_3D (max_3D, max_rw_3D)); stripe_3D = 0
	allocate (stripe_2D (max_2D, MAX(max_rw_2D, MAXVAL(c_map_info(:,:,1,MAP_NUM_FUNC))))); stripe_2D = 0
	allocate (stripe_2D_kind8 (max_2D, max_rw_2D_8)); stripe_2D_kind8 = 0

	allocate(i_s_coord(max_slaves))
	allocate(j_s_coord(max_slaves))
	allocate(il_belt(max_slaves))
	allocate(jl_belt(max_slaves))
	allocate(length_2D(max_slaves))
	
	allocate(c_status_distr(MPI_STATUS_SIZE, max_slaves))
	allocate(c_request_gather(max_slaves))
	allocate(c_request_distr(max_slaves))	
	
	end subroutine c_gd_ini
	
!====================================================================================================================================================
	
	subroutine c_gd_comp_ini(current_comp)
	
	integer :: current_comp
	integer :: sum_stripes
	
	! Get short names of current event and component
	comp_il = c_inf(current_comp) % il
	comp_jl = c_inf(current_comp) % jl
	comp_kl = c_inf(current_comp) % kl
	comp_ml = c_inf(current_comp) % ml

	! Calculate coordinate of slave in stripe
	do s = 1, c_inf(current_comp) % wk(me) % slave_count
		il_belt(s) = c_inf(current_comp) % wk(me) % sl(s) % il_belt
		jl_belt(s) = c_inf(current_comp) % wk(me) % sl(s) % jl_belt
		j_s_coord(s) = c_inf(current_comp) % wk(me) % sl(s) % south - c_inf(current_comp) % wk(me) % stripe_start + 1 
		i_s_coord(s) = c_inf(current_comp) % wk(me) % sl(s) % west
		length_2D(s) = comp_il*c_inf(current_comp) % wk(me) % stripe_count
	end do
	
	end subroutine c_gd_comp_ini

!====================================================================================================================================================

	subroutine c_gather_comp
	
	integer :: st(MPI_STATUS_SIZE)
	
	! Initialization for current event
	c_request_gather = MPI_REQUEST_NULL	
	comp = c_events(c_ev_num) % src
	call c_gd_comp_ini(comp)
	
!=============================================================================
! Coupler workers initiate all 'IRECV' calls, receive one per step and resort 
! it from 'per slave' to 'per data' repsresentation to fill 'stripes'.
! For 4D, 3D, 2D data use 'c_pack_kind4' buffer, for 2D_KIND8 - 'c_pack_kind8'
!=============================================================================

	call c_gd_recv_pack4(ARRAY_TYPE_4D, c_events(c_ev_num) % num_4D, c_pack_kind4, stripe_4D)
	call c_gd_recv_pack4(ARRAY_TYPE_3D, c_events(c_ev_num) % num_3D, c_pack_kind4, stripe_3D)
	call c_gd_recv_pack4(ARRAY_TYPE_2D, c_events(c_ev_num) % num_2D, c_pack_kind4, stripe_2D)

	call c_gd_recv_pack8(ARRAY_TYPE_2D, c_events(c_ev_num) % num_2D_kind8, c_pack_kind8, stripe_2D_kind8)

	CONTAINS
	
	subroutine c_gd_recv_pack4(t, numf, packT, stripeT)
			
		real(kind=4) :: packT(:,:), stripeT(:,:)
		integer :: mpiType = MPI_REAL
#define RECV
#include "c_gd_template.f90"
	end subroutine c_gd_recv_pack4

	subroutine c_gd_recv_pack8(t, numf, packT, stripeT)
			
		real(kind=8) :: packT(:,:), stripeT(:,:)
		integer :: mpiType = MPI_DOUBLE_PRECISION
#define RECV
#include "c_gd_template.f90"
	end subroutine c_gd_recv_pack8
		
	end subroutine c_gather_comp
!====================================================================================================================================================
!==============================================================
! This subroutine distribute data from coupler processor to his 
! slaves. It takes 4D, 3D, 2D stripes (il, jl/run_np(COUPLER)), 
! divide them into parts and sends.
!
! Uses external variables:
!       -c_events(c_ev_num) % dst, comp
!	-c_events(c_ev_num) % num_4D, c_events(c_ev_num) % num_3D, c_comp_num_2D
!==============================================================

	subroutine c_distribute_comp
	
	! Initialization for current event
	c_request_distr = MPI_REQUEST_NULL		
	comp = c_events(c_ev_num) % dst
	call c_gd_comp_ini(comp)

!=============================================================================
! Coupler workers fill send buffers with 'stripe' data, resorting it from 'per
! data' to 'per slave' representation. Then send and wait 4D, 3D, 2D data
! from 'c_pack_kind4' and 2D_KIND8 data from 'c_pack_kind8'
!=============================================================================

	call c_gd_send_pack4(ARRAY_TYPE_4D, c_events(c_ev_num) % num_4D, packT = c_pack_kind4, stripeT = stripe_4D)
	call c_gd_send_pack4(ARRAY_TYPE_3D, c_events(c_ev_num) % num_3D, packT = c_pack_kind4, stripeT = stripe_3D)
	call c_gd_send_pack4(ARRAY_TYPE_2D, c_events(c_ev_num) % num_2D, packT = c_pack_kind4, stripeT = stripe_2D)

	call c_gd_send_pack8(ARRAY_TYPE_2D, c_events(c_ev_num) % num_2D_kind8, packT = c_pack_kind8, stripeT = stripe_2D_kind8)
			
		
	CONTAINS
		
	subroutine c_gd_send_pack4(t, numf, packT, stripeT)
			
		real(kind=4) :: packT(:,:), stripeT(:,:)
		integer :: mpiType = MPI_REAL
#define SEND
#include "c_gd_template.f90"
	end subroutine c_gd_send_pack4

	subroutine c_gd_send_pack8(t, numf, packT, stripeT)
			
		real(kind=8) :: packT(:,:), stripeT(:,:)
		integer :: mpiType = MPI_DOUBLE_PRECISION
#define SEND
#include "c_gd_template.f90"
	end subroutine c_gd_send_pack8
	
	end subroutine c_distribute_comp

!====================================================================================================================================================
	
	end module c_gd_module


