
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!=======================================================
! Initiate dest-type interpolation. 
! Send unsorted and repeated sets of partial sum points. 
! Then check if others want something from me.
!=======================================================

	subroutine c_map_ini_dst_type(src_comp, dst_comp)

	use shared_module
	use c_module
	use c_map_module

	implicit none

	integer src_comp, dst_comp

	integer, allocatable :: src_part(:), sum_owner(:)

	integer w
	integer links_num, num_couplers, num_func, num_cells_send, num_cells_recv

	integer c_status_map_ini(MPI_STATUS_SIZE)
	integer i, counter_cell, counter_sum_num, cur_cell, c_counter
	integer no_cells, mes_size
	integer rank_send, rank_recv

	integer counter

!=================================================================================
! Make queue for sending data, because when message size if rather big(20000 elem)
! realization of MPI can't make SEND without corresponding RECV
!=================================================================================
	do c_counter = 1, run_np(COUPLER)

	if(rank_local == c_counter-1) then

!========================================================
! I send my needs to others if I need something from them
!========================================================
		do w = 1, run_np(COUPLER)

		if(rank_local /= w-1) then

			if(c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS) /= 0) then

				! Set current size and allocate working arrays
				links_num = c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS)
				allocate(src_part(links_num))
				allocate(sum_owner(links_num))

				! Create arrays for numbers of members in every partial sum
				i = 1
				counter_cell = 0
				do while(i <= links_num)

					counter_cell = counter_cell + 1
					cur_cell = c_map_data(src_comp,dst_comp) % dyn_arr(w) % dst(i)
					sum_owner(counter_cell) = cur_cell

					counter_sum_num = 0
					do while(c_map_data(src_comp,dst_comp) % dyn_arr(w) % dst(i) == cur_cell)
						i = i + 1
						counter_sum_num = counter_sum_num + 1
						if(i > links_num) EXIT
					end do

					src_part(counter_cell) = counter_sum_num
	
				end do

				! Allocate and fill 'd_type_num_sums' node in 'c_map_data' structure
				c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS_PART_DST) = counter_cell

				allocate(c_map_data(src_comp,dst_comp) % dyn_arr(w) % d_type_sum_owner(counter_cell))
				c_map_data(src_comp,dst_comp) % dyn_arr(w) % d_type_sum_owner(:) = sum_owner(1:counter_cell)


				! Send cells(all required 'src and wts' and 'd_type_num_sums' array(for decoding 'src') to 'w' neighbour
				call MPI_SEND(	c_map_data(src_comp,dst_comp) % dyn_arr(w) % src,		& 
						links_num,							&
						MPI_INTEGER, 							&
						w-1, 								&
						MAP_DST_TYPE,							&
						comm_local, ierr)

				call MPI_SEND(	c_map_data(src_comp,dst_comp) % dyn_arr(w) % wts,		& 
						links_num,							&
						MPI_DOUBLE_PRECISION, 						&
						w-1, 								&
						MAP_DST_TYPE,							&
						comm_local, ierr)

				call MPI_SEND(	src_part(1:counter_cell),					& 
						counter_cell,							&
						MPI_INTEGER, 							&
						w-1, 								&
						MAP_DST_TYPE,							&
						comm_local, ierr)

				! Free memory before next step
				deallocate(src_part, sum_owner)

			else
				! If I need no points from 'w' neighbour, send control message
				call MPI_SEND(	no_cells,							& 
						1,								&
						MPI_INTEGER, 							&
						w-1, 								&
						MAP_NULL,							&
						comm_local, ierr)
			end if

		end if
		end do

	else

!=========================================
! I check if others need something from me
!=========================================
		call MPI_PROBE(c_counter-1, MPI_ANY_TAG, comm_local, c_status_map_ini, ierr)
		call MPI_GET_COUNT(c_status_map_ini, MPI_INTEGER, mes_size, ierr)

		select case(c_status_map_ini(MPI_TAG))

			case (MAP_DST_TYPE)
				allocate(c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_all(mes_size))
				      
				call MPI_RECV(  c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_all,	& 
						mes_size,									&
						MPI_INTEGER, 									&
						c_counter-1, 									&
						MAP_DST_TYPE,									&
						comm_local, c_status_map_ini, ierr)

				allocate(c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_wts(mes_size))
				      
				call MPI_RECV(  c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_wts,	& 
						mes_size,									&
						MPI_DOUBLE_PRECISION, 								&
						c_counter-1, 									&
						MAP_DST_TYPE,									&
						comm_local, c_status_map_ini, ierr)

				call MPI_PROBE(c_counter-1, MPI_ANY_TAG, comm_local, c_status_map_ini, ierr)
				call MPI_GET_COUNT(c_status_map_ini, MPI_INTEGER, mes_size, ierr)
				c_map_info(src_comp, dst_comp, c_counter, MAP_NUM_LINKS_FOR_SEND_PART_DST) = mes_size
				allocate(c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_num_sums(mes_size))

				call MPI_RECV(  c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % d_type_for_send_num_sums, & 
						mes_size,									&
						MPI_INTEGER, 									&
						c_counter-1, 									&
						MAP_DST_TYPE,									&
						comm_local, c_status_map_ini, ierr)
			case (MAP_NULL)
				call MPI_RECV( 	no_cells,									& 
						mes_size,									&
						MPI_INTEGER, 									&
						c_counter-1, 									&
						MAP_NULL,									&
						comm_local, c_status_map_ini, ierr)
		end select
	end if

	end do

	return
	end
