
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!=====================================================
! Initiate source-type decomposition.
! Sort arrays and ask to send me only unique entires. 
! Then check if others want some cells from me.
!=====================================================

	subroutine c_map_ini_src_type(src_comp, dst_comp)

	use shared_module
	use shared_utils_module	
	use c_module
	use c_map_module

	implicit none

	integer src_comp, dst_comp

	integer, allocatable :: src_copy(:), s_type_un(:), temp_sort(:)

	integer links_num, num_couplers, num_func, num_cells_send, num_cells_recv

	integer c_status_map_ini(MPI_STATUS_SIZE)
	integer i, counter_un, w, c_counter
	integer no_cells, mes_size
	integer rank_send, rank_recv
	
	integer counter

!=================================================================================
! Make queue for sending data, because when message size if rather big(20000 elem)
! realization of MPI can't make SEND without corresponding RECV
!=================================================================================
	do c_counter = 1, run_np(COUPLER)

	if(rank_local == c_counter-1) then

!========================================================
! I send my needs to others if I need something from them
!========================================================
		do w = 1, run_np(COUPLER)

		if(rank_local /= w-1) then

			if(c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS) /= 0) then

				! Set current size and allocate working arrays
				links_num = c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS)
				allocate(src_copy(links_num))	
				allocate(s_type_un(links_num))

				! Sort working vector	      
				src_copy(:) = c_map_data(src_comp, dst_comp) % dyn_arr(w) % src(:)
				allocate(temp_sort((links_num+1)/2))
				call SortMerge(src_copy, links_num, temp_sort)

				! Delete repeated entires and create new unique-entires arrays
				i = 1
				counter_un = 0
				do while(i <= links_num)

					counter_un = counter_un + 1
					s_type_un(counter_un) = src_copy(i)

					do while(s_type_un(counter_un) == src_copy(i))
						i = i + 1
						if(i > links_num) EXIT
					end do

				end do

				! Allocate and fill 's_type_un' node in 'c_map_data' structure
				c_map_info(src_comp, dst_comp, w, MAP_NUM_LINKS_UN_SRC) = counter_un
				allocate(c_map_data(src_comp,dst_comp) % dyn_arr(w) % s_type_addr_in_un(links_num))

				! Save cell adresses of 's_type_un' array for 'src' array
				do i = 1, links_num
					c_map_data(src_comp,dst_comp) % dyn_arr(w) % s_type_addr_in_un(i) = binarySearch_I(s_type_un(1:counter_un), c_map_data(src_comp, dst_comp) % dyn_arr(w) % src(i))
				end do

				! Send my request to neighbours
				call MPI_SEND(	s_type_un(1:counter_un),								& 
						counter_un,										&
						MPI_INTEGER, 										&
						w-1, 											&
						MAP_SRC_TYPE,										&
						comm_local, ierr)
				! Free memory before next step
				deallocate(src_copy, s_type_un, temp_sort)

			else

				! If I need no points from 'w' neighbour, send control message
				call MPI_SEND(	no_cells,										& 
						1,											&
						MPI_INTEGER, 										&
						w-1, 											&
						MAP_NULL,										&
						comm_local, ierr)
			end if
		end if
		end do

	else

!=========================================
! I check if others need something from me
!=========================================
		call MPI_PROBE(c_counter-1, MPI_ANY_TAG, comm_local, c_status_map_ini, ierr)
		call MPI_GET_COUNT(c_status_map_ini, MPI_INTEGER, mes_size, ierr)

		select case(c_status_map_ini(MPI_TAG))

			case (MAP_SRC_TYPE)
				c_map_info(src_comp, dst_comp, c_counter, MAP_NUM_LINKS_FOR_SEND_UN_SRC) = mes_size
				allocate(c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % s_type_for_send_un(mes_size))

				call MPI_RECV(  c_map_data(src_comp, dst_comp) % dyn_arr(c_counter) % s_type_for_send_un(1:mes_size),& 
						mes_size,										&
						MPI_INTEGER, 										&
						c_counter-1, 										&
						MAP_SRC_TYPE,										&
						comm_local, c_status_map_ini, ierr)
			case (MAP_NULL)
				call MPI_RECV( 	no_cells,										& 
						mes_size,										&
						MPI_INTEGER, 										&
						c_counter-1, 										&
						MAP_NULL,										&
						comm_local, c_status_map_ini, ierr)
		end select
	end if

	end do

	return
	end
