
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------



!==================================
! Perform source type interpolation
!==================================

	subroutine c_map_src_type

	use shared_module
	use c_module
	use c_map_module
	
	implicit none

	integer i, j, w, func, n, counter, sr_counter
	integer c_status_map(MPI_STATUS_SIZE)
	integer :: c_status_map_prev_sends(run_np(COUPLER), MPI_STATUS_SIZE)
	integer src_stripe_global_start, dst_stripe_global_start
	integer num_func, num_cells_send, num_cells_recv, comp_src, comp_dst

	! Calcualte main parameters
	comp_src = c_events(c_ev_num) % src
	comp_dst = c_events(c_ev_num) % dst	
	src_stripe_global_start = (c_inf(comp_src) % wk(me) % stripe_start-1)*c_inf(comp_src) % il + 1
	dst_stripe_global_start = (c_inf(comp_dst) % wk(me) % stripe_start-1)*c_inf(comp_dst) % il + 1
	num_func = c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)

	! Waitall previous send
	call MPI_WAITALL(size_local, c_map_send_request(comp_src, comp_dst,:), c_status_map_prev_sends, ierr)
	! Startall RECV(sdelat ewe ranwe??)
	call MPI_STARTALL(size_local, c_map_recv_request(comp_src, comp_dst,:), ierr)

	! Null 'c_map_dst_array'
	c_map_dst_array = 0.

!=============================================
! Send unique points to neighbours who need it
!=============================================
	do w = 1, size_local

		num_cells_send = c_map_info(comp_src, comp_dst, w, MAP_NUM_LINKS_FOR_SEND_UN_SRC)

		if(num_cells_send /= 0) then
		
			sr_counter = 1
			do func = 1, num_func
			do n = 1, num_cells_send
				c_map_send_buf(sr_counter, w) = stripe_2D(c_map_data(comp_src, comp_dst) % dyn_arr(w) % s_type_for_send_un(n) - src_stripe_global_start + 1, func)
				sr_counter = sr_counter + 1
			end do
			end do

			call MPI_STARTALL(1, c_map_send_request(comp_src, comp_dst,w), ierr)
		end if
	end do

!============================
! Mapping cycle for my points
!============================
	do func = 1, num_func
	do n = 1, c_map_info(comp_src, comp_dst, rank_local+1, MAP_NUM_LINKS)
		c_map_dst_array(c_map_data(comp_src, comp_dst) % dyn_arr(rank_local+1) % dst(n) - dst_stripe_global_start + 1, func) = 				&
		c_map_dst_array(c_map_data(comp_src, comp_dst) % dyn_arr(rank_local+1) % dst(n) - dst_stripe_global_start + 1, func) + 				&
		c_map_data(comp_src, comp_dst) % dyn_arr(rank_local+1) % wts(n)*											&
		stripe_2D(c_map_data(comp_src, comp_dst) % dyn_arr(rank_local+1) % src(n) - src_stripe_global_start + 1, func)
	end do
	end do

!==========================================
! Receive data from them, who has my points
!==========================================
	do w = 1, size_local

		! Waitall 1 per step
		call MPI_WAITALL(1, c_map_recv_request(comp_src, comp_dst,w), c_status_map, ierr)

		num_cells_recv = c_map_info(comp_src, comp_dst, w, MAP_NUM_LINKS_UN_SRC)

		if(num_cells_recv /= 0) then

			! Mapping cycle for received points
			sr_counter = 1
			do func = 1, num_func
			do n = 1, c_map_info(comp_src, comp_dst, w, MAP_NUM_LINKS)
				c_map_dst_array(c_map_data(comp_src, comp_dst) % dyn_arr(w) % dst(n) - dst_stripe_global_start + 1, func) = 		&
				c_map_dst_array(c_map_data(comp_src, comp_dst) % dyn_arr(w) % dst(n) - dst_stripe_global_start + 1, func) + 		&
				c_map_data(comp_src, comp_dst) % dyn_arr(w) % wts(n)*									&
				c_map_recv_buf((func-1)*num_cells_recv + c_map_data(comp_src, comp_dst) % dyn_arr(w) % s_type_addr_in_un(n), w)
			end do
			end do

		end if
	end do

!======================================
! Copy data to output array 'stripe_2D'
!======================================
	do func = 1, num_func
	do i = 1, c_inf(comp_dst) % il*c_inf(comp_dst) % wk(me) % stripe_count
		stripe_2D(i,func) = c_map_dst_array(i, func)
	end do
	end do

	return
	end 
