
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------
!==========================================================
! RW module provide all work with file system by netCDF4
!
! Public interface:
!	-c_rw_ini
!	-c_rw_put_data
!	-c_rw_get_data
!	-c_rw_close_files
!===========================================================

	module c_rw_module
	
	use shared_module
	use shared_utils_module
	use c_module
	use c_gd_module
	use netcdf

#include "../../../../coupling/analysis_flag"

	implicit none

	PRIVATE
	! NetCDF ids
	integer nc_id
	integer :: nc_id_fd(NUM_OF_COMP, BUFFER_LEN) = MYNULL
	integer :: nc_id_dg(NUM_OF_COMP) = MYNULL
	integer :: nc_id_it(NUM_OF_COMP) = MYNULL
	integer :: c_rw_it_counts(NUM_OF_COMP) = 0
	integer :: nc_recnum_fd(NUM_OF_COMP, BUFFER_LEN) = 1
	integer :: c_rw_varids(NUM_OF_COMP, ACTION_NUM_FIELDS, ARRAY_TYPE_NUM_FIELDS, BUFFER_LEN) = MYNULL
	integer :: c_rw_infoids(NUM_OF_COMP, ACTION_NUM_FIELDS, VAR_INFO_NUM_FIELDS) = MYNULL	
	integer c_rw_run_id
	integer nc_err
	
	! Names of dimensions and its ids
	character (len = *), parameter :: RW_I_NAME = "i"
	character (len = *), parameter :: RW_J_NAME = "j"
	character (len = *), parameter :: RW_K_NAME = "k"
	character (len = *), parameter :: RW_M_NAME = "m"
	character (len = *), parameter :: RW_T_NAME = "rec"
	character (len = *), parameter :: RW_L_NAME = "info_length"
	integer :: k_dimid, i_dimid, j_dimid, m_dimid, t_dimid, l_dimid
	
	! Offsets and counts for 4D, 3D, 2D data
	integer :: c_rw_start_4D(NUM_OF_COMP, 4), c_rw_count_4D(NUM_OF_COMP, 4)
	integer :: c_rw_recnum(NUM_OF_COMP, ACTION_NUM_FIELDS) = 1
		
	! Work varibles
	integer comp, act, n, t
	character(LEN=NAME_LEN) :: file_name	
	logical file_exists
	
	! Public interface	
	PUBLIC :: c_rw_ini
	PUBLIC :: c_rw_put_data
	PUBLIC :: c_rw_put_intg
	PUBLIC :: c_rw_get_data
	PUBLIC :: c_rw_close_files
	PUBLIC :: nc_rc
	
	contains

!==================================================================================================================================================================================

!=======================================================
! Initializations for time-actions - SAVE_DG and READ_FD
!=======================================================

	subroutine c_rw_ini

	! Work varibles
	character(len = 10) postfix
			
	! Cycle per all components and actions
	do comp = 1, NUM_OF_COMP
	do act = 1, ACTION_NUM_FIELDS
				
		! Define local start position and local count
		c_rw_start_4D(comp, :) = (/ 1, c_inf(comp) % wk(me) % stripe_start, 1, 1 /)	
		c_rw_count_4D(comp, :) = (/ c_inf(comp) % il, c_inf(comp) % wk(me) % stripe_count, c_inf(comp) % kl, c_inf(comp) % ml /)		
			
		select case(act)

		!==================
		case(ACTION_READ_FD)
		
			if(c_inf(comp) % read_fd == 0) CYCLE
			
			do n = 1, c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar_size
				nc_err = NF90_OPEN(TRIM(c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar(n) % file), IOR(NF90_NOWRITE,IOR(NF90_NETCDF4, NF90_MPIIO)),nc_id_fd(comp, n),comm = comm_local,info = MPI_INFO_NULL)
				if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant open file: '//c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % file)
		
				nc_err = NF90_INQ_VARID(nc_id_fd(comp, n), c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % name, c_rw_varids(comp, act, ARRAY_TYPE_2D, n))
				if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant find variable <'//TRIM(c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % name)//'> in file : '//TRIM(c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar(n) % file)//'.')
			end do

			! Define start record
			do n = 1, c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar_size
				nc_recnum_fd(comp,n) = time_start_min/c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % period + 1
			end do
			
		!==================
		case(ACTION_SAVE_DG)

			if(c_inf(comp) % save_dg == 0) CYCLE

			! Make file name and create or open file
			file_name = TRIM(run_solution_folder)//TRIM(c_inf(comp) % name)//'_'//TRIM(I2S(c_inf(comp) % il))//'x'//TRIM(I2S(c_inf(comp) % jl))//	&
			'x'//TRIM(I2S(c_inf(comp) % kl))//"_"//run_experiment//"_DG.nc"

			INQUIRE(file = file_name, exist = file_exists)
			if(file_exists .AND. time_append_dg) then
				! Open file and inquire vars
				! Parallel opening is required to explicitly select collective operation of DG writing (needed on Cray)
				call nc_rc(NF90_OPEN_PAR(TRIM(file_name), IOR(NF90_WRITE, IOR(NF90_NETCDF4, NF90_MPIIO)), ncid = nc_id_dg(comp), comm = comm_local,info = MPI_INFO_NULL))
				
				do n = 1, c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar_size
					nc_err = NF90_INQ_VARID(nc_id_dg(comp), c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % name, c_rw_varids(comp, act, ARRAY_TYPE_2D, n))
					if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant find variable <'//TRIM(c_inf(comp) % rw(act,ARRAY_TYPE_2D) % ar(n) % name)//'> in file : ' &
					//TRIM(c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar(n) % file)//'.')
				end do
				
				do n = 1, c_inf(comp) % rw(act, ARRAY_TYPE_3D) % ar_size
					nc_err = NF90_INQ_VARID(nc_id_dg(comp), c_inf(comp) % rw(act,ARRAY_TYPE_3D) % ar(n) % name, c_rw_varids(comp, act, ARRAY_TYPE_3D, n))
					if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant find variable <'//TRIM(c_inf(comp) % rw(act,ARRAY_TYPE_3D) % ar(n) % name)//'> in file : ' &
					//TRIM(c_inf(comp) % rw(act, ARRAY_TYPE_3D) % ar(n) % file)//'.')
				end do
				
				! Inquire current record number
				call nc_rc( NF90_INQ_DIMID(nc_id_dg(comp), RW_T_NAME, t_dimid))
				call nc_rc( NF90_INQUIRE_DIMENSION(nc_id_dg(comp), t_dimid, len = c_rw_recnum(comp, act)))
				c_rw_recnum(comp, act) = c_rw_recnum(comp, act) + 1
			
				! Inquire info variables
				call nc_rc( NF90_INQ_VARID(nc_id_dg(comp), "real_time", c_rw_infoids(comp, act, VAR_INFO_REAL_TIME)))
				call nc_rc( NF90_INQ_VARID(nc_id_dg(comp), "model_time", c_rw_infoids(comp, act, VAR_INFO_MODEL_TIME)))
				
			else
			
				call nc_rc(NF90_CREATE_PAR(path = TRIM(file_name), cmode = IOR(NF90_CLOBBER, IOR(NF90_NETCDF4, NF90_MPIIO)), ncid = nc_id_dg(comp), comm = comm_local, info = MPI_INFO_NULL))
					
				! Put global attrbutes
				call nc_rc(NF90_PUT_ATT (nc_id_dg(comp), NF90_GLOBAL, "title", TRIM(c_inf(comp) % name)//" diagnostic fields"))
				call nc_rc(NF90_PUT_ATT (nc_id_dg(comp), NF90_GLOBAL, "experiment", run_experiment))
!				call nc_rc(NF90_PUT_ATT (nc_id_dg(comp), NF90_GLOBAL, "missing_value", MISSING_VALUE))
!------->>>
!	add new parameters. used for masking region in plots (ncview, PYTHON). 
!-------
				call nc_rc(NF90_PUT_ATT (nc_id_dg(comp), NF90_GLOBAL,"_FillValue"	, missval))
				call nc_rc(NF90_PUT_ATT (nc_id_dg(comp), NF90_GLOBAL, "missing_value"	, missval))
!-------<<<

					
				! Define the dimensions
				call nc_rc( NF90_DEF_DIM(nc_id_dg(comp), RW_T_NAME, NF90_UNLIMITED, t_dimid) )
				call nc_rc( NF90_DEF_DIM(nc_id_dg(comp), RW_K_NAME, c_inf(comp) % kl, k_dimid) )				
				call nc_rc( NF90_DEF_DIM(nc_id_dg(comp), RW_I_NAME, c_inf(comp) % il, i_dimid) )
				call nc_rc( NF90_DEF_DIM(nc_id_dg(comp), RW_J_NAME, c_inf(comp) % jl, j_dimid) )
				call nc_rc( NF90_DEF_DIM(nc_id_dg(comp), RW_L_NAME, NAME_LEN, l_dimid) )

				! Define of various dims - 3D, 2D
				call c_rw_def_vars(comp, act, nc_id_dg(comp), ARRAY_TYPE_3D, NF90_REAL, (/ i_dimid, j_dimid, k_dimid, t_dimid /))
				call c_rw_def_vars(comp, act, nc_id_dg(comp), ARRAY_TYPE_2D, NF90_REAL, (/ i_dimid, j_dimid, t_dimid /))

				! Define info variables
				call nc_rc( NF90_DEF_VAR(nc_id_dg(comp), "real_time",  NF90_CHAR, (/ l_dimid, t_dimid /), c_rw_infoids(comp, act, VAR_INFO_REAL_TIME)))
				call nc_rc( NF90_DEF_VAR(nc_id_dg(comp), "model_time", NF90_CHAR, (/ l_dimid, t_dimid /), c_rw_infoids(comp, act, VAR_INFO_MODEL_TIME)))

				! End define mode
				call nc_rc( NF90_ENDDEF(nc_id_dg(comp)))
			end if	
			
		!==================
		case(ACTION_SAVE_IT)
			
			if(rank_world /= 0 .OR. c_inf(comp) % save_it == 0) CYCLE
			
			! Make file name and create or open file
			file_name = TRIM(run_solution_folder)//TRIM(c_inf(comp) % name)//'_'//TRIM(I2S(c_inf(comp) % il))//'x'//TRIM(I2S(c_inf(comp) % jl))//	&
			'x'//TRIM(I2S(c_inf(comp) % kl))//"_"//run_experiment//"_IT.nc"
			
			! If we start new experiment, overwrite previous IT-file, in other case continue it
			INQUIRE(file = file_name, exist = file_exists)
			if(time_start_min == 0 .OR. .NOT. file_exists) then 
				call nc_rc(NF90_CREATE(TRIM(file_name), IOR(NF90_NETCDF4,NF90_CLOBBER), nc_id_it(comp)))
				! Put global attrbutes
				call nc_rc(NF90_PUT_ATT (nc_id_it(comp), NF90_GLOBAL, "title", TRIM(c_inf(comp) % name)//" diagnostic integrals"))
				call nc_rc(NF90_PUT_ATT (nc_id_it(comp), NF90_GLOBAL, "period", TRIM(I2S(c_inf(comp) % save_it))//" min") )
				call nc_rc(NF90_PUT_ATT (nc_id_it(comp), NF90_GLOBAL, "experiment", run_experiment))
			else
				call nc_rc(NF90_OPEN(TRIM(file_name), NF90_WRITE, nc_id_it(comp)))
				call nc_rc(NF90_REDEF(nc_id_it(comp)))
			end if
		
		end select	
	end do
	end do
	
	end subroutine c_rw_ini	
!==================================================================================================================================================================================
	
!==================================================================
! Procedure of putting data to opened DG-file and creation of new
! file for every CP-data and corresponding RUN_DATA-file
!==================================================================
	
	subroutine c_rw_put_data
		
	! Work variables
	character(LEN=NAME_LEN) cur_time, cur_date, time_str, model_time
	real(kind=8) wt_start, wt_4D, wt_3D, wt_2D
	integer nc_id !,time_varid

	act = c_events(c_ev_num) % act
	comp = c_events(c_ev_num) % src

	! Compute real time
	call DATE_AND_TIME(cur_date, cur_time)
	time_str = ''
	time_str = cur_date(1:4)//":"//cur_date(5:6)//":"//cur_date(7:8)//" - "//cur_time(1:2)//":"//cur_time(3:4)//":"//cur_time(5:6)
		
	! Compute model time
	call shared_timer_min(REAL(c_events(c_ev_num) % m))
	model_time = TRIM(I2S(time_year,4))//TRIM(I2S(time_month,2))//TRIM(I2S(time_day,2))//TRIM(I2S(time_hour,2))

	select case(act)
	
	case(ACTION_SAVE_CP)
		
		! Make file name and create new file
		file_name = TRIM(run_solution_folder)//TRIM(c_inf(comp) % name)//'_'//TRIM(I2S(c_inf(comp) % il))//'x'//	&
			TRIM(I2S(c_inf(comp) % jl))//'x'//TRIM(I2S(c_inf(comp) % kl))//"_"//run_experiment
		
		if(time_append_cp) then
			if(ANALYSIS_FLAG) then
				file_name = TRIM(file_name)//"_"//TRIM(model_time)//"_an.nc"
			else
				file_name = TRIM(file_name)//"_"//TRIM(model_time)//"_CP.nc"
			end if
		else
			file_name = TRIM(file_name)//"_REWR_CP.nc"
		end if
		
		call nc_rc(NF90_CREATE(TRIM(file_name), IOR(NF90_CLOBBER, IOR(NF90_NETCDF4, NF90_MPIIO)), nc_id, comm = comm_local, info = MPI_INFO_NULL))
					
		! Put global attrbutes
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "title", TRIM(c_inf(comp) % name)//" control point"))
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "experiment", run_experiment))
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "current_min", c_events(c_ev_num) % m))
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "real_time_of_last_CP", TRIM(time_str)))
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "model_time_of_last_CP", TRIM(time_model_date)))
					
!------->>>
!	add new parameters. used for masking region in plots (ncview, PYTHON). 
!-------
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL,"_FillValue"	, missval))
		call nc_rc(NF90_PUT_ATT (nc_id, NF90_GLOBAL, "missing_value"	, missval))
!------->>>

		! Define dimensions
		!call nc_rc( NF90_DEF_DIM(nc_id, RW_T_NAME, NF90_UNLIMITED, t_dimid) )
		call nc_rc( NF90_DEF_DIM(nc_id, RW_K_NAME, c_inf(comp) % kl, k_dimid) )
		call nc_rc( NF90_DEF_DIM(nc_id, RW_I_NAME, c_inf(comp) % il, i_dimid) )
		call nc_rc( NF90_DEF_DIM(nc_id, RW_J_NAME, c_inf(comp) % jl, j_dimid) )
		call nc_rc( NF90_DEF_DIM(nc_id, RW_M_NAME, c_inf(comp) % ml, m_dimid) )
		call nc_rc( NF90_DEF_DIM(nc_id, RW_L_NAME, NAME_LEN, l_dimid) )				
				
		! Define variables of various dims - 4D, 3D, 2D, 2D_KIND8
		call c_rw_def_vars(comp, act, nc_id, ARRAY_TYPE_4D, NF90_REAL, (/ i_dimid, j_dimid, k_dimid, m_dimid /)) 
		call c_rw_def_vars(comp, act, nc_id, ARRAY_TYPE_3D, NF90_REAL, (/ i_dimid, j_dimid, k_dimid /))
		call c_rw_def_vars(comp, act, nc_id, ARRAY_TYPE_2D, NF90_REAL, (/ i_dimid, j_dimid /))
		call c_rw_def_vars(comp, act, nc_id, ARRAY_TYPE_2D_KIND8, NF90_DOUBLE, (/ i_dimid, j_dimid /))
		
		!call nc_rc( NF90_DEF_VAR(nc_id, RW_T_NAME,  NF90_INT, t_dimid, time_varid))
		!call nc_rc( NF90_PUT_ATT(nc_id, time_varid, "units", "minutes since " // trim(i2s(time_start_year)) // "-" // 	& 
		!									trim(i2s(time_start_month,2)) // "-" //	&
		!									trim(i2s(time_start_day,2)))	)
				
		! End define mode.
		call nc_rc( NF90_ENDDEF(nc_id))

		!Writing time stamp (model minutes since experiment start)
		!The trouble is that such timestamp is not a dimension of our model arrays, so they cannot be time-merged by CDO
		!call nc_rc( NF90_PUT_VAR(nc_id, time_varid, c_events(c_ev_num) % m))
		
		! Put 4D, 3D, 2D, 2D_KIND8 data
 		do n = 1, c_events(c_ev_num) % num_4D
 		 	call nc_rc( NF90_PUT_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_4D, n), 			&
					stripe_4D(:,n), 									&
					c_rw_start_4D(comp,1:4), c_rw_count_4D(comp,1:4)))
 		end do

 		do n = 1, c_events(c_ev_num) % num_3D
 			call nc_rc( NF90_PUT_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_3D, n), 			&
					stripe_3D(:,n), 									&
					c_rw_start_4D(comp,1:3), c_rw_count_4D(comp,1:3) ))
 		end do
 
 		do n = 1, c_events(c_ev_num) % num_2D
 			call nc_rc( NF90_PUT_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_2D, n), 			&
					stripe_2D(:,n), 									&
					c_rw_start_4D(comp,1:2), c_rw_count_4D(comp,1:2))) 
 		end do
 
 		do n = 1, c_events(c_ev_num) % num_2D_kind8
 			call nc_rc( NF90_PUT_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_2D_KIND8, n), 		&
 				stripe_2D_kind8(:,n), 									&
 				c_rw_start_4D(comp,1:2), c_rw_count_4D(comp,1:2)))				
 		end do
 		
 		! Close CP-file
 		call nc_rc( NF90_CLOSE(nc_id) )
		
	case(ACTION_SAVE_DG)

		! Put 3D, 2D data
		do n = 1, c_events(c_ev_num) % num_3D
			call nc_rc( NF90_PUT_VAR(nc_id_dg(comp), c_rw_varids(comp, act, ARRAY_TYPE_3D, n), 				&
				stripe_3D(:,n),												& 
				(/ c_rw_start_4D(comp,1:3), c_rw_recnum(comp, act) /), (/ c_rw_count_4D(comp,1:3), 1 /)))
		end do

		do n = 1, c_events(c_ev_num) % num_2D
			!Collective operation of put var is not default on Cray			
			call nc_rc( NF90_VAR_PAR_ACCESS( nc_id_dg(comp), c_rw_varids(comp, act, ARRAY_TYPE_2D, n), NF90_Collective ) )
			
			call nc_rc( NF90_PUT_VAR(nc_id_dg(comp), c_rw_varids(comp, act, ARRAY_TYPE_2D, n), 				&
				stripe_2D(:,n),												& 
				(/ c_rw_start_4D(comp,1:2), c_rw_recnum(comp, act) /), (/ c_rw_count_4D(comp,1:2), 1 /)))
		end do

		! Put time variables
		call nc_rc( NF90_VAR_PAR_ACCESS(nc_id_dg(comp), c_rw_infoids(comp, act, VAR_INFO_REAL_TIME), NF90_Collective) )
		
		call nc_rc( NF90_PUT_VAR(nc_id_dg(comp), c_rw_infoids(comp, act, VAR_INFO_REAL_TIME), 				&
				TRIM(time_str), 											&
				(/ 1, c_rw_recnum(comp, act) /), (/ LEN(TRIM(time_str)), 1 /)))
				
		call nc_rc ( NF90_VAR_PAR_ACCESS(nc_id_dg(comp), c_rw_infoids(comp, act, VAR_INFO_MODEL_TIME), NF90_Collective) )

		call nc_rc( NF90_PUT_VAR(nc_id_dg(comp), c_rw_infoids(comp, act, VAR_INFO_MODEL_TIME),				& 
				TRIM(time_model_date),											& 
				(/1, c_rw_recnum(comp, act) /), (/ LEN(TRIM(time_model_date)), 1 /)))			
				
		! Flush data every 'run_sync_fs' write(or never if equal to 0)
		if(time_sync_fs /= 0 .AND. MOD(c_rw_recnum(comp, act), time_sync_fs) == 0) call nc_rc( NF90_SYNC(nc_id_dg(comp)) )
		
		! Increment record number
		c_rw_recnum(comp, act) = c_rw_recnum(comp, act) + 1	

	
	end select
	
	end subroutine c_rw_put_data
	
!==================================================================================================================================================================================
!=======================================================
! Major coupler receives, averages and saves diagnostics
!=======================================================

	subroutine c_rw_put_intg
		
	! Receive variables
	integer status(MPI_STATUS_SIZE)
	real(kind=8), allocatable :: recv_int(:,:), sum_ints(:,:)
	real(kind=8) :: areaLocal, areaGlobal
	character(NAME_LEN) :: names(BUFFER_LEN)
	
	! netCDF variables
	integer t_dimid
	integer, allocatable :: varids(:)
	
	! Work variables
	integer comp, n, slave
	integer num_steps	
	integer rc

	comp = c_events(c_ev_num) % src
		
	if(rank_world /= 0 .OR. c_inf(comp) % save_it == 0) return
	
!====================================================
! At first time receive names and prepare netCDF file
!====================================================

	! File not yet ENDDEFed - this is first time
	if(NF90_ENDDEF(nc_id_it(comp)) == NF90_NOERR) then

		call nc_rc(NF90_REDEF(nc_id_it(comp)))
		
		! Receive integral counts
		call MPI_RECV(	c_rw_it_counts(comp),					& 
				1,							&
				MPI_INTEGER, 						&
				c_inf(comp) % wk(me)  % slave_start, 			&
				shared_mpi_tags(ACTION_SAVE_IT,COUPLER),		&
				MPI_COMM_WORLD, status, ierr)
				
		! Receive names
		call MPI_RECV(	names,							& 
				NAME_LEN*c_rw_it_counts(comp),				&
				MPI_CHARACTER, 						&
				c_inf(comp) % wk(me)  % slave_start, 			&
				shared_mpi_tags(ACTION_SAVE_IT,COUPLER),		&
				MPI_COMM_WORLD, status, ierr)
				
		if(NF90_DEF_DIM(nc_id_it(comp), RW_T_NAME, NF90_UNLIMITED, t_dimid) /= NF90_NOERR) then
			! Already defined dimensions and variables
			call nc_rc(NF90_INQ_DIMID(nc_id_it(comp), RW_T_NAME, t_dimid))	
			call nc_rc(NF90_INQUIRE_DIMENSION(nc_id_it(comp), t_dimid, len=c_rw_recnum(comp, ACTION_SAVE_IT)))
			c_rw_recnum(comp, ACTION_SAVE_IT) = c_rw_recnum(comp, ACTION_SAVE_IT) + 1
		
			do n = 1, c_rw_it_counts(comp)
				call nc_rc( NF90_INQ_VARID(nc_id_it(comp), TRIM(names(n)),c_rw_varids(comp, ACTION_SAVE_IT, ARRAY_TYPE_1D, n)) )
			end do
		else
			! New file - we should define them
			do n = 1, c_rw_it_counts(comp)
				call nc_rc( NF90_DEF_VAR(nc_id_it(comp), TRIM(names(n)), NF90_DOUBLE, (/ t_dimid /), c_rw_varids(comp, ACTION_SAVE_IT, ARRAY_TYPE_1D, n)) )
			end do
		end if

		call nc_rc( NF90_ENDDEF(nc_id_it(comp)) )
	
	end if

!==================
! Receive integrals
!==================
	num_steps = c_inf(comp) % sync_it
		
	! Allocate arrays and initialize sums
	allocate(recv_int(num_steps, c_rw_it_counts(comp)))
	allocate(sum_ints(num_steps, c_rw_it_counts(comp)))
	sum_ints = 0
	areaGlobal = 0
	
	! Receive values and sum from all processors of component
	do slave = 1, c_inf(comp) % proc_count

		call MPI_RECV(recv_int,						&
			SIZE(recv_int), 					&
			MPI_DOUBLE_PRECISION, 					&
			c_inf(comp) % wk(me)  % slave_start + slave - 1,	&
			shared_mpi_tags(ACTION_SAVE_IT,COUPLER),		&
			MPI_COMM_WORLD, status, ierr)

		call MPI_RECV(areaLocal,					&
			1, 							&
			MPI_DOUBLE_PRECISION, 					&
			c_inf(comp) % wk(me)  % slave_start + slave - 1,	&
			shared_mpi_tags(ACTION_SAVE_IT,COUPLER),		&
			MPI_COMM_WORLD, status, ierr)

			areaGlobal = areaGlobal + areaLocal
			sum_ints = sum_ints + recv_int
	end do
	
	! Average sum per global area
	sum_ints = sum_ints/areaGlobal
			
 	! Put data
 	do n = 1, c_rw_it_counts(comp)
 		call nc_rc( NF90_PUT_VAR(nc_id_it(comp), c_rw_varids(comp, ACTION_SAVE_IT, ARRAY_TYPE_1D, n), sum_ints(:,n), (/ c_rw_recnum(comp, ACTION_SAVE_IT) /), (/ num_steps /)))
 	end do

 	! Flush data every 'time_sync_it' write(or never if equal to 0)
	call nc_rc( NF90_SYNC(nc_id_it(comp)) )
	
 	c_rw_recnum(comp, ACTION_SAVE_IT) = c_rw_recnum(comp, ACTION_SAVE_IT) + num_steps
 
 	deallocate(recv_int, sum_ints)
	
	end subroutine c_rw_put_intg
	
!==================================================================================================================================================================================
	
!===================================================================
! Procedure get data from opened FD-file and open and read data in
! case of one-time IC and CP data
!===================================================================

	subroutine c_rw_get_data

	character(len = NAME_LEN) postfix
	character(len = 20) run_folder
	integer nc_id

	act = c_events(c_ev_num) % act
	comp = c_events(c_ev_num) % dst

	!=========================================
	! Read control point or initial conditions
	!=========================================
	select case(act)

	case(ACTION_READ_CP, ACTION_READ_IC)

		select case(act)
			
		case(ACTION_READ_CP)
			postfix = "_"//run_experiment//"_"//TRIM(time_read_cp)//"_CP.nc"
			run_folder = run_solution_folder
		case(ACTION_READ_IC)
			postfix = "_IC.nc"
			run_folder = run_data_folder
		end select

		file_name = TRIM(run_folder)//TRIM(c_inf(comp) % name)//'_'//TRIM(I2S(c_inf(comp) % il))//'x'//	&
			    TRIM(I2S(c_inf(comp) % jl))//'x'//TRIM(I2S(c_inf(comp) % kl))//TRIM(postfix)

		nc_err = NF90_OPEN(TRIM(file_name), IOR(NF90_NOWRITE, IOR(NF90_NETCDF4, NF90_MPIIO)), nc_id, comm = comm_local, info = MPI_INFO_NULL)
		if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant open file: '// file_name)
			
		do t = 1, ARRAY_TYPE_NUM_FIELDS
		do n = 1, c_inf(comp) % rw(act, t) % ar_size
			nc_err = NF90_INQ_VARID(nc_id, c_inf(comp) % rw(act, t) % ar(n) % name, c_rw_varids(comp, act, t, n))
			if(nc_err /= NF90_NOERR) call shared_error('Coupler: Cant find variable <'//TRIM(c_inf(comp) % rw(act, t) % ar(n) % name)//'> in file : '//TRIM(file_name)//'.')
		end do
		end do

		! Get 4D, 3D, 2D, 2D_KIND8 data
		do n = 1, c_events(c_ev_num) % num_4D
			call nc_rc( NF90_GET_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_4D, n), 			&
				stripe_4D(:,n), 									&
				c_rw_start_4D(comp,1:4), c_rw_count_4D(comp,1:4)))
		end do

		do n = 1, c_events(c_ev_num) % num_3D
			call nc_rc( NF90_GET_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_3D, n), 			&
				stripe_3D(:,n), 									&
				c_rw_start_4D(comp,1:3), c_rw_count_4D(comp,1:3)))
		end do
		
		do n = 1, c_events(c_ev_num) % num_2D
			call nc_rc( NF90_GET_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_2D, n), 			&
				stripe_2D(:,n), 									&
				c_rw_start_4D(comp,1:2), c_rw_count_4D(comp,1:2)))
		end do
		
		do n = 1, c_events(c_ev_num) % num_2D_kind8
			call nc_rc( NF90_GET_VAR(nc_id, c_rw_varids(comp, act, ARRAY_TYPE_2D_KIND8, n), 		&
				stripe_2D_kind8(:,n), 									&
				c_rw_start_4D(comp,1:2), c_rw_count_4D(comp,1:2)))
		end do
		
		! Close files
 		call nc_rc( NF90_CLOSE(nc_id))
				
	case(ACTION_READ_FD)
	
		! Get 2D data		
		do n = 1, c_events(c_ev_num) % num_2D
					
			nc_err = NF90_GET_VAR(nc_id_fd(comp, n), c_rw_varids(comp, act, ARRAY_TYPE_2D, n), 				&
					stripe_2D(:,n),											& 
					(/ c_rw_start_4D(comp,1:2), nc_recnum_fd(comp, n) /), (/ c_rw_count_4D(comp,1:2), 1 /))
														
			! Variable is finished, read from the start
			if(nc_err /= NF90_NOERR) then
				nc_recnum_fd(comp, n) = 1
				
				nc_err = NF90_GET_VAR(nc_id_fd(comp, n), c_rw_varids(comp, act, ARRAY_TYPE_2D, n), 			&
					stripe_2D(:,n),											& 
					(/ c_rw_start_4D(comp,1:2), nc_recnum_fd(comp, n) /), (/ c_rw_count_4D(comp,1:2), 1 /))	
			end if
			
			!if(c_events(c_ev_num) % m /= time_start_min .AND. MOD(c_events(c_ev_num) % m, c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar(n) % period) == 0) &
			 if(                                               MOD(c_events(c_ev_num) % m, c_inf(comp) % rw(act, ARRAY_TYPE_2D) % ar(n) % period) == 0) &
					nc_recnum_fd(comp, n) = nc_recnum_fd(comp, n) + 1			
		end do
	
	end select
	
	end subroutine c_rw_get_data

!==================================================================================================================================================================================

!======================================
! Subroutine for defining new variables
!======================================

	subroutine c_rw_def_vars(comp, act, nc, arrType, dataType, dimids)
	
	integer comp, act, arrType, dataType, nc
	integer dimids(:)
	
	do n = 1, c_inf(comp) % rw(act, arrType) % ar_size
		call nc_rc( NF90_DEF_VAR(nc, c_inf(comp) % rw(act, arrType) % ar(n) % name, dataType, dimids, c_rw_varids(comp, act, arrType, n)) )
		call nc_rc( NF90_PUT_ATT(nc, c_rw_varids(comp, act, arrType, n), "long_name", c_inf(comp) % rw(act, arrType) % ar(n) % lname) )		
		call nc_rc( NF90_PUT_ATT(nc, c_rw_varids(comp, act, arrType, n), "units", c_inf(comp) % rw(act, arrType) % ar(n) % units) )
	end do
	
	end subroutine c_rw_def_vars

!==================================================================================================================================================================================

!=======================================
! Close netCFD files at finalizing phase
!=======================================

	subroutine c_rw_close_files
	
	integer comp, n
	
	! Close files
	do comp = 1, NUM_OF_COMP
		if(nc_id_dg(comp) /= MYNULL) call nc_rc( NF90_CLOSE(nc_id_dg(comp)) )
		if(nc_id_it(comp) /= MYNULL) call nc_rc( NF90_CLOSE(nc_id_it(comp)) )		
	end do

	do comp = 1, NUM_OF_COMP
	do n = 1, c_inf(comp) % rw(ACTION_READ_FD, ARRAY_TYPE_2D) % ar_size
		if(nc_id_fd(comp, act) /= MYNULL) call nc_rc( NF90_CLOSE(nc_id_fd(comp,n)) )		
	end do
	end do

	end subroutine c_rw_close_files

!==================================================================================================================================================================================	

!====================
! Print netCDF errors
!====================

	subroutine nc_rc(status)

	integer, intent ( in) :: status
	
	if(status /= NF90_NOERR) then 
		call shared_error("Coupler netCDF error: "//TRIM(NF90_STRERROR(status)))
	end if
	
	end subroutine nc_rc  
	
!==================================================================================================================================================================================	
	
	end module c_rw_module
	
!==================================================================================================================================================================================

!=======================================================================================
! Get current hour at initialization from CP-file (external, because used in c_register)
!=======================================================================================
	
	subroutine c_rw_get_run_info(comp)

	use shared_utils_module
	use shared_module
	use c_module
	use netcdf
	use c_rw_module
	
	implicit none
	
	integer comp
	integer file_id
	logical file_exists
	character(NAME_LEN) file_name
	
	file_name = TRIM(run_solution_folder)//TRIM(c_inf(comp) % name)//'_'//TRIM(I2S(c_inf(comp) % il))//'x'//	&
			    TRIM(I2S(c_inf(comp) % jl))//'x'//TRIM(I2S(c_inf(comp) % kl))//"_"//run_experiment//"_"//TRIM(time_read_cp)//"_CP.nc"
		
	INQUIRE(file = TRIM(file_name), exist = file_exists)
 
	if(file_exists) then
		call nc_rc(NF90_OPEN(TRIM(file_name), IOR(NF90_NOWRITE, IOR(NF90_NETCDF4, NF90_MPIIO)), file_id, comm = comm_local, info = MPI_INFO_NULL))	
		call nc_rc(NF90_GET_ATT(file_id, NF90_GLOBAL, "current_min", time_start_min) )
		call nc_rc(NF90_CLOSE(file_id))
	else
	!	print *,'WARNING: cannot read time_start_min from the CP-file: ',trim(file_name)
	end if
	!time_start_min=0 !15768000
	print *,TRIM(c_inf(comp) % name),' start minute = ',time_start_min
	
	end subroutine c_rw_get_run_info	
	
	
