
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

	subroutine c_spy_log(ev)
	
	use shared_module
	use c_module
	use c_gd_module
	use shared_utils_module

	implicit none

	integer ev
	integer cur_time_arr(8)
	integer i, j, n, m

!=================================================
! Open logfile and write basic parameters on start
!=================================================
	if(rank_local /= 0) return

	select case (ev)

	case(0)
		
		if(run_log_file_num /= 6) open(run_log_file_num, file = '_log_file.txt')

		call DATE_AND_TIME(VALUES = cur_time_arr)

		write(run_log_file_num, *) '========================================================='
		write(run_log_file_num, *) '============= Running INM RAS coupled model ============='
		write(run_log_file_num, *) '========================================================='	      
		write(run_log_file_num, '(a19,i2.2,a1,i2.2,a1,i2.2)') 'Real time of run: ', cur_time_arr(5),':',cur_time_arr(6),':',cur_time_arr(7)
		write(run_log_file_num, '(a19,i2.2,a1,i2.2,a1,i4.4)') 'Real date of run: ', cur_time_arr(3),'.',cur_time_arr(2),'.',cur_time_arr(1)
		write(run_log_file_num, *) ''

		write(run_log_file_num, *) '=== Main parameters of current run ==='
		write(run_log_file_num, *) ''
		write(run_log_file_num, '(a31, 3x, i2)') 'Number of defined components: ', NUM_OF_COMP

		

		write(run_log_file_num, '(a31, 3x, i2)') 'Number of running components: ', COUNT(c_inf(1:NUM_OF_COMP) % proc_count > 0)
		write(run_log_file_num, *) ''
		write(run_log_file_num, *) 'Component names and number of processors: '

		do i = 0, NUM_OF_COMP
			if(c_inf(i) % proc_count /= 0) write(run_log_file_num, '(a20, a1, i6)') c_inf(i) % name, '', c_inf(i) % proc_count
		end do

		write(run_log_file_num, *) ''
		write(run_log_file_num, '(a15,i3,a8,i2,a9,i2,a6)') 'Run model on: ', time_run_years, ' years, ', time_run_months,' months, ', time_run_days, ' days.'
		write(run_log_file_num, '(a18, a3)') 'Experiment name: ', run_experiment
		write(run_log_file_num, '(a25, a10)') 'Experiment data folder: ', run_data_folder
		write(run_log_file_num, '(a28, l4)') 'Save log information flag: ', run_save_log

		write(run_log_file_num, *) ''

		write(run_log_file_num, *) '=== Mapping table information ==='
		do n = 1, NUM_OF_COMP
		do m = 1, NUM_OF_COMP
			if(c_inf(n) % proc_count /= 0 .AND. c_inf(m) % proc_count /= 0 .AND. c_map_info(n,m,1,MAP_NUM_FUNC) /= 0) then
				write(run_log_file_num, '(a40,a10,a5,a10)') 'Parameters of interpolation between : ',c_inf(n) % name, ' ==> ', c_inf(m) % name
				write(run_log_file_num, 1) 'Mapping type', 	c_map_info(n,m,1, MAP_TYPE)
				write(run_log_file_num, 1) 'Number of fileds', 	c_map_info(n,m,1, MAP_NUM_FUNC)
				write(run_log_file_num, 1) 'Period in minutes', 	c_map_info(n,m,1, MAP_PERIOD_MIN)
			end if
		end do
		end do

		write(run_log_file_num, *) ''

		write(run_log_file_num, *) '=== Coupler main information ==='
		write(run_log_file_num, 1) 'Number of processors', 	run_np(COUPLER)
		write(run_log_file_num, 1) 'Number of events in run', 	c_ev_count
		write(run_log_file_num, 1) 'Period of save in minutes', time_save_cp


		write(run_log_file_num, *) ''
		write(run_log_file_num, *) '=== Coupler memory usage, mb ==='
		write(run_log_file_num, 1) 'Stripe 4D', SIZE(stripe_4D)*4/1000000
		write(run_log_file_num, 1) 'Stripe 3D', SIZE(stripe_3D)*4/1000000
		write(run_log_file_num, 1) 'Stripe 2D', SIZE(stripe_2D)*4/1000000
		write(run_log_file_num, 1) 'Stripe 2D_KIND8', SIZE(stripe_2D_kind8)*8/1000000
		write(run_log_file_num, 1) 'Send-recv vect 4', SIZE(c_pack_kind4)*4/1000000
		write(run_log_file_num, 1) 'Send-recv vect 8', SIZE(c_pack_kind8)*8/1000000	
		write(run_log_file_num, 1) 'Map data'
		write(run_log_file_num, *) ''

		write(run_log_file_num, *) '=== Components main information ==='
		write(run_log_file_num, *) '-------------------------------------------'
		do i = 1, NUM_OF_COMP
			
			if(c_inf(i) % proc_count /= 0) then

				write(run_log_file_num, 2) 'Name of component ',		c_inf(i) % name
				write(run_log_file_num, 1) 'Number of processors', 		c_inf(i) % proc_count
				write(run_log_file_num, 1) 'Slaves in 1 coupler', 		c_inf(i) % wk(me) % slave_count
				write(run_log_file_num, '(a32,i4,a3,i4,a1)') 		'Processor decomp[inp x jnp] [', 		&
											c_inf(i) % inp, ' x ',		&
											c_inf(i) % jnp, ']'
				write(run_log_file_num, '(a32,i4,a3,i4,a3,i4,a3,i4,a1)') 	'Mesh size[i x j x k x m] [', 			&
											c_inf(i) % il, ' x ',	&
											c_inf(i) % jl, ' x ',	&
											c_inf(i) % kl, ' x ',	&
											c_inf(i) % ml, ']'

				write(run_log_file_num, '(a32,i4,a3,i4,a1)') 		'Local size[i_loc x j_loc] [', 		&
											c_inf(i) % wk(me) % sl(1) % il_belt, ' x ',	&
											c_inf(i) % wk(me) % sl(1) % jl_belt, ']'
				if(MOD(c_inf(i) % wk(size_local) % stripe_count,c_inf(i) % wk(me) % sl(1) % jl_belt) /= 0) 		&
				write(run_log_file_num, '(a32,i4,a3,i4,a1)') 		'Last belt size[i_loc x j_loc] [', 		&
											c_inf(i) % wk(me) % sl(1) % il_belt, ' x ',	&
											MOD(c_inf(i) % wk(size_local) % stripe_count,c_inf(i) % wk(me) % sl(1) % jl_belt), ']'											
				write(run_log_file_num, 1) 'Period of save DG in minutes',	c_inf(i) % save_dg
				write(run_log_file_num, 1) 'Period of save IT in minutes',	c_inf(i) % save_it
				write(run_log_file_num, 1) 'Period of read FD in minutes',	c_inf(i) % read_fd
				write(run_log_file_num, *) '-------------------------------------------'				
			end if

		end do

		write(run_log_file_num, '(a, f10.2)') ' System initialization time: ', MPI_WTIME() - wall_time_start_ini
		write(run_log_file_num, *) '-------------------------------------------'		
		write(run_log_file_num, *) ''
		write(run_log_file_num, *) '=== Start model cycle ==='
		write(run_log_file_num, *) ''
		call flush(run_log_file_num)		
  
	case(MYNULL)
  		write(run_log_file_num, '(/a20, 14x,f10.2)') 'Total time of run: ', MPI_WTIME() - wall_time_start_cycle	
		write(run_log_file_num, '(/A/)') '=== All resources are deallocated. End program. ==='	
		call flush(run_log_file_num)
		
		if(run_log_file_num /= 6) close(run_log_file_num)
		

	case default

!==========================
! Write event into log file
!==========================
		call shared_timer_min(REAL(c_events(ev) % m))
		write(run_log_file_num,*) 					'======================================'
		write(run_log_file_num, '(a15, 3x, a21)') 			' Model time:          ', TRIM(time_model_date)		
		write(run_log_file_num, '(a15, 3x, a21)') 			' Event number:        ', TRIM(I2S(ev))
		write(run_log_file_num, '(a15, 3x, a21)') 			' Event type:          ', TRIM(event_type_name(c_events(ev) % act))
		write(run_log_file_num, '(a15, 3x, a21)') 			' Source:              ', TRIM(c_inf(c_events(ev) % src) % name)
		write(run_log_file_num, '(a15, 3x, a21)') 			' Destination:         ', TRIM(c_inf(c_events(ev) % dst) % name)
		write(run_log_file_num, '(a15, 13x,i2,1x,i2,1x,i2,1x,i2)') 	' 4D,3D,2D,2D_8:       ', c_events(ev) % num_4D,  c_events(ev) % num_3D,  c_events(ev) % num_2D,  c_events(ev) % num_2D_kind8
		write(run_log_file_num, '(a15, 14x,f10.2)') 			' Time of run:         ', MPI_WTIME() - wall_time_start_cycle		
		write(run_log_file_num,*) 					'======================================'
		
		if(c_ev_num == c_ev_count) then
			write(run_log_file_num, *) ''
			write(run_log_file_num, *) '=== Finish model cycle ==='
			write(run_log_file_num, *) ''
			write(run_log_file_num, *) '=== Finalizing stage ==='			
		end if
		call flush(run_log_file_num)	

	end select

1	FORMAT(a30, 3x, i10)
2	FORMAT(a30, 3x, a10)

	return
	end









