
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!======================================================
! Contains arrays and variables for exchange procedures
!======================================================

	module comp_halo_update_module

	implicit none

!===============================
! Work arrays for halo functions
!===============================
	real (kind = 4), allocatable :: t4(:,:,:,:)
	real (kind = 8), allocatable :: t8(:,:,:,:)
	real (kind = 4), allocatable :: u4(:,:,:,:), v4(:,:,:,:)
	real (kind = 8), allocatable :: u8(:,:,:,:), v8(:,:,:,:)

!=================================================
! Send-recv vectors V, T for case of REAL datatype
!=================================================
	real (kind=4), allocatable :: vect_south_send4 (:)
	real (kind=4), allocatable :: vect_north_send4 (:)
	real (kind=4), allocatable :: vect_west_send4 (:)
	real (kind=4), allocatable :: vect_east_send4 (:)

	real (kind=4), allocatable :: vect_south_recv4 (:)
	real (kind=4), allocatable :: vect_north_recv4 (:)
	real (kind=4), allocatable :: vect_west_recv4 (:)
	real (kind=4), allocatable :: vect_east_recv4 (:)

!============================================================
! Send-recv vectors V,T for case of DOUBLE PRECISION datatype
!============================================================
	real (kind=8), allocatable :: vect_south_send8 (:)
	real (kind=8), allocatable :: vect_north_send8 (:)
	real (kind=8), allocatable :: vect_west_send8 (:)
	real (kind=8), allocatable :: vect_east_send8 (:)

	real (kind=8), allocatable :: vect_south_recv8 (:)
	real (kind=8), allocatable :: vect_north_recv8 (:)
	real (kind=8), allocatable :: vect_west_recv8 (:)
	real (kind=8), allocatable :: vect_east_recv8 (:)
	
!==========================
! MPI requests and statuses
!==========================
	integer, allocatable::  request_v_lon	(:,:,:,:)
	integer, allocatable::  request_v_lat	(:,:,:,:)
	integer, allocatable::  request_t_lon	(:,:,:,:)
	integer, allocatable::  request_t_lat	(:,:,:,:)
	integer, allocatable::  request_brtr_lat(:)
	integer, allocatable::  request_brtr_lon(:)

	integer, allocatable::  status_v	(:,:)
	integer, allocatable::  status_t	(:,:)
	integer, allocatable::  status_brtr	(:,:)
	
	CONTAINS

!=====================================================================================================================================================================

	subroutine comp_halo_update_alloc

	use shared_module
	use comp_module
	
	implicit none

	integer ext_coeff

!===============================
! Work arrays for halo functions
!===============================
	allocate (t4 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width+1, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	allocate (t8 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width+1, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	allocate (u4 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	allocate (v4 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	allocate (u8 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	allocate (v8 (cmp % west - comp_halo_max_width:cmp % east + comp_halo_max_width, cmp % south - comp_halo_max_width:cmp % north + comp_halo_max_width, cmp % k_size, cmp % m_size ))
	t4 = 0.
	t8 = 0.
	u4 = 0.
	u8 = 0.
	v4 = 0.
	v8 = 0.

!================================================
! Send-recv V,T vectors for case of REAL datatype
!================================================

! Now fast brtr use this vectors, but it has 3 2D functions in one send-recv, so
! in exotic case of small ml and kl we should provide enough memory
	ext_coeff = 1
	if(cmp % m_size == 1 .AND. cmp % k_size == 1) ext_coeff = 2

	allocate (vect_south_send4	(ext_coeff*2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_north_send4	(ext_coeff*2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_west_send4	(ext_coeff*2*cmp % m_size*cmp % k_size*cmp % j_loc_size*(comp_halo_max_width+1)))
	allocate (vect_east_send4	(ext_coeff*2*cmp % m_size*cmp % k_size*cmp % j_loc_size*comp_halo_max_width))
	allocate (vect_south_recv4	(ext_coeff*2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_north_recv4	(ext_coeff*2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_west_recv4	(ext_coeff*2*cmp % m_size*cmp % k_size*cmp % j_loc_size*comp_halo_max_width))
	allocate (vect_east_recv4	(ext_coeff*2*cmp % m_size*cmp % k_size*cmp % j_loc_size*(comp_halo_max_width+1)))
	vect_south_send4 = 0.
	vect_north_send4 = 0.
	vect_west_send4 = 0.
	vect_east_send4 = 0.
	vect_south_recv4 = 0.
	vect_north_recv4 = 0.
	vect_west_recv4 = 0.
	vect_east_recv4 = 0.

!============================================================
! Send-recv V,T vectors for case of DOUBLE PRECISION datatype
!============================================================
	allocate (vect_south_send8	(2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_north_send8	(2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_west_send8	(2*cmp % m_size*cmp % k_size*cmp % j_loc_size*(comp_halo_max_width+1)))
	allocate (vect_east_send8	(2*cmp % m_size*cmp % k_size*cmp % j_loc_size*comp_halo_max_width))
	allocate (vect_south_recv8	(2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_north_recv8	(2*cmp % m_size*cmp % k_size*(cmp % i_loc_size + 2*comp_halo_max_width)*comp_halo_max_width))
	allocate (vect_west_recv8	(2*cmp % m_size*cmp % k_size*cmp % j_loc_size*comp_halo_max_width))
	allocate (vect_east_recv8	(2*cmp % m_size*cmp % k_size*cmp % j_loc_size*(comp_halo_max_width+1)))
	vect_south_send8 = 0.
	vect_north_send8 = 0.
	vect_west_send8 = 0.
	vect_east_send8 = 0.
	vect_south_recv8 = 0.
	vect_north_recv8 = 0.
	vect_west_recv8 = 0.
	vect_east_recv8 = 0.
	
!==========================
! MPI requests and statuses
!==========================
	allocate ( request_v_lon(4, 2:4, 2, 1:comp_halo_stencil+1))
	allocate ( request_v_lat(4, 2:4, 2, 1:comp_halo_stencil+1))
	allocate ( request_t_lon(4, 2:4, 2, 1:comp_halo_stencil+1))
	allocate ( request_t_lat(4, 2:4, 2, 1:comp_halo_stencil+1))
	allocate ( request_brtr_lat(4))
	allocate ( request_brtr_lon(4))
	
	allocate ( status_v	(MPI_STATUS_SIZE,4))
	allocate ( status_t	(MPI_STATUS_SIZE,4))
	allocate ( status_brtr	(MPI_STATUS_SIZE,4))
	
	return
	end subroutine comp_halo_update_alloc
 
!=====================================================================================================================================================================

!============================================================
! Define neighbours and initialize MPI commands. 
! Create 6 diffrerent request sets for 3 cases of 
! array dimensions: 2D, 3D and 4D arrays and
! 2 cases of data size: REAL and DOUBLE_PRECISION. Note, that
! MPI functions use local communicator 'comm', defined in 
! procedure 'shared_ini' for all components.
! Also, all exchanges are defined for all widths of halo:
! from 1 to comp_halo_stencil and for comp_halo_wide.
!============================================================

	subroutine comp_halo_update_ini
	

	use shared_module
	use comp_module

	implicit none
	
	integer dim_num, dat, size_ext, datatype
	integer num_of_func
	integer w, cur_width

!======================
! 1. Compute neighbours
!======================
	comp_southrank	= rank_local - cmp % inp
	comp_westrank	= rank_local - 1
	comp_northrank	= rank_local + cmp % inp
	comp_eastrank	= rank_local + 1

	comp_swrank 	= rank_local - cmp % inp - 1
	comp_serank 	= rank_local - cmp % inp + 1
	comp_nwrank 	= rank_local + cmp % inp - 1
	comp_nerank 	= rank_local + cmp % inp + 1

!============================================================
! If comp_i_cycle=1, making cycling neighbours in i-dimension  
!============================================================
	if(cmp % west == 1) then

		if(comp_i_cycle) then

			comp_westrank = rank_local + cmp % inp - 1
			comp_swrank = rank_local - 1
			comp_nwrank = rank_local + 2*cmp % inp - 1
				
		else

			comp_westrank = MPI_PROC_NULL
			comp_swrank = MPI_PROC_NULL
			comp_nwrank = MPI_PROC_NULL
	
		end if


	end if

	if(cmp % east == cmp % i_size) then 

		if(comp_i_cycle) then

			comp_eastrank = rank_local - cmp % inp + 1
			comp_serank = rank_local - 2*cmp % inp + 1
			comp_nerank = rank_local + 1

		else

			comp_eastrank = MPI_PROC_NULL
			comp_serank = MPI_PROC_NULL
			comp_nerank = MPI_PROC_NULL

		end if
	
	end if

	if(cmp % south == 1) then

		comp_southrank = MPI_PROC_NULL
		comp_swrank = MPI_PROC_NULL
		comp_serank = MPI_PROC_NULL

	end if

!===================================================================
! If i_bsc=1, then making cycling on Bipolar Grinvich in j-dimension
!===================================================================
	if(cmp % north == cmp % j_size) then

		if(comp_bsc_grid) then
		
			if(cmp % inp == 1) then
				comp_northrank = rank_local
			else
				comp_northrank = (size_local  - cmp % inp/2) + ((size_local-cmp % inp/2) - (rank_local + 1))
			end if

			if(cmp % east /= cmp % i_size) then
				comp_nerank = comp_northrank - 1
			else
				comp_nerank = MPI_PROC_NULL
			end if

			if(cmp % west /= 1) then
				comp_nwrank = comp_northrank + 1
			else
				comp_nwrank = MPI_PROC_NULL
			end if
		
		else

			comp_northrank = MPI_PROC_NULL
			comp_nwrank = MPI_PROC_NULL
			comp_nerank = MPI_PROC_NULL
		end if

	end if

!=============================================
! 2. Make MPI_INITs for different common cases
!=============================================

!===================================================
! Start big cycle for all types of exchanges: 
! from 1 to comp_halo_stencil and for comp_halo_wide
!===================================================
	do w = 1, comp_halo_stencil + 1

		cur_width = w
		if(w == comp_halo_stencil + 1) cur_width = comp_halo_wide

!================
! 2.1 For V-cells
!================
		num_of_func = 2 ! U,V
!==========
! REAL data
!==========
		dat = 1
		datatype = MPI_REAL

		do dim_num = 2, 4

			if(dim_num == 2) size_ext = 1				!2D
			if(dim_num == 3) size_ext = cmp % k_size		!3D
			if(dim_num == 4) size_ext = cmp % m_size*cmp % k_size	!4D

			call MPI_SEND_INIT(vect_west_send4,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					201,									&
					comm_local, 								&
					request_v_lon(1,dim_num,dat,w), ierr)
		
			call MPI_SEND_INIT(vect_east_send4,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype, 								&
					comp_eastrank, 								&
					202,									&
					comm_local, 								&
					request_v_lon(2,dim_num,dat,w), ierr)
					
			call MPI_RECV_INIT(vect_east_recv4, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_eastrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lon(3,dim_num,dat,w), ierr)
			
			call MPI_RECV_INIT(vect_west_recv4, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lon(4,dim_num,dat,w), ierr)					
										
			call MPI_SEND_INIT(vect_south_send4,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_southrank, 							&
					203,									&
					comm_local, 								&
					request_v_lat(1,dim_num,dat,w), ierr)

			call MPI_SEND_INIT(vect_north_send4,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_northrank, 							&
					204,									&
					comm_local, 								&
					request_v_lat(2,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_north_recv4, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_northrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lat(3,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_south_recv4, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_southrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lat(4,dim_num,dat,w), ierr)
		end do

!======================
! DOUBLE_PRECISION data
!======================
		dat = 2
		datatype = MPI_DOUBLE_PRECISION
		
		do dim_num = 2, 4

			if(dim_num == 2) size_ext = 1				!2D
			if(dim_num == 3) size_ext = cmp % k_size		!3D
			if(dim_num == 4) size_ext = cmp % m_size*cmp % k_size	!4D

			call MPI_SEND_INIT(vect_west_send8,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					201,									&
					comm_local, 								&
					request_v_lon(1,dim_num,dat,w), ierr)
		
			call MPI_SEND_INIT(vect_east_send8,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype, 								&
					comp_eastrank, 								&
					202,									&
					comm_local, 								&
					request_v_lon(2,dim_num,dat,w), ierr)
					
			call MPI_RECV_INIT(vect_east_recv8, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_eastrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lon(3,dim_num,dat,w), ierr)
			
			call MPI_RECV_INIT(vect_west_recv8, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lon(4,dim_num,dat,w), ierr)					
										
			call MPI_SEND_INIT(vect_south_send8,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_southrank, 							&
					203,									&
					comm_local, 								&
					request_v_lat(1,dim_num,dat,w), ierr)

			call MPI_SEND_INIT(vect_north_send8,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_northrank, 							&
					204,									&
					comm_local, 								&
					request_v_lat(2,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_north_recv8, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_northrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lat(3,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_south_recv8, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_southrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_v_lat(4,dim_num,dat,w), ierr)
		end do

!================
! 2.2 For T-cells
!================

		num_of_func = 1 ! T

!==========
! REAL data
!==========
		dat = 1
		datatype = MPI_REAL

		do dim_num = 2, 4

			if(dim_num == 2) size_ext = 1				!2D
			if(dim_num == 3) size_ext = cmp % k_size		!3D
			if(dim_num == 4) size_ext = cmp % m_size*cmp % k_size	!4D
		
			call MPI_SEND_INIT(vect_west_send4,							&
					size_ext*(cur_width+1)*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					201,									&
					comm_local, 								&
					request_t_lon(1,dim_num,dat,w), ierr)
		
			call MPI_SEND_INIT(vect_east_send4,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype, 								&
					comp_eastrank, 								&
					202,									&
					comm_local, 								&
					request_t_lon(2,dim_num,dat,w), ierr)
					
			call MPI_RECV_INIT(vect_east_recv4, 							&
					size_ext*(cur_width+1)*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_eastrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lon(3,dim_num,dat,w), ierr)
			
			call MPI_RECV_INIT(vect_west_recv4, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lon(4,dim_num,dat,w), ierr)					
										
			call MPI_SEND_INIT(vect_south_send4,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_southrank, 							&
					203,									&
					comm_local, 								&
					request_t_lat(1,dim_num,dat,w), ierr)

			call MPI_SEND_INIT(vect_north_send4,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_northrank, 							&
					204,									&
					comm_local, 								&
					request_t_lat(2,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_north_recv4, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_northrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lat(3,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_south_recv4, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_southrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lat(4,dim_num,dat,w), ierr)
		end do


!======================
! DOUBLE_PRECISION data
!======================
		dat = 2
		datatype = MPI_DOUBLE_PRECISION
		
		do dim_num = 2, 4

			if(dim_num == 2) size_ext = 1				!2D
			if(dim_num == 3) size_ext = cmp % k_size		!3D
			if(dim_num == 4) size_ext = cmp % m_size*cmp % k_size	!4D
		
			call MPI_SEND_INIT(vect_west_send8,							&
					size_ext*(cur_width+1)*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					201,									&
					comm_local, 								&
					request_t_lon(1,dim_num,dat,w), ierr)
		
			call MPI_SEND_INIT(vect_east_send8,							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype, 								&
					comp_eastrank, 								&
					202,									&
					comm_local, 								&
					request_t_lon(2,dim_num,dat,w), ierr)
					
			call MPI_RECV_INIT(vect_east_recv8, 							&
					size_ext*(cur_width+1)*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_eastrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lon(3,dim_num,dat,w), ierr)
			
			call MPI_RECV_INIT(vect_west_recv8, 							&
					size_ext*cur_width*(cmp % j_loc_size)*num_of_func,			&
					datatype,				 				&
					comp_westrank, 								&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lon(4,dim_num,dat,w), ierr)					
										
			call MPI_SEND_INIT(vect_south_send8,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_southrank, 							&
					203,									&
					comm_local, 								&
					request_t_lat(1,dim_num,dat,w), ierr)

			call MPI_SEND_INIT(vect_north_send8,							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_northrank, 							&
					204,									&
					comm_local, 								&
					request_t_lat(2,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_north_recv8, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype,								&
					comp_northrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lat(3,dim_num,dat,w), ierr)

			call MPI_RECV_INIT(vect_south_recv8, 							&
					size_ext*cur_width*(cmp % i_loc_size + 2*cur_width)*num_of_func,	&
					datatype, 								&
					comp_southrank, 							&
					MPI_ANY_TAG,								&
					comm_local, 								&
					request_t_lat(4,dim_num,dat,w), ierr)		
		end do

	end do ! 1, comp_halo_stencil+1

	return
	end subroutine comp_halo_update_ini

!==================================================================================================================================================

!================================================================
! These routine performs update of wide halo belts of calculating 
! domain in T points.
! Types of input data:
! -- kind: 4, 8
! -- dims: 2D, 3D, 4D
! -- update_width: 1-comp_stencil and comp_wide
!
! Use manual for full procedure description.
!================================================================


	subroutine comp_halo_update_T4 (DIM4_t,				&
					 DIM3_t,				&
					 DIM2_t,				&
					 update_width, change_sign_on_bipolar)


	use comp_module
	use shared_module

	implicit none
   
	integer i, j, k, m, counter
	integer dim_num, datatype, cur_update_width_num
	integer k_dim, m_dim

	integer update_width
	integer, optional :: change_sign_on_bipolar
	real (kind = 4), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width, 1:cmp % m_size) :: DIM4_t
	real (kind = 4), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width) :: DIM3_t
	real (kind = 4), optional, dimension(               cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width) :: DIM2_t

!===========================
! We exchange (kind = 4) now
!===========================
	datatype = 1

	if(update_width == comp_halo_wide) then
		cur_update_width_num = comp_halo_stencil+1
	else
		cur_update_width_num = update_width
	end if
      
!===============================
! Make 4D arrays from input data
!===============================
	if(PRESENT(DIM4_t)) then

		dim_num = 4
		k_dim = cmp % k_size
		m_dim = cmp % m_size

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(i,j,k,m) = DIM4_t(k,i,j,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_t)) then

		dim_num = 3
		k_dim = cmp % k_size
		m_dim = 1

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(i,j,k,1) = DIM3_t(k,i,j)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_t)) then

		dim_num = 2
		k_dim = 1
		m_dim = 1

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(i,j,1,1) = DIM2_t(i,j)
		end do
		end do
	end if

!=========================================================
! Create sending-arrays for west-east(+bsc) case 
!=========================================================
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south,cmp % north
		do i = 1, update_width+1
			vect_west_send4(counter) = t4(cmp % west+i-1,j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south,cmp % north
		do i = 1, update_width
			vect_west_send4(counter) = t4(cmp % west+i-1,j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_east_send4(counter) = t4(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_t_lon(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_t_lon(:, dim_num, datatype, cur_update_width_num), status_v , ierr)

!=========================================================
! Copy received belts to work arrays
!=========================================================
	if(cmp % west > 1 .OR. comp_i_cycle) then 
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			t4(cmp % west-update_width-1+i,j,k,m) = vect_west_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then
	
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width+1
			t4(cmp % east+i,j,k,m) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
		
	else if(cmp % east < cmp % i_size .OR. comp_i_cycle) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			t4(cmp % east+i,j,k,m) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

!=========================================================
! Now we have belts for west and east and we can get wide
! halo belts for south and north case(without diagonal)
!=========================================================

!=========================================================
! Create sending-arrays for wide south-north(+bsc) case 
!=========================================================	
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = 1, update_width
	do i = cmp % west - update_width, cmp % east + update_width
		vect_south_send4(counter) = t4(i,cmp % south+j-1,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do

	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then
	
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width+1, cmp % east + update_width+1
			vect_north_send4(counter) = t4(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send4(counter) = t4(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do	
	end if
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_t_lat(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_t_lat(:, dim_num, datatype, cur_update_width_num), status_t , ierr)
	
!=========================================================
! Receive wide south-north belts 
!=========================================================
	if(cmp % south > 1) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(i,cmp % south-update_width-1+j,k,m) = vect_south_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do

	end if

	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	
	else if(cmp % north < cmp % j_size) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t4(i,cmp % north+j,k,m) = vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do

	end if

!===================================================================
! Now we must return changed arrays to parameters of called function
!===================================================================
	if(PRESENT(DIM4_t)) then

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM4_t(k,i,j,m) = t4(i,j,k,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_t)) then

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM3_t(k,i,j) = t4(i,j,k,1)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_t)) then

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM2_t(i,j) = t4(i,j,1,1)
		end do
		end do
	end if

	return
	end subroutine comp_halo_update_T4

!================================================================================================================================================================================


	subroutine comp_halo_update_T8 (DIM4_t,				&
					 DIM3_t,				&
					 DIM2_t,				&
					 update_width, change_sign_on_bipolar)


	use comp_module
	use shared_module

	implicit none
   
	integer i, j, k, m, counter
	integer dim_num, datatype, cur_update_width_num
	integer k_dim, m_dim

	integer update_width
	integer, optional :: change_sign_on_bipolar	
	real (kind = 8), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width, 1:cmp % m_size) :: DIM4_t
	real (kind = 8), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width) :: DIM3_t
	real (kind = 8), optional, dimension(               cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width) :: DIM2_t

!===========================
! We exchange (kind = 8) now
!===========================
	datatype = 2

	if(update_width == comp_halo_wide) then
		cur_update_width_num = comp_halo_stencil+1
	else
		cur_update_width_num = update_width
	end if
      
!===============================
! Make 4D arrays from input data
!===============================
	if(PRESENT(DIM4_t)) then

		dim_num = 4
		k_dim = cmp % k_size
		m_dim = cmp % m_size

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(i,j,k,m) = DIM4_t(k,i,j,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_t)) then

		dim_num = 3
		k_dim = cmp % k_size
		m_dim = 1

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(i,j,k,1) = DIM3_t(k,i,j)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_t)) then

		dim_num = 2
		k_dim = 1
		m_dim = 1

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(i,j,1,1) = DIM2_t(i,j)
		end do
		end do
	end if

!=========================================================
! Create sending-arrays for west-east(+bsc) case 
!=========================================================
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south,cmp % north
		do i = 1, update_width+1
			vect_west_send8(counter) = t8(cmp % west+i-1,j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south,cmp % north
		do i = 1, update_width
			vect_west_send8(counter) = t8(cmp % west+i-1,j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_east_send8(counter) = t8(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_t_lon(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_t_lon(:, dim_num, datatype, cur_update_width_num), status_v , ierr)

!=========================================================
! Copy received belts to work arrays
!=========================================================
	if(cmp % west > 1 .OR. comp_i_cycle) then 
	
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			t8(cmp % west-update_width-1+i,j,k,m) = vect_west_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then
	
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width+1
			t8(cmp % east+i,j,k,m) = vect_east_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
		
	else if(cmp % east < cmp % i_size .OR. comp_i_cycle) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			t8(cmp % east+i,j,k,m) = vect_east_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

!=========================================================
! Now we have belts for west and east and we can get wide
! halo belts for south and north case(without diagonal)
!=========================================================

!=========================================================
! Create sending-arrays for wide south-north(+bsc) case 
!=========================================================
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = 1, update_width
	do i = cmp % west - update_width, cmp % east + update_width
		vect_south_send8(counter) = t8(i,cmp % south+j-1,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
		
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then
	
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width+1, cmp % east + update_width+1
			vect_north_send8(counter) = t8(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send8(counter) = t8(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do	
	end if
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_t_lat(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_t_lat(:, dim_num, datatype, cur_update_width_num), status_t , ierr)
	
!=========================================================
! Receive wide south-north belts 
!=========================================================
	if(cmp % south > 1) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(i,cmp % south-update_width-1+j,k,m) = vect_south_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do

	end if

	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	
	else if(cmp % north < cmp % j_size) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			t8(i,cmp % north+j,k,m) = vect_north_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do

	end if

!===================================================================
! Now we must return changed arrays to parameters of called function
!===================================================================
	if(PRESENT(DIM4_t)) then

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM4_t(k,i,j,m) = t8(i,j,k,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_t)) then

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM3_t(k,i,j) = t8(i,j,k,1)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_t)) then

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM2_t(i,j) = t8(i,j,1,1)
		end do
		end do
	end if

	return
	end subroutine comp_halo_update_T8

	
!=====================================================================================================================================================================

!================================================================
! These routine performs update of wide halo belts of calculating 
! domain in V points(for 2 V-cell functions)
!
! Types of input data:
! -- kind: 4, 8
! -- dims: 2D, 3D, 4D
! -- update_width: [1,comp_stencil] and [comp_wide]
!
! Use manual for full procedure description.
!================================================================

	subroutine comp_halo_update_V4 (DIM4_u, DIM4_v,			&
					 DIM3_u, DIM3_v,			&
					 DIM2_u, DIM2_v,			&
					 update_width, change_sign_on_bipolar)


	use comp_module
	use shared_module

	implicit none
   
	integer i, j, k, m, counter
	integer dim_num, datatype, cur_update_width_num
	integer k_dim, m_dim

	integer update_width
	integer, optional :: change_sign_on_bipolar
	real (kind = 4), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width, 1:cmp % m_size) :: DIM4_u, DIM4_v
	real (kind = 4), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width)   		   :: DIM3_u, DIM3_v
	real (kind = 4), optional, dimension(		    cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width)     	   :: DIM2_u, DIM2_v

!===========================
! We exchange (kind = 4) now
!===========================
	datatype = 1

	if(update_width == comp_halo_wide) then
		cur_update_width_num = comp_halo_stencil+1
	else
		cur_update_width_num = update_width
	end if
      
!===============================
! Make 4D arrays from input data
!===============================
	if(PRESENT(DIM4_u) .AND. PRESENT(DIM4_v)) then

		dim_num = 4
		k_dim = cmp % k_size
		m_dim = cmp % m_size

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(i,j,k,m) = DIM4_u(k,i,j,m)
			v4(i,j,k,m) = DIM4_v(k,i,j,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_u) .AND. PRESENT(DIM3_v)) then

		dim_num = 3
		k_dim = cmp % k_size
		m_dim = 1

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(i,j,k,1) = DIM3_u(k,i,j)
			v4(i,j,k,1) = DIM3_v(k,i,j)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_u) .AND. PRESENT(DIM2_v)) then

		dim_num = 2
		k_dim = 1
		m_dim = 1

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(i,j,1,1) = DIM2_u(i,j)
			v4(i,j,1,1) = DIM2_v(i,j)
		end do
		end do
	end if

!=========================================================
! Create sending-arrays for west-east(+bsc) case 
!=========================================================	
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_west_send4(counter) = u4(cmp % west+i-1,j,k,m)
		counter = counter + 1
		vect_west_send4(counter) = v4(cmp % west+i-1,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do

	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_east_send4(counter) = u4(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
		vect_east_send4(counter) = v4(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_v_lon(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_v_lon(:, dim_num, datatype, cur_update_width_num), status_v , ierr)

!=========================================================
! Copy received belts to work arrays
!=========================================================
	if(cmp % west > 1 .OR. comp_i_cycle) then 
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			u4(cmp % west-update_width-1+i,j,k,m) = vect_west_recv4(counter)
			counter = counter + 1
			v4(cmp % west-update_width-1+i,j,k,m) = vect_west_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

	if(cmp % east < cmp % i_size .OR. comp_i_cycle) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			u4(cmp % east+i,j,k,m) = vect_east_recv4(counter)
			counter = counter + 1
			v4(cmp % east+i,j,k,m) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if	

!=========================================================
! Now we have belts for west and east and we can get wide
! halo belts for south and north case(without diagonal)
!=========================================================

!=========================================================
! Create sending-arrays for wide south-north(+bsc) case 
!=========================================================
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = 1, update_width
	do i = cmp % west - update_width, cmp % east + update_width
		vect_south_send4(counter) = u4(i,cmp % south+j-1,k,m)
		counter = counter + 1
		vect_south_send4(counter) = v4(i,cmp % south+j-1,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send4(counter) = u4(i,cmp % north-update_width-1+j,k,m)
			counter = counter + 1
			vect_north_send4(counter) = v4(i,cmp % north-update_width-1+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do	
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send4(counter) = u4(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
			vect_north_send4(counter) = v4(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_v_lat(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_v_lat(:, dim_num, datatype, cur_update_width_num), status_v , ierr)
	
!=========================================================
! Receive wide south-north belts 
!=========================================================
	if(cmp % south > 1) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(i,cmp % south-update_width-1+j,k,m) = vect_south_recv4(counter)
			counter = counter + 1
			v4(i,cmp % south-update_width-1+j,k,m) = vect_south_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv4(counter)
			counter = counter + 1
			v4(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
		
	else if(cmp % north < cmp % j_size) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u4(i,cmp % north+j,k,m) = vect_north_recv4(counter)
			counter = counter + 1
			v4(i,cmp % north+j,k,m) = vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
!===================================================================
! Now we must return worked arrays to parameters of called function
!===================================================================
	if(PRESENT(DIM4_u) .AND. PRESENT(DIM4_v)) then

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM4_u(k,i,j,m) = u4(i,j,k,m)
			DIM4_v(k,i,j,m) = v4(i,j,k,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_u) .AND. PRESENT(DIM3_v)) then

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM3_u(k,i,j) = u4(i,j,k,1)
			DIM3_v(k,i,j) = v4(i,j,k,1)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_u) .AND. PRESENT(DIM2_v)) then

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM2_u(i,j) = u4(i,j,1,1)
			DIM2_v(i,j) = v4(i,j,1,1)
		end do
		end do
	end if

	return
	end subroutine comp_halo_update_V4

!=================================================================================================================================================================================================

!================================================================
! These routine performs update of wide halo belts of calculating 
! domain in V points(for 2 V-cell functions)
!
! Types of input data:
! -- kind: 4, 8
! -- dims: 2D, 3D, 4D
! -- update_width: [1,comp_stencil] and [comp_wide]
!
! Use manual for full procedure description.
!================================================================

	subroutine comp_halo_update_V8 (DIM4_u, DIM4_v,			&
					 DIM3_u, DIM3_v,			&
					 DIM2_u, DIM2_v,			&
					 update_width, change_sign_on_bipolar)


	use comp_module
	use shared_module

	implicit none
   
	integer i, j, k, m, counter
	integer dim_num, datatype, cur_update_width_num
	integer k_dim, m_dim

	integer update_width
	integer, optional :: change_sign_on_bipolar
	real (kind = 8), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width, 1:cmp % m_size) :: DIM4_u, DIM4_v
	real (kind = 8), optional, dimension(1:cmp % k_size, cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width)   		   :: DIM3_u, DIM3_v
	real (kind = 8), optional, dimension(		    cmp % west - update_width:cmp % east + update_width, cmp % south - update_width:cmp % north + update_width)     	   :: DIM2_u, DIM2_v

!===========================
! We exchange (kind = 8) now
!===========================
	datatype = 2

	if(update_width == comp_halo_wide) then
		cur_update_width_num = comp_halo_stencil+1
	else
		cur_update_width_num = update_width
	end if
      
!===============================
! Make 4D arrays from input data
!===============================
	if(PRESENT(DIM4_u) .AND. PRESENT(DIM4_v)) then

		dim_num = 4
		k_dim = cmp % k_size
		m_dim = cmp % m_size

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(i,j,k,m) = DIM4_u(k,i,j,m)
			v8(i,j,k,m) = DIM4_v(k,i,j,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_u) .AND. PRESENT(DIM3_v)) then

		dim_num = 3
		k_dim = cmp % k_size
		m_dim = 1

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(i,j,k,1) = DIM3_u(k,i,j)
			v8(i,j,k,1) = DIM3_v(k,i,j)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_u) .AND. PRESENT(DIM2_v)) then

		dim_num = 2
		k_dim = 1
		m_dim = 1

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(i,j,1,1) = DIM2_u(i,j)
			v8(i,j,1,1) = DIM2_v(i,j)
		end do
		end do
	end if

!=========================================================
! Create sending-arrays for west-east(+bsc) case 
!=========================================================	
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_west_send8(counter) = u8(cmp % west+i-1,j,k,m)
		counter = counter + 1
		vect_west_send8(counter) = v8(cmp % west+i-1,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do

	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = cmp % south,cmp % north
	do i = 1, update_width
		vect_east_send8(counter) = u8(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
		vect_east_send8(counter) = v8(cmp % east-update_width+i,j,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_v_lon(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_v_lon(:, dim_num, datatype, cur_update_width_num), status_v , ierr)

!=========================================================
! Copy received belts to work arrays
!=========================================================
	if(cmp % west > 1 .OR. comp_i_cycle) then 
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			u8(cmp % west-update_width-1+i,j,k,m) = vect_west_recv8(counter)
			counter = counter + 1
			v8(cmp % west-update_width-1+i,j,k,m) = vect_west_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

	if(cmp % east < cmp % i_size .OR. comp_i_cycle) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south, cmp % north
		do i = 1, update_width
			u8(cmp % east+i,j,k,m) = vect_east_recv8(counter)
			counter = counter + 1
			v8(cmp % east+i,j,k,m) = vect_east_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if	

!=========================================================
! Now we have belts for west and east and we can get wide
! halo belts for south and north case(without diagonal)
!=========================================================

!=========================================================
! Create sending-arrays for wide south-north(+bsc) case 
!=========================================================
	counter = 1
	do m = 1, m_dim
	do k = 1, k_dim
	do j = 1, update_width
	do i = cmp % west - update_width, cmp % east + update_width
		vect_south_send8(counter) = u8(i,cmp % south+j-1,k,m)
		counter = counter + 1
		vect_south_send8(counter) = v8(i,cmp % south+j-1,k,m)
		counter = counter + 1
	end do
	end do
	end do
	end do
	
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send8(counter) = u8(i,cmp % north-update_width-1+j,k,m)
			counter = counter + 1
			vect_north_send8(counter) = v8(i,cmp % north-update_width-1+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do	
	else
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			vect_north_send8(counter) = u8(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
			vect_north_send8(counter) = v8(i,cmp % north-update_width+j,k,m)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if

!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_v_lat(:, dim_num, datatype, cur_update_width_num) , ierr)
	call MPI_WAITALL  (4, request_v_lat(:, dim_num, datatype, cur_update_width_num), status_v , ierr)
	
!=========================================================
! Receive wide south-north belts 
!=========================================================
	if(cmp % south > 1) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(i,cmp % south-update_width-1+j,k,m) = vect_south_recv8(counter)
			counter = counter + 1
			v8(i,cmp % south-update_width-1+j,k,m) = vect_south_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
	if(cmp % north == cmp % j_size .AND. comp_bsc_grid) then

		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv8(counter)
			counter = counter + 1
			v8(cmp % west+(cmp % east-i),cmp % north+update_width+1-j,k,m) = change_sign_on_bipolar*vect_north_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
		
	else if(cmp % north < cmp % j_size) then
		
		counter = 1
		do m = 1, m_dim
		do k = 1, k_dim
		do j = 1, update_width
		do i = cmp % west - update_width, cmp % east + update_width
			u8(i,cmp % north+j,k,m) = vect_north_recv8(counter)
			counter = counter + 1
			v8(i,cmp % north+j,k,m) = vect_north_recv8(counter)
			counter = counter + 1
		end do
		end do
		end do
		end do
	end if
	
!===================================================================
! Now we must return worked arrays to parameters of called function
!===================================================================
	if(PRESENT(DIM4_u) .AND. PRESENT(DIM4_v)) then

		do m = 1, m_dim
		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM4_u(k,i,j,m) = u8(i,j,k,m)
			DIM4_v(k,i,j,m) = v8(i,j,k,m)
		end do
		end do
		end do
		end do
	end if

    	if(PRESENT(DIM3_u) .AND. PRESENT(DIM3_v)) then

		do k = 1, k_dim
		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM3_u(k,i,j) = u8(i,j,k,1)
			DIM3_v(k,i,j) = v8(i,j,k,1)
		end do
		end do
		end do
	end if

	if(PRESENT(DIM2_u) .AND. PRESENT(DIM2_v)) then

		do j = cmp % south - update_width, cmp % north + update_width
		do i = cmp % west - update_width, cmp % east + update_width
			DIM2_u(i,j) = u8(i,j,1,1)
			DIM2_v(i,j) = v8(i,j,1,1)
		end do
		end do
	end if

	return
	end subroutine comp_halo_update_V8

!=================================================================================================================================================================================================

	end module comp_halo_update_module


