
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

#define MEMORY_SIZE 300000

	module ga_module

	use shared_module
	use comp_module
	
	implicit none

#include "mafdecls.fh"
#include "global.fh"

	integer, parameter :: ACTIOM_PUT = 1
	integer, parameter :: ACTION_GET = 2

	! GA variables
	integer g_var
	integer pgroup_wrknst, pgroup_wrk, pgroup_nst
	integer ga_wrk, ga_nst

	! Information tables
	type(VectorType) :: ga_info(2)
	
	
	! Work variables
	logical status

	CONTAINS

!================================================================================================================================================
	
	subroutine ga_constructor(wrk, nst)
	
	integer, intent(in) :: wrk, nst

	integer i
	
	ga_wrk = wrk
	ga_nst = nst
	
	call ga_initialize()
	status = ma_init(MT_F_REAL, MEMORY_SIZE/(size_world), MEMORY_SIZE/(size_world))
		
 	pgroup_wrknst = ga_pgroup_create((/(/ (i, i = proc_first(wrk), proc_first(wrk)+run_np(wrk)-1) /), 	&
										(/ (i, i = proc_first(nst), proc_first(nst)+run_np(nst)-1) /) /), &
										run_np(wrk) + run_np(nst))
									
	pgroup_nst = ga_pgroup_create((/ 	(i, i = proc_first(nst), proc_first(nst)+run_np(nst)-1)	 /), 	&
										run_np(nst))
	
	end subroutine ga_constructor
	
!================================================================================================================================================

	integer function ga_init_wrknst_array()
		
	! Work vars
	integer, allocatable :: map(:)
	integer :: glob_i, glob_j, glob_k, inp, jnp
	integer, allocatable :: south(:), west(:)
	integer j
	integer g_a
	
	call ga_pgroup_set_default(pgroup_wrknst)
	
	! Create global variable
	g_var = ga_create_handle()
	call ga_set_array_name(g_var, "Global int var")
	call ga_set_data(g_var, 1, (/ BUFFER_LEN**2 /), MT_F_INT)
	status =  ga_allocate(g_var)

	! Get global size of 'wrk' side
	if(proc_in(rank_world, ga_wrk)) call nga_put(g_var, 1, 5, (/ cmp % i_size, cmp % j_size, cmp % k_size, cmp % inp, cmp % jnp/), (/ 1 /))
	call ga_sync()
	call nga_get(g_var, 1, 1, glob_i, (/ 1 /))
	call nga_get(g_var, 2, 2, glob_j, (/ 1 /))
	call nga_get(g_var, 3, 3, glob_k, (/ 1 /))
	call nga_get(g_var, 4, 4, inp, (/ 1 /))
	call nga_get(g_var, 5, 5, jnp, (/ 1 /))

	! Get decomposition of 'wrk' side
	allocate(west(run_np(ga_wrk)))	
	allocate(south(run_np(ga_wrk)))
	
	if(proc_in(rank_world, ga_wrk)) then
		call MPI_ALLGATHER(cmp % west,  1, MPI_INTEGER, west,  1, MPI_INTEGER, comm_local,ierr)	
		call MPI_ALLGATHER(cmp % south, 1, MPI_INTEGER, south, 1, MPI_INTEGER, comm_local,ierr)	
		
		call nga_put(g_var, 1, size_local*2, (/ west, south /), (/ 1 /))
	end if
	call ga_sync()
	call nga_get(g_var, 1, run_np(ga_wrk), west, (/ 1 /))
	call nga_get(g_var, run_np(ga_wrk)+1, run_np(ga_wrk)*2, south, (/ 1 /))
	
	! Set global array properties
	g_a = ga_create_handle()
	call ga_set_array_name(g_a, "Global array <>")
	call ga_set_restricted_range(g_a, 0, run_np(ga_wrk)-1)	
	call ga_set_data(g_a, 3, (/ glob_k, glob_i, glob_j /), MT_F_REAL)

  	allocate(map(1 + inp + jnp))
    map(1) = 1
  	map(1+1:inp+1) = west(1:inp)
  	do j = 1, jnp
		map(1+inp+j) = south((j-1)*inp + 1)
  	end do
  	call ga_set_irreg_distr(g_a, map, (/ 1,inp,jnp /) )

	status =  ga_allocate(g_a)
	ga_init_wrknst_array = g_a
	
	if(ga_nodeid() == 0) call ga_summarize(g_a)
		
	end function ga_init_wrknst_array

!================================================================================================================================================

	subroutine ga_reg_array(act, DIM3_arr, period)

	integer act, period
	real (kind=4), dimension(:,:,:), target, optional :: DIM3_arr
	
	integer cur
	
! 	ga_info(act) % ar_size = ga_info(act) % ar_size + 1
! 	cur = ga_info(act) % ar_size 
! 	cmp % inf(act) % ar(cur) % p_3D => DIM3_arr
! 	cmp % inf(act) % ar(cur) % period = period
! 	
! 	ga_temp = ga_init_array()
	
	end subroutine ga_reg_array

!================================================================================================================================================
	
	integer function ga_init_nst_array()

	integer, allocatable :: map(:)	
	integer, allocatable :: south(:), west(:)
	integer ga_nst
	logical status
	integer j
	
	ga_nst = ga_create_handle()
	call ga_set_array_name(ga_nst, "Global array <nst>")
	call ga_set_data(ga_nst, 3, (/ cmp % k_size, cmp % i_size, cmp % j_size /), MT_F_REAL)

	allocate(west (size_local))
	allocate(south(size_local))	
	call MPI_ALLGATHER(cmp % west,  1, MPI_INTEGER, west,  1, MPI_INTEGER, comm_local,ierr)	
	call MPI_ALLGATHER(cmp % south, 1, MPI_INTEGER, south, 1, MPI_INTEGER, comm_local,ierr)	

  	allocate(map(1 + cmp % inp + cmp % jnp))
    map(1) = 1
  	map(1+1:cmp % inp+1) = west(1:cmp % inp)
  	do j = 1, cmp % jnp
		map(1+cmp % inp+j) = south((j-1)*cmp % inp + 1)
  	end do
  	call ga_set_irreg_distr(ga_nst, map, (/ 1,cmp % inp,cmp % jnp /) )

	status =  ga_allocate(ga_nst)
	ga_init_nst_array = ga_nst
		
	end function ga_init_nst_array	
!================================================================================================================================================



	
!================================================================================================================================================
	
	
	subroutine ga_put_array(ga, DIM3_arr, lo, hi)
	
	integer ga, lo(3), hi(3)
	real (kind=4), dimension(:,:,:), target, optional :: DIM3_arr
	
	call nga_put(ga, lo, hi, DIM3_arr, (/SIZE(DIM3_arr,1), SIZE(DIM3_arr,2)/))
	call ga_sync()	
! 	
	end subroutine ga_put_array

!================================================================================================================================================
	
	subroutine ga_get_array(ga, DIM3_arr, lo, hi)
	
	integer ga, lo(3), hi(3)
	real (kind=4), dimension(:,:,:), target, optional :: DIM3_arr
	
	call ga_sync()
	call nga_get(ga, lo, hi, DIM3_arr, (/SIZE(DIM3_arr,1), SIZE(DIM3_arr,2)/))
	
	end subroutine ga_get_array
	
!================================================================================================================================================
































!================================================================================================================================================

	
	subroutine ga_destructor()

 	call ga_pgroup_set_default(ga_pgroup_get_world())
	call ga_terminate()		
	
	end subroutine ga_destructor

!================================================================================================================================================

	end module ga_module