
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!====================================================
! Define constant values for component-coupler system
!====================================================

	module shared_const_module

	implicit none
	save

!================
! Component names
!================
	integer, parameter :: NUM_OF_COMP = 6
	integer, parameter :: COUPLER = 0, COMP_OCN = 1, COMP_ATM = 2, COMP_ICE = 3, COMP_LND = 4, COMP_SEA = 5, COMP_DIA = 6

!=========================
! Component RW information
!=========================
	integer, parameter ::ACTION_NUM_FIELDS			= 6,	&
			      ACTION_SAVE_CP			= 1,	&
			      ACTION_SAVE_DG			= 2,	&
			      ACTION_READ_CP			= 3,	&
			      ACTION_READ_IC			= 4,	&	
			      ACTION_READ_FD			= 5,	&
			      ACTION_SAVE_IT			= 6
	integer, parameter :: ACTION_MAPPING			= ACTION_NUM_FIELDS + 1

	integer, parameter :: INF_SR_NUM_FIELDS		= 2,	&
			      INF_SR_SEND			= 1,	&
			      INF_SR_RECV  			= 2	

	integer, parameter :: 	ARRAY_TYPE_NUM_FIELDS		= 5,	&
				ARRAY_TYPE_4D			= 1,	&
				ARRAY_TYPE_3D			= 2,	&
				ARRAY_TYPE_2D			= 3,	&
				ARRAY_TYPE_2D_KIND8		= 4,	&
				ARRAY_TYPE_1D			= 5
			      
!===============
! Time variables
!===============
	integer, parameter :: 	VAR_INFO_NUM_FIELDS 		= 2,	&
				VAR_INFO_REAL_TIME 		= 1,	&
				VAR_INFO_MODEL_TIME 		= 2


!============================
! Coupler mapping information
!============================
	integer, parameter :: MAP_NUM_FIELDS			= 8,	&
			      MAP_TYPE				= 1,	&	! source or destination mapping type
			      MAP_NUM_LINKS			= 2,	&	! number of my links in each worker
			      MAP_NUM_LINKS_UN_SRC		= 3,	&	! number of my unique links in each worker
			      MAP_NUM_LINKS_FOR_SEND_UN_SRC	= 4,	&	! number of unique links, which another workers require from me
			      MAP_NUM_LINKS_PART_DST		= 5,	&	! number of my partial sums in each worker
			      MAP_NUM_LINKS_FOR_SEND_PART_DST	= 6,	&	! number of partial sums, which another workers require from me
			      MAP_NUM_FUNC 			= 7,	&	! number of interpolated functions
			      MAP_PERIOD_MIN			= 8		! hours between mapping

	integer, parameter :: MAP_NULL 			= 0,	&	! tag for no-need-cells from worker
			      MAP_SRC_TYPE 			= 1,	&	! tag for src type mapping
			      MAP_DST_TYPE 			= 2		! tag for dst type mapping

!=================================================================
! Send-recv parameters from component to coupler on initialization
!=================================================================
	integer, parameter :: NAME_LEN = 50
	integer, parameter :: BUFFER_LEN = 100

!===================================================================
! Structure and time parameters of message for gather-distrib system
!===================================================================
	real(kind = 4), parameter :: MISSING_VALUE = 99999.

	integer, parameter :: MYNULL = -1	
!-------
!	add new parameters. used for masking region in plots (ncview, PYTHON). 
!-------
	real 	(kind = 4), 	parameter :: missval = -9e33
!	real 	(kind = 4), 	parameter :: misValRl4 = -99999.
!	integer	(kind = 4), 	parameter :: misValInt = -99999	

!================
! Run information
!================

	character(LEN=*), parameter :: run_data_folder = 'data/'
	character(LEN=*), parameter :: run_solution_folder = 'solution/'

	end module shared_const_module





