
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!======================
! Contains common utils
!======================

	module shared_utils_module
	
	use shared_const_module
! 	
	implicit none
		
	contains
!================================================================================================================================================

!=========================
! Convert number to string
!=========================
	
	character(NAME_LEN) function I2S(cur_int, out_len)

	integer :: cur_int
	integer, optional :: out_len 
	integer i
	character(LEN = 10) :: str
	
	write(str(1:10),'(i10.10)') cur_int
	
	do i = 1, 10
		if(str(i:i) /= '0') EXIT
		str(i:i) = ''
	end do

	str = ADJUSTL(str)
	
	if(PRESENT(out_len)) then
		do while(LEN(TRIM(str)) < out_len)
			str = '0'//str
		end do
		str = str(1:out_len)
	end if
	
	I2S = str
	
	end function I2S
	
!=========================
! Convert string to number
!=========================
!================================================================================================================================================

	integer function S2I(cur_str)

	character(*) :: cur_str
	
	if(cur_str == '') then 
		S2I = 0
	else
		read(cur_str,*) S2I
	end if
	
	end function S2I
!================================================================================================================================================
	
!========================
! Simple sort of 1D array
!========================

	subroutine SortBubble(vect_in, vect_out, vect_size)

	integer vect_size
	integer, dimension(vect_size) :: vect_in, vect_out
	integer i, cop
	integer :: was_exch

	was_exch = 1
	vect_out = vect_in

	do while(was_exch == 1)

		was_exch = 0
		do i = 1, vect_size - 1
			if (vect_out(i) > vect_out(i+1)) then
				cop = vect_out(i+1)
				vect_out(i+1) = vect_out(i)
				vect_out(i) = cop
				was_exch = 1
			end if
		end do
	end do

	end subroutine SortBubble
!================================================================================================================================================

	subroutine Merge_(A,NA,B,NB,C,NC)
	
	integer, intent(in) :: NA,NB,NC         ! Normal usage: NA+NB = NC
	integer, intent(in out) :: A(NA)        ! B overlays C(NA+1:NC)
	integer, intent(in)     :: B(NB)
	integer, intent(in out) :: C(NC)
	
	integer :: I,J,K
	
	I = 1; J = 1; K = 1;
	do while(I <= NA .and. J <= NB)
	if (A(I) <= B(J)) then
		C(K) = A(I)
		I = I+1
	else
		C(K) = B(J)
		J = J+1
	endif
	K = K + 1
	enddo
	do while (I <= NA)
	C(K) = A(I)
	I = I + 1
	K = K + 1
	enddo
	return
	
	end subroutine Merge_
	
	recursive subroutine SortMerge(A,N,T)
	
	integer, intent(in) :: N
	integer, dimension(N), intent(in out) :: A
	integer, dimension((N+1)/2), intent (out) :: T
	
	integer :: NA,NB,V
	
	if (N < 2) return
	if (N == 2) then
	if (A(1) > A(2)) then
		V = A(1)
		A(1) = A(2)
		A(2) = V
	endif
	return
	endif      
	NA=(N+1)/2
	NB=N-NA
	
	call SortMerge(A,NA,T)
	call SortMerge(A(NA+1),NB,T)
	
	if (A(NA) > A(NA+1)) then
	T(1:NA)=A(1:NA)
	call Merge_(T,NA,A(NA+1),NB,A,N)
	endif
	return
	
	end subroutine SortMerge
!================================================================================================================================================
!==============================
! Binary search in sorted array
!==============================

	function binarySearch_I (a, value)
	
	integer                  :: binarySearch_I
	integer, intent(in), target :: a(:)
	integer, intent(in)         :: value
	integer, pointer            :: p(:)
	integer                  :: mid, offset
	
	p => a
	binarySearch_I = 0
	offset = 0
	do while (size(p) > 0)
		mid = size(p)/2 + 1
		if (p(mid) > value) then
		p => p(:mid-1)
		else if (p(mid) < value) then
		offset = offset + mid
		p => p(mid+1:)
		else
		binarySearch_I = offset + mid    ! SUCCESS!!
		return
		end if
	end do
	
	end function binarySearch_I

!===============================================================================================================================================
!=================================
! Simple search 
!=================================

	integer function INDEX_OF_ELEM_R(elem, vect)

	real(kind=4) :: elem, vect(:)
	
	INDEX_OF_ELEM_R = 1
	
	do while(elem /= vect(INDEX_OF_ELEM_R))
		INDEX_OF_ELEM_R = INDEX_OF_ELEM_R + 1
	end do

	end function INDEX_OF_ELEM_R

!================================================================================================================================================
	
!==================
! Return event name
!==================

	character(NAME_LEN) function event_type_name(ev)

	integer ev

	select case(ev)
		case(ACTION_SAVE_CP) 
			event_type_name = 'Save CP'
		case(ACTION_SAVE_DG) 
			event_type_name = 'Save DG'						
		case(ACTION_READ_CP) 
			event_type_name = 'Read CP'
		case(ACTION_READ_IC) 
			event_type_name = 'Read IC'
		case(ACTION_READ_FD) 
			event_type_name = 'Read FD'
		case(ACTION_MAPPING) 
			event_type_name = 'Mapping'
		case(ACTION_SAVE_IT) 
			event_type_name = 'Save IT'
			
	end select
    
	end function event_type_name
!================================================================================================================================================
	
	subroutine fakeKlMl(t,comp_kl,comp_ml,cur_kl,cur_ml)
	
		integer t, comp_kl, comp_ml, cur_kl, cur_ml
		
		if (t == ARRAY_TYPE_4D) then
			cur_ml = comp_ml
			cur_kl = comp_kl
		end if
		
		if (t == ARRAY_TYPE_3D) then
			cur_ml = 1
			cur_kl = comp_kl
		end if
		
		if (t == ARRAY_TYPE_2D) then
			cur_ml = 1
			cur_kl = 1
		end if	

	end subroutine fakeKlMl
	
!================================================================================================================================================
	!
	!	Here the user can define additional (aperiodic) moments for raising standard periodic events
	!
	logical function is_aperiodic_moment(minute, act)
	
		integer minute,act
	
		is_aperiodic_moment = .FALSE.
		return

		if(act==ACTION_SAVE_CP) then
			if(	MOD(minute,24*60) == 0 			.and.	&
				MOD(minute,24*60*365)>=24*60*302 	.and.	&
				MOD(minute,24*60*365)<24*60*312) is_aperiodic_moment = .TRUE.
		end if
				
	end function is_aperiodic_moment
		
!================================================================================================================================================
	
	
	
	
	
	
	
	
	end module shared_utils_module
	
	