
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!============================================================================
! This serial program try to decompose domains between components and coupler
!============================================================================

	program test_decomp
	
	use shared_module
	use shared_utils_module
	use comp_module	
	
	character(NAME_LEN) :: arg
	integer :: counter

	run_phase = 'SHR'
	run_log_file_num = 6
	rank_world = 0
	
	if(COMMAND_ARGUMENT_COUNT() <= 2 .OR. MOD(COMMAND_ARGUMENT_COUNT()-2, 5) /= 0) call shared_error("Usage: ./test_decomp CPL 2 OCN 360 180 2D 100 ATM 640 400 1D 10")
	run_comps =  (COMMAND_ARGUMENT_COUNT()-2)/5
	
	! Get coupler size
	call GET_COMMAND_ARGUMENT(2, arg)
	read(arg, *) run_np(COUPLER)
	
	write(*,*)
	! Read component information and try to decompose
	do counter = 1, run_comps
		call GET_COMMAND_ARGUMENT(2 + (counter-1)*5 + 1, cmp % name)
		call GET_COMMAND_ARGUMENT(2 + (counter-1)*5 + 2, arg)
		read(arg, *) cmp % i_size
		call GET_COMMAND_ARGUMENT(2 + (counter-1)*5 + 3, arg)
		read(arg, *) cmp % j_size
		call GET_COMMAND_ARGUMENT(2 + (counter-1)*5 + 4, arg)
		read(arg, *) cmp % decomp_type
		call GET_COMMAND_ARGUMENT(2 + (counter-1)*5 + 5, arg)
		read(arg, *) cmp % proc_count

		call comp_decompose
		
		write(*,*) "Decomposition of component "//TRIM(cmp % name)//" has been done successfully."
		write(*,*) "Process mesh: "//TRIM(I2S(cmp % inp))//"x"//TRIM(I2S(cmp % jnp))
		if(MOD(cmp % j_size, cmp % jnp) == 0) then
			write(*,*) "Local domain size (equal domains): "//TRIM(I2S(cmp % i_loc_size))//"x"//TRIM(I2S(cmp % j_size/cmp % jnp))
		else
			write(*,*) "Local domain size (unequal domains, maximum): "//TRIM(I2S(cmp % i_loc_size))//"x"//TRIM(I2S(cmp % j_size/(cmp % jnp-1)))
		end if
		write(*,*)
		
	end do
	
	end program test_decomp