
!--------------------------------------
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!--------------------------------------

!======================================================================
! Compare global mapping with parallel: insert this routine after c_map
!======================================================================

	subroutine test_global_map

	use shared_module
	use shared_utils_module	
	use c_module
	use c_rw_module	
	use netcdf
	use c_gd_module

	implicit none

	! netCDF variables
	integer, parameter :: 	NC 		= 1,	&
				NUM_LINK	= 2, 	&
				NUM_WT		= 3, 	&
				SRC_ADDR	= 4, 	&
				DST_ADDR	= 5, 	&
				WTS		= 6
	integer netids(NUM_OF_COMP,NUM_OF_COMP, 6)
	character(100) :: dimname
	
	! Mapping information
	integer num_links, num_wts
	integer, allocatable ::	src_grid_addr(:), dst_grid_addr(:)
	real (kind = 8), allocatable :: wts_map(:,:)

	! Global mapping data    
	real(kind=4) global_src(c_inf(c_events(c_ev_num) % src) % il*c_inf(c_events(c_ev_num) % src) % jl, c_map_info(c_events(c_ev_num) % src, c_events(c_ev_num) % dst, 1, MAP_NUM_FUNC))
	real(kind=4) dst_temp(c_inf(c_events(c_ev_num) % dst) % il*c_inf(c_events(c_ev_num) % dst) % jl, c_map_info(c_events(c_ev_num) % src, c_events(c_ev_num) % dst, 1, MAP_NUM_FUNC))

	! Work variables
	character(100) file_name
	logical file_exists
	integer n, func, i, j, counter, r
	integer,parameter :: test_out = 0
	integer comp_src, comp_dst
	
	comp_src = c_events(c_ev_num) % src
	comp_dst = c_events(c_ev_num) % dst
	
!===================================================
! Open netCDF file, read mapping sizes and mesh data
!===================================================
	
	! Built name of file
	file_name = 	TRIM(run_data_folder)							//	&
				TRIM(c_inf(comp_src) % name)				//	&
				'_'								//	&
				TRIM(I2S(c_inf(comp_src) % il))//	&
				'x'								//	&
				TRIM(I2S(c_inf(comp_src) % jl))//	&
				'_to_'								//	&
				TRIM(c_inf(comp_dst) % name)				//	&
				'_'								//	&
				TRIM(I2S(c_inf(comp_dst) % il))	//	&
				'x'								//	&
				TRIM(I2S(c_inf(comp_dst) % jl))	//	&
				".nc"
				
	! Open netCDF file
	INQUIRE(file = file_name, exist = file_exists)
	if(file_exists) then

		call nc_rc(NF90_OPEN(TRIM(file_name), NF90_NOWRITE, netids(comp_src, comp_dst, NC)))
		
		call nc_rc(NF90_INQ_DIMID(netids(comp_src, comp_dst, NC), "num_links", netids(comp_src, comp_dst, NUM_LINK)))
		call nc_rc(NF90_INQ_DIMID(netids(comp_src, comp_dst, NC), "num_wgts", netids(comp_src, comp_dst, NUM_WT)))
		call nc_rc(NF90_INQ_VARID(netids(comp_src, comp_dst, NC), "src_address", netids(comp_src, comp_dst, SRC_ADDR)))
		call nc_rc(NF90_INQ_VARID(netids(comp_src, comp_dst, NC), "dst_address", netids(comp_src, comp_dst, DST_ADDR)))
		call nc_rc(NF90_INQ_VARID(netids(comp_src, comp_dst, NC), "remap_matrix", netids(comp_src, comp_dst, WTS)))
	else
		call shared_error("Cant open interpolation weights file - "//TRIM(file_name))
	end if	
	
 	call nc_rc( NF90_INQUIRE_DIMENSION(netids(comp_src, comp_dst, NC), netids(comp_src, comp_dst, NUM_LINK), dimname, num_links))
 	call nc_rc( NF90_INQUIRE_DIMENSION(netids(comp_src, comp_dst, NC), netids(comp_src, comp_dst, NUM_WT), dimname, num_wts))
		
	! Read map data and close file
	allocate(src_grid_addr(num_links))
	allocate(dst_grid_addr(num_links))
	allocate(wts_map(num_wts, num_links))

	call nc_rc( NF90_GET_VAR(netids(comp_src, comp_dst, NC), netids(comp_src, comp_dst, SRC_ADDR), src_grid_addr))
	call nc_rc( NF90_GET_VAR(netids(comp_src, comp_dst, NC), netids(comp_src, comp_dst, DST_ADDR), dst_grid_addr))
	call nc_rc( NF90_GET_VAR(netids(comp_src, comp_dst, NC), netids(comp_src, comp_dst, WTS), wts_map))
	
	call nc_rc( NF90_CLOSE(netids(comp_src, comp_dst, NC)))

!================
! Make test field
!================
	global_src = 0.
	do func = 1, c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)
		counter = 1
		do i = 1, c_inf(comp_src) % il*c_inf(comp_src) % jl
			global_src(i,func) = REAL(func) + 0.001*REAL(counter)
			counter = counter + 1
		end do
	end do

	if(rank_local == 0 .AND. test_out == 1) then
		write(*,*) '================================== Global test field:', c_inf(comp_src) % name
		do func = 1, c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)
			write(*,*) ' 2D function number: ', func
			do j =  c_inf(comp_src) % jl, 1, -1
				write(*,'(1000f11.7)')(global_src((j-1)*c_inf(comp_src) % il + i, func), i = 1, c_inf(comp_src) % il )
			end do
			write(*,*) ''
		end do
	end if

!==========================
! Test global interpolation
!==========================
	dst_temp = 0.

	do func = 1, c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)
	do n = 1, num_links
		dst_temp(dst_grid_addr(n),func) = dst_temp(dst_grid_addr(n),func) + wts_map(1,n)*global_src(src_grid_addr(n),func)
	end do
	end do

	if(rank_local == 0 .AND. test_out == 1) then
		write(*,*) '================================== Global mapping result:', c_inf(comp_dst) % name
		do func = 1, c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)
			write(*,*) ' 2D function number: ', func
			do j =  c_inf(comp_dst) % jl, 1, -1
				write(*,'(1000f7.4)')(dst_temp((j-1)*c_inf(comp_dst) % il + i, func), i = 1, c_inf(comp_dst) % il )
			end do
			write(*,*) ''
		end do
	end if

!==================================================
! Check errors between global and parallel versions
!==================================================
	do r = 0, size_local
	if(rank_local == r) then
		write(*,*) '================================================================'
		write(*,*) 'Rank '//TRIM(I2S(rank_local,2))//' checks for mapping errors between: '//TRIM(c_inf(comp_src) % name)//" and "//TRIM(c_inf(comp_dst) % name)
		write(*,*) '================================================================'

		do func = 1, c_map_info(comp_src, comp_dst, 1, MAP_NUM_FUNC)
		do i = 1, c_inf(comp_dst) % il*c_inf(comp_dst) % wk(me) % stripe_count
			if(ABS(dst_temp((c_inf(comp_dst) % wk(me) % stripe_start-1)*c_inf(comp_dst) % il + 1 + i - 1, func) - stripe_2D(i, func)) > 0.001) then
				write(*,*) 'Error in mapping!'
				write(*,*) 'func:', func
				write(*,*) 'i:', i
				write(*,*) 'Points:', dst_temp((c_inf(comp_dst) % wk(me) % stripe_start-1)*c_inf(comp_dst) % il + 1 + i - 1, func), stripe_2D(i, func)
				write(*,*) 'Error:', ABS(dst_temp((c_inf(comp_dst) % wk(me) % stripe_start-1)*c_inf(comp_dst) % il + 1 + i - 1, func) - stripe_2D(i, func))
				stop
			end if
		end do
		end do
		call sleep(1)
	end if
	call MPI_BARRIER(comm_local, ierr)
	end do

	deallocate(src_grid_addr, dst_grid_addr,wts_map)

	return
	end



