!========================================================================================
! This module global object ActionHandler of type Actions.
! ActionHandler controls all properties of actions: comparison,
! relative priority and services wich can process it. 
!========================================================================================

module actions_module

use const_module
use utils_module, only : search_linear

implicit none

character(*), parameter, private :: ActionTypes(13) =       (/  "READ_FD", "SEND_MP" , "RECV_MP", "SAVE_CP", "SAVE_DG", "SAVE_IT",  "SEND_DAS","RECV_AOI", "RECV_ENKF", "SEND_NST", "RECV_NST", "STOP", "ERROR"/)
integer, parameter, private :: ActionPriority(13) =         (/          1,          2,         3,         4,         5,         6,           7,         8,           9,         10,       11,       12,      13/)
character(*), parameter, private :: ActionService(13) =     (/      "IOD",    "CPL"  ,     "CPL",     "IOD",     "IOD",     "IOD",       "AOI",     "AOI",       "AOI",      "NST",     "NST",      "",  ""    /)


type, private :: Actions
private

    

 contains
    !=======================
    ! ===== PUBLIC API =====
    procedure, public :: less => less_priority      ! comparison "<" operator for <act1> and <act2>
    procedure, public :: priority                   ! priority of action <act> (like hash)
    procedure, public :: service => get_service     ! what service process action <act>
    ! ===== PUBLIC API =====    
    !=======================
end type

    type(Actions) :: ActionHandler

 CONTAINS

!========================================================================================

logical function less_priority(this, act1, act2)
    class(Actions) :: this
    character(*), intent(in) :: act1, act2
    character(STATUS_LEN) :: status    
    less_priority = this % priority(act1, status) < this % priority(act2, status)    
end function

!========================================================================================

!========================================================================================

integer function priority(this, act, status)
    class(Actions) :: this
    character(*), intent(in) :: act
    character(STATUS_LEN), intent(inout) :: status
    integer :: ind
    
    ind = search_linear(act, ActionTypes, 1, SIZE(ActionTypes))    
    
    if (ind == 0) then
        status = "Actions error: unknown Action type <"//TRIM(act)//">" 
    else
        status = ""
        priority = ActionPriority(ind)
    end if
    
end function

!========================================================================================

character(3) function get_service(this, act, status)
    class(Actions) :: this
    character(*), intent(in) :: act
    character(STATUS_LEN), intent(inout) :: status
    integer :: ind
    
    ind = search_linear(act, ActionTypes, 1, SIZE(ActionTypes))    
    
    if (ind == 0) then
        status = "Actions error: unknown Action type <"//TRIM(act)//">" 
    else 
        status = ""
        get_service = ActionService(ind)
    end if
end function

!========================================================================================
            
end module
