!========================================================================================
! Class of simple clock. You can define start and end dates set tick period. 
! Clock uses DateTime class for add_time operations.
!========================================================================================

module clock_module

use datetime_module
implicit none

type :: Clock
private
    type(DateTime) :: cur_dt_, start_dt_, end_dt_
    integer :: dh_, dm_, ds_    
    logical :: is_set_ = .FALSE., is_stopped_ = .FALSE.
    
 contains                                   ! ===== PUBLIC API =====
    procedure, public :: set                ! set timer on (start_date, end_date) with tick period (dh, dm, ds)
    procedure, public :: tick               ! make one tick - just add time (dh, dm, ds)
    procedure, public :: is_stopped         ! .TRUE. when time is equal (end_date)
    procedure, public :: current_datetime   ! return current DateTime object
end type
 
 CONTAINS

!========================================================================================

subroutine set(this, start_dt, end_dt, dh, dm, ds)
    class(Clock), intent(inout) :: this
    type(DateTime), intent(in) :: start_dt, end_dt
    integer, intent(in) :: dh, dm, ds

    if (dh == 0 .AND. dm == 0 .AND. ds == 0) then
        write(*,*) "Error: Timer <tick> : ticking period cant be equal to zero"
        call exit(1)
    endif
    
    this % start_dt_ = start_dt
    this % cur_dt_ = this % start_dt_
    this % end_dt_ = end_dt    
    this % dh_ = dh    
    this % dm_ = dm
    this % ds_ = ds

    this % is_set_ = .TRUE.
    this % is_stopped_ = .FALSE.
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      perform one tick of clock. At the end set STOPPED flag to .TRUE.
!                   Note: if last tick greater then end_dt, force it to end_dt
!----------------------------------------------------------------------------------------

subroutine tick(this)
    class(Clock), intent(inout) :: this

    if(this % is_stopped_) return

    if (.NOT. this % is_set_) then
        write(*,*) "Error: Timer <tick> : ticking before set time of clock."
        call exit(1)
    endif

    call this % cur_dt_ % add_time(h = this % dh_, m = this % dm_, s = this % ds_)
    
    if(date2sec(this % cur_dt_) >= date2sec(this % end_dt_)) then
        this % is_stopped_ = .TRUE.
        this % cur_dt_ = this % end_dt_
    end if
        
end subroutine

!========================================================================================

logical function is_stopped(this)
    class(Clock) :: this
    is_stopped = this % is_stopped_
end function

!========================================================================================

type(DateTime) function current_datetime(this)
    class(Clock) :: this
    current_datetime = this % cur_dt_
end function

!========================================================================================
end module
