! TODO сделать перегрузку при регистрации массивовв


    module component_module

    use const_module
    use utils_array_module
    use decomposition_module, only: Decomposition
    use communicator_ga_module
    use array_info_storage_module, HashStorageArray => HashStorage 
    use model_info_module
    use experiment_info_module
    use component_splitter_module
    use array_info_module
    use datetime_module
    use clock_module
    use var_event_normal_module
    use var_event_synced_module
    use event_scheduler_module
    use event_module
    use debug_module
    implicit none
    character(NAME_LEN) array_opt
type, abstract, public :: Component
private

    type(HashStorageArray) :: array_storage_            
    type(Communicator), pointer :: comm_
    type(EventScheduler) :: ev_scheduler_
    type(Clock) :: model_clock_
     
    type(Decomposition) :: dec_
    integer :: timestep_sec_
    
    character, allocatable :: ev_packed_(:)    
    real(kind=4), allocatable :: adapter_3D4_(:,:,:)
    real(kind=8), allocatable :: adapter_3D8_(:,:,:)
    
    
 contains
    !========================
    ! ===== PUBLIC API ======
    procedure, public :: init => init_component             ! Constructor: get pointers and init storage
    procedure, public :: destroy => destroy_component       ! Destructor: destroy storage

    procedure, public :: register_model                     ! Put model info to ModelsInfo container
    generic, public :: register_array => register_array_real4_2D, register_array_real4_3D, register_array_real8_2D, register_array_real8_3D
    procedure, public :: register_periodic_event            ! Put periodic event to EventScheduler
    procedure, public :: register_synced_event              ! Synchromize with time file via FDAdapter
    procedure, public :: raise_event                        ! Raise component event
    procedure, public :: model_cycle                        ! perform model cycle from <start_date> to <end_date>
        
    procedure, public :: fatal_error => comp_fatal_error    ! Synchromized error and exit
    procedure, public :: model_time    
    procedure, public :: file_prefix                        ! Generate file prefix, e.g. <./data/OCN_tst>
    
    ! ===== PUBLIC API ======
    !========================
    
    procedure, private :: try_register_comp_ga
    procedure, private :: put_to_ga
    procedure, private :: get_from_ga    
    procedure, private :: handle_event
    procedure, private :: get_array_info
    procedure, private :: send_request
    procedure, private :: block_until_ga_status_becomes
    
    procedure, private :: register_array_real4_2D
    procedure, private :: register_array_real8_2D
    procedure, private :: register_array_real4_3D
    procedure, private :: register_array_real8_3D

    ! Abstract interfaces for components
    PROCEDURE(I_ini_reg_comp), DEFERRED :: ini_reg_comp
    PROCEDURE(I_ini_reg_data), DEFERRED :: ini_reg_data
    PROCEDURE(I_ini_allocate), DEFERRED :: ini_allocate
    PROCEDURE(I_ini_main), DEFERRED :: ini_main
    PROCEDURE(I_make_step), DEFERRED :: make_step
    PROCEDURE(I_finalize), DEFERRED :: finalize
    
END TYPE Component

ABSTRACT INTERFACE
    
    subroutine I_ini_reg_comp(this)
        import Component
        class(Component) :: this
    end subroutine  

    subroutine I_ini_reg_data(this)
        import Component
        class(Component) :: this
    end subroutine
    
    subroutine I_ini_allocate(this)
        import Component
        class(Component) :: this
    end subroutine
    
    subroutine I_ini_main(this)
        import Component
        class(Component) :: this
    end subroutine  
    
    subroutine I_make_step(this)
        import Component
        class(Component) :: this
    end subroutine  
    
    subroutine I_finalize(this)
        import Component
        class(Component) :: this
    end subroutine 
    
END INTERFACE

 CONTAINS

!========================================================================================
    
subroutine init_component(this, comm) 
    class(Component) :: this
    type(Communicator), intent(in), target :: comm
    
    this % comm_ => comm
    
    call this % array_storage_ % init()
    call this % ev_scheduler_ % init()
    
    if (.NOT. ALLOCATED(this % ev_packed_)) allocate(this % ev_packed_(event_pack_size()))        

end subroutine

subroutine destroy_component(this) 
    class(Component) :: this
    
    call this % array_storage_ % destroy()
    call this % ev_scheduler_ % destroy()
    
    if (ALLOCATED(this % ev_packed_)) deallocate(this % ev_packed_)       
    if (ALLOCATED(this % adapter_3D4_)) deallocate(this % adapter_3D4_)
    if (ALLOCATED(this % adapter_3D8_)) deallocate(this % adapter_3D8_)
                    
end subroutine 
!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      decompose model and put model in ModelsInfo container
! Parameters:
!   decomp          - object containing model decomposition
!   timestep_sec    - model timestep
!----------------------------------------------------------------------------------------
    
subroutine register_model(this, decomp, timestep_sec)
    class (Component) :: this
    type(Decomposition), intent(in) :: decomp
    integer, intent(in) :: timestep_sec
 
    this % dec_ = decomp
    ! Put information about myself in ModelInfo array (to sync it later)
    ModelsInfo(CompSplitter % i_am_id()) = ModelInfo(CompSplitter % i_am(), ModelsInfo(CompSplitter % i_am_id()) % model_name(), this % dec_, timestep_sec)
    
    ! Set model_clock (because all pre-cycle events should have start_date time)
    call this % model_clock_ % set(ExpInfo % start_date(), ExpInfo % end_date(), 0, 0, timestep_sec)
    this % timestep_sec_ = timestep_sec
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      get array address from user and put info to interanl hash
! Parameters:
!   name            - string name of array
!   arr_*           - array you want to register
!   indexing        - string representation of indexing (supported: "ijk", "kij")
!   lname           - not used now
!   units           - not used now
!----------------------------------------------------------------------------------------

#define _REG_ARRAY_(_FUNC_, _TYPE_, _DIMS_) \
subroutine _FUNC_(this, arr_name, indexing, arr); \
    class(Component) :: this; \
    character(*), intent(in) :: arr_name; \
    character(*), intent(in), optional :: indexing; \
    _TYPE_, _DIMS_, target :: arr; \
    type(ArrayInfo) :: arr_info; \
    call arr_info % init(arr_name, "", "", arr = arr, indx = indexing); \
    if (this % array_storage_ % has_key(arr_name)) call this % fatal_error("array name <"//arr_name//"> is already in use."); \
    call this % array_storage_ % put(arr_name, arr_info); \
end subroutine

_REG_ARRAY_(register_array_real4_2D, real(4), dimension(:,:))
_REG_ARRAY_(register_array_real8_2D, real(8), dimension(:,:))
_REG_ARRAY_(register_array_real4_3D, real(4), dimension(:,:,:))
_REG_ARRAY_(register_array_real8_3D, real(8), dimension(:,:,:))

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      adapter: preprocess_event and call ev_container
!   arr_name        - string name of array
!   act             - action to perform (e.g. SAVE_DG)
!   src             - (optional) source of data (component, file name) 
!   dst             - (optional) destination of data (component, file name)
!   info            - (optional) string with additional information to service
!   dh, dm, ds      - (optional) (hour, minute, second) period for event
!----------------------------------------------------------------------------------------

subroutine register_periodic_event(this, arr_name, act, src, dst, info, dd, dh, dm, ds, delay)
    class(Component) :: this
    character(*), optional, intent(in) :: src, dst, info
    character(*), intent(in) :: arr_name, act
    integer, intent(in), optional :: dd, dh, dm, ds, delay
    type(Event) :: ev
    type(DateTime) :: event_date
    type(ArrayInfo) :: arr_info

    arr_info = this % get_array_info(arr_name)    
    ev = Event(arr_info = arr_info, act = act, owner = CompSplitter % i_am(), src = src, dst = dst, &
                          dt = this % model_time(), file_prefix = this % file_prefix(), info = info)
    event_date =  ExpInfo % start_date()
    if ( PRESENT(delay) ) call event_date % add_time (0,0,0, delay)
    call this % ev_scheduler_ % add( &
        new_VarEventNormal( ev, event_date, &
                            MERGE(dd,0,PRESENT(dd)), MERGE(dh,0,PRESENT(dh)), MERGE(dm,0,PRESENT(dm)), MERGE(ds,0,PRESENT(ds))))
end subroutine
    
!========================================================================================
    
subroutine register_synced_event(this, arr_name, src, start_date)
    class(Component) :: this
    character(*), intent(in) :: src, arr_name
    type(DateTime), intent(in), optional :: start_date
    type(Event) :: ev
    type(ArrayInfo) :: arr_info

    arr_info = this % get_array_info(arr_name)    
    ev = Event(arr_info = arr_info, act = "READ_FD", owner = CompSplitter % i_am(), src = src, dst = CompSplitter % i_am())
    
    call this % ev_scheduler_ % add( &
        new_VarEventSynced( ev, &
                            MERGE(start_date, ExpInfo % start_date(), PRESENT(start_date))))        

end subroutine
    
!========================================================================================

subroutine raise_event(this, arr_name, act, src, dst, dt_rec)
    class(Component) :: this
    character(*), intent(in) :: act
    character(*), optional, intent(in) :: src, dst, arr_name
    type(DateTime), intent(in), optional :: dt_rec
    type(Event) :: ev
       
    type(ArrayInfo) :: arr_info
    
    if (PRESENT(arr_name)) arr_info = this % get_array_info(arr_name)

    ev = Event(arr_info = arr_info, act = act, owner = CompSplitter % i_am(), src = src, dst = dst, &
                          dt = this % model_time(), dt_rec = dt_rec, file_prefix = this % file_prefix())  
        
    call this % handle_event(ev)
 
end subroutine

!========================================================================================

type(DateTime) function model_time(this)
    class(Component) :: this
    model_time = this % model_clock_ % current_datetime()
end function
    
!========================================================================================

subroutine model_cycle(this)
    use event_vector_module, VectorEvent => VectorStorage   

    class(Component) :: this
    integer :: i
    type(VectorEvent) :: new_events
    type(Event) :: ev !,evtest
    
    ! Initialization loop
    call this % ev_scheduler_ % get_all_events(new_events)
    do i = 1, new_events % length()
        ev = new_events % get(i)
        call this % send_request(ev)
        call this % try_register_comp_ga(ev)
    end do

    ! This means we stop registration
    call this % raise_event(act = "STOP")
    
    ! We should be sure that all are registered now
    call CompSplitter % barrier()
        
    ! Iterate while clock is ticking
    do while (.TRUE.)
        ! Ask EventScheduler about upcoming events and raise them
        call this % ev_scheduler_ % gather_events(this % model_time(), new_events)
        
        !if (this % model_clock_ % current_datetime() == ExpInfo % start_date() .and. TRIM(ModelsInfo(CompSplitter % i_am_id()) % model_name())=="ATM_NCAR") then
	!	do i = 1, new_events % length()
	!		evtest=new_events % get(i)
	!		print *, 'Event schedule tester ---> ', trim(evtest % time()), trim(evtest % action()), trim(evtest % src()), trim(evtest % dst()), trim(evtest % arr_name())
	!	end do
	!end if	
		
        do i = 1, new_events % length()
            call this % handle_event(new_events % get(i))
        end do

        if (this % model_clock_ % is_stopped()) EXIT        
        
        ! Models INI_MAIN on first step
        if (this % model_clock_ % current_datetime() == ExpInfo % start_date()) &
            call this % ini_main()
        
        ! Make all physics of the step
        call this % make_step()
        
        ! Make one clock tick   
        call this % model_clock_ % tick()
    end do 

    ! At the end send STOP signal to services
    call this % raise_event(act = "STOP")
    
    
end subroutine

!========================================================================================


subroutine comp_fatal_error(this, msg)
    class(Component) :: this
    character(*), intent(in) :: msg
    call CompSplitter % sync_error("Component "//TRIM(ModelsInfo(CompSplitter % i_am_id()) % model_name())//": "//msg, "LCL")
end subroutine

!========================================================================================


!========================================================================================
! PRIVATE
!========================================================================================

type(ArrayInfo) function get_array_info(this, arr_name)
    class(Component) :: this
    character(*), intent(in) :: arr_name
    get_array_info = this % array_storage_ % get(arr_name)
    if (get_array_info % arr_name() == "") call this % fatal_error("you are trying to add event for unregistered array <"//TRIM(arr_name)//">")   
end function 

character(LONG_NAME_LEN) function file_prefix(this)
    class(Component) :: this    
    file_prefix = TRIM(ExpInfo % data_dir())//"/"//TRIM(ModelsInfo(CompSplitter % i_am_id()) % model_name())//"_"//TRIM(ExpInfo % exp_name())
end function

!========================================================================================
    
subroutine handle_event(this, ev)
    class(Component) :: this
    type(Event) :: ev
    character(100) :: ga_name
    type(DateTime) :: dt,dt_tmp
    character(30) :: status
    character(10) :: yyyymmdd
        
    ! Set current model time
    dt = this % model_time()
    call ev % set_time(dt % to_string())
        
    call Debugger % log_msg("Component start handle: "//TRIM(ev % to_string()))

    ! Send request and perform event-specific actions
    select case (ev % action())

        case("SAVE_CP", "SAVE_DG", "SEND_MP","SEND_DAS","SEND_NST")

            ! CP has time as part of its name
            if (ev % action() == "SAVE_CP") then
                ! Dont save CP on start
                if (dt == ExpInfo % start_date()) return

                yyyymmdd = dt % to_string()
                call ev % set_dst(TRIM(this % file_prefix())//"_CP_"//yyyymmdd//".nc")

            end if

        
            ! Wait untill buffer become empty, put to it, sync, mark as full and send request
            call this % block_until_ga_status_becomes(ev % ga_name(), "free")
                
            call this % put_to_ga(ev)
            call this % comm_ % sync(CompSplitter % i_am_id())                    
            if (CompSplitter % is_first_rank()) call this % comm_ % put_info(ev % ga_name(), COMM_GA_STATUS, "full")
            call this % send_request(ev) 
                                
        case("READ_FD","RECV_AOI","RECV_ENKF","RECV_NST")
            
            ! Special case: if read_cp but we set search_cp to false - skip action (easy for client)
            if (TRIM(ev % info()) == "READ_CP" .AND. ExpInfo % search_cp() == .false.) return
           ! Try register, wait until full, get data, mark as free
            call this % send_request(ev) 
            call Debugger % log_msg("After send request "//ev % ga_name())            

            call this % block_until_ga_status_becomes(ev % ga_name(), "full")
            
            if (ev % action() == "RECV_ENKF") then 
                write(*,*),TRIM(ev % ga_name()) ," DAS RECV INFO", TRIM(this % comm_ % get_info(ev % ga_name(), COMM_GA_OPT))
                array_opt = this % comm_ % get_info(ev % ga_name(), COMM_GA_OPT)
                if(CompSplitter % is_first_rank()) call this % comm_ % put_info(ev % ga_name(), COMM_GA_STATUS, "0") 
            endif
            call this % get_from_ga(ev)
            call this % comm_ % sync(CompSplitter % i_am_id())                
            if(CompSplitter % is_first_rank()) call this % comm_ % put_info(ev % ga_name(), COMM_GA_STATUS, "free")            

        case("RECV_MP")
            ! This is push-event, so no request: just register, wait, get, mark as free
            call this % block_until_ga_status_becomes(ev % ga_name(), "full")
            
            call this % get_from_ga(ev)
            
            ! Check time correspondence
            status = TRIM(this % comm_ % get_info(ev % ga_name(), COMM_GA_TIME))
            if(ev % time() /= TRIM(status)) then
                call this % fatal_error("Received mapping event time for array <"//TRIM(ev % arr_name())// &
                                         "> ("//TRIM(status)//") "//" do not correspond to model time ("//TRIM(ev % time())//"). Check mapping periods.")
            end if
                
            call this % comm_ % sync(CompSplitter % i_am_id())
            call Debugger % log_msg("Finish receieve.")
            if(CompSplitter % is_first_rank()) call this % comm_ % put_info(ev % ga_name(), COMM_GA_STATUS, "free") 
        
        case("STOP")
            call this % send_request(ev)           
        
        case("ERROR")
            call this % send_request(ev)           
            call CompSplitter % barrier()
        
        case default
            call this % fatal_error("raise event: uknown action: <"//TRIM(ev % action())//">")
    end select   
end subroutine

subroutine try_register_comp_ga(this, ev)
    class(Component) :: this
    type(Event) :: ev
    character(STATUS_LEN) :: status
    ! Lazy: if not registered: register, return
    if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
        call this % comm_ % init_array(ev % ga_name(), ev % arr_type(), ev % arr_dims(), &
                                        CompSplitter % i_am_id(), this % dec_, &
                                        CompSplitter % comp_id(ActionHandler % service(ev % action(),status)))
                                        
    end if
end subroutine
       
!========================================================================================
       
subroutine block_until_ga_status_becomes(this, ga_name, status)
    class(Component) :: this
    character(*), intent(in) :: ga_name, status

    if (CompSplitter % is_first_rank()) then
        do while (TRIM(this % comm_ % get_info(ga_name, COMM_GA_STATUS)) /= status); end do
    end if
    call this % comm_ % sync(CompSplitter % i_am_id())  
end subroutine
    
 
!========================================================================================

subroutine put_to_ga(this, ev)
    class(Component) :: this
    type(Event), intent(in) :: ev
    type(ArrayInfo) :: arr_info

    arr_info = this % get_array_info(ev % arr_name())
              
    ! Put data depends on type and dims (TODO in templates)
    select case (ev % arr_dims()) 
        case("2D")
        if (ev % arr_type() == "real4") call this % comm_ % put(ev % ga_name(), this % dec_ % lower_bound_2D(), this % dec_ % upper_bound_2D(), arr_info % p_2D4())
        if (ev % arr_type() == "real8") call this % comm_ % put(ev % ga_name(), this % dec_ % lower_bound_2D(), this % dec_ % upper_bound_2D(), arr_info % p_2D8())

        case("3D")
        if (ev % arr_type() == "real4") call this % comm_ % put(ev % ga_name(), this % dec_ % lower_bound_3D(), this % dec_ % upper_bound_3D(), arr_info % p_3D4())
        if (ev % arr_type() == "real8") call this % comm_ % put(ev % ga_name(), this % dec_ % lower_bound_3D(), this % dec_ % upper_bound_3D(), arr_info % p_3D8())
    end select

end subroutine

!========================================================================================

subroutine get_from_ga(this, ev)
    class(Component) :: this
    type(Event), intent(in) :: ev
    type(ArrayInfo) :: arr_info

    arr_info = this % get_array_info(ev % arr_name())

    select case (ev % arr_dims()) 
        case("2D")
        if (ev % arr_type() == "real4") call this % comm_ % get(ev % ga_name(), this % dec_ % lower_bound_2D(), this % dec_ % upper_bound_2D(), arr_info % p_2D4())
        if (ev % arr_type() == "real8") call this % comm_ % get(ev % ga_name(), this % dec_ % lower_bound_2D(), this % dec_ % upper_bound_2D(), arr_info % p_2D8())

        case("3D")
        if (ev % arr_type() == "real4") then
            if (.NOT. ALLOCATED(this % adapter_3D4_)) allocate(this % adapter_3D4_(this % dec_ % i_size_loc(), this % dec_ % j_size_loc(), this % dec_ % k_size()))        
            call this % comm_ % get(ev % ga_name(), this % dec_ % lower_bound_3D(), this % dec_ % upper_bound_3D(), this % adapter_3D4_)
            call arr_info % set_data(this % adapter_3D4_)
        else if (ev % arr_type() == "real8") then
            if (.NOT. ALLOCATED(this % adapter_3D8_)) allocate(this % adapter_3D8_(this % dec_ % i_size_loc(), this % dec_ % j_size_loc(), this % dec_ % k_size()))        
            call this % comm_ % get(ev % ga_name(), this % dec_ % lower_bound_3D(), this % dec_ % upper_bound_3D(), this % adapter_3D8_)
            call arr_info % set_data(this % adapter_3D8_)
        end if
    end select

end subroutine
     
!========================================================================================

subroutine send_request(this, ev)
    class(Component) :: this
    type(Event), intent(in) :: ev
    integer :: ierr
    character(STATUS_LEN) :: status
    
    ! Only major rank send message
    if (.NOT. CompSplitter % is_first_rank()) return
            
    call event_to_mpi_pack(ev, this % ev_packed_)
    call MPI_SEND(this % ev_packed_, SIZE(this % ev_packed_), MPI_PACKED, 0, ActionHandler % priority(ev % action(), status), MPI_COMM_WORLD, ierr)
    
end subroutine


end module

