!========================================================================================      
! Useful clients
!========================================================================================      

module cpl_comp_clients_module

implicit none

 CONTAINS
!========================================================================================      
! Register all groups in Communicator

subroutine comm_register_groups(comm)
    use const_module
    use component_splitter_module
    use communicator_ga_module
    implicit none   
    type(Communicator), intent(inout) :: comm    
    integer :: i
    integer, allocatable :: proc_list_src(:), proc_list_dst(:)
    
    ! If model, register all others as subscribers
    if (CompSplitter % is_model()) then
    
        call CompSplitter % proc_list(list = proc_list_src)
        do i = 1, CompSplitter % comp_defined()
            if (CompSplitter % is_launched(id = i) .AND. &
                .NOT. CompSplitter % is_model(id = i) .OR. &
                CompSplitter % i_am_id() == i) then   
                    call CompSplitter % proc_list(list = proc_list_dst, id = i)
                    call comm % init_group(CompSplitter % i_am_id(), proc_list_src, i, proc_list_dst)
                    deallocate(proc_list_dst)
            end if
        end do
        deallocate(proc_list_src)
    
    else
    
        call CompSplitter % proc_list(list = proc_list_dst)
        
        do i = 1, CompSplitter % comp_defined()
            if (CompSplitter % is_launched(id = i) .AND. &
                CompSplitter % is_model(id = i) .OR. &
                CompSplitter % i_am_id() == i) then
                call CompSplitter % proc_list(id = i, list = proc_list_src)
                call comm % init_group(i, proc_list_src, CompSplitter % i_am_id(), proc_list_dst)
                deallocate(proc_list_src)
            end if
        end do
        deallocate(proc_list_dst)
    end if
 
end subroutine


subroutine show_run_models()
    use component_splitter_module
    use communicator_ga_module
    use model_info_module

    integer :: comp, serv

    if (CompSplitter % rank_world() == 0) then
        do comp = 1, SIZE(ModelsInfo)
            if (CompSplitter % is_launched(CompSplitter % comp_name(comp)) .AND. CompSplitter % is_model(CompSplitter % comp_name(comp))) call ModelsInfo(comp) % show()
        end do
        
        do serv = 1, size(SERVICE_COMPS)
            if (CompSplitter % is_launched(name = SERVICE_COMPS(serv))) then
                write(*,'(a, a)') "Service name:                 ", TRIM(SERVICE_COMPS(serv))
                write(*,'(a, a)') "Number of processes:          ", n2s(CompSplitter % comm_local_size(name = SERVICE_COMPS(serv)))
            end if
        end do
                write(*,'(a)') '-----------------------------------------------------------------------'
        
        
    end if
    

end subroutine


!========================================================================================


end module