module debug_module

use component_splitter_module
use utils_module

integer, parameter, private :: DEBUG_STREAM = 333
    
type :: Debug
private
    character(100) :: filename_
    logical :: is_active = .false.
    
    
 contains 

    generic :: log_msg => log_msg_only, log_msg_i4, log_msg_arr_i4, log_msg_r4, log_msg_r8, log_msg_str

    procedure, public :: init => init_debugger
    procedure, public :: destroy => destroy_debugger

    procedure, private :: log_msg_only
    procedure, private :: log_msg_i4
    procedure, private :: log_msg_arr_i4
    procedure, private :: log_msg_r4
    procedure, private :: log_msg_r8
    procedure, private :: log_msg_str
    
    
end type

type(Debug) :: Debugger

    
 contains

!========================================================================================

subroutine init_debugger(this)
    class(Debug) :: this
    
#ifdef DEBUG    
    this % is_active = .true.
#endif
    if (.NOT. this % is_active) return

    this % filename_ = TRIM(CompSplitter % i_am())//TRIM(n2s(CompSplitter % rank_local()))//".debug.txt"   
    open(DEBUG_STREAM, file = this % filename_, status = 'replace')
    
end subroutine
 
subroutine destroy_debugger(this)
    class(Debug) :: this

    if (.NOT. this % is_active) return
    close(DEBUG_STREAM)
    
end subroutine

!========================================================================================

subroutine log_msg_only(this, msg)
    class(Debug) :: this
    character(*), intent(in) :: msg
    integer :: cur_time_arr(8)
    character(100) :: time_str

    if (.NOT. this % is_active) return
    call DATE_AND_TIME(VALUES = cur_time_arr)
    time_str = TRIM(n2s(cur_time_arr(5),2))//':'//TRIM(n2s(cur_time_arr(6),2))//':'//TRIM(n2s(cur_time_arr(7),2))//">"
    
    write(DEBUG_STREAM, *) TRIM(time_str)//": "//TRIM(msg)
end subroutine

#define _LOG_MSG_(_FUNC_,_TYPE_, _DIMS_)    \
subroutine _FUNC_ (this, msg, val);         \
    class(Debug) :: this;                   \
    character(*), intent(in) :: msg;        \
    _TYPE_, intent(in) :: val _DIMS_;       \
    if (.NOT. this % is_active) return;     \
    write(DEBUG_STREAM, *) msg, val;        \
end subroutine

_LOG_MSG_(log_msg_i4,       integer, )
_LOG_MSG_(log_msg_arr_i4,   integer, (:))
_LOG_MSG_(log_msg_r4,       real(4), )
_LOG_MSG_(log_msg_r8,       real(8), )
_LOG_MSG_(log_msg_str,      character(*), )

 
!========================================================================================
 
end module