!========================================================================================
! Small class for all informtation about experiment
!========================================================================================

module experiment_info_module

use const_module
use datetime_module
use component_splitter_module, only: CompSplitter, sync_error
implicit none

type, private :: ExperimentInfo
private
    character(SHORT_NAME_LEN) :: exp_name_
    type(DateTime) :: start_date_, end_date_, exp_start_date_
    character(NAME_LEN) :: data_dir_
    logical :: silent_, search_cp_
 contains                                               ! ===== PUBLIC API =====
    procedure, public :: init => init_command_line      ! takes number of first argument in command line to read (now read only exp_name)
    procedure, public :: show => show_exp_info          ! print experiment informtation
    
    
    procedure, public :: exp_name => get_exp_name    ! accessor
    procedure, public :: namelist_name               ! accessor
    procedure, public :: start_date                  ! accessor
    procedure, public :: exp_start_date              ! accessor
    procedure, public :: end_date                    ! accessor
    procedure, public :: data_dir
    procedure, public :: silent
    procedure, public :: search_cp
       
    procedure, private :: read_namelist    
end type

!========================================================================================
type(ExperimentInfo) :: ExpInfo

 CONTAINS

!========================================================================================
  
subroutine init_command_line(this, start_arg)
    class(ExperimentInfo) :: this
    integer, intent(in) :: start_arg
    call GET_COMMAND_ARGUMENT(start_arg, this % exp_name_)
    call this % read_namelist("configure/exp_"//TRIM(this % exp_name_)//".in")
end subroutine
  
!============================================================================================

character(SHORT_NAME_LEN) function get_exp_name(this)
    class(ExperimentInfo) :: this
    get_exp_name = this % exp_name_
end function

character(SHORT_NAME_LEN) function namelist_name(this)
    class(ExperimentInfo) :: this
    namelist_name = "exp_"//TRIM(this % exp_name_)//".in"
end function

character(NAME_LEN) function data_dir(this)
    class(ExperimentInfo) :: this
    data_dir = this % data_dir_
end function


type(DateTime) function start_date(this)
    class(ExperimentInfo) :: this
    start_date = this % start_date_
end function

type(DateTime) function exp_start_date(this)
    class(ExperimentInfo) :: this
    exp_start_date = this % exp_start_date_
end function

type(DateTime) function end_date(this)
    class(ExperimentInfo) :: this
    end_date = this % end_date_
end function

logical function silent(this)
    class(ExperimentInfo) :: this
    silent = this % silent_
end function

logical function search_cp(this)
    class(ExperimentInfo) :: this
    search_cp = this % search_cp_
end function

!============================================================================================

subroutine show_exp_info(this)
    class(ExperimentInfo) :: this
    
    write(*,'(a)') '======================================================================='
    write(*,'(a, a)')  'Experiment name:              ', TRIM(this % exp_name_)
    write(*,'(a, a)')  'Data directory:               ', TRIM(this % data_dir_)
    write(*,'(a, a)')  'Start date:                   ', TRIM(this % start_date_ % to_string()) 
    write(*,'(a, a)')  'End date:                     ', TRIM(this % end_date_ % to_string())
    write(*,'(a, l)')  'Start from CP:               ',  this % search_cp_
    write(*,'(a, l)')  'Silent mode:                 ',  this % silent_
    write(*,'(a)') '======================================================================='
        
end subroutine

subroutine read_namelist(this, namelist_path)
    class(ExperimentInfo) :: this    
    character(*), intent(in) :: namelist_path   
    logical :: file_exists
    character(NAME_LEN) :: data_dir, run_start_date, exp_start_date
    integer :: run_days
    logical :: search_cp, silent
    namelist /EXP_INFO/ data_dir, run_start_date, exp_start_date, run_days, search_cp
    namelist /RUN_PARAMS/ silent

    INQUIRE(file = namelist_path, exist = file_exists)
    if(file_exists) then
        open(11, file = namelist_path)
        read(11, nml = EXP_INFO)
        read(11, nml = RUN_PARAMS)
        close(11)
    else
        call CompSplitter % sync_error("ExperimentInfo: cant find input namelist: <"//TRIM(namelist_path)//">", "GLB")
    end if

    if (run_days <= 0) &
        call CompSplitter % sync_error("ExperimentInfo: <run_days> parameter of input namelist should be postive integer", "GLB")
    
    this % start_date_ = DateTime(run_start_date)
    this % exp_start_date_ = DateTime(exp_start_date)
    this % end_date_ = this % start_date_
    call this % end_date_ % add_time(d = run_days) 
    
    this % data_dir_ = data_dir
    this % silent_ = silent
    this % search_cp_ = search_cp
    
end subroutine

!============================================================================================

end module