#define __MODULE_NAME__         halo_updater4_module
#define __CLASS_NAME__          HaloUpdater4
#define __DATA_TYPE__           real(kind=4)
#define __MPI_DATATYPE__        MPI_REAL
#include "HaloUpdaterBase.f90"

#define __MODULE_NAME__         halo_updater8_module
#define __CLASS_NAME__          HaloUpdater8
#define __DATA_TYPE__           real(kind=8)
#define __MPI_DATATYPE__        MPI_DOUBLE_PRECISION
#include "HaloUpdaterBase.f90"

!========================================================================================

module halo_updater_module
use halo_updater4_module
use halo_updater8_module
use component_splitter_module
implicit none
    
type :: HaloUpdater
private
    type(HaloUpdater4) :: hupd4
    type(HaloUpdater8) :: hupd8
    logical :: is_init = .false.
 contains
 
    procedure, public :: init => init_halo_updater
    procedure, public :: destroy => destroy_halo_updater
    generic, public :: update => update2D4, update3D4, update2D8, update3D8

    procedure, private :: update2D4
    procedure, private :: update3D4
    procedure, private :: update2D8
    procedure, private :: update3D8
end type 

type(HaloUpdater) :: HaloUpdateMaster
 contains

!========================================================================================
 
subroutine init_halo_updater(this, decomp, max_halo_width)
    class(HaloUpdater) :: this
    type(Decomposition), intent(in) :: decomp
    integer, intent(in) :: max_halo_width

    call this % hupd4 % init(decomp, max_halo_width)
    call this % hupd8 % init(decomp, max_halo_width)
    
    this % is_init = .true.
    
end subroutine

!========================================================================================

subroutine destroy_halo_updater(this)
    class(HaloUpdater) :: this

    call this % hupd4 % destroy()
    call this % hupd8 % destroy()
    
end subroutine

!========================================================================================

#define _HALO_UPDATE_(__FUNC__, __TYPE__, __DIMS__, __OBJ__, __ARG__) \
subroutine __FUNC__(this, arr, update_width, grid_type, change_sign_on_bipolar); \
    class(HaloUpdater) :: this; \
    __TYPE__, __DIMS__ :: arr; \
    integer, intent(in) :: update_width; \
    logical, optional :: change_sign_on_bipolar; \
    character(1), intent(in) :: grid_type; \
    if (this % is_init == .false.) & \
    call CompSplitter % sync_error("HaloUpdater: called update() without call to init().", mode = "LCL"); \
    call __OBJ__ % update(__ARG__ = arr, update_width = update_width, grid_type = grid_type, change_sign_on_bipolar = change_sign_on_bipolar); \
end subroutine

_HALO_UPDATE_(update2D4, real(4), dimension(:,:), this % hupd4, DIM2_arr)    
_HALO_UPDATE_(update3D4, real(4), dimension(:,:,:), this % hupd4, DIM3_arr)    
_HALO_UPDATE_(update2D8, real(8), dimension(:,:), this % hupd8, DIM2_arr)    
_HALO_UPDATE_(update3D8, real(8), dimension(:,:,:), this % hupd8, DIM3_arr)    
    

!========================================================================================


end module