module CONCRETE_MODULE_FLIBS
    use CONCRETE_MODULE, DICT_DATA => CONCRETE_TYPE
    implicit none

    ! The length of the keys
    integer, parameter :: DICT_KEY_LENGTH = 30
    ! The "null" value for these data
    type(DICT_DATA) :: DICT_NULL
    
    include "dictionary.f90"
    
end module

module OUTPUT_MODULE

    use CONCRETE_MODULE_FLIBS        
    implicit none
    
    TYPE, PUBLIC :: HashStorage

    PRIVATE
        type(DICT_STRUCT), pointer :: table_

      CONTAINS

        PROCEDURE, PUBLIC :: init => init_storage
        PROCEDURE, PUBLIC :: destroy => destroy_storage

        PROCEDURE, PUBLIC :: put
        PROCEDURE, PUBLIC :: get
        procedure, public :: has_key
    
    END TYPE HashStorage

 CONTAINS

    subroutine init_storage(this)
        class(HashStorage) :: this
        type(DICT_DATA) :: data
        
        call dict_create(this % table_, "", data)
    
    end subroutine init_storage
 
    subroutine destroy_storage(this)
        class(HashStorage) :: this
        
        call dict_destroy(this % table_)
        
    end subroutine destroy_storage
 
    subroutine put(this, name, node)
        class(HashStorage) :: this
        character(*), intent(in) :: name
        type(DICT_DATA), intent(in) :: node

        if (LEN(TRIM(name)) > DICT_KEY_LENGTH) then
            write(*,*) 'Error: HashStorage: cant add key - too long: '//TRIM(name)
            call exit(1)
        end if
        call dict_add_key(this % table_, TRIM(name), node)
        
    end subroutine

    type(DICT_DATA) function get(this, name)
            
        class(HashStorage) :: this
        character(*), intent(in) :: name
        
        get = dict_get_key(this % table_, TRIM(name))
    end function
    
    logical function has_key(this, name)
            
        class(HashStorage) :: this
        character(*), intent(in) :: name
        
        has_key = dict_has_key(this % table_, TRIM(name))
    end function

end module