module point_saver_module

use file_handler_nc_module

implicit none

! One PointSaver object control storage of one point. For optimization purposes this 
! process is buffered.
!======================================================================================== 

type :: PointSaver 
private
    character(50) :: varname_
    integer :: cursize_
    integer :: flush_period_

    type(FileHandler_NC) :: fh_

 contains
    procedure, public :: init => init_point_saver           ! constructor get variable name and buffer len
    procedure, public :: destroy => destroy_point_saver     ! destructor flush buffers
    procedure, public :: put => put_point                   ! put data to file (or buffer)
    

end type

 CONTAINS

!======================================================================================== 

subroutine init_point_saver(this, varname, flush_period) 
    class(PointSaver) :: this
    character(*), intent(in) :: varname
    integer, intent(in) :: flush_period
 
    this % cursize_ = 0
    this % flush_period_ = flush_period
    
    call this % fh_ % open_or_create_file(varname//"_IT.nc")
    call this % fh_ % create_dim("rec", 1)
    call this % fh_ % create_time_dim()
    call this % fh_ % put_att("title", "point diagnostic for variable "//varname, is_global = .TRUE.)
    call this % fh_ % open_or_create_var(varname, "real8", dim1_name = "rec", dimt_name = "")
 
end subroutine 
 
subroutine destroy_point_saver(this) 
    class(PointSaver) :: this
    
    call this % fh_ % close_file()
 
end subroutine 
 
!======================================================================================== 

subroutine put_point(this, val, dt) 
    class(PointSaver) :: this
    real(8), intent(in) :: val
    type(DateTime), intent(in) :: dt
    integer :: i

    call this % fh_ % put((/ val /), (/ 1 /), (/ 1 /), dt)
    
    this % cursize_ = this % cursize_ + 1
    if (this % cursize_ == this % flush_period_) then
        call this % fh_ % flush()
        this % cursize_ = 0
    end if
    
end subroutine    
    
    

end module









