    ! Be sure, that all have received request in one time
    ! #DEADLOCK: CPL 1 receive msg and look that array still full - put it to queue
    !            CPL 2 receive msg and look that array is free - process it




!========================================================================================
! Derived class for CPL service.
!========================================================================================

module service_cpl_module

use utils_module
use service_module
use mapper_module
use decomposition_module
use event_module
use component_splitter_module
use communicator_ga_module
use model_info_module
use cpl_comp_clients_module
use experiment_info_module
use actions_module
use event_vector_module, VectorEvent => VectorStorage    
use string_hash_module, StingHash => HashStorage
use statistics_module
#ifdef DEBUG
use debug_module
#endif

implicit none

type, extends(Service) :: ServiceCPL
private
    type(Mapper) :: mappers_(NUMBER_OF_COMPONENTS, NUMBER_OF_COMPONENTS)
    type(VectorEvent) :: ev_accum_
    
 contains  
    !=======================
    ! ===== PUBLIC API =====
    procedure, public :: init => init_cpl
    procedure, public :: destroy => destroy_cpl
    procedure, public :: handle_request => handle_cpl
    procedure, public :: after_init => after_init_cpl
    
    procedure, private :: perform_mapping
    procedure, private :: send_map_to_dst
    procedure, private :: make_filename
    procedure, private :: corr_ga_name
    
    procedure, private :: process_event
    procedure, private :: process_queued
    procedure, private :: is_ga_free
    procedure, private :: has_new_msg
    procedure, private :: is_pair_registered
    ! ===== PUBLIC API =====
    !=======================    
end type

 CONTAINS

!========================================================================================
 
subroutine init_cpl(this)
    class(ServiceCPL) :: this
    integer :: s, r
    integer, allocatable :: serv_ranks(:)
 
    call this % ev_accum_ % init()
     
end subroutine

subroutine destroy_cpl(this)
    class(ServiceCPL) :: this

    call this % ev_accum_ % destroy()
    
end subroutine
 
 subroutine after_init_cpl(this)
    class(ServiceCPL) :: this
    
end subroutine
!========================================================================================
! Problem is: we want not to get coupler events which can deadlock him
! So we need check if final buffer ready.
! But operation if "free" is not synced: CPL 1 could see "no", and then CPL 2
! could see "yes" in a second.
! Solution: we broadcast check of first cpl to all others.
! Can we do better? #TODO

logical function is_ga_free(this, name)
    class(ServiceCPL) :: this
    character(*), intent(in) :: name
    
    if (CompSplitter % is_first_rank()) then
        is_ga_free = TRIM(this % comm_ % get_info(name, COMM_GA_STATUS)) == "free"
    end if
    call broadcast(is_ga_free, 0, CompSplitter % comm_local())
 
end function

!========================================================================================

logical function has_new_msg(this)
    class(ServiceCPL) :: this
    integer :: status(MPI_STATUS_SIZE), ierr
    
    if (CompSplitter % is_first_rank()) then
        call MPI_IPROBE(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, has_new_msg, status, ierr)
    end if
    call broadcast(has_new_msg, 0, CompSplitter % comm_local())
                
                
end function


!========================================================================================

subroutine process_queued(this)
    class(ServiceCPL) :: this
    type(Event) :: ev_queued
    integer :: counter

    ! Do while queue is not empty or why no new messages
    do while (.NOT. (this % ev_accum_ % length() == 0 .OR. this % has_new_msg()))     
    
        counter = 1
        
        ! Check queue for ready events
        do while (counter <= this % ev_accum_ % length()) 
            ev_queued = this % ev_accum_ % get(counter)

#ifdef DEBUG
            call Debugger % log_msg("Consider: "//TRIM(ev_queued % to_string()))
#endif        

            if (this % is_ga_free(this % corr_ga_name(ev_queued))) then
#ifdef DEBUG
                call Debugger % log_msg("Get and process: "//TRIM(ev_queued % to_string()))
#endif            
                call this % process_event(ev_queued)
                call this % ev_accum_ % delete(ind = counter, num = 1)
            else
                counter = counter + 1
            end if
        end do  
    
    end do
    
                
end subroutine          

!========================================================================================

subroutine handle_cpl(this, ev)
    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev
    
   select case (ev % action())
    
        case("RECV_MP")
            if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
                call this % try_register_service_ga(ev)
#ifdef DEBUG
                call Debugger % log_msg ("Registered: "//TRIM(ev % to_string()))
#endif                
            end if

        case("SEND_MP")
            if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
                call this % try_register_service_ga(ev)
#ifdef DEBUG
                call Debugger % log_msg ("Registered: "//TRIM(ev % to_string()))
#endif                
                return
            else
                call this % ev_accum_ % append(ev)
                call this % process_queued()
            end if
                    
        ! Error case
        case("ERROR")   
            call this % report_handle(ev)
            call sleep(1)
            call exit(1)    

        case default
            call this % process_queued()
            
    end select

end subroutine

!========================================================================================

! Create corresponding name using this SEND_MP event
character(NAME_LEN) function corr_ga_name(this, ev)
    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev
    
    corr_ga_name = ev % make_ga_name(ev % dst(), ev % arr_name(), "RECV_MP")
end function


!========================================================================================

logical function is_pair_registered(this, ev)
    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev
    
    
    is_pair_registered = this % comm_ % is_registered(ev % ga_name()) .AND. &
                         this % comm_ % is_registered(this % corr_ga_name(ev))
    
end function

!========================================================================================

subroutine process_event(this, ev)
    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev

#ifdef DEBUG
    call Debugger % log_msg ("Process event: "//TRIM(ev % to_string()))
#endif
    
    call this % perform_mapping(ev)
    call this % send_map_to_dst(ev)    
    
    call this % report_handle(ev)    
end subroutine

!========================================================================================

subroutine perform_mapping(this, ev)
    use decomposition_module ! #BUG - cant use it in module header

    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev
    type(Decomposition) :: src_dec, cpl_src_dec, cpl_dst_dec
    character(100) :: filename
    integer :: src_id, dst_id, service_id

    ! Shortcuts
    src_id = CompSplitter % comp_id(ev % src())
    dst_id = CompSplitter % comp_id(ev % dst())
    service_id = CompSplitter % i_am_id()
    cpl_src_dec = this % morph_to_decomp_1D(ev % src())
    cpl_dst_dec = this % morph_to_decomp_1D(ev % dst())
    
    ! Lazy mapper initialization
    if (.NOT. this % mappers_(src_id, dst_id) % is_initialized()) then
        call this % mappers_(src_id, dst_id) % init(cpl_src_dec, cpl_dst_dec)  
        call this % mappers_(src_id, dst_id) % read_and_analize(this % make_filename(ev, cpl_src_dec, cpl_dst_dec))
        call Stats % init_time_mark("service_cpl_without_init", "LCL")    
     end if
    
    ! Wait until ga is full
!     do while (TRIM(this % comm_ % get_info(ev % ga_name(), COMM_GA_STATUS)) == "free"); end do
    
    ! Interpolate, passing src_array directly to subroutine
    call this % mappers_(src_id, dst_id) % interpolate(this % comm_ % id(ev % ga_name()))
    
    ! Ensure that all finish interpolation and mark array as free
    call this % comm_ % sync(service_id)
    
    if(CompSplitter % is_first_rank()) call this % comm_ % put_info(ev % ga_name(), COMM_GA_STATUS, "free")
        
end subroutine

!========================================================================================

subroutine send_map_to_dst(this, ev)
    use decomposition_module

    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev   
    type(Decomposition) :: dst_dec, cpl_dst_dec
    integer :: src_id, dst_id, service_id
    
    ! Shortcuts
    src_id = CompSplitter % comp_id(ev % src())
    dst_id = CompSplitter % comp_id(ev % dst())
    service_id = CompSplitter % i_am_id()
    cpl_dst_dec = this % morph_to_decomp_1D(ev % dst())
    
    ! Wait until ga is free
!     do while (TRIM(this % comm_ % get_info(this % corr_ga_name(ev), COMM_GA_STATUS)) == "full"); end do   
      
    ! Put data to destination ga    
    call this % comm_ % put(this % corr_ga_name(ev), cpl_dst_dec % lower_bound_2D(), cpl_dst_dec % upper_bound_2D(), this % mappers_(src_id, dst_id) % get_result_2D())
    
    ! Ensure that all put and change status
    call this % comm_ % sync(service_id)
    
    if(CompSplitter % is_first_rank()) then
        call this % comm_ % put_info(this % corr_ga_name(ev), COMM_GA_TIME, ev % time())
        call this % comm_ % put_info(this % corr_ga_name(ev), COMM_GA_STATUS, "full")
    end if
    
end subroutine

!========================================================================================

character(100) function make_filename(this, ev, src_dec, dst_dec)
    use decomposition_module

    class(ServiceCPL) :: this
    type(Event), intent(in) :: ev
    type(Decomposition), intent(in) :: src_dec, dst_dec
    integer :: src_id, dst_id

    src_id = CompSplitter % comp_id(ev % src())
    dst_id = CompSplitter % comp_id(ev % dst())
    
    make_filename = TRIM(ExpInfo % data_dir())//"/"// &
                    TRIM(ModelsInfo(src_id) % model_name())//'_' &
                    //TRIM(src_dec % to_string_2D())// &
                    '_to_'// &
                    TRIM(ModelsInfo(dst_id) % model_name())//'_' &
                    //TRIM(dst_dec % to_string_2D())// &
                    ".nc"
end function


!========================================================================================
 
end module

