

!========================================================================================
! Class RequestQueue.
! Provide MPI client-server interaction by Event's.
!========================================================================================

module service_dtr_module

use utils_module
use actions_module
use service_module
use Int4_vector_module, Int4_vector => VectorStorage
use Int4_data_module
use debug_module

implicit none

type, extends(Service) :: ServiceDTR 
private
    type(Int4_vector) :: all_service_ranks_

 contains  
    !=======================
    ! ===== PUBLIC API =====
    procedure, public :: init => init_dtr
    procedure, public :: destroy => destroy_dtr    
    procedure, public :: handle_request => handle_dtr
    procedure, public :: after_init => after_init_dtr
    
    ! ===== PUBLIC API =====
    !=======================    
end type

 CONTAINS

subroutine init_dtr(this)
    class(ServiceDTR) :: this
    integer :: s, r
    integer, allocatable :: serv_ranks(:)
 
    call this % all_service_ranks_ % init()
 
    ! Get ranks of all services
    do s = 1, SIZE(SERVICE_COMPS)
        if (SERVICE_COMPS(s) == "DTR") cycle;
        call CompSplitter % proc_list(name = SERVICE_COMPS(s), list = serv_ranks)
        do r = 1, SIZE(serv_ranks)
            call this % all_service_ranks_ % append(Int4_data(serv_ranks(r)))
        end do
        deallocate(serv_ranks)
    end do
    
end subroutine

subroutine destroy_dtr(this)
    class(ServiceDTR) :: this

    ! Get ranks of all services
    call this % all_service_ranks_ % destroy() 
    
end subroutine

subroutine after_init_dtr(this)
    class(ServiceDTR) :: this
    
end subroutine

 
!========================================================================================

subroutine handle_dtr(this, ev)
    class(ServiceDTR) :: this
    type(Event), intent(in) :: ev
    integer :: i
    integer :: dummy
    character(STATUS_LEN) :: status
    type(Int4_data) :: node
    integer :: ierr
    integer, allocatable :: err_ranks(:)
    
    ! check if it is valid event
    dummy = ActionHandler % priority(ev % action(), status)
    if (status /= "") call CompSplitter % sync_error("Service DTR: "//TRIM(status), "LCL")
        
    if (ev % action() == "ERROR") then
        call CompSplitter % proc_list(name = ev % dst(), list = err_ranks)
        call this % all_service_ranks_ % destroy()
        call this % all_service_ranks_ % init()
        do i = 1, SIZE(err_ranks)
            call this % all_service_ranks_ % append(Int4_data(err_ranks(i)))
        end do
    end if
    
    ! Just fanout message to all service ranks (not for itself)
    call event_to_mpi_pack(ev, this % ev_packed_)
    
    do i = 1, this % all_service_ranks_ % length()
        node = this % all_service_ranks_ % get(i)
        call MPI_SEND(this % ev_packed_, SIZE(this % ev_packed_), MPI_PACKED, node % value, 1, MPI_COMM_WORLD, ierr)
    end do

    call Debugger % log_msg ("Fanaout: "//TRIM(ev % to_string()))
    
    
end subroutine
 
!========================================================================================
 
end module
