!========================================================================================

module time_label_module
type :: TimeLabel
    real(8) :: start = -1
    real(8) :: finish = -1
    character(3) :: mode = "" 
end type

type(TimeLabel) :: EMPTY_DATA
end module

#define CONCRETE_TYPE TimeLabel
#define CONCRETE_MODULE time_label_module
#define CONCRETE_MODULE_FLIBS time_label_module_
#define OUTPUT_MODULE time_label_hash_module
#include "HashStorage.f90"

!========================================================================================

module statistics_module

use component_splitter_module
use time_label_hash_module, TimeLabelHash => HashStorage
use time_label_module
use mpi
use utils_module
use component_splitter_module

implicit none

type :: Statistics
private
  type(TimeLabelHash) :: labels_
    
    
    
 contains 

    procedure, public :: init => init_stats
    procedure, public :: destroy => destroy_stats

    procedure, public :: init_time_mark
    procedure, public :: pause_time_mark
    procedure, public :: get_time_dif
    procedure, public :: print_time_dif
    
end type

type(Statistics) :: Stats

    
 contains

!========================================================================================
 
subroutine init_stats(this)
    class(Statistics) :: this

    call this % labels_ % init()
    
end subroutine
 
subroutine destroy_stats(this)
    class(Statistics) :: this
   
    call this % labels_ % destroy()
    
end subroutine

!========================================================================================

subroutine init_time_mark(this, label, mode, msg)
    class(Statistics) :: this
    character(*), intent(in) :: label, mode
    character(*), intent(in), optional :: msg
    type(TimeLabel) :: tl

    call CompSplitter % barrier(mode = mode, msg = msg)

    tl % mode = mode
    tl % start = MPI_WTIME()
    call this % labels_ % put(label, tl)
    
end subroutine

subroutine pause_time_mark(this, label)
    class(Statistics) :: this
    character(*), intent(in) :: label
    type(TimeLabel) :: tl

    tl = this % labels_ % get(label)
    
    if (tl % finish == EMPTY_DATA % finish) then
        call CompSplitter % barrier(mode = tl % mode)
        tl % finish = MPI_WTIME()
        call this % labels_ % put(label, tl)
    end if
    
end subroutine

real(8) function get_time_dif(this, label)
    class(Statistics) :: this
    character(*), intent(in) :: label
    type(TimeLabel) :: tl
    integer :: ierr
    
    call this % pause_time_mark(label)

    tl = this % labels_ % get(label)
    if (tl % start == EMPTY_DATA % start) then
        get_time_dif = 0
    else
        get_time_dif = tl % finish - tl % start
    end if
end function

subroutine print_time_dif(this, label, msg)
    class(Statistics) :: this
    character(*), intent(in) :: label, msg
    character(50) :: msg_
    real(8) :: time_dif
    type(TimeLabel) :: tl
   
    msg_ = msg
    time_dif = this % get_time_dif(label)
    tl = this % labels_ % get(label)
    
    call CompSplitter % barrier(mode = tl % mode, msg = msg_//" | "//TRIM(n2s(time_dif))//" sec.")
        
end subroutine
    

!========================================================================================
 
 
end module