
program test_array_info_module
    use test_manager_module
    use array_info_module

    implicit none

    type(TestManager) :: tm

   
    call tm % init("seq")

    call test_helper(tm, il = 2, jl = 3, kl = 4)
    call test_helper(tm, il = 2, jl = 4, kl = 3)
    call test_helper(tm, il = 3, jl = 2, kl = 4)
    call test_helper(tm, il = 3, jl = 4, kl = 2)
    call test_helper(tm, il = 4, jl = 2, kl = 3)
    call test_helper(tm, il = 4, jl = 3, kl = 2)

    call tm % report_tests()
    
    call tm % destroy()
           
 contains
 
 subroutine test_helper(tm, il, jl, kl)
    type(TestManager), intent(in) :: tm
    integer, intent(in) :: il, jl, kl
    real(4) :: arr2D4(il, jl)
    real(4) :: arr3D4(il, jl, kl)
    real(4) :: arr3D4_kij(kl, il, jl)
    type(ArrayInfo) :: inf1, inf2, inf3
    integer :: i 

    !==============
    ! Check 2D case
    !==============
    arr2D4 = reshape((/ (i, i = 1, SIZE(arr2D4) ) /), shape(arr2D4))
    call inf1 % init(name = "arr1", lname = "arr1_long", units = "arr1_units", indx = "ij", arr = arr2D4)
    call tm % assert(inf1 % arr_name() == "arr1", "check 2D array name")
    call tm % assert(inf1 % arr_type() == "real4", "check 2D array type")
    call tm % assert(inf1 % arr_dims() == "2D", "check 2D array dims")
    call tm % assert(ALL(inf1 % p_2D4() == arr2D4), "check 2D pointer")
    call inf1 % set_data(arr2D4)
    call tm % assert(ALL(inf1 % p_2D4() == arr2D4), "set ij data for ij indexing")
    call tm % assert(ALL(arr2D4 == reshape((/ (i, i = 1, SIZE(arr2D4) ) /), shape(arr2D4))), "data is set correctly?")

    !==================
    ! Check 3D ijk case
    !==================
    ! Model data is ijk, is external data ijk?
    arr3D4 = reshape((/ (i, i = 1, SIZE(arr3D4)) /), shape(arr3D4))
    call inf2 % init(name = "arr2", lname = "arr2_long", units = "arr2_units", indx = "ijk", arr = arr3D4)
    call tm % assert(inf2 % arr_name() == "arr2", "check 3D array name")
    call tm % assert(inf2 % arr_type() == "real4", "check 3D array type")
    call tm % assert(inf2 % arr_dims() == "3D", "check 3D array dims")
    call tm % assert(ALL(inf2 % p_3D4() == arr3D4), "get 3D pointer for ijk indexing")

    ! Model data is ijk, set it from external ijk.
    call inf2 % set_data(arr3D4)
    call tm % assert(ALL(inf2 % p_3D4() == arr3D4), "set ijk data for ijk indexing")
    call tm % assert(ALL(arr3D4 == reshape((/ (i, i = 1, SIZE(arr3D4)) /), shape(arr3D4))), "data is set correctly?")

    !==================
    ! Check 3D kij case
    !==================    
    ! Model data is kij, is external data ijk?
    arr3D4_kij = indexing_ijk_to_kij(arr3D4)    
    call inf3 % init(name = "arr3", lname = "", units = "", indx = "kij", arr = arr3D4_kij)
    call tm % assert(ALL(inf3 % p_3D4() == arr3D4), "check 3D pointer for kij indexing")
    
    ! Model data is kij, set it from external ijk.
    call inf3 % set_data(arr3D4)
    call tm % assert(ALL(inf3 % p_3D4() == arr3D4), "set kij data from ijk indexing")
    call tm % assert(ALL(arr3D4_kij == indexing_ijk_to_kij(arr3D4)), "data is set correctly?")
   
 end subroutine
 
   
end