program test_datetime
    use test_manager_module
    use datetime_module
    implicit none
    
    type(TestManager) :: tst
    type(DateTime) :: dat, dat2, start_date, end_date
    logical :: is_ok
    integer(kind=8) :: i
    character(200) :: msg
    
    call tst % init("seq")
    
    dat = DateTime(1980, 3, 15, 12, 15, 11)
    call tst % assert(dat % to_string() == "1980-03-15 12:15:11", "initialize by date")
        
    dat = DateTime("1980-03-15 12:15:11")
    call tst % assert(dat % year == 1980 .AND. dat % month == 3 .AND. dat % day == 15 .AND. &
                        dat % hour == 12 .AND. dat % minute == 15 .AND. dat % second == 11, "initialize by string")

    call tst % assert(is_leap_year(1980), "1980 is leap year")
    call tst % assert(.NOT. is_leap_year(1981), "1981 is not leap year")
    call tst % assert(is_leap_year(2000), "1980 is leap year")
                            
    call tst % assert(dat % year_day() == 31 + 29 + 15, "year_day")
                        
    call tst % assert(dat % year_sec() == (31 + 29 + 15 - 1)*60*60*24 + 12*60*60 + 15*60 + 11, "year_sec")
                        
    call dat % add_time(20, 0, 0)
    call tst % assert(dat % to_string() == "1980-04-04 12:15:11", "add days")
    
    call dat % add_time(0, 26, 0)
    call tst % assert(dat % to_string() == "1980-04-05 14:15:11", "add hours")
    
    call dat % add_time(0, 0, 65)
    call tst % assert(dat % to_string() == "1980-04-05 15:20:11", "add minutes")

    call dat % add_time(s = 61)
    call tst % assert(dat % to_string() == "1980-04-05 15:21:12", "add seconds")
    
    call dat % add_time(366, 25, 61)
    call tst % assert(dat % to_string() == "1981-04-07 17:22:12", "add time")

    dat2 = sec2date(date2sec(dat))
    call tst % assert(dat2 % to_string() == dat % to_string(), "dat2num and num2date")
    
    call tst % assert(sec2date(date2sec(DateTime("2001-01-01 00:00:00"))) == DateTime("2001-01-01 00:00:00"), "check 1st of January")
    call tst % assert(sec2date(date2sec(DateTime("2001-01-01 00:00:46"))) == DateTime("2001-01-01 00:00:46"), "check 1st of January after champaign")
    call tst % assert(sec2date(date2sec(DateTime("2001-01-02 00:00:00"))) == DateTime("2001-01-02 00:00:00"), "check 2st of January")

    
    call tst % assert(date2sec(sec2date(31536000)) == 31536000, "check New Year")
    
!   This is very long test (all seconds in year), uncomment it if you want to be sure about DateTime  
    start_date = DateTime("1948-12-30 11:00:00")
    end_date =   DateTime("1949-01-02 12:00:00")
    msg = "check all secs in ["// start_date % to_string() // ", "// end_date % to_string()//"]"
    do i = date2sec(start_date), date2sec(end_date)
        if (date2sec(sec2date(i)) /= i) then
            call tst % assert(.false., msg)
            EXIT
        end if
    end do
    call tst % assert(.true., msg)
            
    call tst % report_tests()
    call tst % destroy()    
                      
end