    
program test_file_handler_nc
    use test_global_domain_module
    use test_manager_module
   
    use component_splitter_module
    use communicator_ga_module    
    use decomposition_module
    
    implicit none

    integer, parameter :: il = 180, jl = 160, kl = 10
    integer, parameter :: rank_check = 0

    type(TestGlobalDomain) :: tgd
    type(TestManager) :: tm
    type(Communicator) :: comm_ga    
    type(Decomposition) :: ocn_decomp
    integer :: ocn_id, cpl_id
    integer, allocatable :: proc_list_ocn(:), proc_list_cpl(:)
    logical :: ans  
    integer :: w, e, s, n  
      
    call CompSplitter % init(COMMAND_ARGUMENT_COUNT()/2)
    ocn_id = CompSplitter % comp_id("OCN")
    cpl_id = CompSplitter % comp_id("CPL")
    
    call tm % init("par")

    call CompSplitter % show_memory_usage("after CompSplitter % init", 0)
    
!========================================================================================    
    
    ! Init comm, add groups
    
    call comm_ga % init(CompSplitter % comp_defined(), CompSplitter % comm_local_size())
    call tm % assert(.TRUE., "initialize")
    
    call CompSplitter % proc_list("OCN", list = proc_list_ocn)
    call CompSplitter % proc_list("CPL", list = proc_list_cpl)
    
    call comm_ga % init_group(ocn_id, proc_list_ocn, cpl_id, proc_list_cpl)
    call tm % assert(.TRUE., "add group for OCN and CPL")
    
    
    ! Ocn make decomposition and broadcast it to everyone
    if (CompSplitter % i_am() == "OCN") then
        ocn_decomp = Decomposition(il, jl, kl, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local())
    end if
    call ocn_decomp % broadcast(CompSplitter % proc_first("OCN"), CompSplitter % comm_world())
    call tgd % init(ocn_decomp, t_size = 10, rank_local = rank_check)
    w = ocn_decomp % west(rank_check)
    e = ocn_decomp % east(rank_check)
    s = ocn_decomp % south(rank_check)
    n = ocn_decomp % north(rank_check)
    
    call CompSplitter % show_memory_usage("after TestGlobalDomain", 0)
    
!======================================================================================== 

    ! Put/get test for 2D
    call comm_ga % sync(ocn_id, cpl_id)
    call tm % assert(.true., "perform synchronize")

    call comm_ga % init_array(arr_name = "glob_2D", datatype = "real4", dimnum = "2D", holder_id = ocn_id, holder_decomp = ocn_decomp, subscriber_id = cpl_id)
    call tm % assert(.TRUE., "initialize 2D shared global array")
    
    if (CompSplitter % i_am() == "CPL") then
        call comm_ga % put("glob_2D", (/ 1, 1 /), (/ il, jl /), tgd % glob(:,:,1,1))
    !    call comm_ga % acc("glob_2D", (/ 1, 1 /), (/ il, jl /), tgd % glob(:,:,1,1), real(1.) )
    !    call comm_ga % acc("glob_2D", (/ 1, 1 /), (/ il, jl /), tgd % glob(:,:,1,1), real(-1.) )
    end if

    call tm % assert(.TRUE., "CPL put 2D global data")
    
    call comm_ga % sync(ocn_id, cpl_id)
    call tm % assert(.TRUE., "synchronize")
        
    call comm_ga % get("glob_2D", (/ w, s /), (/ e, n /), tgd % loc_2D)
    !write(*,*), 'glob', tgd % glob(1,1,1,1), ' loc ', tgd % loc_2D( w, s )

    call tm % assert(tgd % is_correct(arr_type = "2D"), "all get 2D local patch defined in parameters")

! !========================================================================================    

    ! Put/get test for 2D kind 8
    call comm_ga % init_array(arr_name = "glob_2D8", datatype = "real8", dimnum = "2D", holder_id = ocn_id, holder_decomp = ocn_decomp, subscriber_id = cpl_id)
    call tm % assert(.TRUE., "Initialize 2D real8 shared global array")
    
    if (CompSplitter % i_am() == "CPL") then
        call comm_ga % put("glob_2D8", (/ 1, 1 /), (/ il, jl /), REAL(tgd % glob(:,:,1,1), kind = 8))
    !    call comm_ga % acc("glob_2D8", (/ 1, 1 /), (/ il, jl /), REAL(tgd % glob(:,:,1,1), kind = 8), real( 1 , kind = 8) )
    !    call comm_ga % acc("glob_2D8", (/ 1, 1 /), (/ il, jl /), REAL(tgd % glob(:,:,1,1), kind = 8), real( -1 , kind = 8))
    endif
    call tm % assert(.TRUE., "CPL put 2D global data")

    call comm_ga % sync(ocn_id, cpl_id)
    call tm % assert(.TRUE., "synchronize")
    
    call comm_ga % get("glob_2D8", (/ w, s /), (/ e, n /), tgd % loc_2D8)
    call tm % assert(tgd % is_correct(arr_type = "2D8"), "all get 2D8 local patch defined in parameters")
 
! !========================================================================================    
       
    ! Put/get test for 3D
    call comm_ga % init_array(arr_name = "glob_3D", datatype = "real4", dimnum = "3D", holder_id = ocn_id, holder_decomp = ocn_decomp, subscriber_id = cpl_id)
    call tm % assert(.TRUE., "Initialize 2D real8 shared global array")
          
    if (CompSplitter % i_am() == "OCN") call comm_ga % put("glob_3D", (/ 1, 1, 1 /), (/ il, jl, kl /), tgd % glob(:,:,:,1))          
    call tm % assert(.TRUE., "OCN put 3D global data")

    call comm_ga % sync(ocn_id, cpl_id)
    call tm % assert(.TRUE., "synchronize")
        
    call comm_ga % get("glob_3D", (/ w, s, 1 /), (/ e, n, kl /), tgd % loc_3D)                   
    call tm % assert(tgd % is_correct(arr_type = "3D"), "all get 3D local patch defined in parameters")
! 
!     
! !========================================================================================    
    ! Test for only OCN array
            
    call comm_ga % init_group(ocn_id, proc_list_ocn)
    call tm % assert(.TRUE., "add group for only OCN")
            
    ! This cause WARINNG about memory reallocation when ga_allocate. #TODO
    ! Only from np > 1, and for private arrays
    if (CompSplitter % i_am() == "OCN") call comm_ga % init_array(arr_name = "priv_2D", datatype = "real4", dimnum = "2D", &
                                            holder_id = ocn_id, holder_decomp = ocn_decomp)
    call tm % assert(.TRUE., "initialize private global ocean array")
    
    if (CompSplitter % i_am() == "OCN") call comm_ga % put("priv_2D", (/ 1, 1 /), (/ il, jl /), tgd % glob(:,:,1,1))
    call tm % assert(.TRUE., "OCN put 2D global data")
        
    if (CompSplitter % i_am() == "OCN") call comm_ga % sync(ocn_id)
    call tm % assert(.TRUE., "synchronize")
    
    if (CompSplitter % i_am() == "OCN") then
        call comm_ga % get("priv_2D", (/ w, s /), (/ e, n /), tgd % loc_2D)
        ans = tgd % is_correct(arr_type = "2D")
    end if
    call tm % assert(CompSplitter % i_am() /= "OCN" .OR. ans, "OCN get 2D local patch defined in parameters")
!         
! !========================================================================================    
    ! Test for shared array with customized dims
    
    ! Dont forget to set group back
!     call comm_ga % set_group(ocn_id, cpl_id)

    
    call comm_ga % init_array(arr_name = "glob_2D_cust", datatype = "real4", dim1_len = il + 2, dim2_len = jl + 2, &
                              holder_id = ocn_id, holder_size = CompSplitter % comm_local_size("OCN"), subscriber_id = cpl_id)
    call tm % assert(.TRUE., "initialize 2D shared global array with customized dims")
    
    if (CompSplitter % i_am() == "CPL") call comm_ga % put("glob_2D_cust", (/ 1, 1 /), (/ il, jl /), tgd % glob(:,:,1,1))    
    call tm % assert(.TRUE., "CPL put 2D some data")

    call comm_ga % sync(ocn_id, cpl_id)
    call tm % assert(.TRUE., "synchronize")
    
    call comm_ga % get("glob_2D_cust", (/ w, s /), (/ e, n /), tgd % loc_2D)   
    call tm % assert(tgd % is_correct(arr_type = "2D"), "all get 2D local patch defined in parameters")    
!      
!========================================================================================    
    ! Test info field
! 

    call tm % assert(TRIM(comm_ga % get_info("glob_2D", COMM_GA_STATUS)) == "free", "check status of new array")
    
    call comm_ga % put_info(arr_name = "glob_2D", field = COMM_GA_STATUS, value = "hello")
    call tm % assert(TRIM(comm_ga % get_info("glob_2D", COMM_GA_STATUS)) == "hello", "check status")
    
    call comm_ga % put_info(arr_name = "glob_2D", field = COMM_GA_TIME, value = "1900-01-01 12:15:00")
    call tm % assert(TRIM(comm_ga % get_info("glob_2D", COMM_GA_TIME)) == "1900-01-01 12:15:00", "check time")
!     

!========================================================================================    
    ! WARNING 
    ! This test doesn't work for strange reason: cpl_id became equal to 0
    ! If you place write(*,*) cpl_id there - everything is fine
    ! Looks like memory corruption somewhere TODO

    ! Create array with same name but other size
    
!     call comm_ga % put_info(arr_name = "glob_2D_cust", field = COMM_GA_STATUS, value = "hello")
    
!     call comm_ga % init_array(arr_name = "glob_2D_cust", datatype = "real4", dim1_len = il + 20, dim2_len = jl + 20, &
!                               holder_id = ocn_id, holder_size = CompSplitter % comm_local_size("OCN"), subscriber_id = cpl_id)
                              
!     call tm % assert(TRIM(comm_ga % get_info("glob_2D_cust", COMM_GA_STATUS)) == "free", "create array with same name but other size")
                                                            
    call comm_ga % destroy_array("glob_2D_cust")
    call tm % assert(.TRUE., "destroy_array")          
    
!========================================================================================  
    ! Finalize

    deallocate(proc_list_cpl, proc_list_ocn)


    call comm_ga % destroy()
    call tm % assert(.TRUE., "destroy communicator")    
    
    call tm % report_tests()
    call tm % destroy()
    
end program
