    
program test_halo_updater
    use test_manager_module
    use component_splitter_module
    use decomposition_module
    use halo_updater_module
    use test_global_domain_module
    use utils_module
    implicit none
    
    integer, parameter :: il = 180, jl = 160, kl = 2    
    type(TestManager) :: tm
    
    type(Decomposition) :: decomp
    type(TestGlobalDomain) :: global  
    integer :: wb, eb, sb, nb, kb, ierr
    integer :: test_halo
    real(4), allocatable :: adapter_kij_4(:,:,:)
    integer, allocatable :: land_mask(:,:)
    
    call CompSplitter % init(COMMAND_ARGUMENT_COUNT()/2)
    call tm % init("par")

    decomp = Decomposition(il, jl, kl, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local())
    if (tm % rank() == 0) call decomp % show()
    
! ===========================================
! Simple case: halo_width = 1, all flags, 2D
! ===========================================
    
    call test_helper(1, 1, .false., .false., "T")
    call test_helper(1, 1, .true.,  .false., "T")
    call test_helper(1, 1, .false., .true.,  "T")
    call test_helper(1, 1, .true.,  .true.,  "T")
    
    call test_helper(1, 1, .false., .false., "V")
    call test_helper(1, 1, .true.,  .false., "V")
    call test_helper(1, 1, .false., .true.,  "V")
    call test_helper(1, 1, .true.,  .true.,  "V")
  
!=======================================
! halo_width = 3
!=======================================  
    
    call test_helper(3, 1, .true., .true., "T")
    call test_helper(3, 2, .true., .true., "T")
    call test_helper(3, 3, .true., .true., "T")
    
    call test_helper(3, 1, .true., .true., "V")
    call test_helper(3, 2, .true., .true., "V")
    call test_helper(3, 3, .true., .true., "V")
    
!=======================================
! Masked halo tests
!=======================================  

    ! Make Decomposition for this mask to get new borders
    decomp = Decomposition(il, jl, kl, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local())

    ! Create mask
    allocate(land_mask(decomp % inp(), decomp % jnp())); land_mask = 1
    land_mask(1,1) = 0    
        
    if (tm % rank() == 0) call decomp % show()
        
    ! Call tests cases
    call test_helper(1, 1, .false., .false., "V", land_mask)
    call test_helper(1, 1, .true.,  .false., "V", land_mask)
    call test_helper(1, 1, .false., .true.,  "V", land_mask)
    call test_helper(1, 1, .true.,  .true.,  "V", land_mask)

    call test_helper(1, 1, .false., .false., "T", land_mask)
    call test_helper(1, 1, .true.,  .false., "T", land_mask)
    call test_helper(1, 1, .false., .true.,  "T", land_mask)
    call test_helper(1, 1, .true.,  .true.,  "T", land_mask)
    
    deallocate(land_mask)
    
! Finalize
    call HaloUpdateMaster % destroy()

    call global % destroy()
    call tm % report_tests()
    call tm % destroy()
    
 contains

subroutine test_helper(max_halo_width, test_halo, is_icycle, is_bsc, grid_type, land_mask) 
    integer, intent(in) :: max_halo_width, test_halo
    logical, intent(in) :: is_icycle, is_bsc
    integer, intent(in), optional :: land_mask(:,:)
    character(1), intent(in) :: grid_type
    character(100) :: descr
    
    descr = "width = "//TRIM(n2s(test_halo))//" of max_halo_width = "//TRIM(n2s(max_halo_width))//" ssc: "//MERGE("T", "F", is_icycle)//" bsc: "//MERGE("T", "F", is_bsc)
    
    ! Rebuild decomp
    decomp = Decomposition(il, jl, kl, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local(), is_icycle = is_icycle, is_bsc = is_bsc)
    if (PRESENT(land_mask)) then
        ! Apply some test mask, if present
        call decomp % build_proc_decomp(land_mask)
        
        ! If some rank is cut by decomp mask, just assert .TRUE.
        if (ALL(decomp % ij_by_rank(CompSplitter % rank_local()) == (/ -1, -1/))) then
            call tm % assert(.TRUE., "")
            call tm % assert(.TRUE., "")
            call tm % assert(.TRUE., "")
            call tm % assert(.TRUE., "")
            return
        end if
    end if

    ! Get short boundaries for comparison
    wb = decomp % west()
    eb = decomp % east()
    sb = decomp % south()
    nb = decomp % north()

    ! Initialize global domain object for comparison
    call global % init(decomp, t_size = 1, max_halo_width = max_halo_width, rank_local = CompSplitter % rank_local())
    call global % halo_update(grid_type)
    
    ! Initialize updater
    call HaloUpdateMaster % init(decomp, max_halo_width)
    call tm % assert(.TRUE., "initialize max_halo_width: "//n2s(max_halo_width))

    ! Udpdate and global update 2D
    call global % reset_local_arrays("halo")
    call HaloUpdateMaster % update(global % loc_2D(wb-test_halo:eb+test_halo, sb-test_halo:nb+test_halo), test_halo, grid_type)
    call tm % assert(global % is_correct(arr_type = "2D", halo_width = test_halo), "2D real(4): "//TRIM(descr))

    ! Udpdate and global update 2D8
    call global % reset_local_arrays("halo")
    call HaloUpdateMaster % update(global % loc_2D8(wb-test_halo:eb+test_halo, sb-test_halo:nb+test_halo), test_halo, grid_type)
    call tm % assert(global % is_correct(arr_type = "2D8", halo_width = test_halo), "2D real(8): "//TRIM(descr))
        
    ! Now HaloUpdaterT receives 3D array in (k,i,j) indexing, so revert indexing
    if (allocated(adapter_kij_4)) deallocate(adapter_kij_4)
    allocate(adapter_kij_4(kl, wb - test_halo : eb + test_halo, sb -test_halo : nb+ test_halo))    
    call global % reset_local_arrays("halo")
    adapter_kij_4 = 0
    adapter_kij_4 = indexing_ijk_to_kij(global % loc_3D(wb - test_halo : eb + test_halo, sb -test_halo : nb+ test_halo, :))
    call HaloUpdateMaster % update(adapter_kij_4, test_halo, grid_type)
    global % loc_3D(wb - test_halo : eb + test_halo, sb -test_halo : nb+ test_halo, :) = indexing_kij_to_ijk(adapter_kij_4)
    call tm % assert(global % is_correct(arr_type = "3D", halo_width = test_halo), "3D real(4): "//TRIM(descr))
    
end subroutine

end