program test_mapper_bench
    use test_manager_module
    use mapper_module
    use communicator_ga_module
    use test_global_domain_module
    use file_handler_nc_module
    implicit none
    
    type(FileHandler_NC) :: handler
    type(TestManager) :: tm
    type(Decomposition) :: src_decomp_par, dst_decomp_par
    type(Communicator) :: comm_ga
    type(Mapper) :: map_par
   
    character(100) :: size_arg, filename
    character(100) :: src_name, dst_name
    integer :: src_i, src_j, dst_i, dst_j
    
    real, allocatable :: src_data_global(:,:), src_data_global_1D(:), dst_data_global_1D(:)
    integer :: i, link
    
    integer, allocatable :: src_addr(:), dst_addr(:)
    real (kind = 8), allocatable :: wts_map(:,:)
    integer :: num_of_cycles
    real(8) :: start_time
    
    num_of_cycles = 100
        
    call tm % init("par")
    
    ! Read command line args
    if (tm % rank() == 0 .AND. COMMAND_ARGUMENT_COUNT() /= 6) then
        write(*,*) "Wrong arguments. Usage: ./exe OCN 180 90 ATM_NCAR 192 94"
        call exit(1)
    end if
    
    call GET_COMMAND_ARGUMENT(1, src_name)
    call GET_COMMAND_ARGUMENT(2, size_arg)    
    read(size_arg, *) src_i
    call GET_COMMAND_ARGUMENT(3, size_arg)    
    read(size_arg, *) src_j
    call GET_COMMAND_ARGUMENT(4, dst_name)
    call GET_COMMAND_ARGUMENT(5, size_arg)    
    read(size_arg, *) dst_i
    call GET_COMMAND_ARGUMENT(6, size_arg)    
    read(size_arg, *) dst_j


    ! Allocate global data
    allocate(src_data_global(src_i, src_j))
       
    ! Build decompositions and filename
    src_decomp_par = Decomposition(src_i, src_j, 10, "1D", tm % comm_size(), tm % rank())
    dst_decomp_par = Decomposition(dst_i, dst_j, 10, "1D", tm % comm_size(), tm % rank())
    
    filename = "./data/"//TRIM(src_name)//'_'//TRIM(src_decomp_par % to_string_2D())// &
            '_to_'//TRIM(dst_name)//'_'//TRIM(dst_decomp_par % to_string_2D())//".nc"

    call tm % assert(.TRUE., "Data allocation")
            
    ! Prepare communicator
    call comm_ga % init(1, tm % comm_size())
    call comm_ga % init_group(1, (/ (i, i = 0, tm % comm_size()-1) /))    
    call comm_ga % init_array("src_data_global", "real4", "2D", 1, src_decomp_par, 1)

    call tm % assert(.TRUE., "Init communicator")
    
    ! Put test data to src array
    src_data_global = tm % rank()
    call comm_ga % put("src_data_global", (/ 1, 1 /), (/ src_i, src_j /), src_data_global)
    call comm_ga % sync(1, 1)    

    call tm % assert(.TRUE., "Put test data to GA")

    !=================================
    ! Parallel interpolation by Mapper
    !=================================
    call map_par % init(src_decomp_par, dst_decomp_par)
    call tm % assert(.TRUE., "Initialize parallel")
    
    call map_par % read_and_analize(filename)
    call tm % assert(.TRUE., "Read and analize")
    
    ! Perform num_of_cycles test interpolation
    call tm % barrier()
    start_time = MPI_WTIME()
    do i = 1, num_of_cycles
        call map_par % interpolate(comm_ga % id("src_data_global"))
    end do
    call tm % barrier()
    if (tm % rank() == 0) write(*,*) "Benchmark time with ", num_of_cycles, " cycles : ", MPI_WTIME() - start_time

    call map_par % destroy()
    call tm % assert(.TRUE., "Destroy")     

    call tm % report_tests()
    
    call comm_ga % destroy()
    call tm % destroy()    
    
end program