
program test_file_handler_nc
    use event_scheduler_module
    use test_manager_module
    use var_event_normal_module
    use var_event_synced_module
    use event_vector_module, VectorEvent => VectorStorage 
    use event_module
    use array_info_module
    
    implicit none

    type(TestManager) :: tm
    type(EventScheduler) :: evs
    type(VarEventNormal), pointer :: ve_normal
    type(VarEventSynced), pointer :: ve_synced
    
    type(Event) :: ev
    type(VectorEvent) :: ev_vector
    type(DateTime) :: dt
    type(FileHandler_NC) :: fh   
    type(ArrayInfo) :: inf
    real(4) :: arr2D(2,2)
   
    call tm % init("seq")
  
    !========================
    ! Test normal event
    !========================
    ve_normal => new_VarEventNormal(ev, DateTime("1900-02-01 00:00:00"), dd = 1, dh = 0, dm = 0, ds = 0)

    call tm % assert(ve_normal % update(DateTime("1900-01-01 00:00:00"), ev) == .FALSE., "past event is not raised")
    call tm % assert(ve_normal % update(DateTime("1900-02-01 00:00:00"), ev), "zero event is raised")
    call tm % assert(ve_normal % update(DateTime("1900-03-01 00:00:00"), ev), "periodic event is raised")
    call tm % assert(ve_normal % update(DateTime("1900-03-01 00:00:12"), ev) == .FALSE., "non periodic event is not raised")
    deallocate(ve_normal)

    !========================
    ! Test synced event
    !========================
    
    ! Make fake FD file
    call fh % create_file("test.nc")
    call fh % create_dim("i", 1)
    call fh % create_time_dim()
    call fh % create_var("SNOW", "real4", "i", dimt_name = "TIME") 
    call fh % put( (/ 1 /), (/ 1 /), (/ 1 /), dt = DateTime("1900-01-02 00:00:00"))
    call fh % put( (/ 1 /), (/ 1 /), (/ 1 /), dt = DateTime("1900-01-05 00:00:00"))
    call fh % put( (/ 1 /), (/ 1 /), (/ 1 /), dt = DateTime("1900-01-08 00:00:00"))
    call fh % put( (/ 1 /), (/ 1 /), (/ 1 /), dt = DateTime("1900-02-14 00:00:00"))
    
    ! Test
    call inf % init("SNOW", "", "", indx = "ij", arr = arr2D)
    ve_synced => new_VarEventSynced(Event(arr_info = inf, act = "READ_FD", owner = "", src = "test.nc"), DateTime("1900-01-06 00:00:00"))
 
    call tm % assert(ve_synced % update(DateTime("1988-01-05 00:00:00"), ev) == .FALSE., "nothing happens on this date")
    call tm % assert(ve_synced % update(DateTime("1988-01-08 00:00:00"), ev) == .TRUE., "raise event on new date")
    call tm % assert(ev % time_rec() == "1900-01-08 00:00:00", "time_rec of it equal to record time")

    deallocate(ve_synced)
    
    !========================
    ! Test container
    !========================
    
    call evs % init()
  
    call evs % add(new_VarEventNormal(Event(arr_info = inf, act = "SAVE_DG", owner = ""), &
                                          DateTime("1900-02-01 00:00:00"), dd = 1, dh = 0, dm = 0, ds = 0))
    call evs % add(new_VarEventSynced(Event(arr_info = inf, act = "READ_FD", owner = "", &
                                          src = "test.nc"), DateTime("1900-02-01 00:00:00")))
    
    call evs % gather_events(DateTime("1900-02-14 00:00:00"), ev_vector)

    ev = ev_vector % get(1)
    call tm % assert(ev % time() == "1900-02-14 00:00:00" .AND. ev % action() == "READ_FD", "FD raised")

    ev = ev_vector % get(2)
    call tm % assert(ev % time() == "1900-02-14 00:00:00" .AND. ev % action() == "SAVE_DG" , "DG raised")
    
    call ev_vector % destroy()
    
    call evs % destroy()
    
    !========================    
    ! Finalize
    !========================
    
    if (tm % rank() == 0) call SYSTEM("rm -f *.nc")  
    call tm % report_tests()
    call tm % destroy()    
        
end program