#include "o_defines.f90"
#include "ocn_2Ddomain"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------



	module cpl_ocn_module
	
	use comp_module
	use o_grid_module
	use o_par_module
	use o_driver_module
	use o_analysis_module
	use o_diagn_means_module
	use o_diagn_basic_module
	use o_diagn_surface_module
	use o_diagn_mld_module
	use o_diagn_a2o_module
	implicit none
	
!==================================================================================
! Define type 'ocn', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_ocn
	
	CONTAINS
		PROCEDURE :: ini_params => o_ini_params
		PROCEDURE :: ini_reg_comp => o_ini_reg_comp
		PROCEDURE :: ini_reg_data => o_ini_reg_data
		PROCEDURE :: ini_allocate => o_ini_allocate
		PROCEDURE :: ini_main => o_ini_main
		PROCEDURE :: main_step => o_main_step
	END TYPE cpl_ocn	

	CONTAINS
	
!================================================================================================================================================================
	subroutine o_ini_params(this)

		class(cpl_ocn) :: this
		call o_par_init()
		call o_driver_ini_params
	
	end subroutine o_ini_params

!================================================================================================================================================================
	subroutine o_ini_reg_comp(this)

		class(cpl_ocn) :: this
		call this % reg_model(	reg_np = run_np(COMP_OCN),				&
					reg_i_size = il-2, 					&
					reg_j_size = jl-2,					&
					reg_k_size = kl,					&
					reg_m_size = ml,					&
					reg_decomp_type_2D = .TRUE.,				&
					reg_i_cycle = MERGE(.TRUE.,.FALSE.,i_cycle == 1),	&
					reg_bsc_grid=MERGE(.TRUE.,.FALSE.,bsc_flag == 1),	&
					reg_time_steps_day = ocn_steps_in_day,			&
					reg_time_save_it = 0,				&
					reg_read_cp = ocn_read_cp,				&
					reg_halo_stencil = 2,					&
					reg_halo_wide = 1					)
		call o_driver_subdomains(cmp % west, cmp % east, cmp % south, cmp % north)
				
	end subroutine o_ini_reg_comp

!================================================================================================================================================================
	
	subroutine o_ini_reg_data(this)
		
	class(cpl_ocn) :: this
	character(100) field_name
	character (len=2) name_k
	character (len=4) name_z

!	write(*,*) "iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)",iwest(myrank),ieast(myrank),jsouth(myrank),jnorth(myrank)
!	write(*,*) "o_ini_reg_data(this)"
	write(*,*) 'ocn-ice mapping periods ',ocn_ice_mapping,ice_ocn_mapping

!-------
!	OCN. ACTION_READ_IC
!-------
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM2_arr=kv_real			(  odom	 ),name="kv")
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM2_arr=kt_real			(  odom	 ),name="kt")
	
#if ANALYSIS_FLAG == 1
	!call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM3_main_kind_arr=t_c		(:,odom,1),name="t_an")
	!call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM3_main_kind_arr=t_c		(:,odom,2),name="s_an")
	
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=mht_adv_v_mean % collector3d(:,odom),name='mht_adv_v_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_kind8_arr=v_c			(:,odom  ),name='v_run_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_kind8_arr=t_c			(:,odom,1),name='t_run_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=ssl_c		(  odom  ),name='ssl_run_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=v_c		(1,odom  ),name='test_input')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=v_c		(2,odom  ),name='test_output')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_arr=cv			(:,odom  ),name='cv')
#elif ANALYSIS_FLAG == 0	
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM3_main_kind_arr=t_c		(:,odom,1),name="t_an")
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_IC, DIM3_main_kind_arr=t_c		(:,odom,2),name="s_an")
!-------
!	OCN. ACTION_READ_CP
!-------
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM3_main_kind_arr=t_c		(:,odom,1),name='t1_cp_save' 	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM3_main_kind_arr=t_c		(:,odom,2),name='t2_cp_save' 	)
!	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM4_main_kind_arr=t_c		(:,odom,:),name='t_cp_save' 	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM3_main_kind_arr=u_c		(:,odom  ),name='u_cp_save' 	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM3_main_kind_arr=v_c		(:,odom	 ),name='v_cp_save' 	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_main_kind_arr=w_surf_c	(  odom	 ),name='wsurf_cp_save')
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_kind8_arr=u_bt_a_f		(  odom	 ),name='ubt_cp_save'	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_kind8_arr=v_bt_a_f		(  odom	 ),name='vbt_cp_save'	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_kind8_arr=h_bt_a_f		(  odom	 ),name='hbt_cp_save'	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_kind8_arr=ssl_c		(  odom	 ),name='ssl_cp_save'	)
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_arr=hice			(  odom	 ),name='hice_cp_save')
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_arr=aice			(  odom	 ),name='aice_cp_save')
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_main_kind_arr=dx3_v_l	(2,odom	 ),name='dx3_v_l_cp_save')
	!call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_arr=tice			(  odom	 ),name='tice')	!diagnostic
	
!-------
!	OCN. ACTION_SAVE_CP
!-------
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_main_kind_arr=t_c		(:,odom,1),name='t1_cp_save' 	)
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_main_kind_arr=t_c		(:,odom,2),name='t2_cp_save' 	)
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_main_kind_arr=u_c		(:,odom  ),name='u_cp_save' 	)
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_main_kind_arr=v_c		(:,odom	 ),name='v_cp_save' 	)
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_main_kind_arr=w_surf_c	(  odom	 ),name='wsurf_cp_save')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=u_bt_a_f		(  odom	 ),name='ubt_cp_save' )
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=v_bt_a_f		(  odom	 ),name='vbt_cp_save' )
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=h_bt_a_f		(  odom	 ),name='hbt_cp_save' )
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_kind8_arr=ssl_c		(  odom	 ),name='ssl_cp_save' )
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=hice			(  odom	 ),name='hice_cp_save')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=aice			(  odom	 ),name='aice_cp_save')	
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_main_kind_arr=dx3_v_l		(2,odom  ),name='dx3_v_l_cp_save' )

!-------
!	OCN. ACTION_SAVE_DG	OCN solution
!-------
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_kind8_arr=ssl_c	(odom),period=dg_period,name='ssl',long_name='SSL')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_kind8_arr=h_bt_a_f	(odom),period=dg_period,name='h_bt_a_f' )	
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_main_kind_arr=w_surf_c(odom),period=dg_period,name='w_surf')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM3_arr=kh	(:,odom,1),period=dg_period,name='kh',long_name='KH')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM3_arr=test3d(:,odom),period=dg_period,name='slopetest',long_name='slopetest')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM3_main_kind_arr=t_c(:,odom,1),period=dg_period,name='T3d',long_name='T3d')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=aice		(odom),period=dg_period,name='Aice')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=hice		(odom),period=dg_period,name='Hice')
	
	do k = 1, kl
	if (k==1 .or. k==5 .or. k==17 .or. k==24) then
		write (name_k, '(i2.2)') k 
		write (name_z, '(i4.4)') int(z(k)) 
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_main_kind_arr=t_c	(k,odom,1),period=dg_period,name='t'//name_k,long_name='T-'//name_z//'m',units='deg.C')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_main_kind_arr=t_c	(k,odom,2),period=dg_period,name='s'//name_k,long_name='S-'//name_z//'m',units='psu')
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_main_kind_arr=u_c	(k,odom  ),period=dg_period,name='u'//name_k,long_name='U-'//name_z//'m',units='m/s')
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_main_kind_arr=v_c	(k,odom  ),period=dg_period,name='v'//name_k,long_name='V-'//name_z//'m',units='m/s')
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_collector_kind_arr=t_mean % collector3d(k,odom),period=dg_period,name='t_mean_'//name_k,long_name='T-'//name_z//'m',units='deg.C')
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_collector_kind_arr=s_mean % collector3d(k,odom),period=dg_period,name='s_mean_'//name_k,long_name='S-'//name_z//'m',units='psu')
	end if
	end do
		
	!------------For eddy experiments-------------
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=mht_subgrid_u_mean % collector3d(:,odom),name='mht_subgrid_u_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=mht_subgrid_v_mean % collector3d(:,odom),name='mht_subgrid_v_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=mht_adv_u_mean % collector3d(:,odom),name='mht_adv_u_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=mht_adv_v_mean % collector3d(:,odom),name='mht_adv_v_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=ke_mean % collector3d(:,odom),name='ke_mean')
	
	!-------------Basic mean fields---------------
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=u_mean % collector3d(:,odom),name='u_mean')	
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=v_mean % collector3d(:,odom),name='v_mean')	
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=t_mean % collector3d(:,odom),name='t_mean')	
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM3_collector_kind_arr=s_mean % collector3d(:,odom),name='s_mean')		
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=ssl_mean % collector2d(odom),name='ssl_mean')		

	!--------------Surface fluxes and parameters-----------------
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=ssr_mean % collector2d(odom),name='ssr_mean')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=str_mean % collector2d(odom),name='str_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=sns_mean % collector2d(odom),name='sns_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=ltn_mean % collector2d(odom),name='ltn_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=prec_mean % collector2d(odom),name='prec_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=evap_mean % collector2d(odom),name='evap_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=wind_stress_x_t_mean % collector2d(odom),name='windsterss_x_t_mean')		
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=wind_stress_y_t_mean % collector2d(odom),name='windsterss_y_t_mean')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=total_surf_heat_mean % collector2d(odom),name='total_surf_hea_mean')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_collector_kind_arr=under_mld_ssr_mean % collector2d(odom),name='under_mld_ssr_mean')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=tair02(odom),name='tair02')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=tice(odom),name='tice')
	
	!--------------mixed layer depth-----------------------------
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=mld(odom),name='mld')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=mld_int(odom),name='mld_int')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=mld_max(odom),name='mld_max')	
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=convadjes_perstep(odom),name='convadjes_perstep')	
	
	if(TRIM(atm_model_name) == 'ATM_NCAR') then

		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom),period=6*60,name='T_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom),period=6*60,name='Q_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom),period=6*60,name='U_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom),period=6*60,name='V_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=LWDN_MOD_atm	(odom),period=6*60,name='LWDN' )
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=SWDN_MOD_atm	(odom),period=6*60,name='SWDN' ) 	
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=RAIN_atm	(odom),period=6*60,name='RAIN')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=SNOW_atm	(odom),period=6*60,name='SNOW' )
			
	else if(atm_model_name == 'ATM_INTERIM') then
	
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom), period = 6*60, name = 't2m')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom), period = 6*60, name = 'd2m')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom), period = 6*60, name = 'u10')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom), period = 6*60, name = 'v10') 	
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=SNOW_atm	(odom), period = 6*60, name = 'sst') 	!Interim SST
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=LWDN_MOD_atm	(odom), period = 6*60, name = 'strd' )
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=SWDN_MOD_atm	(odom), period = 6*60, name = 'ssrd' ) 	
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=RAIN_atm	(odom), period = 6*60, name = 'tp')	!Interim full precipitation
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=aice_atm	(odom), period = 6*60, name = 'ci')
		!call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=runoff	(odom), period = 6*60, name = 'ro' )

		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=tair02	(odom), period = dg_period, name = 'tair02')
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=tdew02	(odom), period = dg_period, name = 'tdew02')
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=wind_speed_x_t	(odom), period = dg_period, name = 'wind_speed_x_t')
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=wind_speed_y_t	(odom), period = dg_period, name = 'wind_speed_y_t') 	
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=str	(odom), period = dg_period, name = 'str' )
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=ssr	(odom), period = dg_period, name = 'ssr' ) 	
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=prec	(odom), period = dg_period, name = 'prec')
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=nudge_sst(odom),period = dg_period, name = 'sst' )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=temp1	(odom), period=dg_period,name='aice_diff')
		call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=nudge_aice(odom),period = dg_period, name = 'nudge_aice' )
		
		
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=temp1	(odom),name='sst_diff')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=SNOW_atm	(odom),name='sst_raw')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=nudge_sst	(odom),name='sst_nudged')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_CP, DIM2_arr=temp2	(odom),name='ssl_corrector')
	
	else if(TRIM(atm_model_name) == 'ATM_GFS') then

		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom),period=3*60,name='T')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom),period=3*60,name='Q')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom),period=3*60,name='U')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom),period=3*60,name='V')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=LWDN_MOD_atm	(odom),period=3*60,name='LWDN' )
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=SWDN_MOD_atm	(odom),period=3*60,name='SWDN' ) 	
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=RAIN_atm	(odom),period=3*60,name='RAIN')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=SNOW_atm	(odom),period=3*60,name='SNOW' )
		!call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=runoff		(odom),period=3*60,name='runoff' )
	
	end if
	
	
#endif

	if(TRIM(atm_model_name) == 'ATM_INM') 	then
		
		!-----Mapping from ATM_INM------
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=atm_ocn_windstr_x(odom),period=atm_ocn_mapping,name='ssu_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=atm_ocn_windstr_y(odom),period=atm_ocn_mapping,name='ssv_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=str_from_atm	(odom),period=atm_ocn_mapping,name='tr_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=ssr_from_atm	(odom),period=atm_ocn_mapping,name='sr_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=sns_from_atm	(odom),period=atm_ocn_mapping,name='sh_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=ltn_from_atm	(odom),period=atm_ocn_mapping,name='lh_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=evap_from_atm	(odom),period=atm_ocn_mapping,name='e_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=prec_from_atm	(odom),period=atm_ocn_mapping, name='p_atm' )
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=t_atm		(odom),period=atm_ocn_mapping, name ='t2_atm' )
		!call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=from_atm1(odom),period=atm_ocn_mapping,name='tester_back')

		!additional parameters from atm-inm
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom), period = atm_ocn_mapping, name = 'u10m')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom), period = atm_ocn_mapping, name = 'v10m')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom), period = atm_ocn_mapping, name = 't2m')
		call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom), period = atm_ocn_mapping, name = 'sq2m')
		
		!-----Mapping to ATM-INM------
		call this % reg_array(send_to=COMP_ATM,action=ACTION_MAPPING,DIM2_arr=ocn_atm_sst  (  odom  ),period=ocn_atm_mapping,map_type=MAP_DST_TYPE,name='sst')
		call this % reg_array(send_to=COMP_ATM,action=ACTION_MAPPING,DIM2_arr=aice	   (  odom  ),period=ocn_atm_mapping,map_type=MAP_DST_TYPE, name = 'ice_compactness' )
		call this % reg_array(send_to=COMP_ATM,action=ACTION_MAPPING,DIM2_arr=tice         (  odom  ),period=ocn_atm_mapping,map_type=MAP_DST_TYPE, name = 'ice_temperature' )
		
		!------diagnostics-----
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=wind_stress_x_t_mean % collector2d(odom),name='windstr_x_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=wind_stress_y_t_mean % collector2d(odom),name='windstr_y_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=total_surf_heat_mean % collector2d(odom),name='total_heat_mean',	period=dg_period  )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=str_mean % collector2d(odom),name='str_mean',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ssr_mean % collector2d(odom),name='ssr_mean',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=sns_mean % collector2d(odom),name='sns_mean',		period=dg_period  )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ltn_mean % collector2d(odom),name='ltn_mean',		period=dg_period  )
		
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=atm_ocn_windstr_x	(odom),name='a2o_windstrx',	period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=atm_ocn_windstr_y	(odom),name='a2o_windstry',	period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=str		(odom),name='str',		period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ssr		(odom),name='ssr',		period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=sns		(odom),name='sns',		period=dg_period  )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ltn		(odom),name='ltn',		period=dg_period  )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=evap		(odom),name='evap',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=prec		(odom),name='prec',		period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=aice		(odom),name='aice',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=tice		(odom),name='tice',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=U_10_MOD_atm	(odom),name='u10',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=V_10_MOD_atm	(odom),name='v10',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=T_10_MOD_atm	(odom),name='t2',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=Q_10_MOD_atm	(odom),name='q2',	period=dg_period )
	
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=t_atm		(odom),name='t_atm',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=temp2		(odom),name='t_atm_translated',	period=dg_period )
		
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=atm_ocn_windstr_x_mean % collector2d(odom),name='a2o_strx_mean',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=atm_ocn_windstr_y_mean % collector2d(odom),name='a2o_stry_mean',		period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=sns_from_atm_mean % collector2d(odom),name='a2o_sns_mean',			period=dg_period  )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ltn_from_atm_mean % collector2d(odom),name='a2o_ltn_mean',			period=dg_period  )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=total_surf_heat_from_atm_mean % collector2d(odom),name='a2o_total_mean',	period=dg_period  )
		
	end if  !if(TRIM(atm_model_name) == 'ATM_INM')
	
	if(TRIM(atm_model_name) == 'ATM_INMCM4') then

		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom),period=24*60,name='T_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom),period=24*60,name='Q_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom),period=24*60,name='U_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom),period=24*60,name='V_10_MOD_atm')
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=LWDN_MOD_atm	(odom),period=24*60,name='LWDN' )
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=SWDN_MOD_atm	(odom),period=24*60,name='SWDN' ) 	
		call this % reg_array(recv_from=COMP_ATM,action=ACTION_MAPPING, DIM2_arr=RAIN_atm	(odom),period=24*60,name='RAIN')
		
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ssr_mean % collector2d(odom),name='ssr_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=str_mean % collector2d(odom),name='str_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=sns_mean % collector2d(odom),name='sns_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ltn_mean % collector2d(odom),name='ltn_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=prec_mean % collector2d(odom),name='prec_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=evap_mean % collector2d(odom),name='evap_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=total_surf_heat_mean % collector2d(odom),name='total_surfheat_mean',	period=dg_period )
		
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=ssl_mean % collector2d(odom),name='ssl_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=aice_mean % collector2d(odom),name='aice_mean',	period=dg_period )
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_collector_kind_arr=hice_mean % collector2d(odom),name='hice_mean',	period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM3_collector_kind_arr=t_mean % collector3d(:,odom),name='t_mean',		period=dg_period )
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM3_collector_kind_arr=s_mean % collector3d(:,odom),name='s_mean',		period=dg_period )
	end if
	
	
	
	
!-------
!	ACTION_MAPPING. send_to=COMP_DIA
!-------
	if(TRIM(cmp % names(COMP_DIA)) == 'DIA_CORE') then
		call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING, DIM2_arr=kt_real(odom),period=6*60,name='kt_for_diagn',map_type=MAP_SRC_TYPE)
		!do k=1,kl
			!write(field_name,'(A,i2.2)') 't_for_diagn_',k
			!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING,DIM2_main_kind_arr=t_c(k,odom,1),period=6*60,name=trim(field_name),map_type=MAP_SRC_TYPE)

			!write(field_name,'(A,i2.2)') 's_for_diagn_',k
			!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING,DIM2_main_kind_arr=t_c(k,odom,2),period=6*60,name=trim(field_name),map_type=MAP_SRC_TYPE)

			!write(field_name,'(A,i2.2)') 'v_for_diagn_',k
			!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING, DIM2_arr=v_dg(k,odom),period=6*60,name=trim(field_name),map_type=MAP_SRC_TYPE)
		!end do

		!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING, DIM2_arr=flux_n_dg_adv 	(odom),period=6*60,name='flux_n_dg_adv',map_type=MAP_SRC_TYPE)
		!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING, DIM2_arr=flux_n_dg_lapl	(odom),period=6*60,name='flux_n_dg_lapl',map_type=MAP_SRC_TYPE)
		!call this % reg_array(send_to=COMP_DIA,action=ACTION_MAPPING, DIM2_arr=flux_n_dg_bh  	(odom),period=6*60,name='flux_n_dg_bh',map_type=MAP_SRC_TYPE)
	end if

	
	if(trim(ice_model_name) == 'ICE_CICE') then

		!-----Mapping to ICE_CICE-----
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_sst   (odom),period=ocn_ice_mapping,name='sst')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_sss   (odom),period=ocn_ice_mapping,name='sss')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_frzmlt(odom),period=ocn_ice_mapping,name='frzmlt')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_uocn  (odom),period=ocn_ice_mapping,name='uocn')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_vocn  (odom),period=ocn_ice_mapping,name='vocn')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_tltx  (odom),period=ocn_ice_mapping,name='ss_tltx')
		call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_tlty  (odom),period=ocn_ice_mapping,name='ss_tlty')
		!call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ocn_ice_test  (odom),period=ocn_ice_mapping,name='ocn_ice_test')			

		if(TRIM(atm_model_name) == 'ATM_INM') 	then
			!----re-sending atmospheric fluxes to ice-----
			!----Note that prec is already in m/s---------
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=atm_ocn_windstr_x(odom),period=ocn_ice_mapping,name='ssu_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=atm_ocn_windstr_y(odom),period=ocn_ice_mapping,name='ssv_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=str		(odom),period=ocn_ice_mapping,name='tr_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=ssr		(odom),period=ocn_ice_mapping,name='sr_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=sns_from_atm	(odom),period=ocn_ice_mapping,name='sh_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=ltn_from_atm	(odom),period=ocn_ice_mapping,name='lh_a2i',map_type=MAP_SRC_TYPE)
			!call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=evap_from_atm	(odom),period=ocn_ice_mapping,name='e_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=prec		(odom),period=ocn_ice_mapping, name='p_a2i',map_type=MAP_SRC_TYPE)
			!call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=t_atm		(odom),period=ocn_ice_mapping, name ='t2_a2i',map_type=MAP_SRC_TYPE)
			!call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=from_atm1(odom),period=ocn_ice_mapping,name='tester_back',map_type=MAP_SRC_TYPE)
			
			!additional parameters from atm-inm to ice
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm	(odom),period=ocn_ice_mapping,name='u10m_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm	(odom),period=ocn_ice_mapping,name='v10m_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm	(odom),period=ocn_ice_mapping,name='t2m_a2i',map_type=MAP_SRC_TYPE)
			call this % reg_array(send_to=COMP_ICE,action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm	(odom),period=ocn_ice_mapping,name='sq2m_a2i',map_type=MAP_SRC_TYPE)
					
		end if
		
		!-----Mapping from ICE_CICE-----
		!'ice_ocn_stress_x_v', 'ice_ocn_stress_x_v' come to T-nodes. 
		!in order to use in OCN they should be interpolated to V-nodes

		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_aice		(odom),period=ice_ocn_mapping,name='aice')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_stress_x_v	(odom),period=ice_ocn_mapping,name='strocnxt')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_stress_y_v	(odom),period=ice_ocn_mapping,name='strocnyt')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_ssr_ai		(odom),period=ice_ocn_mapping,name='fswthru_ai')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_water_ai	(odom),period=ice_ocn_mapping,name='fresh_ai')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_heat_ai		(odom),period=ice_ocn_mapping,name='fhocn_ai')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_salt_ai		(odom),period=ice_ocn_mapping,name='fsalt_ai')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_hice		(odom),period=ice_ocn_mapping,name='vice')
		!call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_test		(odom),period=ice_ocn_mapping,name='ice_ocn_test')
		call this % reg_array(recv_from=COMP_ICE,action=ACTION_MAPPING,DIM2_arr=ice_ocn_tice		(odom),period=ice_ocn_mapping,name='tice')

		!-----diagnostics-----
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_sal_dg	(odom),period=dg_period,name='ice_ocn_salt')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_hea_dg	(odom),period=dg_period,name='ice_ocn_heat')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_wat_dg	(odom),period=dg_period,name='ice_ocn_water')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_stx_dg	(odom),period=dg_period,name='ice_ocn_stress_x_v')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_sty_dg	(odom),period=dg_period,name='ice_ocn_stress_y_v')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_ssr_dg	(odom),period=dg_period,name='ice_ocn_ssr')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_test	(odom),period=dg_period,name='ice_ocn_test')
		
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_sst_dg	(odom),period=dg_period,name='ocn_ice_sst')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_sss_dg	(odom),period=dg_period,name='ocn_ice_sss')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_uoc_dg	(odom),period=dg_period,name='ocn_ice_uocn')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_voc_dg	(odom),period=dg_period,name='ocn_ice_vocn')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_tlx_dg	(odom),period=dg_period,name='ocn_ice_tltx')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_tly_dg	(odom),period=dg_period,name='ocn_ice_tlty')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_frz_dg	(odom),period=dg_period,name='ocn_ice_frzmlt')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=aice		(odom),period=dg_period,name='Aice')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ice_ocn_aic_dg	(odom),period=dg_period,name='ice_ocn_Aice')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=ocn_ice_test	(odom),period=dg_period,name='ocn_ice_test')
	end if		!if(trim(ice_model_name) == 'ICE_CICE')

	
	if(TRIM(lnd_model_name) == 'LND_CORE') then
		call this % reg_array(recv_from=COMP_LND,action=ACTION_MAPPING, DIM2_arr=runoff		(odom),period=(365*1440)/12,name='runoff' )
		!call this % reg_array(recv_from=COMP_LND,action=ACTION_MAPPING, DIM2_arr=surface_salt	(odom),period=(365*1440)/12,name='salt' )
	end if
	
	
	if(TRIM(lnd_model_name) == 'LND_DAS') then
		call this % reg_array(recv_from=COMP_LND,action=ACTION_MAPPING, DIM2_arr=runoff		(odom),period=24*60,name='runoff' )
		call this % reg_array(recv_from=COMP_LND,action=ACTION_MAPPING, DIM2_arr=nudge_sst	(odom),period=24*60,name='nudge_sst' )
		!call this % reg_array(recv_from=COMP_LND,action=ACTION_MAPPING, DIM2_arr=nudge_aice	(odom),period=24*60,name='nudge_aice' )
		
		
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=nudge_aice	(odom),period=dg_period,name='nudge_aice')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=nudge_sst	(odom),period=dg_period,name='nudge_sst')
		call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=temp1		(odom),period=dg_period,name='sst_diff')
		!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG, DIM2_arr=temp2		(odom),period=dg_period,name='aice_diff')
	end if
	
	end subroutine o_ini_reg_data

!================================================================================================================================================================
	
	subroutine o_ini_allocate(this)
		
		class(cpl_ocn) :: this
#if ANALYSIS_FLAG == 0
		call o_driver_allocate
#elif ANALYSIS_FLAG == 1
        call o_analysis_allocate
#endif

	end subroutine o_ini_allocate
!================================================================================================================================================================

	subroutine o_ini_main(this)
		
		class(cpl_ocn) :: this
#if ANALYSIS_FLAG == 0
		call o_driver_ini_main
#elif ANALYSIS_FLAG == 1
        call o_analysis_ini_main
#endif
	end subroutine o_ini_main
				
!================================================================================================================================================================

	subroutine o_main_step(this)
	integer   (kind=4), allocatable:: itest (:,:)
	character (kind=1), allocatable:: bt    (:,:)
	real(8) area_local,dare,ssl_local,flux_local,averages_array(3)
	real start,finish
	real (kind=4) land_value
	class(cpl_ocn) :: this

	
	call shared_timer
#if ANALYSIS_FLAG == 1	
	call o_analysis_main_step
#elif ANALYSIS_FLAG == 0

!-------
!	timer
!-------
	if (ocn_rank_local == 0 .and. mod(time_l,ocn_steps_in_day/24).eq.0 ) call cpu_time(start)

!------->>>
!	unmask land points for the use in model (it was masked for saving to CP file at previous time-step)
!-------
	if (mod((time_l-1)*(24*60/ocn_steps_in_day),time_save_cp).eq.0) then
		land_value = 0.	
		call o_mask_land_CP (land_value)
		if (rank_local == 0) print *, "call o_mask_land_CP (missing_value) ", land_value
	end if
!-------<<<

!------->>>
!	unmask land for saving to DG file
!-------
	if(time_l_save_dg /= 0 .AND. MOD(time_l-1, time_l_save_dg) == 0) then
		land_value = 0.	
		call o_mask_land_DG (land_value)
	end if
!-------<<<


!------->>>
!	handle functions recieved from ICE_CICE model. do it after mapping
!-------
	if(trim(ice_model_name) == "ICE_CICE" .and. (.not. cice_and_schrum .or. jsouth(myrank) .ge. jl/2-10)) &
		call make_ice_ocn_fluxes	
	
	!Halo exchange must be called on all cores, including Schrum cores (in case of reduced CICE grid)
	if(trim(ice_model_name) == "ICE_CICE")	& 
		call comp_halo_update_T4(DIM2_t = aice, update_width = 1, change_sign_on_bipolar = 1)
!-------<<<

	if(trim(atm_model_name) == "ATM_INM") call make_ocn_atm_fluxes


	call o_driver_main_step

!------->>>
!	prepare functions for sending to ICE_CICE model. do it before mapping
!-------
	if(trim(ice_model_name) == 'ICE_CICE' .and. MOD(time_l,ocn_ice_mapping*minday/24/60)==0 .and. (.not. cice_and_schrum .or. jsouth(myrank) .ge. jl/2-10)) &
		call make_ocn_ice_fluxes
!-------<<<


!==================================================
!	some diagnostics
!==================================================
		
	area_local=0.
	ssl_local=0.
	flux_local=0.
	do j=jsouth(myrank),jnorth(myrank)
	do i=iwest (myrank),ieast (myrank)
		dare=area_tbox(i,j)*ct3(1,i,j)
		area_local=area_local+dare
		ssl_local=ssl_local+ssl_c(i,j)*dare
		flux_local=flux_local+temp2(i,j)*dare
	end do
	end do
	
	!allreducing ssl (for every step diagnostics) and freeze_solution (for solution crash control)
	call comp_p % reduce_itg(area_local, [real(kind=8) :: ssl_local,freeze_solution,flux_local], averages_array)    
	freeze_solution=averages_array(2)
	if(myrank==1 .and. mod(time_l_in_run,2)==0 .and. time_l_in_run>2) then
		write(103,'(3(e20.12))') Real(time_l_in_run,8)*0.5_8,averages_array(3)
	end if
	
!------->>>
!	Accumulate mean arrays(and finalize,if finalize_condition == .TRUE.)
!-------
	call o_diagn_mld
	call o_diagn_basic(time_l_save_dg)
	call o_diagn_surface(time_l_save_dg)
	call o_diagn_time_mean(dim3_arr_in=sqrt(u_c(:,:,:)*u_c(:,:,:)+v_c(:,:,:)*v_c(:,:,:)), arr_out=ke_mean, finalize_condition=(MOD(time_l,time_l_save_cp) == 0))
!-------<<<

!------->>>
!	mask land for saving to CP file
!-------
	if (mod( (time_l*(24*60/ocn_steps_in_day)),time_save_cp).eq.0) then
		call o_mask_land_CP (missval)
		!if (ocn_rank_local == 0) print *, "call o_mask_land (missing_value) ", missval
	end if
!-------<<<


!------->>>
!	mask land for saving to DG file
!-------
	if(time_l_save_dg /= 0 .AND. MOD(time_l, time_l_save_dg) == 0) then
		call o_mask_land_DG (missval)
	end if
!-------<<<

!-------
!	timer
!-------
	if (ocn_rank_local == 0 .and. mod(time_l,ocn_steps_in_day/24).eq.0 ) then
		call cpu_time(finish)
		print *, "### OCN step walltime ### ", finish - start
	end if
#endif
	end subroutine o_main_step
		
!================================================================================================================================================================
	
	end module cpl_ocn_module
