
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS) 
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------


subroutine o_ini_reg_data_interim(this)
	
#if CMF_VERSION == 2
    use cpl_ocn_module
	
	class(cpl_ocn) :: this
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=T_10_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'v2t')
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=Q_10_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'v2d')
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=U_10_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'v10u')
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=V_10_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'v10v') 	
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=SNOW_atm	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'sst') 	!Interim SST
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=LWDN_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'strd' )
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=SWDN_MOD_atm(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'ssrd' ) 	
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=RAIN_atm	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'tp')	!Interim full precipitation
	call this % reg_array(recv_from=COMP_ATM, action=ACTION_MAPPING, DIM2_arr=runoff	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = 6*60, name = 'ro' )

	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=tair02	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'tair02')
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=tdew02	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'tdew02')
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=wind_speed_x_t	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'wind_speed_x_t')
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=wind_speed_y_t	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'wind_speed_y_t') 	
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=str	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'str' )
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=ssr	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'ssr' ) 	
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=prec	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'prec')
	call this % reg_array(send_to=COUPLER, action=ACTION_SAVE_DG, DIM2_arr=nudge_sst	(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)), period = dg_period, name = 'sst' )	
#elif CMF_VERSION == 3
    integer :: this
#endif
	
end subroutine o_ini_reg_data_interim
