!========================================================================================
! This is automatically generated adapter for CMF-system.
! WARNING: for correct work don't change any names generated by generate_comp.sh script.
!
! Subroutines will be called by main program. 
! Fill them with appropriate actions, suitable for your model.
!========================================================================================

!--------------------------------------
!		
!	Marchuk Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Marchuk Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF3.0
!
! authors: 
!	2014    / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2019.06 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! references:
!	Ushakov K. V., Ibrayev R. A. (2018) Assessment of mean world ocean meridional heat 
!		transport characteristics by a high-resolution model. 
!		Russian Journal of Earth Sciences, 2018, vol. 18, ES1004, doi: 10.2205/2018ES000616
!	Kalmykov V.V., Ibrayev R.A., Kaurkin M.N., Ushakov K.V. (2018) Compact Modeling Framework v3.0 
!		for high-resolution global ocean–ice–atmosphere models. Geosci. Model Dev., 11, 3983-3997. 
!		https://doi.org/10.5194/gmd-11-3983-2018
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibrayev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------


module ocn_cpl_adapter_module

use component_module
use o_par_module
use o_driver_module

implicit none

type, extends(Component) :: ocn_cpl_adapter
 contains
    procedure :: ini_reg_comp
    procedure :: ini_allocate
    procedure :: ini_reg_data    
    procedure :: ini_main
    procedure :: make_step
    procedure :: finalize 
end type

 CONTAINS

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register your model in the system.
! Here you should (probably) read or define necessary information and 
! must call this % register_model()
!----------------------------------------------------------------------------------------

subroutine ini_reg_comp(this)
    class(ocn_cpl_adapter) :: this
    logical :: read_proc_decomp = .false.
    character(NAME_LEN) :: land_file
    
    ! Initiate ocean parameters here
    call o_driver_ini_params()
    time_l_save_cp=cp_period/(24*60/minday)
    time_l_save_dg=dg_period/(24*60/minday)
    
    ! If we want read predefine processor decomposition, set file name to read from
    if (read_proc_decomp) then
        land_file = this % file_prefix()//"_IC.nc"
    else
        land_file = ""
    end if

    ! Build decomposition of grid (or read it from IC-file)
    ocn_decomp = Decomposition(il-2, jl-2, kl, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local(), &
                                proc_decomp_file = land_file, is_icycle = i_cycle == 1, is_bsc = bsc_flag == 1)

    
    ! Call registration routine of CMF (it needs only size of grid and timestep in whole(!) seconds)
    call this % register_model(decomp = ocn_decomp, timestep_sec = 24*60*60/ocn_steps_in_day) 
          
    ! Initiate o_par_module
    call o_par_init(this)

    ! Get Decomposition of ocean and pass it to driver routine
    call o_driver_subdomains(ocn_decomp % west(), ocn_decomp % east(), ocn_decomp % south(), ocn_decomp % north())
    
        
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to allocate all your data, which will interact with system.
! This is necessary, because you ini_reg_data() use adresses of your arrays and 
! they should be allocated at time of registration.
! Here you should do all array allocations.
!----------------------------------------------------------------------------------------

subroutine ini_allocate(this)
    class(ocn_cpl_adapter) :: this

    call o_driver_allocate()
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register all data you want to interact with system.
! Here you should call this % register_array(), this % register_*_event() or
! this % raise_event() to get result immediately.
!----------------------------------------------------------------------------------------

subroutine ini_reg_data(this)
    class(ocn_cpl_adapter) :: this

    integer :: w, e, s, n
    w = iwest(myrank)
    e = ieast(myrank)
    s = jsouth(myrank)
    n = jnorth(myrank)
    
    call o_par_after_sync()
    
    call ini_reg_data_ic(this, w, e, s, n)
    call ini_reg_data_dg(this, w, e, s, n)
    call ini_reg_data_cp(this, w, e, s, n)
    !call ini_reg_data_cp_add(this, w, e, s, n)
    if (TRIM(atm_model_name) == "ATM_NCAR") call ini_reg_data_atm_ncar(this, w, e, s, n)
    if (TRIM(atm_model_name) == "ATM_GFS") call ini_reg_data_atm_gfs(this, w, e, s, n)
    if (TRIM(atm_model_name) == "ATM_INTERIM") call ini_reg_data_atm_interim(this, w, e, s, n)
    if (TRIM(lnd_model_name) == "LND_CORE") call ini_reg_data_lnd_core(this, w, e, s, n)
    if (TRIM(ice_model_name) == "ICE_CICE") call ini_reg_data_ice_cice(this, w, e, s, n)
!==============================
!Data Assimilation Service DAS
! work only with real8
!==============================
    if ( ocn_das_on ) call ini_reg_data_service_das(this, w, e, s, n)    

        
end subroutine
!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program before main time cycle.
! Here you should do all model preparation actions.
!----------------------------------------------------------------------------------------

subroutine ini_main(this)
    class(ocn_cpl_adapter) :: this
    
#if CMF_VERSION == 3
	!As in CMF2.0, we initialize time_l and time_l_in_run outside of o_driver_module.f90
	!Calculate time_l as difference between exp_start and run_start + 1
	!So time_l means the No of step that is being made (1st, 2nd, 3rd, ...)
	!Or, in other words, the number of steps that will be completed when the current model step is finished
	time_l = (date2sec(ExpInfo % start_date()) - date2sec(ExpInfo % exp_start_date())) / (24*60*60) * ocn_steps_in_day + 1
	time_l_in_run = 1
#endif
	call o_driver_ini_main()
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at every step of time cycle. 
! Here you should represent one time step of your model with all physics and internal 
! exhanges (like halo MPI-updates).
!----------------------------------------------------------------------------------------

subroutine make_step(this)
	class(ocn_cpl_adapter) :: this
	real start,finish
	real(8) area_local,dare,ssl_local,averages_array(2)
    
	call shared_timer
#if ANALYSIS_FLAG == 1	
	call o_analysis_main_step
#elif ANALYSIS_FLAG == 0


	!---cpu usage timer---
	if (ocn_rank_local == 0 .and. mod(time_l,ocn_steps_in_day/24).eq.0 ) call cpu_time(start)

	!---unmask land points for the use in model (it was masked for saving to CP or DG file at previous time-step)---
	if (mod(time_l-1,time_l_save_cp).eq.0) call o_mask_land_CP(0.)
	if(time_l_save_dg /= 0 .AND. MOD(time_l-1, time_l_save_dg) == 0) call o_mask_land_DG(0.)

	!---handle functions recieved from ICE_CICE model. do it after mapping---
	if(trim(ice_model_name) == "ICE_CICE") then
		call make_ice_ocn_fluxes	
		call HaloUpdateMaster % update(aice(:,:), update_width = 1, grid_type = 'T') 
	end if
    
	!====main step==========\
	!========================\
	call o_driver_main_step() !
	!========================/
	!=======================/

	!---prepare functions for sending to ICE_CICE model. do it before mapping---
	if(trim(ice_model_name) == 'ICE_CICE' .and. MOD(time_l,ocn_ice_mapping*minday/24/60)==0 ) call make_ocn_ice_fluxes

!==================================================
!	some diagnostics
!==================================================
		
	!area_local=0.
	!ssl_local=0.
	!do j=jsouth(myrank),jnorth(myrank)
	!do i=iwest (myrank),ieast (myrank)
	!	dare=area_tbox(i,j)*ct3(1,i,j)
	!	area_local=area_local+dare
	!	ssl_local=ssl_local+ssl_c(i,j)*dare
	!end do
	!end do
	
	!allreducing ssl (for every step diagnostics) and freeze_solution (for solution crash control)
	!averages_array(1) = UtilsAllReduce(area_local, ssl_local,  ocn_comm)
	!averages_array(2) = UtilsAllReduce(area_local, dble(freeze_solution), ocn_comm)
	!freeze_solution=averages_array(2)

!------->>>
!	Accumulate mean arrays(and finalize,if finalize_condition == .TRUE.)
!	The moment of finalization is verified for CMF3. For example, if time_l_save_cp=minday and this step is the last in day, then time_l=minday (see subr. ini_main),
!	and the mean arrays are finalized. The time of the next Component.f90 step is 00:00, so the finalized arrays are written to CP.
!	The same is true for masking CP with missval.
!-------
	!call o_diagn_mld
	call o_diagn_surface
	!call o_diagn_surface_integrals
	!call o_diagn_a2o_module
	!call o_diagn_basic
	!call o_diagn_time_mean(dim3_arr_in=u_c*u_c+v_c*v_c, arr_out=ke_mean, finalize_condition=(MOD(time_l,time_l_save_cp) == 0))
!-------<<<

	!---mask land for saving to CP or DG file---
	if (mod(time_l,time_l_save_cp).eq.0) call o_mask_land_CP (missval)
	if(time_l_save_dg /= 0 .AND. MOD(time_l, time_l_save_dg) == 0) call o_mask_land_DG (missval)

	!---timer---
	if (ocn_rank_local == 0 .and. mod(time_l,ocn_steps_in_day/24).eq.0 ) then
		call cpu_time(finish)
		print *, "### OCN step walltime ### ", finish - start
	end if	
	
	!As in CMF2.0, we increment these counters only when all user-defined subroutines have completed
	time_l = time_l + 1
	time_l_in_run = time_l_in_run + 1
	!call test_ps % put(real(1., 8), dt = cmp_ptr % model_time())
	
#endif
    
    
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at the end.
! Here you should do all postprocesing actions and memory deallocations.
!----------------------------------------------------------------------------------------

subroutine finalize(this)        
    class(ocn_cpl_adapter) :: this   
    call o_driver_destroy()
    call o_par_destroy()
end subroutine

!========================================================================================

!========================================================================================
! Some ocean specific helpers
!========================================================================================

subroutine ini_reg_data_ic(obj, w, e, s, n) 
    class(Component), intent(in) :: obj
    integer, intent(in) :: w, e, s, n

    call obj % register_array(arr_name = "kv", arr = kv_real(w:e, s:n))
    call obj % register_array(arr_name = "kt", arr = kt_real(w:e, s:n))
    call obj % register_array(arr_name = "t_an", arr = t_c(:, w:e, s:n, 1), indexing = "kij")
    call obj % register_array(arr_name = "s_an", arr = t_c(:, w:e, s:n, 2), indexing = "kij")

    call obj % register_periodic_event(arr_name = "t_an", act = "READ_IC")
    call obj % register_periodic_event(arr_name = "s_an", act = "READ_IC")
    call obj % register_periodic_event(arr_name = "kv", act = "READ_IC")
    call obj % register_periodic_event(arr_name = "kt", act = "READ_IC")

end subroutine

!========================================================================================

subroutine ini_reg_data_cp(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
	
	call obj % register_array(arr_name = "t_cp",      arr = t_c(:, w:e, s:n, 1), indexing = "kij")
	call obj % register_array(arr_name = "s_cp",      arr = t_c(:, w:e, s:n, 2), indexing = "kij")
	call obj % register_array(arr_name = "u_cp",      arr = u_c(:, w:e, s:n),    indexing = "kij")
	call obj % register_array(arr_name = "v_cp",      arr = v_c(:, w:e, s:n),    indexing = "kij")
	call obj % register_array(arr_name = "wsurf_cp",  arr = w_surf_c(w:e, s:n))
	call obj % register_array(arr_name = "ubt_cp",    arr = u_bt_a_f(w:e, s:n))
	call obj % register_array(arr_name = "vbt_cp",    arr = v_bt_a_f(w:e, s:n))
	call obj % register_array(arr_name = "hbt_cp",    arr = h_bt_a_f(w:e, s:n))
	call obj % register_array(arr_name = "ssl_cp",    arr = ssl_c(w:e, s:n))
	call obj % register_array(arr_name = "hice_cp",   arr = hice(w:e, s:n))
	call obj % register_array(arr_name = "aice_cp",   arr = aice(w:e, s:n))
	call obj % register_array(arr_name = "dx3_v_l_cp",arr = dx3_v_l(2, w:e, s:n))
	!call obj % register_array(arr_name = "tice_cp",   arr = tice(w:e, s:n))
	
	if(ExpInfo % search_cp()) then
		call obj % register_periodic_event(arr_name = "t_cp",         act = "READ_CP")
		call obj % register_periodic_event(arr_name = "s_cp",         act = "READ_CP")
		call obj % register_periodic_event(arr_name = "u_cp",         act = "READ_CP")
		call obj % register_periodic_event(arr_name = "v_cp",         act = "READ_CP")
		call obj % register_periodic_event(arr_name = "wsurf_cp",     act = "READ_CP")
		call obj % register_periodic_event(arr_name = "ubt_cp",       act = "READ_CP")
		call obj % register_periodic_event(arr_name = "vbt_cp",       act = "READ_CP")
		call obj % register_periodic_event(arr_name = "hbt_cp",       act = "READ_CP")
		call obj % register_periodic_event(arr_name = "ssl_cp",       act = "READ_CP")
		call obj % register_periodic_event(arr_name = "hice_cp",      act = "READ_CP")
		call obj % register_periodic_event(arr_name = "aice_cp",      act = "READ_CP")
		call obj % register_periodic_event(arr_name = "dx3_v_l_cp",   act = "READ_CP")
		!call obj % register_periodic_event(arr_name = "tice_cp",      act = "READ_CP")
	end if  
	
	call obj % register_periodic_event(arr_name = "t_cp",         act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "s_cp",         act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "u_cp",         act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "v_cp",         act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "wsurf_cp",     act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "ubt_cp",       act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "vbt_cp",       act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "hbt_cp",       act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "ssl_cp",       act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "hice_cp",      act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "aice_cp",      act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "dx3_v_l_cp",   act = "SAVE_CP", dm = cp_period)
	!call obj % register_periodic_event(arr_name = "tice_cp",      act = "SAVE_CP", dm = cp_period)
	
end subroutine

!========================================================================================

subroutine ini_reg_data_cp_add(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
	
	!------------For eddy experiments-------------
	call obj % register_array(arr_name = "mht_sub_u_mea",	arr = mht_subgrid_u_mean % collector3d(:,w:e,s:n), indexing = "kij")
	call obj % register_array(arr_name = "mht_sub_v_mea",	arr = mht_subgrid_v_mean % collector3d(:,w:e,s:n), indexing = "kij")
	call obj % register_array(arr_name = "mht_adv_u_mea",	arr = mht_adv_u_mean % collector3d(:,w:e,s:n), indexing = "kij")
	call obj % register_array(arr_name = "mht_adv_v_mea",	arr = mht_adv_v_mean % collector3d(:,w:e,s:n), indexing = "kij")
	call obj % register_array(arr_name = "ke_mean",		arr = ke_mean % collector2d(w:e,s:n))
	
	!-------------Basic mean fields---------------
	call obj % register_array(arr_name = "u_mean",		arr = u_mean % collector3d(:,w:e,s:n), indexing = "kij")	
	call obj % register_array(arr_name = "v_mean",		arr = v_mean % collector3d(:,w:e,s:n), indexing = "kij")		
	call obj % register_array(arr_name = "t_mean",		arr = t_mean % collector3d(:,w:e,s:n), indexing = "kij")	
	call obj % register_array(arr_name = "s_mean",		arr = s_mean % collector3d(:,w:e,s:n), indexing = "kij")			
	call obj % register_array(arr_name = "ssl_mean",	arr = ssl_mean % collector2d(w:e,s:n))			
	
	!--------------Surface fluxes and parameters-----------------
	call obj % register_array(arr_name = "ssr_mean",	arr = ssr_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "str_mean",	arr = str_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "sns_mean",	arr = sns_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "ltn_mean",	arr = ltn_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "prec_mean",	arr = prec_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "evap_mean",	arr = evap_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "windstr_u_mea",	arr = wind_stress_x_t_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "windstr_v_mea",	arr = wind_stress_y_t_mean % collector2d(w:e,s:n))			
	call obj % register_array(arr_name = "sst_bias",    	arr = sst_diff(w:e, s:n))
	call obj % register_array(arr_name = "aice_bias",    	arr = temp1(w:e, s:n))	
	call obj % register_array(arr_name = "ssl_corrector",	arr = temp2(w:e, s:n))
	
	!--------------mixed layer depth-----------------------------
	call obj % register_array(arr_name = "mld",  	arr = mld(w:e, s:n))
	call obj % register_array(arr_name = "mld_max",	arr = mld_max(w:e, s:n))
	
	
!	call obj % register_periodic_event(arr_name = "mht_sub_u_mea",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "mht_sub_v_mea",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "mht_adv_u_mea",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "mht_adv_v_mea",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "ke_mean",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "u_mean",		act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "v_mean",		act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "t_mean",		act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "s_mean",		act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "ssl_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "ssr_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "str_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "sns_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "ltn_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "prec_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "evap_mean",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "windstr_u_mea",	act = "SAVE_CP", dm = cp_period)
	call obj % register_periodic_event(arr_name = "windstr_v_mea",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "sst_bias",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "aice_bias",	act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "ssl_corrector",	act = "SAVE_CP", dm = cp_period)	
!	call obj % register_periodic_event(arr_name = "mld",		act = "SAVE_CP", dm = cp_period)
!	call obj % register_periodic_event(arr_name = "mld_max",	act = "SAVE_CP", dm = cp_period)
	
end subroutine

!========================================================================================

subroutine ini_reg_data_dg(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n

	call obj % register_array(arr_name = "TSURF",       arr = t_c(1,w:e, s:n,1))
	call obj % register_array(arr_name = "SSURF",       arr = t_c(1,w:e, s:n,2))
	call obj % register_array(arr_name = "SSL",       	arr = ssl_c(w:e, s:n))
	!call obj % register_array(arr_name = "T10",       	arr = t_c(10,w:e, s:n,1))
	!call obj % register_array(arr_name = "S10",       	arr = t_c(10,w:e, s:n,2))

	!call obj % register_array(arr_name = "WSURF",      	arr = w_surf_c(w:e, s:n))
	!call obj % register_array(arr_name = "USURF",      	arr = u_c(1,w:e, s:n))
	call obj % register_array(arr_name = "tair02",      	arr = tair02(w:e, s:n))
	!call obj % register_array(arr_name = "tdew02",      	arr = tdew02(w:e, s:n))
	!call obj % register_array(arr_name = "sst_bias",  	arr = sst_diff(w:e, s:n))
	!call obj % register_array(arr_name = "nudge_sst",      arr = nudge_sst(w:e, s:n))
	!call obj % register_array(arr_name = "wind_u",		arr = wind_speed_x_t(w:e, s:n))
	!call obj % register_array(arr_name = "wind_v",		arr = wind_speed_y_t(w:e, s:n))
	!call obj % register_array(arr_name = "str",      	arr = str(w:e, s:n))
	call obj % register_array(arr_name = "ssr_mean",      	arr = ssr_mean % collector2d(w:e,s:n))
	!call obj % register_array(arr_name = "prec",      	arr = prec(w:e, s:n))
	
	call obj % register_periodic_event(arr_name = "TSURF",	act = "SAVE_DG", dm = dg_period)     
	call obj % register_periodic_event(arr_name = "SSURF",	act = "SAVE_DG", dm = dg_period)     
	call obj % register_periodic_event(arr_name = "SSL",	act = "SAVE_DG", dm = dg_period)
	!call obj % register_periodic_event(arr_name = "T10",	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "S10",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "WSURF",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "USURF",	act = "SAVE_DG", dm = dg_period)         
 	call obj % register_periodic_event(arr_name = "tair02",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "tdew02",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "sst_bias",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "nudge_sst",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "wind_u",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "wind_v",	act = "SAVE_DG", dm = dg_period)   
	!call obj % register_periodic_event(arr_name = "str",		act = "SAVE_DG", dm = dg_period)
	call obj % register_periodic_event(arr_name = "ssr_mean",	act = "SAVE_DG", dm = dg_period)
	!call obj % register_periodic_event(arr_name = "prec",		act = "SAVE_DG", dm = dg_period)
	
end subroutine	
	
!========================================================================================

subroutine ini_reg_data_atm_ncar(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
		
	call obj % register_array(arr_name = "T_10_MOD",    arr = T_10_MOD_atm(w:e, s:n)); T_10_MOD_atm = 0
	call obj % register_array(arr_name = "Q_10_MOD",    arr = Q_10_MOD_atm(w:e, s:n)); Q_10_MOD_atm = 0
	call obj % register_array(arr_name = "U_10_MOD",    arr = U_10_MOD_atm(w:e, s:n)); U_10_MOD_atm = 0
	call obj % register_array(arr_name = "V_10_MOD",    arr = V_10_MOD_atm(w:e, s:n)); V_10_MOD_atm = 0
	call obj % register_array(arr_name = "LWDN_MOD",    arr = LWDN_MOD_atm(w:e, s:n)); LWDN_MOD_atm = 0
	call obj % register_array(arr_name = "SWDN_MOD",    arr = SWDN_MOD_atm(w:e, s:n)); SWDN_MOD_atm = 0
	call obj % register_array(arr_name = "RAIN",        arr = RAIN_atm(w:e, s:n)); RAIN_atm = 0
	call obj % register_array(arr_name = "SNOW",        arr = SNOW_atm(w:e, s:n)); SNOW_atm = 0
	
	
	call obj % register_periodic_event(arr_name = "T_10_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    !60*60*3 seconds is the time of the first record in CNYFv2 and IAFv2
	call obj % register_periodic_event(arr_name = "Q_10_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "U_10_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "V_10_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "LWDN_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "SWDN_MOD", act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "RAIN",     act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
	call obj % register_periodic_event(arr_name = "SNOW",     act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    

!	call obj % register_periodic_event(arr_name = "T_10_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "Q_10_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "U_10_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "V_10_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "LWDN_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "SWDN_MOD", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "RAIN",     act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "SNOW",     act = "SAVE_DG", dm = dg_period)     
    
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_interim(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
		
	call obj % register_array(arr_name = "t2m",    arr = T_10_MOD_atm(w:e, s:n)); T_10_MOD_atm = 0
	call obj % register_array(arr_name = "d2m",    arr = Q_10_MOD_atm(w:e, s:n)); Q_10_MOD_atm = 0
	call obj % register_array(arr_name = "u10",    arr = U_10_MOD_atm(w:e, s:n)); U_10_MOD_atm = 0
	call obj % register_array(arr_name = "v10",    arr = V_10_MOD_atm(w:e, s:n)); V_10_MOD_atm = 0
	call obj % register_array(arr_name = "sst",        arr = SNOW_atm(w:e, s:n)); SNOW_atm = 0
	call obj % register_array(arr_name = "strd",    arr = LWDN_MOD_atm(w:e, s:n)); LWDN_MOD_atm = 0
	call obj % register_array(arr_name = "ssrd",    arr = SWDN_MOD_atm(w:e, s:n)); SWDN_MOD_atm = 0
	call obj % register_array(arr_name = "tp",        arr = RAIN_atm(w:e, s:n)); RAIN_atm = 0
	call obj % register_array(arr_name = "ci",        arr = aice_atm(w:e, s:n)); aice_atm = 0
!	call obj % register_array(arr_name = "ro",        arr = runoff(w:e, s:n)); runoff = 0			!runoff replaced by CORE data
	
	call obj % register_periodic_event(arr_name = "t2m", act = "RECV_MP", src = "ATM", dh = 6)
	call obj % register_periodic_event(arr_name = "d2m", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "u10", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "v10", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "sst", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "strd", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "ssrd", act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "tp",   act = "RECV_MP", src = "ATM", dh = 6)    
	call obj % register_periodic_event(arr_name = "ci",   act = "RECV_MP", src = "ATM", dh = 6)    
!	call obj % register_periodic_event(arr_name = "ro",   act = "RECV_MP", src = "ATM", dh = 6)
	
!	call obj % register_periodic_event(arr_name = "t2m", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "d2m", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "u10", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "v10", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "sst", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "strd", act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "ssrd",     act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "tp",     act = "SAVE_DG", dm = dg_period)     
!	call obj % register_periodic_event(arr_name = "ro",    act = "SAVE_DG", dm = dg_period)     
    
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_gfs(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
		
	call obj % register_array(arr_name = "t2m",    arr = T_10_MOD_atm(w:e, s:n)); T_10_MOD_atm = 0
	call obj % register_array(arr_name = "q2m",    arr = Q_10_MOD_atm(w:e, s:n)); Q_10_MOD_atm = 0
	call obj % register_array(arr_name = "u10",    arr = U_10_MOD_atm(w:e, s:n)); U_10_MOD_atm = 0
	call obj % register_array(arr_name = "v10",    arr = V_10_MOD_atm(w:e, s:n)); V_10_MOD_atm = 0
	call obj % register_array(arr_name = "trdi",   arr = LWDN_MOD_atm(w:e, s:n)); LWDN_MOD_atm = 0
	call obj % register_array(arr_name = "srdi",   arr = SWDN_MOD_atm(w:e, s:n)); SWDN_MOD_atm = 0
	call obj % register_array(arr_name = "tprr",   arr = RAIN_atm(w:e, s:n)); RAIN_atm = 0
	call obj % register_array(arr_name = "tprs",   arr = SNOW_atm(w:e, s:n)); SNOW_atm = 0
	
	
	call obj % register_periodic_event(arr_name = "t2m", act = "RECV_MP", src = "ATM", dh = 3)
	call obj % register_periodic_event(arr_name = "q2m", act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "u10", act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "v10", act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "trdi",act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "srdi",act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "tprr",act = "RECV_MP", src = "ATM", dh = 3)    
	call obj % register_periodic_event(arr_name = "tprs",act = "RECV_MP", src = "ATM", dh = 3)    
	
	! call obj % register_periodic_event(arr_name = "t2m", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "q2m", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "u10", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "v10", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "trdi", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "srdi", act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "tprr",act = "SAVE_DG", dm = dg_period)     
	! call obj % register_periodic_event(arr_name = "tprs",act = "SAVE_DG", dm = dg_period)     
    
end subroutine

!========================================================================================

subroutine ini_reg_data_ice_cice(obj, w, e, s, n) 
	class(Component), intent(in) :: obj
	integer, intent(in) :: w, e, s, n
		
	call obj % register_array(arr_name = "sst_OI",		arr = ocn_ice_sst(w:e, s:n))
	call obj % register_array(arr_name = "sss_OI",    	arr = ocn_ice_sss(w:e, s:n))
	call obj % register_array(arr_name = "frzmlt_OI",    	arr = ocn_ice_frzmlt(w:e, s:n))
	call obj % register_array(arr_name = "uocn_OI",    	arr = ocn_ice_uocn(w:e, s:n))
	call obj % register_array(arr_name = "vocn_OI",    	arr = ocn_ice_vocn(w:e, s:n))
	call obj % register_array(arr_name = "ss_tltx_OI",    	arr = ocn_ice_tltx(w:e, s:n))
	call obj % register_array(arr_name = "ss_tlty_OI",	arr = ocn_ice_tlty(w:e, s:n))
	!call obj % register_array(arr_name = "test_OI",        arr = ocn_ice_test(w:e, s:n))
	
	call obj % register_array(arr_name = "aice_IO",		arr = ice_ocn_aice(w:e, s:n))
	call obj % register_array(arr_name = "strocnxt_IO",    	arr = ice_ocn_stress_x_v(w:e, s:n))
	call obj % register_array(arr_name = "strocnyt_IO",    	arr = ice_ocn_stress_y_v(w:e, s:n))
	call obj % register_array(arr_name = "fswthru_IO",    	arr = ice_ocn_ssr_ai(w:e, s:n))
	call obj % register_array(arr_name = "fresh_IO",    	arr = ice_ocn_water_ai(w:e, s:n))
	call obj % register_array(arr_name = "fhocn_IO",    	arr = ice_ocn_heat_ai(w:e, s:n))
	call obj % register_array(arr_name = "fsalt_IO",	arr = ice_ocn_salt_ai(w:e, s:n))
	call obj % register_array(arr_name = "vice_IO",		arr = ice_ocn_hice(w:e, s:n))
	call obj % register_array(arr_name = "tice_IO",		arr = ice_ocn_tice(w:e, s:n))
	!call obj % register_array(arr_name = "test_IO",        arr = ice_ocn_test(w:e, s:n))
	
	call obj % register_array(arr_name = "ice_ocn_salt",	arr = ice_ocn_sal_dg(w:e, s:n))
	call obj % register_array(arr_name = "ice_ocn_heat",	arr = ice_ocn_hea_dg(w:e, s:n))
	call obj % register_array(arr_name = "ice_ocn_water",	arr = ice_ocn_wat_dg(w:e, s:n))
	call obj % register_array(arr_name = "IO_stress_x_v",	arr = ice_ocn_stx_dg(w:e, s:n))
	call obj % register_array(arr_name = "IO_stress_y_v",	arr = ice_ocn_sty_dg(w:e, s:n))
	call obj % register_array(arr_name = "ice_ocn_ssr",	arr = ice_ocn_ssr_dg(w:e, s:n))
	!call obj % register_array(arr_name = "ice_ocn_test",	arr = ice_ocn_test(w:e, s:n))
	
	call obj % register_array(arr_name = "ocn_ice_sst",		arr = ocn_ice_sst_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_sss",		arr = ocn_ice_sss_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_uocn",		arr = ocn_ice_uoc_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_vocn",		arr = ocn_ice_voc_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_tltx",		arr = ocn_ice_tlx_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_tlty",		arr = ocn_ice_tly_dg(w:e, s:n))
	call obj % register_array(arr_name = "ocn_ice_frzml",		arr = ocn_ice_frz_dg(w:e, s:n))
	call obj % register_array(arr_name = "ice_ocn_Aice",		arr = ice_ocn_aic_dg(w:e, s:n))
	!call obj % register_array(arr_name = "ocn_ice_test",		arr = ocn_ice_test(w:e, s:n))
	
	
	call obj % register_periodic_event(arr_name = "sst_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "sss_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "frzmlt_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "uocn_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "vocn_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "ss_tltx_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	call obj % register_periodic_event(arr_name = "ss_tlty_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)    
	!call obj % register_periodic_event(arr_name = "test_OI", 	act = "SEND_MP", dst = "ICE", dm = ocn_ice_mapping)
	
	call obj % register_periodic_event(arr_name = "aice_IO",     act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "strocnxt_IO", act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "strocnyt_IO", act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "fswthru_IO",  act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "fresh_IO",    act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "fhocn_IO",    act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)    
	call obj % register_periodic_event(arr_name = "fsalt_IO",    act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)
	call obj % register_periodic_event(arr_name = "vice_IO",     act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)       
	call obj % register_periodic_event(arr_name = "tice_IO",     act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)       
	!call obj % register_periodic_event(arr_name = "test_IO",    act = "RECV_MP", src= "ICE", dm = ice_ocn_mapping)       

	!call obj % register_periodic_event(arr_name = "ice_ocn_salt", 	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ice_ocn_heat", 	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ice_ocn_water", 	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "IO_stress_x_v",act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "IO_stress_y_v",act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ice_ocn_ssr", 	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ice_ocn_test",	act = "SAVE_DG", dm = dg_period)     
	
	!call obj % register_periodic_event(arr_name = "ocn_ice_sst",	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ocn_ice_sss",    act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ocn_ice_uocn",	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ocn_ice_vocn",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "ocn_ice_tltx",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "ocn_ice_tlty",	act = "SAVE_DG", dm = dg_period)     
	!call obj % register_periodic_event(arr_name = "ocn_ice_frzml",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "ice_ocn_Aice",	act = "SAVE_DG", dm = dg_period)         
	!call obj % register_periodic_event(arr_name = "ocn_ice_test",	act = "SAVE_DG", dm = dg_period)      
    
end subroutine



subroutine ini_reg_data_lnd_core(obj, w, e, s, n) 
    class(Component), intent(in) :: obj
    integer, intent(in) :: w, e, s, n
    
    call obj % register_array(arr_name = "runoff",          arr = runoff(w:e, s:n))
!     call obj % register_array(arr_name = "salt",            arr = surface_salt(w:e, s:n))
        
    call obj % register_periodic_event(arr_name = "runoff",       act = "RECV_MP", src = "LND", dh = 24)    

    call obj % register_periodic_event(arr_name = "runoff",       act = "SAVE_DG",  dm = dg_period)
    
!     call obj % register_periodic_event(arr_name = "salt",         act = "RECV_MP", src = "LND", dh = 24)    
   
end subroutine

!========================================================================================

subroutine ini_reg_data_service_das(obj, w, e, s, n)
    class(Component), intent(in) :: obj

    integer, intent(in) :: w, e, s, n

		call obj % register_array(arr_name = "t_f",      arr = t_c(:, w:e, s:n, 1), indexing = "kij")
		call obj % register_array(arr_name = "s_f",      arr = t_c(:, w:e, s:n, 2), indexing = "kij")
        call obj % register_array(arr_name = "ssl_f",    arr = ssl_c( w:e, s:n),	indexing = "ij")
        !!call obj % register_array(arr_name = "aice",     arr = ice_ocn_aice ( w:e, s:n),	indexing = "ij")

        call obj % register_periodic_event( arr_name = "t_f",     act = "SEND_DAS" , dh = 24, delay = 60*10)
        call obj % register_periodic_event( arr_name = "s_f",     act = "SEND_DAS" , dh = 24, delay = 60*10)
        call obj % register_periodic_event( arr_name = "ssl_f",   act = "SEND_DAS" , dh = 24, delay = 60*10)
        !!call obj % register_periodic_event( arr_name = "aice",    act = "SEND_DAS" , dh = 24)

        call obj % register_periodic_event( arr_name = "t_f",     act = "RECV_ENKF" , dh = 24, delay = 60*10)
        call obj % register_periodic_event( arr_name = "s_f",     act = "RECV_ENKF" , dh = 24, delay = 60*10)
        call obj % register_periodic_event( arr_name = "ssl_f",   act = "RECV_ENKF" , dh = 24, delay = 60*10)
        !!call obj % register_periodic_event( arr_name = "aice",    act = "RECV_ENKF" , dh = 24)      
end subroutine

end module
    