!========================================================================================
! This is automatically generated adapter for CMF-system.
! WARNING: for correct work don't change any names generated by generate_comp.sh script.
!
! Subroutines will be called by off_main program. 
! Fill them with appropriate actions, suitable for your model.
!========================================================================================
!--------------------------------------
!		
!	Marchuk Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Marchuk Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF3.0
!
! authors: 
!	2014    / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2019.06 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! references:
!	Ushakov K. V., Ibrayev R. A. (2018) Assessment of mean world ocean meridional heat 
!		transport characteristics by a high-resolution model. 
!		Russian Journal of Earth Sciences, 2018, vol. 18, ES1004, doi: 10.2205/2018ES000616
!	Kalmykov V.V., Ibrayev R.A., Kaurkin M.N., Ushakov K.V. (2018) Compact Modeling Framework v3.0 
!		for high-resolution global ocean–ice–atmosphere models. Geosci. Model Dev., 11, 3983-3997. 
!		https://doi.org/10.5194/gmd-11-3983-2018
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibrayev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------



module ocn_off_adapter_module
use offline_module
use o_grid_module
use o_driver_module

implicit none
    
type, extends(Offline) :: ocn_off_adapter
 contains
    procedure, public :: make_grid
    procedure, public :: make_ic
end type


	!====> ice start <====!
	logical :: ice_grid = .FALSE.
	!====>  ice end  <====!
    
 CONTAINS

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by off_main program to get information about model grid.
! Here you should build your grid and then call this % register_grid(grid).
!----------------------------------------------------------------------------------------

	subroutine make_grid(this, name, dims, grid_type)
		class(ocn_off_adapter) :: this
		
		! These parameters are passed from main program - you can use it
		character(*), intent(in) :: name, grid_type
		integer, intent(in) :: dims(3)

		! This object you should fill
		type(ModelGrid) :: grid
		
		write(*,'(A$)') 'Building global OCN grid'
				
		
		! Set grid size
		il = dims(1) + 2
		jl = dims(2) + 2
		kl = dims(3)
		
		write(*,'(A,i4,i4,i4,A$)') ' with dimensions:',il,jl,kl,'...'
		
		! Initiate parameters (WARNING: it overrides il, jl, kl)
		call o_driver_ini_params()
		
		! Initiate domains for global serial case
		call set_grid_subdomains(rank = 1, iw = 2, ie = il-1, js = 2, jn = jl-1)
		
		! Allocate grid arrays
		call allocate_o_grid_coord_module()
		
		! Make grid arrays x1_t, x2_t, x1_v, x2_v
		call o_ini_set_grid_coordinate()

		! Initialize grid object and allocate arrays
		call grid % init(name, (/ il -2, jl -2, kl /) , grid_type) 
		
		select case (grid_type)
		
		case("T")       
			do j = 2, jl-1
			do i = 2, il-1
			grid % center_lat(i-1,j-1)   = x2_t(i,j)
			grid % corner_lat(1,i-1,j-1) = x2_v(i,j)
			grid % corner_lat(2,i-1,j-1) = x2_v(i-1,j)
			grid % corner_lat(3,i-1,j-1) = x2_v(i-1,j-1)
			grid % corner_lat(4,i-1,j-1) = x2_v(i,j-1)

			grid % center_lon(i-1,j-1  ) = x1_t(i,j)
			grid % corner_lon(1,i-1,j-1) = x1_v(i,j)
			grid % corner_lon(2,i-1,j-1) = x1_v(i-1,j)
			grid % corner_lon(3,i-1,j-1) = x1_v(i-1,j-1)
			grid % corner_lon(4,i-1,j-1) = x1_v(i,j-1)
			end do
			end do
			
		case("V")
		
			do j = 2, jl-1
			do i = 2, il-1
			grid % center_lat(i-1,j-1)   = x2_v(i,j)
			grid % corner_lat(1,i-1,j-1) = x2_t(i,j)
			grid % corner_lat(2,i-1,j-1) = x2_t(i+1,j)
			grid % corner_lat(3,i-1,j-1) = x2_t(i+1,j+1)
			grid % corner_lat(4,i-1,j-1) = x2_t(i,j+1)

			grid % center_lon(i-1,j-1  ) = x1_v(i,j)
			grid % corner_lon(1,i-1,j-1) = x1_t(i,j)
			grid % corner_lon(2,i-1,j-1) = x1_t(i+1,j)
			grid % corner_lon(3,i-1,j-1) = x1_t(i+1,j+1)
			grid % corner_lon(4,i-1,j-1) = x1_t(i,j+1)
			end do
			end do
			
		case default
			write(*,*) "Error: no grid type defined for "//grid_type// " case."
			stop
		
		end select

			!====> ice start <====!	
			if (ice_grid .and. grid_type .eq. "V") then
				write(*, '(A$)') 'Writing additional grid data...'
				call additional_dump_grid
			endif
			!====>  ice end  <====!
		
		write(*,'(A)') ' success.'
		
		call o_make_mask(grid % mask2D) 
		grid % level(1:kl) = z(1:kl)
		
		! Register grid if Offline block
		call this % register_model_grid(grid)    
    
		CONTAINS

			! Create ocean mask, using ETOPO-file and maybe user redefine procedure.
			! If file not found, topography will be empty
			subroutine o_make_mask(mask2D)

				integer, intent(inout) :: mask2D(:,:)
				character(100) :: filename
				character (kind=1), allocatable:: bt    (:,:)
				logical :: exists
				type(FileHandler_NC) :: fh_mask
					
				filename = TRIM(DATA_FOLDER)//TRIM(name)//"_"//TRIM(n2s(dims(1)))//'x'//TRIM(n2s(dims(2)))//"_V"//"_topo.nc"
				INQUIRE(file = filename, exist = exists)
				if(.NOT. exists) then
					mask2D = 1
					write(*,'(A)') 'Adding ocean mask... failed: no interpolated topography file <'//TRIM(filename)//">. Mask will be empty."
				else    
					call fh_mask % open_file(filename)
					call fh_mask % open_var("topo")
					call fh_mask % get(hv(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank)), lo = (/ 1, 1 /), hi = dims(1:2))
					call fh_mask % close_file()
					
					! Make hv,kv,kt
					hv = MERGE(-hv, REAL(0, kind=4), hv < 0)    
					
					call o_user_redefine_topo()
						
					call o_ini_bottom(hv, .true.)

					! Add mask for T-points
					!missval or misValInt or smth like this may be used here instead of 0
					mask2D = MERGE(1, 0, kt(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank)) > 0)

					!====> ice start <====!
					! Dump additional grid information of kv and kt with halo data
					if (ice_grid) then
						write(*, '(A$)') ' writing additional grid mask data...'
						call additional_dump_mask
						write(*,'(A)') ' success.'
					endif
					!====>  ice end  <====!
					
					call fh_mask % create_file("./data/OCN.mask.nc")
					call fh_mask % create_dim('i', dims(1))
					call fh_mask % create_dim('j', dims(2))
					call fh_mask % create_dim('k', dims(3))
					
					call fh_mask % create_var("center_lat", "real4", "i", "j")
					call fh_mask % put(sngl(grid % center_lat), (/ 1, 1 /), hi = (/ dims(1), dims(2) /))
					call fh_mask % put_att("units", "degrees_north")        
					call fh_mask % put_att("long_name", "latitude")
					call fh_mask % put_att("axis", "Y") 
					
					call fh_mask % create_var("center_lon", "real4", "i", "j")
					call fh_mask % put(sngl(grid % center_lon), (/ 1, 1 /), hi = (/ dims(1), dims(2) /))
					call fh_mask % put_att("units", "degrees_east")        
					call fh_mask % put_att("long_name", "longitude")   
					call fh_mask % put_att("axis", "X") 
					
					call fh_mask % create_var("mask2D", "real4", "i", "j")
					call fh_mask % put(sngl(mask2D), (/ 1, 1 /), hi = (/ dims(1), dims(2) /))
					call fh_mask % put_att("units", "unitless")        
					call fh_mask % put_att("long_name", "mask")   
					call fh_mask % close_file()
					write(*,'(A)') 'Adding ocean mask... success.'

					!print out this % mask2D
					write(*,'(A$)') 'printing ASCII mask2D... '
					allocate (bt    (2:il-1, 2:jl-1))
					call charar(mask2D,bt,il-2, jl-2)
					call out_map('t02','_this % mask2D',bt,x1_v,x2_v,il-2, jl-2)
					!call charar(kt,bt,il,jl)
					!call out_map('t02','_kt',bt,x1_t,x2_t,il,jl)
					deallocate(bt)
					write(*,'(A)') 'done.'
					
					!call grid_analysis (this)
					
				end if
			end subroutine o_make_mask
		!end contains
	end subroutine make_grid

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main to add to IC-file all data you want.
! Here you should (probably) create FileHandler_NC object and put data.
!----------------------------------------------------------------------------------------

	subroutine make_ic(this, name, dims, grid_type)
		class(ocn_off_adapter) :: this
		
		! These parameters are passed from main program - you can use it
		character(*), intent(in) :: name, grid_type
		integer, intent(in) :: dims(3)
		type(FileHandler_NC) :: fh_ic    
		character(100) :: lev_filename    
			
		write(*,'(A$)') 'Adding ocean IC...'
			
		! Put topo, kv, kt to IC-file
		call fh_ic % open_file(this % get_ic_filename())
		call fh_ic % create_var("topo", "real4", "i", "j")
		call fh_ic % put_att("units", "meters")
		call fh_ic % put_att("long_name", "Bottom topography")
		call fh_ic % put(hv(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank)), lo = (/ 1, 1 /), hi = dims(1:2))       
		call fh_ic % create_var("kv", "real4", "i", "j")
		call fh_ic % put_att("units", "unitless")     
		call fh_ic % put_att("long_name", "kv-array")     
		call fh_ic % put(REAL(kv(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank))), lo = (/ 1, 1 /), hi = dims(1:2))   
		call fh_ic % create_var("kt", "real4", "i", "j")
		call fh_ic % put_att("units", "unitless")     
		call fh_ic % put_att("long_name", "kt-array")     
		call fh_ic % put(REAL(kt(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank))), lo = (/ 1, 1 /), hi = dims(1:2))   
		
		! Process levitus IC
		lev_filename = TRIM(DATA_FOLDER)//TRIM(name)//"_"//TRIM(n2s(dims(1)))//'x'//TRIM(n2s(dims(2)))//'_T'
		call process_levitus(TRIM(lev_filename)//"_t_an_z.nc", "t_an", fh_ic)
		call process_levitus(TRIM(lev_filename)//"_s_an_z.nc", "s_an", fh_ic)
			
		call fh_ic % close_file()

		write(*,'(A)') 'success.'

		contains
    
			subroutine process_levitus(filename, var_name, fh_ic)
				character(*), intent(in) :: filename, var_name
				type(FileHandler_NC) :: fh_ic, fh_lev
				real(kind=4), dimension(iwest(myrank):ieast(myrank), jsouth(myrank):jnorth(myrank), kl) :: var
				real(4) :: mis_val
				
				! Read var from interpolated file
				call fh_lev % open_file(filename)
				call fh_lev % open_var(var_name)
				call fh_lev % get_att(att_name = "_FillValue", att_val = mis_val)
				call fh_lev % get(var, lo = (/ 1, 1, 1 /), hi = dims , dt = DateTime("1900-01-01 00:00:00"))
				call fh_lev % close_file()
				
				! Process it
				call process_levitus_var(var, var_name, mis_val)
				
				! Save processed var to IC-file
				call fh_ic % create_var(var_name, "real4", "i", "j", "k")
				call fh_ic % put(var, (/ 1, 1, 1 /), hi = dims)
				call fh_ic % put_att("units", "")        
				call fh_ic % put_att("long_name", "Levitus initial conditions")   
			end subroutine process_levitus
		!end contains
	end subroutine make_ic
    
!======================================================================================

	subroutine additional_dump_mask
		!-------
		!	editted by: Ibrayev, Rashit, 2016.08.05
		!	translated to CMF3 structure: Konstantin Ushakov, 2018.02
		!-------
		type(FileHandler_NC) :: fh_mask
		
		call fh_mask % open_file(TRIM(add_grid_info_filename()))
		call fh_mask % open_or_create_var("kv", "real8", "i", "j")
		call fh_mask % put(DBLE(kv(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_mask % put_att("units", "levels")        
		call fh_mask % put_att("long_name", "bottom topography of the V-grid")   
		
		call fh_mask % open_or_create_var("kt", "real8", "i", "j")
		call fh_mask % put(DBLE(kt(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_mask % put_att("units", "levels")        
		call fh_mask % put_att("long_name", "bottom topography of the T-grid")
		call fh_mask % close_file()
	
	end subroutine additional_dump_mask
	
!=======================================================================================

	subroutine additional_dump_grid
		!-------
		!	editted by: Ibrayev, Rashit, 2016.08.05
		!	translated to CMF3 structure: Konstantin Ushakov, 2018.02
		!-------
		type(FileHandler_NC) :: fh_grid

		!vectors to compute angle between x direction and true east (of spherical long-lat system)
		real(kind=4), allocatable, dimension(:,:) :: tsc_u, tsc_v, ssc_u, ssc_v
		real(kind=8), allocatable, dimension(:,:) :: angle
		allocate(angle(il, jl))

		! Make grid arrays dx1_**_ - required for ice cice grid generation
		call allocate_o_grid_step_module
		call o_ini_set_grid_step
		
		call fh_grid % create_file(TRIM(add_grid_info_filename()))
		call fh_grid % create_dim('i', il)
		call fh_grid % create_dim('j', jl)
		call fh_grid % create_dim('k', kl)
				
		call fh_grid % create_var("x1_v", "real8", "i", "j")
		call fh_grid % put(DBLE(x1_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "degrees")        
		call fh_grid % put_att("long_name", "longitudes of V-grid points")   
		
		call fh_grid % create_var("x2_v", "real8", "i", "j")
		call fh_grid % put(DBLE(x2_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "degrees")        
		call fh_grid % put_att("long_name", "latitudes of V-grid points")   
	
		call fh_grid % create_var("x1_t", "real8", "i", "j")
		call fh_grid % put(DBLE(x1_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "degrees")        
		call fh_grid % put_att("long_name", "longitudes of T-grid points")   
		
		call fh_grid % create_var("x2_t", "real8", "i", "j")
		call fh_grid % put(DBLE(x2_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "degrees")        
		call fh_grid % put_att("long_name", "latitudes of T-grid points")   
		
		!---V-steps---
		call fh_grid % create_var("dx1_nw_v", "real8", "i", "j")
		call fh_grid % put(DBLE(dx1_nw_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "NW-ern size of V-box in x-dimension")   

		call fh_grid % create_var("dx1_ne_v", "real8", "i", "j")
		call fh_grid % put(DBLE(dx1_ne_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "NE-ern size of V-box in x-dimension")
		
		call fh_grid % create_var("dx2_en_v", "real8", "i", "j")
		call fh_grid % put(DBLE(dx2_en_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "EN-ern size of V-box in y-dimension")   
		
		call fh_grid % create_var("dx2_es_v", "real8", "i", "j")
		call fh_grid % put(DBLE(dx2_es_v(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "ES-ern size of V-box in y-dimension")   
		
		!---the same for t-steps---
		call fh_grid % create_var("dx1_nw_t", "real8", "i", "j")
		call fh_grid % put(DBLE(dx1_nw_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "NW-ern size of T-box in x-dimension")   

		call fh_grid % create_var("dx1_ne_t", "real8", "i", "j")
		call fh_grid % put(DBLE(dx1_ne_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "NE-ern size of T-box in x-dimension")
		
		call fh_grid % create_var("dx2_en_t", "real8", "i", "j")
		call fh_grid % put(DBLE(dx2_en_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "EN-ern size of T-box in y-dimension")   
		
		call fh_grid % create_var("dx2_es_t", "real8", "i", "j")
		call fh_grid % put(DBLE(dx2_es_t(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "meters")        
		call fh_grid % put_att("long_name", "ES-ern size of T-box in y-dimension")

		!angle = angle to rotate FROM true east TO bipolar east (lies in [-pi,pi])
		if (bsc_flag .eq. 0) then
			angle=0.
		else
			allocate(tsc_u(il, jl), tsc_v(il, jl))
			allocate(ssc_u(il, jl), ssc_v(il, jl))
			ssc_u = 10.0
			ssc_v = 0.0
			call ssc_tsc_rotate_t(ssc_u, ssc_v, tsc_u, tsc_v, 1, il, 1, jl)
			angle=-asin(tsc_v/ssc_u)
			deallocate(tsc_u,tsc_v,ssc_u,ssc_v)
		end if
		
		call fh_grid % create_var("angle", "real8", "i", "j")
		call fh_grid % put(DBLE(angle(1:il,1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
		call fh_grid % put_att("units", "radians in [-pi,pi]")        
		call fh_grid % put_att("long_name", "angle FROM east TO bipolar east in T-points")   
		call fh_grid % close_file()
		
	end subroutine additional_dump_grid

!========================================================================================	
	
	function add_grid_info_filename
		character(len=256) add_grid_info_filename
		add_grid_info_filename = "./data/ADD_GRID_INFO_OCN_"//TRIM(n2s(il-2))//'x'//TRIM(n2s(jl-2))//".nc"
	end function add_grid_info_filename    

!========================================================================================

!--------------------------------------
!	The following procedures analyse model grid and
!	- output horisontal grid step array to file
!	- output horisontal grid statistics
!	2016.08.05
!	author: Ibrayev, Rashit
!-------
!--------------------------------------


	function dgrid_filename
		character(len=256) dgrid_filename
		dgrid_filename = TRIM(data_folder)//"grid_step_ocn_"//TRIM(n2s(il-2))//'x'//TRIM(n2s(jl-2))//".nc"
	end function dgrid_filename

!~~~~~~~
	subroutine grid_analysis
!~~~~~~~
	
	type(FileHandler_NC) :: fh_grid
	integer	(kind=4) j0_grid, j1_grid
	
	real (kind=8), allocatable:: dx	(:,:)
	real (kind=8), allocatable:: dy	(:,:)
	real (kind=8), allocatable:: dgrid(:,:)

	write(*,*) 'subroutine grid_analysis start '
	write(*,*) 'subroutine grid_analysis (il,jl)= ',il,jl

	allocate (dx (1:il,1:jl))
	allocate (dy (1:il,1:jl))
	allocate (dgrid (1:il,1:jl))

	dx = 0.
	dy = 0.
	dgrid = 0.

	call allocate_o_grid_step_module
	call o_ini_set_grid_step


!	write(*,*) 'subroutine grid_analysis #01'
!-------
!	compute 'normal' grid steps for V-cells
!-------
	area_tbox = 0.
	do j = 2, jl-1
	do i = 2, il-1
	if(kv(i,j) > 0) then
     	dx(i,j) = dx1_nw_v(i,j) + dx1_ne_v(i,j)
	dy(i,j) = dx2_en_v(i,j) + dx2_es_v(i,j)
	end if
	end do
	end do

!	write(*,*) 'subroutine grid_analysis #02'
!-------
!	mean grid step of V-boxes
!-------
	area_vbox = 0.
	do j = 2, jl-1
	do i = 2, il-1
		if(kv(i,j) > 0) then
			area_vbox (i,j) =				&
     				(dx1_nw_v(i,j)*dx2_en_v(i-1,j)+	&
     				 dx2_es_v(i,j)*dx1_ne_v(i,j-1))+	&
     				(dx1_ne_v(i,j)*dx2_en_v(i,j)+		&
     				 dx2_es_v(i-1,j)*dx1_nw_v(i,j-1))

	dgrid (i,j) = sqrt(area_vbox (i,j))
		end if
	end do
	end do

	open (888,file = TRIM(data_folder)//"grid_mmm_ocn_"//TRIM(n2s(il-2))//'x'//TRIM(n2s(jl-2)))
	write(888,'(a,2i5)') '         (il,jl)=', il,jl
	write(888,*) '        min,      max,     aver,   i_min,   j_min,   i_max,   j_max,  n_aver'

!-------
!	compute min, max, average grid steps (dx,dy,sqrt(area_vbox)) for V-cells. all grid
!-------
	j0_grid = 2
	j1_grid = jl-1
	write(888,'(30x,a,2i5)') 'grid all, (j0_grid, j1_grid)=',j0_grid, j1_grid
	call step_mmm('dx',dx,j0_grid, j1_grid)
	call step_mmm('dy',dy,j0_grid, j1_grid)
	call step_mmm('dm',dgrid,j0_grid, j1_grid)

	if (bsc_flag == 1) then
!-------
!	compute min, max, average grid steps (dx,dy,sqrt(area_vbox)) for V-cells. ssc grid
!-------
	j0_grid = 2
	j1_grid = jl_ssc-1
	write(888,'(30x,a,2i5)') 'grid ssc, (j0_grid, j1_grid)=',j0_grid, j1_grid
	call step_mmm('dx',dx,j0_grid, j1_grid)
	call step_mmm('dy',dy,j0_grid, j1_grid)
	call step_mmm('dm',dgrid,j0_grid, j1_grid)

	j0_grid = 2
	j1_grid = jl_ssc
	write(888,'(30x,a,2i5)') 'grid ssc, (j0_grid, j1_grid)=',j0_grid, j1_grid
	call step_mmm('dx',dx,j0_grid, j1_grid)
	call step_mmm('dy',dy,j0_grid, j1_grid)
	call step_mmm('dm',dgrid,j0_grid, j1_grid)

!-------
!	compute min, max, average grid steps (dx,dy,sqrt(area_vbox)) for V-cells. bsc grid
!-------
	j0_grid = jl_ssc+2
	j1_grid = jl-1
	write(888,'(30x,a,2i5)') 'grid bsc, (j0_grid, j1_grid)=',j0_grid, j1_grid
	call step_mmm('dx',dx,j0_grid, j1_grid)
	call step_mmm('dy',dy,j0_grid, j1_grid)
	call step_mmm('dm',dgrid,j0_grid, j1_grid)

	j0_grid = jl_ssc+1
	j1_grid = jl-1
	write(888,'(30x,a,2i5)') 'grid bsc, (j0_grid, j1_grid)=',j0_grid, j1_grid
	call step_mmm('dx',dx,j0_grid, j1_grid)
	call step_mmm('dy',dy,j0_grid, j1_grid)
	call step_mmm('dm',dgrid,j0_grid, j1_grid)

	end if	!	if (bsc_flag = 1) then

	close(888)

!-------
!	save grid steps
!-------
	write(*,*) 'subroutine grid_analysis printing '

	call fh_grid % open_file(TRIM(dgrid_filename()))
	call fh_grid % create_var("dgrid", "real8", "i", "j")
	call fh_grid % put(dgrid(1:il, 1:jl), (/ 1, 1 /), hi = (/ il, jl /))
	call fh_grid % put_att("units", "meters")        
	call fh_grid % put_att("long_name", "grid step")   
	
	call fh_grid % create_var("dx", "real8", "i", "j")
	call fh_grid % put(dx(1:il, 1:jl), (/ 1, 1 /), hi = (/ il, jl /))
	call fh_grid % put_att("units", "meters")        
	call fh_grid % put_att("long_name", "x step")   
	
	call fh_grid % create_var("dy", "real8", "i", "j")
	call fh_grid % put(dy(1:il, 1:jl), (/ 1, 1 /), hi = (/ il, jl /))
	call fh_grid % put_att("units", "meters")        
	call fh_grid % put_att("long_name", "y step")   
	
!-------
!	save model bottom topography for plot
!-------
	hv = 0.
	do j = 1, jl
	do i = 1, il
	if (kv(i,j) >= 1) then
	hv(i,j) = zw(kv(i,j)+1)
	end if
	end do
	end do

	call fh_grid % create_var("hv", "real8", "i", "j")
	call fh_grid % put(dble(hv(1:il, 1:jl)), (/ 1, 1 /), hi = (/ il, jl /))
	call fh_grid % put_att("units", "meters")        
	call fh_grid % put_att("long_name", "model topography, kl descrete levels")   
	
	call fh_grid % close_file()

	end subroutine grid_analysis


!~~~~~~~
	subroutine step_mmm(dname,dstep,j0_grid, j1_grid)
!~~~~~~~

!-------
!	find min, max, mean of 'dstep' array
!	basic subroutine for analysing model grid and
!	- output horisontal grid step array to file
!	- output horisontal grid statistics
!	2016.08.05
!	author: Ibrayev, Rashit
!-------

!	class(off_ocn) :: this

	character(len=2) dname
	real	(kind=4) dstep_min, dstep_max, dstep_average
	integer	(kind=4) i_dstep_min, j_dstep_min, i_dstep_max, j_dstep_max, n_dstep_average
	integer	(kind=4) j0_grid, j1_grid
	real (kind=8) dstep (il,jl)

!-------
!	compute min, max, average of 'dstep' array
!-------
	dstep_min = 999999.
	i_dstep_min = 0
	j_dstep_min = 0

	dstep_max = 0.
	i_dstep_max = 0
	j_dstep_max = 0

	dstep_average = 0.
	n_dstep_average = 0

	do j = j0_grid, j1_grid
	do i = 2, il-1
	if(kv(i,j) > 0) then

	if ( dstep(i,j) < dstep_min ) then
		dstep_min = dstep(i,j)
		i_dstep_min = i
		j_dstep_min = j
	end if

	if ( dstep(i,j) > dstep_max ) then
		dstep_max = dstep(i,j)
		i_dstep_max = i
		j_dstep_max = j
	end if

	dstep_average = dstep_average + dstep(i,j)
	n_dstep_average = n_dstep_average + 1

	end if
	end do
	end do

	if (n_dstep_average /= 0 ) dstep_average = dstep_average / real(n_dstep_average)

!-------
!	save grid steps
!-------
	write(888,'(a2,3f10.2,5i9)') dname,dstep_min, dstep_max, dstep_average, i_dstep_min, j_dstep_min, i_dstep_max, j_dstep_max, n_dstep_average

	end subroutine step_mmm

end module