
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

module o_par_module

#if CMF_VERSION == 2
    use shared_module
    use comp_module
    use comp_halo_update_module 
#elif CMF_VERSION == 3
    use component_splitter_module
    use halo_updater_module
    use decomposition_module
    use model_info_module
    use experiment_info_module
    use point_saver_module
    use component_module
#endif

    implicit none

    integer :: ocn_comm
    integer :: ocn_comm_size
    integer :: ocn_rank_local
    
    character(NAME_LEN) :: atm_model_name
    character(NAME_LEN) :: lnd_model_name
    character(NAME_LEN) :: ice_model_name
    logical :: cice_and_schrum=.false.
    integer :: ice_remapping_mode=1		!0 - traditional mapping (fluxes are per m2 of ice), 1 - NEMO-type mapping (fluxes are per m2 of grid cell). 
						!This applies to salt, heat, water and sst remapped from CICE to OCN  

#if CMF_VERSION == 2

#elif CMF_VERSION == 3
    type(Decomposition) :: ocn_decomp
    integer :: time_l, time_l_in_run
    class(Component), pointer :: cmp_ptr
    type(PointSaver) :: test_ps
    character(NAME_LEN) :: run_experiment='aaa'
    integer :: time_l_save_cp, time_l_save_dg
    integer time_year,time_month,time_day,time_hour,time_min,time_sec
    
#endif

 contains

#if CMF_VERSION == 2

subroutine o_par_init()
    ocn_comm = comm_local
    ocn_rank_local = rank_local
    ocn_comm_size = size_local
    
    atm_model_name = cmp % names(COMP_ATM)
    lnd_model_name = cmp % names(COMP_LND)
    ice_model_name = cmp % names(COMP_ICE)
end subroutine

#elif CMF_VERSION == 3

subroutine o_par_init(cmp)
    class(Component), intent(in), target :: cmp
    cmp_ptr => cmp
    
    ocn_comm = CompSplitter % comm_local()
    ocn_rank_local = CompSplitter % rank_local()
    ocn_comm_size = CompSplitter % comm_local_size()
    ocn_decomp = ModelsInfo(CompSplitter % i_am_id()) % decomp()
    
    run_experiment=trim(ExpInfo % exp_name())

    !call test_ps % init("ocn_var1", 100)
    
end subroutine

subroutine o_par_after_sync()
    
    atm_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("ATM")) % model_name())
    lnd_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("LND")) % model_name())
    ice_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("ICE")) % model_name())

end subroutine

subroutine shared_timer
	type(DateTime) :: dt
	dt = cmp_ptr % model_time()
	time_year = dt % year
	time_month = dt % month
	time_day = dt % day
	time_hour = dt % hour
	time_min = dt % minute
	time_sec = dt % second
end subroutine
	
#endif

#if CMF_VERSION == 2

subroutine o_par_destroy()
  
end subroutine

#elif CMF_VERSION == 3

subroutine o_par_destroy()
    
    !call test_ps % destroy()
    
end subroutine

#endif
 
!----------------------------------------------------------------------------------------
! Decription: Print message and stop program
! Parameters:
!    msg      - string message
!    mode     - 'LCL' - all ocean cores, 'IND' - only one core
!----------------------------------------------------------------------------------------

subroutine ocn_error(msg, mode)
    character(*), intent(in) :: msg
    character(3), intent(in) :: mode

#if CMF_VERSION == 2
    call shared_error(msg)
#elif CMF_VERSION == 3
    call CompSplitter % sync_error(msg, mode)
#endif
    
end subroutine    

end module

