
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	module for sea ice coupling
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!		(based on the code provided by Schrum, Corinna, 
!		Institute for Oceanography, University of Hamburg, Germany, Corinna.Schrum@hzg.de)
!
! reference:
!	Ushakov K.V., Grankina T.B., Ibrayev R.A., Gromov I.V., 2016. 
!		Simulation of Arctic and North Atlantic ocean water and ice seasonal characteristics 
!		by the INMIO-CICE coupled model. IOP Conf. Series: Earth and Environmental Science. 48(2016) 012013.
!	Schrum C., and J. Backhaus, 1999. 
!		Sensitivity of atmosphere-ocean heat exchange and heat content in North Sea and Baltic Sea. A comparative Assessment. 
!		Tellus 51A, 1999, 526-549.
!--------------------------------------

!============================
! Arrays for SLAV atm model
!============================

	module atm_module
	save

!------
!	data sent to atm
! 	need, because of coupler bug with send-receive of real*8 arrays
!------
	real (kind=4), allocatable:: ocn_atm_sst	(:,:)
	real (kind=4), allocatable:: atm_ocn_windstr_x	(:,:)
	real (kind=4), allocatable:: atm_ocn_windstr_y	(:,:)
	end module atm_module


!=======
	subroutine allocate_atm_module
!=======
	use o_grid_module
	use atm_module
	implicit none
	
	allocate (ocn_atm_sst		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (atm_ocn_windstr_x	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (atm_ocn_windstr_y	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	
	ocn_atm_sst = 0.0
	atm_ocn_windstr_x = 0.0
	atm_ocn_windstr_y = 0.0
	end


!=======
	subroutine make_ocn_atm_fluxes	!Data to be sent to atm
!=======
	use o_basic_module
	use atm_module
	implicit none
	
	ocn_atm_sst=t_c(1,:,:,1)
	
	end 
