
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	sea ice thermodynamic model
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!		(based on the code provided by Schrum, Corinna, 
!		Institute for Oceanography, University of Hamburg, Germany, Corinna.Schrum@hzg.de)
!
! reference:
!	Schrum C., and J. Backhaus, 1999. 
!		Sensitivity of atmosphere-ocean heat exchange and heat content in North Sea and Baltic Sea. A comparative Assessment. 
!		Tellus 51A, 1999, 526-549.
!--------------------------------------


!---------------------------------------------------------------------
!	2006.10.24
!	copy from cs_9/icetd1.f
!	add air-ice-sea fluxes from cs_9/sstd33.f
!
!	Routine that arranges calculating of heat fluxes 
!	thermodynamic ice-model, 
!	for details and discription see info_eis_hamsom.txt 
!	all units are in SI (J, kg, m)
!---------------------------------------------------------------------

	subroutine ice_td_schrum


	use o_grid_module
	use o_basic_module
	use o_tf_module
	use o_td_module
	use ice_module
	use o_d_vmc_1_module
	use test_ocean_module
	use o_par_module
	implicit none
	
	integer idmax(1)
	real khwat
	real atem,cp1,cp2,cp_w,cpwat,d_w,da1,da2,dh_ibot,ctime
	real dh_ice,dh_ice1,dh_itop,dh_new,h_i,h_ice,patmo
	real ds_w,dt_w,dt_w1,dtime,e10,esatice,evapice,f_ice,f_wat
	real s_i,s_w,t_w,tf_ibot,tf_itop,tfreez,esati,sens,evapa
	integer (kind=4)	ice
	logical :: ice_relax=.false.

	
	!----Interim ice relaxation----------
	if(ice_relax) then
		do j=jsouth(myrank)-1	,jnorth(myrank)+1
		do i=iwest (myrank)-1	,ieast (myrank)+1
			da1=(nudge_aice(i,j)-aice(i,j))*htsec/3600./24.
			
			if( aice(i,j).gt.epsice ) then
				hice(i,j)=hice(i,j)*(aice(i,j)+da1)/aice(i,j)
			else
				hice(i,j)=max(0.,h0_ice*da1)
			end if
			temp1(i,j)=aice(i,j)-nudge_aice(i,j)
			aice(i,j)=aice(i,j)+da1
		end do
		end do
	end if
	
	
	mask_ice=0
	ice=0
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		if(kt(i,j).gt.0 .and. hice(i,j).gt.epsice .and. aice(i,j).gt.epsice ) then
			mask_ice(i,j)=1
			ice=1
		end if
	end do
	end do

	if(ice.eq.0) go to 1

!---------------------------------------------------------------------
!1.	compute heat fluxes at air-ice
!---------------------------------------------------------------------
!-------
!1.0.1.	find temperature of ice
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		if(mask_ice(i,j).eq.1)	then
			t_w = t_f(1,i,j,1)
			s_w = t_f(1,i,j,2)+S_ref
			if(s_w.lt.0.) s_w=0.
			s_i=0.2*s_w
			!tf_ibot= tfreez(s_w,0.)
			tf_itop= tfreez(s_i,0.)
			tice(i,j)=min (tf_itop,tair10(i,j))
		end if
	end do
	end do

!-------
!1.1.	compute downward solar radiation fluxes at ice surface
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		if(mask_ice(i,j).eq.1)	ssr_ice(i,j)=ssr(i,j)*(1-albedo_ice)
	end do
	end do

!-------
!1.2.	compute upward thermal radiation fluxe
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		!Is the second **4 term necessary???
		if(mask_ice(i,j).eq.1)	str_ice(i,j)=str(i,j)-	5.67e-8*(tice(i,j)+273.16)**4 !+ 5.67e-8*(t_f(1,i,j,1)+273.16)**4
	end do
	end do

!-------
!1.3.	sensible heat flux at air-ice interface 
!-------
	patmo=1013.           ! atmospheric pressure [mb]
	select case (atm_model_name)
		case("ATM_NCAR","ATM_INTERIM","ATM_GFS","ATM_INMCM4")
			do j=jsouth(myrank)-1	,jnorth(myrank)+1
			do i=iwest (myrank)-1	,ieast (myrank)+1
				if(mask_ice(i,j).eq.1)	then
					e10=patmo*q10(i,j)/(0.622+0.378*q10(i,j))
					esatice=esati(tice(i,j))	![Pa] saturation vapor pressure over the ice
					sns_ice(i,j)=-sens(	wind_speed_m_t(i,j),	tair10(i,j),tice(i,j),e10,esatice)
!-------
!1.4.	latent heat flux at air-ice interface 
!-------
					evapice=evapa(	wind_speed_m_t(i,j),tair10(i,j),tice(i,j),e10,esatice)
					ltn_ice(i,j)=-evapice*(2.5008e6 - 2.3e3 * tair10(i,j))*0.51975
				end if
			end do
			end do
		case("ATM_INM")
			sns_ice=sns
			ltn_ice=ltn
		case default
			call ocn_error('no ice procedures for atmosphere <'//atm_model_name//'>.', 'LCL')
	end select
!-------
!	heat balance at the air-ice interface 
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		if(mask_ice(i,j).eq.1)	qh_ai(i,j)=ssr_ice(i,j)+str_ice(i,j)+sns_ice(i,j)+ltn_ice(i,j)
	end do
	end do

1	continue

	ctime=1.
	dtime=htsec*ctime
	!h0_ice	= 0.2	!m, thickness of normal ice, used in 'aice'
	!hice0	= 0.05	!m, minimum value of ice thickness
	d_w = densit	!kg/m**3
!	khwat=20.e-4	![m**2/s]
	khwat=c1Ahv+c2ahv(1)

!-------
!	compute heat fluxes at ice-sea interface and 
!	solve ice thermodynamics equation
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		if(kt(i,j).gt.0) then

			t_w = t_f(1,i,j,1)  
			s_w = t_f(1,i,j,2)+S_ref 
!-------
!			control negative salinity
!-------
			if (s_w.lt.0.) s_w=0.

			s_i=0.2*s_w	
			tf_ibot= tfreez(s_w,0.) 
			tf_itop= tfreez(s_i,0.)
			tice(i,j)=min (tf_itop,tair10(i,j))
!-------
!			Specific heat of water
!-------
			cp1=4219.8-6.6 *s_w		!these values *1000.
			cp2=4080. -2.47*s_w
			atem=0.05*t_w
			atem=min(atem,1.)
			atem=max(atem,0.)
			cp_w=(atem*cp2+(1.-atem)*cp1)	!J/(kg*K)
			cpwat = cp_w*densit*dx3_t_c(2,i,j) !hzw(2)
			dh_ice1=0.

!-------
!			there is ice on water, determine  heat-fluxes, for new ice 
!			surface temperature tice(1,i,j) using the old ice surface 
!			temperatures tp 
!-------
			if(mask_ice(i,j).eq.1) then
				h_i = hice(i,j)/aice(i,j)	!real thickness of ice (in the part aice)
!-------
!				heat flux through the ice (f_ice) and ice-sea interface (f_wat)
!				at time step 'L-1'
!-------
				f_ice = -k_ice*(tf_ibot - tice(i,j))/h_i
				!f_wat = -cp_w*d_w*khwat*(t_w - tf_ibot)/(0.5*hz(1)+ssl_c(i,j))   !*(0.5*dzv)
				f_wat = -cp_w*d_w*khwat*(t_w - tf_ibot)/(0.5*dx3_t_c(2,i,j))   !*(0.5*dzv)
!-------
!				ocean temperature change due to heat loss to the ice 
!-------
				dt_w1 = dtime*f_wat/cpwat
!-------
!				ice production or ice melting at the bottom due to (im-)balance between 
!				heat flux through the ice and oceanic heat loss (fc and fw) 
!-------
				dh_ibot =  dtime*(f_wat - f_ice)/(d_ice*q_ice) 
				!temp1(i,j)=temp1(i,j)+dh_ibot*aice(1,i,j)
!-------
!				ice melting at the surface of the ice (at tf_itop!) 
!				melting rate xmelt (only negative --> melting!) 
!-------
				dh_itop=(f_ice-qh_ai(i,j))*dtime/(d_ice*q_ice)
				if(dh_itop.gt.0.) dh_itop=0.	!no freezing at the ice top surface
				!temp2(i,j)=temp2(i,j)+dh_itop*aice(1,i,j)
!-------
!				ice thickness change due to freezing and melting 
!				at the bottom (dh_ibot) 
!				and melting at the top (dh_itop) 
!-------
				dh_ice1 = dh_ibot + dh_itop

!-------
!				determine temperature change due to heat-flux (loss) to the  atmosphere   !commented out since it is calculated in ordinary way in td_scalar
!-------
				!dt_w2 = t_source(1,i,j,1) * dtime / (cp_w*densit)

!-------
!				change of water temperature at surface layer 
!				due to heat loss to ice and to air
!-------
				dt_w=aice(i,j)*dt_w1 !+ (1.0-aice(i,j))*dt_w2 
				t_w=t_w+dt_w	
			end if		

!-------
!			produce ice thickness in open water 
!-------
			dh_new=0.
			if(t_w.lt.tf_ibot) then
				dt_w=tf_ibot-t_w 
				dh_new=cpwat*dt_w/(d_ice*q_ice) 
				!temp3(i,j)=temp3(i,j)+dh_new
				t_w=tf_ibot	
			end if
!-------
!			change of average ice thickness
!-------
!! 			in next line is it right to multiply dg_new by (1-A)   ??? No!
			dh_ice = aice(i,j)*dh_ice1 + dh_new
			hice(i,j) = hice(i,j) + dh_ice

!-------
!			increment of water salinity due to thermodynamic change of ice-thickness 
!-------
			ds_w= (s_w-s_i)*d_ice*dh_ice/(d_w*dx3_t_c(2,i,j))

!-------
!			change t,s of the upper boxes
!-------
			t_f(1,i,j,1) = t_w
			t_f(1,i,j,2) = t_f(1,i,j,2) +ds_w

!-------
!			change of ice compactness due to freezing or melting 
!-------
						da1=0.
			if(dh_new.gt.0.)	da1=(dh_new/h0_ice)*(1.-aice(i,j))
						h_ice=hice(i,j)
			if(h_ice.lt.hice0) 	h_ice=hice0
						da2=0.
			if(dh_ice.lt.0.)	da2=( aice(i,j)/(2.*h_ice) )*dh_ice
			aice(i,j)=aice(i,j)+da1+da2

			if((hice(i,j).lt.epsice).or.(aice(i,j).lt.epsice)) then 
				hice(i,j) = 0.0 
				aice(i,j) = 0.0 
			end if 
			
			aice(i,j)=min(1.,aice(i,j))
		end if		!(kt(i,j).gt.0) then
	end do
	end do

!----------------------------------------------------------------------
!	Heat flux in the sea: Qh= Cp*densit* Qh',
!	where Qh' is in water =-{K*dT/dz},
!	in heat transport eq. at the sea surface Qh' is prescribed.
!	Qh'=Qh/(Cp*densit), value of Qh' is in [Celcius*m/s]
!	where Cp=4217 J/kg*K - specific heat of sea water
!	densit=1017 kg/m**3 - sea water density, see subr. setup
!----------------------------------------------------------------------
	end 



!~~~~~~
	real function tfreez (s,p) 
!~~~~~~
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
!	freezing point of sea water (millero 1978) !p-term commnted out!!
	data ca/-0.0575/,cb/1.710523e-3/,cc/-2.154996e-4/,cd/-7.53e-3/
	tfreez = s*(ca+cb*sqrt(s)+cc*s)  !+cd*p 
	return 
	end 




      function esata(tt) 
      IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
      ai    = (7.5*tt)/(237.3+tt) 
      esata = 610.7*(10.**ai) 	![Pa]
      return 
      end 

      function esati(tt)
      IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
      ai    = (9.5*tt)/(265.5+tt) 
      esati = 610.7*(10.**ai)		! [Pa]
      return 
      end 

      function sens(u,ta,ts,ea,es)
      IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
      us=u
      if (u.le.0.3) u=0.3
      if (u.gt.50.) u=50.
      if (u.le.2.2) goto 20 
      if (u.le.5.) goto 30 
      if (u.le.8.) goto 40 
      if (u.le.25.) goto 50 
      a=1.652
      b=-0.017 
      ch=a+b*u
      goto 60 
50    continue 
      a=1.17 
      b=0.0075 
      c=-0.00045 
      ch=a+b*u+c*((u-8.)**2)
      goto 60 
40    continue 
      a=1.15 
      b=0.01 
      ch=a+b*u
      goto 60 
30    continue 
      a=0.927 
      b=0.0546 
      ch=a+b*u
      goto 60 
20    continue 
      b=1.185 
      p=-0.157 
      ch=b*(u**p)
60    continue
      snul=((ts-ta)+0.0011*(es-ea))/(u**2)
      s=snul*abs(snul)/(abs(snul)+0.01)
      s=max(s,-3.3)
      if (s.gt.0.) ch=ch*(1.e-3+0.63e-3*sqrt(s))
      if (s.le.0.) ch=ch*(1.e-4+3.e-5*s+9.e-3*exp(4.8*s))
      u=us 
      csens=1.28*1015.*ch*u 
      sens=csens*(ts-ta)	! [W/m**2]
      return 
      end 

      function evapa(u,ta,ts,ea,es)
      IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
      ra=ea/(461.*(ta+273.15))
      rs=es/(461.*(ts+273.15))
      evapa=1.6e-3*u*(rs-ra)	! [kg/m**2*s]
      return
      end

   
