!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2017.01.31 / Koromyslov, Alexandr, INM RAS, Russia, kor175@mail.ru
!	2017.09.01 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Ushakov K.V., Ibrayev R.A. Simulation of the global ocean thermohaline state by the 
!		eddy-resolving INMIO model configuration. IOP Conference Series: Earth and Environmental Science. 2017. 8 pp. 
!
!--------------------------------------
#if CMF_VERSION == 2
#include "o_defines.f90"
#include "ocn_2Ddomain"

	module o_analysis_module
	
	use c_module
	use c_rw_module
	use comp_module
	use c_gd_module
	use shared_module
	use shared_utils_module
	use netcdf
	use o_driver_module
	use o_basic_module
	use o_analysis_utils_module
	use o_td_adv_cd
	
	implicit none

	character(LEN=NAME_LEN) :: file_name, model_time
	integer :: a_counter = 0
	real (kind=main_kind), allocatable:: a_3d_read	(:,:,:),a_2d_read(:,:)
	
        character (len=16) :: mean_date
        integer:: out_unit=42
        real(8) :: mean_result
        
	CONTAINS

!================================================================================================================================================================

        subroutine o_analysis_allocate

        call allocate_o_grid_coord_module; deallocate(hv)
	call allocate_o_grid_step_module; deallocate(dx3_t_l,dx3_v_f,dx3_t_f,dx3_t_c,dx3_v_l)	
        call allocate_o_basic_module; deallocate(d_c,d_potential_c)
	!call allocate_ice_module; deallocate(tice,ssr_ice,str_ice,sns_ice,ltn_ice,qh_ai,mask_ice,ice_ocn_hice)
        
        !allocate (u_bt_a_f	(   	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (v_bt_a_f	(   	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (h_bt_a_f	(   	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (w_surf_c	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (ct3v		(kl,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	
	allocate (func_u_e	(kl  ,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (func_v_n	(kl  ,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )

	allocate (flow_u_e	(kl+1, 	iwest (myrank)-2:ieast (myrank)+2, jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_v_n	(kl,   	iwest (myrank)-2:ieast (myrank)+2, jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_w_b	(kl+1,  iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )	

	allocate (flow_t_n	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_e	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_b	(0:kl ,	iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )

	!----Arrays needed for FCTZ scheme-------
	!allocate (func_v_e	(kl  ,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (func_u_n	(kl  ,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (func_u_b	(kl+1,	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )	
	!allocate (adv_t	(kl  ,	iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (t_f 		(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	
	call o_diagn_mht_init
	
	!allocate (a_3d_read(kl,iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)))
	!allocate (a_2d_read(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank)))
	
	end subroutine        
        
!================================================================================================================================================================

	subroutine o_analysis_ini_main
	
	integer :: counter
	integer :: ierr
	real (kind=4) land_value
	
#if CMF_VERSION == 3
	! Calculate time_l as difference between exp_start and run_start
	time_l = (date2sec(ExpInfo % start_date()) - date2sec(ExpInfo % exp_start_date())) / (24*60*60) * ocn_steps_in_day + 1
	time_l_in_run = 1
#endif
	
!===================================================================
! Prepare kv and kt, which are read from file in REAL representation
!===================================================================
!=====================================================================
! Make first halo exchanges. Don't forget make initial exchanges for
! all arrays from control point, because they are stored without halo.
!=====================================================================
#if CMF_VERSION == 2
	call comp_halo_update_V4 (DIM2_u = kv_real, DIM2_v = kv_real, update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_T4 (DIM2_t = kt_real, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	! Init global HaloUpdate object
	call HaloUpdateMaster % init(decomp = ocn_decomp, max_halo_width = 2)
	
	call HaloUpdateMaster % update(kv_real, update_width = 1, grid_type = 'V')    
	call HaloUpdateMaster % update(kt_real, update_width = 1, grid_type = 'T')    
#endif
	kv = NINT(kv_real)
	kt = NINT(kt_real)
	
	kv=min(kv,kl)
	kt=min(kt,kl)

!===============================================================
! This cycle make queue for set_grid, because since 1/12 grade
! arrays _ssc and _bsc need more then 1 gb RAM per processor.
! For medium grid you can use casual version of set_grid calls.
!===============================================================
	if(ocn_rank_local == 0) write(*,'(A$)') 'Filtered <set_grid> method... '
 	do counter = 0, 7
 		if(MOD(ocn_rank_local,8) == counter) then
 			call o_ini_set_grid_coordinate
 			call o_ini_set_grid_step
 		end if
 
  	      call MPI_BARRIER(ocn_comm, ierr)
  	end do
	if(ocn_rank_local == 0) write(*,'(A)') 'complete.'

!================
! Set mesh arrays
!================
	call o_ini_cv_ct
	call o_ini_area_box
	
	!---short notations for array boundaries----
	iwe=iwest(myrank)
	iea=ieast(myrank)
	jso=jsouth(myrank)
	jno=jnorth(myrank)
	
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
		dx3_v_c(:,i,j) = hzw(:)
		dx3_v_c(2,i,j) = hzw(2) + 0.25*	( (ssl_c(i+1,j+1)+ssl_c(i,j)) + (ssl_c(i,j+1)+ssl_c(i+1,j)) )
	end do
	end do
	
	!Clear output text files	
	open (unit=out_unit,file="results2d.dat",action="write",status="replace")
	close (out_unit)	
	open (unit=out_unit,file="results3d.dat",action="write",status="replace")
	close (out_unit)
        
        end subroutine

!================================================================================================================================================================
	
	subroutine o_analysis_main_step
	
!------->>>
!       DG analysis section
!-------     	
	integer ii,jj
	if(dg_period /= 0 .AND. MOD(time_l*(24*60/ocn_steps_in_day), dg_period) == 0) then
		
!		mean_date=TRIM(I2S(time_year,4))//"."//TRIM(I2S(time_month,2))//"."//TRIM(I2S(time_day,2))//"-"//TRIM(I2S(time_hour,2))//":00"!"1958.01.01-00:00"!
!
!		!------------Test. read 2d array from DG-file-------
!		call read_dg_file("mask_oceans",return_array_2d=a_2d_read,is_2d=.TRUE.,input_dg_file="mean_test.nc")
!		v_c(1,odom)=a_2d_read
!				
!		do ii=2,il-1
!		do jj=2,jl-1
!			call mean_S(dim2_arr_in=v_c(1,:,:), mask_2d_an = ct3(1,:,:),result_out=mean_result,is_T=.TRUE.,		&
!				lat1=sngl(max(-90.,x2_t(ii,jj)-2.)),								&
!				lat2=sngl(min(90.,x2_t(ii,jj)+2.)),								&
!				lon1=sngl(max(0.,modulo(x1_t(ii,jj),360.)-2.)),							&
!				lon2=sngl(min(359.9,modulo(x1_t(ii,jj),360.)+2.)))
!			v_c(2,ii,jj)=mean_result
!		end do
!		end do
!		return
!		!------End of the test------------
!		
!		
!		!calculate average by S (here on T-grid)
!		call mean_S(dim2_arr_in=v_c(1,:,:), mask_2d_an = ct3(1,:,:),result_out=mean_result,is_T=.TRUE.,lat1=-25.,lat2=-15.,lon1=60.,lon2=65.)
!		!write result to dat file
!		open (unit=out_unit,file="results2d.dat",access="append",action="write",decimal='comma')
!		write(out_unit,*) mean_date,mean_result
!		close (out_unit)
		
		print *,'time_l, time_l_in_run = ',time_l,time_l_in_run,' which is ',time_l_in_run/ocn_steps_in_day,' days'
		
		!temporarily reading into v_c, because dummy argument is 3D, not 4D
		call read_dg_file(var_to_read="t_mean",return_array_3d=v_c(:,odom),is_3d=.TRUE.,input_dg_file='l00_1979-1982_eq-tmean_runmean45days.nc')
		t_c(:,odom,1)=v_c(:,odom)
		!do k=1,kl
		!	t_c(k,iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank),1)=a_3d_read(k,:,:)
		!end do

		call read_dg_file(var_to_read="v_mean",return_array_3d=v_c(:,odom),is_3d=.TRUE.,input_dg_file='l00_1979-1982_eq-vmean_runmean45days.nc')
		!do k=1,kl
		!	v_c(k,iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank))=a_3d_read(k,:,:)
		!end do
		
		call read_dg_file(var_to_read="ssl_mean",return_array_2d=ssl_c(odom),is_2d=.TRUE.,input_dg_file='l00_1979-1982_eq-sslmean_runmean45days.nc')
		!ssl_c(iwest(myrank):ieast(myrank),jsouth(myrank):jnorth(myrank))=a_2d_read(:,:)
		
		t_c(:,:,:,1)=merge(t_c(:,:,:,1),0.,ct3(1:kl,:,:)>0.01)
		v_c=merge(v_c,0.,cv(1:kl,:,:)>0.01)
				
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			dx3_v_c(2,i,j) = hzw(2) + 0.25*	( (ssl_c(i+1,j+1)+ssl_c(i,j)) + (ssl_c(i,j+1)+ssl_c(i+1,j)) )
		end do
		end do
		
		
		
!	  	!calculate average by z (here on V-grid)
!	 	call mean_z(dim3_arr_in=a_3d_tst,dim2_arr_out=a_2d_tst,is_V=.TRUE.,z1=4,z2=10)	!available parameters z1, z2(z1<z2) to specify vertical levels range
!	  	!calculate average by S (here on V-grid)
!	  	call mean_S(dim2_arr_in=a_2d_tst, result_out=mean_result,is_V=.TRUE.)
		
		!open (unit=out_unit,file="meantest-3d.dat",action="write",access='append',decimal='comma')
		!write(out_unit,*) mean_date,v_c(2,17,10:20)
		!close (out_unit)
		
		!if ( ( MOD( a_counter, time_l_save_cp ) == 0 ) .AND. ( rank_local == 0 ) ) then model_time = TRIM(I2S(time_year,4))//TRIM(I2S(time_month,2))//TRIM(I2S(time_day,2))//TRIM(I2S(time_hour,2))
		
		!calculating transport of heat by mean flow (since into t_c, v_c and ssl_c we have read the mean values from a (possibly CDO-modified) CP-file)
		!call o_td_scalar_advection_fctz(1,t_c,t_c,dx3_t_c,mht_adv_u_mean,mht_adv_v_mean)
		call o_td_scalar_advection_cd(0,u_c,v_c,t_c,mht_adv_u_mean,mht_adv_v_mean)
	
	endif
	a_counter = a_counter + 1

	end subroutine 

!================================================================================================================================================================
	
    
	end module o_analysis_module
#endif