!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Offline and online integral diagnostics module
!	
! authors: 
!	2017.10.31 / Gromov, Igor, IO RAS, Russia, gromov@phystech.edu
!
!--------------------------------------
#if CMF_VERSION == 2
#include "o_defines.f90"
#include "ocn_2Ddomain"

	module o_analysis_utils_module
	
	use c_module
	use c_rw_module
	use comp_module
	use c_gd_module
	use shared_module
	use shared_utils_module
	use netcdf
	use o_driver_module

	implicit none
	
	CONTAINS
	
	!This subroutine counts average by z-axis
	!vertical levels range may be specified
	subroutine mean_z(dim3_arr_in, dim2_arr_out,z1,z2,is_V, is_T)
	real(kind=main_kind),intent(in):: dim3_arr_in (kl,	iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(kind=main_kind):: dim2_arr_out( 	iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8):: z_total
	integer, optional :: z1,z2		!specify levels to count average between
	logical, optional :: is_V, is_T  	!this defines which grid to use
	integer :: z_1,z_2			!vertical levels range
	dim2_arr_out=0.
	!set vertical levels range for cycle
	if(.not.present(z1))then
	  z_1=1
	else
	  z_1=z1
	end if
	if(.not.present(z2))then
	  z_2=49
	else
	  z_2=z2
	end if
	if(z_2<z_1)call shared_error("subroutine mean_z. z_2 < z_1")
	!process errors in number of grid types specified
	if(.not.present(is_T).and..not.present(is_V))call shared_error("subroutine mean_z. not specified grid type")
	if(present(is_T).and.present(is_V))call shared_error("subroutine mean_z. both grid types specified")
	
	do i=iwest (myrank),ieast (myrank)
	  do j=jsouth(myrank),jnorth(myrank)
	    z_total=0.						!depth accumulator
	    if(present(is_T))then
	      do k=z_1,min(z_2,kt(i,j))
		dim2_arr_out(i,j)=dim2_arr_out(i,j)+dim3_arr_in(k,i,j) * ct3(k+1,i,j) *  hzw(k+1)
		z_total=z_total+ct3(k+1,i,j) *  hzw(k+1)
	      end do
	    end if
	    if(present(is_V))then
	      do k=z_1,min(z_2,kv(i,j))
		dim2_arr_out(i,j)=dim2_arr_out(i,j)+dim3_arr_in(k,i,j) * cv(k+1,i,j) * hzw(k+1)
		z_total=z_total+cv(k+1,i,j) * hzw(k+1)
	      end do
	    end if
	    if(z_total > 0) then
	      dim2_arr_out(i,j)=dim2_arr_out(i,j) / z_total
	    else
	      dim2_arr_out(i,j)=0!maybe here must be missval
	    end if
	  end do
	end do
	end subroutine mean_z
	
	!This subroutine counts average by area
	!coordinates range may be specified
	
! 	Входные параметры:
! 	- двумерный массив типа main_kind размеров il, jl.
! 	- двумерная маска типа real(4) тех же размеров. Маска может принимать
! 	значения в промежутке [0,1], т. е. возможны неполностью морские ячейки.
! 	- 4 числа типа real, обозначающих две долготные и две широтные границы
! 	сферического прямоугольника. Долгота задаётся в пределах [0,360)1
! 	.
! 
! 	- 1 или 2 логических флага, определяющих тип сетки, на которой задан массив
! 	(T или V)
! 	• Выходной параметр: среднее значение входного массива по указанному
! 	прямоугольнику.
! 	• Осреднение должно учитывать площади ячеек и маску. Считать, что если центр
! 	ячейки
! 	попал в рассматриваемый прямоугольник, то вся ячейка попала в него.
	
	subroutine mean_S(dim2_arr_in, mask_2d_an, result_out,lon1, lon2, lat1, lat2,is_V, is_T,diag_missval)
	real(kind=main_kind),intent(in) :: dim2_arr_in(odom)
	real(kind=4),intent(in) 	:: mask_2d_an (odom)
	real, optional			:: lat1, lat2, lon1, lon2!specify rectangle to count average in
	integer				:: i1(2),i2(2),j1(2),j2(2)
	real				:: i_1,i_2,i_3,i_4,j_1,j_2
	logical, optional 		:: is_V, is_T  	!this defines which grid to use
	real (kind=8), allocatable	:: x1_temp(:,:)
	real (kind=8)			:: accum, total_area
	real(kind=main_kind), optional 	:: result_out
	real, optional			:: diag_missval
	real 				:: mean_S_missval
	!check given data
	if(.not.present(is_T).and..not.present(is_V))	call shared_error("subroutine mean_S. not specified grid type")
 	if(present(is_T).and.present(is_V))		call shared_error("subroutine mean_S. both grid types specified")
	
	
	allocate (x1_temp	(1:il,1:jl))
	!if lan1>lot2 set needed lats to { [lon1, 360]or[0, lon2] }
	i_1 = lon1
	i_4 = lon2
	if(lon1 .lt. lon2)then
	  i_2 = lon2
	  i_3 = lon1
	else
	  i_2 = 360
	  i_3 = 0
	end if
	!convert lat-lon to grid coords
	!in model start lat coordinate is set in ocn_list - x1_ssc_west_south
	if(present(is_T))x1_temp=modulo(x1_t,360.)
	if(present(is_V))x1_temp=modulo(x1_v,360.)
	
! 	  mean_S_missval = diag_missval
! 	else
! 	  mean_S_missval = missval
! 	end if
	accum = 0.			!variables to keep
	total_area = 0.			!sum of data and area
	do i=iwest (myrank),ieast (myrank)
	  do j=jsouth(myrank),jnorth(myrank)
	      if(present(is_T))then
		if( (((x1_temp(i,j).ge.i_1).and.(x1_temp(i,j).le.i_2)).or. &	!if lan1>lot2 needed lats are { [lon1, 360]or[0, lon2] }
		     ((x1_temp(i,j).ge.i_3).and.(x1_temp(i,j).le.i_4))).and. &	!if lan1<lot2 needed lats are { [lon1, lon2]or[lon1, lon2]=[lon1, lon2] }
		     ((x2_t(i,j)-lat1)*(x2_t(i,j)-lat2).le.0))then		!x2_t is between 
		  if(present(diag_missval))then
		    if(dim2_arr_in(i,j) .ne. diag_missval)then			!check if not missval
		      accum = accum + dim2_arr_in(i,j)  *  area_tbox(i,j)
		      total_area = total_area +   area_tbox(i,j)
		    end if
		  else
		    if(dim2_arr_in(i,j) .ne. missval)then			!check if not missval
		      accum = accum + dim2_arr_in(i,j)  *  area_tbox(i,j) * mask_2d_an(i,j)
		      total_area = total_area +   area_tbox(i,j) * mask_2d_an(i,j)
		    end if
		  end if
		  
		end if
	      end if
	      if(present(is_V))then
		if( (((x1_temp(i,j).ge.i_1).and.(x1_temp(i,j).le.i_2)).or. &
		     ((x1_temp(i,j).ge.i_3).and.(x1_temp(i,j).le.i_4))).and. &
		     ((x2_v(i,j)-lat1)*(x2_v(i,j)-lat2).le.0))then
		  if(present(diag_missval))then
		    if(dim2_arr_in(i,j) .ne. diag_missval)then			!check if not missval
		      accum = accum + dim2_arr_in(i,j)  *  area_vbox(i,j)
		      total_area = total_area +   area_vbox(i,j)
		    end if
		  else
		    if(dim2_arr_in(i,j) .ne. missval)then			!check if not missval
		      accum = accum + dim2_arr_in(i,j)  *  area_vbox(i,j) * mask_2d_an(i,j)
		      total_area = total_area +   area_vbox(i,j) * mask_2d_an(i,j)
		    end if
		  end if
		end if
	      end if
	  end do
	end do
	if(total_area>0)then
	  accum=accum/total_area
	else
	  accum=0
	  !write(*,*)"total_area=0"
	end if
	result_out = accum
	end subroutine mean_S
	
	subroutine read_dg_file(var_to_read,return_array_2d,return_array_3d,&
				is_3d,is_2d,input_dg_file)
    
	use netcdf
	use shared_module
	implicit none
        character (len = *) 				:: var_to_read
        real (kind=main_kind),optional 	:: return_array_2d(odom),return_array_3d(kl,odom)
	logical,optional 				:: is_3d,is_2d
        character (len = *),optional 			:: input_dg_file
	
        integer 		:: ncid, RecordDimID, nRecords
        integer,allocatable 	:: start(:), count(:)
    
        integer :: var_varid, record_number, Records
        integer :: i
    
        character(len = nf90_max_name) :: RecordDimName
        character (len = NAME_LEN) :: FILE_DG_NAME_ANALYSIS
        real (kind=main_kind), allocatable		:: temp_array_3d(:,:,:)
        
        !Process errors in array dimensions
	if(.not.present(is_2d).and..not.present(is_3d))call shared_error("subroutine read_dg_file. array dimensions")
	if(present(is_3d).and.present(is_2d))call shared_error("subroutine read_dg_file. array is 2d and 3d simultaeonusly")
        
        if(.not.present(input_dg_file))then	
	  FILE_DG_NAME_ANALYSIS = TRIM(run_solution_folder)//TRIM(cmp % name )//'_'//TRIM(I2S(cmp % i_size))//'x'//   &
			TRIM(I2S(cmp % j_size))//'x'//TRIM(I2S(cmp % k_size))//"_"//run_experiment//"_DG.nc"
	else
	  FILE_DG_NAME_ANALYSIS = TRIM(run_solution_folder)//input_dg_file
	end if
	record_number = INT(time_l*(24*60/ocn_steps_in_day)/ dg_period)		!what record in file to read
	
	!open DG file
	call check( nf90_open(FILE_DG_NAME_ANALYSIS, nf90_nowrite, ncid) )
	if(present(is_2d))then
	  allocate(count(3))
	  allocate(start(3))
	  count = (/ il-2,jl-2, 1 /)!il-2,jl-2 because there is no haloes in nc file
	  start = (/ 1, 1, 1 /)
	  call check( nf90_inq_varid(ncid, var_to_read, var_varid) )
	  start(3) = record_number
	  call check( nf90_get_var(ncid, var_varid, return_array_2d, start = start, count = count) )
	end if
	
	if(present(is_3d))then
	  allocate(count(4))
	  allocate(start(4))
	  allocate(temp_array_3d(odom,kl))
	  count = (/ il-2,jl-2, kl,1 /)!il-2,jl-2 because there is no haloes in nc file
	  start = (/ 1, 1, 1, 1 /)
	  call check( nf90_inq_varid(ncid, var_to_read, var_varid) )
	  start(4) = record_number
	  call check( nf90_get_var(ncid, var_varid, temp_array_3d, start = start, count = count) )
	  
	  do k = 1,kl
	  do i = iwest (myrank),ieast (myrank)
	  do j = jsouth(myrank),jnorth(myrank)
	    return_array_3d(k,i,j) = temp_array_3d (i,j,k)
	  enddo
	  enddo
	  enddo
	  
	  deallocate(count)
	  deallocate(start)
	  deallocate(temp_array_3d)
	end if
	  ! Close the file. This frees up any internal netCDF resources
	  ! associated with the file.
      
	  
	  call check( nf90_close(ncid) )
	
	end subroutine read_dg_file
	
! 	Написать процедуру, принимающую на входе
! 	- название диагностического файла (опционально),
! 	- 4 границы сферического прямоугольника,
! 	- количество num_fields осредняемых полей,
! 	- массив строк размера num_fields с названиями полей,
! 	- границы записей по времени (integer rec_start, rec_end),
! 	- значение пропуска данных (missing value)
! 	- название ascii-файла для записи результатов. Путь также от папки solution.
! 	Процедура должна создать ascii-файл и записать в него num_fields+1 колонок чисел –
! 	номер записи и соответствующие осреднённые (c учётом площади ячеек и пропуска
! 	данных) значения указанных полей. Считать, что все поля заданы на T-сетке и все
! 	ячейки полностью морские или сухопутные.
	subroutine offline_diagn_T( input_dg_file,lat1, lat2, lon1, lon2,&
				    num_fields,fields_names, rec_start, rec_end,&
				    missing_value,output_filename,mask_lvl,out_val)
        character (len = *),optional 	:: input_dg_file
	real, optional			:: lat1, lat2, lon1, lon2
	integer				:: num_fields,fields_dims(num_fields)
	integer				:: rec_start, rec_end,recs
	integer,optional		:: mask_lvl
	character (len =*),dimension(num_fields) :: fields_names
	real				:: missing_value
        character (len = *) 		:: output_filename
        real(8)		:: mean
	real (kind=main_kind), allocatable:: a_2d_tst	(:,:)
        integer:: out_unit=43
        integer :: rec_cur,rec_lvl
	real(8), optional			:: out_val
        
        rec_cur = INT(time_l*(24*60/ocn_steps_in_day)/ dg_period)
        
        !in begin of file write exp and var names
	if(rec_cur.eq.1)then	
	  open (unit=out_unit,file=output_filename,action="write",status="replace")
	  write(out_unit,"(A3) ",advance="no")run_experiment
	  do recs=1,num_fields
	    write(out_unit,"(A12) ",advance="no")TRIM(fields_names(recs))
	  end do
	  write(out_unit,*)" "
	  close (out_unit)
        end if
        
        if(present(mask_lvl))then
	  rec_lvl=mask_lvl
	else
	  rec_lvl=1
	end if
	
	allocate (a_2d_tst (odom))
        
        if((rec_cur-rec_end)*(rec_cur-rec_start).le.0)then!if current recorf in nc file is in borders
	  open (unit=out_unit,file=output_filename,access="append",action="write",decimal='comma')
	  write(out_unit,"(I3) ",advance="no")rec_cur
	  do recs=1,num_fields
	  if(present(input_dg_file))then
	      call read_dg_file(TRIM(fields_names(recs)),return_array_2d=a_2d_tst,is_2d=.TRUE.,input_dg_file=input_dg_file)
	    else
	      call read_dg_file(TRIM(fields_names(recs)),return_array_2d=a_2d_tst,is_2d=.TRUE.)
	    end if
		!calculate average by S (here on T-grid)
	    call mean_S(dim2_arr_in=a_2d_tst, mask_2d_an=ct3(rec_lvl,odom),result_out=mean,&
			is_T=.TRUE.,lat1 = lat1, lat2 = lat2, lon1 = lon1, lon2 = lon2,diag_missval=missing_value)
	    write(out_unit,"(e12.3) ",advance="no")mean
	  end do
	  write(out_unit,*)" "
	  close (out_unit)
	  !write result to dat file
        end if!end do
        
 	if(present(out_val))out_val=mean
        deallocate(a_2d_tst)
	end subroutine offline_diagn_T
	
	
	subroutine offline_diagn_V( input_dg_file,lat1, lat2, lon1, lon2,&
				    num_fields,fields_names, rec_start, rec_end,&
				    missing_value,output_filename,mask_lvl,out_val)
        character (len = *),optional 	:: input_dg_file
	real, optional			:: lat1, lat2, lon1, lon2
	integer				:: num_fields
	integer				:: rec_start, rec_end,recs
	integer,optional		:: mask_lvl
	character (len =*),dimension(2*num_fields) :: fields_names
	real				:: missing_value
        character (len = *) 		:: output_filename
        real(8)		:: mean_u,mean_v,mean_ke
	real (kind=main_kind), allocatable:: a_2d_U	(:,:),a_2d_V	(:,:)
        integer:: out_unit=43,i,j
        integer :: rec_cur,rec_lvl
        
	real(8), optional			:: out_val
        
	!write(*,*)iwest (myrank),ieast (myrank),jsouth(myrank),jnorth(myrank)
        
        rec_cur = INT(time_l*(24*60/ocn_steps_in_day)/ dg_period)
        
        if(rec_cur.eq.1)then	
	  open (unit=out_unit,file=output_filename,action="write",status="replace")
	  write(out_unit,"(A3) ",advance="no")run_experiment
	  do recs=1,num_fields
	    write(out_unit,"(A12) ",advance="no")TRIM(fields_names(2*recs-1))//'+'//TRIM(fields_names(2*recs))
	  end do
	  write(out_unit,*)" "
	  close (out_unit)
        end if
        
        if(present(mask_lvl))then
	  rec_lvl=mask_lvl
	else
	  rec_lvl=1
	end if
	
        
	if((rec_cur-rec_end)*(rec_cur-rec_start).le.0)then!do rec_count=rec_start,min(rec_end,rec_max)
        allocate (a_2d_U (odom))!	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1))
        allocate (a_2d_V (odom))!	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1))
	  open (unit=out_unit,file=output_filename,access="append",action="write",decimal='comma')
	  write(out_unit,"(I3) ",advance="no")rec_cur
	  do recs=1,num_fields
	  !U
	    if(present(input_dg_file))then
	      call read_dg_file(TRIM(fields_names(2*recs-1)),return_array_2d=a_2d_U,is_2d=.TRUE.,input_dg_file=input_dg_file)
	    else
	      call read_dg_file(TRIM(fields_names(2*recs-1)),return_array_2d=a_2d_U,is_2d=.TRUE.)
	    end if
	  !V
	  if(present(input_dg_file))then
	      call read_dg_file(TRIM(fields_names(2*recs)),return_array_2d=a_2d_V,is_2d=.TRUE.,input_dg_file=input_dg_file)
	    else
	      call read_dg_file(TRIM(fields_names(2*recs)),return_array_2d=a_2d_V,is_2d=.TRUE.)
	    end if
	    
	  !write(*,*)"2 = ",MAXVAL(a_2d_U),MINVAL(a_2d_U),MAXVAL(a_2d_V),MINVAL(a_2d_V)
	  
	  where(a_2d_U.ne.missing_value)a_2d_U=0.5*(a_2d_U**2+a_2d_V**2)
	  !write(*,*)"4 = ",MAXVAL(a_2d_U),MINVAL(a_2d_U)
	  call mean_S(dim2_arr_in=a_2d_U, mask_2d_an=cv(rec_lvl,odom),result_out=mean_ke,&
		      is_v=.TRUE.,lat1 = lat1, lat2 = lat2, lon1 = lon1, lon2 = lon2,&
		      diag_missval=missing_value)
	  write(out_unit,"(e12.3) ",advance="no")mean_ke
	  end do
	  write(out_unit,*)" "
	  close (out_unit)
	  !write result to dat file
 	if(present(out_val))out_val=mean_ke
        deallocate(a_2d_U)
        deallocate(a_2d_V)
        end if!end do
	end subroutine offline_diagn_V
	
	
	!This subroutine processes errors in NetCDF operations
	subroutine check(status)
		integer, intent ( in) :: status
	      
		if(status /= nf90_noerr) then 
		  print *, trim(nf90_strerror(status))
		  stop "Stopped"
		end if
	end subroutine check  
	
	end module o_analysis_utils_module
#endif
	