
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations. 
!	compute barotropic components
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
!--------------------------------------

!-------
!	compute sea surface level and full baroclinic currents at 'l+1'	
!	using the solution of barotropic shallow water equation
!-------

	subroutine o_d_make_ssl_f(ctime)
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_d_brtr_sngl_module
	implicit none
	integer ctime
!-------
!#1.	find sea surface level at 'l+1'
!-------

	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	flow_v_n_2d (i,j) = 									&
			(v_bt_a_f(	i-1	,j) + v_bt_a_f(	i	,j)) * 0.5 * 		&
			(dx1_nw_t(	i	,j) + dx1_ne_t(	i	,j))
	end do
	end do

	do j=jsouth(myrank)		,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	flow_u_e_2d (i,j) =									& 
     		(u_bt_a_f(    i  ,j  ) + u_bt_a_f(    i  ,j-1)  ) * 0.5 *			&
     		(dx2_en_t(    i  ,j  ) + dx2_es_t(    i  ,j  )  )
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
		flow_v_s_2d (i,j) = flow_v_n_2d (i,j-1)
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
		flow_u_w_2d (i,j) = flow_u_e_2d (i-1,j)
	end do
	end do

	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
		if(kt(i,j) > 0) &								 !!!!!!! correcting kt()>0 else ct3 on poles = 0 and diveide by zero
			div_2d(i,j)=(	(flow_u_e_2d(i,j)-flow_u_w_2d(i,j)) +		&
     					(flow_v_n_2d(i,j)-flow_v_s_2d(i,j))   ) *	&
     						area_tbox_v(i,j)/ct3(1,i,j)
	end do
	end do

	select case (ctime)
		case(1)
			do j = jsouth(myrank), jnorth(myrank)+1
			do i = iwest (myrank), ieast (myrank)+1
				if(kt(i,j)> 0) then
					ssl_f(i,j) = ssl_c(i,j)+(water_ssf(i,j)-div_2d(i,j))*htsec
				end if
			end do
			end do
		case(2)
			do j = jsouth(myrank), jnorth(myrank)+1
			do i = iwest (myrank), ieast (myrank)+1
				if(kt(i,j)> 0) then
					ssl_f(i,j) = dx3_t_l(2,i,j)-hzw(2)+(water_ssf(i,j)-div_2d(i,j))*htsec*ctime
				end if
			end do
			end do
	end select

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
		w_surf_c(i,j)=	div_2d(i,j)
	end do
	end do
	end

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine o_d_make_v_f
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
!-------
!	compute sea surface level and full baroclinic currents at 'l+1'	
!	using the solution of barotropic shallow water equation
!-------

!-------
!#2.2	compute depth average curents from baroclinic solution
!-------
	u_ksum_f = 0
	v_ksum_f = 0

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		u_ksum_f(i,j)=u_ksum_f(i,j)+u_f(k,i,j) * dx3_v_f(k+1,i,j)
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		v_ksum_f(i,j)=v_ksum_f(i,j)+v_f(k,i,j) * dx3_v_f(k+1,i,j)
	end do
	end do
	end do
	
	k=1
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if( k.le.kv(i,j)) then
		u_ksum_f(i,j)=u_ksum_f(i,j)/(zw(kv(i,j)+1)+ssl_v_f(i,j))
		v_ksum_f(i,j)=v_ksum_f(i,j)/(zw(kv(i,j)+1)+ssl_v_f(i,j))
	end if
	end do
	end do

!-------
!#2.3	compute pure baroclinic solution
!-------
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		u_f(k,i,j)=u_f(k,i,j)-u_ksum_f(i,j)
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		v_f(k,i,j)=v_f(k,i,j)-v_ksum_f(i,j)
	end do
	end do
	end do

!-------
!#2.4	total velocity = baroclinic + barotropic velocity
!-------
	k=1
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if( k.le.kv(i,j)) then
		u_ksum_f(i,j) = u_bt_a_f(i,j) / (zw(kv(i,j)+1)+ssl_v_f(i,j) )
		v_ksum_f(i,j) = v_bt_a_f(i,j) / (zw(kv(i,j)+1)+ssl_v_f(i,j) )
	end if
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		u_f(k,i,j)=u_f(k,i,j) + u_ksum_f(i,j)
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		v_f(k,i,j)=v_f(k,i,j) + v_ksum_f(i,j)
	end do
	end do
	end do

	return
	end

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine o_d_brtr_w_surface(ctime)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!-------
!	vertical velocity at sea surface
!	to be used in sub. W at the next step
!-------

	use o_grid_module
	use o_basic_module
	use o_d_brcl_module
	implicit none
	integer ctime

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	if(kt(i,j).gt.0 ) then
	w_surf_c(i,j)=	- (ssl_f(i,j)-ssl_c(i,j))/(htsec*ctime)	&
     			+ water_ssf(i,j)
	end if
	end do
	end do

	return
	end
	
	
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine o_d_make_dx3___f
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module

!-------
!	compute vertical grid steps for T-column
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	k = 1
!	dx3_t_f(k+1,i,j) = hzw(k+1)
	dx3_t_f(k+1,i,j) = hzw(k+1) + ssl_f(i,j)
!z	do k = 2, kl
!z	dx3_t_f(k+1,i,j) = hzw(k+1)
!z	end do
	end do
	end do
!-------
!	compute sea surface level at v-points
!-------
!	if(mod(time_l,minday)==0) then
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	if(kv(i,j).gt.0 ) then
	ssl_v_f (i,j)=0.25*						&
     			( (ssl_f(i+1,j+1)+ssl_f(i,j))			&
     			 +(ssl_f(i  ,j+1)+ssl_f(i+1,j))	)
	end if
	end do
	end do
!	end if
!-------
!	compute vertical grid steps for V-column
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	k = 1
!	dx3_v_f(k+1,i,j) = hzw(k+1)
	dx3_v_f(k+1,i,j) = hzw(k+1) + ssl_v_f(i,j)
!z	do k = 2, kl
!z	dx3_v_f(k+1,i,j) = hzw(k+1)
!z	end do
	end do
	end do


	return
	end

	
