
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations. 
!	module for barotropic components
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru 
!--------------------------------------

	
!~~~~~~
	module o_d_brtr_sngl_module
!~~~~~~

!-------
!	sweq, single ear algorithm
!-------

	implicit none
	save

	real (kind=8), allocatable:: u_bt_c		(:,:)
	real (kind=8), allocatable:: v_bt_c		(:,:)
	real (kind=8), allocatable:: h_bt_c		(:,:)

	real (kind=8), allocatable:: u_bt_f		(:,:)
	real (kind=8), allocatable:: v_bt_f		(:,:)
	real (kind=8), allocatable:: h_bt_f		(:,:)

!z	real (kind=8), allocatable:: u_bt_a_c		(:,:)
!z	real (kind=8), allocatable:: v_bt_a_c		(:,:)
	real (kind=8), allocatable:: h_bt_a_c		(:,:)
	real (kind=8), allocatable:: h_bt_a_l		(:,:)
	
!z	real (kind=8), allocatable:: h_bt_a_f		(:,:)

	real (kind=8), allocatable:: flux_u_n_2d	(:,:)
	real (kind=8), allocatable:: flux_u_e_2d	(:,:)
	real (kind=8), allocatable:: flux_v_n_2d	(:,:)
	real (kind=8), allocatable:: flux_v_e_2d	(:,:)
	real (kind=8), allocatable:: u_lapl_2d		(:,:)
	real (kind=8), allocatable:: v_lapl_2d		(:,:)
	real (kind=8), allocatable:: ru_2d		(:,:)
	real (kind=8), allocatable:: rv_2d		(:,:)
	real gamma_sweq
	
	end module o_d_brtr_sngl_module


!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine allocate_o_d_brtr_sngl_module
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!-------
!	sweq, single ear algorithm
!-------
	use o_d_brtr_sngl_module
	use o_grid_module
	use o_par_module
	
	namelist /OCN_SWEQ/ gamma_sweq	
	
	!-----reading and validating of namelist parameters-----
	open(8,file = namelist_path,err=1)
	read(8, nml = OCN_SWEQ, err=1)
	close(8)
		
	if(gamma_sweq.lt.0. .or. gamma_sweq.gt.0.25) print *,'WARNING: possibly unstable time staggerring in SWEQ'
	!-------------------------------------------
	
	
	allocate (u_bt_c	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (v_bt_c	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (h_bt_c	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )

	allocate (u_bt_f	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (v_bt_f	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (h_bt_f	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )

!z	allocate (u_bt_a_c	(   	iwest (myrank)-1:ieast (myrank)+1,
!z     &					jsouth(myrank)-1:jnorth(myrank)+1) )
!z	allocate (v_bt_a_c	(   	iwest (myrank)-1:ieast (myrank)+1,
!z     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (h_bt_a_c	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (h_bt_a_l	(   	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )     
	u_bt_c	 = 0.
	v_bt_c	 = 0.
	h_bt_c	 = 0.

	u_bt_f	 = 0.
	v_bt_f	 = 0.
	h_bt_f	 = 0.

!z	u_bt_a_c = 0.
!z	v_bt_a_c = 0.
	h_bt_a_c = 0.
	h_bt_a_l = 0.
	
!	allocate (h_bt_a_f	(   	iwest (myrank)-1:ieast (myrank)+1,
!     &					jsouth(myrank)-1:jnorth(myrank)+1) )
!	h_bt_a_f = 0.

	allocate (flux_u_n_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (flux_u_e_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (flux_v_n_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (flux_v_e_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (u_lapl_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (v_lapl_2d	(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	flux_u_n_2d	= 0.
	flux_u_e_2d	= 0.
	flux_v_n_2d	= 0.
	flux_v_e_2d	= 0.
	u_lapl_2d	=0.
	v_lapl_2d	=0.

	allocate (ru_2d		(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (rv_2d		(	iwest (myrank)-1:ieast (myrank)+1,
     &					jsouth(myrank)-1:jnorth(myrank)+1) )
	ru_2d	=0.
	rv_2d	=0.

	return
1	call ocn_error('Cannot read SWEQ nameleist! Stop.', 'LCL')
	end
