
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations. 
!	shallow water equations
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Dyakonov, Gleb, IO RAS, Russia, glebruler@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Killworth, P. D., D. Stainforth, D. J. Webb, and S. Paterson , 1991. 
!		The development of a free surface Bryan-Cox-Semtner model. 
!		J. Phys. Oceanogr.,21, 1333–1348.
!	Griffies S., 2004. Fundamentals of Ocean Climate Models. Princeton University Press. 528 pp.
!--------------------------------------


	subroutine o_d_brtr_sweq_sngl

!-------
!	integrate shallow water equations by predictor-corrector scheme
!	at interval from 'l' till 'l+1' with 'tau' time-step
!	initiate shallow_water_equations: u_bt_a_f=v_bt_a_f=0
!-------
	use o_grid_module
	use o_basic_module
	use o_d_brcl_module
	use o_d_brtr_sngl_module
	use o_par_module
	implicit none
	
	integer	p		!barotropic step counter
	real(kind=8)	c	!denominator for averaging in the end of cycle
	real(kind=8) taucor,gridsteps_frame
!-------
!#1	
!-------
!-------
!#1.3	continue initiate functions
!-------
	u_bt_c  = u_bt_a_f
	v_bt_c  = v_bt_a_f
	
	if(time_l==1) then 
		h_bt_c=ssl_c(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	else
		h_bt_c=h_bt_a_f
	end if
	h_bt_f=h_bt_c
	h_bt_a_f = h_bt_c
	!h_bt_a_c = h_bt_a_f		!saving averaged barotropic level in the beginning of SWEQ to change it into h_bt_a_l after o_d_make_ssl_f
	
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		gh_mask(i,j)=cv(1,i,j)*gravit*(zw(kv(i,j)+1) + 0.25*((h_bt_c  (i,j)+h_bt_c  (i+1,j+1))+(h_bt_c  (i,j+1)+h_bt_c  (i+1,j))         ))
	end do
	end do

!-------
!        **********  ***      ***   **********   *****        ***********
!       ********** *   *      **   ********** *   ***          ******** *
!       ***       **    *    **    ***       **   ***          ***      *
!       ***              *  **     ***            ***          ***
!       ***               ***      ***            ***          ******
!       ***               **       ***            ***          ***
!       ***       **      **       ***       **   ***      **  ***      *
!       ********** *      **       ********** *   ********* *  ******** *
!        **********      ****       **********   ***********  ***********
!-------

	do p=1,sweq_pl*2
		!--------------------------
		!predictor step for barotropic level (eq. 11.10 of MOM5 Elements)-----
		!--------------------------
		if(gamma_sweq.gt.1.e-7) then
			!if (time_l_in_run .le. 100) print *,'-----Running SWEQ predictor-----'
			do j=jsouth(myrank)-1	,jnorth(myrank)+1
			do i=iwest (myrank)-1	,ieast (myrank)+1
				u_bt_c(i,j) = cv(1,i,j)*u_bt_c(i,j)
				v_bt_c(i,j) = cv(1,i,j)*v_bt_c(i,j)
			end do
			end do

			do j=jsouth(myrank)-1	,jnorth(myrank)+1
			do i=iwest (myrank)	,ieast (myrank)+1
				flow_v_n_2d (i,j) = 							&
					(  v_bt_c(    i-1,j  ) +   v_bt_c(    i  ,j  )   ) * 0.5 *	&
					(dx1_nw_t(    i  ,j  ) + dx1_ne_t(    i  ,j  )   )
			end do
			end do

			do j=jsouth(myrank)	,jnorth(myrank)+1
			do i=iwest (myrank)-1	,ieast (myrank)+1
				flow_u_e_2d (i,j) = 							&
					(  u_bt_c(    i  ,j  ) +   u_bt_c(    i  ,j-1)  ) * 0.5 *	&
					(dx2_en_t(    i  ,j  ) + dx2_es_t(    i  ,j  )  )
			end do
			end do

			do j=jsouth(myrank)	,jnorth(myrank)+1
			do i=iwest (myrank)	,ieast (myrank)+1
				div_2d(i,j)=(	(flow_u_e_2d(i,j)-flow_u_e_2d(i-1,j)) +				&
						(flow_v_n_2d(i,j)-flow_v_n_2d(i,j-1))   ) * 			&
						area_tbox_v(i,j)*ct3v(1,i,j)
			end do
			end do

			do j=jsouth(myrank)	,jnorth(myrank)+1
			do i=iwest (myrank)	,ieast (myrank)+1
				h_bt_f(i,j) =	h_bt_c(i,j) + (	water_ssf(i,j) - div_2d(i,j) ) * tau * gamma_sweq
			end do
			end do

#if CMF_VERSION == 2    
			call comp_halo_update_T8(DIM2_t = h_bt_f, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
			call HaloUpdateMaster % update(h_bt_f, update_width = 1, grid_type = 'T')
#endif  		
		end if
		!-----------------------------------
		!barotropic momentum equations
		!-----------------------------------
		
		!-----predicted (noncorrected) elevation at eastern and northern walls of v-box-----
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
			func_h_e_vbox (i,j)=								&
				(h_bt_f(i+1,j+1)*dx2_es_v(i,j)+h_bt_f(i+1,j)*dx2_en_v(i,j))/		&
				(                dx2_es_v(i,j)+              dx2_en_v(i,j))
		end do
		end do
		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			func_h_n_vbox (i,j)=								&
				(h_bt_f(i,j+1)*dx1_ne_v(i,j)+h_bt_f(i+1,j+1)*dx1_nw_v(i,j))/		&
				(              dx1_ne_v(i,j)+                dx1_nw_v(i,j))
		end do
		end do

		!elevation gradients
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			grad_h_x (i,j) = 								&
				(func_h_e_vbox(i  ,j  ) - func_h_e_vbox(i-1,j  )) /			&
				(dx1_ne_t(i,j) + dx1_nw_t(i+1,j))
		end do
		end do
		do j=jsouth(myrank)	,jnorth(myrank)                      
		do i=iwest (myrank)	,ieast (myrank)
			grad_h_y (i,j) = 								&
				(func_h_n_vbox(i  ,j  ) - func_h_n_vbox(i  ,j-1)) /			&
				(dx2_es_t(i,j+1) + dx2_en_t(i,j))
		end do
		end do

		!-----2D-laplace momentum diffusion term
		if(am_lapl_2d.gt.1.e-7) then
			if (time_l_in_run .le. 1) print *,'-----Running SWEQ Laplace-----'
			k = 1
			do j=jsouth(myrank)	,jnorth(myrank)
			do i=iwest (myrank)-1	,ieast (myrank)
				gridsteps_frame=(dx2_en_v(i,j)+dx2_es_v(i,j)) / (dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
								
				!No slip boundary condition
				flux_u_e_2d(i,j) = (								&
					u_bt_c (i+1,j)*(2.-cv(k,i  ,j) ) - 					&
					u_bt_c (i  ,j)*(2.-cv(k,i+1,j) )   ) *					&
					gridsteps_frame
				flux_v_e_2d(i,j) = (								&
					v_bt_c (i+1,j)*(2.-cv(k,i  ,j) ) - 					&
					v_bt_c (i  ,j)*(2.-cv(k,i+1,j) )   ) *					&
					gridsteps_frame
			end do
			end do

			do j=jsouth(myrank)-1	,jnorth(myrank)
			do i=iwest (myrank)	,ieast (myrank)
				gridsteps_frame = (dx1_nw_v(i,j)+dx1_ne_v(i,j)) / (dx2_en_t(i,j+1)+dx2_es_t(i,j+1))
				
				!No slip boundary condition
				flux_u_n_2d(i,j) = (								&
					u_bt_c (i,j+1)*(2.-cv(k,i  ,j) )-					&
					u_bt_c (i,j  )*(2.-cv(k,i,j+1) ) ) *					&
					gridsteps_frame
				flux_v_n_2d(i,j) = (								&
					v_bt_c (i,j+1)*(2.-cv(k,i  ,j) )-					&
					v_bt_c (i,j  )*(2.-cv(k,i,j+1) ) ) *					&
					gridsteps_frame
			end do
			end do
	
			do j=jsouth(myrank)	,jnorth(myrank)
			do i=iwest (myrank)	,ieast (myrank)
				if(1.le.kv(i,j)) then	
					u_lapl_2d(i,j) = am_lapl_2d * area_vbox_v(i,j) * (			&
						(flux_u_e_2d(i  ,j  ) - flux_u_e_2d(i-1,j  )) +		&
						(flux_u_n_2d(i  ,j  ) - flux_u_n_2d(i  ,j-1))   )
			
					v_lapl_2d(i,j) = am_lapl_2d * area_vbox_v(i,j) * (			&
						(flux_v_e_2d(i  ,j  ) - flux_v_e_2d(i-1,j  )) +		&
						(flux_v_n_2d(i  ,j  ) - flux_v_n_2d(i  ,j-1))   )
				end if
			end do
			end do
			
		end if

		!-----final formula with explicit coriolis-----
!		do j=jsouth(myrank)	,jnorth(myrank)
!		do i=iwest (myrank)	,ieast (myrank)
!		u_bt_f(i,j) = (	-gh_mask(i,j)*grad_h_x(i,j)
!     &			+rx(i,j)
!     &			+coriolis(i,j)*v_bt_c(i,j)
!     &			+u_lapl_2d(i,j)			)
!		end do
!		end do
!		do j=jsouth(myrank)	,jnorth(myrank)
!		do i=iwest (myrank)	,ieast (myrank)
!		v_bt_f(i,j) = (	-gh_mask(i,j)*grad_h_y(i,j)
!     &			+ry(i,j)
!     &			-coriolis(i,j)*u_bt_c(i,j)
!     &			+v_lapl_2d(i,j)			)
!		end do
!		end do
!		u_bt_f = tau*u_bt_f + u_bt_c
!		v_bt_f = tau*v_bt_f + v_bt_c


		!-----final formula for barotropic "velocities" with implicit coriolis-----
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			ru_2d(i,j) = u_bt_c(i,j) + 				&
				tau *	(-gh_mask(i,j)*grad_h_x(i,j)		&
					+rx(i,j)				&
					+u_lapl_2d(i,j)			)
		
			rv_2d(i,j) = v_bt_c(i,j) +				&
				tau *	(-gh_mask(i,j)*grad_h_y(i,j)		&
					+ry(i,j)				&
					+v_lapl_2d(i,j)			)
		end do
		end do
		
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			taucor=tau*coriolis(i,j)
			
			u_bt_f(i,j) =	(  ru_2d(i,j) 				&
				+ taucor * rv_2d(i,j) ) /	&
				(1. + taucor*taucor)
			v_bt_f(i,j) = (	  rv_2d(i,j) 				&
				- taucor * ru_2d(i,j) ) /		&
				(1. + taucor*taucor)
		end do
		end do

		do j=jsouth(myrank)-1	,jnorth(myrank)+1
		do i=iwest (myrank)-1	,ieast (myrank)+1
			u_bt_f(i,j) = cv(1,i,j)*u_bt_f(i,j)
			v_bt_f(i,j) = cv(1,i,j)*v_bt_f(i,j)
		end do
		end do
		
#if CMF_VERSION == 2    
        call comp_halo_update_V8(DIM2_u = u_bt_f, DIM2_v = v_bt_f,update_width = 1, change_sign_on_bipolar = -1)
#elif CMF_VERSION == 3
        call HaloUpdateMaster % update(u_bt_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
        call HaloUpdateMaster % update(v_bt_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
#endif		
		
		!--------------------------
		!corrector step for barotropic level (eq. 11.12 of MOM5 Elements)-----
		!--------------------------
		do j=jsouth(myrank)-1	,jnorth(myrank)+1
		do i=iwest (myrank)	,ieast (myrank)+1
			flow_v_n_2d (i,j) = 							&
     				(  v_bt_f(    i-1,j  ) +   v_bt_f(    i  ,j  )   ) * 0.5 *	&
     				(dx1_nw_t(    i  ,j  ) + dx1_ne_t(    i  ,j  )   )
		end do
		end do

		do j=jsouth(myrank)	,jnorth(myrank)+1
		do i=iwest (myrank)-1	,ieast (myrank)+1
			flow_u_e_2d (i,j) = 							&
     				(  u_bt_f(    i  ,j  ) +   u_bt_f(    i  ,j-1)  ) * 0.5 *	&
     				(dx2_en_t(    i  ,j  ) + dx2_es_t(    i  ,j  )  )
		end do
		end do

		do j=jsouth(myrank)	,jnorth(myrank)+1
		do i=iwest (myrank)	,ieast (myrank)+1
			div_2d(i,j)=(	(flow_u_e_2d(i,j)-flow_u_e_2d(i-1,j)) +				&
					(flow_v_n_2d(i,j)-flow_v_n_2d(i,j-1))   ) * 			&
					area_tbox_v(i,j)*ct3v(1,i,j)
		end do
		end do

		do j=jsouth(myrank)	,jnorth(myrank)+1
		do i=iwest (myrank)	,ieast (myrank)+1
			h_bt_f(i,j) =	h_bt_c(i,j) + (	water_ssf(i,j) - div_2d(i,j) ) * tau
		end do
		end do
		
#if CMF_VERSION == 2    
        call comp_halo_update_T8(DIM2_t = h_bt_f, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
        call HaloUpdateMaster % update(h_bt_f, update_width = 1, grid_type = 'T')
#endif  		
		!--------------------------
		!accumulating means and reinitializing for next step of the cycle
		!--------------------------
		u_bt_a_f = u_bt_a_f + u_bt_f	
		v_bt_a_f = v_bt_a_f + v_bt_f	
		h_bt_a_f = h_bt_a_f + h_bt_f	

		u_bt_c = u_bt_f
		v_bt_c = v_bt_f
		h_bt_c = h_bt_f

	end do		!do p=0,pl
	

!-------
!#3.	average solution
!-------

	c=1.d0/dble(2*sweq_pl+1)
	u_bt_a_f = c*u_bt_a_f
	v_bt_a_f = c*v_bt_a_f
	h_bt_a_f = c*h_bt_a_f

	!call greenwich_test(u_bt_a_f,p,'ubt')

	return                                                           
	end                                                              

	subroutine greenwich_test(u_array,p,text)
	use o_grid_module
	use o_par_module
	implicit none
	
	integer p
	real(8) u_array	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8) store1	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8) store2	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	character(3) text
	
	j=jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		if(j==jl-1) then
			store1(i,j-1)=u_array(i,j)
		end if
	end do
		
#if CMF_VERSION == 2    
        call comp_halo_update_V8(DIM2_u = store1, DIM2_v = store2,update_width = 1, change_sign_on_bipolar = -1)
#elif CMF_VERSION == 3
        call HaloUpdateMaster % update(store1, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
        call HaloUpdateMaster % update(store2, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
#endif	
		
	j=jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		if(j==jl-1 .and. i.le.il/2) then
			if(abs(u_array(i,j)-store1(i,j+1)).gt.1.e-7) then
				print *,'BAROTROPIC ERROR ',p,text,i,u_array(i,j),store1(i,j+1)
				!call shared_error('barotropic stop')
			end if
		end if
	end do
	end
	
!*************************************************************************************************************************

	subroutine greenwich_symmetrize(u_array,v_array,change)
	use o_grid_module
	use o_par_module
	implicit none
	
	real(8) u_array	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8) v_array	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8) store1	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(8) store2	(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	integer change
	
	j=jnorth(myrank)
	if(j==jl-1) then
		do i=iwest (myrank)-1	,ieast (myrank)+1
			store1(i,j-1)=u_array(i,j)
			store2(i,j-1)=v_array(i,j)
		end do
	end if
		
#if CMF_VERSION == 2    
        call comp_halo_update_V8(DIM2_u = store1, DIM2_v = store2,update_width = 1, change_sign_on_bipolar = change)
#elif CMF_VERSION == 3
        call HaloUpdateMaster % update(store1, update_width = 1, grid_type = 'V', change_sign_on_bipolar = (change==-1))
        call HaloUpdateMaster % update(store2, update_width = 1, grid_type = 'V', change_sign_on_bipolar = (change==-1))
#endif	
		
	j=jnorth(myrank)
	if(j==jl-1) then
		do i=iwest (myrank)-1	,ieast (myrank)+1
			u_array(i,j)=(u_array(i,j)+store1(i,j+1))*0.5
			v_array(i,j)=(v_array(i,j)+store2(i,j+1))*0.5
		end do
	end if
	end
	
	
	