
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations.
!	Governs the integration of dynamical equations
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!--------------------------------------

!===============================================
! Governs the integration of dynamical equations
! Computes u, v, ssl at 'l+1' timestep
!-------
!	E U L E R  S C H E M E: 
!			*_c - Functions at (l) time step
!			*_f - Functions at (l+1) time step
!-------
!	L E A P F R O G   T I M E S T E P:
!			*_l - Functions at (l-1) time step
!			*_c - Functions at (l) time step
!			*_f - Functions at (l+1) time step
!===============================================

	subroutine o_d_main
	use o_par_module
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	implicit none

	!---summing sea surface water fluxes---
	call o_d_sswf

	!---calculating biharmonic filter terms---
	if(mixh_m_mode .ge. 1) call o_d_mixh_m_bh

	!---advection terms---
	call o_d_am_cd

	!---summing all rhs terms, calculating new barotropic and baroclinic velocities,---
	!---also calculating new ssl,dx3 and surface w (for l+1 time step)---
	if(MOD((time_l-1), lmats) == 0) then
		call o_d_motor (1.,u_c,v_c,u_c,v_c,u_c,v_c,dx3_v_c)
	else
		call o_d_motor (2.,u_l,v_l,u_c,v_c,u_l,v_l,dx3_v_l)
	end if

	!---summing barotropic and baroclinic velocities---
	call o_d_make_v_f
	end
	