
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations.
!	Horizontal Mixing Terms for Momentum equations
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, T. B. Grankina, and R. A. Ibraev, 2016. Modeling the Water Circulation 
!		in the North Atlantic in the Scope of the CORE-II Experiment. 
!		Izvestiya, Atmospheric and Oceanic Physics, Vol. 52, No. 4, pp. 365–375
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

#include "o_defines.f90"
!--------------------------------------
!      	compute Horizontal Mixing Terms for Momentum equations
!	Laplase:	free slip b.c.
!--------------------------------------
	subroutine o_d_mixh_m_lapl (u,v)

	use o_grid_module
	use o_basic_module
	use o_d_brcl_module
	use o_par_module
	implicit none

	real (kind=main_kind) u(kl, iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	real (kind=main_kind) v(kl, iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	real (kind=main_kind) gridsteps_frame	!registry for saving large gridsteps combinations ("crocodiles") 
	
	if(am_lapl_mode == 2 .or. am_bh_mode == 2) call o_d_smagorinsky(u,v)	!Compute Smagorinsky viscosities
!-------
!		5-point Laplase, at solid boundaries - no friction
!-------
!-------
!		flux at the eastern walls of v-box
!-------
	flux_u_e=0.
	flux_v_e=0.
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=1,kv(i,j)
		gridsteps_frame=cv(k,i+1,j) *				&
     			(dx2_en_v(i,j)+dx2_es_v(i,j))/			&
     			(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))*		&
			(dx3_t_c(k+1,i+1,j+1)+dx3_t_c(k+1,i+1,j))*0.5
		
		flux_u_e(k,i,j) = (u(k,i+1,j) - u(k,i  ,j)) * gridsteps_frame
		flux_v_e(k,i,j) = (v(k,i+1,j) - v(k,i  ,j)) * gridsteps_frame
	end do
	end do
	end do

!-------
!		flux at the northern walls of v-box
!-------
	flux_u_n=0.
	flux_v_n=0.
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		gridsteps_frame = cv(k,i,j+1) *						&
     			(dx1_nw_v(i,j)+dx1_ne_v(i,j)     ) /				&
     			(dx2_en_t(i,j+1)+dx2_es_t(i,j+1))*				&
			(dx3_t_c(k+1,i+1,j+1)+dx3_t_c(k+1,i,j+1))*0.5
	
		flux_u_n(k,i,j) = (u(k,i,j+1) - u(k,i,j  ) ) * gridsteps_frame
		flux_v_n(k,i,j) = (v(k,i,j+1) - v(k,i,j  ) ) * gridsteps_frame
	end do
	end do
	end do

!-------
!		5-point Laplase, at solid boundaries 'v'=0,
!-------
!-------
!		flux at the eastern walls of v-box
!-------
!	do j=jsouth(myrank)	,jnorth(myrank)
!	do i=iwest (myrank)-1	,ieast (myrank)
!	do k=1,kl
!	flux_u_e(k,i,j) = 1. * (
!     &			u(k,i+1,j)*(2.-cv(k,i  ,j) ) - 
!     &			u(k,i  ,j)*(2.-cv(k,i+1,j) )   ) *
!     &			(dx2_en_v(i,j)+dx2_es_v(i,j)       ) / 
!     &				(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
!	end do
!	end do
!	end do
 
!	do j=jsouth(myrank)	,jnorth(myrank)
!	do i=iwest (myrank)-1	,ieast (myrank)
!	do k=1,kl
!	flux_v_e(k,i,j) = 1. * (
!     &			v(k,i+1,j)*(2.-cv(k,i  ,j) ) - 
!     &			v(k,i  ,j)*(2.-cv(k,i+1,j) )   ) *
!     &			(dx2_en_v(i,j)+dx2_es_v(i,j)       ) /
!     &				(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
!	end do
!	end do
!	end do

!-------
!		flux at the northern walls of v-box
!-------
!	do j=jsouth(myrank)-1	,jnorth(myrank)
!	do i=iwest (myrank)	,ieast (myrank)
!	do k=1,kl
!	flux_u_n(k,i,j) = 1. * (
!     &			u(k,i,j+1)*(2.-cv(k,i  ,j) )-
!     &			u(k,i,j  )*(2.-cv(k,i,j+1) ) ) *
!     &			(dx1_nw_v(i,j)+dx1_ne_v(i,j)     ) /
!     &				(dx2_en_t(i,j+1)+dx2_es_t(i,j+1))
!	end do
!	end do
!	end do

!	do j=jsouth(myrank)-1	,jnorth(myrank)
!	do i=iwest (myrank)	,ieast (myrank)
!	do k=1,kl
!	flux_v_n(k,i,j) = 1. * (
!     &			v(k,i,j+1)*(2.-cv(k,i  ,j) )-
!     &			v(k,i,j  )*(2.-cv(k,i,j+1) ) ) *
!     &			(dx1_nw_v(i,j)+dx1_ne_v(i,j)     ) /
!     &				(dx2_en_t(i,j+1)+dx2_es_t(i,j+1))
!	end do
!	end do
!	end do

!-------
!	pure Laplase without viscosity coefficient - to be used in biharmonics
!-------
	if(mixh_m_mode.gt.0) then
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kv(i,j)
			u_laplace_pure(k,i,j) = area_vbox_v(i,j) * (			&
				(flux_u_e(k,i  ,j  ) - flux_u_e(k,i-1,j  )) +			&
				(flux_u_n(k,i  ,j  ) - flux_u_n(k,i  ,j-1))   )
			v_laplace_pure(k,i,j) = area_vbox_v(i,j) * (			&
				(flux_v_e(k,i  ,j  ) - flux_v_e(k,i-1,j  )) +			&
				(flux_v_n(k,i  ,j  ) - flux_v_n(k,i  ,j-1))   )
		end do
		end do
		end do
	
#if CMF_VERSION == 2    
		call comp_halo_update_V(DIM3_u = u_laplace_pure, DIM3_v = v_laplace_pure, update_width = 1, change_sign_on_bipolar = -1)
#elif CMF_VERSION == 3
		call HaloUpdateMaster % update(u_laplace_pure, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
		call HaloUpdateMaster % update(v_laplace_pure, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
#endif
	end if

!-------
!		Laplase with viscosity coefficient - to be used here
!-------
	if(mixh_m_mode.lt.2) then	!mixh_m_mode=2 means only biharmonics, no Laplace friction
		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
		do k=1,kv(i,j)
			flux_u_e(k,i,j)=flux_u_e(k,i,j)*am_lapl_e(k,i,j)
			flux_v_e(k,i,j)=flux_v_e(k,i,j)*am_lapl_e(k,i,j)
			flux_u_n(k,i,j)=flux_u_n(k,i,j)*am_lapl_n(k,i,j)
			flux_v_n(k,i,j)=flux_v_n(k,i,j)*am_lapl_n(k,i,j)
		end do
		end do
		end do
		
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kv(i,j)
			u_laplace(k,i,j) = area_vbox_v(i,j) * (			&
				(flux_u_e(k,i  ,j  ) - flux_u_e(k,i-1,j  )) +			&
				(flux_u_n(k,i  ,j  ) - flux_u_n(k,i  ,j-1))   )

			v_laplace(k,i,j) = area_vbox_v(i,j) * (			&
				(flux_v_e(k,i  ,j  ) - flux_v_e(k,i-1,j  )) +			&
				(flux_v_n(k,i  ,j  ) - flux_v_n(k,i  ,j-1))   )
		end do
		end do
		end do
	end if
end

!--------------------------------------
!	compute friction terms for momentum equation by bi-harmonic operator
!		with laplace terms computed before. At solid boundaries - no biharmonic flux
!--------------------------------------

	subroutine o_d_mixh_m_bh

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	implicit none
	real (kind=main_kind) gridsteps_frame	!registry for saving large gridsteps combinations ("crocodiles") 
	
!-------
!		flux at the eastern walls of v-box
!-------
	flux_u_e=0.
	flux_v_e=0.
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=1,kv(i,j)
		gridsteps_frame = am_bh_e(k,i,j)*cv(k,i+1,j)*				&
     			(dx2_en_v(i,j)+dx2_es_v(i,j)       ) /				&
     			(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j)) *				&
     			(dx3_t_c(k+1,i+1,j+1)+dx3_t_c(k+1,i+1,j))*0.5
		
		flux_u_e(k,i,j) = (							&
     			u_laplace_pure(k,i+1,j)/dx3_v_f(k+1,i+1,j) -			&
     			u_laplace_pure(k,i  ,j)/dx3_v_f(k+1,i  ,j)   ) * gridsteps_frame

		flux_v_e(k,i,j) = (							&
     			v_laplace_pure(k,i+1,j)/dx3_v_f(k+1,i+1,j) -			&
     			v_laplace_pure(k,i  ,j)/dx3_v_f(k+1,i  ,j)   ) * gridsteps_frame
	end do
	end do
	end do

!-------
!		flux at the northern walls of v-box
!-------
	flux_u_n=0.
	flux_v_n=0.
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		gridsteps_frame = am_bh_n(k,i,j)*cv(k,i,j+1)*				&
     			(dx1_nw_v(i,j)+dx1_ne_v(i,j)     ) /				&
     			(dx2_en_t(i,j+1)+dx2_es_t(i,j+1)) * 				&
			(dx3_t_c(k+1,i+1,j+1)+dx3_t_c(k+1,i,j+1))*0.5     			
		
		flux_u_n(k,i,j) = (							&
     			u_laplace_pure(k,i,j+1)/dx3_v_f(k+1,i,j+1) -			&
     			u_laplace_pure(k,i,j  )/dx3_v_f(k+1,i,j  )   ) * gridsteps_frame
	
		flux_v_n(k,i,j) = (							&
     			v_laplace_pure(k,i,j+1)/dx3_v_f(k+1,i,j+1) -			&
     			v_laplace_pure(k,i,j  )/dx3_v_f(k+1,i,j  )   ) * gridsteps_frame
	end do
	end do
	end do

!-------
!		Laplase (Laplace) = 
!		bi-harmonic operator
!-------
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		u_laplace2(k,i,j) = area_vbox_v(i,j) * (					&
     		(flux_u_e(k,i  ,j  ) - flux_u_e(k,i-1,j  )) +					&
     		(flux_u_n(k,i  ,j  ) - flux_u_n(k,i  ,j-1))   )

		v_laplace2(k,i,j) = area_vbox_v(i,j) * (					&
     		(flux_v_e(k,i  ,j  ) - flux_v_e(k,i-1,j  )) +					&
     		(flux_v_n(k,i  ,j  ) - flux_v_n(k,i  ,j-1))   )
	end do
	end do
	end do
end
	
!--------------------------------------
!	compute Smagorinsky viscosities (rememeber to multiply by c_smag_lapl and c_smag_bh in o_d_motor.f90)
!--------------------------------------
	subroutine o_d_smagorinsky(u,v)
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_basic_module
	use o_par_module

	implicit none
	real (kind=main_kind) u(kl, iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	real (kind=main_kind) v(kl, iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	real delta,deformation,delta_equator
	!real (kind=8) peak1,peak2,peak3,peak4
	
	delta_equator=x2_ssc_step*pi180*radius
	
	!coefficients from (Griffies,Hallberg,2000, eq.12,13)
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
		!delta=min(dx2_es_v(i,j)+dx2_en_v(i,j),dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
		delta=sqrt((dx2_es_v(i,j)+dx2_en_v(i,j)) * (dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j)) )
		func_u_e(1,i,j)=(delta/pi)**2
		func_u_e(2,i,j)=0.125*(delta*delta/pi)**2
		func_u_e(3,i,j)=am_lapl*delta/delta_equator		!background viscosity
		func_u_e(4,i,j)=am_bh*(delta/delta_equator)**3		!background biharmonic viscosity
	end do
	end do
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		!delta=min(dx2_es_t(i,j+1)+dx2_en_t(i,j+1),dx1_nw_v(i,j)+dx1_ne_v(i,j))
		delta=sqrt(  (dx2_es_t(i,j+1)+dx2_en_t(i,j+1))*(dx1_nw_v(i,j)+dx1_ne_v(i,j))  )
		func_u_n(1,i,j)=(delta/pi)**2
		func_u_n(2,i,j)=0.125*(delta*delta/pi)**2
		func_u_n(3,i,j)=am_lapl*delta/delta_equator		!background viscosity
		func_u_n(4,i,j)=am_bh*(delta/delta_equator)**3		!background biharmonic viscosity
	end do
	end do
		
	!Tension and shearing at eastern wall of v-box
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=1,kv(i,j)
		!tension (Dt)
		flux_u_e(k,i,j)=(u(k,i+1,j)-u(k,i,j))/(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))-	&
				(v(k,i,j+1)+v(k,i+1,j+1)-v(k,i,j-1)-v(k,i+1,j-1))*0.5/(dx2_es_v(i,j+1)+dx2_en_v(i,j)+dx2_es_v(i,j)+dx2_en_v(i,j-1))
		
		!shearing (Ds)
		flux_v_e(k,i,j)=(u(k,i,j+1)+u(k,i+1,j+1)-u(k,i,j-1)-u(k,i+1,j-1))*0.5/(dx2_es_v(i,j+1)+dx2_en_v(i,j)+dx2_es_v(i,j)+dx2_en_v(i,j-1))+	&
				(v(k,i+1,j)-v(k,i,j))/(dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
		
		deformation=sqrt(flux_u_e(k,i,j)*flux_u_e(k,i,j) + flux_v_e(k,i,j)*flux_v_e(k,i,j))
				
		!Smagorinsky Laplace and bh coefficients
		if(am_lapl_mode==2) am_lapl_e(k,i,j)=max(func_u_e(1,i,j)*deformation*c_smag_lapl*c_smag_lapl,func_u_e(3,i,j))
		if(am_bh_mode==2)   am_bh_e(k,i,j)=min(-func_u_e(2,i,j)*deformation*c_smag_bh*c_smag_bh,func_u_e(4,i,j))
	end do
	end do
	end do
	
	!Tension and shearing at northern wall of v-box
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kv(i,j)
		!tension (Dt)
		flux_u_n(k,i,j)=(u(k,i+1,j)+u(k,i+1,j+1)-u(k,i-1,j)-u(k,i-1,j+1))*0.5/(dx1_ne_v(i-1,j)+dx1_ne_v(i,j)+dx1_nw_v(i,j)+dx1_nw_v(i+1,j))-	&
				(v(k,i,j+1)-v(k,i,j))/(dx2_en_t(i,j+1)+dx2_es_t(i,j+1))
				
		!shearing (Ds)
		flux_v_n(k,i,j)=(u(k,i,j+1)-u(k,i,j))/(dx2_en_t(i,j+1)+dx2_es_t(i,j+1))+	&
				(v(k,i+1,j)+v(k,i+1,j+1)-v(k,i-1,j)-v(k,i-1,j+1))*0.5/(dx1_ne_v(i-1,j)+dx1_ne_v(i,j)+dx1_nw_v(i,j)+dx1_nw_v(i+1,j))
		
		deformation=sqrt(flux_u_n(k,i,j)*flux_u_n(k,i,j) + flux_v_n(k,i,j)*flux_v_n(k,i,j))
		
		!Smagorinsky Laplace and bh coefficients
		if(am_lapl_mode==2) am_lapl_n(k,i,j)=max(func_u_n(1,i,j)*deformation*c_smag_lapl*c_smag_lapl,func_u_n(3,i,j))
		if(am_bh_mode==2)   am_bh_n(k,i,j)=min(-func_u_n(2,i,j)*deformation*c_smag_bh*c_smag_bh,func_u_n(4,i,j))
	end do
	end do
	end do
	
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=1,kl
		if(abs(cv(k,i,j)*cv(k,i+1,j)).lt.0.1) am_bh_e(k,i,j)=0. !ah_bh_e(k,i,j,1)*td_bh_order
		if(abs(cv(k,i,j)*cv(k,i,j+1)).lt.0.1) am_bh_n(k,i,j)=0. !ah_bh_n(k,i,j,1)*td_bh_order
	end do
	end do
	end do
	
!	!Empirical addition to damp w_surf noise
!#if CMF_VERSION == 2    
!        call comp_halo_update_T(DIM2_t = w_surf_c, update_width = 1, change_sign_on_bipolar = 1)
!#elif CMF_VERSION == 3
!	call HaloUpdateMaster % update(w_surf_c, update_width = 1, grid_type = 'T')
!#endif 	
!	do j=jsouth(myrank)	,jnorth(myrank)
!	do i=iwest (myrank)	,ieast (myrank)
!		peak1=ceiling(ct3(1,i  ,j+1)*ct3(1,i  ,j)*ct3(1,i  ,j-1)-0.001)	*(w_surf_c(i  ,j+1)-w_surf_c(i  ,j))*(w_surf_c(i  ,j)-w_surf_c(i  ,j-1))
!		peak2=ceiling(ct3(1,i+1,j+1)*ct3(1,i+1,j)*ct3(1,i+1,j-1)-0.001)	*(w_surf_c(i+1,j+1)-w_surf_c(i+1,j))*(w_surf_c(i+1,j)-w_surf_c(i+1,j-1))
!		peak3=ceiling(ct3(1,i+1,j+1)*ct3(1,i,j+1)*ct3(1,i-1,j+1)-0.001)	*(w_surf_c(i+1,j+1)-w_surf_c(i,j+1))*(w_surf_c(i,j+1)-w_surf_c(i-1,j+1))
!		peak4=ceiling(ct3(1,i+1,j  )*ct3(1,i  ,j)*ct3(1,i-1,j  )-0.001)	*(w_surf_c(i+1,j  )-w_surf_c(i,j  ))*(w_surf_c(i,j  )-w_surf_c(i-1,j  ))
!		
!		if(sign(1.,w_surf_c(i  ,j+1)*w_surf_c(i  ,j)).gt.-0.9 .or. sign(1.,w_surf_c(i  ,j+1)*w_surf_c(i  ,j-1)).lt.0.9) peak1=0.
!		if(sign(1.,w_surf_c(i+1,j+1)*w_surf_c(i+1,j)).gt.-0.9 .or. sign(1.,w_surf_c(i+1,j+1)*w_surf_c(i+1,j-1)).lt.0.9) peak2=0.
!		if(sign(1.,w_surf_c(i+1,j+1)*w_surf_c(i,j+1)).gt.-0.9 .or. sign(1.,w_surf_c(i+1,j+1)*w_surf_c(i-1,j+1)).lt.0.9) peak3=0.
!		if(sign(1.,w_surf_c(i+1,j  )*w_surf_c(i,j  )).gt.-0.9 .or. sign(1.,w_surf_c(i+1,j  )*w_surf_c(i-1,j  )).lt.0.9) peak4=0.
!		
!		peak1=-min(0.,peak1,peak2,peak3,peak4)/1.e-13
!		peak1= min(peak1,2.)
!		
!		am_lapl_n(1:kv(i,j),i,j)=max(am_lapl_n(1:kv(i,j),i,j),peak1*func_u_n(3,i,j))
!		am_lapl_e(1:kv(i,j),i,j)=max(am_lapl_e(1:kv(i,j),i,j),peak1*func_u_e(3,i,j))
!	end do
!	end do
		
	!-------------------------Smoothing-maximizing viscosities--------------------
	if(am_lapl_mode==2) then	
#if CMF_VERSION == 2    
		call comp_halo_update_V4(DIM3_u = am_lapl_e, DIM3_v = am_lapl_n, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
		call HaloUpdateMaster % update(am_lapl_e, update_width = 1, grid_type = 'V')
		call HaloUpdateMaster % update(am_lapl_n, update_width = 1, grid_type = 'V')
#endif	
	
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
		do k=1,kv(i,j)
			flux_u_e(k,i,j)=maxval(am_lapl_e(k,i:i+1,j-1:j+1)*cv(k,i:i+1,j-1:j+1))
		end do
		end do
		end do
		
		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kv(i,j)
			flux_u_n(k,i,j)=maxval(am_lapl_n(k,i-1:i+1,j:j+1)*cv(k,i-1:i+1,j:j+1))
		end do
		end do
		end do
		
		do j=jsouth(myrank)-1	,jnorth(myrank)
		if(j.le.jl-3) then
			do i=iwest (myrank)-1	,ieast (myrank)
				am_lapl_e(:,i,j)=flux_u_e(:,i,j)
				am_lapl_n(:,i,j)=flux_u_n(:,i,j)
			end do
		end if
		end do
	end if
	
	if(am_bh_mode==2) then	
#if CMF_VERSION == 2    
		call comp_halo_update_V4(DIM3_u = am_bh_e, DIM3_v = am_bh_n, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
		call HaloUpdateMaster % update(am_bh_e, update_width = 1, grid_type = 'V')
		call HaloUpdateMaster % update(am_bh_n, update_width = 1, grid_type = 'V')
#endif	
	
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
		do k=1,kv(i,j)
			flux_v_e(k,i,j)=minval(am_bh_e(k,i:i+1,j-1:j+1))
		end do
		end do
		end do
		
		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kv(i,j)
			flux_v_n(k,i,j)=minval(am_bh_n(k,i-1:i+1,j:j+1))
		end do
		end do
		end do
		
		do j=jsouth(myrank)-1	,jnorth(myrank)
		if(j.le.jl-3) then
			do i=iwest (myrank)-1	,ieast (myrank)
				am_bh_e(:,i,j)=flux_v_e(:,i,j)
				am_bh_n(:,i,j)=flux_v_n(:,i,j)
			end do
		end if
		end do
	end if
	end

!--------------------------------------
!	initialize time-constant viscosities
!--------------------------------------
	subroutine o_d_mixh_ini
	use o_grid_module
	use o_d_brcl_module

	implicit none
	real delta,delta_equator	!gridsteps - in current point and on the equator
	
	!initialize constant global viscosities
	am_lapl_e=am_lapl
	am_lapl_n=am_lapl
	am_bh_e=am_bh
	am_bh_n=am_bh
		
	!make viscosities space-varying according to gridsteps
	delta_equator=x2_ssc_step*pi180*radius
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
		!delta=min(dx2_es_v(i,j)+dx2_en_v(i,j),dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j))
		delta=sqrt((dx2_es_v(i,j)+dx2_en_v(i,j)) * (dx1_nw_t(i+1,j)+dx1_ne_t(i+1,j)) )
		if(am_lapl_mode == 1) am_lapl_e(:,i,j)=am_lapl*delta/delta_equator		!viscosity
		if(am_bh_mode == 1) am_bh_e(:,i,j)=am_bh*(delta/delta_equator)**3		!biharmonic viscosity
	end do
	end do
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		!delta=min(dx2_es_t(i,j+1)+dx2_en_t(i,j+1),dx1_nw_v(i,j)+dx1_ne_v(i,j))
		delta=sqrt(  (dx2_es_t(i,j+1)+dx2_en_t(i,j+1))*(dx1_nw_v(i,j)+dx1_ne_v(i,j))  )
		if(am_lapl_mode == 1) am_lapl_n(:,i,j)=am_lapl*delta/delta_equator		!viscosity
		if(am_bh_mode == 1) am_bh_n(:,i,j)=am_bh*(delta/delta_equator)**3		!biharmonic viscosity
	end do
	end do
	end	
	