#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations.
!	transformation of variables between 
!	tripolar system of coordinates <==> spherical system of coordinates
!
! authors: 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Murray R.J. 1996. 
!		Explicit Generation of Orthogonal Grids for Ocean Models,
!		J. Comp. Physics, 126, 251-273.
!	Ibrayev R.A., Khabeev R.N., Ushakov K.V., 2012. 
!		Eddy-resolving 1/10 Model of the World Ocean. 
!		Izvestiya, Atmospheric and Oceanic Physics, 2012, Vol. 48, No. 1, pp. 37–46.
!--------------------------------------



	subroutine ssc_tsc_rotate_t(ssc_u,ssc_v,tsc_u,tsc_v,i1,i2,j1,j2)!interpolation to model tsc-grid must be done before calling this subroutine  
	use o_grid_module
	use o_bsc_module
	implicit none
	integer i1,i2,j1,j2											!i1,i2,j1,j2 - bounds of arrays
	real(4) tsc_u(i1:i2,j1:j2),tsc_v(i1:i2,j1:j2)
	real(4) ssc_u(i1:i2,j1:j2),ssc_v(i1:i2,j1:j2)					!tsc_u and ssc_u are both on model grid, but ssc_u is not rotated to bipolar coordinate directions
	real(8) spb,slb,cpb,clb,bsc_u,bsc_v,rot_coef1,rot_coef2,rot_coef0

	do j=j1,j2														!Rotating the velocity vector on the 3-polar grid. Cycle over 3-polar grid nodes.
	do i=max(2,i1),i2  !min(i2,il-1)
		if(j.lt.jl_ssc) then
			tsc_u(i,j)=ssc_u(i,j)
			tsc_v(i,j)=ssc_v(i,j)
		else
			if(i.le.il/2) then
				i_bsc=il_bsc-j+jl_ssc
				j_bsc=i-1
			else
				i_bsc=j-jl_ssc+1
				j_bsc=il_ssc-i+1
			end if	
		
			slb= dsin (x1_bsc_t(i_bsc)*3.1415926536d0/180.d0)
			clb= dcos (x1_bsc_t(i_bsc)*3.1415926536d0/180.d0)
			spb= dsin (x2_bsc_t(j_bsc)*3.1415926536d0/180.d0)
			cpb= dcos (x2_bsc_t(j_bsc)*3.1415926536d0/180.d0)
	
			rot_coef0=1.d0/dsqrt(1.d0-clb**2*cpb**2)
			rot_coef1=clb*spb*rot_coef0
			rot_coef2=slb*rot_coef0
			
			bsc_u=-rot_coef1*ssc_u(i,j)-rot_coef2*ssc_v(i,j)
			bsc_v= rot_coef2*ssc_u(i,j)-rot_coef1*ssc_v(i,j)

			if(i.le.il/2) then
				tsc_u(i,j)=bsc_v
				tsc_v(i,j)=-bsc_u
			else
				tsc_u(i,j)=-bsc_v
				tsc_v(i,j)=bsc_u
			end if	
		end if
	end do
	end do
	end

!----------------------------------------------------------------------------------------------------------------------------------------------
	subroutine tsc_ssc_rotate_v(tsc_u,tsc_v,ssc_u,ssc_v,i1,i2,j1,j2) !interpolation to plot ssc-grid must be done after calling this subroutine  
	use o_grid_module
	use o_bsc_module
	implicit none
!	integer i1,i2,j1,j2,i,j											!i1,i2,j1,j2 - bounds of arrays
	integer i1,i2,j1,j2			!i1,i2,j1,j2 - bounds of arrays
	real(kind=collector_kind) tsc_u(i1:i2,j1:j2),tsc_v(i1:i2,j1:j2)
	real(kind=collector_kind) ssc_u(i1:i2,j1:j2),ssc_v(i1:i2,j1:j2)					!tsc_u and ssc_u are both on model grid, but ssc_u is not rotated to bipolar coordinate directions
	real(8) cpb,clb,spb,slb,bsc_u,bsc_v

	do j=j1,min(j2,jl-1)											!Rotating the velocity vector on the 3-polar grid. Cycle over 3-polar grid nodes.
	do i=max(2,i1),min(i2,il-1)
		if(j.lt.jl_ssc) then
			ssc_u(i,j)=tsc_u(i,j)	
			ssc_v(i,j)=tsc_v(i,j)	
		else
			if(i.le.il/2) then
				i_bsc=(il_bsc-1)-(j-jl_ssc)
				j_bsc=i-1
				bsc_u=-tsc_v(i,j)
				bsc_v=tsc_u(i,j)
			else
				i_bsc=j-jl_ssc+1
				j_bsc=il_ssc-i
				bsc_u=tsc_v(i,j)
				bsc_v=-tsc_u(i,j)
			end if	
			
			slb=dsin(x1_bsc_v(i_bsc)*3.1415926536d0/180.d0)  !slb - sine lambda bipolar
			clb=dcos(x1_bsc_v(i_bsc)*3.1415926536d0/180.d0)
			spb=dsin(x2_bsc_v(j_bsc)*3.1415926536d0/180.d0)
			cpb=dcos(x2_bsc_v(j_bsc)*3.1415926536d0/180.d0)

			ssc_v (i,j)=-slb/dsqrt(1.d0-cpb**2*clb**2)*bsc_u-clb/dsqrt(1.d0-cpb**2*clb**2)*bsc_v*spb
			ssc_u (i,j)=-clb/dsqrt(1.d0-cpb**2*clb**2)*bsc_u*spb+ slb/dsqrt(1.d0-cpb**2*clb**2)*bsc_v
		end if
	end do
	end do
	end



