
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!
!	ocean dynamic equations.
!	total water flux at the sea surface
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

	subroutine o_d_sswf

!--------------------------------------
!	compute water sea surface flux = P+M-E
!--------------------------------------
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_tf_module
	use ice_module
	use o_par_module

	implicit none
	real(8) local_integral,local_area,global_average_sswf(1)
	
!-------
!	ssh change due to precipitation and evaporation (under ice water does not evaporate)
!-------
	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
	if(kt(i,j).gt.0) then
	water_ssf(i,j) =  dble(prec(i,j)) 							&
			+                  dble(ice_ocn_water(i,j)	/densit)		&
			- (1.-aice(i,j)) * dble(evap(i,j)		/densit)
	end if
	end do
	end do
	
!-----------------------------------------------------------------------------------
!	calculating and subtracting average water surface flux in standalone experiments	
!----------------------------------------------------------------------------------	
	if(atm_model_name=="ATM_NCAR") then
		local_integral=	sum( water_ssf(iwe:iea,jso:jno)*ct3(1,iwe:iea,jso:jno)*area_tbox(iwe:iea,jso:jno) )
		local_area=	sum( 				ct3(1,iwe:iea,jso:jno)*area_tbox(iwe:iea,jso:jno) )

#if CMF_VERSION == 2    
		call comp_p % reduce_itg(local_area, [real(kind=8) :: local_integral], global_average_sswf)
#elif CMF_VERSION == 3
		global_average_sswf = UtilsAllReduce(local_area, local_integral, ocn_comm)
#endif 	
	
!-----------------------------------------------------------------------------------
!	subtracting average water surface flux	
!----------------------------------------------------------------------------------	
		do j = jsouth(myrank), jnorth(myrank)+1
		do i = iwest (myrank), ieast (myrank)+1
			if(kt(i,j).gt.0) water_ssf(i,j) = water_ssf(i,j) - global_average_sswf(1)
		end do
		end do
	end if
	end 
