#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!
!	ocean dynamic equations.
!	part of momentum equation - solver for implicit Coriolis formulation
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!--------------------------------------


!~~~~~~
	subroutine o_d_vel_lp(Arix,Brix,Crix,alfa1,Ru0,Rv0,u0,v0,N,kl)
!~~~~~~

!--------------------------------------
!     Solve the equation for V:	(A*B**(-1)*A-B)*v = A*B**(-1)*rv-ru
!--------------------------------------

	!use o_grid_module
	IMPLICIT REAL(kind=main_kind)(A-H, O-Z), INTEGER(I-N)
	double Precision A(kl,kl),A2(kl,kl),alfa,sum,ud(kl),vd(kl)
	double Precision ABD(7,kl)
        dimension ipvt(kl)
        dimension u0(kl),v0(kl),Ru0(kl),Rv0(kl)
        dimension Arix(kl),Brix(kl),Crix(kl)
!-------
! 0.	Define marix A(kl,kl)
!-------
	alfa=dble(alfa1)	
	do j=1,kl
	do i=1,kl
	A(i,j)=0.0d0
	end do
	end do
	do i=2,N
		A(i,i-1)=-dble(Arix(i)	)
	end do
	do i=1,N
		A(i,i  )= dble(Crix(i)	)
	end do
	do i=1,N-1
		A(i,i+1)=-dble(Brix(i)	)
	end do
!-------
! 1. Solve the equation for V:	(A*B**(-1)*A-B)*v = A*B**(-1)*rv-ru
!-------
!-------
! 1.1.	Multiple A2=A*A
!-------
	do j=1,N
	do i=1,N
	sum=0.0d0
		do k=1,N
		sum=sum+A(i,k)*A(k,j)
		end do
	A2(i,j)=sum
	end do
	end do
!-------
! 1.2.	Compute alfa**(-1)*A2 - E*alfa
!-------
	do j=1,N
	do i=1,N
	A2(i,j)=(1.d0/alfa)*A2(i,j)
	end do
	end do

	do i=1,N
	A2(i,i)=A2(i,i)+alfa
	end do
!-------
! 1.3.	Compute the right side vector
!-------
	do i=1,N
	vd(i)=0.0d0
		do j=1,N
		vd(i)=vd(i)+A(i,j)*dble(rv0(j))
		end do
	end do

	do i=1,N
	vd(i)=(1./alfa)*vd(i)-dble(ru0(i))
	end do
!-------
! 1.4.	The problem for v:	A2*v = rv
!-------
!-------
! 1.4.1.	The band matrix ofr Subr. DGBFA
!-------
	ML=2
	MU=2
	LDA=2*ml+mu+1
                   M = ML + MU + 1
                   DO 20 J = 1, N
                      I1 = MAX0(1, J-MU)
                      I2 = MIN0(N, J+ML)
                      DO 10 I = I1, I2
                         K = I - J + M
                         ABD(K,J) = A2(I,J)
10    CONTINUE
20    CONTINUE
!-------
! 1.4.2.	Factorise the 'A2' matrix
!-------
	Call dGBFA(ABD,LDA,N,ML,MU,IPVT,INFO)
	if(info.ne.0) write(*,*) 'Subr.SGBFA tells',info
!-------
! 1.4.3.	Solve the equation for v
!-------
	job=0
	Call dGBSL(ABD,LDA,N,ML,MU,IPVT,vd,JOB)
!-------
! 2.	Compute the solution for 'u':	u = -B**(-1)*A*v+B**(-1)*rv
!-------
	do i=1,N
	u0(i)=0.
		do j=1,N
		u0(i)=u0(i)+A(i,j)*vd(j)
		end do
	end do

	do i=1,N
	u0(i)=(1./alfa1)*(-u0(i)+rv0(i))
	end do

	do i=1,N
	v0(i)=vd(i)
	end do
	return
	end
