
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean dynamic equations.
!	continuty equation
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

!--------------------------------------
!		compute vertical velocity from continuty equation
!		between k & k+1 levels for t-points at 'j+1' latitude
!		and v-points at 'j' latitude
!		integration begins from sea surface (z=0)
!--------------------------------------
	
	subroutine o_d_w
	use o_grid_module
	use o_basic_module
	use o_d_brcl_module
	use test_ocean_module
	implicit none

!z	flow_u_n = 0.	!array used for divergence

!-------
!		Compute vertical velocity at z=0
!-------
	k=1
	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	if(kt(i,j).gt.0) w(k,i,j)=w_surf_c(i,j)
	end do
	end do
	
!-------
!		flow through the walls of t-box
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)
	!flow_v_n (k,i,j) =													&
     	!	(     v_c(k  ,i-1,j  ) +      v_c(k  ,i  ,j  )   ) * 0.5 *	&
     	!	( dx3_v_c(k+1,i-1,j  ) +  dx3_v_c(k+1,i  ,j  )   ) * 0.5 *	&
     	!	(dx1_nw_t(    i  ,j  ) + dx1_ne_t(    i  ,j  )   )
		flow_v_n(k,i,j) =									&
			(v_c(k,i-1,j)*dx3_v_c(k+1,i-1,j) + v_c(k,i,j)*dx3_v_c(k+1,i,j)   ) * 0.5 *	&
			(dx1_nw_t(i,j) + dx1_ne_t(i,j))
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1								
	do i=iwest (myrank)-1	,ieast (myrank)+1							
	do k=1, kt(i,j)
	!flow_u_e (k,i,j) =													&
     	!	(     u_c(k  ,i  ,j  ) +      u_c(k  ,i  ,j-1)  ) * 0.5 *	&
     	!	( dx3_v_c(k+1,i  ,j  ) +  dx3_v_c(k+1,i  ,j-1)  ) * 0.5 *	&
     	!	(dx2_en_t(    i  ,j  ) + dx2_es_t(    i  ,j  )  )
		flow_u_e (k,i,j) =									&
			(u_c(k,i,j)*dx3_v_c(k+1,i,j) + u_c(k,i,j-1)*dx3_v_c(k+1,i,j-1)) * 0.5 *	&
			(dx2_en_t(i,j) + dx2_es_t(i,j))
	end do	!do k=1,kl
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)
	flow_v_s (k,i,j) = flow_v_n (k,i,j-1)
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)
	flow_u_w (k,i,j) = flow_u_e (k,i-1,j)
	end do	!do k=1,kl
	end do
	end do

!-------
!		Compute divergence
!-------
	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1,kl
	flow_u_n(k,i,j) = area_tbox_v(i,j) * (                  &
     		(flow_u_e(k,i  ,j  ) - flow_u_w(k,i,j)) +       &
     		(flow_v_n(k,i  ,j  ) - flow_v_s(k,i,j))  )
	end do
	end do
	end do

!-------
!		Compute vertical velocity in water column, k.gt.1
!-------
!-------
!	integrate from top to bottom
!-------
	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
		if (kt(i,j).ge.1) then

			do k=1, kt(i,j)-1
				w(k+1,i,j)=( w(k,i,j)*ct3(k,i,j) - flow_u_n(k,i,j) )/ct3(k+1,i,j)
			end do

			k = kt(i,j)
			w(k+1,i,j)=( w(k,i,j)*ct3(k,i,j) - flow_u_n(k,i,j) )/ct3(k,i,j)

		end if
	end do
	end do

!z	do j=jsouth(myrank)	,jnorth(myrank)
!z	do i=iwest (myrank)	,ieast (myrank)
!z		temp1(i,j)=w(kt(i,j)+1,i,j)
!z	end do
!z	end do

	return
	end	

