!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	diagnostical parameters of solution 
!	Module defining 2D space-averaging techniques
!	This is a system file.
!
! authors: 
!	2020.07 / Ushakov Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------
module o_diagn_horizon_module
	
	CONTAINS
	
	function o_diagn_horizon_mean(field,level,over_ice) result(mean)
		use o_grid_module
		use ice_module
		use o_basic_module
		use o_par_module
		implicit none
		real(4) field(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
		real(8) dare,area_local,field_local,averages(1),mean
		integer level
		logical over_ice
		
		area_local=0.
		field_local=0.
		
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			dare=area_tbox(i,j)*ct3(level,i,j)
			area_local=area_local+dare
			if(over_ice) then !include or not field over ice
				field_local=field_local+dare*field(i,j)
			else
				field_local=field_local+dare*(1.-aice(i,j))*field(i,j)
			end if
		end do
		end do
		
#if CMF_VERSION == 2
		call comp_p % reduce_itg(area_local, [real(kind=8) :: field_local], averages)
#elif CMF_VERSION == 3
		averages(1) = UtilsAllReduce(area_local, field_local,  ocn_comm)
#endif	
		mean=averages(1)
	end
end module o_diagn_horizon_module