
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	diagnostical parameters of solution 
!
! authors: 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
!--------------------------------------


!---------------------------------------------------------------------------	
!	Take u,v arrays and calculate various horizontal means
!---------------------------------------------------------------------------
#if CMF_VERSION == 2
subroutine o_UV_horizons(u_array,v_array)
	use o_grid_module
	use o_par_module
	use o_basic_module
	implicit none

	real(8) ke_local_integral(kl+1),ke_global_mean(kl+1),volume_local_v(kl+1),temp(1)
	real(8),dimension(kl,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) :: u_array,v_array,gridbox_volume_v
	
	do i=iwe,iea
	do j=jso,jno
	do k=1,kl
		gridbox_volume_v(k,i,j)= area_vbox(i,j) * cv(k,i,j) * dx3_v_c(k+1,i,j)
	end do
	end do
	end do
		
	!Horizontal means
	do k=1,kl
		volume_local_v(k)=sum(	gridbox_volume_v(k,iwe:iea,jso:jno) )
		ke_local_integral(k)=sum(  (u_array(k,iwe:iea,jso:jno)**2 + v_array(k,iwe:iea,jso:jno)**2)  *  gridbox_volume_v(k,iwe:iea,jso:jno)   )    
		call comp_p % reduce_itg(volume_local_v(k), [real(kind=8) :: ke_local_integral(k)],temp )
		ke_global_mean(k)=temp(1)
	end do
		
	!Full W.O. mean. "Level kl+1" means "average over whole il*jl*kl"
	volume_local_v(kl+1)=sum(volume_local_v(1:kl))
	ke_local_integral(kl+1)=sum(  ke_local_integral(1:kl)   )    
	call comp_p % reduce_itg(volume_local_v(kl+1), [real(kind=8) :: ke_local_integral(kl+1)],temp )
	ke_global_mean(kl+1)=temp(1)

	if(myrank==1) write(100,'(51f8.3)') sngl(time_l)/minday,ke_global_mean
	
end
#endif


!----------------------------------------------------------	
!	Growing annual means for T,S,U,V every time step	(update: not necessary annual, the period is specified by ave_finishing_period_days)
!----------------------------------------------------------
!	subroutine o_annual_means_write
!	use o_core_diagnostics_module
!	use o_grid_module
!	use o_basic_module
!	use o_tf_module
!	use ice_module
!	use o_td_module
!	use o_par_module
!
!	implicit none
!	real(8) :: temp=1.d0
!	real(8) moc_column(0:kl)
!	real(8) local_integral,local_area,global_annualmean_heatflux(1)
!	
!	i1=iwest(myrank)
!	i2=ieast(myrank)
!	j1=jsouth(myrank)
!	j2=jnorth(myrank)
!		
!	!computing horizontal means from time means
!	if(mod(time_l,ave_finishing_period_days*minday)==1) then
!		
!		!to test conservativity we substitute t_annualmean by t_c
!		!t_annualmean=t_c
!
!		t_annualmean=0.
!		u_annualmean=0.
!		v_annualmean=0.
!		moc_core2_annualmean=0.
!		aice_marchmean=0.
!		hice_marchmean=0.
!		mld_marchmean=0.
!		counter_marchmean=0
!		heatflux_annualmean=0.
!	end if
!	
!	!summing instantaneous fields
!	if(mod(time_l,ave_adding_period_hours*minday/24) == 1 ) then
!		t_annualmean=t_annualmean+t_c
!		u_annualmean=u_annualmean+u_c
!		v_annualmean=v_annualmean+v_c
!		heatflux_annualmean=heatflux_annualmean+ssr*(1.-albedo_ocean)+str+sns+ltn
!		
!		do j=jsouth(myrank)-1,jnorth(myrank)+1
!		do i=iwest (myrank)-1,ieast (myrank)+1
!			moc_column=0.
!			do k=1,kl
!				moc_column(k)=moc_column(k-1)+cv(k,i,j)*v_c(k,i,j)*dx3_v_c(k+1,i,j)*(dx1_nw_v(i,j)+dx1_ne_v(i,j))
!				moc_core2_annualmean(k,i,j)=moc_core2_annualmean(k,i,j)+moc_column(k)
!			end do
!			
!		end do
!		end do
!#if CMF_VERSION == 2  
!		call shared_timer
!		if(time_month==3) then
!			aice_marchmean=aice_marchmean+aice(:,:)
!			hice_marchmean=hice_marchmean+hice(:,:)
!			mld_marchmean=mld_marchmean+mld
!			counter_marchmean=counter_marchmean+1
!			!if(time_day_in_year==74 .and. myrank==1) print *,'----Vesna!-----'
!		end if
!#endif
!		end if
!	
!	!dividing the sums to make the time means. After this ocean step they may be written to CP.
!	if(mod(time_l,ave_finishing_period_days*minday)==0) then
!		t_annualmean=t_annualmean/(  (ave_finishing_period_days*24)/ave_adding_period_hours  )
!		u_annualmean=u_annualmean/(  (ave_finishing_period_days*24)/ave_adding_period_hours  )
!		v_annualmean=v_annualmean/(  (ave_finishing_period_days*24)/ave_adding_period_hours  )
!		heatflux_annualmean=heatflux_annualmean/(  (ave_finishing_period_days*24)/ave_adding_period_hours  )
!		moc_core2_annualmean=moc_core2_annualmean/(  (ave_finishing_period_days*24)/ave_adding_period_hours  )
!		aice_marchmean=aice_marchmean/counter_marchmean
!		hice_marchmean=hice_marchmean/counter_marchmean
!		mld_marchmean=mld_marchmean/counter_marchmean
!		!call o_horizons(t_annualmean,u_annualmean,v_annualmean) !calculating tem_global_mean(1:kl+1)  !temporary not used
!#if CMF_VERSION == 2  		
!		!-----heatflux global and annual mean------
!		local_integral=	sum( heatflux_annualmean(i1:i2,j1:j2)*	ct3(1,i1:i2,j1:j2)*area_tbox(i1:i2,j1:j2) )
!		local_area=	sum( 					ct3(1,i1:i2,j1:j2)*area_tbox(i1:i2,j1:j2) )
!		call comp_p % reduce_itg(local_area, [real(kind=8) :: local_integral], global_annualmean_heatflux)
!		if(myrank==1) print *,'Global annualmean ocean heat flux = ',sngl(global_annualmean_heatflux(1)),' Wt' 
!#endif
!	end if
!	
!	
!	!-----------saving integral heatflux----------------
!	local_integral=	sum( (ssr(i1:i2,j1:j2)*(1.-albedo_ocean)+str(i1:i2,j1:j2)+sns(i1:i2,j1:j2)+ltn(i1:i2,j1:j2)) * 	ct3(1,i1:i2,j1:j2)*area_tbox(i1:i2,j1:j2) )
!	local_area=	sum( 												ct3(1,i1:i2,j1:j2)*area_tbox(i1:i2,j1:j2) )
!#if CMF_VERSION == 2  	
!	call comp_p % send_intg	(local_area,		&	
!				[local_integral], 	&
!				['instant global heatflux'] )
!#endif	
!	!call comp_p % send_intg	(temp,									&	!writing only global annual means + their year numbers (+step number - for control)
!	!	[real(kind=8) :: tem_global_mean(kl+1,1),tem_global_mean(kl+1,2),time_year,time_l], 	&
!	!	[character(NAME_LEN) :: T_names(kl+1,1),T_names(kl+1,2),'year','step'] )
!	end
		
		
!---------------------------------------------------------------------------	
!	Take t,u,v arrays and calculate their horizontal means - tem_global_mean and ke_global_mean
!---------------------------------------------------------------------------
!	subroutine o_horizons(t_array,u_array,v_array)
!	use o_core_diagnostics_module
!	use o_grid_module
!    use o_par_module
!
!	use o_basic_module
!	implicit none
!	real(8) tem_local_integral(kl+1,ml),ke_local_integral(kl+1),ssl_local_integral,volume_local_t(kl+1),volume_local_v(kl+1),area_local_t,temp(1)
!	real(8) t_array (kl,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1, ml	)
!	real(8) u_array (kl,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1	)
!	real(8) v_array (kl,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1	)
!	
!	!Grid boxes volumes including ct3 (i1,i2,j1,j2 are stored and initialized in o_core_diagnostics_module)
!	do i=i1,i2
!	do j=j1,j2
!	do k=1,kl
!		gridbox_volume_t(k,i,j)= area_tbox(i,j) * dble(ct3(k,i,j)) * dble(dx3_t_c(k+1,i,j))
!		!gridbox_volume_v(k,i,j)= area_vbox(i,j) * dble( cv(k,i,j)) * dble(dx3_v_c(k+1,i,j))
!	end do
!	end do
!	end do
!		
!	!Horizontal means
!	do k=1,kl
!		volume_local_t(k)=sum(	gridbox_volume_t(k,i1:i2,j1:j2) )
!		volume_local_v(k)=sum(	gridbox_volume_v(k,i1:i2,j1:j2) )
!		
!		do m=1,ml
!			tem_local_integral(k,m)=sum(  dble(t_array(k,i1:i2,j1:j2,m))  *  gridbox_volume_t(k,i1:i2,j1:j2)  ) 
!#if CMF_VERSION == 2    
!            call comp_p % reduce_itg(volume_local_t(k), [real(kind=8) :: tem_local_integral(k,m)],temp )
!#elif CMF_VERSION == 3
!            temp(1) = UtilsAllReduce(volume_local_t(k), tem_local_integral(k,m), ocn_comm)
!#endif 			
!			tem_global_mean(k,m)=temp(1)
!		end do
!
!		!ke_local_integral(k)=sum(  dble(u_array(k,i1:i2,j1:j2)**2 + v_array(k,i1:i2,j1:j2)**2)  *  gridbox_volume_v(k,i1:i2,j1:j2)   )    
!		!call comp_p % reduce_itg(volume_local_v(k), [real(kind=8) :: ke_local_integral(k)],temp )
!		!ke_global_mean(k)=temp(1)
!	end do
!		
!	!Full W.O. mean. "Level kl+1" means "average over whole il*jl*kl"
!	volume_local_t(kl+1)=sum(volume_local_t(1:kl))
!	!volume_local_v(kl+1)=sum(volume_local_v(1:kl))
!	
!	do m=1,ml
!		tem_local_integral(kl+1,m)=sum(  tem_local_integral(1:kl,m)  ) 
!#if CMF_VERSION == 2    
!        call comp_p % reduce_itg(volume_local_t(kl+1), [real(kind=8) :: tem_local_integral(kl+1,m)],temp )
!#elif CMF_VERSION == 3
!        temp(1) = UtilsAllReduce(volume_local_t(kl+1), tem_local_integral(kl+1,m), ocn_comm)
!#endif 		
!		tem_global_mean(kl+1,m)=temp(1)
!	end do
!
!	!ke_local_integral(kl+1)=sum(  ke_local_integral(1:kl)   )    
!	!call comp_p % reduce_itg(volume_local_v(kl+1), [real(kind=8) :: ke_local_integral(kl+1)],temp )
!	!ke_global_mean(kl+1)=temp(1)
!	
!	!SSL global mean
!	ssl_local_integral=sum(  ssl_c(i1:i2,j1:j2)  *  area_tbox(i1:i2,j1:j2) * dble(ct3(1,i1:i2,j1:j2))  ) 
!	area_local_t      =sum(                         area_tbox(i1:i2,j1:j2) * dble(ct3(1,i1:i2,j1:j2))  )
!#if CMF_VERSION == 2    
!    call comp_p % reduce_itg(area_local_t, [real(kind=8) :: ssl_local_integral],temp )
!#elif CMF_VERSION == 3
!        temp(1) = UtilsAllReduce(area_local_t, ssl_local_integral, ocn_comm)
!#endif      
!	
!	ssl_global_mean=temp(1)
!	
!	end