
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	diagnostical parameters of solution 
!	Module defining variables and subroutines needed for mixed layer depth calculation
!	This is a self-allocating module
! authors: 
!	2017.11 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
! reference:
!	K. V. Ushakov, R. A. Ibraev, Physical Oceanography, 2018.
!--------------------------------------



#include "o_defines.f90"
module o_diagn_mld_module
	use o_grid_module
	use o_basic_module
	use o_par_module
	save
    
	real (kind=4), allocatable :: mld(:,:),mld_max(:,:)			!current mixed layer depth, maximum mixed layer depth
	real (kind=4), allocatable :: mld_int(:,:)				!current mixed layer depth (number of w-horizon)
	real (kind=4), allocatable :: convadjes(:,:),convadjes_perstep(:,:)	!calculating convection adjustment events
    
	CONTAINS
  
	subroutine o_diagn_mld_allocate
			allocate (mld 	   		(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
			allocate (mld_max  		(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
			allocate (mld_int    		(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
			!allocate (convadjes		(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
			!allocate (convadjes_perstep	(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
			mld_max=0.
			mld=0.
			mld_int=0.
			!convadjes=0.
			!convadjes_perstep=0.
	end subroutine o_diagn_mld_allocate
	
	subroutine o_diagn_mld
		implicit none
		real(8) sigma_0,sigma_h,rho_from_theta
		
		mld=0.
		mld_int=0.
		!if(mod(time_l_in_run,minday*365*3)==0) then
		!	if(myrank==1) print *,'RESETTING MLD_MAX'
		!	mld_max=0.
		!end if
			
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			if(kt(i,j)>0) then
				sigma_0=rho_from_theta(dble(t_c(1,i,j,2)),dble(t_c(1,i,j,1)),0.d0)  		!sigma_0 - potential density at the surface
				do k=2,kt(i,j)
					sigma_h=rho_from_theta(dble(t_c(k,i,j,2)),dble(t_c(k,i,j,1)),0.d0)  	!sigma_h - potential density at the k level				
					if(abs(sigma_h-sigma_0).ge.0.125) then
						mld(i,j)=zw(k)
						mld_int(i,j)=k
						goto 1
					end if
				end do
				mld(i,j)=zw(kt(i,j)+1)
				mld_int(i,j)=kt(i,j)+1
	1			continue			
			end if
		end do
		end do
		mld_max=max(mld_max,mld)
		
	end subroutine o_diagn_mld
	
end module o_diagn_mld_module