#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

	module o_driver_module
	
	use o_par_module	
	use o_grid_module
	use o_td_module
	use o_td_mixing
	use o_tf_module	
	use o_d_brcl_module
	use o_d_vmc_1_module
	use o_basic_module
	use ice_module
	use atm_module
	use test_ocean_module
	use o_core_diagnostics_module
	use o_diagn_basic_module
	use o_diagn_surface_module
	use o_diagn_mht_module
	use o_diagn_mld_module
	use o_td_adv_fctz
	
	implicit none
	integer dg_period, cp_period
	character(NAME_LEN) das_recv_info
	CONTAINS
	
!================================================================================================================================================================
subroutine o_driver_ini_params
		
	namelist /OCN_MAIN/ 	il, jl, kl, ml, ocn_steps_in_day, ocn_read_cp, dg_period, cp_period, ocn_das_on
	namelist /OCN_PHYSICS/ 	am_lapl_mode, am_bh_mode, am_lapl, am_bh, c_smag_lapl, c_smag_bh, am_lapl_2d, ah_mode, ah_bh_mode, ah, ah_bh, sweq_pl, omega, gravit, densit, gsolar1, zsolar1, v_mix_mod, c1amv, c2amv_all, c1ahv, c2ahv_all, albedo_ocean, albedo_ice, atm_model,t_potential_convert
	namelist /OCN_GRID/ 	i_cycle, j_cycle, ssc_flag, bsc_flag, x1_ssc_size, x2_ssc_size, x1_ssc_west_south, x2_ssc_west_south, x2_ssc_step, pool_min_number, i_solution_control, j_solution_control, k_solution_control
	namelist /OCN_SCHEMES/ 	scalar_scheme_time, scalar_scheme_advection, its, mixh_m_mode, mixh_t_mode, coriolis_alfa, coriolis_gamma, coriolis_betta
	namelist /INMIO_SLAV/ 	ao_fluxes_mode, ao_conserv_fluxes
	
	!Read namelist and get basic ocean parameters
	write(namelist_path,'(A)') './configure/ocn_list.in'
	
	open(8,file = namelist_path,err=1)
	read(8, nml = OCN_MAIN)
	read(8, nml = OCN_PHYSICS)
	read(8, nml = OCN_GRID)
	read(8, nml = OCN_SCHEMES)
	close(8)
	
	open(9,file = './configure/coupling_list.in',err=1)
	read(9, nml = INMIO_SLAV)	
	close(9)
	
	tau = (24./REAL(ocn_steps_in_day)*3600)/sweq_pl
	lmats	= ocn_steps_in_day
	hthour = 24./REAL(ocn_steps_in_day)
	mltke	= 0
	
	if(bsc_flag==0) then
		jl_ssc=jl
	else 
		jl_ssc=NINT(x2_ssc_size/x2_ssc_step)+2
	end if
	
	il_ssc	= il
	il_bsc = (jl-jl_ssc)*2
	jl_bsc = il/2

	minday = ocn_steps_in_day
	key_lb=0
	
	robert_asselin	= 0.0
	htsec = hthour*3600.
	
	c2amv=c2amv_all
	c2ahv=c2ahv_all
	
	!print *, '*MAIN*', il,jl,kl,ml,ocn_steps_in_day,ocn_read_cp,dg_period
	if(ocn_rank_local==1) print *, '*PHYSICS*', am_lapl_mode, am_bh_mode, am_lapl, am_bh, c_smag_lapl, c_smag_bh, am_lapl_2d, ah_mode, ah_bh_mode, ah, ah_bh, sweq_pl, omega, gravit, densit, gsolar1, zsolar1, v_mix_mod, c1amv, c2amv_all, c1ahv, c2ahv_all, albedo_ocean, albedo_ice, atm_model,t_potential_convert
	!print *, '*GRID*', i_cycle, j_cycle, ssc_flag, bsc_flag, x1_ssc_size, x2_ssc_size, x1_ssc_west_south, x2_ssc_west_south, x2_ssc_step, pool_min_number
	!print *, 'SCHEMES', scalar_scheme_time, scalar_scheme_advection, its, mixh_m_mode, mixh_t_mode, coriolis_alfa, coriolis_gamma, coriolis_betta
	if(ocn_rank_local==1) print *, '*COUPLING*', ao_fluxes_mode
	
	ice_ocn_mapping=2*1440/ocn_steps_in_day
	ocn_ice_mapping=2*1440/ocn_steps_in_day
	
	atm_ocn_mapping=24 !120
	ocn_atm_mapping=24 !240
	
	return
1	call ocn_error('Cannot read ocean nameleist! Stop.', 'LCL')
end subroutine o_driver_ini_params
!================================================================================================================================================================

subroutine o_driver_subdomains(iw, ie, js, jn)
    integer, intent(in) :: iw, ie, js, jn
				
	call set_grid_subdomains(ocn_rank_local + 1, iw + 1, ie + 1, js + 1, jn + 1)
	
end subroutine

!================================================================================================================================================================
	
subroutine o_driver_allocate
		
	call allocate_o_grid_coord_module
	call allocate_o_grid_step_module	
	call allocate_o_basic_module
	call allocate_o_d_brcl_module
	call allocate_o_td_module
	call allocate_o_d_brtr_sngl_module
	call allocate_ice_module
	if(trim(atm_model_name)=='ATM_INM') call allocate_atm_module
	call allocate_o_tf_module

	call allocate_test_ocean_module	
	if(scalar_scheme_advection==3) call allocate_o_som_module
	if(scalar_scheme_advection==2) call allocate_o_td_adv_fctz
	call o_diagn_basic_module_init		!time means of u,v,t,s,ssl
	call o_diagn_surface_module_init	!time means of ssr,str,sns,ltn,prec,evap,wind_stresses,total_surf_heat
	!call o_diagn_a2o_module_init
	call o_diagn_mht_init
	call o_diagn_mld_allocate
	
end subroutine o_driver_allocate
!================================================================================================================================================================

subroutine o_driver_ini_main
		
	integer :: counter
	integer :: ierr
	real (kind=4) land_value
	
!===================================================================
! Prepare kv and kt, which are read from file in REAL representation
!===================================================================
#if CMF_VERSION == 2
	call comp_halo_update_V4 (DIM2_u = kv_real, DIM2_v = kv_real, update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_T4 (DIM2_t = kt_real, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	! Init global HaloUpdate object
	call HaloUpdateMaster % init(decomp = ocn_decomp, max_halo_width = 2)
	
	call HaloUpdateMaster % update(kv_real, update_width = 1, grid_type = 'V')    
	call HaloUpdateMaster % update(kt_real, update_width = 1, grid_type = 'T')    
#endif
	kv = NINT(kv_real)
	kt = NINT(kt_real)
	ocn_ice_test=kt_real+1

!------->>>
!	unmask CP arrays. Fill land with land_value = 0.
!-------
	land_value = 0.	
	call o_mask_land_CP (land_value)
	!print *, "call o_mask_land (missing_value) ", land_value
!-------<<<
	
!===============================================================
! This cycle make queue for set_grid, because since 1/12 grade
! arrays _ssc and _bsc need more then 1 gb RAM per processor.
! For medium grid you can use casual version of set_grid calls.
!===============================================================
	if(ocn_rank_local == 0) write(*,'(A$)') 'Filtered <set_grid> method... '
 	do counter = 0, 7
 		if(MOD(ocn_rank_local,8) == counter) then
 			call o_ini_set_grid_coordinate
 			call o_ini_set_grid_step
 		end if
 
  	      call MPI_BARRIER(ocn_comm, ierr)
  	end do
	if(ocn_rank_local == 0) write(*,'(A)') 'complete.'

! 	do counter = 0, size-1
! 		if(rank == counter) then
! 			call o_ini_set_grid_coordinate
! 			call o_ini_set_grid_step
! 		end if
! 
! 	      call MPI_BARRIER(comm, ierr)
!  	end do

! 	if(ocn_rank_local == 0) write(*,*) 'Full parallel set grid method start.'
! 	call o_ini_set_grid_coordinate
! 	call o_ini_set_grid_step
! 	if(ocn_rank_local == 0) write(*,*) 'Full parallel set grid method finished.'

!================
! Set mesh arrays
!================
	call o_ini_cv_ct
	call o_ini_set_physics
	call o_ini_area_box
	
	!---short notations for array boundaries----
	iwe=iwest(myrank)
	iea=ieast(myrank)
	jso=jsouth(myrank)
	jno=jnorth(myrank)
	
	if(t_potential_convert == 1 .AND. time_l == 1) call o_ini_convert()
	
	!treating salinity as an anomaly above S_ref (usually S_ref=35 psu or =0 psu)
	t_c(:,:,:,2)=t_c(:,:,:,2)-S_ref
	
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k = 1, kl
		if(k.gt.kt(i,j)) t_c(k,i,j,:)=0.
		if(k.le.kt(i,j) .and. t_c(k,i,j,2).lt.0.001) then
			write(*,'(a,3i,2f8.3)')'WARNING: possibly unfilled T-box initial condition at ', i,j,k,t_c(k,i,j,:)
			!t_c(k,i,j,:)=t_c(max(k-1,1),i,j,:)
		end if
	end do
	end do
	end do
	
	!Initial ad hoc correction of ssl field
	ssl_c(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)=h_bt_a_f
	dx3_v_l(2,:,:)=hzw(2)+h_bt_a_f
	
!=====================================================================
! Make first halo exchanges. Don't forget make initial exchanges for
! all arrays from control point, because they are stored without halo.
!=====================================================================
#if CMF_VERSION == 2
	call comp_halo_update_T (DIM4_t = t_c, update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_V (DIM3_u = u_c, DIM3_v = v_c, update_width = 1, change_sign_on_bipolar = -1)
	call comp_halo_update_V8(DIM2_u = u_bt_a_f, DIM2_v = v_bt_a_f, update_width = 1, change_sign_on_bipolar = -1)
	call comp_halo_update_T8(DIM2_t = h_bt_a_f, update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_T (DIM2_t = w_surf_c, update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_T8(DIM2_t = ssl_c, update_width = 2, change_sign_on_bipolar = 1)
	
	call comp_halo_update_T4(DIM2_t = hice(:,:), update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_T4(DIM2_t = aice(:,:), update_width = 1, change_sign_on_bipolar = 1)
	call comp_halo_update_V (DIM2_u = dx3_v_l(2,:,:),DIM2_v = dx3_v_l(3,:,:),update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	call HaloUpdateMaster % update(t_c(:,:,:,1), update_width = 1, grid_type = 'T')   
	call HaloUpdateMaster % update(t_c(:,:,:,2), update_width = 1, grid_type = 'T')   
	call HaloUpdateMaster % update(u_c, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)   
	call HaloUpdateMaster % update(v_c, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)   
	call HaloUpdateMaster % update(u_bt_a_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)   
	call HaloUpdateMaster % update(v_bt_a_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)   
	call HaloUpdateMaster % update(h_bt_a_f, update_width = 1, grid_type = 'T') 
	call HaloUpdateMaster % update(w_surf_c, update_width = 1, grid_type = 'T') 
	call HaloUpdateMaster % update(ssl_c, update_width = 2, grid_type = 'T') 
	call HaloUpdateMaster % update(hice(:,:), update_width = 1, grid_type = 'T') 
	call HaloUpdateMaster % update(aice(:,:), update_width = 1, grid_type = 'T') 
	call HaloUpdateMaster % update(dx3_v_l(2,:,:), update_width = 1, grid_type = 'V') 
	call HaloUpdateMaster % update(dx3_v_l(3,:,:), update_width = 1, grid_type = 'V')
#endif
	call o_ini_initiate_arrays()

!============================================
! Initiate laplacian of u,v,t,s and get it's halo
!============================================
	call o_d_mixh_ini	!initiate viscosities
	call o_td_mixh_ini	!initiate diffusivities
	call o_d_mixh_m_lapl(u_c,v_c)
	call o_td_mixh (t_c,mht_subgrid_u_mean,mht_subgrid_v_mean)
#if CMF_VERSION == 2	
	call comp_halo_update_V (DIM3_u = u_laplace, DIM3_v = v_laplace, update_width = 1, change_sign_on_bipolar = -1)
	call comp_halo_update_T (DIM4_t = t_laplace, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	call HaloUpdateMaster % update(u_laplace, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
	call HaloUpdateMaster % update(v_laplace, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
	call HaloUpdateMaster % update(t_laplace(:,:,:,1), update_width = 1, grid_type = 'T')
	call HaloUpdateMaster % update(t_laplace(:,:,:,2), update_width = 1, grid_type = 'T')
#endif
!=============================================
! Copy data for 'l' timestep to 'l-1' timestep
!=============================================
	u_l = u_c
	v_l = v_c

	t_f	= t_c
	t_l	= t_c
	ssl_f	= ssl_c			!subr. make dx3___f works with ssl_f

	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	do k = 2, kl
		dx3_t_f(k+1,i,j) = hzw(k+1)
		dx3_v_f(k+1,i,j) = hzw(k+1)
	end do
	end do
	end do
	
	call o_d_make_dx3___f
	dx3_t_c = dx3_t_f
	dx3_v_c = dx3_v_f
	forall(k=3:kl+1) dx3_v_l(k,:,:)=hzw(k)   !dx3_v_l is saved in CP, dx3_t_l need not be saved since T leap-frog is not called on restart step
	hice=max(0.,hice)
	
	ocn_atm_sst=t_c(1,:,:,1)	!sst must be sent to ATM_INM on initialization
end subroutine o_driver_ini_main
				
!================================================================================================================================================================

	subroutine o_driver_main_step

!-------
!	one may be confused due to old version of designations of timestep for arrays:
!	NOW (with CMF2.0)
!	time timestep counter is equal to 'l' and we are finding solution at 'l' timestep
!	that means that f_c is the solution at 'l-1' timestep
!			f_l is the solution at 'l-2' timestep
!	we compute f_f, the solution at 'l' timestep
!-------
	real(8) averages_array(2)  
	integer lmats_save

	lmats_save=lmats
#if CMF_VERSION == 3
	call HaloUpdateMaster % update(t_c(:,:,:,1), update_width = 1, grid_type = 'T')
	call HaloUpdateMaster % update(t_c(:,:,:,2), update_width = 1, grid_type = 'T')
	call HaloUpdateMaster % update(u_c, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
	call HaloUpdateMaster % update(v_c, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
	call HaloUpdateMaster % update(ssl_c, update_width = 2, grid_type = 'T')	
	call o_mask_land_CP (real(0.))

	if (TRIM(array_opt) == "new") then
		if (ocn_rank_local == 0) write(*,*), "New Assimilated Data come!"
		array_opt = "used"
		
		ssl_f=ssl_c
		call o_d_make_dx3___f
		
		dx3_t_c = dx3_t_f
		dx3_v_c = dx3_v_f
		
		h_bt_a_f=ssl_c(iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
		
		lmats=1
		
		!dx3_t_l = dx3_t_c
		!dx3_v_l = dx3_v_c
	end if
#endif
	
!========================================
! 2.  Calculate current 'l' timestep data
!========================================
	
	!-----Attention: Patch for bypassing HMC initialization bug-----
	!if(time_l_in_run.le.minday) then
	!	runoff=0.
	!	surface_salt=t_c(1,:,:,2)+S_ref
	!end if
	
	if(mod(time_l,15*lmats)==1) then
		!Ad hoc correction of ssl field
		temp2=ssl_c(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)-h_bt_a_f
		ssl_c(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)=h_bt_a_f
		dx3_v_l(2,:,:)=hzw(2)+h_bt_a_f
		
		ssl_f	= ssl_c
		call o_d_make_dx3___f !make dx3_t_f, ssl_v_f and dx3_v_f
		dx3_t_c = dx3_t_f
		dx3_v_c = dx3_v_f
		
		!dx3_t_l(2,:,:)=dx3_v_l(2,:,:)
		if (ocn_rank_local == 0) print *,'****Correcting sea level***'
	end if
	
	!---Symmetrizing bipolar Greenwich ---
	if(j_cycle==1 .and. MOD(time_l,lmats) == 1) then
		do k = 1, kl
			call greenwich_symmetrize(u_c(k,:,:),v_c(k,:,:),-1)
			call greenwich_symmetrize(u_l(k,:,:),v_l(k,:,:),-1)
		end do		
		call greenwich_symmetrize(u_bt_a_f,v_bt_a_f,-1)
	end if
	
	!---formal tests of ocean solution---
	call o_online_tests
	if(skip_ocean_calculations==1)	goto 1		!skip all the ocean calculations in case of problems in solution 
		
	!---Correct UV-field for safety---
	u_c=u_c*cv(1:kl,:,:)
	v_c=v_c*cv(1:kl,:,:)
	
	!---top fluxes including CORE data---
	call o_tf_main

	!---compute W---
	call o_d_w

	!---compute density---
	call o_td_state_mcdougall2003
	!call o_td_state_unesco1983
	
	!---compute vertical mixing coefficients---
  	call o_vmc_drive

	!---compute solar radiation intensities at horizons---
	call o_td_solar
	
	!---compute t_source for thermodynamical equations---
	call o_td_source

!=================================================
! 3.  Calculate next 'l+1' timestep data
!=================================================
		
	!---compute 'u,v,ssl'---
	call o_d_main

	!---compute and exchange T,S and other scalars---
	call o_td_main(time_l)
#if CMF_VERSION == 2	
	call comp_halo_update_T(DIM4_t = t_f, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	call HaloUpdateMaster % update(t_f(:,:,:,1), update_width = 1, grid_type = 'T')
	call HaloUpdateMaster % update(t_f(:,:,:,2), update_width = 1, grid_type = 'T')
#endif
	
	!3.6.2	integrate 'hice', 'aice' to 'l+1', modify t_f at 'l+1' (is not called if the atmosphere component is absent)
    if ((atm_model_name =="ATM_NCAR" 		.or.  &
         atm_model_name =="ATM_GFS" 		.or.  &
         atm_model_name =="ATM_INTERIM" 	.or.  &
         atm_model_name =="ATM_INMCM4" 	.or.  &
         atm_model_name =="ATM_INM" ) 		.and. & 
         (ice_model_name .ne. "ICE_CICE" .or. (cice_and_schrum.and.jsouth(myrank).lt.jl/2-10)))	call ice_td_schrum()
	
	!---Update 'u,v' halo at 'l+1'---
#if CMF_VERSION == 2    
	call comp_halo_update_V(DIM3_u = u_f, DIM3_v = v_f, update_width = 1, change_sign_on_bipolar = -1)
#elif CMF_VERSION == 3
	call HaloUpdateMaster % update(u_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
	call HaloUpdateMaster % update(v_f, update_width = 1, grid_type = 'V', change_sign_on_bipolar = .TRUE.)
#endif	
	
	!---compute and update laplacian of 'u,v' that will be used on 'l+1' step ---
	if(MOD(time_l,lmats) /= 0)	call o_d_mixh_m_lapl (u_c,v_c)
	if(MOD(time_l,lmats) == 0)	call o_d_mixh_m_lapl (u_f,v_f)
		
	!---compute and update laplacian of 't' that will be used on 'l+1' step ---
	if (scalar_scheme_time.eq.0) call o_td_mixh(t_f,mht_subgrid_u_mean,mht_subgrid_v_mean)
	if (scalar_scheme_time.eq.1) then
			if(mod(time_l,lmats).ne.0) call o_td_mixh(t_c,mht_subgrid_u_mean,mht_subgrid_v_mean)
			if(mod(time_l,lmats).eq.0) call o_td_mixh(t_f,mht_subgrid_u_mean,mht_subgrid_v_mean)
	end if		

	!---robert-asselin time filter---
	if(abs(robert_asselin).gt.1.e-4 .and. mod(time_l-1,lmats).ne.0) then
		u_c = u_c + robert_asselin * (u_l - 2.*u_c + u_f)
		v_c = v_c + robert_asselin * (v_l - 2.*v_c + v_f)
		t_c = t_c + robert_asselin * (t_l - 2.*t_c + t_f)
		dx3_t_c = dx3_t_c + robert_asselin * (dx3_t_l - 2.*dx3_t_c + dx3_t_f)
		dx3_v_c = dx3_v_c + robert_asselin * (dx3_v_l - 2.*dx3_v_c + dx3_v_f)
	end if

!=================================================
! 4.2 Reassign arrays for next timestep
!=================================================

	u_l = u_c
	u_c = u_f
	v_l = v_c
	v_c = v_f
	t_l = t_c
	t_c = t_f
	ssl_c = ssl_f

	dx3_t_l = dx3_t_c
	dx3_v_l = dx3_v_c
	dx3_t_c = dx3_t_f
	dx3_v_c = dx3_v_f
	
	lmats=lmats_save

1	continue

	!printing out the solution in a specified point
	if(mod(time_l,1).eq.0	.and.	j_solution_control.ge.jsouth(myrank)		&
				.and.	j_solution_control.le.jnorth(myrank)		&
				.and.	i_solution_control.ge. iwest(myrank)		&
				.and.	i_solution_control.le. ieast(myrank) ) then
		write(*,'(a3,a4,i7, i5,i2.2,i2.2, a1,i2.2,a1,i2.2,  5f13.8,1x,e16.8)')						& 
			run_experiment,												&
			' ocn',												&
			time_l_in_run,												&
			time_year,time_month,time_day,'/',time_hour,':',time_min,						&
			t_c	(k_solution_control,	i_solution_control,j_solution_control,1),				&
			u_c	(k_solution_control,	i_solution_control,j_solution_control),					&
			ssl_c	(			i_solution_control,j_solution_control),					&
			aice	(			i_solution_control,j_solution_control),					&
			w	(kt(i_solution_control,j_solution_control)+1,i_solution_control,j_solution_control),		&
			averages_array(1)
	end if

	end subroutine
		
!================================================================================================================================================================
	subroutine o_driver_destroy()
	
!	call destroy_o_grid_module()
!   ....
	
	end subroutine
	end module o_driver_module
