
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	bottom
!	compute model topography
!	clean model topography:
!	 - make halo and symmetry
!	 - remove single cell islands
!	 - delete single holes
!	 - delete overlapping nodes
!	 - remove small lakes
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!	2017.04 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru
!
! reference:
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!--------------------------------------

!====================================================
! Compute bottom topography arrays 'kv' and 'kt'.
! Compute height of partial bottom v-box and t-boxes.
!====================================================

	subroutine o_ini_bottom(topo, is_print)

	use o_grid_module

	implicit none
	
	real(4), intent(in) :: topo(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	logical, intent(in) :: is_print
	
	real(kind=4) hv_0(iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
! 	real(kind=4), dimension(iwest(myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) :: kv_for_exch, kt_for_exch
    character (kind=1), allocatable:: bt    (:,:)

	write(*,'(A$)') 'Ocean bottom tuning...'
	
! Initiate
	kv = 0
	kt = 0
	hv_0 = topo
	
	do i = iwest(myrank), ieast(myrank)
	do j = jsouth(myrank), jnorth(myrank)
		if(hv_0(i,j) <= 0.0) hv_0(i,j) = 0.
	end do
	end do

! Compute 'kv'
	do k = 1,kl
	do i = iwest(myrank), ieast(myrank)
	do j = jsouth(myrank), jnorth(myrank)
		if(hv_0(i,j) <= zw(k+1) .AND. hv_0(i,j) > zw(k)) kv(i,j) = k
	end do
	end do
	end do

	do i = iwest(myrank), ieast(myrank)
	do j = jsouth(myrank), jnorth(myrank)
		if(kv(i,j) == 1) kv(i,j) = 2
	end do
	end do

	do i = iwest(myrank), ieast(myrank)
	do j = jsouth(myrank), jnorth(myrank)
		if(hv_0(i,j) >= z(kl)) kv(i,j) = kl
	end do
	end do
!-------
! 	hand made correction of 'kv'.
!	. . . /release/~configure/off_ocn_configure.f90: subroutine o_user_bottom_tuning
!-------
	call o_user_bottom_tuning
	
	call kv_make_halo_and_symmetry (kv,0)

	call kv_clean_single_cell_islands

!!	call kv_clean_single_cell_peninsulas

	call kv_clean_holes

	call kv_clean_overlapping

	call kv_make_halo_and_symmetry (kv,0)

	call o_ini_bottom_remove_lakes
        write(*,'(A)') '################ after o_ini_bottom_remove_lakes'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)


	call kv_final_tuning		!	kv-tuning due to various physical reasons

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_holes
        write(*,'(A)') '################ after clean_holes'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_overlapping
        write(*,'(A)') '################ after clean_overlapping'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_overlapping
        write(*,'(A)') '################ after clean_overlapping'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_holes
        write(*,'(A)') '################ after clean_holes'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_holes
        write(*,'(A)') '################ after clean_holes'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_overlapping
        write(*,'(A)') '################ after clean_overlapping'
	call kv_check_symmetry (kv)

	call kv_make_halo_and_symmetry (kv,0)
	call kv_clean_holes
        write(*,'(A)') '################ after clean_holes'
	call kv_check_symmetry (kv)


!	call kv_make_halo_and_symmetry (kv,0)


		do i = iwest(myrank), ieast(myrank)
		do j = jsouth(myrank), jnorth(myrank)
		if(kv(i,j) == 1) kv(i,j) = 2
		end do
		end do

!-------
!	compute 'kt'
!-------
		do i = iwest(myrank), ieast(myrank)
		do j = jsouth(myrank), jnorth(myrank)
		kt(i,j) =  MAX0(kv(i-1,j-1),kv(i-1,j), kv(i,j-1), kv(i,j))
		end do
		end do

	write(*,'(A)') ' success.'
	
!	print out topography
	if (is_print) then
        write(*,'(A)') 'start printing kv'
        allocate (bt    (il,jl))
        call charar(kv,bt,il,jl)
        call out_map('t02','_kv',bt,x1_v,x2_v,il,jl)
        call charar(kt,bt,il,jl)
        call out_map('t02','_kt',bt,x1_t,x2_t,il,jl)
        deallocate(bt)
        write(*,'(A)') 'finish printing kv'
	end if
        
	return
	end




!~~~~~~~
	subroutine kv_make_halo_and_symmetry (matrix,key_matrix)
!~~~~~~~

	use o_grid_module
	implicit none
	integer (kind=4) matrix(il,jl)
	integer (kind=4) key_matrix

!-------
!	make borders in case of non-i_cycle domain
!-------
	if(i_cycle == 0) then
		do j = 1,jl
			matrix( 1:2,j) = 0
			matrix(il-1:il,j) = 0
		end do
	end if

!-------		
!	make global halo update
!-------
	if(i_cycle == 1) then
		do j = 1,jl
			matrix( 1,j) = matrix(il-1,j)
			matrix(il,j) = matrix(   2,j)
		end do
	end if

!-------
!	make borders in case of non-bsc_flag configuration
!-------
	if (bsc_flag == 0) then
		do i = 1,il
			matrix(i,jl-1:jl)=0
		end do
	end if

!-------		
!	make symmetry relatively bsc-greenwich
!-------
	if (bsc_flag == 1) then
		do i = 1,il
			matrix(i,jl)=matrix(il-1-(i-2),jl-2)
		end do
	if (key_matrix == 1) then
		do i = 1,il
			matrix(i,jl)=0
		end do
	end if
	end if

!-------
!	make kv equal at bsc greenwich on both bsc hemispheres 
!-------
	if (bsc_flag == 1) then
	! II quadrant
	do i = 2,il/2
			matrix(i,jl-1)=matrix(il-1-(i-2),jl-1)
		end do
	end if
	print *,'kv_make_halo_and_symmetry has been done'	
	return
	end




!~~~~~~~
	subroutine kv_check_symmetry (matrix)
!~~~~~~~

	use o_grid_module
	implicit none
	integer (kind=4) matrix(il,jl)

!-------		
!	check global halo update
!-------
	i = 0
	j = 0
	if(i_cycle == 1) then
		do j = 1,jl
			if( matrix( 1,j) .ne. matrix(il-1,j) ) print *,'kv_check_symmetry #1',i,j,matrix( 1,j),matrix(il-1,j)
			if( matrix(il,j) .ne. matrix(   2,j) ) print *,'kv_check_symmetry #1',i,j,matrix(il,j),matrix(   2,j)
		end do
	end if


!-------		
!	check symmetry relatively bsc-greenwich
!-------
	i = 0
	j = 0
	if (bsc_flag == 1) then
		do i = 1,il
			if( matrix(i,jl) .ne. matrix(il-1-(i-2),jl-2) ) print *,'kv_check_symmetry #2',i,j,matrix(i,jl),matrix(il-1-(i-2),jl-2)
		end do
	end if

!-------
!	check kv equal at bsc greenwich on both bsc hemispheres 
!-------
	i = 0
	j = 0
	if (bsc_flag == 1) then
	! II quadrant
	do i = 2,il/2
			if( matrix(i,jl-1) .ne. matrix(il-1-(i-2),jl-1) )  print *,'kv_check_symmetry #3',i,j,matrix(i,jl-1),matrix(il-1-(i-2),jl-1)
		end do
	end if
	print *,'kv_check_symmetry has been done'	
	return
	end




!~~~~~~~
	subroutine kv_clean_single_cell_islands
!~~~~~~~

!-------
!	delete single V-cell islands
!-------

	use o_grid_module
	
	implicit none

	integer iterat, k1, k2, k3, k4
	integer node, kvsw, kvnw, kvse, kvne, maxw, maxl

!	do k = kl, 1, -1
	k = 1
	do iterat = 1, 10

!-------
!	delete single V-cell islands
!-------
!	do i = iwest(myrank), ieast(myrank)
!	do j = jsouth(myrank), jnorth(myrank)
	do j = 2,jl-1
	do i = 2,il-1
	if(kv(i,j) == 0) then
		k1=0
		k2=0
		k3=0
		k4=0
		if(kv(i-1,j  ) > 0) k1 = 1
		if(kv(i+1,j  ) > 0) k2 = 1
		if(kv(i  ,j-1) > 0) k3 = 1
		if(kv(i  ,j+1) > 0) k4 = 1

		if(k1+k2+k3+k4 >= 3) kv(i,j)=kv(i,j) + 1
	end if
	end do
	end do

!	call kv_make_halo_and_symmetry (kv)

	end do	
!	end do

	print *,'kv_clean_single_cell_islands has been done'	
	return
	end




!~~~~~~~
	subroutine kv_clean_single_cell_peninsulas
!~~~~~~~

!-------
!	delete single V-cell peninsulas
!-------

	use o_grid_module
	
	implicit none

	integer iterat, k1, k2, k3, k4
	integer node, kvsw, kvnw, kvse, kvne, maxw, maxl

!	do k = kl, 1, -1
	k = 1
	do iterat = 1, 10

!-------
!	delete single V-cell west-east peninsula
!-------
!	do i = iwest(myrank), ieast(myrank)
!	do j = jsouth(myrank), jnorth(myrank)
	do j = 2,jl-1
	do i = 2,il-1
	if(kv(i,j) == 0) then
		if(	kv(i-1,j  ) == 0 .and. 		&
			kv(i+1,j  ) == 0 .and. 		&
			kv(i  ,j-1) >= 0 .and.		&
			kv(i  ,j+1) >= 0		) kv(i,j)=kv(i,j) + 1

	end if
	end do
	end do
!-------
!	delete single V-cell south-north peninsula
!-------
!	do i = iwest(myrank), ieast(myrank)
!	do j = jsouth(myrank), jnorth(myrank)
	do j = 2,jl-1
	do i = 2,il-1
	if(kv(i,j) == 0) then
		if(	kv(i-1,j  ) >= 0 .and. 		&
			kv(i+1,j  ) >= 0 .and. 		&
			kv(i  ,j-1) == 0 .and.		&
			kv(i  ,j+1) == 0		) kv(i,j)=kv(i,j) + 1

	end if
	end do
	end do

!	call kv_make_halo_and_symmetry (kv)

	end do	
!	end do

	print *,'kv_clean_single_cell_peninsulas has been done'	
	return
	end




!~~~~~~~
	subroutine kv_clean_holes
!~~~~~~~

!-------
!	Correct array 'kv': 	delete single V-box holes and channels
!-------

	use o_grid_module
	
	implicit none

	integer iterat, k1, k2, k3, k4
	integer node, kvsw, kvnw, kvse, kvne, maxw, maxl

!-------
!	delete single V-box holes and channels
!-------
	do k = kl, 1, -1
	do iterat = 1, 10

!	do i = iwest(myrank), ieast(myrank)
!	do j = jsouth(myrank), jnorth(myrank)
	do j = 2,jl-1
	do i = 2,il-1
	if(kv(i,j) == k) then
		k1=1
		k2=1
		k3=1
		k4=1
		if(kv(i-1,j  ) < kv(i,j)) k1 = 0
		if(kv(i+1,j  ) < kv(i,j)) k2 = 0
		if(kv(i  ,j-1) < kv(i,j)) k3 = 0
		if(kv(i  ,j+1) < kv(i,j)) k4 = 0
	
!	single cell hole and channels
		if(k1+k2+k3+k4 <= 1) kv(i,j)=kv(i,j)-1
!	single cell hole
!		if(k1+k2+k3+k4 == 0) kv(i,j)=kv(i,j)-1

	end if
	end do
	end do

!	call kv_make_halo_and_symmetry (kv)

	end do	
	end do

	print *,'kv_clean_holes has been done'	
	return
	end




!~~~~~~~
	subroutine kv_clean_overlapping
!~~~~~~~

!-------
!	Correct array 'kv': 	prevent overlapping T-points
!-------

	use o_grid_module
	
	implicit none

	integer iterat, k1, k2, k3, k4
	integer node, kvsw, kvnw, kvse, kvne, maxw, maxl

!-------
!	prevent overlapping T-points
!-------
	node=1 !calculator of corrected T-points
	do while (node.gt.0)
		node=0                                                           
		do j=2,jl-1	!running through T-points
		do i=2,il-1
			if(max(kv(i-1,j-1),kv(i-1,j),kv(i,j-1),kv(i,j)).gt.0) then
				
				!--------
				!NE and SW V-boxes are deeper then SE and NW
				!--------
				kvsw=kv(i-1 ,j-1)                                                 
				kvnw=kv(i-1 ,j  )                                                 
				kvse=kv(i   ,j-1)                                                 
				kvne=kv(i   ,j  )                                                 
				if(min(kvsw,kvne).gt.max(kvse,kvnw) ) then
					node=node+1                                                     
					maxw=max(kvsw,kvne)                                            
					maxl=max(kvse,kvnw)                                            
					if(kvsw.eq.maxw) kv(i-1,j-1)=maxl                               
					if(kvne.eq.maxw) kv(i  ,j  )=maxl                               
!	print *,'clean_kv #2.1, node=', node,i,j	
				end if

				!-------
				!SE and NW V-boxes are deeper then NE and SW
				!-------
				kvsw=kv(i-1 ,j-1)                                                 
				kvnw=kv(i-1 ,j  )                                                 
				kvse=kv(i   ,j-1)                                                 
				kvne=kv(i   ,j  )                                                 
				if(min(kvse,kvnw).gt.max(kvsw,kvne)) then
					node=node+1
					maxw=max(kvse,kvnw)
					maxl=max(kvsw,kvne)
					if(kvse.eq.maxw) kv(i  ,j-1)=maxl
					if(kvnw.eq.maxw) kv(i-1,j  )=maxl
!	print *,'clean_kv #2.2, node=', node,i,j
				end if
			end if
		end do
		end do

!	call kv_make_halo_and_symmetry (kv)

	end do

	print *,'kv_clean_overlapping has been done', node
	return
	end




!~~~~~~~
	subroutine kv_final_tuning
!~~~~~~~

!-------
!	Kv-tuning due to various physical reasons
!-------
	use o_grid_module
	implicit none
	integer iter,maxstep
	real(kind=4), dimension(iwest(myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) :: kv_temp
		
	!The Hudson bay must be erased because it is close to bipolar pole and causes instability
	!do i = iwest(myrank), ieast(myrank)
	!do j = jsouth(myrank), jnorth(myrank)
	!	if(x1_v(i,j).le.-83. .and. x2_v(i,j).ge.53. .and. x2_v(i,j).le. 66.8) kv(i,j)=0
	!end do
	!end do
	
	!Making Bosphorus
	!if(exp_configuration==0 .or. exp_configuration==9 .or. exp_configuration==2) then
	!	do i = iwest(myrank), ieast(myrank)
	!	do j = jsouth(myrank), jnorth(myrank)
	!		if(abs(x1_v(i,j)-29.1).le.0.1 .and. abs(x2_v(i,j)-41.1).le.0.2 .and. kv(i,j).eq.0) kv(i,j)=4. 
	!	end do
	!	end do
	!end if
	
	!Smoothing topography steps
	maxstep=10 !maximum allowed topography step (in kv-array)
	do iter=1,10
		kv_temp=kv
!		do i = iwest(myrank), ieast(myrank)
!		do j = jsouth(myrank), jnorth(myrank)
	do j = 2,jl-1
	do i = 2,il-1
			if(kv_temp(i,j).gt.kv(i+1,j)+maxstep) kv_temp(i,j)=max(2,kv(i+1,j)+maxstep)
			if(kv_temp(i,j).gt.kv(i-1,j)+maxstep) kv_temp(i,j)=max(2,kv(i-1,j)+maxstep)
			if(kv_temp(i,j).gt.kv(i,j+1)+maxstep) kv_temp(i,j)=max(2,kv(i,j+1)+maxstep)
			if(kv_temp(i,j).gt.kv(i,j-1)+maxstep) kv_temp(i,j)=max(2,kv(i,j-1)+maxstep)
		end do
		end do
		kv=kv_temp

	end do

	print *,'kv_final_tuning has been done'
	return
	end




!~~~~~~~
	subroutine o_ini_bottom_remove_lakes
!~~~~~~~

!-------
!	 Remove small lakes from bottom topography 'kv'
!-------

	use o_grid_module

	integer (kind=4), allocatable :: mask(:,:)
	
	integer (kind=4) iteration
	integer (kind=4) counter
	
	integer i0, j0, marker, n, n_pool
	integer :: calculate_iter=0

	allocate (mask(il,jl))

!-------
!	initiate
!-------
	mask = 0
	
!-------
!	set initial point for masking the world ocean basin
!-------

	i = il/4
	do j = jl/2,2,-1
		if (kv(i,j) > 0) then
			i0 = i
			j0 = j
		end if
	end do

!-------
!	mark pool #n
!-------

	do j = 2, jl-1
	do i = 2, il-1
		if (kv(i,j) > 0) mask(i,j) = 9999
	end do
	end do

	n_pool = 1
	mask(i0, j0) = n_pool
	iteration = 0

!-------
!	start iteration
!-------
100	continue
!	print *,'Lake removal iteration',calculate_iter,n_pool
	calculate_iter=calculate_iter+1
	iteration = iteration + 1
	marker = 0

	do j = 2,jl
	do i = 2,il-1
		if (mask(i,j) == n_pool) then
			if (mask(i,j-1) == 9999) then
				mask(i,j-1) = n_pool
				marker = 1
			end if
		end if
	end do
	end do

	do j = 2,jl-2
	do i = 2,il-1
		if (mask(i,j) == n_pool) then
			if (mask(i,j+1) == 9999) then
				mask(i,j+1) = n_pool
				marker = 1
			end if
		end if
	end do
	end do

	do j = 2,jl-1
	do i = 3,il-2
		if (mask(i,j) == n_pool) then

			if (mask(i-1,j) == 9999) then
				mask(i-1,j) = n_pool
				marker = 1
			end if

			if (mask(i+1,j) == 9999) then
				mask(i+1,j ) = n_pool
				marker = 1
			end if

		end if
	end do
	end do

!	call kv_make_halo_and_symmetry (mask)

	if (marker == 1) go to 100

!-------
!	now the whole sub-region was marked
!	next: find one more pool
!-------

	do j = 2,jl-1
	do i = 2,il-1
		if (mask(i,j) == 9999) then
			n_pool = n_pool+1
			mask (i,j) = n_pool
			print *,'Starting pool ',n_pool,' from ',i,j
			if(n_pool.eq.9999) Stop 'Pool numbers exhausted!'
			iteration = 0
			go to 101
		end if
	end do
	end do

101	continue

	if (iteration == 0) go to 100

!	print *,'Finished pool ',n_pool,' last (i,j)=',i,j,'iteration=',iteration

!-------
!	finish iteration
!-------



!-------
!	now all water points was marked. remove small lakes from 'mask'
!-------
	do n = 1,n_pool

!-------
!	count how many points in each pool
!-------
		counter = 0
		do j = 1,jl
		do i = 1,il
			if (mask (i,j) == n) then
				counter = counter + 1
				i0 = i
				j0 = j
			end if
		end do
		end do

		if (counter <= pool_min_number) then

			do j = 1,jl
			do i = 1,il
			if (mask (i,j).eq.n) mask (i,j) = 0
			end do
			end do
		end if
	end do

!-------
!	remove small lakes from 'kv'
!-------
	do j = 1,jl
	do i = 1,il
		if (mask(i,j) == 0) kv(i,j) = 0
	end do
	end do

!-------
!	Verifying kv-symmetricity on bipolar greenwich
!-------	
!	if(j_cycle == 1) then
!		do i = 2,il-1
!			if( kv(i,jl-1) .ne. kv( (il-1) - (i-2), jl-1)) then
!				print *,'j_cycle error after lake removal',i,kv(i,jl-1),kv(il-1-i+2,jl-1)
!				stop
!			end if
!		end do
!	end if

	print *,'small lakes has been removed'	
	
	deallocate(mask)

	return
	end

