
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	bottom (and lateral) topography masks
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

!==============================
! Compute mask arrays cv and ct
!==============================

	subroutine o_ini_cv_ct

	use o_grid_module
	use o_basic_module
	use o_d_brcl_module
    use o_par_module

    
	implicit none

	integer counter

!===========
! Compute cv
!===========
	cv = 0.
	do k = 1, kl+1
	do j = jsouth(myrank)-1, jnorth(myrank)+1
	do i = iwest (myrank)-1, ieast (myrank)+1
		if(k <= kv(i,j)) cv(k,i,j) = 1.
	end do
	end do
	end do

!===========
! Compute ct
!===========
	ct3 = 0.
	ct3v= 0.
	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
	do k = 1,kt(i,j)
		ct3(k,i,j) = 0.25*(cv(k,i-1,j  ) + cv(k,i  ,j  ) + cv(k,i-1,j-1) + cv(k,i  ,j-1))
		ct3v(k,i,j) = 1./ct3(k,i,j)
	end do
	end do
	end do

!=======================================================
! Get halo for ct3. Halo updates is written for kl size,
! so exchange 'kl+1' level with 2D halo update.
!=======================================================
#if CMF_VERSION == 2    
    !Found some trouble while exchanging an 1:kl section
    do k=1,kl+1
	call comp_halo_update_T4(DIM2_t = ct3(k,:,:), update_width = 1, change_sign_on_bipolar = 1)
    end do
    call comp_halo_update_T4(DIM3_t = ct3v(1:kl,:,:), update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
    call HaloUpdateMaster % update(ct3(1:kl,:,:), update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(ct3(kl+1,:,:), update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(ct3v(1:kl,:,:),update_width = 1, grid_type = 'T')
#endif

!============
! Test output
!============
! 	call MPI_BARRIER(comm_local, ierr)
! 	do counter = 0, cmp % proc_count-1
! 
! 		if(rank_local == counter) then
! 
! 			write(*,*) 'Local grid function on initialization, rank = ', rank_local
! 
! 			write(*,*) '=== hv'
! 			do j=jnorth(myrank)+1, jsouth(myrank)-1, -1
! 				write(*,'(2000f8.1)')(hv(i,j), i=iwest(myrank)-1,ieast(myrank)+1)
! 			end do
! 			write(*,*) ''
! 
! 			write(*,*) '=== kv'
! 			do j=jnorth(myrank)+1, jsouth(myrank)-1, -1
! 				write(*,'(2000i3)')(kv(i,j), i=iwest(myrank)-1,ieast(myrank)+1)
! 			end do
! 			write(*,*) ''
! 
! 			write(*,*) '=== cv on k = 1'
! 			do j=jnorth(myrank)+1, jsouth(myrank)-1, -1
! 				write(*,'(2000i1)')(INT(cv(1,i,j)), i=iwest(myrank)-1,ieast(myrank)+1)
! 			end do
! 			write(*,*) ''
! 
! 			call flush(6)
! 			call sleep(1)
! 
! 		end if
! 		call MPI_BARRIER(comm_local, ierr)
! 	end do


	return
	end

