
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	grid
!	compute grid variables
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Murray R.J. 1996. 
!		Explicit Generation of Orthogonal Grids for Ocean Models,
!		J. Comp. Physics, 126, 251-273.
!	Ibrayev R.A., Khabeev R.N., Ushakov K.V., 2012. 
!		Eddy-resolving 1/10 Model of the World Ocean. 
!		Izvestiya, Atmospheric and Oceanic Physics, 2012, Vol. 48, No. 1, pp. 37–46.
!--------------------------------------

!--------------------------------------
!		define grid in 3-d space and time
!
!ssc - spherical system of coordinates
!bsc - bipolar system of coordinates
!tsc - tripolar system of coordinates
!----------------------------------------------------------------------

	subroutine o_ini_set_grid_coordinate

	use o_grid_module
	use o_ssc_module
	use o_bsc_module

	implicit none

	real (kind=8) x1_pole_radian, x2_pole_radian, x2_0_radian
	real (kind=8) dx1_radian
	
!-------
!		parameters of the ocean ,earth and basin                            
!-------
	pi	= asin(1.)*2. !3.14159                                                        
	pi180	= pi/180.                                                     
	radius	= 0.6371229e7	!meters
	radv	= 1./radius


!-------
!#1		compute parameters for ssc & bsc
!-------

		call o_ini_set_grid_z

!-------
!#2	compose united array of lengths of boxes
!-------



!-------
!#2.1	add spherical system of coordinates
!-------

c	print *,'#g001#',myrank,size

	if (ssc_flag .eq. 1)	then

		call o_ini_set_grid_ssc_coordinate

!-------
!#2.3	compose united array of v & t-point coordinates
!-------

!-------
!#2.3.1.	add ssc part of the grid
!-------

	do j = 1,jl_ssc
	do i = 1,il_ssc
	x1_v (i,j) = x1_ssc_v (i  )
	x1_t (i,j) = x1_ssc_t (i  )
	x2_v (i,j) = x2_ssc_v (  j)
	x2_t (i,j) = x2_ssc_t (  j)
	end do
	end do

	!Some haloes need in off-block
	if(i_cycle==1) then
		do j = 1,jl
!			x1_t(il,j) = x1_t(2,j)
!			x2_t(il,j) = x2_t(2,j)
		end do
	end if
			
	
	end if	!if (ssc_flag .eq. 1)	then

c	print *,'#g004#',myrank,size


!-------
!#2.2	include bipolar system of coordinates
!-------

	if (bsc_flag .eq. 1) then

		x1_bsc_west_south = x1_ssc_west_south
		x2_bsc_west_south = x2_ssc_west_south + x2_ssc_size


c	print *,'#b001#',myrank,size

		call o_ini_set_grid_bsc_coordinate


!-------
!#2.3	compose united array of v & t-point coordinates
!-------
!----------------------------------------------------------
!2.3.2.	transformation from bsc to ssc 
!		of coordinates of v-points in bsc (degrees) |
!	make united array of coordinates of v-points
!----------------------------------------------------------


	x1_bsc_v_radian = x1_bsc_v*pi180
	x2_bsc_v_radian	= x2_bsc_v*pi180

	x1_pole_radian	= x1_pole*pi180	!The ssc x1 coordinate of bsc North Pole 
	x2_0_radian	= x2_0*pi180

! add II quadrant of bipolar part of grid

	do j = jl_ssc, jl
	do i = 2, il/2

		i_bsc = (il_bsc-1)-(j-jl_ssc)
		j_bsc = (i-1)


		dx1_radian = - atan (
     &				sin(x1_bsc_v_radian (i_bsc))/
     &				tan(x2_bsc_v_radian (j_bsc))  )

		if(x2_bsc_v (j_bsc).ge.0.) dx1_radian = dx1_radian + pi
		x1_v (i,j) = x1_pole_radian + dx1_radian


		x2_v (i,j) = asin( 
     &			(   ( 1. + cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			  - ( 1. - cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			   *(tan(x2_0_radian*0.5))**2		   )
     &			/
     &			(   ( 1. + cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			  + ( 1. - cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			   *(tan(x2_0_radian*0.5))**2		   )  )

	end do		
	end do		

! add I quadrant of bipolar part of grid

	do j = jl_ssc, jl
	do i = il/2+1, il-1
	
	i_bsc = (j-jl_ssc+1)
	j_bsc = (il-1) - i + 1


	dx1_radian = - atan (
     &				sin(x1_bsc_v_radian (i_bsc))/
     &				tan(x2_bsc_v_radian (j_bsc))  )

	if(x2_bsc_v (j_bsc).ge.0.) dx1_radian = dx1_radian + pi
	if(dx1_radian      .lt.0.) dx1_radian = dx1_radian + 2.*pi
	x1_v (i,j) = x1_pole_radian + dx1_radian
	if(x1_v(i,j).lt.0) x1_v(i,j)=x1_v(i,j)+2.*pi   !!!!!correcting line!!!!
	
	x2_v (i,j) = asin( 
     &			(   ( 1. + cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			  - ( 1. - cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			   *(tan(x2_0_radian*0.5))**2		   )
     &			/
     &			(   ( 1. + cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			  + ( 1. - cos(x1_bsc_v_radian(i_bsc))
     &				  *cos(x2_bsc_v_radian(j_bsc)) )
     &			   *(tan(x2_0_radian*0.5))**2		   )  )

	end do		
	end do		


! transform to angle degrees
	do j = jl_ssc, jl
	do i = 2, il-1
		x1_v (i,j) = x1_v (i,j)*180./pi
		x2_v (i,j) = x2_v (i,j)*180./pi
	end do		
	end do		



! add 'frame' points in normal case


	do j = jl_ssc, jl
		x1_v (1,j) = x1_v (2,j)
		x2_v (1,j) = x2_v (2,j)
	end do

	do j = jl_ssc, jl
		x1_v (il,j) = x1_v (il-1,j)
		x2_v (il,j) = x2_v (il-1,j)
	end do

! add 'frame' points for periodic boundary b.c. 

	if(i_cycle .eq. 1) then

		do j = jl_ssc, jl
			x1_v (1,j) = x1_v (il-1,j)
			if(x1_v(il-1,j).gt.0)	x1_v (1,j) = x1_v (il-1,j)-360.
			x2_v (1,j) = x2_v (il-1,j)
		end do

		do j = jl_ssc, jl
			x1_v (il,j) = x1_v (2,j)
			x2_v (il,j) = x2_v (2,j)
		end do
	
	end if	!	if(i_cycle .eq. 1) then

!-------
!2.3.2.+	calculate coordinates of t-points
!-------
	do j = jl_ssc+1, jl
	do i = 2,        il
		x1_t (i,j) = 0.25*	( x1_v (i-1,j-1) 
     &				 +x1_v (i  ,j-1)
     &				 +x1_v (i-1,j  )
     &				 +x1_v (i  ,j  ) )
		x2_t (i,j) = 0.25*	( x2_v (i-1,j-1) 
     &				 +x2_v (i  ,j-1)
     &				 +x2_v (i-1,j  )
     &				 +x2_v (i  ,j  ) )
	end do
	end do

	!Some haloes need in off-block
	do i = 2,il
		x1_t(i,jl) = x1_t(il+2-i,jl-1)
		x2_t(i,jl) = x2_t(il+2-i,jl-1)
	end do
	
	!Make poles in one point
	do j = jl_ssc+1,jl
		x1_t(2,j) = x1_t(2,jl_ssc)
		x2_t(2,j) = x2_t(2,jl_ssc)

		x1_t(il,j) = x1_t(il,jl_ssc)
		x2_t(il,j) = x2_t(il,jl_ssc)

		x1_t(il/2+1,j) = x1_t(il/2+1,jl_ssc)
		x2_t(il/2+1,j) = x2_t(il/2+1,jl_ssc)
	end do

	end if	!if (bsc_flag .eq. 1)	then



!-------
!		time grid
!--

	return                                                            
	end                                                               


!~~~~~~
	subroutine o_ini_set_grid_step
!~~~~~~

	use o_grid_module
	use o_ssc_module
	use o_bsc_module

	implicit none

	real (kind=8) x1_pole_radian, x2_pole_radian, x2_0_radian
	real (kind=8) dx1_radian

	integer counter

!-------
!		parameters of the ocean ,earth and basin                            
!-------
	!pi	= 3.14159                                                        
	!pi180	= pi/180.                                                     
	!radius	= 0.64e7	!meters
	!radv	= 1./radius


!-------
!#1		compute parameters for ssc & bsc
!-------

		call o_ini_set_grid_z

!-------
!#2	compose united array of lengths of boxes
!-------



!-------
!#2.1	add spherical system of coordinates
!-------

c	print *,'#g001#',myrank,size

	if (ssc_flag .eq. 1)	then

		call o_ini_set_grid_ssc_step


c	print *,'#g002#',myrank,size

	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	if (j.le.jl_ssc) then
	dx1_nw_v (i,j) = dx1_nw_ssc_v (i,j) 
	dx1_ne_v (i,j) = dx1_ne_ssc_v (i,j) 
	dx2_en_v (i,j) = dx2_en_ssc_v (i,j) 
	dx2_es_v (i,j) = dx2_es_ssc_v (i,j) 
	end if
	end do
	end do


	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	if (j.le.jl_ssc) then
	dx1_nw_t (i,j) = dx1_nw_ssc_t (i,j) 
	dx1_ne_t (i,j) = dx1_ne_ssc_t (i,j) 
	dx2_en_t (i,j) = dx2_en_ssc_t (i,j) 
	dx2_es_t (i,j) = dx2_es_ssc_t (i,j) 
	end if
	end do
	end do


!----------------------
!	Deallocate memory
!----------------------

!	Ssc arrays

	deallocate ( dx1_nw_ssc_v )
	deallocate ( dx1_ne_ssc_v )
	deallocate ( dx1_nw_ssc_t )
	deallocate ( dx1_ne_ssc_t )

	deallocate ( dx2_en_ssc_v )
	deallocate ( dx2_es_ssc_v )
	deallocate ( dx2_en_ssc_t )
	deallocate ( dx2_es_ssc_t )



	end if	!if (ssc_flag .eq. 1)	then

c	print *,'#g004#',myrank,size


!-------
!#2.2	include bipolar system of coordinates
!-------

	if (bsc_flag .eq. 1) then

	x1_bsc_west_south = x1_ssc_west_south
	x2_bsc_west_south = x2_ssc_west_south + x2_ssc_size


c	print *,'#b001#',myrank,size

		call o_ini_set_grid_bsc_step



c	print *,'#b002#',myrank,size

!-------
!2.2.1.	lengths of lateral boundaries of v-cells as it is seen from ssc
!-------

	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	if (j.ge.jl_ssc) then

!-------
!2.2.1.1.	II quadrant of bsc from bsc-south pole to bsc-north pole
!-------

!	left line

	if (i.eq.1) then
	j_bsc = 1
	i_bsc = il_bsc-(j-jl_ssc)-1

	dx1_nw_v(i,j) = dx2_ws_bsc_v(i_bsc,j_bsc)
	dx1_ne_v(i,j) = dx2_wn_bsc_v(i_bsc,j_bsc)
	dx2_en_v(i,j) = dx1_nw_bsc_v(i_bsc,j_bsc)
	dx2_es_v(i,j) = dx1_ne_bsc_v(i_bsc,j_bsc)	
	end if


!2.2.1.2.	internal points

	if (i.ge.2. and .i.le.il/2) then
	j_bsc = i-1
	i_bsc = il_bsc-(j-jl_ssc)-1

	dx1_nw_v(i,j) = dx2_ws_bsc_v(i_bsc,j_bsc)
	dx1_ne_v(i,j) = dx2_wn_bsc_v(i_bsc,j_bsc)
	dx2_en_v(i,j) = dx1_nw_bsc_v(i_bsc,j_bsc)
	dx2_es_v(i,j) = dx1_ne_bsc_v(i_bsc,j_bsc)
	end if

!-------
!2.2.1.3.	I quadrant of bsc from bsc-south pole to bsc-north pole
!-------

!	internal points

	if (i.ge.il/2+1. and .i.le.il-1) then
	j_bsc = (jl_bsc-1) - (i-(il/2+1))
	i_bsc = 1 + (j-jl_ssc)

	dx1_nw_v(i,j) = dx2_en_bsc_v(i_bsc,j_bsc)
	dx1_ne_v(i,j) = dx2_es_bsc_v(i_bsc,j_bsc)
	dx2_en_v(i,j) = dx1_se_bsc_v(i_bsc,j_bsc)
	dx2_es_v(i,j) = dx1_sw_bsc_v(i_bsc,j_bsc)
	end if


!2.2.1.4.	right line

	if (i.eq.il) then
	j_bsc = 1
	i_bsc = 1 + (j-jl_ssc)

	dx1_nw_v(i,j) = dx2_en_bsc_v(i_bsc,j_bsc)
	dx1_ne_v(i,j) = dx2_es_bsc_v(i_bsc,j_bsc)
	dx2_en_v(i,j) = dx1_se_bsc_v(i_bsc,j_bsc)
	dx2_es_v(i,j) = dx1_sw_bsc_v(i_bsc,j_bsc)
	end if

	
	end if	!if (j.ge.jl_ssc) then

	end do
	end do





!-------
!2.2.2.	lengths of lateral boundaries of t-cells as it is seen from ssc
!-------


!-------
!2.2.2.1.	II quadrant of bsc from bsc-south pole to bsc-north pole
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1

	if (j.ge.jl_ssc) then


!	north-east corner, left line

	if (i.eq.1) then
	j_bsc = 1
	i_bsc = il_bsc-(j-jl_ssc)
	dx1_ne_t(i,j) = dx2_wn_bsc_t(i_bsc,j_bsc)
	dx2_en_t(i,j) = dx1_nw_bsc_t(i_bsc,j_bsc)
	end if


!2.2.2.1.1.	north-east corner, internal points

	if (i.ge.2. and .i.le.il/2) then
	j_bsc = i-1
	i_bsc = il_bsc-(j-jl_ssc)
	dx1_ne_t(i,j) = dx2_wn_bsc_t(i_bsc,j_bsc)
	dx2_en_t(i,j) = dx1_nw_bsc_t(i_bsc,j_bsc)
	end if

!2.2.2.2.	north-west line, internal points

	if (i.ge.3. and .i.le.il/2+1) then
	j_bsc = i-1
	i_bsc = il_bsc-(j-jl_ssc)
	dx1_nw_t(i,j) = dx2_ws_bsc_t(i_bsc,j_bsc)
	end if

!2.2.2.2.1.	north-west line, left line

	if (i.eq.2) then
	j_bsc = 2
	i_bsc = il_bsc-(j-jl_ssc)
	dx1_nw_t(i,j) = dx2_ws_bsc_t(i_bsc,j_bsc)
	end if



	if (j.ge.jl_ssc+1) then

!2.2.2.3.	east-south line, internal points

	if (i.ge.2. and .i.le.il/2) then
	j_bsc = i-1
	i_bsc = il_bsc-(j-jl_ssc)
	dx2_es_t(i,j) = dx1_ne_bsc_t(i_bsc,j_bsc)
	end if

!2.2.2.3.1.	east-south line, left line

	if (i.eq.1) then
	j_bsc = 1
	i_bsc = il_bsc-(j-jl_ssc)
	dx2_es_t(i,j) = dx1_ne_bsc_t(i_bsc,j_bsc)
	end if

	end if	!if (j.ge.jl_ssc+1) then

	end if	!if (j.ge.jl_ssc) then

	end do
	end do




!-------
!2.2.3.1.	I quadrant of bsc from bsc-south pole to bsc-north pole
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	
	if (j.ge.jl_ssc) then


!2.2.3.1.1.	north-east corner, internal points

	if (i.ge.il/2+1. and .i.le.il-1) then
	j_bsc = jl_bsc - (i-(il/2+1))
	i_bsc = 1 + (j-jl_ssc)
	dx1_ne_t(i,j) = dx2_es_bsc_t(i_bsc,j_bsc)
	dx2_en_t(i,j) = dx1_se_bsc_t(i_bsc,j_bsc)
	end if

!	north-east corner, left line
	if (i.eq.il) then
	j_bsc = 2
	i_bsc = 1 + (j-jl_ssc)
	dx1_ne_t(i,j) = dx2_es_bsc_t(i_bsc,j_bsc)
	dx2_en_t(i,j) = dx1_se_bsc_t(i_bsc,j_bsc)
	end if


!2.2.3.2.	north-west line, internal points

	if (i.ge.il/2+2. and .i.le.il) then
	j_bsc = jl_bsc - (i-(il/2+1))
	i_bsc = 1 + (j-jl_ssc)
	dx1_nw_t(i,j) = dx2_en_bsc_t(i_bsc,j_bsc)
	end if

!2.2.3.2.1.	north-west line, left line

	if (i.eq.il/2+1) then
	j_bsc = jl_bsc - 1
	i_bsc = 1 + (j-jl_ssc)
	dx1_nw_t(i,j) = dx2_en_bsc_t(i_bsc,j_bsc)
	end if

	if (j.ge.jl_ssc+1) then

!2.2.3.3.	east-south line, internal points

	if (i.ge.il/2+1. and .i.le.il-1) then
	j_bsc = jl_bsc - (i-(il/2+1))
	i_bsc = 1 + (j-jl_ssc)
	dx2_es_t(i,j) = dx1_sw_bsc_t(i_bsc,j_bsc)
	end if

!2.2.3.3.1.	east-south line, left line

	if (i.eq.il) then
	j_bsc = jl_bsc - 1
	i_bsc = 1 + (j-jl_ssc)
	dx2_es_t(i,j) = dx1_sw_bsc_t(i_bsc,j_bsc)
	end if
	
	end if	!if (j.ge.jl_ssc+1) then

	end if	!if (j.ge.jl_ssc) then
	
	end do
	end do


c----------------------
c	Deallocate memory
c----------------------
c	Bsc arrays

	deallocate (dx1_nw_bsc_v )
	deallocate (dx1_ne_bsc_v )
	deallocate (dx1_sw_bsc_v )
	deallocate (dx1_se_bsc_v )

	deallocate (dx2_en_bsc_v )
	deallocate (dx2_es_bsc_v )
	deallocate (dx2_wn_bsc_v )
	deallocate (dx2_ws_bsc_v )

	deallocate (dx1_ne_bsc_t )
	deallocate (dx1_nw_bsc_t )
	deallocate (dx1_se_bsc_t )
	deallocate (dx1_sw_bsc_t )

	deallocate (dx2_en_bsc_t )
	deallocate (dx2_wn_bsc_t )
	deallocate (dx2_es_bsc_t )
	deallocate (dx2_ws_bsc_t )


	end if	!if (bsc_flag .eq. 1)	then

	return                                                            
	end                                                               

