
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	initiate
!	physical and numerical scheme constants
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

!===================================================
! Define physical and scheme parameters of the model
!===================================================

	subroutine o_ini_set_physics
	use o_grid_module
	use o_d_brcl_module
	use o_d_vmc_1_module
	implicit none

!=========================================
! Compute coriolis parameter
!=========================================
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		coriolis(i,j) = 2.*omega*sin(x2_v(i,j)*pi180)
	end do
	end do


!=========================================
! Explicit or implicit Coriolis term
!	coriolis_alfa*U('l+1') + 
!	coriolis_gamma*U('l') + 
!	coriolis_betta*U('l-1')
!=========================================
!	coriolis_alfa = 1.0 !1.0
!	coriolis_betta = 0.0
!	coriolis_gamma = 0.0 !0.0


	nloop = 20
	iprmix = 0

	!===================================================================
!		Parameters to define vertical eddy diffusion
!	v_mix_mod=0	vertical mixing coeff. are constant
!		Km = c1Amv
!		Kh = c1Ahv
!	v_mix_mod=1:	munk-anderson parameterization
!		km(i,k,j)=c1Amv / (1.+c3Amv*riz(i,k))**nAmv + c2Amv(k)
!		kh(i,k)=c1Ahv/(1.+c3Ahv*rI)**nAhv+c2Ahv(k)
!	v_mix_mod=2:	Mellor-Yamada level 2.5 turbulent closure model
!	v_mix_mod=3:	kpp
!===================================================================
	if(v_mix_mod==0) mix_unstable_levels = 1
	if(v_mix_mod==1) mix_unstable_levels = 1
	if(v_mix_mod==2) mix_unstable_levels = 0
	if(v_mix_mod==3) mix_unstable_levels = 0
	if(v_mix_mod==4) mix_unstable_levels = 1
	
!==============================================
! Constants for munk-anderson parameterization
!==============================================
	if(v_mix_mod==1 .or. v_mix_mod==4) then
		c3Amv = 1.
		namv  = 1		!see correction in munk1.f

		c3Ahv = 1.
		nahv  = 1		!see correction in munk1.f

		dvc  = .01		!m/c	???
	end if

	return                                                            
	end                                                               
