
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	masking output arrays (land = missvalue)
!
! authors: 
!	2017.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!
! reference:
!	Ibrayev R.A., Kalnitskiy L.Yu., Ushakov K.V., 2017. 
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------


!~~~~~~~
	subroutine o_mask_land_DG(land_value)
!~~~~~~~

!--------------------------------------
!	The following procedures used for masking arrays in CP, DG files
!	2016.10.15
!	author: Ibrayev, Rashit
!-------

!-------
!	this subroutine is used for masking solution domain when writing to DG file
!	put 'land_value' to land points 
!	in case of writing data to DG file 'land_value = -99999' ==> that is convenient for plotting with 'ncview', 'PyNGL'
!	the subroutine also used before subroutine o_driver_main_step, because some arrays masked by -99999 at previous time-step
!	can be used at current time-step 
!	in that case 'land_value = 0' ==> in equations by default functions at land points are ==0
!-------
!	ATTENTION
!	masked array should be the same arrays as in subroutine o_ini_reg_data(this) for ACTION_SAVE_DG
!-------

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_d_brtr_sngl_module
	use ice_module	
	implicit none
	real (kind=4) land_value

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kt(i,j)+1,kl
	do m=1,ml
	t_c(k,i,j,m) = land_value
	end do
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,kl
	u_c(k,i,j) = land_value
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,kl
	v_c(k,i,j) = land_value
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kv(i,j) == 0) then
!	u_bt_a_f(i,j) = land_value
!	v_bt_a_f(i,j) = land_value
	end if	
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then
	w_surf_c(i,j) = land_value
!	h_bt_a_f(i,j) = land_value
	ssl_c	(i,j) = land_value
	hice	(i,j) = land_value
	aice	(i,j) = land_value
	end if
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,2
!	dx3_v_l(k,i,j) = land_value
	end do
	end do
	end do

!-------
!	These arrays are additional to DG
!-------
	ice_ocn_ssr_dg	= ice_ocn_ssr
	ice_ocn_hea_dg	= ice_ocn_heat
	ice_ocn_sal_dg	= ice_ocn_salt
	ice_ocn_stx_dg	= ice_ocn_stress_x_v
	ice_ocn_sty_dg	= ice_ocn_stress_y_v
	ice_ocn_wat_dg	= ice_ocn_water
	ice_ocn_aic_dg	= ice_ocn_aice

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then

	ice_ocn_ssr_dg	(i,j) = land_value
	ice_ocn_hea_dg	(i,j) = land_value
	ice_ocn_sal_dg	(i,j) = land_value
	ice_ocn_stx_dg	(i,j) = land_value
	ice_ocn_sty_dg	(i,j) = land_value
	ice_ocn_wat_dg	(i,j) = land_value
	ice_ocn_aic_dg	(i,j) = land_value
	end if
	end do
	end do

	ocn_ice_frz_dg	= ocn_ice_frzmlt
	ocn_ice_tlx_dg	= ocn_ice_tltx
	ocn_ice_tly_dg	= ocn_ice_tlty
	ocn_ice_uoc_dg	= ocn_ice_uocn
	ocn_ice_voc_dg	= ocn_ice_vocn
	ocn_ice_sst_dg	= ocn_ice_sst
	ocn_ice_sss_dg	= ocn_ice_sss

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then
	ocn_ice_frz_dg		(i,j) = land_value
	ocn_ice_tlx_dg		(i,j) = land_value
	ocn_ice_tly_dg		(i,j) = land_value
	ocn_ice_uoc_dg		(i,j) = land_value
	ocn_ice_voc_dg		(i,j) = land_value
	ocn_ice_sst_dg		(i,j) = land_value
	ocn_ice_sss_dg		(i,j) = land_value
	end if
	end do
	end do



	if(myrank .eq. 1) write(*,*) 'DG files filled with land_value=',land_value

	return
	end


!~~~~~~~
	subroutine o_mask_land_CP(land_value)
!~~~~~~~

!-------
!	this subroutine is used for masking solution domain when writing/reading to/from CP file
!	put 'land_value' to land points 
!	in case of reading data from CP file 'land_value = 0' ==> in equations by default functions at land points are ==0
!	in case of writing data to CP file 'land_value = -99999' ==> that is convenient for plotting with 'ncview', 'PyNGL'
!-------

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_d_brtr_sngl_module
	use ice_module	
	implicit none
	real (kind=4) land_value

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kt(i,j)+1,kl
	do m=1,ml
	t_c(k,i,j,m) = land_value
	end do
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,kl
	u_c(k,i,j) = land_value
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,kl
	v_c(k,i,j) = land_value
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kv(i,j) == 0) then
	u_bt_a_f(i,j) = land_value
	v_bt_a_f(i,j) = land_value
	end if	
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then
	w_surf_c(i,j) = land_value
	h_bt_a_f(i,j) = land_value
	ssl_c	(i,j) = land_value
	hice	(i,j) = land_value
	aice	(i,j) = land_value
	end if
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=kv(i,j)+1,2
	dx3_v_l(k,i,j) = land_value
	end do
	end do
	end do

!-------
!	These arrays are additional to CP
!	Arrays from CICE without additional efforts cann't be masked (One have to interfere in c_main.f90).
!	that is why we have to use additinal arrays
!-------
	ice_ocn_ssr_dg	= ice_ocn_ssr
	ice_ocn_hea_dg	= ice_ocn_heat
	ice_ocn_sal_dg	= ice_ocn_salt
	ice_ocn_stx_dg	= ice_ocn_stress_x_v
	ice_ocn_sty_dg	= ice_ocn_stress_y_v
	ice_ocn_wat_dg	= ice_ocn_water
	ice_ocn_aic_dg	= ice_ocn_aice

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then
	ice_ocn_ssr_dg	(i,j) = land_value
	ice_ocn_hea_dg	(i,j) = land_value
	ice_ocn_sal_dg	(i,j) = land_value
	ice_ocn_stx_dg	(i,j) = land_value
	ice_ocn_sty_dg	(i,j) = land_value
	ice_ocn_wat_dg	(i,j) = land_value
	ice_ocn_aic_dg	(i,j) = land_value
	end if
	end do
	end do

	ocn_ice_frz_dg	= ocn_ice_frzmlt
	ocn_ice_tlx_dg	= ocn_ice_tltx
	ocn_ice_tly_dg	= ocn_ice_tlty
	ocn_ice_uoc_dg	= ocn_ice_uocn
	ocn_ice_voc_dg	= ocn_ice_vocn
	ocn_ice_sst_dg	= ocn_ice_sst
	ocn_ice_sss_dg	= ocn_ice_sss

	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	if(kt(i,j) == 0) then
	ocn_ice_frz_dg		(i,j) = land_value
	ocn_ice_tlx_dg		(i,j) = land_value
	ocn_ice_tly_dg		(i,j) = land_value
	ocn_ice_uoc_dg		(i,j) = land_value
	ocn_ice_voc_dg		(i,j) = land_value
	ocn_ice_sst_dg		(i,j) = land_value
	ocn_ice_sss_dg		(i,j) = land_value
	end if
	end do
	end do


	if(myrank .eq. 1) write(*,*) 'CP files filled with land_value=',land_value

	return
	end


