
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!
! reference:
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

!========================================================
! This file contains 2 exotic subroutines:
! -for fast brtr_sweq(explicit using u_bt, v_bt, h_bt)
! -for dx-boundaries exchanges(old style of 8 neighbours)
!========================================================

!==================================================================================
! Special halo update subroutine for BRTR function in fast SWEQ, without arguments 
! and allocates. It uses u_bt_f_ad, v_bt_f_ad, h_bt_f_ad directly.
!==================================================================================

	subroutine o_p_halo_update_BRTR

	use o_grid_module
	use o_d_brtr_wide_module
	use comp_halo_update_module
	use comp_module
	use shared_module

	implicit none

	integer counter

!=========================================================
! Create sending-arrays for west-east(+bsc) case 
!=========================================================	
	counter = 1
	do j = jsouth(myrank),jnorth(myrank)
	do i = 1, comp_halo_wide
		vect_west_send4(counter) = u_bt_f_ad(iwest(myrank)+i-1,j)
		counter = counter + 1
		vect_west_send4(counter) = v_bt_f_ad(iwest(myrank)+i-1,j)
		counter = counter + 1
	end do
	end do
	
	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then

		do j = jsouth(myrank),jnorth(myrank)
		do i = 1, comp_halo_wide+1
			vect_west_send4(counter) = h_bt_f_ad(iwest(myrank)+i-1,j)
			counter = counter + 1
		end do
		end do
	else
		do j = jsouth(myrank),jnorth(myrank)
		do i = 1, comp_halo_wide
			vect_west_send4(counter) = h_bt_f_ad(iwest(myrank)+i-1,j)
			counter = counter + 1
		end do
		end do
	end if	

	counter = 1
	do j = jsouth(myrank),jnorth(myrank)
	do i = 1, comp_halo_wide
		vect_east_send4(counter) = u_bt_f_ad(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		vect_east_send4(counter) = v_bt_f_ad(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		vect_east_send4(counter) = h_bt_f_ad(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
	end do
	end do
	
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_brtr_lon(:), ierr)
	call MPI_WAITALL  (4, request_brtr_lon(:), status_brtr, ierr)

!=========================================================
! Copy received belts to work arrays
!=========================================================
	if(iwest(myrank) > 2 .OR. comp_i_cycle) then 
		
		counter = 1
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide
			u_bt_f_ad(iwest(myrank)-comp_halo_wide-1+i,j) = vect_west_recv4(counter)
			counter = counter + 1
			v_bt_f_ad(iwest(myrank)-comp_halo_wide-1+i,j) = vect_west_recv4(counter)
			counter = counter + 1
			h_bt_f_ad(iwest(myrank)-comp_halo_wide-1+i,j) = vect_west_recv4(counter)
			counter = counter + 1
		end do
		end do
	end if

	if(ieast(myrank) < il-1 .OR. comp_i_cycle) then
		
		counter = 1
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide
			u_bt_f_ad(ieast(myrank)+i,j) = vect_east_recv4(counter)
			counter = counter + 1
			v_bt_f_ad(ieast(myrank)+i,j) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
	end if
	
	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then
	
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide+1
			h_bt_f_ad(ieast(myrank)+i,j) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
		
	else if(ieast(myrank) < il-1 .OR. comp_i_cycle) then
		
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide
			h_bt_f_ad(ieast(myrank)+i,j) = vect_east_recv4(counter)
			counter = counter + 1
		end do
		end do
	end if
	
!=========================================================
! Now we have belts for west and east and we can get wide
! halo belts for south and north case(without diagonal)
!=========================================================

!=========================================================
! Create sending-arrays for wide south-north(+bsc) case 
!=========================================================	
	counter = 1
	do j = 1, comp_halo_wide
	do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
		vect_south_send4(counter) = u_bt_f_ad(i,jsouth(myrank)+j-1)
		counter = counter + 1
		vect_south_send4(counter) = v_bt_f_ad(i,jsouth(myrank)+j-1)
		counter = counter + 1
		vect_south_send4(counter) = h_bt_f_ad(i,jsouth(myrank)+j-1)
		counter = counter + 1
	end do
	end do
	
	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			vect_north_send4(counter) = u_bt_f_ad(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
			vect_north_send4(counter) = v_bt_f_ad(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
		end do
		end do
	
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide+1, ieast(myrank) + comp_halo_wide+1
			vect_north_send4(counter) = h_bt_f_ad(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
		end do
		end do	
	else
		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			vect_north_send4(counter) = u_bt_f_ad(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
			vect_north_send4(counter) = v_bt_f_ad(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
			vect_north_send4(counter) = h_bt_f_ad(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
		end do
		end do
	end if
		
!=========================================================
! Start INIT-operations
!=========================================================	
	call MPI_STARTALL (4, request_brtr_lat(:), ierr)
	call MPI_WAITALL  (4, request_brtr_lat(:), status_brtr, ierr)
	
!=========================================================
! Receive wide south-north belts 
!=========================================================
	if(jsouth(myrank) > 2) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			u_bt_f_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = vect_south_recv4(counter)
			counter = counter + 1
			v_bt_f_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = vect_south_recv4(counter)
			counter = counter + 1
			h_bt_f_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = vect_south_recv4(counter)
			counter = counter + 1
		end do
		end do
	end if
	
	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			u_bt_f_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = -vect_north_recv4(counter)
			counter = counter + 1
			v_bt_f_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = -vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			h_bt_f_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
		
	else if(jnorth(myrank) < jl-1) then
		
		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank) - comp_halo_wide, ieast(myrank) + comp_halo_wide
			u_bt_f_ad(i,jnorth(myrank)+j) = vect_north_recv4(counter)
			counter = counter + 1
			v_bt_f_ad(i,jnorth(myrank)+j) = vect_north_recv4(counter)
			counter = counter + 1
			h_bt_f_ad(i,jnorth(myrank)+j) = vect_north_recv4(counter)
			counter = counter + 1
		end do
		end do
	end if
	
	return
	end

	
!=========================================================================================================================================================================================

!================================================
! Initialization subroutine for fast SWEQ exhange
!================================================

	subroutine o_p_halo_update_BRTR_ini
	
	use comp_halo_update_module
	use shared_module
	use comp_module

	implicit none

	call MPI_SEND_INIT(vect_west_send4,							&
			(comp_halo_wide+1)*(cmp % j_loc_size)*3,				&
			MPI_REAL,				 				&
			comp_westrank, 								&
			201,									&
			comm_local, 								&
			request_brtr_lon(1), ierr)
		
	call MPI_SEND_INIT(vect_east_send4,							&
			comp_halo_wide*(cmp % j_loc_size)*3,					&
			MPI_REAL, 								&
			comp_eastrank, 								&
			202,									&
			comm_local, 								&
			request_brtr_lon(2), ierr)
					
	call MPI_RECV_INIT(vect_east_recv4, 							&
			(comp_halo_wide+1)*(cmp % j_loc_size)*3,				&
			MPI_REAL,				 				&
			comp_eastrank, 								&
			MPI_ANY_TAG,								&
			comm_local, 								&
			request_brtr_lon(3), ierr)
			
	call MPI_RECV_INIT(vect_west_recv4, 							&
			comp_halo_wide*(cmp % j_loc_size)*3,					&
			MPI_REAL,				 				&
			comp_westrank, 								&
			MPI_ANY_TAG,								&
			comm_local, 								&
			request_brtr_lon(4), ierr)					
										
	call MPI_SEND_INIT(vect_south_send4,							&
			comp_halo_wide*(cmp % i_loc_size + 2*comp_halo_wide)*3,			&
			MPI_REAL,								&
			comp_southrank, 							&
			203,									&
			comm_local, 								&
			request_brtr_lat(1), ierr)

	call MPI_SEND_INIT(vect_north_send4,							&
			comp_halo_wide*(cmp % i_loc_size + 2*comp_halo_wide)*3,			&
			MPI_REAL, 								&
			comp_northrank, 							&
			204,									&
			comm_local, 								&
			request_brtr_lat(2), ierr)

	call MPI_RECV_INIT(vect_north_recv4, 							&
			comp_halo_wide*(cmp % i_loc_size + 2*comp_halo_wide)*3,			&
			MPI_REAL,								&
			comp_northrank, 							&
			MPI_ANY_TAG,								&
			comm_local, 								&
			request_brtr_lat(3), ierr)

	call MPI_RECV_INIT(vect_south_recv4, 							&
			comp_halo_wide*(cmp % i_loc_size + 2*comp_halo_wide)*3,			&
			MPI_REAL, 								&
			comp_southrank, 							&
			MPI_ANY_TAG,								&
			comm_local, 								&
			request_brtr_lat(4), ierr)
	
	return
	end

!=========================================================================================================================================================================================
	
!============================================================
! Special subroutine of halo update for exotic dx_*** updates
!============================================================

	subroutine o_p_halo_update_DX

	use o_grid_module
	use o_d_brtr_wide_module
	use comp_halo_update_module
	use comp_module
	use shared_module

	implicit none
	
	real (kind=8) :: dx_south_send8(8*cmp % i_loc_size*comp_halo_wide)
	real (kind=8) :: dx_north_send8(8*cmp % i_loc_size*comp_halo_wide)
	real (kind=8) :: dx_west_send8	(8*cmp % j_loc_size*comp_halo_wide)
	real (kind=8) :: dx_east_send8	(8*cmp % j_loc_size*comp_halo_wide)
	real (kind=8) :: dx_se_send8	(8*comp_halo_wide*comp_halo_wide)
	real (kind=8) :: dx_sw_send8	(8*comp_halo_wide*comp_halo_wide)
	real (kind=8) :: dx_nw_send8	(8*comp_halo_wide*comp_halo_wide)
	real (kind=8) :: dx_ne_send8	(8*comp_halo_wide*comp_halo_wide)

	real (kind=8) :: dx_south_recv8(8*cmp % i_loc_size*comp_halo_wide)
	real (kind=8) :: dx_north_recv8(8*cmp % i_loc_size*comp_halo_wide)
	real (kind=8) :: dx_west_recv8	(8*cmp % j_loc_size*comp_halo_wide)
	real (kind=8) :: dx_east_recv8	(8*cmp % j_loc_size*comp_halo_wide)
	real (kind=8) :: dx_se_recv8	(8*comp_halo_wide*comp_halo_wide)	
	real (kind=8) :: dx_sw_recv8	(8*comp_halo_wide*comp_halo_wide)
	real (kind=8) :: dx_nw_recv8	(8*comp_halo_wide*comp_halo_wide)
	real (kind=8) :: dx_ne_recv8	(8*comp_halo_wide*comp_halo_wide)
	
	integer status_dx(MPI_STATUS_SIZE, 16)
	integer request_dx(16)
	integer counter

!======================
! Create sending-arrays
!======================
	counter = 1
	do j = 1, comp_halo_wide
	do i = iwest(myrank), ieast(myrank)
		dx_south_send8(counter) = dx1_ne_v_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx1_nw_v_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx2_en_v_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx2_es_v_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx1_ne_t_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx1_nw_t_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx2_en_t_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_south_send8(counter) = dx2_es_t_ad	(i,jsouth(myrank)+j-1)
		counter = counter + 1
	end do
	end do

	counter = 1
	do j = 1, comp_halo_wide
	do i = iwest(myrank), ieast(myrank)
		dx_north_send8(counter) = dx1_ne_v_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx1_nw_v_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx2_en_v_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx2_es_v_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx1_ne_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx1_nw_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx2_en_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_north_send8(counter) = dx2_es_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
	end do
	end do

	counter = 1
	do j = jsouth(myrank),jnorth(myrank)
	do i = 1, comp_halo_wide
		dx_west_send8(counter) = dx1_ne_v_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx1_nw_v_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx2_en_v_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx2_es_v_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx1_ne_t_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx1_nw_t_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx2_en_t_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
		dx_west_send8(counter) = dx2_es_t_ad	(iwest(myrank)+i-1,j)
		counter = counter + 1
	end do
	end do

	counter = 1
	do j = jsouth(myrank),jnorth(myrank)
	do i = 1, comp_halo_wide
		dx_east_send8(counter) = dx1_ne_v_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx1_nw_v_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx2_en_v_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx2_es_v_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx1_ne_t_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx1_nw_t_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx2_en_t_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
		dx_east_send8(counter) = dx2_es_t_ad	(ieast(myrank)-comp_halo_wide+i,j)
		counter = counter + 1
	end do
	end do

	counter = 1
	do j = 1, comp_halo_wide
	do i = 1, comp_halo_wide
		dx_sw_send8(counter) = dx1_ne_v_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx1_nw_v_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx2_en_v_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx2_es_v_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx1_ne_t_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx1_nw_t_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx2_en_t_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_sw_send8(counter) = dx2_es_t_ad	(iwest(myrank)+i-1,jsouth(myrank)+j-1)
		counter = counter + 1
	end do
	end do

	counter = 1
	do j = 1, comp_halo_wide
	do i = 1, comp_halo_wide
		dx_se_send8(counter) = dx1_ne_v_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx1_nw_v_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx2_en_v_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx2_es_v_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx1_ne_t_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx1_nw_t_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx2_en_t_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1
		dx_se_send8(counter) = dx2_es_t_ad	(ieast(myrank)-comp_halo_wide+i,jsouth(myrank)+j-1)
		counter = counter + 1			
	end do
	end do

	counter = 1
	do j = 1, comp_halo_wide
	do i = 1, comp_halo_wide
		dx_nw_send8(counter) = dx1_ne_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx1_nw_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx2_en_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx2_es_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx1_ne_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx1_nw_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx2_en_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_nw_send8(counter) = dx2_es_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1			
	end do
	end do

	counter = 1
	do j = 1, comp_halo_wide
	do i = 1, comp_halo_wide
		dx_ne_send8(counter) = dx1_ne_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx1_nw_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx2_en_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx2_es_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx1_ne_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx1_nw_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx2_en_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1
		dx_ne_send8(counter) = dx2_es_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
		counter = counter + 1			
	end do
	end do

!==============================================================
! Bipolar exchange case if (comp_bsc_grid). Replace north,
! north-west and north-east sending belts.
!==============================================================

	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then
!======
! North
!======
		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank), ieast(myrank)
			dx_north_send8(counter) = dx1_ne_v_ad	(i,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1
			dx_north_send8(counter) = dx1_nw_v_ad	(i,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1

			dx_north_send8(counter) = dx2_en_v_ad	(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
			dx_north_send8(counter) = dx2_es_v_ad	(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1

			dx_north_send8(counter) = dx1_ne_t_ad	(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
			dx_north_send8(counter) = dx1_nw_t_ad	(i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1

			dx_north_send8(counter) = dx2_en_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
			dx_north_send8(counter) = dx2_es_t_ad	(i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
		end do
		end do
!===========
! North-west
!===========
		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx_nw_send8(counter) = dx1_ne_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1
			dx_nw_send8(counter) = dx1_nw_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide-1
			dx_nw_send8(counter) = dx2_en_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
			dx_nw_send8(counter) = dx2_es_v_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide+1
			dx_nw_send8(counter) = dx1_ne_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j-1)
			counter = counter + 1
			dx_nw_send8(counter) = dx1_nw_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j-1)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx_nw_send8(counter) = dx2_en_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
			dx_nw_send8(counter) = dx2_es_t_ad	(iwest(myrank)+i-1,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
		end do
		end do
!===========
! North-east
!===========
		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx_ne_send8(counter) = dx1_ne_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1
			dx_ne_send8(counter) = dx1_nw_v_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide-2+j)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide+1
			dx_ne_send8(counter) = dx2_en_v_ad	(ieast(myrank)-comp_halo_wide-1+i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
			dx_ne_send8(counter) = dx2_es_v_ad	(ieast(myrank)-comp_halo_wide-1+i,jnorth(myrank)-comp_halo_wide-1+j)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide-1
			dx_ne_send8(counter) = dx1_ne_t_ad	(ieast(myrank)-comp_halo_wide+i+1,jnorth(myrank)-comp_halo_wide+j-1)
			counter = counter + 1
			dx_ne_send8(counter) = dx1_nw_t_ad	(ieast(myrank)-comp_halo_wide+i+1,jnorth(myrank)-comp_halo_wide+j-1)
			counter = counter + 1
		end do
		end do

		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx_ne_send8(counter) = dx2_en_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
			dx_ne_send8(counter) = dx2_es_t_ad	(ieast(myrank)-comp_halo_wide+i,jnorth(myrank)-comp_halo_wide+j)
			counter = counter + 1
		end do
		end do

	end if

!========================================
! Send and eceive MPI-operations
!========================================
	call MPI_ISEND(dx_south_send8,				&
			comp_halo_wide*(cmp % i_loc_size)*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_southrank, 			&
     			201, 					&
			comm_local, request_dx(1), ierr)

	call MPI_ISEND(dx_north_send8,				&
     			comp_halo_wide*(cmp % i_loc_size)*8,	&
			MPI_DOUBLE_PRECISION, 			&
     			comp_northrank, 			&
     			202,					&
     			comm_local, request_dx(2), ierr)

	call MPI_ISEND(dx_west_send8,				&
     			comp_halo_wide*(cmp % j_loc_size)*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_westrank, 				&
     			203,					&
     			comm_local, request_dx(3), ierr)
	
	call MPI_ISEND(dx_east_send8,				&
     			comp_halo_wide*(cmp % j_loc_size)*8,	&
			MPI_DOUBLE_PRECISION, 			&
     			comp_eastrank, 				&
     			204,					&
     			comm_local, request_dx(4),ierr)

	call MPI_ISEND(dx_sw_send8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_swrank, 				&
     			205,					&
     			comm_local, request_dx(5), ierr)

	call MPI_ISEND(dx_se_send8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_serank, 				&
     			206,					&
     			comm_local, request_dx(6), ierr)		

	call MPI_ISEND(dx_nw_send8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,		 	&
     			comp_nwrank, 				&
     			207,					&
     			comm_local, request_dx(7), ierr)

	call MPI_ISEND(dx_ne_send8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_nerank, 				&
     			208,					&
     			comm_local, request_dx(8), ierr)

	call MPI_IRECV(dx_north_recv8, 			&
     			comp_halo_wide*(cmp % i_loc_size)*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_northrank, 			&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(9), ierr)

	call MPI_IRECV(dx_south_recv8, 			&
     			comp_halo_wide*(cmp % i_loc_size)*8,	&
			MPI_DOUBLE_PRECISION, 			&
     			comp_southrank, 			&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(10), ierr)
	
	call MPI_IRECV(dx_east_recv8, 				&
     			comp_halo_wide*(cmp % j_loc_size)*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_eastrank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(11), ierr)
		
	call MPI_IRECV(dx_west_recv8, 				&
     			comp_halo_wide*(cmp % j_loc_size)*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_westrank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(12), ierr)

	call MPI_IRECV(dx_ne_recv8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_nerank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(13), ierr)

	call MPI_IRECV(dx_nw_recv8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION,			&
     			comp_nwrank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(14), ierr)

	call MPI_IRECV(dx_se_recv8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION, 			&
     			comp_serank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(15), ierr)

	call MPI_IRECV(dx_sw_recv8,				&
			comp_halo_wide*comp_halo_wide*8,	&
			MPI_DOUBLE_PRECISION, 			&
     			comp_swrank, 				&
     			MPI_ANY_TAG,				&
     			comm_local, request_dx(16), ierr)
     			
	call MPI_WAITALL  (16, request_dx(:), status_dx , ierr)

!===================================
! Copy received arrays to halo belts
!===================================

! South belt
	if(jsouth(myrank) > 2) then
		
		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank), ieast(myrank)
			dx1_ne_v_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(i,jsouth(myrank)-comp_halo_wide-1+j) = dx_south_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! North belt
	if(jnorth(myrank) < jl-1) then
		
		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank), ieast(myrank)
			dx1_ne_v_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(i,jnorth(myrank)+j) = dx_north_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! West belt
	if(iwest(myrank) > 2 .OR. comp_i_cycle) then 
		
		counter = 1
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(iwest(myrank)-comp_halo_wide-1+i,j) = dx_west_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! East belt
	if(ieast(myrank) < il-1 .OR. comp_i_cycle) then
		
		counter = 1
		do j = jsouth(myrank), jnorth(myrank)
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(ieast(myrank)+i,j) = dx_east_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! South-west belt
	if(jsouth(myrank) > 2 .AND. (iwest(myrank) > 2 .OR. comp_i_cycle)) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(iwest(myrank)-comp_halo_wide-1+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_sw_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! South-east belt
	if(jsouth(myrank) > 2 .AND. (ieast(myrank) < il-1 .OR. comp_i_cycle)) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(ieast(myrank)+i,jsouth(myrank)-comp_halo_wide-1+j) = dx_se_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! North-west belt	
	if(jnorth(myrank) < jl-1 .AND. (iwest(myrank) > 2 .OR. comp_i_cycle)) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(iwest(myrank)-comp_halo_wide-1+i,jnorth(myrank)+j) = dx_nw_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

! North-east belt	
	if(jnorth(myrank) < jl-1 .AND. (ieast(myrank) < il-1 .OR. comp_i_cycle)) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = 1, comp_halo_wide
			dx1_ne_v_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx1_nw_v_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx2_es_v_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx1_nw_t_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
			dx2_es_t_ad(ieast(myrank)+i,jnorth(myrank)+j) = dx_ne_recv8(counter)
			counter = counter + 1
		end do
		end do

	end if

!==========================================================
! Bipolar exchange case if (comp_bsc_grid). Receive north,
! north-west and north-east belts. Note, that funcrions
! are mirrow-replaced(nw->ne, ne->nw etc).
!==========================================================

!======
! North
!======
	if(jnorth(myrank) == jl-1 .AND. comp_bsc_grid) then

		counter = 1
		do j = 1, comp_halo_wide
		do i = iwest(myrank), ieast(myrank)
			dx1_nw_v_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1
			dx1_ne_v_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1

			dx2_es_v_ad(iwest(myrank)+(ieast(myrank)-i)-1,jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_en_v_ad(iwest(myrank)+(ieast(myrank)-i)-1,jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1

			dx1_nw_t_ad(iwest(myrank)+(ieast(myrank)-i)+1,jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1
			dx1_ne_t_ad(iwest(myrank)+(ieast(myrank)-i)+1,jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1

			dx2_es_t_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1
			dx2_en_t_ad(iwest(myrank)+(ieast(myrank)-i),jnorth(myrank)+comp_halo_wide+1-j) = dx_north_recv8(counter)
			counter = counter + 1
		end do
		end do
!===========
! North-west
!===========
		if(iwest(myrank) /= 2) then

			counter = 1
			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide
				dx1_nw_v_ad(iwest(myrank)-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
				dx1_ne_v_ad(iwest(myrank)-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide-1
				dx2_es_v_ad(iwest(myrank)-i-1,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
				dx2_en_v_ad(iwest(myrank)-i-1,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide+1
				dx1_nw_t_ad(iwest(myrank)+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
				dx1_ne_t_ad(iwest(myrank)+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide
				dx2_es_t_ad(iwest(myrank)-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
				dx2_en_t_ad(iwest(myrank)-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_nw_recv8(counter)
				counter = counter + 1
			end do
			end do

		end if
!===========
! North-east
!===========
		if(ieast(myrank) /= il-1) then

			counter = 1
			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide
				dx1_nw_v_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
				dx1_ne_v_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide+1
				dx2_es_v_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
				dx2_en_v_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide-1
				dx1_nw_t_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
				dx1_ne_t_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
			end do
			end do

			do j = 1, comp_halo_wide
			do i = 1, comp_halo_wide
				dx2_es_t_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
				dx2_en_t_ad(ieast(myrank)+comp_halo_wide+1-i,jnorth(myrank)+comp_halo_wide+1-j) = dx_ne_recv8(counter)
				counter = counter + 1
			end do
			end do

		end if

	end if

	return
	end
