#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	second order momentum scheme for tracers advection
!
! authors: 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Prather, M. J. (1986), Numerical advection by conservation of second-order moments, J. Geophys. Res., 91(D6), 6671–6681, doi:10.1029/JD091iD06p06671.
!--------------------------------------

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	module o_som_module
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	save
		!moments
		real(kind=main_kind), allocatable :: S0(:,:,:,:),SX(:,:,:,:),SXX(:,:,:,:),SY(:,:,:,:),SYY(:,:,:,:),SXY(:,:,:,:),SZ(:,:,:,:),SZZ(:,:,:,:),SXZ(:,:,:,:),SYZ(:,:,:,:),SM(:,:,:)
		real(kind=main_kind) alf,alfq,alf1,alf1q,temptm
		real(kind=main_kind), allocatable :: mask2(:,:,:,:)	!mask - where leave 2nd order and where drop to 1st order moments
		integer ip1,jp1,kp1
	end module o_som_module
	
	subroutine allocate_o_som_module
		use o_grid_module
		use o_som_module
		implicit none
	
		allocate (S0 (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SX (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SXX(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SY (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SYY(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SXY(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SZ (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SZZ(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SXZ(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SYZ(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))
		allocate (SM (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl   ))		
		allocate (mask2 (iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl,ml))		
		
		sx=0.
		sxx=0.
		sy=0.
		syy=0.
		sxy=0.
		sz=0.
		szz=0.
		sxz=0.
		syz=0.
		sm=0.
		s0=0.
		if(myrank==1) print *, 'Starting SOM2'
	end
		
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine o_td_scalar_advection_som(u0,v0,t0,dx3_t0,dx3_v0,w0,ctime)
!	(integration of advection term of scalar equation)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_som_module
	implicit none
	
	integer k1,k2
	real(kind=main_kind),dimension (kl,	iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1) u0,v0
	real(kind=main_kind),dimension (kl+1,	iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1) dx3_t0,dx3_v0,w0
	real(kind=main_kind),dimension (kl,	iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,ml) t0
	
	!recalculate zero-order moments at each step
	do i=iwest(myrank)-1,ieast(myrank)+1
	do j=jsouth(myrank)-1,jnorth(myrank)+1
	do k=1,kt(i,j)
		sm(i,j,k)=area_tbox(i,j)*dx3_t0(k+1,i,j)*ct3(k,i,j)			!t-box volume
		s0(i,j,k,:)=t0(k,i,j,:)*sm(i,j,k)					!tracer content in the box
	end do
	end do
	end do
	
	call advect_som_x(u0,dx3_v0,ctime)	
	call advect_som_y(v0,dx3_v0,ctime)
	call advect_som_z(w0,ctime)

	!---------Monotonizator---------
	mask2=1.
	do m=1,ml
		call exchange_moment(s0(:,:,:,m),1)
		
		do i=iwest(myrank),ieast(myrank)
		do j=jsouth(myrank),jnorth(myrank)
			do k=1,kt(i,j)
				if((s0(i,j,k,m)-s0(i-1,j,k,m))*(s0(i,j,k,m)-s0(i+1,j,k,m))*cv(k,i-1,j)*cv(k,i+1,j).gt.0) mask2(i-1:i+1,j,k,m)=0.
				if((s0(i,j,k,m)-s0(i,j-1,k,m))*(s0(i,j,k,m)-s0(i,j+1,k,m))*cv(k,i,j-1)*cv(k,i,j+1).gt.0) mask2(i,j-1:j+1,k,m)=0.
			end do
			
			do k=2,kt(i,j)-1
				if((s0(i,j,k,m)-s0(i,j,k-1,m))*(s0(i,j,k,m)-s0(i,j,k+1,m)).gt.0) mask2(i,j,k-1:k+1,m)=0.
			end do
		end do
		end do
	end do
	sxx=sxx*mask2	!applying mask - dropping to 1st order moments. This must be done twice (here and after exchanges), to provide correct parallelism
	syy=syy*mask2
	szz=szz*mask2
	sxy=sxy*mask2
	sxz=sxz*mask2
	syz=syz*mask2
	!----------end monotonizator----------------
	
	do m=1,ml
		call exchange_moment(sx(:,:,:,m),-1)
		call exchange_moment(sy(:,:,:,m),-1)
		call exchange_moment(sz(:,:,:,m),1)
		call exchange_moment(sxx(:,:,:,m),1)
		call exchange_moment(syy(:,:,:,m),1)
		call exchange_moment(szz(:,:,:,m),1)
		call exchange_moment(sxy(:,:,:,m),1)
		call exchange_moment(sxz(:,:,:,m),-1)
		call exchange_moment(syz(:,:,:,m),-1)
	end do	
	
	sxx=sxx*mask2	!second part of monotonizator
	syy=syy*mask2
	szz=szz*mask2
	sxy=sxy*mask2
	sxz=sxz*mask2
	syz=syz*mask2
	
	
	do m = 1,ml
	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1		,kt(i,j)
		t_f(k,i,j,m) = s0(i,j,k,m)*area_tbox_v(i,j)
	end do
	end do
	end do			
	end do			
	end
	
	
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	subroutine advect_som_x(u0,dx3_v0,ctime)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_basic_module
	use o_grid_module
	use o_som_module
	use o_d_brcl_module, only: flow_v_e,flow_w_b
    use o_par_module

	implicit none
	
	real(kind=main_kind),dimension(iwest(myrank)-1:ieast(myrank)+1,ml) :: F0,FX,FXX,FY,FYY,FXY,fz,fzz,fxz,fyz	
	real(kind=main_kind),dimension(iwest(myrank)-1:ieast(myrank)+1   ) :: FM
	real(kind=main_kind) u0 (kl,	 iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(kind=main_kind) dx3_v0(kl+1,iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	integer ctime
	
	!U-flow through the right faces of the T-boxes. Be careful: defined only till kt, so you must prevent use this array on interfaces between water and land boxes.
	do j=jsouth(myrank)  ,jnorth(myrank)+1
	do i=iwest (myrank)-1,ieast (myrank)+1
	do k=1,kt(i,j)
		flow_v_e (k,i,j) =									&
				(u0(k,i,j)*dx3_v0(k+1,i,j) + u0(k,i,j-1)*dx3_v0(k+1,i,j-1)) * 0.5 *	&
				(dx2_en_t(i,j) + dx2_es_t(i,j))
	end do
	end do
	end do
	
	!We also need flow_v_e at line jsouth-1. Exchange must be carefull since flow_v_e is defined not in T-points
	flow_w_b(1:kl,:,jnorth(myrank)+1)=flow_v_e(1:kl,:,jnorth(myrank)+1) 					!saving (north+1) line
#if CMF_VERSION == 2    
    call comp_halo_update_T (DIM3_t = flow_v_e, update_width = 1, change_sign_on_bipolar = -1)      !during exchange the (north+1) line is distorted on bipolar greenwich
#elif CMF_VERSION == 3
    call HaloUpdateMaster % update(flow_v_e, update_width = 1, grid_type = 'T', change_sign_on_bipolar = .TRUE.)
#endif      	
	flow_v_e(1:kl,:,jnorth(myrank)+1)=flow_w_b(1:kl,:,jnorth(myrank)+1)					!restoring (north+1) line
		
	do k=1,kl
	do j=jsouth(myrank)-1,jnorth(myrank)+1
		
		!marshing through T-boxes. i - index of current t-box, ip1=i+1
		i=iwest(myrank)-1
		do ip1=iwest(myrank),ieast(myrank)+1
			
			if(k.gt.kt(i,j) .or. k.gt. kt(ip1,j)) goto 26
			
			if(flow_v_e (k,i,j).lt.0.) goto 24					!if flow>0 then working with box #i, else with box #ip1
			
			fm(i)=flow_v_e(k,i,j)*htsec*ctime					!volume transported from box #i through its right face
			alf=fm(i)/sm(i,j,k)							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.-alf
			alf1q=alf1*alf1
			sm(i,j,k)=sm(i,j,k)-fm(i)						!remaining volume			
			
			do m=1,ml
				!amounts of moments transported
				f0(i,m)=alf*(s0(i,j,k,m)+alf1*(sx(i,j,k,m)+(alf1-alf)*sxx(i,j,k,m)))
				fx(i,m)=alfq*(sx(i,j,k,m)+3.*alf1*sxx(i,j,k,m))
				fxx(i,m)=alf*alfq*sxx(i,j,k,m)
				fy(i,m)=alf*(sy(i,j,k,m)+alf1*sxy(i,j,k,m))
				fz(i,m)=alf*(sz(i,j,k,m)+alf1*sxz(i,j,k,m))
				fxy(i,m)=alfq*sxy(i,j,k,m)
				fxz(i,m)=alfq*sxz(i,j,k,m)
				fyy(i,m)=alf*syy(i,j,k,m)
				fzz(i,m)=alf*szz(i,j,k,m)
				fyz(i,m)=alf*syz(i,j,k,m)

				!readjust moments remaining in the box
				s0(i,j,k,m)=s0(i,j,k,m)-f0(i,m)
				sx(i,j,k,m)=alf1q*(sx(i,j,k,m)-3.*alf*sxx(i,j,k,m))
				sxx(i,j,k,m)=sxx(i,j,k,m)*alf1*alf1q
				sy(i,j,k,m)=sy(i,j,k,m)-fy(i,m)
				syy(i,j,k,m)=syy(i,j,k,m)-fyy(i,m)
				sz(i,j,k,m)=sz(i,j,k,m)-fz(i,m)
				szz(i,j,k,m)=szz(i,j,k,m)-fzz(i,m)
				
				sxy(i,j,k,m)=alf1q*sxy(i,j,k,m)
				sxz(i,j,k,m)=alf1q*sxz(i,j,k,m)
				syz(i,j,k,m)=syz(i,j,k,m)-fyz(i,m)
			end do
			goto 26
				
24			fm(i)=-flow_v_e (k,i,j)*htsec*ctime					!volume transported from box #ip1 through its left face
			alf=fm(i)/sm(ip1,j,k) 							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.0-alf
			alf1q=alf1*alf1
			sm(ip1,j,k)=sm(ip1,j,k)-fm(i)						!remaining volume
			
			do m=1,ml
				!amounts of moments transported
				f0(i,m)=alf*(s0(ip1,j,k,m)-alf1*(sx(ip1,j,k,m)-(alf1-alf)*sxx(ip1,j,k,m)))
				fx(i,m)=alfq*(sx(ip1,j,k,m)-3.*alf1*sxx(ip1,j,k,m))
				fxx(i,m)=alf*alfq*sxx(ip1,j,k,m)
				fy(i,m)=alf*(sy(ip1,j,k,m)-alf1*sxy(ip1,j,k,m))
				fz(i,m)=alf*(sz(ip1,j,k,m)-alf1*sxz(ip1,j,k,m))
				
				fxy(i,m)=alfq*sxy(ip1,j,k,m)
				fxz(i,m)=alfq*sxz(ip1,j,k,m)
				
				fyy(i,m)=alf*syy(ip1,j,k,m)
				fzz(i,m)=alf*szz(ip1,j,k,m)
				fyz(i,m)=alf*syz(ip1,j,k,m)

				!readjust moments remaining in the box
				s0(ip1,j,k,m)=s0(ip1,j,k,m)-f0(i,m)
				sx(ip1,j,k,m)=alf1q*(sx(ip1,j,k,m)+3.*alf*sxx(ip1,j,k,m))
				sxx(ip1,j,k,m)=alf1*alf1q*sxx(ip1,j,k,m)
				sy(ip1,j,k,m)=sy(ip1,j,k,m)-fy(i,m)
				syy(ip1,j,k,m)=syy(ip1,j,k,m)-fyy(i,m)
				sz(ip1,j,k,m)=sz(ip1,j,k,m)-fz(i,m)
				
				szz(ip1,j,k,m)=szz(ip1,j,k,m)-fzz(i,m)
				sxy(ip1,j,k,m)=alf1q*sxy(ip1,j,k,m)
				sxz(ip1,j,k,m)=alf1q*sxz(ip1,j,k,m)
				syz(ip1,j,k,m)=syz(ip1,j,k,m)-fyz(i,m)
			end do
26			i=ip1
		end do		
				
		!put the temporary moments (fi) into appropriate neighboring boxes
		i=iwest(myrank)-1
		do ip1=iwest(myrank),ieast(myrank)+1 
		   
			if(k.gt.kt(i,j) .or. k.gt. kt(ip1,j)) goto 36
			
			if(flow_v_e (k,i,j).lt.0.) goto 34
			
			sm(ip1,j,k)=sm(ip1,j,k)+fm(i)
			alf=fm(i)/sm(ip1,j,k)
			alf1=1.0-alf
			do m=1,ml
				temptm=alf*s0(ip1,j,k,m)-alf1*f0(i,m)
				s0(ip1,j,k,m)=s0(ip1,j,k,m)+f0(i,m)
				sxx(ip1,j,k,m)=alf*alf*fxx(i,m)+alf1*alf1*sxx(ip1,j,k,m)+5.*(alf*alf1*(sx(ip1,j,k,m)-fx(i,m))-(alf1-alf)*temptm)
				sx(ip1,j,k,m)=alf*fx(i,m)+alf1*sx(ip1,j,k,m)+3.*temptm
				sxy(ip1,j,k,m)=alf*fxy(i,m)+alf1*sxy(ip1,j,k,m)+3.*(-alf1*fy(i,m)+alf*sy(ip1,j,k,m))
				sxz(ip1,j,k,m)=alf*fxz(i,m)+alf1*sxz(ip1,j,k,m)+3.*(-alf1*fz(i,m)+alf*sz(ip1,j,k,m))
				
				sy(ip1,j,k,m)=sy(ip1,j,k,m)+fy(i,m)
				syy(ip1,j,k,m)=syy(ip1,j,k,m)+fyy(i,m)
				
				sz(ip1,j,k,m)=sz(ip1,j,k,m)+fz(i,m)
				szz(ip1,j,k,m)=szz(ip1,j,k,m)+fzz(i,m)
				syz(ip1,j,k,m)=syz(ip1,j,k,m)+fyz(i,m)
			end do
			goto 36

34			sm(i,j,k)=sm(i,j,k)+fm(i)
			alf=fm(i)/sm(i,j,k)
			alf1=1.0-alf
			do m=1,ml
				temptm=-alf*s0(i,j,k,m)+alf1*f0(i,m)
				s0(i,j,k,m)=s0(i,j,k,m)+f0(i,m)
				sxx(i,j,k,m)=alf*alf*fxx(i,m)+alf1*alf1*sxx(i,j,k,m)+5.*(alf*alf1*(-sx(i,j,k,m)+fx(i,m))+(alf1-alf)*temptm)
				sx(i,j,k,m)=alf*fx(i,m)+alf1*sx(i,j,k,m)+3.*temptm
				sxy(i,j,k,m)=alf*fxy(i,m)+alf1*sxy(i,j,k,m)+3.*(alf1*fy(i,m)-alf*sy(i,j,k,m))
				sxz(i,j,k,m)=alf*fxz(i,m)+alf1*sxz(i,j,k,m)+3.*(alf1*fz(i,m)-alf*sz(i,j,k,m))
				
				sy(i,j,k,m)=sy(i,j,k,m)+fy(i,m)
				syy(i,j,k,m)=syy(i,j,k,m)+fyy(i,m)
				
				sz(i,j,k,m)=sz(i,j,k,m)+fz(i,m)
				szz(i,j,k,m)=szz(i,j,k,m)+fzz(i,m)
				syz(i,j,k,m)=syz(i,j,k,m)+fyz(i,m)
			end do
36			i=ip1
		end do
	end do
	end do
	end 

	
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	subroutine advect_som_y(v0,dx3_v0,ctime)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_basic_module
	use o_grid_module
	use o_som_module
	use o_d_brcl_module
	implicit none
	
	real(kind=main_kind),dimension(jsouth(myrank)-1:jnorth(myrank)+1,ml) :: F0,FX,FXX,FY,FYY,FXY,fz,fzz,fxz,fyz	
	real(kind=main_kind),dimension(jsouth(myrank)-1:jnorth(myrank)+1   ) :: FM
	real(kind=main_kind) v0 (kl,	 iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	real(kind=main_kind) dx3_v0(kl+1,iwest (myrank)-1:ieast (myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
	integer ctime
	
	do k=1,kl
	do i=iwest(myrank),ieast(myrank)
		
		!marshing through T-boxes. j - index of current t-box, jp1=j+1
		j=jsouth(myrank)-1
		do jp1=jsouth(myrank),jnorth(myrank)+1
			
			if(k.gt.kt(i,j) .or. k.gt. kt(i,jp1)) goto 126
					
			flow_v_n(k,i,j) =									&
				(v0(k,i-1,j)*dx3_v0(k+1,i-1,j) + v0(k,i,j)*dx3_v0(k+1,i,j)   ) * 0.5 *	&
				(dx1_nw_t(i,j) + dx1_ne_t(i,j))
				
			if(flow_v_n (k,i,j).lt.0.) goto 124					!if flow>0 then working with box #i, else with box #ip1
			
			fm(j)=flow_v_n(k,i,j)*htsec*ctime					!volume transported from box #i through its right face
			alf=fm(j)/sm(i,j,k)							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.-alf
			alf1q=alf1*alf1
			sm(i,j,k)=sm(i,j,k)-fm(j)						!remaining volume			
			
			do m=1,ml
				!amounts of moments transported
				f0(j,m)=alf*(s0(i,j,k,m)+alf1*(sy(i,j,k,m)+(alf1-alf)*syy(i,j,k,m)))
				fy(j,m)=alfq*(sy(i,j,k,m)+3.*alf1*syy(i,j,k,m))
				fyy(j,m)=alf*alfq*syy(i,j,k,m)
				fx(j,m)=alf*(sx(i,j,k,m)+alf1*sxy(i,j,k,m))
				fz(j,m)=alf*(sz(i,j,k,m)+alf1*syz(i,j,k,m))
				
				fxy(j,m)=alfq*sxy(i,j,k,m)
				fyz(j,m)=alfq*syz(i,j,k,m)
				
				fxx(j,m)=alf*sxx(i,j,k,m)
				fzz(j,m)=alf*szz(i,j,k,m)
				fxz(j,m)=alf*sxz(i,j,k,m)

				!readjust moments remaining in the box
				s0(i,j,k,m)=s0(i,j,k,m)-f0(j,m)
				
				sy(i,j,k,m)=alf1q*(sy(i,j,k,m)-3.*alf*syy(i,j,k,m))
				syy(i,j,k,m)=syy(i,j,k,m)*alf1*alf1q
				sx(i,j,k,m)=sx(i,j,k,m)-fx(j,m)
				sxx(i,j,k,m)=sxx(i,j,k,m)-fxx(j,m)
				sz(i,j,k,m)=sz(i,j,k,m)-fz(j,m)
				
				szz(i,j,k,m)=szz(i,j,k,m)-fzz(j,m)
				sxy(i,j,k,m)=alf1q*sxy(i,j,k,m)
				syz(i,j,k,m)=alf1q*syz(i,j,k,m)
				sxz(i,j,k,m)=sxz(i,j,k,m)-fxz(j,m)
			end do
			goto 126
				
124			fm(j)=-flow_v_n (k,i,j)*htsec*ctime					!volume transported from box #ip1 through its left face
			alf=fm(j)/sm(i,jp1,k) 							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.0-alf
			alf1q=alf1*alf1
			sm(i,jp1,k)=sm(i,jp1,k)-fm(j)						!remaining volume
			
			do m=1,ml
				!amounts of moments transported
				f0(j,m)=alf*(s0(i,jp1,k,m)-alf1*(sy(i,jp1,k,m)-(alf1-alf)*syy(i,jp1,k,m)))
				fy(j,m)=alfq*(sy(i,jp1,k,m)-3.*alf1*syy(i,jp1,k,m))
				fyy(j,m)=alf*alfq*syy(i,jp1,k,m)
				fx(j,m)=alf*(sx(i,jp1,k,m)-alf1*sxy(i,jp1,k,m))
				fz(j,m)=alf*(sz(i,jp1,k,m)-alf1*syz(i,jp1,k,m))
				
				fxy(j,m)=alfq*sxy(i,jp1,k,m)
				fyz(j,m)=alfq*syz(i,jp1,k,m)
				
				fxx(j,m)=alf*sxx(i,jp1,k,m)
				fzz(j,m)=alf*szz(i,jp1,k,m)
				fxz(j,m)=alf*sxz(i,jp1,k,m)

				!readjust moments remaining in the box
				s0(i,jp1,k,m)=s0(i,jp1,k,m)-f0(j,m)
				sy(i,jp1,k,m)=alf1q*(sy(i,jp1,k,m)+3.*alf*syy(i,jp1,k,m))
				syy(i,jp1,k,m)=alf1*alf1q*syy(i,jp1,k,m)
				sx(i,jp1,k,m)=sx(i,jp1,k,m)-fx(j,m)
				sxx(i,jp1,k,m)=sxx(i,jp1,k,m)-fxx(j,m)
				sz(i,jp1,k,m)=sz(i,jp1,k,m)-fz(j,m)
				szz(i,jp1,k,m)=szz(i,jp1,k,m)-fzz(j,m)
				
				sxy(i,jp1,k,m)=alf1q*sxy(i,jp1,k,m)
				syz(i,jp1,k,m)=alf1q*syz(i,jp1,k,m)
				sxz(i,jp1,k,m)=sxz(i,jp1,k,m)-fxz(j,m)
			end do
126			j=jp1
		end do		
				
		!put the temporary moments (fi) into appropriate neighboring boxes
		j=jsouth(myrank)-1
		do jp1=jsouth(myrank),jnorth(myrank)+1
		   
			if(k.gt.kt(i,j) .or. k.gt. kt(i,jp1)) goto 136
			
			if(flow_v_n (k,i,j).lt.0.) goto 134
			
			sm(i,jp1,k)=sm(i,jp1,k)+fm(j)
			alf=fm(j)/sm(i,jp1,k)
			alf1=1.0-alf
			do m=1,ml
				temptm=alf*s0(i,jp1,k,m)-alf1*f0(j,m)
				s0(i,jp1,k,m)=s0(i,jp1,k,m)+f0(j,m)
				syy(i,jp1,k,m)=alf*alf*fyy(j,m)+alf1*alf1*syy(i,jp1,k,m)+5.*(alf*alf1*(sy(i,jp1,k,m)-fy(j,m))-(alf1-alf)*temptm)
				sy(i,jp1,k,m)=alf*fy(j,m)+alf1*sy(i,jp1,k,m)+3.*temptm
				sxy(i,jp1,k,m)=alf*fxy(j,m)+alf1*sxy(i,jp1,k,m)+3.*(-alf1*fx(j,m)+alf*sx(i,jp1,k,m))
				syz(i,jp1,k,m)=alf*fyz(j,m)+alf1*syz(i,jp1,k,m)+3.*(-alf1*fz(j,m)+alf*sz(i,jp1,k,m))
				
				sx(i,jp1,k,m)=sx(i,jp1,k,m)+fx(j,m)
				sxx(i,jp1,k,m)=sxx(i,jp1,k,m)+fxx(j,m)
				
				sz(i,jp1,k,m)=sz(i,jp1,k,m)+fz(j,m)
				szz(i,jp1,k,m)=szz(i,jp1,k,m)+fzz(j,m)
				sxz(i,jp1,k,m)=sxz(i,jp1,k,m)+fxz(j,m)
			end do
			goto 136

134			sm(i,j,k)=sm(i,j,k)+fm(j)
			alf=fm(j)/sm(i,j,k)
			alf1=1.0-alf
			do m=1,ml
				temptm=-alf*s0(i,j,k,m)+alf1*f0(j,m)
				s0(i,j,k,m)=s0(i,j,k,m)+f0(j,m)
				syy(i,j,k,m)=alf*alf*fyy(j,m)+alf1*alf1*syy(i,j,k,m)+5.*(alf*alf1*(-sy(i,j,k,m)+fy(j,m))+(alf1-alf)*temptm)
				sy(i,j,k,m)=alf*fy(j,m)+alf1*sy(i,j,k,m)+3.*temptm
				sxy(i,j,k,m)=alf*fxy(j,m)+alf1*sxy(i,j,k,m)+3.*(alf1*fx(j,m)-alf*sx(i,j,k,m))
				syz(i,j,k,m)=alf*fyz(j,m)+alf1*syz(i,j,k,m)+3.*(alf1*fz(j,m)-alf*sz(i,j,k,m))
				
				sx(i,j,k,m)=sx(i,j,k,m)+fx(j,m)
				sxx(i,j,k,m)=sxx(i,j,k,m)+fxx(j,m)
				sz(i,j,k,m)=sz(i,j,k,m)+fz(j,m)
				szz(i,j,k,m)=szz(i,j,k,m)+fzz(j,m)
				sxz(i,j,k,m)=sxz(i,j,k,m)+fxz(j,m)
			end do
136			j=jp1
		end do
	end do
	end do
	end 

!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
	subroutine advect_som_z(w0,ctime)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_basic_module
	use o_grid_module
	use o_som_module
	use o_d_brcl_module
	implicit none
	
	real(kind=main_kind),dimension(kl,ml) :: F0,FX,FXX,FY,FYY,FXY,fz,fzz,fxz,fyz	
	real(kind=main_kind),dimension(kl   ) :: FM
	real(kind=main_kind) w0(kl+1,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)
	integer ctime
	
	do i=iwest(myrank),ieast(myrank)
	do j=jsouth(myrank),jnorth(myrank)
		
		!marshing through T-boxes. k - index of current t-box, kp1=k+1
		k=1
		do kp1=2,kt(i,j)
			
			flow_w_b (k,i,j) = w(k+1,i,j)*ct3(k+1,i,j)*area_tbox(i,j)
			if(flow_w_b (k,i,j).lt.0.) goto 224					!if flow>0 then working with box #k, else with box #kp1
			
			fm(k)=flow_w_b(k,i,j)*htsec*ctime					!volume transported from box #k through its bottom face
			alf=fm(k)/sm(i,j,k)							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.-alf
			alf1q=alf1*alf1
			!sm(i,j,k)=sm(i,j,k)-fm(k)						!remaining volume. Is not used any more if advect_som_z is the last of 3 advections			
			
			do m=1,ml
				!amounts of moments transported
				f0(k,m)=alf*(s0(i,j,k,m)+alf1*(sz(i,j,k,m)+(alf1-alf)*szz(i,j,k,m)))
				fz(k,m)=alfq*(sz(i,j,k,m)+3.*alf1*szz(i,j,k,m))
				fzz(k,m)=alf*alfq*szz(i,j,k,m)
				fy(k,m)=alf*(sy(i,j,k,m)+alf1*syz(i,j,k,m))
				fx(k,m)=alf*(sx(i,j,k,m)+alf1*sxz(i,j,k,m))
				
				fyz(k,m)=alfq*syz(i,j,k,m)
				fxz(k,m)=alfq*sxz(i,j,k,m)
				
				fyy(k,m)=alf*syy(i,j,k,m)
				fxx(k,m)=alf*sxx(i,j,k,m)
				fxy(k,m)=alf*sxy(i,j,k,m)

				!readjust moments remaining in the box
				s0(i,j,k,m)=s0(i,j,k,m)-f0(k,m)
				sz(i,j,k,m)=alf1q*(sz(i,j,k,m)-3.*alf*szz(i,j,k,m))
				szz(i,j,k,m)=szz(i,j,k,m)*alf1*alf1q
				sy(i,j,k,m)=sy(i,j,k,m)-fy(k,m)
				syy(i,j,k,m)=syy(i,j,k,m)-fyy(k,m)
				
				sx(i,j,k,m)=sx(i,j,k,m)-fx(k,m)
				sxx(i,j,k,m)=sxx(i,j,k,m)-fxx(k,m)
				
				syz(i,j,k,m)=alf1q*syz(i,j,k,m)
				sxz(i,j,k,m)=alf1q*sxz(i,j,k,m)
				sxy(i,j,k,m)=sxy(i,j,k,m)-fxy(k,m)
			end do
			goto 226
				
224			fm(k)=-flow_w_b (k,i,j)*htsec*ctime					!volume transported from box #kp1 through its up face
			alf=fm(k)/sm(i,j,kp1) 							!transported part (=transported volume/whole volume)
			alfq=alf*alf
			alf1=1.0-alf
			alf1q=alf1*alf1
			sm(i,j,kp1)=sm(i,j,kp1)-fm(k)						!remaining volume
			
			do m=1,ml
				!amounts of moments transported
				f0(k,m)=alf*(s0(i,j,kp1,m)-alf1*(sz(i,j,kp1,m)-(alf1-alf)*szz(i,j,kp1,m)))
				fz(k,m)=alfq*(sz(i,j,kp1,m)-3.*alf1*szz(i,j,kp1,m))
				fzz(k,m)=alf*alfq*szz(i,j,kp1,m)
				fy(k,m)=alf*(sy(i,j,kp1,m)-alf1*syz(i,j,kp1,m))
				fx(k,m)=alf*(sx(i,j,kp1,m)-alf1*sxz(i,j,kp1,m))
				
				fyz(k,m)=alfq*syz(i,j,kp1,m)
				fxz(k,m)=alfq*sxz(i,j,kp1,m)
				
				fyy(k,m)=alf*syy(i,j,kp1,m)
				fxx(k,m)=alf*sxx(i,j,kp1,m)
				fxy(k,m)=alf*sxy(i,j,kp1,m)

				!readjust moments remaining in the box
				s0(i,j,kp1,m)=s0(i,j,kp1,m)-f0(k,m)
				sz(i,j,kp1,m)=alf1q*(sz(i,j,kp1,m)+3.*alf*szz(i,j,kp1,m))
				szz(i,j,kp1,m)=alf1*alf1q*szz(i,j,kp1,m)
				sy(i,j,kp1,m)=sy(i,j,kp1,m)-fy(k,m)
				syy(i,j,kp1,m)=syy(i,j,kp1,m)-fyy(k,m)
				
				sx(i,j,kp1,m)=sx(i,j,kp1,m)-fx(k,m)
				sxx(i,j,kp1,m)=sxx(i,j,kp1,m)-fxx(k,m)
				
				syz(i,j,kp1,m)=alf1q*syz(i,j,kp1,m)
				sxz(i,j,kp1,m)=alf1q*sxz(i,j,kp1,m)
				sxy(i,j,kp1,m)=sxy(i,j,kp1,m)-fxy(k,m)
			end do
226			k=kp1
		end do		
				
		!put the temporary moments (fi) into appropriate neighboring boxes
		k=1
		do kp1=2,kt(i,j)
		   
			if(flow_w_b (k,i,j).lt.0.) goto 234
			
			sm(i,j,kp1)=sm(i,j,kp1)+fm(k)
			alf=fm(k)/sm(i,j,kp1)
			alf1=1.0-alf
			do m=1,ml
				temptm=alf*s0(i,j,kp1,m)-alf1*f0(k,m)
				s0(i,j,kp1,m)=s0(i,j,kp1,m)+f0(k,m)
				
				szz(i,j,kp1,m)=alf*alf*fzz(k,m)+alf1*alf1*szz(i,j,kp1,m)+5.*(alf*alf1*(sz(i,j,kp1,m)-fz(k,m))-(alf1-alf)*temptm)
				sz(i,j,kp1,m)=alf*fz(k,m)+alf1*sz(i,j,kp1,m)+3.*temptm
				syz(i,j,kp1,m)=alf*fyz(k,m)+alf1*syz(i,j,kp1,m)+3.*(-alf1*fy(k,m)+alf*sy(i,j,kp1,m))
				sxz(i,j,kp1,m)=alf*fxz(k,m)+alf1*sxz(i,j,kp1,m)+3.*(-alf1*fx(k,m)+alf*sx(i,j,kp1,m))
				
				sy(i,j,kp1,m)=sy(i,j,kp1,m)+fy(k,m)
				syy(i,j,kp1,m)=syy(i,j,kp1,m)+fyy(k,m)
				sx(i,j,kp1,m)=sx(i,j,kp1,m)+fx(k,m)
				sxx(i,j,kp1,m)=sxx(i,j,kp1,m)+fxx(k,m)
				sxy(i,j,kp1,m)=sxy(i,j,kp1,m)+fxy(k,m)
			end do
			goto 236

234			sm(i,j,k)=sm(i,j,k)+fm(k)
			alf=fm(k)/sm(i,j,k)
			alf1=1.0-alf
			do m=1,ml
				temptm=-alf*s0(i,j,k,m)+alf1*f0(k,m)
				s0(i,j,k,m)=s0(i,j,k,m)+f0(k,m)
				
				szz(i,j,k,m)=alf*alf*fzz(k,m)+alf1*alf1*szz(i,j,k,m)+5.*(alf*alf1*(-sz(i,j,k,m)+fz(k,m))+(alf1-alf)*temptm)
				sz(i,j,k,m)=alf*fz(k,m)+alf1*sz(i,j,k,m)+3.*temptm
				syz(i,j,k,m)=alf*fyz(k,m)+alf1*syz(i,j,k,m)+3.*(alf1*fy(k,m)-alf*sy(i,j,k,m))
				sxz(i,j,k,m)=alf*fxz(k,m)+alf1*sxz(i,j,k,m)+3.*(alf1*fx(k,m)-alf*sx(i,j,k,m))
				
				sy(i,j,k,m)=sy(i,j,k,m)+fy(k,m)
				syy(i,j,k,m)=syy(i,j,k,m)+fyy(k,m)
				sx(i,j,k,m)=sx(i,j,k,m)+fx(k,m)
				sxx(i,j,k,m)=sxx(i,j,k,m)+fxx(k,m)
				sxy(i,j,k,m)=sxy(i,j,k,m)+fxy(k,m)
			end do
236			k=kp1
		end do
	end do
	end do
	end 
		
	
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	subroutine exchange_moment(moment,bipolar)
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	use o_grid_module
    use o_par_module

    use o_d_brcl_module
	implicit none
	
	real(kind=main_kind) moment(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1,kl)
	integer bipolar
	
	do i=iwest(myrank),ieast(myrank)
	do j=jsouth(myrank),jnorth(myrank)
	do k=1,kt(i,j)
		flow_v_e(k,i,j)=moment(i,j,k)
	end do
	end do
	end do

#if CMF_VERSION == 2    
    call comp_halo_update_T (DIM3_t = flow_v_e, update_width = 1, change_sign_on_bipolar = bipolar)
#elif CMF_VERSION == 3
    call HaloUpdateMaster % update(flow_v_e, update_width = 1, grid_type = 'T', change_sign_on_bipolar = MERGE(.true., .false., bipolar == -1))
#endif 	
	do i=iwest(myrank)-1,ieast(myrank)+1
	do j=jsouth(myrank)-1,jnorth(myrank)+1
	do k=1,kt(i,j)
		moment(i,j,k)=flow_v_e(k,i,j)
	end do
	end do
	end do
	end
	
	