
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	upwind scheme for advection
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!--------------------------------------


!~~~~~~
	subroutine o_td_scalar_advection_uw
!~~~~~~

!--------------------------------------
!	integration of advection term of scalar equation 
!	t(l+1) = t(l)+advection(t(l))*dt
!--------------------------------------
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
!z	use o_lb_module
	implicit none
	
!z	real (kind=8)	c_ssh_2
!z	real (kind=4)	cin,cfr
!z	real (kind=4)	h


!-------
!		flow through the walls of t-box
!-------

!*******
!		repeat calculation of flows from subr. w_3d_exp
!		for sea and land v-cells separated by solid boundaries
!*******

!-------
!		flow through the walls of t-box
!-------

	do j=jsouth(myrank)-1	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)

!	flow_v_n (k,i,j) = 
!     &		(     v_c(k  ,i-1,j  ) +      v_c(k  ,i  ,j  )   ) * 0.5 *
!     &		( dx3_v_c(k+1,i-1,j  ) +  dx3_v_c(k+1,i  ,j  )   ) * 0.5 *
!     &		(dx1_nw_t(    i  ,j  ) + dx1_ne_t(    i  ,j  )   )
	
	flow_v_n(k,i,j) =
     & (v_c(k,i-1,j)*dx3_v_c(k+1,i-1,j) + v_c(k,i,j)*dx3_v_c(k+1,i,j))*0.5*
     & (dx1_nw_t(i,j) + dx1_ne_t(i,j))
	
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)-1	,ieast (myrank)+1
	do k=1, kt(i,j)

!	flow_u_e (k,i,j) = 
!     &		(     u_c(k  ,i  ,j  ) +      u_c(k  ,i  ,j-1)  ) * 0.5 *
!     &		( dx3_v_c(k+1,i  ,j  ) +  dx3_v_c(k+1,i  ,j-1)  ) * 0.5 *
!     &		(dx2_en_t(    i  ,j  ) + dx2_es_t(    i  ,j  )  )

	flow_u_e (k,i,j) =	
     & (u_c(k,i,j)*dx3_v_c(k+1,i,j) + u_c(k,i,j-1)*dx3_v_c(k+1,i,j-1))*0.5*
     & (dx2_en_t(i,j) + dx2_es_t(i,j))
	
	end do	!do k=1,kl
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)
	flow_v_s (k,i,j) = flow_v_n (k,i,j-1)
	end do
	end do
	end do

	do j=jsouth(myrank)	,jnorth(myrank)+1
	do i=iwest (myrank)	,ieast (myrank)+1
	do k=1, kt(i,j)
	flow_u_w (k,i,j) = flow_u_e (k,i-1,j)
	end do	!do k=1,kl
	end do
	end do


	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	if (kt(i,j).ge.1) then

	do k = 1, kt(i,j)-1
	flow_w_b(k+1,i,j) = w(k+1,i,j)*ct3(k+1,i,j)
	end do

	k = kt(i,j)
	flow_w_b(k+1,i,j) = w(k+1,i,j)*ct3(k,i,j)

	end if
	end do
	end do



	do m = 1,ml

	flow_t_e = 0.
	flow_t_n = 0.
	flow_t_w = 0.
	flow_t_s = 0.
	flow_t_b = 0.
	adv_t = 0.

!-------
!			upwind scheme for horizontal advection
!-------

	do j = jsouth(myrank)-1	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1,kl
	if(k.le.kt(i,j) ) then
	if(	flow_v_n(k,i,j).ge.0.)	then
		flow_t_n(k,i,j) = flow_v_n(k,i,j)*t_c(k,i,  j,m  )
			else
		flow_t_n(k,i,j) = flow_v_n(k,i,j)*t_c(k,i,  j+1,m)
	end if
	end if
	end do
	end do
	end do			!do k=1,kl

	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)-1	,ieast (myrank)
	do k = 1,kl
	if(k.le.kt(i,j) ) then
	if(	flow_u_e(k,i,j).ge.0.)	then
		flow_t_e(k,i,j) = flow_u_e(k,i,j)*t_c(k,i  ,j,m  )
			else
		flow_t_e(k,i,j) = flow_u_e(k,i,j)*t_c(k,i+1,j,m  )
	end if
	end if
	end do
	end do
	end do			!do k=1,kl

	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1,kl
	flow_t_s(k,i,j) = flow_t_n (k,i,j-1)
	end do
	end do
	end do			!do k=1,kl

	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1,kl
	flow_t_w(k,i,j) = flow_t_e (k,i-1,j)
	end do
	end do
	end do			!do k=1,kl

!-------
!			upwind scheme for vertical advection
!-------
	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	if (kt(i,j).ge.1) then

	do k = 1, kt(i,j)-1
        if(flow_w_b(k+1,i,j).ge.0.)	then
				flow_t_b(k,i,j) = flow_w_b(k+1,i,j)*t_c(k,i,j,m)
				else
				flow_t_b(k,i,j) = flow_w_b(k+1,i,j)*t_c(k+1,i,j,m)
			        end if
	end do

	k = kt(i,j)
				flow_t_b(k,i,j) = flow_w_b(k+1,i,j)*t_c(k,i,j,m)

       	end if
	end do
	end do			!do k=1,kl


	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
c	do k = 1,kl
c	if(k.le.kt(i,j) ) then
	do k = 1, kt(i,j)
	adv_t (k,i,j) = area_tbox_v(i,j) * (
     &		(flow_t_e(k,i  ,j  ) - flow_t_w(k,i,j)) +
     &		(flow_t_n(k,i  ,j  ) - flow_t_s(k,i,j))  )
c     &		(flow_t_e(k,i  ,j  ) - flow_t_e(k,i-1,j  )) +
c     &		(flow_t_n(k,i  ,j  ) - flow_t_n(k,i  ,j-1)))

c	end if
	end do

	end do
	end do			!do k=1,kl

	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1		,kt(i,j)
c	t_f(k,i,j,m) = t_c(k,i,j,m)*ct3(k,i,j)*dx3_t_c(k+1,i,j)
	t_f(k,i,j,m) = t_c(k,i,j,m)*ct3(k,i,j)*dx3_t_c(k+1,i,j) 
     &				- htsec*adv_t (k,i,j)
	end do
	end do
	end do			!do k=1,kl

	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1		,kt(i,j)
	t_f(k,i,j,m) = t_f(k,i,j,m) - 
     &			htsec*(flow_t_b(k,i,j)-flow_t_b(k-1,i,j))
	end do
	end do
	end do			!do k=1,kl

	
	end do			!do m=1,ml

	return
	end	
