#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	horizontal mixing terms
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

	module o_td_mixing
	contains
	subroutine o_td_mixh (t,mht_u,mht_v)

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_core_diagnostics_module
	use o_par_module
	use o_diagn_means_module
	implicit none

	real (kind=main_kind)	t	(kl,iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1, ml)
	type(mean_field),optional :: mht_u,mht_v
	
!-------
! Compute turbulent fluxes at eastern and northern walls of T-box
! Careful! Positive directions for these fluxes are opposite to those of velocities
! At solid boundaries - zero flux
!-------
	
	do m=1,ml
		!diffusive fluxes through the walls of t-box. Will be [K*m3/sec] after multiplication by ah.
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
		do k=1,kl
			flux_u_e (k,i,j)=	0.25 *(cv(k,i,j-1)+cv(k,i,j)) * (dx3_v_f(k+1,i,j-1)+dx3_v_f(k+1,i,j)) *		&
     				( t(k,i+1,j,m) - t(k,i,j,m) ) /				&
     				( dx1_nw_v(i,j-1)+dx1_ne_v(i,j-1) ) *		&
     				( dx2_en_t(i,j)+dx2_es_t(i,j) )
		end do
		end do
		end do

		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kl
			flux_u_n (k,i,j) = 	0.25*(cv(k,i-1,j  )+cv(k,i,j  )) * (dx3_v_f(k+1,i-1,j  )+dx3_v_f(k+1,i,j  )) *	&
     				( t(k,i,j+1,m)-t(k,i,j,m) ) /				&
     				( dx2_en_v(i-1,j)+dx2_es_v(i-1,j) ) *		&
     				( dx1_nw_t(i,j)+dx1_ne_t(i,j) )
		end do
		end do
		end do

		!-------
		! pure Laplase - to be used in biharmonics
		!-------
 		if(mixh_t_mode .ge. 1) then
 			do j=jsouth(myrank)	,jnorth(myrank)
			do i=iwest (myrank)	,ieast (myrank)
			do k=1,kt(i,j)
				t_laplace_pure(k,i,j,m) = area_tbox_v(i,j) * (		&
					(flux_u_e(k,i,j) - flux_u_e(k,i-1,j  )) +	&
					(flux_u_n(k,i,j) - flux_u_n(k,i  ,j-1))   )
			end do
			end do
			end do
		end if	
		
			
		!-------
		! Laplase with diffusion coefficients - to be used here
		!-------
		if(mixh_t_mode.lt.2) then
			do j=jsouth(myrank)-1	,jnorth(myrank)
			do i=iwest (myrank)-1	,ieast (myrank)
			do k=1,kt(i,j)
				flux_u_e(k,i,j)=flux_u_e(k,i,j)*ah_e(k,i,j,m)
				flux_u_n(k,i,j)=flux_u_n(k,i,j)*ah_n(k,i,j,m)
			end do
			end do
			end do
		
			do j=jsouth(myrank)	,jnorth(myrank)
			do i=iwest (myrank)	,ieast (myrank)
			do k=1,kt(i,j)
				t_laplace(k,i,j,m) = area_tbox_v(i,j) * (		&
					(flux_u_e(k,i,j) - flux_u_e(k,i-1,j  )) +	&
					(flux_u_n(k,i,j) - flux_u_n(k,i  ,j-1))   )			
			end do
			end do
			end do
			
 			if(present(mht_u).and.present(mht_v).and.m==1) then
				do j = jsouth(myrank)	,jnorth(myrank)
				do i = iwest (myrank)	,ieast (myrank)
				do k = 1, kt(i,j)
					flux_u_e(k,i,j)=-flux_u_e(k,i,j)/( dx2_en_t(i,j)+dx2_es_t(i,j) )
					flux_u_n(k,i,j)=-flux_u_n(k,i,j)/( dx1_nw_t(i,j)+dx1_ne_t(i,j) )
				end do
				end do
				end do
				call o_diagn_time_mean(dim3_arr_in=flux_u_e, arr_out=mht_u, finalize_condition=(MOD(time_l,time_l_save_cp) == 0))
				call o_diagn_time_mean(dim3_arr_in=flux_u_n, arr_out=mht_v, finalize_condition=(MOD(time_l,time_l_save_cp) == 0))
			
				!rotate heat flux arrays if finalized
				if(MOD(time_l,time_l_save_cp) == 0) then
					do k=1,kl
						call tsc_ssc_rotate_v(	mht_u % collector3d(k,:,:), mht_v % collector3d(k,:,:),			&
									mht_u % collector3d(k,:,:), mht_v % collector3d(k,:,:),			&
									iwest(myrank)-1,ieast(myrank)+1,jsouth(myrank)-1,jnorth(myrank)+1)	
					end do
					if(mixh_t_mode==1) then
						mht_u % collector3d=mht_u % collector3d*2.	!We have to *2 because we have incremented mht_u%counter twice (with laplace and biharmonic terms)
						mht_v % collector3d=mht_v % collector3d*2.
					end if
				end if
			end if
		end if
	end do
#if CMF_VERSION == 2    
    if(mixh_t_mode.ge.1) call comp_halo_update_T (DIM4_t = t_laplace_pure, update_width = 1, change_sign_on_bipolar = 1)      
#elif CMF_VERSION == 3
    if(mixh_t_mode.ge.1) call HaloUpdateMaster % update(t_laplace_pure(:,:,:,1), update_width = 1, grid_type = 'T')
    if(mixh_t_mode.ge.1) call HaloUpdateMaster % update(t_laplace_pure(:,:,:,2), update_width = 1, grid_type = 'T')
#endif 
	end subroutine o_td_mixh

	
	subroutine o_td_mixh_bh(mht_u,mht_v)

!--------------------------------------
!	compute friction terms for t and s equation by bi-harmonic operator
!		with laplace terms computed before
!--------------------------------------
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_core_diagnostics_module
	use o_diagn_means_module
	implicit none

	type(mean_field),optional :: mht_u,mht_v
	do m=1,ml
	!-------
	!		Compute turbulent fluxes at eastern and northern walls of T-box
	!		At solid boundaries - zero flux
	!		Careful: see comments in o_td_mixh
	!-------

		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)-1	,ieast (myrank)
		do k=1,kl
			flux_u_e (k,i,j)= (t_laplace_pure(k,i+1,j,m)*ct3v(k,i+1,j)/dx3_t_c(k+1,i+1,j) - t_laplace_pure(k,i,j,m)*ct3v(k,i,j)/dx3_t_c(k+1,i,j) )*ah_bh_e(k,i,j,m) /	&
     				( dx1_nw_v(i,j-1)+dx1_ne_v(i,j-1) ) *				&
     				( dx2_en_t(i,j)+dx2_es_t(i,j) ) * 0.25 * (cv(k,i,j-1)+cv(k,i,j)) * (dx3_v_c(k+1,i,j-1)+dx3_v_c(k+1,i,j))
		end do
		end do
		end do

		do j=jsouth(myrank)-1	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kl
			flux_u_n (k,i,j) = 	(t_laplace_pure(k,i,j+1,m)*ct3v(k,i,j+1)/dx3_t_c(k+1,i,j+1)-t_laplace_pure(k,i,j,m)*ct3v(k,i,j)/dx3_t_c(k+1,i,j) )*ah_bh_n(k,i,j,m) /	&
     				( dx2_en_v(i-1,j)+dx2_es_v(i-1,j) ) *				&
     				( dx1_nw_t(i,j)+dx1_ne_t(i,j) ) * 0.25 * (cv(k,i-1,j  )+cv(k,i,j  )) * (dx3_v_c(k+1,i-1,j  )+dx3_v_c(k+1,i,j  ))
		end do
		end do
		end do

!-------
!		biharmonic operator
!-------
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kt(i,j)
			t_laplace2(k,i,j,m) = area_tbox_v(i,j) * (				&
     				(flux_u_e(k,i,j) - flux_u_e(k,i-1,j  )) +			&
     				(flux_u_n(k,i,j) - flux_u_n(k,i  ,j-1))   )
		end do
		end do
		end do
		
		if(present(mht_u).and.present(mht_v).and.m==1) then
			do j = jsouth(myrank)	,jnorth(myrank)
			do i = iwest (myrank)	,ieast (myrank)
			do k = 1, kt(i,j)
				flux_u_e(k,i,j)=-flux_u_e(k,i,j)*td_bh_order/( dx2_en_t(i,j)+dx2_es_t(i,j) )
				flux_u_n(k,i,j)=-flux_u_n(k,i,j)*td_bh_order/( dx1_nw_t(i,j)+dx1_ne_t(i,j) )
			end do
			end do
			end do
			
			call o_diagn_time_mean(dim3_arr_in=flux_u_e, arr_out=mht_u, finalize_condition=(MOD(time_l,time_l_save_cp) == 0).and. (mixh_t_mode==2))	!If mixh_t_mode<2 then no finalization or rotation here. They will be in o_td_mixh.
			call o_diagn_time_mean(dim3_arr_in=flux_u_n, arr_out=mht_v, finalize_condition=(MOD(time_l,time_l_save_cp) == 0).and. (mixh_t_mode==2))
			
			!rotate heat flux arrays if finalized (if there is no laplace viscosity)
			if(MOD(time_l,time_l_save_cp) == 0 .and. mixh_t_mode==2) then
				do k=1,kl
					call tsc_ssc_rotate_v(	mht_u % collector3d(k,:,:), mht_v % collector3d(k,:,:),			&
								mht_u % collector3d(k,:,:), mht_v % collector3d(k,:,:),			&
								iwest(myrank)-1,ieast(myrank)+1,jsouth(myrank)-1,jnorth(myrank)+1)	
				end do
			end if
		
		
		end if
	end do	
	t_laplace2=t_laplace2*td_bh_order	
	end subroutine o_td_mixh_bh

	
	!--------------------------------------
	!	compute diffusivities (rememeber to multiply later by td_bh_order)
	!--------------------------------------
	subroutine o_td_mixh_ini
	use o_grid_module
	use o_td_module
	implicit none
	real delta,delta_equator
	
	ah_e=ah
	ah_n=ah
	ah_bh_e=ah_bh
	ah_bh_n=ah_bh

	!diffusivity coefficients scaling as in ORCA12
	delta_equator=x2_ssc_step*pi180*radius
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
		delta=min(dx2_es_t(i,j)+dx2_en_t(i,j),dx1_nw_v(i,j-1)+dx1_ne_v(i,j-1))
		if(ah_mode   ==1) ah_e(:,i,j,:)=ah*delta/delta_equator
		if(ah_bh_mode==1) ah_bh_e(:,i,j,:)=ah_bh*(delta/delta_equator)**3
	end do
	end do
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		delta=min(dx2_es_v(i-1,j)+dx2_en_v(i-1,j),dx1_nw_t(i,j)+dx1_ne_t(i,j))
		if(ah_mode   ==1) ah_n(:,i,j,:)=ah*delta/delta_equator
		if(ah_bh_mode==1) ah_bh_n(:,i,j,:)=ah_bh*(delta/delta_equator)**3
	end do
	end do


	!Scaling coefficient to provide numerical accuracy. Fluxes will be multiplied by it in the end of o_tb_mixh_bh
	td_bh_order=1.e9	
	
	ah_bh_e=ah_bh_e/td_bh_order
	ah_bh_n=ah_bh_n/td_bh_order

	end subroutine o_td_mixh_ini
	end module o_td_mixing