#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	module for variables
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

!----------------------------------------------------------------------
!	variables  used in thermodynamic part
!----------------------------------------------------------------------

	module o_td_module
	save

	real (kind=main_kind), allocatable:: t_f	(:,:,:,:)
	real (kind=main_kind), allocatable:: t_l	(:,:,:,:)
	real (kind=main_kind), allocatable:: t_laplace	(:,:,:,:)
	real (kind=main_kind), allocatable:: t_laplace2(:,:,:,:)
	real (kind=main_kind), allocatable:: t_laplace_pure(:,:,:,:)
	real (kind=main_kind), allocatable:: t_source	(:,:,:,:)
	real (kind=4), allocatable:: kh		(:,:,:,:)
	!real (kind=4), allocatable:: kh_v	(:,:,:,:)
	real (kind=4), allocatable:: efsolar	(:,:,:)
	real (kind=8), allocatable:: r_scalar	(:,:,:)
	!real (kind=4), allocatable:: sbl_depth (:,:) ! surface boundary layer depth in kpp mixing scheme
	!real (kind=4), allocatable:: bbl_depth (:,:) ! bottom boundary layer depth in kpp mixing scheme
	real (kind=4), allocatable:: ah_e	(:,:,:,:)
	real (kind=4), allocatable:: ah_n	(:,:,:,:)
	real (kind=4), allocatable:: ah_bh_e	(:,:,:,:)
	real (kind=4), allocatable:: ah_bh_n	(:,:,:,:)
	
!-------
!	scalar advection operator arrays
!-------
	real (kind=main_kind), allocatable:: flow_t_n	(:,:,:)
	real (kind=main_kind), allocatable:: flow_t_e	(:,:,:)
	real (kind=main_kind), allocatable:: flow_t_s	(:,:,:)
	real (kind=main_kind), allocatable:: flow_t_w	(:,:,:)
	real (kind=main_kind), allocatable:: flow_t_b	(:,:,:)
	real (kind=main_kind), allocatable:: adv_t	(:,:,:)

!------
!		parameters of solar penetration
!------
	real (kind=4) 	gsolar1,zsolar1,zsolar1v
!z	real (kind=4) 	gsolar2,zsolar2,zsolar2v
	real (kind=4) 	ah, c_ah_bh, ah_bh, c_ah

	integer (kind=4) scalar_scheme_advection
	integer (kind=4) scalar_scheme_time
	real(kind=4) albedo_ocean, albedo_ice
	integer ah_mode,ah_bh_mode
	real(4) td_bh_order	
	
	end module o_td_module



	subroutine allocate_o_td_module

	use o_grid_module
	use o_td_module
  
	implicit none

	allocate (t_f 		(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (t_l 		(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (t_laplace	(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (t_laplace2(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (t_laplace_pure(kl  ,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (t_source	(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (kh		(kl  ,	iwest (myrank)  :ieast (myrank), 	jsouth(myrank):jnorth(myrank),		ml))
	!allocate (kh_v		(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1, ml))
	allocate (efsolar	(kl+1,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1))
	t_f      = 0.
	t_l      = 0.
	t_laplace=0.
	t_laplace2=0.
	t_laplace_pure=0.
	t_source = 0.
	kh	 = 0.
	!kh_v = 0.
	efsolar	 = 0.

	allocate (r_scalar	(kl,	iwest (myrank)  :ieast (myrank),	jsouth(myrank)  :jnorth(myrank)  ) )
	!allocate (sbl_depth	(		iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )
	!allocate (bbl_depth	(		iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (ah_e		(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (ah_n		(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (ah_bh_e	(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	allocate (ah_bh_n	(kl,	iwest (myrank)-1:ieast (myrank)+1, 	jsouth(myrank)-1:jnorth(myrank)+1,	ml))
	r_scalar = 0.
	!sbl_depth = 0.
	!bbl_depth = 0.

!-------
!	scalar advection operator arrays
!-------
	allocate (flow_t_n	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_e	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_s	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_w	(kl  ,	iwest (myrank)-2:ieast (myrank)+2,	jsouth(myrank)-2:jnorth(myrank)+2) )
	allocate (flow_t_b	(0:kl ,	iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (adv_t		(kl  ,	iwest (myrank)-1:ieast (myrank)+1,	jsouth(myrank)-1:jnorth(myrank)+1) )
	flow_t_n = 0.
	flow_t_e = 0.
	flow_t_s = 0.
	flow_t_w = 0.
	flow_t_b = 0.
	adv_t = 0.	

	ah_e=0.
	ah_n=0.
	ah_bh_e=0.
	ah_bh_n=0.
	
	return
	end

