
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	compute sources and solve eqations
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------


!-------
! compute heat and salt sources under the (1-Aice) part 
! of the T-cell sea surface
!-------

	subroutine o_td_scalar(ctime)

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_tf_module
	use ice_module
	use o_d_vmc_1_module
	use o_vmc_GM_skew_module	
	
	implicit none
	
	real (kind=4)	ctime
	real (kind=8) arix(kl),brix(kl),crix(kl),rrix(kl),frix(kl)
	real (kind=8) bbrix(kl),rrrix(kl),cc(kl)
	integer kk, kz1, kz2
	
	select case (mixh_t_mode)	!if mixh_t_mode==2 then just t_laplace2=t_laplace2
	case(0)
		t_laplace2 =  t_laplace
	case(1)
		t_laplace2 =  t_laplace2+t_laplace
	end select
	!r_scalar = 0.

	do m = 1,ml
!-------
!	Compute the right side for implicit equation on 'l+1' timestep
!	in r_scalar we collect K*dx3*ct3=K*m3/(dx*dy) or psu*dx3*ct3 from various sources (advection, diffuzion, surface fluxes,...)
!	i.e. we collect total physical content of the box divided by dx*dy
!-------
	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1,kt(i,j)
	r_scalar(k,i,j)= t_f(k,i,j,m) + ctime*htsec * (t_laplace2(k,i,j,m) + ct3(k,i,j) * t_source(k,i,j,m) * dx3_t_c(k+1,i,j) )  !t_f and t_laplace2 are already implicitly multiplied by dx3*ct3, while t_source isn't
	end do
	end do
	end do

	!If GM is used then add offdiagonal fluxes
	if(v_mix_mod == 4) then
		do j = jsouth(myrank)	,jnorth(myrank)
		do i = iwest (myrank)	,ieast (myrank)
		do k = 1,kt(i,j)
			r_scalar(k,i,j)=r_scalar(k,i,j)+ctime*htsec*ct3(k,i,j)*offdiagonal_GM_fluxes_sum(k,i,j,m)
		end do
		end do
		end do
	end if
!-------
! Part 4.2	Solve the implicit equation for scalar function
!		-A(k)*T(k-1) + C(k)*T(k) - B(k)*t(k+1) = R(k)
!-------
		kk=1
		do j = jsouth(myrank)	,jnorth(myrank)
		do i = iwest (myrank)	,ieast (myrank)
			if( kt(i,j).gt.0 ) then
				!-------
				! Part 4.3.	Compute the matrix terms
				!-------

				k = 1
				kz1 = 1
				kz2 = kt(i,j)

				arix(1) = 0.0
				do k = 2,kz2
					arix(k) = kh(k,i,j,m)*ct3(k,i,j)*hzv(k)! *hkm(k)
				end do

				do k = 1,kz2-1
					brix(k) = kh(k+1,i,j,m)*ct3(k+1,i,j)*hzv(k+1) !*hkp(k)
				end do
				brix(kz2) = 0.

				if(kz2.ge.2) then
				!-------
				!	This part is valid for the Crunk-Nicholson scheme for vertical diffusion 
				!	here we add an 1/2 of vertical diffusion for explicit time-step
				!	if you want to use completely implicit time scheme then
				!	comment the following '**' operators before the net comments
				!-------
					do k = 1,kz2
						arix(k) = ctime*htsec*0.5*arix(k)
						brix(k) = ctime*htsec*0.5*brix(k)
						crix(k) = arix(k)+brix(k)
					end do

					k = 1
					cc(k) =	crix(k)*t_c(k  ,i,j,m)-brix(k)*t_c(k+1,i,j,m)
					do k = 2,kz2-1
						cc(k) = -arix(k)*t_c(k-1,i,j,m)+crix(k)*t_c(k  ,i,j,m)-brix(k)*t_c(k+1,i,j,m)
					end do
					k = kz2
					cc(k) = -arix(k)*t_c(k-1,i,j,m)+crix(k)*t_c(k  ,i,j,m)

					do k = 1,kz2
						r_scalar(k,i,j) = r_scalar(k,i,j)-cc(k)
					end do
				end if
			
				!-------
				!		The end of part concerning to the Crunk-Nickolson scheme
				!-------
				do k = 1,kz2
					crix(k) = arix(k)+brix(k)+ct3(k,i,j)*dx3_t_f(k+1,i,j)
				end do
	
				!-------
				! Part 4.4.	Solve the Matrix Equation by factorising Method
				!-------
				if(kz2.eq.1) then
					k = 1
					t_f(k,i,j,m) = r_scalar(k,i,j) / crix(k)
				end if
		
				if(kz2.ge.2) then
					do k = 1,kz2
						rrix(k) = r_scalar(k,i,j)
					end do
					call factordp(kz1,kz2,arix,brix,crix,rrix,frix,bbrix,rrrix)
					do k = 1,kz2
						t_f(k,i,j,m) = frix(k)
					end do
				end if
			end if
		end do	
		end do	

	end do	!do m=1,ml
	return
	end
