
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	heat source due to solar radiation penetration into the sea
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

!------
! Parameterisation of dependance of solar penetration on the bottom depth
! efsolar at bottom = 0, that means no solar radiation 
! penetrates through bottom
!------

	subroutine o_td_solar

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module

	!use shared_module

	implicit none

	real (kind=4) zsolar (iwest(myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)   
	real (kind=4) zsolarv(iwest(myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1)   
	real (kind=4) depth

	efsolar	= 0.
	zsolar	= 0.
	zsolarv	= 0.
	k = 1

!------
!	solar radiance penetration depends on wavelength
!	1. for long-wave range e-folding length scale 'zsolar2'~ 1-3m
!		long-wave solar radiation penetrates only in upper 1-2-3 m, that is why 
!		we assume that all this energy will be accumulated in upper cell layer
!	2. for short-wave range (ultraviolet, visible) e-folding length scale 'zsolar1'~ 15m!	
!	only short-wave will penetrate in deeper layers
!------

!------
!#1. 	attenuation function at sea surface solar radiation
!------

	k = 1
	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
		if (kt(i,j) >= 1) then
			efsolar(k,i,j) = 1.
		end if
	end do
	end do

!------
!#2. 	add attenuation function for short-wave part of solar radiation
!------
!------
!#2.1 	parameterization of e-folding length scale for shallow regions 
!------

	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
		if (kt(i,j) >= 1) then
			depth = zw(kt(i,j)+1) + REAL(ssl_c(i,j))
			zsolar(i,j) = zsolar1
			if (depth < 100.) zsolar(i,j) = zsolar1*( depth/100. )
			zsolarv (i,j) = 1./zsolar (i,j)
		end if
	end do
	end do


!------
!#2.2. 	add attenuation function for short-wave part of solar radiation
!------
	do j = jsouth(myrank), jnorth(myrank)+1
	do i = iwest (myrank), ieast (myrank)+1
	do k = 1,kl
		!remember that efsolar at bottom = 0, that means no solar radiation 
		!if(ABS(ssl_c(i,j))   > 60) write(*,*) 'Error in solar: ssl_c become too big:', time_l, ssl_c(i,j), i,j
		!if(ABS(zsolarv(i,j)) > 60) write(*,*) 'Error in solar: zsolarv become too big:', time_l, zsolarv(i,j), i,j
	
		if (k < kt(i,j)) then
			depth = zw(k+1) + REAL(ssl_c(i,j))
			efsolar(k+1,i,j) = gsolar1 * EXP(-depth * zsolarv(i,j))
		end if
	end do
	end do
	end do

	return
	end
