#include "o_defines.f90"


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	heat and salt sources at air-sea interface
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------


!-------
! Compute heat and salt sources at air-sea interface
! as if there is no ice
!-------

	subroutine o_td_source
	use o_par_module
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_tf_module
	use o_td_module
	use ice_module
	use test_ocean_module
	implicit none

	real c4217d, c4217dv,Vpiston,freez_point
	real(8) local_integral,local_area,global_average_sflux(1)
	logical,parameter :: sst_relax=.false.
	

	c4217d = 4217.*densit		!4217 J/kg/K - water heat capacity
	c4217dv = 1./c4217d
	Vpiston = 0. !50./4./365./24./3600.   !50m/4years
	t_source = 0.  !Changed 26.08.16
!-------
! Heat source from air-sea interface: t_source=[K/s]
!-------
	m = 1
	k = 1
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest (myrank), ieast (myrank)
		if(kt(i,j) > 0) then
			t_source(k,i,j,m) = c4217dv * (										&
     			(ssr(i,j)*(1.-albedo_ocean)*( efsolar(k,i,j)-efsolar(k+1,i,j) )	&
     			+str(i,j)														&
     			+sns(i,j)														&
     			+ltn(i,j)	) / dx3_t_c(k+1,i,j) )
		end if
	end do
	end do

	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest (myrank), ieast (myrank)
	do k = 2,kt(i,j)
		t_source(k,i,j,m) = c4217dv*((ssr(i,j)*(1.-albedo_ocean)*(efsolar(k,i,j)-efsolar(k+1,i,j)))*hzwv(k+1))
	end do
	end do
	end do

!-------
! Heat source: modify source due to ice covered
!-------
	m = 1
	do j = jsouth(myrank)	,jnorth(myrank)
	do i = iwest (myrank)	,ieast (myrank)
	do k = 1,kt(i,j)
		t_source(k,i,j,m) = t_source(k,i,j,m)*(1. - aice(i,j))
	end do
	end do
	end do
	
!-------
! Heat source due to temperature of precipitation and melting of snow (it has been already multiplied by (1-aice) in o_tf_par_read_core)
!-------	
	k=1
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest (myrank), ieast (myrank)
		if(kt(i,j) > 0) then
			t_source(k,i,j,m)=t_source(k,i,j,m)+prec_tflux(i,j)/dx3_t_c(k+1,i,j)
		end if
	end do
	end do
	
!---------
! Adding fluxes from ice-ocean interface. CICE fluxes are per square unit of ice (not gridbox)
!---------
	if(trim(ice_model_name)=="ICE_CICE") then
		k=1
		m=1
		do j = jsouth(myrank)	,jnorth(myrank)
		do i = iwest (myrank)	,ieast (myrank)
			if(kt(i,j) > 0) t_source(k,i,j,m) = t_source(k,i,j,m) + c4217dv*	&
				(	ice_ocn_ssr(i,j) * ( efsolar(k,i,j)-efsolar(k+1,i,j) ) +		&
					ice_ocn_heat(i,j)		)/dx3_t_c(k+1,i,j)
		end do
		end do
		
		do j = jsouth(myrank), jnorth(myrank)
		do i = iwest (myrank), ieast (myrank)
		do k = 2,kt(i,j)
			t_source(k,i,j,m) = t_source(k,i,j,m) + c4217dv*ice_ocn_ssr(i,j)*(efsolar(k,i,j)-efsolar(k+1,i,j))*hzwv(k+1)
		end do
		end do
		end do
		
		k=1
		m=2
		do j = jsouth(myrank)	,jnorth(myrank)
		do i = iwest (myrank)	,ieast (myrank)
			if(kt(i,j) > 0) t_source(k,i,j,m) = t_source(k,i,j,m) + ice_ocn_salt(i,j)/densit/dx3_t_c(k+1,i,j)
		end do
		end do
		
		
	end if
	
!-------
! Salt source due to NCAR CORE relaxation or Heat source due to Interim relaxation: t_source=[psu/s] (see *) or [K/s]
!-------
	if((trim(lnd_model_name)=="LND_CORE".or.trim(lnd_model_name)=="LND_DAS") .and. Vpiston.gt.1e-9) then
		k=1
		m=2
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			if(k.le.kt(i,j)) then
				t_source(k,i,j,m) = Vpiston*(surface_salt(i,j)-S_ref-t_c(k,i,j,m))	!(*) it is [psu*m3/m2/s] division by dx3 will be later after averaging  
			end if
		end do
		end do	
	
		!-----------------------------------------------------------------------------------
		!	calculating and subtracting average salt restoring flux
		!----------------------------------------------------------------------------------	
		local_integral=	sum( t_source(k,iwe:iea,jso:jno,m)*	ct3(1,iwe:iea,jso:jno)*area_tbox(iwe:iea,jso:jno) )
		local_area=	sum( 					ct3(1,iwe:iea,jso:jno)*area_tbox(iwe:iea,jso:jno) )

#if CMF_VERSION == 2    
		call comp_p % reduce_itg(local_area, [real(kind=8) :: local_integral], global_average_sflux)
#elif CMF_VERSION == 3
		global_average_sflux = UtilsAllReduce(local_area, local_integral, ocn_comm)
#endif  			
	
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			if(k.le.kt(i,j)) then
				t_source(k,i,j,m) = (t_source(k,i,j,m)-global_average_sflux(1))/dx3_t_c(k+1,i,j)
			end if
		end do
		end do	
	end if
	
	sst_diff=nudge_sst-t_c(1,:,:,1)
	if(sst_relax) then
		k=1
		m=1
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			if(k.le.kt(i,j)) then
				!relaxing surface temperature with characteristic time = 3600*8 seconds.
				!no relaxation under ice
				if(nudge_sst(i,j).gt.-1.8) then
					!t_source(k,i,j,m) = t_source(k,i,j,m) + (nudge_sst(i,j)-t_c(k,i,j,m))*(1. - aice(i,j))/(3600.*8.)
				!else 
				!	if(t_c(k,i,j,1).gt. freez_point(t_c(k,i,j,2),0.d0)) &
				!		t_source(k,i,j,m) = t_source(k,i,j,m) + (freez_point(t_c(k,i,j,2),0.d0)-t_c(k,i,j,m))*(1. - aice(i,j))/(3600.*48.)
				end if
			end if
		end do
		end do		
	end if
	

#if CMF_VERSION == 2    
	call comp_halo_update_T (DIM4_t = t_source, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
	call HaloUpdateMaster % update(t_source(:,:,:,1), update_width = 1, grid_type = 'T')
	call HaloUpdateMaster % update(t_source(:,:,:,2), update_width = 1, grid_type = 'T')
#endif	
end
