
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	atmospheric boundary layer model
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Launiainen, J. and Vihma, T., 1990.
!	Derivation of turbulent surface fluxes – an iterative flux-profile method allowing arbitrary observing heights, 
!	Environ. Softw., 5(3), 113–124.
!--------------------------------------


!~~~~~~
	subroutine o_tf_abl_convert(	patmsi,
     &				tair,
     &				Tdew,
     &				sphumi,
     &				vobs,
     &				ts,
     &					tau,qhsns,qhltn,evapor,
     &					ncount,T10,q_10,rll)
!~~~~~~
!	input:	tdew	=>	tdew02(i,j)
!	output:	sphumi	=>	shumid(i,j)
!
!
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	real lambda

	z_v=10. !! look through the code if not
	z_t=2.
	z_q=2.

	rlinfi=10.
	iterat=25
	crit=0.001
	badarea1=0.5
	badarea2=1.6
	ncount=0

	tobs=tair+0.01*z_t
	t0=273.15+(tobs+ts)/2. ! mean absolute temperature of the surface layer
c	Patmo=1013.            ! atmospheric pressure [mb]
	patmo=patmsi/100.      ! si --> mb
	gravsi=9.8             ! acceleration due to gravity
	xk=0.405               ! von Karman constant 
	cp=1004.               ! specific heat
	lambda=2.5e6-2375.*ts

c	       calculation of 'ee' - saturation pressure of water vapour
c	if(Tdew.le.0.) ee=exp(-6141./(Tdew+273.15)+24.3)
c	if(Tdew.gt.0.) ee=exp(-6763.6/(Tdew+273.15)-4.9283*
c     +  log(Tdew+273.15)+54.23)
	if(Tdew.lt.0.) ee=exp(6141./(Tdew+273.15)-24.3)
	if(Tdew.ge.0.) ee=exp(6763.6/(Tdew+273.15)+4.9283*
     &	 log(Tdew+273.15)-54.23)

c	       calculation of 'esur' - saturation pressure
c       	at the temperature of a surface
c	if(Ts.le.0.) esur=exp(-6141./(Ts+273.15)+24.3)
c	if(Ts.gt.0.) esur=exp(-6763.6/(Ts+273.15)-4.9283*
c     +  log(Ts+273.15)+54.23)
	if(Ts.lt.0.) esur=exp(6141./(Ts+273.15)-24.3)
	if(Ts.ge.0.) esur=exp(6763.6/(Ts+273.15)+4.9283*
     &   log(Ts+273.15)-54.23)
	
c		calculation of specific humidity
c	sphumi=0.622*ee  /(Patmo-0.378*ee)   ! at a temperature obs. height
c	spsur =0.622*esur/(Patmo-0.378*esur) ! at a surface 
	sphumi=0.622/(Patmo*ee  -0.378)		!	ee=1./ee
	spsur =0.622/(Patmo*esur-0.378)		!	esur=1./esur

c	       Stanton number and Dalton number assumed equal


c	zr0=4.5e-5
c	zrtq=zr0


c	if(z_v.eq.10.) then
	v10=Vobs
	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
	zr0=10.*exp(-xk/sqrt(cdtmp))
	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if

	c4=gravsi/(t0*vobs*vobs)
	c31=ts-tobs
	c32=0.61*t0*(spsur-sphumi)
	c1=log(z_v/zr0)
	c3=c32/log(z_q/zrtq)+c31/log(z_t/zrtq)
	rll=-c4*c3*c1*c1

	c99=0.7*c4*z_v*z_v*(-c31/z_t-c32/z_q)

	if(c99.ge.badarea1) then
	if(c99.ge.badarea2) iterat=0
	do ii=1,iterat
	rll=rlinfi*real(ii)/real(iterat)
c	if(z_v.ne.10.) then
c	call v10z0iter(xk,vobs,z_v,rll,zr0,v10)
c	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
c	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if
	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	c21=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	c22=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	c3=c32/c22+c31/c21
	phi=-c4*c3*c1*c1
	if(phi.le.rll) then
	rll=0.5*(rll+phi)
	go to 559
	end if
	end do
	rll=rlinfi
	iterat=0
559	continue
	ncount=ii
	end if

	do ii=1,iterat

c	if(z_v.ne.10.) then
c	call v10z0iter(xk,vobs,z_v,rll,zr0,v10)
c	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
c	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if

	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	c21=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	c22=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	c3=c31/c21+c32/c22
	phi=-c4*c3*c1*c1

	c5=psimprime(zr0*rll)*zr0-psimprime(z_v*rll)*z_v
	c5=2.*c1*c3*c5
	c61=psihprime(zrtq*rll)*zrtq-psihprime(z_t*rll)*z_t
	c62=psihprime(zrtq*rll)*zrtq-psihprime(z_q*rll)*z_q
	c6=-c31*c61/(c21*c21)-c32*c62/(c22*c22)
	c6=c6*c1*c1
	phiprime=-c4*(c5+c6)

c	phiprime doesn't consider zr0 and zrtq dependence on rll,
c	which is the case if z_v .ne. 10.;
c	this probably can slow down convergence
	
	diff=(phi-rll)/(phiprime-1.)
	d1=phi-rll
	rll=rll-diff
	if(abs(d1).lt.crit) go to 555

	end do				!ii=1,iterat

555	continue

	ncount=ncount+ii

	rll=min(rll,rlinfi)

	rhol=349./t0 ! air density

	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	uscale=vobs*xk/c1
	tau=rhol*uscale*uscale
	
c	tau=tau/2.

	c2=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	qhsns=rhol*cp*uscale*xk*(tobs-ts)/c2
	c3=log(10./z_t)-psih(10.*rll)+psih(z_t*rll)
	t10=tobs+c3*(tobs-ts)/c2

	c2=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	evapor=-rhol*uscale*xk*(sphumi-spsur)/c2
	qhltn=-lambda*evapor
	c3=log(10./z_q)-psih(10.*rll)+psih(z_q*rll)
	q_10=sphumi+c3*(sphumi-spsur)/c2

	t10=t10-0.1
c	Tobs=Tobs-0.01*z_t	

c	hsens=hsens*rhol*cp*uscale
c	evapor=evapor*rhol*uscale
c	qhsns=-hsens
c	qhltn=-evapor*lambda
c	eva=evapor
	
	return
	end



c--------------------------------------
c	the only difference with convert20.f is that humidity 
c	is prescribed through specific humidity instead of dew point temperature
c--------------------------------------

	subroutine o_tf_abl_convert_sh(	patmsi,
     &					tair,
     &					sphumi,
     &					vobs,
     &					ts,
     &					tau,qhsns,qhltn,evapor,
     &					ncount,T10,q_10,rll)

	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	real lambda

	z_v=10. !! look through the code if not
	z_t=2.
	z_q=2.

	rlinfi=10.
	iterat=25
	crit=0.001
	badarea1=0.5
	badarea2=1.6
	ncount=0

	tobs=tair+0.01*z_t
	t0=273.15+(tobs+ts)/2. ! mean absolute temperature of the surface layer
c	Patmo=1013.            ! atmospheric pressure [mb]
	patmo=patmsi/100.      ! si --> mb
	gravsi=9.8             ! acceleration due to gravity
	xk=0.405               ! von Karman constant 
	cp=1004.               ! specific heat
	lambda=2.5e6-2375.*ts

c	       calculation of 'ee' - saturation pressure of water vapour
c	if(Tdew.lt.0.) ee=exp(6141./(Tdew+273.15)-24.3)
c	if(Tdew.ge.0.) ee=exp(6763.6/(Tdew+273.15)+4.9283*
c     &				log(Tdew+273.15)-54.23)

c	       calculation of 'esur' - saturation pressure
c       	at the temperature of a surface
	if(Ts.lt.0.) esur=exp(6141./(Ts+273.15)-24.3)
	if(Ts.ge.0.) esur=exp(6763.6/(Ts+273.15)+4.9283*
     &				log(Ts+273.15)-54.23)
	
c		calculation of specific humidity
c	sphumi=0.622/(Patmo*ee  -0.378)		!	ee=1./ee
	spsur =0.622/(Patmo*esur-0.378)		!	esur=1./esur

c	       Stanton number and Dalton number assumed equal


c	zr0=4.5e-5
c	zrtq=zr0


c	if(z_v.eq.10.) then
	v10=Vobs
	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
	zr0=10.*exp(-xk/sqrt(cdtmp))
	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if

	c4=gravsi/(t0*vobs*vobs)
	c31=ts-tobs
	c32=0.61*t0*(spsur-sphumi)
	c1=log(z_v/zr0)
	c3=c32/log(z_q/zrtq)+c31/log(z_t/zrtq)
	rll=-c4*c3*c1*c1

	c99=0.7*c4*z_v*z_v*(-c31/z_t-c32/z_q)

	if(c99.ge.badarea1) then
	if(c99.ge.badarea2) iterat=0
	do ii=1,iterat
	rll=rlinfi*real(ii)/real(iterat)
c	if(z_v.ne.10.) then
c	call v10z0iter(xk,vobs,z_v,rll,zr0,v10)
c	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
c	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if
	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	c21=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	c22=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	c3=c32/c22+c31/c21
	phi=-c4*c3*c1*c1
	if(phi.le.rll) then
	rll=0.5*(rll+phi)
	go to 559
	end if
	end do
	rll=rlinfi
	iterat=0
559	continue
	ncount=ii
	end if

	do ii=1,iterat

c	if(z_v.ne.10.) then
c	call v10z0iter(xk,vobs,z_v,rll,zr0,v10)
c	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
c	zrtq=10.*exp(-xk*sqrt(cdtmp)/(0.32e-3+0.63*cdtmp))
c	end if

	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	c21=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	c22=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	c3=c31/c21+c32/c22
	phi=-c4*c3*c1*c1

	c5=psimprime(zr0*rll)*zr0-psimprime(z_v*rll)*z_v
	c5=2.*c1*c3*c5
	c61=psihprime(zrtq*rll)*zrtq-psihprime(z_t*rll)*z_t
	c62=psihprime(zrtq*rll)*zrtq-psihprime(z_q*rll)*z_q
	c6=-c31*c61/(c21*c21)-c32*c62/(c22*c22)
	c6=c6*c1*c1
	phiprime=-c4*(c5+c6)

c	phiprime doesn't consider zr0 and zrtq dependence on rll,
c	which is the case if z_v .ne. 10.;
c	this probably can slow down convergence
	
	diff=(phi-rll)/(phiprime-1.)
	d1=phi-rll
	rll=rll-diff
	if(abs(d1).lt.crit) go to 555

	end do				!ii=1,iterat

555	continue

	ncount=ncount+ii

	rll=min(rll,rlinfi)

	rhol=349./t0 ! air density

	c1=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
	uscale=vobs*xk/c1
	tau=rhol*uscale*uscale
	
c	tau=tau/2.

	c2=log(z_t/zrtq)-psih(z_t*rll)+psih(zrtq*rll)
	qhsns=rhol*cp*uscale*xk*(tobs-ts)/c2
	c3=log(10./z_t)-psih(10.*rll)+psih(z_t*rll)
	t10=tobs+c3*(tobs-ts)/c2

	c2=log(z_q/zrtq)-psih(z_q*rll)+psih(zrtq*rll)
	evapor=-rhol*uscale*xk*(sphumi-spsur)/c2
	qhltn=-lambda*evapor
	c3=log(10./z_q)-psih(10.*rll)+psih(z_q*rll)
	q_10=sphumi+c3*(sphumi-spsur)/c2

	t10=t10-0.1
c	Tobs=Tobs-0.01*z_t	

c	hsens=hsens*rhol*cp*uscale
c	evapor=evapor*rhol*uscale
c	qhsns=-hsens
c	qhltn=-evapor*lambda
c	eva=evapor
	
	return
	end





c	subroutine v10z0iter(xk,vobs,z_v,rll,zr0,v10)
c
c	double precision c1,c2,cdtmp,phi,phiprime,dd
c
c	do j=1,4
c	c1=log(10./zr0)-psim(10.*rll)+psim(zr0*rll)
c	c2=log(z_v/zr0)-psim(z_v*rll)+psim(zr0*rll)
c	v10=vobs*c1/c2
c	cdtmp=(1.1-0.017*v10+0.0025*v10*v10)*1.e-3
c	phi=10.*exp(-xk/sqrt(cdtmp))
c	phiprime=zr0*xk*0.5
c	phiprime=phiprime/(cdtmp*sqrt(cdtmp))
c	phiprime=phiprime*(-0.017+0.005*v10)*1.e-3
c	phiprime=phiprime*vobs*(rll*psimprime(zr0*rll)-1./zr0)
c	phiprime=phiprime*(c2-c1)/(c2*c2)
c	dd=(phi-zr0)/(phiprime-1.)
c	zr0=zr0-dd
c	if(abs(dd).le.zr0*1.e-6) return
c	end do
c
c	return
c	end

	function psim(t)
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	if(t.gt.0.) then
c	psim=-0.7*t-0.75*(t-100./7.)*exp(-0.35*t)-75./7.
	psim=-0.7*t-0.75*(t-100./7.)*exp(-0.35*t)
	return
	end if
	g1=16.
	tm0=1.-g1*t
	tm1=sqrt(tm0)
	tm2=sqrt(tm1)
c	psim=log(0.125*(1.+2.*tm2+tm1)*(1.+tm1))-2.*(atan(tm2)-atan(1.))
	psim=log((1.+2.*tm2+tm1)*(1.+tm1))-2.*atan(tm2)
	return
	end
	
	function psih(t)
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	if(t.gt.0.) then
c	psih=-0.7*t-0.75*(t-100./7.)*exp(-0.35*t)-75./7.
	psih=-0.7*t-0.75*(t-100./7.)*exp(-0.35*t)
	return
	end if
	g2=16.
	tm0=sqrt(1.-g2*t)
c	psih=2.*log(0.5*(1.+tm0))
	psih=2.*log(1.+tm0)
	return
	end

	function psimprime(t)
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	if(t.gt.0.) then
	psimprime=-0.7-exp(-0.35*t)*(4.5-t*0.35*0.75)
	return
	end if
	g1=16.
	tm0=1.-g1*t
	tm1=sqrt(tm0)
	tm2=sqrt(tm1)
	psimprime=-g1/(tm0+tm1+tm2+tm1*tm2)
	return
	end

	function psihprime(t)
	IMPLICIT REAL(A-H, O-Z), INTEGER(I-N)
	if(t.gt.0.) then
	psihprime=-0.7-exp(-0.35*t)*(4.5-t*0.35*0.75)
	return
	end if
	g2=16.
	tm0=1.-g2*t
	tm1=sqrt(tm0)
	psihprime=-g2/(tm1+tm0)
	return
	end
	
