
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	Govern the input of atmospheric forcing parameters
!
! authors:
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------

!===================================================
! Govern the input of atmospheric forcing parameters
!===================================================

	subroutine o_tf_main

	use o_tf_module
	use o_basic_module
	use o_grid_module
	use o_td_module
	use ice_module
	use o_par_module 

	implicit none

!-------
!#1.	input air-sea parameters or fluxes
!-------

	select case (atm_model_name)
	
		case("ATM_NCAR","ATM_INTERIM","ATM_GFS","ATM_INMCM4")
			!print *,'-----------------------------Running common ABL----<<<<<<<<<<<<<<'
			call o_tf_par_read_core
			call o_tf_abl_drive
		case("ATM_INM")
			if (ao_fluxes_mode == 1) then
				call INM_atm_fluxes_convert
			else if (ao_fluxes_mode == 2) then
				call o_tf_par_read_core
				call o_tf_abl_drive
			else
				Stop 'o_tf_main ao_fluxes_mode selection error'
			end if
		case default
			if(atm_model.eq.1) then 
				call o_tf_par_art
				call o_tf_abl_drive
			end if
			if(atm_model.gt.1) call o_tf_flux_art
	end select
	end
	
!===================================================
! Convert INM tmospheric fluxes (received in o_ini_reg_data) to ocean model units
!===================================================	
	subroutine INM_atm_fluxes_convert
	use o_tf_module
	use o_grid_module
	use o_basic_module
	use atm_module
	use o_par_module
	use test_ocean_module
	implicit none

		!======get fluxes from arrays-receivers and make necessary preprocessing======
		
		!rotate wind stress to bipolar grid lines
		call ssc_tsc_rotate_t(	atm_ocn_windstr_x,atm_ocn_windstr_y,wind_stress_x_t,wind_stress_y_t,iwest(myrank)-1,ieast(myrank)+1,jsouth(myrank)-1,jnorth(myrank)+1)  
		wind_stress_x_t=min(wind_stress_x_t,3.)
		wind_stress_x_t=max(wind_stress_x_t,-3.)
		wind_stress_y_t=min(wind_stress_y_t,3.)
		wind_stress_y_t=max(wind_stress_y_t,-3.)
			
		prec=prec_from_atm/(atm_ocn_mapping*60.)/1000.			!prec is received as kg/m2 (integrated through the period between mappings) and must be converted to m/s
		evap=evap_from_atm/(atm_ocn_mapping*60.)
		sns=sns_from_atm
		ltn=ltn_from_atm
		ssr=ssr_from_atm
		str=str_from_atm
		tair10=T_10_MOD_atm-273.15

		!======maintaining strict flux conservativity=======
		if(ao_conserv_fluxes) then
			!call flux_land_correct(wind_stress_x_t)
			!call flux_land_correct(wind_stress_y_t)
			call flux_land_correct(prec)
			call flux_land_correct(evap)
			call flux_land_correct(sns)
			call flux_land_correct(ltn)
			call flux_land_correct(ssr)
			call flux_land_correct(str)
			call flux_land_correct(t_atm)	!test array, not used in real calculations
		
			do j = jsouth(myrank), jnorth(myrank)
			do i = iwest (myrank), ieast (myrank)
				if(kt(i,j).ne.0) then
					wind_stress_x_t(i,j)=wind_stress_x_t(i,j)/ct3(1,i,j)
					wind_stress_y_t(i,j)=wind_stress_y_t(i,j)/ct3(1,i,j)
					prec(i,j)=prec(i,j)/ct3(1,i,j)
					evap(i,j)=evap(i,j)/ct3(1,i,j)
					sns(i,j)=sns(i,j)/ct3(1,i,j)
					ltn(i,j)=ltn(i,j)/ct3(1,i,j)
					ssr(i,j)=ssr(i,j)/ct3(1,i,j)
					str(i,j)=str(i,j)/ct3(1,i,j)
					temp2(i,j)=t_atm(i,j)/ct3(1,i,j)	!diagnostic output
				end if
			end do
			end do
		end if
			
		!prec and evap must be exchanged (because water_ssf is used with ears in sweq)
		!heat fluxes need not be exchanged
#if CMF_VERSION == 2    
		call comp_halo_update_T4 (DIM2_t = wind_stress_x_t, update_width = 1, change_sign_on_bipolar = -1)  
		call comp_halo_update_T4 (DIM2_t = wind_stress_y_t, update_width = 1, change_sign_on_bipolar = -1)
		call comp_halo_update_T4 (DIM2_t = prec, update_width = 1, change_sign_on_bipolar = 1)  
		call comp_halo_update_T4 (DIM2_t = evap, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
		call HaloUpdateMaster % update(wind_stress_x_t, update_width = 1, grid_type = 'T', change_sign_on_bipolar = .TRUE.)
		call HaloUpdateMaster % update(wind_stress_y_t, update_width = 1, grid_type = 'T', change_sign_on_bipolar = .TRUE.)
		call HaloUpdateMaster % update(prec, update_width = 1, grid_type = 'T')
		call HaloUpdateMaster % update(evap, update_width = 1, grid_type = 'T')
#endif 	
		!interpolate wind stress to V points (used in o_d_motor)
		do j = jsouth(myrank), jnorth(myrank)
		do i = iwest (myrank), ieast (myrank)
			if(kv(i,j) > 0) wind_stress_x_v(i,j)=0.25*(	(wind_stress_x_t(i+1,j+1)+	&
									wind_stress_x_t(i  ,j  ))+	&
									(wind_stress_x_t(i+1,j  )+	&
									wind_stress_x_t(i  ,j+1)) )
			if(kv(i,j) > 0)  wind_stress_y_v(i,j)=0.25*(	(wind_stress_y_t(i+1,j+1)+	&
									wind_stress_y_t(i  ,j  ))+	&
									(wind_stress_y_t(i+1,j  )+	&
									wind_stress_y_t(i  ,j+1)) )
		end do
		end do
	end
	
!========================================================================================
! Move to ocean the fluxes from atm that have occasionally fallen to land
!========================================================================================
	subroutine flux_land_correct(flux)
	use o_grid_module
	use o_tf_module
	!use o_basic_module
	implicit none
	
		real flux(iwest(myrank)-1:ieast(myrank)+1,jsouth(myrank)-1:jnorth(myrank)+1)
		integer ee,ww,ss,nn,rad,i1,j1
		
		do j = jsouth(myrank), jnorth(myrank)
		do i = iwest (myrank), ieast (myrank)
			if(flux(i,j).ne.0 .and. kt(i,j)==0) then
				if(inew(i,j)==0)  then
					! Search with increasing radius
					do rad = 1, 100
						ss = MAX(j-rad, jsouth(myrank))
						nn = MIN(j+rad, jnorth(myrank))
						ww = MAX(i-rad, iwest(myrank))
						ee = MIN(i+rad, ieast(myrank))
			
						if(ANY(kt(ww:ee, ss:nn) > 0)) then
							do j1=ss,nn
							do i1=ww,ee
								if(kt(i1,j1)>0) then
									inew(i,j)=i1
									jnew(i,j)=j1
									write(*,'(a,x,6i,f8.3)'),'----Translating ',i,j,inew(i,j),jnew(i,j),kt(i,j),kt(i1,j1),flux(i,j)
									goto 1
								end if
							end do
							end do
						end if
					end do
					write(*,'(a,2i)') 'Warning: translation not found for point ',i,j
					inew(i,j)=-1	!(i,j) inspected but no route found
				end if
				
1				if(inew(i,j)>0) then	!means that this (i,j) has been inspected and obtained a translation route 
					flux(inew(i,j),jnew(i,j))=flux(inew(i,j),jnew(i,j))+flux(i,j)*area_tbox(i,j)/area_tbox(inew(i,j),jnew(i,j))
					flux(i,j)=0.
				end if
			end if
		end do
		end do
	end
				
