
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	ocean thermodynamic equations
!	module for atmosphere - ocean flux variables
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------


!~~~~~~
	module o_tf_module
!~~~~~~

	implicit none
	save

!------
!		air-sea flux array
!------
	real (kind=4), allocatable:: ssr		(:,:)
	real (kind=4), allocatable:: str		(:,:)
	real (kind=4), allocatable:: sns		(:,:)
	real (kind=4), allocatable:: ltn		(:,:)
	real (kind=4), allocatable:: prec		(:,:)
	real (kind=4), allocatable:: prec_from_atm	(:,:)
	real (kind=4), allocatable:: sns_from_atm	(:,:)
	real (kind=4), allocatable:: ltn_from_atm	(:,:)
	real (kind=4), allocatable:: ssr_from_atm	(:,:)
	real (kind=4), allocatable:: str_from_atm	(:,:)
	real (kind=4), allocatable:: prec_tflux		(:,:)
	real (kind=4), allocatable:: t_atm		(:,:)
	real (kind=4), allocatable:: evap		(:,:)
	real (kind=4), allocatable:: evap_from_atm	(:,:)
	real (kind=4), allocatable:: wind_stress_x_t	(:,:)
	real (kind=4), allocatable:: wind_stress_y_t	(:,:)
	real (kind=4), allocatable:: wind_stress_m_t	(:,:)
	real (kind=4), allocatable:: wind_speed_x_t	(:,:)
	real (kind=4), allocatable:: wind_speed_y_t	(:,:)
	real (kind=4), allocatable:: wind_speed_m_t	(:,:)
	real (kind=4), allocatable:: wind_speed_a_s	(:,:)
!	real (kind=4), allocatable:: clouds		(:,:)
	real (kind=4), allocatable:: tair02		(:,:)
	real (kind=4), allocatable:: tdew02		(:,:)
	real (kind=4), allocatable:: shumid		(:,:)
	real (kind=4), allocatable:: patm		(:,:)
	real (kind=4), allocatable:: tair10		(:,:)
	real (kind=4), allocatable:: q10		(:,:)
!z	real (kind=4), allocatable:: relhum10		(:,:)
	real (kind=4), allocatable:: wind_stress_x_v	(:,:)
	real (kind=4), allocatable:: wind_stress_y_v	(:,:)
	real (kind=4), allocatable:: runoff		(:,:)
	real (kind=4), allocatable:: runoff2		(:,:)
	real (kind=4), allocatable:: surface_salt	(:,:)
	real (kind=4), allocatable:: sst		(:,:)	
	real (kind=4), allocatable:: sst_diff		(:,:)	!diagnostic: sst_model-sst_forcing
	real (kind=4), allocatable:: nudge_sst		(:,:)	
	real (kind=4), allocatable:: nudge_aice		(:,:)	
	real (kind=4), allocatable:: total_surf_heat	(:,:)
	real (kind=4), allocatable:: under_mld_ssr	(:,:)
	integer, allocatable:: inew	(:,:)			!point to where fluxes are translated from land point (i,j) 
	integer, allocatable:: jnew	(:,:)
!------
!	array used in atmospheric boundary layer model (subr. convert)
!------
	real (kind=4), allocatable:: rl			(:,:)

!------
!	periods for data exchange in [min]
!					60*1     min =    60 min ~> 1h, 
!					60*2     min =   120 min ~> 2h, 
!					60*6     min =   360 min ~> 6h, 
!					60*24    min =  1440 min ~> 1d, 
!					60*24*5  min =  7200 min ~> 5d, 
!					60*24*10 min = 14400 min ~> 10d, 
!					60*24*30 min = 43200 min ~> 30d
!------
	integer :: atm_ocn_mapping != 120  !in minutes
	integer :: ocn_atm_mapping != 240  !in minutes
	
	integer :: ice_ocn_mapping != 20 !24
	integer :: ocn_ice_mapping != 20 !24

	integer ao_fluxes_mode		!read from namelist, 1 - ocean absorbs fluxes calculated by SLAV, 2 - by INMIO
	logical ao_conserv_fluxes	!read from namelist, is the interpolation conservative (true) or smoothing (false)
	
	end module o_tf_module

!~~~~~~
	subroutine allocate_o_tf_module
!~~~~~~

	use o_grid_module
	use o_tf_module


!------
!		air-sea flux array
!------
	allocate (ssr		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (str		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (sns		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (ltn		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (prec		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (evap_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (ltn_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (sns_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (ssr_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (str_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (prec_tflux	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (t_atm		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )	
	allocate (evap		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )	
	allocate (prec_from_atm	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )		
!	allocate (clouds	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (tair02	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (tdew02	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (shumid	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (patm		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (tair10	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (q10		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (runoff	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (runoff2	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (surface_salt	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )	
	allocate (nudge_sst		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (nudge_aice	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (sst		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (sst_diff	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (total_surf_heat(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (under_mld_ssr	(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (inew		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (jnew		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	
	ssr		=0.
	str		=0.
	sns		=0.
	ltn		=0.
	prec		=0.
	prec_from_atm	=0.
	sns_from_atm	=0.
	ltn_from_atm	=0.
	ssr_from_atm	=0.
	str_from_atm	=0.
	prec_tflux	=0.
	evap		=0.
	evap_from_atm	=0.	
!	clouds		=0.
	tair02		=0.
	tdew02		=0.
	shumid		=0.
	patm		=0.
	tair10		=0.
	q10		=0.
	runoff		=0.
	surface_salt	=0.
	nudge_sst	=0.
	nudge_aice	=0.
	runoff2		=0.
	sst		=0.
	sst_diff	=0.
	total_surf_heat =0.
	under_mld_ssr 	=0.
	inew=0
	jnew=0

	allocate (wind_stress_x_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_stress_y_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_stress_m_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_speed_x_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_speed_y_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_speed_m_t(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_speed_a_s(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_stress_x_v(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	allocate (wind_stress_y_v(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	
	wind_stress_x_t	=0.
	wind_stress_y_t	=0.
	wind_stress_m_t	=0.
	wind_speed_x_t 	=0.
	wind_speed_y_t 	=0.
	wind_speed_m_t 	=0.
	wind_speed_a_s 	=0.
	wind_stress_x_v	=0.
	wind_stress_y_v	=0.

!------
!		array used in atmospheric boundary layer model (subr. convert)
!------
	allocate (rl		(	iwest (myrank)-1:ieast (myrank)+1, jsouth(myrank)-1:jnorth(myrank)+1) )
	rl	= 0.

	end
