
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	CORE data processing and additional routines
!
!
! authors: 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!--------------------------------------



!~~~~~~~
	subroutine o_tf_par_read_core
!~~~~~~~
	use test_ocean_module
	use o_grid_module
	use o_td_module
	use o_tf_module
	use o_basic_module
	use o_par_module
	use ice_module
	
	implicit none
	integer rad,sou,nor,wes,eas,ii,jj
	real(8) local_integral,local_area,global_average_sswf(1)
	real :: wind_threshold=25.
	real wind_scaling
	
	!---------
	!copying data from test ocean arrays to native ocean arrays. 
	!Here will be also temporal interpolation
	!Rain and snow are temporarily merged into prec
	!---------
	if(mod(minday,24).eq.0 .and. mod(time_l-1,minday/24) .ne.0 ) return
	
	if(lnd_model_name=="LND_CORE") then
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			if(kt(i,j)==0 .and. runoff(i,j).gt.0.) then  !if some runoff occasionally comes to land move it to nearest ocean point
				do rad = 1, MAX(ieast(myrank)-iwest(myrank),jnorth(myrank)-jsouth(myrank))
					sou = MAX(j-rad, jsouth(myrank))
					nor = MIN(j+rad, jnorth(myrank))
					wes = MAX(i-rad, iwest(myrank))
					eas = MIN(i+rad, ieast(myrank))

					if(ANY(kt(wes:eas, sou:nor) .gt. 0)) then
						do ii=wes,eas
						do jj=sou,nor
							if(kt(ii,jj).gt.0) runoff(ii,jj)=runoff(ii,jj)+runoff(i,j)/COUNT(kt(wes:eas, sou:nor) .gt. 0)
						end do
						end do
						runoff(i,j)=0.
						exit
					else !if(time_l_in_run.lt.minday/6) then
						print *,'Warning: found no ocean points for runoff from ',i,j
					end if
				end do
			end if
		end do
		end do
	end if
	
	
	select case (atm_model_name)
	case("ATM_NCAR")
				
		!wind_speed_x_t=U_10_MOD_atm
		!wind_speed_y_t=V_10_MOD_atm
		wind_speed_x_t=max(-15.,U_10_MOD_atm)	!Outdated operation since there is wind_threshold now
		wind_speed_x_t=min(15.,wind_speed_x_t)
		wind_speed_y_t=max(-15.,V_10_MOD_atm)
		wind_speed_y_t=min(15.,wind_speed_y_t)
		tair02=T_10_MOD_atm-273.15
		shumid=Q_10_MOD_atm
		prec=(RAIN_atm+SNOW_atm)/densit
		str=LWDN_MOD_atm-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		ssr=SWDN_MOD_atm
		prec_tflux=-SNOW_atm*merge(1.-aice,1.,trim(ice_model_name)=='ICE_CICE')/densit*78.	![K*m/s] - temperature flux by melting snow. 78=melting heat(J/kg)/heat capacity(J/kg/K)  

	case ("ATM_INTERIM")		!Interim data is stored with scale factors and offsets (see netcdf documentation)
		!------------------for MHI 2010.01.01 - 2018.06.30 ------------------------------
		wind_speed_x_t=U_10_MOD_atm*0.00424590398992683+34.2944468722238
		wind_speed_y_t=V_10_MOD_atm*0.00277015278912139+3.53934826100778
		tair02=T_10_MOD_atm*0.00203636576556611+258.458622320047-273.15
		tdew02=Q_10_MOD_atm*0.00195197522964933+243.274139979182-273.15
		nudge_sst=SNOW_atm*0.000632885012369722+288.664844690306-273.15
		!prec=(RAIN_atm*6.54784668362868e-06+0.214546744338633+runoff*2.77622505744363e-06+0.0909657902321978	)/12./3600.  !prec and runoff are accumulated for 12 hours
		prec=(RAIN_atm*6.54784668362868e-06+0.214546744338633							)/12./3600.  !runoff is taken from LND_CORE
		str=(LWDN_MOD_atm*403.806889277158+12768598.6590554)/12./3600.-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		ssr=(SWDN_MOD_atm*559.588115911068+18335464.205942)/12./3600.
		nudge_aice=aice_atm*1.52594875864068e-05+0.499992370256207
		
		!------------------for 2001-2013------------------------------
		!wind_speed_x_t=U_10_MOD_atm*0.0015557333523461-3.61003079636367
		!wind_speed_y_t=V_10_MOD_atm*0.00140508130430828+4.18342526696503
		!tair02=T_10_MOD_atm*0.0020387826582838+258.199297991483-273.15
		!tdew02=Q_10_MOD_atm*0.0019801136353476+242.352128839667-273.15
		!nudge_sst=SNOW_atm*0.000634591273432851+288.615771571551-273.15
		!!prec=(RAIN_atm*6.5447033258556e-06+0.21444374909044+runoff*3.57085421180932e-06+0.117002609104144)/12./3600.  
		!prec=(RAIN_atm*6.5447033258556e-06+0.21444374909044						)/12./3600.  
		!str=(LWDN_MOD_atm*422.760120855142+12007954.1199396)/12./3600.-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		!ssr=(SWDN_MOD_atm*569.291685105214+18653411.3541574)/12./3600.
			
		!------------------for 1991-2000------------------------------
		!wind_speed_x_t=U_10_MOD_atm*0.00130511042549054-3.433330777869
		!wind_speed_y_t=V_10_MOD_atm*0.00136506543093925+0.203335106444687
		!tair02=T_10_MOD_atm*0.00205943568179629+257.899238836847-273.15
		!tdew02=Q_10_MOD_atm*0.00196060199989891+243.918209152125-273.15
		!nudge_sst=SNOW_atm*0.000608472067507973+288.232361779591-273.15
		!!prec=(RAIN_atm*7.97644449370106e-06+0.26135618018896+runoff*3.49797488419479e-06+0.114614645055527)/12./3600.  
		!prec=(RAIN_atm*7.97644449370106e-06+0.26135618018896						)/12./3600.  
		!str=(LWDN_MOD_atm*425.210931896907+12021815.3945341)/12./3600.-0.0000000567*(t_c(1,:,:,1)+273.15)**4
		!ssr=(SWDN_MOD_atm*567.510353562327+18595044.2448232)/12./3600.
				
		!------------------for 1979-1990------------------------------
		!wind_speed_x_t=U_10_MOD_atm*0.00136057089528129+0.515249890333609
		!wind_speed_y_t=V_10_MOD_atm*0.00123852503111803+1.8194872438321
		!tair02=T_10_MOD_atm*0.00205283393235266+257.467410777858-273.15
		!tdew02=Q_10_MOD_atm*0.00195595285761622+243.165701405895-273.15
		!nudge_sst=SNOW_atm*0.000594471785684312+288.165230107857-273.15
		!!prec=(RAIN_atm*6.68417526622767e-06+0.219013686660271+runoff*3.68768233522972e-06+0.120830599396137)/12./3600.  
		!prec=(RAIN_atm*6.68417526622767e-06+0.219013686660271						)/12./3600.  
		!str=(LWDN_MOD_atm*433.385927700548+12567900.3070362)/12./3600.-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		!ssr=(SWDN_MOD_atm*564.853981963286+18508005.573009)/12./3600.

		!In Interim, the first accumulated data come at 12:00 of the first day. Before 12:00 there is "rubbish".
		if(time_l<=minday/2) then
			prec=0.
			ssr=0.
			str=0.
			runoff=0.
		end if
		prec_tflux=0.	!Bugfixed!	!No snow melting heat in Interim, since precipitation above water is assumed to be rain. 

	case("ATM_GFS")
		wind_speed_x_t=U_10_MOD_atm
		wind_speed_y_t=V_10_MOD_atm
		tair02=T_10_MOD_atm
		shumid=Q_10_MOD_atm
		prec=(RAIN_atm+SNOW_atm)/1000. !convert from mm/s to m/s
		str=LWDN_MOD_atm-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		ssr=SWDN_MOD_atm
		prec_tflux=-SNOW_atm*merge(1.-aice,1.,trim(ice_model_name)=='ICE_CICE')/densit*78.	![K*m/s] - temperature flux by melting snow. 78=melting heat(J/kg)/heat capacity(J/kg/K)
		
	case("ATM_INM")
		wind_speed_x_t=U_10_MOD_atm
		wind_speed_y_t=V_10_MOD_atm
		tair02=T_10_MOD_atm-273.15
		shumid=Q_10_MOD_atm

		prec=prec_from_atm/(atm_ocn_mapping*60.)/1000.			!prec is received as kg/m2 (integrated through the period between mappings) and must be converted to m/s
		str=str_from_atm
		ssr=ssr_from_atm
		!sns,ltn calculated by bulk formulae
		prec_tflux=0.
	case("ATM_INMCM4")
		wind_speed_x_t=U_10_MOD_atm
		wind_speed_y_t=V_10_MOD_atm
		tair02=T_10_MOD_atm-273.15
		shumid=Q_10_MOD_atm
		prec=RAIN_atm/1000.
		str=LWDN_MOD_atm-0.0000000567*(t_c(1,:,:,1)+273.16)**4
		ssr=SWDN_MOD_atm
		prec_tflux=0.	!it is necessary here!
		
		runoff=0.
		do j=jsouth(myrank),jnorth(myrank)
		do i=iwest (myrank),ieast (myrank)
			!300e12=Total year runoff [kg], split into 48 (hand tuned number) gridcells
			if(i<=23 .and. j>=144 .and. kt(i,j)>0) runoff(i,j)=300e12/(365.*86400.*48.*area_tbox(i,j)*ct3(1,i,j))	!kg/m2/s
		end do
		end do
		
	end select
	
	!Combine precipitation, runoff and aice
	prec=prec*merge(1.-aice,1.,ice_model_name=='ICE_CICE') !In Schrum the precipitation fully leaks through ice, in CICE - not (?)
	prec=prec+runoff/1000.	!1000=fresh water density
	prec_tflux=prec_tflux+prec*t_c(1,:,:,1)		![K*m/s] - add temperature flux, brought by precipitation. We propose that T_prec=T_ocean.
	
	patm=100000.
	do j=jsouth(myrank),jnorth(myrank)
	do i=iwest (myrank),ieast (myrank)
		wind_scaling=sqrt(wind_speed_x_t(i,j)*wind_speed_x_t(i,j)+wind_speed_y_t(i,j)*wind_speed_y_t(i,j))/wind_threshold
		
		if( wind_scaling > 1. .and. kt(i,j)>0) then
			wind_speed_x_t(i,j)=wind_speed_x_t(i,j)/wind_scaling
			wind_speed_y_t(i,j)=wind_speed_y_t(i,j)/wind_scaling
			!print *,'Forcing warning in ocean: wind speed,i,j = ',wind_scaling*wind_threshold,i,j
		end if
	end do
	end do
			
	do j=jsouth(myrank)-1,jnorth(myrank)+1
	do i=iwest (myrank)-1,ieast (myrank)+1
	    if(tair02(i,j)<-270.) tair02(i,j)=0.
		!if(kt(i,j)==0 .and. runoff(i,j)>0) print *,'Achtung! Possibly wrong runoff interpolation',i,j,kt(i,j),runoff(i,j)
		if(kt(i,j)==0) then
			wind_speed_x_t(i,j)=0.
			wind_speed_y_t(i,j)=0.
			tair02(i,j)=0.
			tdew02(i,j)=0.
			prec(i,j)=0.
			str(i,j)=0.
			ssr(i,j)=0.
			shumid(i,j)=0.
			surface_salt(i,j)=0.
		end if
	end do	
	end do

	if(bsc_flag==1) call ssc_tsc_rotate_t(	wind_speed_x_t,wind_speed_y_t,	wind_speed_x_t,wind_speed_y_t,	iwest(myrank)-1,ieast(myrank)+1,jsouth(myrank)-1,jnorth(myrank)+1)

#if CMF_VERSION == 2    
    call comp_halo_update_T4(DIM2_t = wind_speed_x_t, update_width = 1, change_sign_on_bipolar = -1)
    call comp_halo_update_T4(DIM2_t = wind_speed_y_t, update_width = 1, change_sign_on_bipolar = -1)
    call comp_halo_update_T4(DIM2_t = tair02, update_width = 1, change_sign_on_bipolar = 1)
    call comp_halo_update_T4(DIM2_t = prec, update_width = 1, change_sign_on_bipolar = 1)
    call comp_halo_update_T4(DIM2_t = ssr, update_width = 1, change_sign_on_bipolar = 1)
    call comp_halo_update_T4(DIM2_t = str, update_width = 1, change_sign_on_bipolar = 1)
    call comp_halo_update_T4(DIM2_t = shumid, update_width = 1, change_sign_on_bipolar = 1)	!Is humidity exchange really necessary?
    call comp_halo_update_T4(DIM2_t = tdew02, update_width = 1, change_sign_on_bipolar = 1)
    call comp_halo_update_T4(DIM2_t = nudge_aice, update_width = 1, change_sign_on_bipolar = 1)
#elif CMF_VERSION == 3
    call HaloUpdateMaster % update(wind_speed_x_t, update_width = 1, grid_type = 'T', change_sign_on_bipolar = .TRUE.)
    call HaloUpdateMaster % update(wind_speed_y_t, update_width = 1, grid_type = 'T', change_sign_on_bipolar = .TRUE.)
    call HaloUpdateMaster % update(tair02, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(prec, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(ssr, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(str, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(shumid, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(tdew02, update_width = 1, grid_type = 'T')
    call HaloUpdateMaster % update(nudge_aice, update_width = 1, grid_type = 'T')
#endif  

	do j=jsouth(myrank)-1,jnorth(myrank)+1
	do i=iwest (myrank)-1,ieast (myrank)+1
			if(kt(i,j).gt.0) then
				if(abs(wind_speed_x_t(i,j)).lt.0.01) wind_speed_x_t(i,j)=sign(0.01,wind_speed_x_t(i,j))
				wind_speed_m_t(i,j)=sqrt(wind_speed_x_t(i,j)*wind_speed_x_t(i,j)+wind_speed_y_t(i,j)*wind_speed_y_t(i,j))
			end if
	end do
	end do
end

	


