!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	parameterization of verical mixing
!
! authors: 
!	2017.09 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Griffies S.M. The Gent-McWilliams skew flux. JPO, 1998, 28, 831-841
!
!--------------------------------------

	module o_vmc_GM_skew_module
	use o_td_module
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use test_ocean_module
	use o_diagn_mld_module

	real (kind=4), allocatable :: offdiagonal_GM_fluxes_sum(:,:,:,:)  !verical difference of (-kappa*2*S*grad_h_T) - see formula (29) of (Griffies, 1998) 
	real (kind=4), allocatable :: kh_GM(:,:,:,:)	!GM addition to ordinary kh		
	real (kind=4), allocatable :: tneutr(:,:,:)
	real(8) dxv,dyv,gdv,dtdx,dtdy
	
	contains
	
	subroutine o_vmc_GM_skew
		
		implicit none
		real(8) press(kl),rho_from_theta
		real slope
		
		if(.not.allocated(offdiagonal_GM_fluxes_sum)) then
			allocate(offdiagonal_GM_fluxes_sum	(kl,iwest (myrank):ieast (myrank),jsouth(myrank):jnorth(myrank),ml) )
			allocate(kh_GM				(kl,iwest (myrank):ieast (myrank),jsouth(myrank):jnorth(myrank),ml) )
			allocate(tneutr				(kl,iwest (myrank):ieast (myrank),jsouth(myrank):jnorth(myrank)) )
			offdiagonal_GM_fluxes_sum=0.
			kh_GM=0.
			tneutr=0.
		end if
	
		do k=1, kl
			!pressure [db] on the upper interface of the (k,i,j) cell
			press(k)=densit*gravit*zw(k)*0.0001
		end do
		
		gdv=densit/gravit
		!kh_GM=0.
		flux_u_e=0.
		flux_u_n=0.
		
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kt(i,j)
			
			!dxv=1./(dx1_nw_v(i-1,j-1)+dx1_ne_v(i-1,j-1)+dx1_nw_v(i,j-1)+dx1_ne_v(i,j-1))
			!dyv=1./(dx2_es_v(i-1,j-1)+dx2_en_v(i-1,j-1)+dx2_es_v(i-1,j)+dx2_en_v(i-1,j))
			
			!d rho / dx in T-nodes
			!func_u_e(k,i,j)=gdv*(d_c(k,i+1,j)-d_c(k,i-1,j))*dxv
			func_u_e(k,i,j)=(	(d_c(k,i+1,j)-d_c(k,i,j))*(cv(k,i  ,j)+cv(k,i  ,j-1))/(dx1_nw_v(i  ,j-1)+dx1_ne_v(i  ,j-1)) +			&
						(d_c(k,i,j)-d_c(k,i-1,j))*(cv(k,i-1,j)+cv(k,i-1,j-1))/(dx1_nw_v(i-1,j-1)+dx1_ne_v(i-1,j-1))	)*0.25*gdv
		
			!d rho / dy in T-nodes
			!func_u_n(k,i,j)=gdv*(d_c(k,i,j+1)-d_c(k,i,j-1))*dyv
			func_u_n(k,i,j)=(	(d_c(k,i,j+1)-d_c(k,i,j))*(cv(k,i,j  )+cv(k,i-1,j  ))/(dx2_es_v(i-1,j  )+dx2_en_v(i-1,j  )) +			&
						(d_c(k,i,j)-d_c(k,i,j-1))*(cv(k,i,j-1)+cv(k,i-1,j-1))/(dx2_es_v(i-1,j-1)+dx2_en_v(i-1,j-1))	)*0.25*gdv

			if(k.gt.1) then
				!d rho / dz on the upper interface of the (k,i,j) T-cell
				!Note that this is locally referenced rho, i.e. rho referenced to upper interface of the k-th cell
				func_u_b(k,i,j)=(	rho_from_theta(	dble(t_c(k  ,i,j,2)),dble(t_c(k  ,i,j,1)),press(k)) -		&
							rho_from_theta(	dble(t_c(k-1,i,j,2)),dble(t_c(k-1,i,j,1)),press(k))) * hzv(k)
			
				if(abs(func_u_b(k,i,j)).lt.1.e-10) func_u_b(k,i,j)=sign(1.e-10,func_u_b(k,i,j))
				
				!slope components on the upper interface of the (k,i,j) T-cell 
				flux_u_e(k,i,j)=-0.5*(func_u_e(k-1,i,j)+func_u_e(k,i,j))/func_u_b(k,i,j)
				flux_u_n(k,i,j)=-0.5*(func_u_n(k-1,i,j)+func_u_n(k,i,j))/func_u_b(k,i,j)
			
				slope=sqrt(flux_u_e(k,i,j)*flux_u_e(k,i,j)+flux_u_n(k,i,j)*flux_u_n(k,i,j))
				call taper(slope,sngl(zw(k)+ssl_c(i,j)),mld(i,j),tneutr(k,i,j))
				
				test3d(k,i,j)=tneutr(k,i,j)*flux_u_e(k,i,j)
				!tn=1.
				
				!restricting slope components
				!flux_u_e(k,i,j)=min(flux_u_e(k,i,j),0.01)
				!flux_u_n(k,i,j)=min(flux_u_n(k,i,j),0.01)
				
				!flux_u_e(k,i,j)=max(flux_u_e(k,i,j),-0.01)
				!flux_u_n(k,i,j)=max(flux_u_n(k,i,j),-0.01)
				
			end if
		end do 
		end do
		end do
		
		do m=1,ml
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
			do k=1,kt(i,j)
			
				!Tracers' gradient components in T-nodes
				!dtdx=(t_c(k,i+1,j,m)-d_c(t,i-1,j,m))*dxv
				!dtdy=(t_c(k,i,j+1,m)-d_c(t,i,j-1,m))*dyv
				flow_t_e(k,i,j)=(	(t_c(k,i+1,j,m)-t_c(k,i,j,m))*(cv(k,i  ,j)+cv(k,i  ,j-1))/(dx1_nw_v(i  ,j-1)+dx1_ne_v(i  ,j-1)) +			&
							(t_c(k,i,j,m)-t_c(k,i-1,j,m))*(cv(k,i-1,j)+cv(k,i-1,j-1))/(dx1_nw_v(i-1,j-1)+dx1_ne_v(i-1,j-1))	)*0.25
				
				flow_t_n(k,i,j)=(	(t_c(k,i,j+1,m)-t_c(k,i,j,m))*(cv(k,i,j  )+cv(k,i-1,j  ))/(dx2_es_v(i-1,j  )+dx2_en_v(i-1,j  )) +			&
							(t_c(k,i,j,m)-t_c(k,i,j-1,m))*(cv(k,i,j-1)+cv(k,i-1,j-1))/(dx2_es_v(i-1,j-1)+dx2_en_v(i-1,j-1))	)*0.25
			end do
			
			do k=2,kt(i,j)
				!temporarily kh_GM stores -2S*grad_h_T*tn on upper interface of k-th box (2 cancels with 0.5)
				kh_GM(k,i,j,m)=-(	flux_u_e(k,i,j)*(flow_t_e(k,i,j)+flow_t_e(k-1,i,j)) + 		&
							flux_u_n(k,i,j)*(flow_t_n(k,i,j)+flow_t_n(k-1,i,j))	)*tneutr(k,i,j)
				
				!test3d(k,i,j)=flow_t_e(k,i,j)!kh_GM(k,i,j,1)
			end do
		
			!do k=1,kt(i,j)
				!if(zw(k).le.mld(i,j)) kh_GM(k,i,j,m)=0.
			!end do
						
			do k=1,kt(i,j)-1
				!(inward flux minus outward flux)
				offdiagonal_GM_fluxes_sum(k,i,j,m)=kh_GM(k,i,j,m)-kh_GM(k+1,i,j,m)
			end do
			k=kt(i,j)
			if(k>0) offdiagonal_GM_fluxes_sum(k,i,j,m)=kh_GM(k,i,j,m)
			
		end do
		end do
		end do
			
		
		do j=jsouth(myrank)	,jnorth(myrank)
		do i=iwest (myrank)	,ieast (myrank)
		do k=1,kt(i,j)
			!kappa to be used in (Griffies, 1998, eq. 29)
			kh_GM(k,i,j,:)=0.25*(ah_e(k,i,j,:)+ah_e(k,i-1,j,:)+ah_n(k,i,j,:)+ah_n(k,i,j-1,:))
			
			!final value of offdiagonal fluxes sum [(K*m3/s)/dxdy]
			offdiagonal_GM_fluxes_sum(k,i,j,:)=kh_GM(k,i,j,:)*offdiagonal_GM_fluxes_sum(k,i,j,:)
	
			!finishing kh_GM
			kh_GM(k,i,j,:)=kh_GM(k,i,j,:)*(flux_u_e(k,i,j)*flux_u_e(k,i,j) + flux_u_n(k,i,j)*flux_u_n(k,i,j))*tneutr(k,i,j)
			
			!test3d(k,i,j)=offdiagonal_GM_fluxes_sum(k,i,j,1) !(flux_u_e(k,i,j)*flux_u_e(k,i,j) + flux_u_n(k,i,j)*flux_u_n(k,i,j))
			!if(zw(k).le.mld(i,j)) kh_GM(k,i,j,:)=0.
		
		end do
		end do
		end do
	
		kh=kh+kh_GM
	
	end subroutine o_vmc_GM_skew
	
	subroutine taper(slope,depth,mld,tn)
		
		implicit none
		real slope,depth,mld
		real ksi,r,tsine,dsv,ttanh,tn
		
		real :: lambda=50000
		real :: turb_layer_min=100.
		real :: smax=0.01
		real :: ds=0.001
		
		ksi=max(lambda*slope,mld,turb_layer_min)
		r=depth/ksi
		dsv=1./ds
		
		if(r.le.1) then
			tsine=0.5*(1.+sin(3.1415926*(r-0.5)))
		else
			tsine=1.
		end if
		
		ttanh=0.5*(1.+tanh((smax-slope)*dsv))
		
		tn=tsine*ttanh
	end subroutine taper
	
	
	end module o_vmc_GM_skew_module
