!#include "cppdefs.h"

!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	parameterization of verical mixing
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------


	subroutine o_vmc_drive
	
!--------------------------------------
!	subroutine which governs calculatiion of vertical mixing coefficients
!--------------------------------------

	use o_par_module
 
	use o_basic_module
	use o_grid_module
	use o_d_brcl_module

	use o_td_module
	use o_d_vmc_1_module
	use o_vmc_GM_skew_module
!	use o_d_vmc_2_module
!	use kpp_module
	implicit none

!-------
!	set constant values to vertical mixing coefficients for momentum,heat & salinity
!-------
	if(v_mix_mod == 0) then
		km = c1amv
		kh = c1ahv
	end if


!-------
!	munk-anderson parameterization of vertical mixing coefficients for momentum,heat & salinity
!-------
	if(v_mix_mod == 1) then
		call o_vmc_munk
		return
	end if


!-------
!	mellor-yamada 2.5 model
!	!!!	km, kh=kh(khq) are defined at 'l' timestep
!-------
	if(v_mix_mod == 2) then
		Stop '*********Mellor-Yamada not implemented yet***********'
		!call advection_tke
		!call mixh_q
		!adv_q=0
		!q_hmix=0
		!call mell_yam
	end if


!-------
! k-profile parametrization
!-------
	if(v_mix_mod == 3) then
		Stop '*********KPP not tested yet***********'
!		!-----------uncomment this line!------------  call o_vmc_kpp_main 
!
!		!Please, check sign change on bipolar greenwich!!
!#if CMF_VERSION == 2    
!        call comp_halo_update_V4(DIM3_u = kh_v(:,:,:,1), DIM3_v = kh_v(:,:,:,2), update_width = 1, change_sign_on_bipolar = 1)
!#elif CMF_VERSION == 3
!        call HaloUpdateMaster % update(kh_v(:,:,:,1), update_width = 1, grid_type = 'V')
!        call HaloUpdateMaster % update(kh_v(:,:,:,2), update_width = 1, grid_type = 'V')
!#endif
!!z		u_f = 0.
!!z		v_f = 0.
!
!		do m=1,ml
!		do k=1,kl
!		do j=jsouth(myrank)	,jnorth(myrank) 
!		do i=iwest (myrank)	,ieast (myrank)
!			if(kt(i,j).ge.k) then 
!				!The following sum must be rewritten in X-way for equator-symmetricity (as for w in d_am_cd)
!				kh(k,i,j,m) = (	(kh_v(k,i-1,j-1,m)*cv(k,i-1,j-1)+				&
!     						 kh_v(k,i  ,j  ,m)*cv(k,i  ,j  ))+				&
!     						(kh_v(k,i-1,j  ,m)*cv(k,i-1,j  )+				&
!     						 kh_v(k,i  ,j-1,m)*cv(k,i  ,j-1)) )/				&
!						(  (cv(k,i-1,j-1)+cv(k,i,j))+(cv(k,i-1,j)+cv(k,i,j-1))  )
!		end if
!		end do 
!		end do 
!		end do 
!		end do 
!		return
	end if

!----------------	
! Gent-McWilliams in (Griffies, 1998) formulation	
!----------------	
	if(v_mix_mod == 4) then
		call o_vmc_munk
		!km=10.e-4
		call o_vmc_GM_skew
	end if
	
	end

