
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	parameterization of verical mixing
!	munk-anderson 
!
! authors: 
!	2010.05 / Ibrayev, Rashit, INM RAS, Russia, ibrayev@mail.ru 
!
! reference:
!	Munk, W. H. and Anderson, E. R.1948.
!		Note on the theory of the thermocline, 
!		J. Mar. Res., 7, 276–295, 1948.
!--------------------------------------



	subroutine o_vmc_munk

!--------------------------------------
!	If v_mix_mod=1 calculate coefficients of vertical eddy viscosity
!		and diffusivity by formula: amz=av1/(1+av4*ri)**2+av2
!		kh=amz/(1+av4*ri)+av3
!--------------------------------------

	use o_basic_module
	use o_grid_module
	use o_d_brcl_module
	use o_td_module
	use o_d_vmc_1_module
	implicit none
	real dvdzc,c
	
!-------
!		density at v-points
!-------
	flux_u_n = 0.	!density at v-points

!	do j=jsouth(myrank)-1	,jnorth(myrank)
!	do i=iwest (myrank)-1	,ieast (myrank)
!	do k=1,kv(i,j)
!	flux_u_n(k,i,j)=0.25*( d_c(k,i+1,j+1)
!     &			+d_c(k,i+1,j  )
!     &			+d_c(k,i  ,j+1)
!     &			+d_c(k,i  ,j  )	)
!	end do
!	end do
!	end do

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=1,kv(i,j)
		flux_u_n(k,i,j)=0.25*( (d_c(k,i+1,j+1)+d_c(k,i,j))
     &			+		(d_c(k,i  ,j+1)+d_c(k,i+1,j))  )
	end do
	end do
	end do
	
	
	
!-------
!		vertical gradient of density between at v-points
!-------
	flux_v_n = 0.	!vertical gradient of density

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
		flux_v_n(k,i,j)=hzv(k)*(flux_u_n(k,i,j)-flux_u_n(k-1,i,j))
	end do
	end do
	end do

!-------
!		gradient of currents
!-------
	flow_u_n = 0.	!vertical gradient of u-velocity
	flow_v_n = 0.	!vertical gradient of u-velocity

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
		flow_u_n  (k,i,j) = hzv(k)*(u_c(k,i,j)-u_c(k-1,i,j))
	end do
	end do
	end do

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
		flow_v_n(k,i,j) = hzv(k)*(v_c(k,i,j)-v_c(k-1,i,j))
	end do
	end do
	end do

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
	flow_v_n(k,i,j) =	flow_u_n(k,i,j)*flow_u_n(k,i,j) + 
     &				flow_v_n(k,i,j)*flow_v_n(k,i,j)
	end do
	end do
	end do

!-------
!		richardson number
!-------
	flux_u_n = 0.	!richardson number

	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
		dvdzc=(dvc*hzv(k))**2
		flux_u_n(k,i,j)=1.
		if(flow_v_n(k,i,j) .gt. dvdzc) then
			flux_u_n(k,i,j) = flux_v_n(k,i,j) / flow_v_n(k,i,j)
		else
			flux_u_n(k,i,j) = flux_v_n(k,i,j) / dvdzc
	end if
	end do
	end do
	end do

!-------
!		vertical eddy mixing coefficients for momentum
!-------
	do j=jsouth(myrank)-1	,jnorth(myrank)
	do i=iwest (myrank)-1	,ieast (myrank)
	do k=2,kv(i,j)
c		km(k,i,j) = c1amv/(1.+c3amv*flux_u_n(k,i,j))**namv+c2amv(k)
		km(k,i,j) = c1amv/(1.+c3amv*flux_u_n(k,i,j))+c2amv(k)	!for the case namv=1
	end do
	end do
	end do

!-------
!		vertical eddy mixing coefficients for heat & salinity
!-------
!-------
!		formula accordinq Munk & anderson
!-------
	m = 1
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
	do k=2,kt(i,j)
		c=1./(	cv(k,i  ,j  )+
     &			cv(k,i-1,j  )+
     &			cv(k,i  ,j-1)+
     &			cv(k,i-1,j-1) )

		kh(k,i,j,m) = c1ahv/(1.+c*c3ahv*
     &			(	( flux_u_n(k,i  ,j  )
     &				 +flux_u_n(k,i-1,j-1))
     &			+	( flux_u_n(k,i  ,j-1)
     &				 +flux_u_n(k,i-1,j  ))	))+c2ahv(k)	!for the case nahv=1
	end do
	end do
	end do

	k=1
	do j=jsouth(myrank)	,jnorth(myrank)
	do i=iwest (myrank)	,ieast (myrank)
		if(k.le.kt(i,j)) kh(k,i,j,m) = c1ahv+c2ahv(k)
	end do
	end do

!-------
!	the same coefficients for other tracers
!-------
	do j=jsouth(myrank)	,jnorth(myrank)
	do m=2,ml
	do i=iwest (myrank)	,ieast (myrank)
	do k=1,kl
		kh(k,i,j,m) = kh(k,i,j,1)
	end do
	end do
	end do
	end do
	return
	end
