#define TEST_KIND 4


!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
! authors: 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!
!--------------------------------------


!     c_main  allocate.
!
!	call test_halo_update(2,cmp % i_size+1,2,cmp % j_size+1)
!	call test_halo_update_spec(2,cmp % i_size+1,2,cmp % j_size+1)


	subroutine test_halo_update(g_west, g_east,g_south, g_north)
	
!=============================================================
!     halo-.
!      , 
!       .
!=============================================================

	use o_grid_module
	use shared_module
	use comp_module
	use comp_halo_update_module

	implicit none

	integer, parameter :: test_cur_halo_width = 1
	integer, parameter :: test_output = 0

	integer, parameter :: k_level_for_check = 2, m_level_for_check = 2

	integer test_rank, counter
	integer g_south, g_north, g_west, g_east


! 4D, 3D, 2D data
	real(kind = 4) global_seq(g_west:g_east, g_south:g_north)

	real(kind = TEST_KIND), dimension (cmp % k_size, g_west-test_cur_halo_width:g_east+test_cur_halo_width, g_south-test_cur_halo_width:g_north+test_cur_halo_width, cmp % m_size)  :: g_test_u4, g_test_v4, g_test_t4
	real(kind = TEST_KIND), dimension (cmp % k_size, g_west-test_cur_halo_width:g_east+test_cur_halo_width, g_south-test_cur_halo_width:g_north+test_cur_halo_width) :: g_test_u3, g_test_v3, g_test_t3
	real(kind = TEST_KIND), dimension (             g_west-test_cur_halo_width:g_east+test_cur_halo_width, g_south-test_cur_halo_width:g_north+test_cur_halo_width) :: g_test_u2, g_test_v2, g_test_t2

	real(kind = TEST_KIND), dimension (cmp % k_size, iwest(myrank)-test_cur_halo_width:ieast(myrank)+test_cur_halo_width, jsouth(myrank)-test_cur_halo_width:jnorth(myrank)+test_cur_halo_width, cmp % m_size) :: test_u4, test_v4, test_t4
	real(kind = TEST_KIND), dimension (cmp % k_size, iwest(myrank)-test_cur_halo_width:ieast(myrank)+test_cur_halo_width, jsouth(myrank)-test_cur_halo_width:jnorth(myrank)+test_cur_halo_width) :: test_u3, test_v3, test_t3
	real(kind = TEST_KIND), dimension (             iwest(myrank)-test_cur_halo_width:ieast(myrank)+test_cur_halo_width, jsouth(myrank)-test_cur_halo_width:jnorth(myrank)+test_cur_halo_width) :: test_u2, test_v2, test_t2

! Fill global array
	counter = 1
	do j = g_south, g_north
	do i = g_west, g_east
		global_seq(i,j) = counter
		counter = counter + 1
	end do
	end do

! 4D arrays
	test_u4 = 0
	test_v4 = 0
	test_t4 = 0
	g_test_u4 = 0
	g_test_v4 = 0
	g_test_t4 = 0

	do m = 1,ml
	do k = 1,kl
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest(myrank), ieast(myrank)
		test_u4(k,i,j,m) = 100 + m*10 + k + global_seq(i,j)*0.001 
		test_v4(k,i,j,m) = 200 + m*10 + k + global_seq(i,j)*0.001 
		test_t4(k,i,j,m) = 300 + m*10 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do
	end do

	do m = 1,ml
	do k = 1,kl
	do j = g_south, g_north
	do i = g_west, g_east
		g_test_u4(k,i,j,m) = 100 + m*10 + k + global_seq(i,j)*0.001 
		g_test_v4(k,i,j,m) = 200 + m*10 + k + global_seq(i,j)*0.001 
		g_test_t4(k,i,j,m) = 300 + m*10 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do
	end do


! 3D arrays
	test_u3 = 0
	test_v3 = 0
	test_t3 = 0
	g_test_u3 = 0
	g_test_v3 = 0
	g_test_t3 = 0

	do k = 1,kl
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest(myrank), ieast(myrank)
		test_u3(k,i,j) = 10 + k + global_seq(i,j)*0.001 
		test_v3(k,i,j) = 20 + k + global_seq(i,j)*0.001 
		test_t3(k,i,j) = 30 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do

	do k = 1,kl
	do j = g_south, g_north
	do i = g_west, g_east
		g_test_u3(k,i,j) = 10 + k + global_seq(i,j)*0.001 
		g_test_v3(k,i,j) = 20 + k + global_seq(i,j)*0.001 
		g_test_t3(k,i,j) = 30 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do


! 2D arrays
	test_u2 = 0
	test_v2 = 0
	test_t2 = 0
	g_test_u2 = 0
	g_test_v2 = 0
	g_test_t2 = 0

	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest(myrank), ieast(myrank)
		test_u2(i,j) = 1 + global_seq(i,j)*0.001 
		test_v2(i,j) = 2 + global_seq(i,j)*0.001 
		test_t2(i,j) = 3 + global_seq(i,j)*0.001 
	end do
	end do

	do j = g_south, g_north
	do i = g_west, g_east
		g_test_u2(i,j) = 1 + global_seq(i,j)*0.001 
		g_test_v2(i,j) = 2 + global_seq(i,j)*0.001 
		g_test_t2(i,j) = 3 + global_seq(i,j)*0.001 
	end do
	end do

! Test-zone

#if TEST_KIND == 4
		call comp_halo_update_V4(DIM4_u = test_u4, DIM4_v = test_v4, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
		call comp_halo_update_T4(DIM4_t = test_t4,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	
		call comp_halo_update_V4(DIM3_u = test_u3, DIM3_v = test_v3, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
		call comp_halo_update_T4(DIM3_t = test_t3,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)

		call comp_halo_update_V4(DIM2_u = test_u2, DIM2_v = test_v2, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
		call comp_halo_update_T4(DIM2_t = test_t2,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
#else 	
 		call comp_halo_update_V8(DIM4_u = test_u4, DIM4_v = test_v4, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
 		call comp_halo_update_T8(DIM4_t = test_t4,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
  
  		call comp_halo_update_V8(DIM3_u = test_u3, DIM3_v = test_v3, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
  		call comp_halo_update_T8(DIM3_t = test_t3,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
  
  		call comp_halo_update_V8(DIM2_u = test_u2, DIM2_v = test_v2, 	update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
  		call comp_halo_update_T8(DIM2_t = test_t2,			update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
#endif

! Exchange global arrays

	if(comp_i_cycle) then

		do m = 1,ml
		do k = 1,kl
		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u4(k,g_west-i,j,m) =  g_test_u4(k,g_east-i+1,j,m)
			g_test_v4(k,g_west-i,j,m) =  g_test_v4(k,g_east-i+1,j,m)
			g_test_t4(k,g_west-i,j,m) =  g_test_t4(k,g_east-i+1,j,m)
		end do
		end do
		end do
		end do

		do m = 1,ml
		do k = 1,kl
		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u4(k,g_east+i,j,m) =  g_test_u4(k,g_west+i-1,j,m)
			g_test_v4(k,g_east+i,j,m) =  g_test_v4(k,g_west+i-1,j,m)
			g_test_t4(k,g_east+i,j,m) =  g_test_t4(k,g_west+i-1,j,m)
		end do
		end do
		end do
		end do


		do k = 1,kl
		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u3(k,g_west-i,j) =  g_test_u3(k,g_east-i+1,j)
			g_test_v3(k,g_west-i,j) =  g_test_v3(k,g_east-i+1,j)
			g_test_t3(k,g_west-i,j) =  g_test_t3(k,g_east-i+1,j)
		end do
		end do
		end do

		do k = 1,kl
		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u3(k,g_east+i,j) =  g_test_u3(k,g_west+i-1,j)
			g_test_v3(k,g_east+i,j) =  g_test_v3(k,g_west+i-1,j)
			g_test_t3(k,g_east+i,j) =  g_test_t3(k,g_west+i-1,j)
		end do
		end do
		end do


		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u2(g_west-i,j) =  g_test_u2(g_east-i+1,j)
			g_test_v2(g_west-i,j) =  g_test_v2(g_east-i+1,j)
			g_test_t2(g_west-i,j) =  g_test_t2(g_east-i+1,j)
		end do
		end do

		do i = 1, test_cur_halo_width
		do j = g_south-test_cur_halo_width, g_north+test_cur_halo_width
			g_test_u2(g_east+i,j) =  g_test_u2(g_west+i-1,j)
			g_test_v2(g_east+i,j) =  g_test_v2(g_west+i-1,j)
			g_test_t2(g_east+i,j) =  g_test_t2(g_west+i-1,j)
		end do
		end do

	end if

	if(comp_bsc_grid) then

		do m = 1,ml
		do k = 1,kl
		do j = 1, test_cur_halo_width
		do i = g_west, g_east
			g_test_u4(k,i,g_north+j,m) = -g_test_u4(k,g_east+g_west-i,g_north-j,m)
			g_test_v4(k,i,g_north+j,m) = -g_test_v4(k,g_east+g_west-i,g_north-j,m)
			g_test_t4(k,i,g_north+j,m) = -g_test_t4(k,g_east+g_west-i+1,g_north-j+1,m)
		end do
		end do
		end do
		end do

		do k = 1,kl
		do j = 1, test_cur_halo_width
		do i = g_west, g_east
			g_test_u3(k,i,g_north+j) = -g_test_u3(k,g_east+g_west-i,g_north-j)
			g_test_v3(k,i,g_north+j) = -g_test_v3(k,g_east+g_west-i,g_north-j)
			g_test_t3(k,i,g_north+j) = -g_test_t3(k,g_east+g_west-i+1,g_north-j+1)
		end do
		end do
		end do

		do j = 1, test_cur_halo_width
		do i = g_west, g_east
			g_test_u2(i,g_north+j) = -g_test_u2(g_east+g_west-i,g_north-j)
			g_test_v2(i,g_north+j) = -g_test_v2(g_east+g_west-i,g_north-j)
			g_test_t2(i,g_north+j) = -g_test_t2(g_east+g_west-i+1,g_north-j+1)
		end do
		end do

	end if

! Compare local and global results except top corners

	call sleep(1)

	call MPI_BARRIER(comm_local, ierr)
	do test_rank = 0, size_local-1

		if(rank_local == test_rank) then

		write(*,*) 'After test halo update: Processor number ===============', rank_local

 			do m = 1,ml
 			do k = 1,kl
 			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
 			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
 				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then
 					if(ABS(g_test_u4(k,i,j,m) - test_u4(k,i,j,m)) > 0.00000001) write(*,*) 'u4 error:', i,j,k,m
 					if(ABS(g_test_v4(k,i,j,m) - test_v4(k,i,j,m)) > 0.00000001) write(*,*) 'v4 error:', i,j,k,m
 					if(ABS(g_test_t4(k,i,j,m) - test_t4(k,i,j,m)) > 0.00000001) write(*,*) 't4 error:', i,j,k,m
 				end if
 			end do
 			end do
 			end do
 			end do

 			do k = 1,kl
 			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
 			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
 				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then
 					if(ABS(g_test_u3(k,i,j) - test_u3(k,i,j)) > 0.000000001) write(*,*) 'u3 error:', i,j,k
 					if(ABS(g_test_v3(k,i,j) - test_v3(k,i,j)) > 0.000000001) write(*,*) 'v3 error:', i,j,k
 					if(ABS(g_test_t3(k,i,j) - test_t3(k,i,j)) > 0.000000001) write(*,*) 't3 error:', i,j,k
 				end if
 			end do
 			end do
 			end do

			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then 
					if(ABS(g_test_u2(i,j) - test_u2(i,j)) > 0.000000001) write(*,*) 'u2 error:', i,j
					if(ABS(g_test_v2(i,j) - test_v2(i,j)) > 0.000000001) write(*,*) 'v2 error:', i,j
					if(ABS(g_test_t2(i,j) - test_t2(i,j)) > 0.000000001) write(*,*) 't2 error:', i,j
				end if
			end do
			end do

		call flush(6)
!		call sleep(1)

		end if

		call MPI_BARRIER(comm_local, ierr)
	end do

	call sleep(2)
	
! Check for exotic exhange with ':'(part of whole array)

	test_u3 = 0
	test_v3 = 0
	test_t3 = 0
	test_u4 = 0
	test_v4 = 0
	test_t4 = 0

	do k = 1,kl
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest(myrank), ieast(myrank)
		test_u3(k,i,j) = 10 + k + global_seq(i,j)*0.001 
		test_v3(k,i,j) = 20 + k + global_seq(i,j)*0.001 
		test_t3(k,i,j) = 30 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do

	do m = 1,ml
	do k = 1,kl
	do j = jsouth(myrank), jnorth(myrank)
	do i = iwest(myrank), ieast(myrank)
		test_u4(k,i,j,m) = 100 + m*10 + k + global_seq(i,j)*0.001 
		test_v4(k,i,j,m) = 200 + m*10 + k + global_seq(i,j)*0.001 
		test_t4(k,i,j,m) = 300 + m*10 + k + global_seq(i,j)*0.001 
	end do
	end do
	end do
	end do

#if TEST_KIND == 4	
! 3D in 4D
	call comp_halo_update_V4(DIM3_u = test_u4(:,:,:,m_level_for_check), DIM3_v = test_v4(:,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T4(DIM3_t = test_t4(:,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
! 2D in 4D
	call comp_halo_update_V4(DIM2_u = test_u4(k_level_for_check,:,:,m_level_for_check), DIM2_v = test_v4(k_level_for_check,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T4(DIM2_t = test_t4(k_level_for_check,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
! 2D in 3D
	call comp_halo_update_V4(DIM2_u = test_u3(k_level_for_check,:,:), DIM2_v = test_v3(k_level_for_check,:,:), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T4(DIM2_t = test_t3(k_level_for_check,:,:), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
#else
! 3D in 4D
	call comp_halo_update_V8(DIM3_u = test_u4(:,:,:,m_level_for_check), DIM3_v = test_v4(:,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T8(DIM3_t = test_t4(:,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
! 2D in 4D
	call comp_halo_update_V8(DIM2_u = test_u4(k_level_for_check,:,:,m_level_for_check), DIM2_v = test_v4(k_level_for_check,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T8(DIM2_t = test_t4(k_level_for_check,:,:,m_level_for_check), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
! 2D in 3D
	call comp_halo_update_V8(DIM2_u = test_u3(k_level_for_check,:,:), DIM2_v = test_v3(k_level_for_check,:,:), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
	call comp_halo_update_T8(DIM2_t = test_t3(k_level_for_check,:,:), update_width = test_cur_halo_width, change_sign_on_bipolar = -1)
#endif

	call MPI_BARRIER(comm_local, ierr)
	do test_rank = 0, size_local-1

		if(rank_local == test_rank) then

		write(*,*) 'After update with part of arrays: Processor number ===============', rank_local

! ! 3D in 4D
 			do k = 1,kl
 			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
 			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
 				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then
 					if(ABS(g_test_u4(k,i,j,m_level_for_check) - test_u4(k,i,j,m_level_for_check)) > 0.00001) write(*,*) 'u4 error:', i,j,k,m
 					if(ABS(g_test_v4(k,i,j,m_level_for_check) - test_v4(k,i,j,m_level_for_check)) > 0.00001) write(*,*) 'v4 error:', i,j,k,m
 					if(ABS(g_test_t4(k,i,j,m_level_for_check) - test_t4(k,i,j,m_level_for_check)) > 0.00001) write(*,*) 't4 error:', i,j,k,m
 				end if
 			end do
 			end do
 			end do

! 2D in 4D
 			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
 			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
 				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then
 					if(ABS(g_test_u4(k_level_for_check,i,j,m_level_for_check) - test_u4(k_level_for_check,i,j,m_level_for_check)) > 0.00001) write(*,*) 'u4 error:', i,j,k,m
 					if(ABS(g_test_v4(k_level_for_check,i,j,m_level_for_check) - test_v4(k_level_for_check,i,j,m_level_for_check)) > 0.00001) write(*,*) 'v4 error:', i,j,k,m
 					if(ABS(g_test_t4(k_level_for_check,i,j,m_level_for_check) - test_t4(k_level_for_check,i,j,m_level_for_check)) > 0.00001) write(*,*) 't4 error:', i,j,k,m
 				end if
 			end do
 			end do

! 2D in 3D
 			do j = jsouth(myrank)-test_cur_halo_width, jnorth(myrank)+test_cur_halo_width
 			do i = iwest(myrank)-test_cur_halo_width, ieast(myrank)+test_cur_halo_width
 				if(.NOT.( (i > g_east-1 .AND. j > g_north) .OR. (i < g_west+1 .AND. j > g_north) )) then
 					if(ABS(g_test_u3(k_level_for_check,i,j) - test_u3(k_level_for_check,i,j)) > 0.00001) write(*,*) 'u3 error:', i,j,k
 					if(ABS(g_test_v3(k_level_for_check,i,j) - test_v3(k_level_for_check,i,j)) > 0.00001) write(*,*) 'v3 error:', i,j,k
 					if(ABS(g_test_t3(k_level_for_check,i,j) - test_t3(k_level_for_check,i,j)) > 0.00001) write(*,*) 't3 error:', i,j,k
 				end if
 			end do
 			end do


		call flush(6)
!		call sleep(1)

		end if

		call MPI_BARRIER(comm_local, ierr)
	end do

	call sleep(4)	
! Output

	if(test_output == 1) then

	call MPI_BARRIER(comm_local, ierr)
	do test_rank = 0, size_local-1

		if(rank_local == test_rank) then

		write(*,*) 'After test halo update: Processor number ===============', rank_local

! 4D arrays

		write(*,*) ' U4 ========='
		do m = 1,ml
		write(*,*) 'm =', m
			do k = 1,kl
				write(*,*) 'k = ', k
				do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
					write(*,'(1000f9.3)')(test_u4(k,i,j,m), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
				end do
				write(*,*)
			end do
		write(*,*)
		end do

		write(*,*) ' V4 ========='
		do m = 1,ml
		write(*,*) 'm =', m
			do k = 1,kl
				write(*,*) 'k = ', k
				do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
					write(*,'(1000f9.3)')(test_v4(k,i,j,m), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
				end do
				write(*,*)
			end do
		write(*,*)
		end do

		write(*,*) ' T4 ========='
		do m = 1,ml
		write(*,*) 'm =', m
			do k = 1,kl
				write(*,*) 'k = ', k
				do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
					write(*,'(1000f9.3)')(test_t4(k,i,j,m), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
				end do
				write(*,*)
			end do
		write(*,*)
		end do

! 3D arrays
		write(*,*) ' U3 ========='
		do k = 1,kl
			write(*,*) 'k = ', k
			do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
				write(*,'(1000f9.3)')(test_u3(k,i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
			end do
			write(*,*)
		end do

		write(*,*) ' V3 ========='
		do k = 1,kl
			write(*,*) 'k = ', k
			do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
				write(*,'(1000f9.3)')(test_v3(k,i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
			end do
			write(*,*)
		end do

		write(*,*) ' T3 ========='
		do k = 1,kl
			write(*,*) 'k = ', k
			do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
				write(*,'(1000f9.3)')(test_t3(k,i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
			end do
			write(*,*)
		end do

! 2D arrays
		write(*,*) ' U2 ========='
		do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
			write(*,'(1000f9.3)')(test_u2(i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
		end do

		write(*,*) ' V2 ========='
		do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
			write(*,'(1000f9.3)')(test_v2(i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
		end do

		write(*,*) ' T2 ========='
		do j = jnorth(myrank) + test_cur_halo_width, jsouth(myrank)-test_cur_halo_width, -1
			write(*,'(1000f9.3)')(test_t2(i,j), i = iwest(myrank)-test_cur_halo_width,ieast(myrank)+test_cur_halo_width)
		end do

		call flush(6)
		call sleep(1)

		end if

		call MPI_BARRIER(comm_local, ierr)
	end do

	end if

	return 
	end
