! #define GA

!===============================================
! Main program and entry point of overall system
!===============================================

	program c_main

	use shared_const_module
	use c_module
	use comp_module
	use shared_module
	use c_rw_module
	use c_map_module
	use shared_utils_module
	use comp_halo_update_module	

#ifdef GA
	use ga_module
#endif
	
#include "cpp_modules.h"
	
	implicit none

	integer counter, c, mc
	real(kind=8) wt_write_s, wt_write_f, wt_write_g
	character(NAME_LEN) comp_arg, np_arg
	integer np
	namelist /OCN_MAIN/ 	il, jl, kl, ml, ocn_steps_in_day, ocn_read_cp, dg_period, cp_period

#include "cpp_objects.h"
#include "analysis_flag"

!	if(rank_local == 0) call shared_memory_usage("start", 'coupler at start')
	
!==============================================================================================================
!==============================================================================================================
! Section 1 : Shared initialization
!==============================================================================================================
!==============================================================================================================
	run_phase = 'SHR'
	
	call shared_ini_params			! Initialization of MPI and shared parameters
		
	do counter = 1, run_comps
		call GET_COMMAND_ARGUMENT((counter-1)*2+1, comp_arg)
		call GET_COMMAND_ARGUMENT((counter-1)*2+2, np_arg)
		read(np_arg, *) np

		select case (comp_arg)
			case('CPL')
				run_np(COUPLER) =  np
#include "cpp_case_args.h"
			case default			
				call shared_error("No "//TRIM(comp_arg)//" component is currently compiled.")
		end select
	end do
	
	call shared_ini_split			! Split MPI_COMM_WORLD between components
		
	select case (proc_where())		! Get base class pointer for every component 
#include "cpp_case_pntr.h"
	end select
	
#ifdef GA
	call ga_constructor(COMP_OCN, COMP_SEA)
#endif	
!==============================================================================================================
!==============================================================================================================
! Section 2 : Personal initialization: basics
!==============================================================================================================
!==============================================================================================================
	run_phase = 'INI'
	
	if(.NOT. proc_in(rank_world,COUPLER)) then

		call comp_p % ini_params	! Read and define runtime parameters
		call comp_p % ini_reg_comp	! Register model grid in coupler
		
       ! Allocate module data and call some initializations
        if(comp_halo_max_width /= 0) then
          call comp_halo_update_alloc
          call comp_halo_update_ini
        end if
        
   		call comp_p % ini_allocate	! Allocate all data structures
		call comp_p % ini_reg_data	! Mark send-recv data 
		call comp_p % reg_sendrecv	! Register send-recv data
	else	
		call MPI_BARRIER(comm_local, ierr)	
		wall_time_start_ini = MPI_WTIME()
		
		call c_ini_register		! Receive data from components and registrate them		
		call c_ini_make_events	! Build coupler table for all events in the system		
		call c_map_ini			! Make mapping initialization
		call c_rw_ini			! Make IO initialization	
		call c_gd_ini			! Allocate work data structures		
		call c_spy_log(0)		! Write main run information to log-file	
	end if	
	
! 	if(rank_local == 0) call shared_memory_usage(cmp % name, 'after ini_basics')
	
!==============================================================================================================
!==============================================================================================================
! Section 3 : Read initial conditions
!==============================================================================================================
!==============================================================================================================

	run_phase = 'RUN'

	if(.NOT. proc_in(rank_world,COUPLER)) then
	
	if(ANY(cmp % inf(COUPLER, cmp % me, ACTION_READ_IC,:) % ar_size /= 0)) call comp_p % recv_data(COUPLER, ACTION_READ_IC)
	if(comp_read_cp) call comp_p % recv_data(COUPLER, ACTION_READ_CP)

!==============================================================================================================
!==============================================================================================================
! Section 4 : Personal initialization: main
!==============================================================================================================
!==============================================================================================================
		
	call comp_p % ini_main
	
! 	if(rank_local == 0) call shared_memory_usage(cmp % name, 'after ini_main')
	
	if(time_l_read_fd /=0)													&
		call comp_p % recv_data(COUPLER, ACTION_READ_FD)
	
 	do mc = 1, NUM_OF_COMP
 		if(run_np(mc) /= 0 .AND. cmp % inf(cmp % me,mc,ACTION_MAPPING,ARRAY_TYPE_2D) % ar_size /= 0) 			&
		call comp_p % send_data(mc, ACTION_MAPPING)
 	end do

  	do mc = 1, NUM_OF_COMP
 		if(run_np(mc) /= 0 .AND. cmp % inf(mc,cmp % me,ACTION_MAPPING,ARRAY_TYPE_2D) % ar_size /= 0) 			&
 		call comp_p % recv_data(mc, ACTION_MAPPING)
 	end do
	
!==============================================================================================================
!==============================================================================================================
! Section 5 : Main calculation cycle
!==============================================================================================================
!==============================================================================================================

	! Synchronize and set timer
	call MPI_BARRIER(comm_local, ierr)	
	wall_time_start_cycle = MPI_WTIME()
				
	do time_l_in_run = 1, time_run_steps
	!call shared_timer
		! Receive file data
		if(time_l_read_fd /= 0 .AND. MOD(time_l, time_l_read_fd) == 0) 									&
			call comp_p % recv_data(COUPLER, ACTION_READ_FD)
				
		! All physics calculations and internal halo updates of one time step
		call comp_p % main_step
		
		! Send data for mapping every periodic timestep.
		do mc = 1, NUM_OF_COMP
			if(run_np(mc) /= 0 .AND. cmp % inf(cmp % me,mc,ACTION_MAPPING,ARRAY_TYPE_2D) % ar_size /= 0 .AND.		&
			MOD(time_l, INT(cmp % inf(cmp % me,mc,ACTION_MAPPING,ARRAY_TYPE_2D) % ar(1) % period/time_step_min)) == 0)					& 
			call comp_p % send_data(mc, ACTION_MAPPING)
		end do
	
		! Receive mapped data at start or every periodic timestep
		do mc = 1, NUM_OF_COMP
			if(run_np(mc) /= 0 .AND. cmp % inf(mc,cmp % me,ACTION_MAPPING,ARRAY_TYPE_2D) % ar_size /= 0 .AND.					&
			MOD(time_l, INT(cmp % inf(mc,cmp % me,ACTION_MAPPING,ARRAY_TYPE_2D) % ar(1) % period/time_step_min)) == 0) 				&
				call comp_p % recv_data(mc, ACTION_MAPPING)
		end do

		! Save control point in files every 'time_l_save' step and also at specified aperiodic moments.
		if(	((time_l_save_cp /= 0 .AND. MOD(time_l,time_l_save_cp) == 0) 										.OR.	&
			(abs(time_l*time_step_min-nint(time_l*time_step_min))<1e-7 .AND. is_aperiodic_moment(nint(time_l*time_step_min),ACTION_SAVE_CP)))	.AND.	&
			ANY(cmp%inf(cmp%me,COUPLER,ACTION_SAVE_CP,:)%ar_size /= 0)) 	then
				call comp_p % send_data(COUPLER, ACTION_SAVE_CP)
				if(ANALYSIS_FLAG) call MPI_BARRIER(mpi_comm_world, ierr)	!synchronyzation with CPL to prevent loss of some analysis data
		end if
		
		! Save diagnostics every 'time_save_dg' step
		if(time_l_save_dg /= 0 .AND. MOD(time_l, time_l_save_dg) == 0) 	&
			call comp_p % send_data(COUPLER, ACTION_SAVE_DG)

! 		if(rank_local == 0 .AND. MOD(time_l_in_run, 1) == 0 .AND. cmp % me == COMP_OCN) 						&
! 			write(*,*) 'Ocean global step :', time_l, MPI_WTIME() - wall_time_start
		! Save integrals every 'time_sync_it' day
		
		if(time_save_it /= 0 .AND. MOD(time_l, time_sync_it*time_l_save_it) == 0) 							&
			call comp_p % send_intg()


		time_l = time_l + 1
	end do
	
	end if
	
	if(proc_in(rank_world,COUPLER)) then

		if(ANALYSIS_FLAG) then
                	open(8,file = './configure/ocn_list.in')
                	read(8, nml = OCN_MAIN)
                	close(8)
                	c_ev_count = 1
               		do i = 2, INT(time_days_in_run/INT(time_save_cp/(24*60))) + 1
                		c_ev_count = c_ev_count + 1
                		!c_events(c_ev_count) % act = ACTION_READ_CP
                    		!c_events(c_ev_count) % src = COUPLER
                    		!c_events(c_ev_count) % dst = COMP_OCN
                    		!c_events(c_ev_count) % m = time_save_cp * (i - 1)
                    		!c_events(c_ev_count) % num_4D = c_inf(COMP_OCN) % rw(ACTION_READ_CP,ARRAY_TYPE_4D) % ar_size
                    		!c_events(c_ev_count) % num_3D = c_inf(COMP_OCN) % rw(ACTION_READ_CP,ARRAY_TYPE_3D) % ar_size
                    		!c_events(c_ev_count) % num_2D = c_inf(COMP_OCN) % rw(ACTION_READ_CP,ARRAY_TYPE_2D) % ar_size
				!c_events(c_ev_count) % num_2D_kind8 = c_inf(COMP_OCN) % rw(ACTION_READ_CP,ARRAY_TYPE_2D_KIND8) % ar_size
				
				c_events(c_ev_count) % act = ACTION_SAVE_CP
				c_events(c_ev_count) % src = COMP_OCN
				c_events(c_ev_count) % dst = COUPLER
				c_events(c_ev_count) % m = time_save_cp * (i - 1)
				c_events(c_ev_count) % num_4D = c_inf(COMP_OCN) % rw(ACTION_SAVE_CP, ARRAY_TYPE_4D) % ar_size
				c_events(c_ev_count) % num_3D = c_inf(COMP_OCN) % rw(ACTION_SAVE_CP, ARRAY_TYPE_3D) % ar_size
				c_events(c_ev_count) % num_2D = c_inf(COMP_OCN) % rw(ACTION_SAVE_CP, ARRAY_TYPE_2D) % ar_size
				c_events(c_ev_count) % num_2D_kind8 = c_inf(COMP_OCN) % rw(ACTION_SAVE_CP, ARRAY_TYPE_2D_KIND8) % ar_size
				
                	enddo
	        endif
	
		! Synchronize and set timer
		call MPI_BARRIER(comm_local, ierr)
		wall_time_start_cycle = MPI_WTIME()
	
		! Wait for event and make apropriate action
		do c_ev_num = 1, c_ev_count

			!if (ANALYSIS_FLAG) then
			!	call shared_timer_min(REAL(c_events(c_ev_num) % m)) 
               		!	time_read_cp = TRIM(I2S(time_start_year))//TRIM(I2S(time_month,2))//TRIM(I2S(time_day,2))//"00"
                	!endif
		
			select case (c_events(c_ev_num) % act)

				case(ACTION_READ_IC)
					call c_rw_get_data
					call c_distribute_comp

				case(ACTION_READ_CP)
					call c_rw_get_data
					call c_distribute_comp

				case(ACTION_READ_FD)
					call c_rw_get_data
					call c_distribute_comp

				case(ACTION_SAVE_CP)

					call c_gather_comp
					call c_rw_put_data
					if(ANALYSIS_FLAG) call MPI_BARRIER(mpi_comm_world, ierr)	!synchronyzation with OCN to prevent loss of some analysis data
					
				case(ACTION_SAVE_DG)
					call c_gather_comp
					call c_rw_put_data

				case(ACTION_MAPPING)
					call c_gather_comp
					if(c_map_info(c_events(c_ev_num) % src,c_events(c_ev_num) % dst,1,MAP_TYPE) == MAP_SRC_TYPE) call c_map_src_type
					if(c_map_info(c_events(c_ev_num) % src,c_events(c_ev_num) % dst,1,MAP_TYPE) == MAP_DST_TYPE) call c_map_dst_type
!					call test_global_map
					call c_distribute_comp
					
				case(ACTION_SAVE_IT)
					call c_rw_put_intg
			end select
			call MPI_BARRIER(comm_local, ierr)
			! Write log information
			if(run_save_log) call c_spy_log(c_ev_num)
			
		end do

	end if

!==============================================================================================================
!==============================================================================================================
! Section 6 : Finalizing procedures
!==============================================================================================================
!==============================================================================================================
	call MPI_BARRIER(MPI_COMM_WORLD, ierr)
	
	if(proc_in(rank_world,COUPLER)) then

!	      	call c_fin_deallocate_arrays

		! Close all used netCDF files
		call c_rw_close_files()

		call MPI_BARRIER(comm_local, ierr)
		call c_spy_log(MYNULL)

	else
		!call comp_p % finalize
	end if

#ifdef GA	
	call ga_destructor()
#endif

	call MPI_BARRIER(MPI_COMM_WORLD, ierr)
	call MPI_FINALIZE(ierr)

	stop
	end

