!========================================================================================
! This is automatically generated adapter for CMF-system.
! WARNING: for correct work don't change any names generated by generate_comp.sh script.
!
! Subroutines will be called by main program. 
! Fill them with appropriate actions, suitable for your model.
!========================================================================================

module atm_ncar_cpl_adapter_module

use component_module
use decomposition_module
implicit none

type, extends(Component) :: atm_ncar_cpl_adapter
 contains
    procedure :: ini_reg_comp
    procedure :: ini_allocate
    procedure :: ini_reg_data    
    procedure :: ini_main
    procedure :: make_step
    procedure :: finalize 
end type

    integer atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day
    integer atm_forcing_type
    logical atm_read_cp
   
    real (kind=4), allocatable :: t_10(:,:), q_10(:,:), u_10(:,:), v_10(:,:)
    real (kind=4), allocatable :: swdn(:,:), lwdn(:,:), rain(:,:), snow(:,:)
    type(Decomposition) :: dec
    
 CONTAINS

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register your model in the system.
! Here you should (probably) read or define necessary information and 
! must call this % register_model()
!----------------------------------------------------------------------------------------

subroutine ini_reg_comp(this)
	class(atm_ncar_cpl_adapter) :: this
    
	namelist /ATM_MAIN/ atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day, atm_read_cp
	namelist /ATM_PHYSICS/ atm_forcing_type
	
	open(9, file= "./configure/atm_list.in")
	read(9, nml = ATM_MAIN)
	read(9, nml = ATM_PHYSICS)
	close(9)

	dec = Decomposition(atm_i_size, atm_j_size, atm_k_size, "1D", CompSplitter % comm_local_size(), CompSplitter % rank_local())
    
	! Call registration routine of CMF (it needs only size of grid and timestep in whole(!) seconds)
	call this % register_model(decomp = dec, timestep_sec = 60*60) 
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to allocate all your data, which will interact with system.
! This is necessary, because you ini_reg_data() use adresses of your arrays and 
! they should be allocated at time of registration.
! Here you should do all array allocations.
!----------------------------------------------------------------------------------------

subroutine ini_allocate(this)
    class(atm_ncar_cpl_adapter) :: this

    allocate(t_10(dec % i_size_loc(), dec % j_size_loc())); t_10 = 0
    allocate(q_10(dec % i_size_loc(), dec % j_size_loc())); q_10 = 0
    allocate(u_10(dec % i_size_loc(), dec % j_size_loc())); u_10 = 0
    allocate(v_10(dec % i_size_loc(), dec % j_size_loc())); v_10 = 0
    allocate(lwdn(dec % i_size_loc(), dec % j_size_loc())); lwdn = 0
    allocate(swdn(dec % i_size_loc(), dec % j_size_loc())); swdn = 0
    allocate(rain(dec % i_size_loc(), dec % j_size_loc())); rain = 0
    allocate(snow(dec % i_size_loc(), dec % j_size_loc())); snow = 0
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register all data you want to interact with system.
! Here you should call this % register_array(), this % register_*_event() or
! this % raise_event() to get result immediately.
!----------------------------------------------------------------------------------------

subroutine ini_reg_data(this)
    class(atm_ncar_cpl_adapter) :: this

    !==================================================================
    !------------corrected normal year forcing-------------------------    
    !==================================================================
    
    ! Register required arrays
    call this % register_array(arr_name = "T_10_MOD",   arr = t_10)
    call this % register_array(arr_name = "Q_10_MOD",   arr = q_10)
    call this % register_array(arr_name = "U_10_MOD",   arr = u_10)
    call this % register_array(arr_name = "V_10_MOD",   arr = v_10)
    call this % register_array(arr_name = "LWDN_MOD",   arr = lwdn)
    call this % register_array(arr_name = "SWDN_MOD",   arr = swdn)
    call this % register_array(arr_name = "RAIN",       arr = rain)
    call this % register_array(arr_name = "SNOW",       arr = snow)
    
    call this % register_array(arr_name = "T_10MOD_AI",   arr = t_10)
    call this % register_array(arr_name = "Q_10MOD_AI",   arr = q_10)
    call this % register_array(arr_name = "U_10MOD_AI",   arr = u_10)
    call this % register_array(arr_name = "V_10MOD_AI",   arr = v_10)
    call this % register_array(arr_name = "LWDNMOD_AI",   arr = lwdn)
    call this % register_array(arr_name = "SWDNMOD_AI",   arr = swdn)
    call this % register_array(arr_name = "RAIN_AI",       arr = rain)
    call this % register_array(arr_name = "SNOW_AI",       arr = snow)
    
    !Create SyncedGenerators to sync with NCAR files
    !CORE-I atmosphere
    if(atm_forcing_type==1) then	
	call this % register_synced_event(arr_name = "T_10_MOD",    src = "./data/CNYFv2/t_10.15JUNE2009.nc")    
	call this % register_synced_event(arr_name = "Q_10_MOD",    src = "./data/CNYFv2/q_10.15JUNE2009.nc")    
	call this % register_synced_event(arr_name = "U_10_MOD",    src = "./data/CNYFv2/u_10.15JUNE2009.nc")    
	call this % register_synced_event(arr_name = "V_10_MOD",    src = "./data/CNYFv2/v_10.15JUNE2009.nc")    
	call this % register_synced_event(arr_name = "LWDN_MOD",    src = "./data/CNYFv2/ncar_rad.15JUNE2009.nc") 
	call this % register_synced_event(arr_name = "SWDN_MOD",    src = "./data/CNYFv2/ncar_rad.15JUNE2009.nc") 
	call this % register_synced_event(arr_name = "RAIN",        src = "./data/CNYFv2/ncar_precip.15JUNE2009.nc") 
	call this % register_synced_event(arr_name = "SNOW",        src = "./data/CNYFv2/ncar_precip.15JUNE2009.nc") 
    end if
    
    !CORE-II atmosphere
    if(atm_forcing_type==2) then
	call this % register_synced_event(arr_name = "T_10_MOD",    src = "./data/IAFv2/t_10.1948-2009.23OCT2012.nc")    
	call this % register_synced_event(arr_name = "Q_10_MOD",    src = "./data/IAFv2/q_10.1948-2009.23OCT2012.nc")    
	call this % register_synced_event(arr_name = "U_10_MOD",    src = "./data/IAFv2/u_10.1948-2009.23OCT2012.nc")    
	call this % register_synced_event(arr_name = "V_10_MOD",    src = "./data/IAFv2/v_10.1948-2009.23OCT2012.nc")    
	call this % register_synced_event(arr_name = "LWDN_MOD",    src = "./data/IAFv2/ncar_rad.1948-2009.23OCT2012.nc") 
	call this % register_synced_event(arr_name = "SWDN_MOD",    src = "./data/IAFv2/ncar_rad.1948-2009.23OCT2012.nc") 
	call this % register_synced_event(arr_name = "RAIN",        src = "./data/IAFv2/ncar_precip.1948-2009.23OCT2012.nc") 
	call this % register_synced_event(arr_name = "SNOW",        src = "./data/IAFv2/ncar_precip.1948-2009.23OCT2012.nc")     
    end if    
    
    !call this % register_periodic_event(arr_name = "T_10_MOD", act = "SAVE_DG", dh = 6)

!     ! Create PeriodicGenerators to send it to OCN
    call this % register_periodic_event(arr_name = "T_10_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3) !60*60*3 seconds is the time of the first record in CNYFv2 and IAFv2
    call this % register_periodic_event(arr_name = "Q_10_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "U_10_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "V_10_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "LWDN_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "SWDN_MOD",  act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "RAIN",      act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    call this % register_periodic_event(arr_name = "SNOW",      act = "SEND_MP", dst = "OCN", dh = 6, delay = 60*60*3)
    
    ! Create PeriodicGenerators to send it to ICE
    if(TRIM(ModelsInfo(CompSplitter % comp_id("ICE")) % model_name())=='ICE_CICE') then
	call this % register_periodic_event(arr_name = "T_10MOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "Q_10MOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "U_10MOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "V_10MOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "LWDNMOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "SWDNMOD_AI",  act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "RAIN_AI",      act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
	call this % register_periodic_event(arr_name = "SNOW_AI",      act = "SEND_MP", dst = "ICE", dh = 6, delay = 60*60*3)
    end if
    
    
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program before main time cycle.
! Here you should do all model preparation actions.
!----------------------------------------------------------------------------------------

subroutine ini_main(this)
    class(atm_ncar_cpl_adapter) :: this

end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at every step of time cycle. 
! Here you should represent one time step of your model with all physics and internal 
! exhanges (like halo MPI-updates).
!----------------------------------------------------------------------------------------

subroutine make_step(this)
    class(atm_ncar_cpl_adapter) :: this
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at the end.
! Here you should do all postprocesing actions and memory deallocations.
!----------------------------------------------------------------------------------------

subroutine finalize(this)        
    class(atm_ncar_cpl_adapter) :: this        

    deallocate(t_10, q_10, u_10, v_10)
    deallocate(swdn, lwdn, rain, snow)
    
end subroutine

!========================================================================================

end module
    