	
	module cpl_atm_gfs_module
	
	use comp_module
	use shared_module
	
	implicit none

!==================================================================================
! Define type 'cpl_atm', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_atm_gfs
	
	CONTAINS
		PROCEDURE :: ini_params => a_ini_params
		PROCEDURE :: ini_reg_comp => a_ini_reg_comp
		PROCEDURE :: ini_reg_data => a_ini_reg_data
		PROCEDURE :: ini_allocate => a_ini_allocate
		PROCEDURE :: ini_main => a_ini_main
		PROCEDURE :: main_step => a_main_step
	END TYPE cpl_atm_gfs
	
	
	integer atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day
	integer atm_forcing_type
	logical atm_read_cp
	
	real (kind=4), allocatable :: t_2(:,:), q_2(:,:), u_10(:,:), v_10(:,:)
	real (kind=4), allocatable :: swdn(:,:), lwdn(:,:), rain(:,:), snow(:,:), runoff(:,:)
	
	CONTAINS
	
!================================================================================================================================================================
	subroutine a_ini_params(this)
		
	class(cpl_atm_gfs) :: this
		namelist /ATM_MAIN/ atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day, atm_read_cp
		
		open(9, file= "./comp_atm_gfs/atm_list.in")
		read(9, nml = ATM_MAIN)
		close(9)

	end subroutine a_ini_params
!================================================================================================================================================================

	subroutine a_ini_reg_comp(this)
		
	class(cpl_atm_gfs) :: this

		call this % reg_model(	reg_np = run_np(COMP_ATM),		&
					reg_i_size = atm_i_size, 		&
					reg_j_size = atm_j_size,		&
					reg_k_size = atm_k_size,		&
					reg_m_size = atm_m_size,		&
					reg_decomp_type_2D = .FALSE.,		&
					reg_i_cycle = .TRUE.,			&
					reg_bsc_grid = .FALSE.,			&
					reg_time_steps_day = atm_steps_in_day,	&
					reg_time_save_it = 0,			&
					reg_read_cp = atm_read_cp,		&
					reg_halo_stencil = 0,			&
					reg_halo_wide = 0			)
	
	end subroutine a_ini_reg_comp

!================================================================================================================================================================

	subroutine a_ini_reg_data(this)
		
	class(cpl_atm_gfs) :: this
	
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_2, name = 't2m', period = 3*60, file = "./data/GFS/t2m.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_2, name = 'q2m', period = 3*60, file = "./data/GFS/q2m.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'u10', period = 3*60, file = "./data/GFS/u10.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'v10', period = 3*60, file = "./data/GFS/v10.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'trdi', period = 3*60, file = "./data/GFS/trdi.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'srdi', period = 3*60, file = "./data/GFS/srdi.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'tprr', period = 3*60, file = "./data/GFS/tprr.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'tprs', period = 3*60, file = "./data/GFS/tprs.nc"  )
		!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff, name = 'Foxx_o_roff', period = 1440, file = "./data/GFS/runoff_core.nc"  )
	
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=t_2, period = 3*60, name = 'T', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=q_2, period = 3*60, name = 'Q', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=u_10, period = 3*60, name = 'U', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=v_10, period = 3*60, name = 'V', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 3*60, name = 'LWDN', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=swdn, period = 3*60, name = 'SWDN', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=rain, period = 3*60, name = 'RAIN', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=snow, period = 3*60, name = 'SNOW', map_type = MAP_SRC_TYPE )
		!call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=runoff, period = 3*60, name = 'runoff', map_type = MAP_SRC_TYPE )

		!--------------functions to be transfered from ATM to ICE-------------
		if(TRIM(cmp % names(COMP_ICE))=="ICE_CICE") then
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=t_2, period = 3*60, name = 'T', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=q_2, period = 3*60, name = 'Q', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=u_10, period = 3*60, name = 'U', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=v_10, period = 3*60, name = 'V', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 3*60, name = 'LWDN', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=swdn, period = 3*60, name = 'SWDN', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=rain, period = 3*60, name = 'RAIN', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=snow, period = 3*60, name = 'SNOW', map_type = MAP_SRC_TYPE )
		end if

		!-------------------------diagnostics---------------------------------
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=t_2, name='T2m_GFS',period=7200)
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=lwdn, name='LWDN_MOD',period=1440)
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=rain, name='RAIN',period=1440)
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=snow, name='snow_for3',period=360)
		
	end subroutine a_ini_reg_data	
	
	
!================================================================================================================================================================
	
	subroutine a_ini_allocate(this)
		
	class(cpl_atm_gfs) :: this
	
		allocate(t_2(cmp % i_loc_size, cmp % j_loc_size))
		allocate(q_2(cmp % i_loc_size, cmp % j_loc_size))
		allocate(u_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(v_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(lwdn(cmp % i_loc_size, cmp % j_loc_size))
		allocate(swdn(cmp % i_loc_size, cmp % j_loc_size))
		allocate(rain(cmp % i_loc_size, cmp % j_loc_size))
		allocate(snow(cmp % i_loc_size, cmp % j_loc_size))
		!allocate(runoff(cmp % i_loc_size, cmp % j_loc_size))
	
	end subroutine a_ini_allocate
!================================================================================================================================================================

	subroutine a_ini_main(this)
	class(cpl_atm_gfs) :: this
	end subroutine a_ini_main
				
!================================================================================================================================================================

	subroutine a_main_step(this)
	class(cpl_atm_gfs) :: this
	end subroutine a_main_step
		
!================================================================================================================================================================
	
	end module cpl_atm_gfs_module
	
