module cpl_atm_inmcm4_module
	
	use comp_module
	use shared_module
	implicit none

!==================================================================================
! Define type 'cpl_atm', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_atm_inmcm4
	
	CONTAINS
		PROCEDURE :: ini_params => a_ini_params
		PROCEDURE :: ini_reg_comp => a_ini_reg_comp
		PROCEDURE :: ini_reg_data => a_ini_reg_data
		PROCEDURE :: ini_allocate => a_ini_allocate
		PROCEDURE :: ini_main => a_ini_main
		PROCEDURE :: main_step => a_main_step
	END TYPE cpl_atm_inmcm4
	
	
	integer atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day
	logical atm_read_cp
	
	real (kind=4), allocatable :: t_10(:,:), q_10(:,:), u_10(:,:), v_10(:,:)
	real (kind=4), allocatable :: swdn(:,:), lwdn(:,:), rain(:,:)
	
	CONTAINS
	
!================================================================================================================================================================
	subroutine a_ini_params(this)
		class(cpl_atm_inmcm4) :: this
		namelist /ATM_MAIN/ atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day, atm_read_cp
		
		open(9, file= "./comp_atm_inmcm4/atm_inmcm4_list.in")
		read(9, nml = ATM_MAIN)
		close(9)
	end subroutine a_ini_params
!================================================================================================================================================================

	subroutine a_ini_reg_comp(this)
		class(cpl_atm_inmcm4) :: this
		call this % reg_model(	reg_np = run_np(COMP_ATM),		&
					reg_i_size = atm_i_size, 		&
					reg_j_size = atm_j_size,		&
					reg_k_size = atm_k_size,		&
					reg_m_size = atm_m_size,		&
					reg_decomp_type_2D = .FALSE.,		&
					reg_i_cycle = .TRUE.,			&
					reg_bsc_grid = .FALSE.,			&
					reg_time_steps_day = atm_steps_in_day,	&
					reg_time_save_it = 0,			&
					reg_read_cp = atm_read_cp,		&
					reg_halo_stencil = 0,			&
					reg_halo_wide = 0			)
	end subroutine a_ini_reg_comp

!================================================================================================================================================================

	subroutine a_ini_reg_data(this)
		
		class(cpl_atm_inmcm4) :: this
		
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'tas', 	period = (365*1440)/12, file = "./data/INMCM4/tas.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'huss', period = (365*1440)/12, file = "./data/INMCM4/huss.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'uas', 	period = (365*1440)/12, file = "./data/INMCM4/uas.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'vas', 	period = (365*1440)/12, file = "./data/INMCM4/vas.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'rlds', period = (365*1440)/12, file = "./data/INMCM4/rlds.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'rsds', period = (365*1440)/12, file = "./data/INMCM4/rsds.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'pr', 	period = (365*1440)/12, file = "./data/INMCM4/pr.nc") !_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"  )
		
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=t_10, period = 24*60, name = 'T_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=q_10, period = 24*60, name = 'Q_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=u_10, period = 24*60, name = 'U_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=v_10, period = 24*60, name = 'V_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 24*60, name = 'LWDN_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=swdn, period = 24*60, name = 'SWDN_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=rain, period = 24*60, name = 'RAIN', map_type = MAP_SRC_TYPE )

		!--------------functions to be transfered from ATM to ICE-------------
		if(TRIM(cmp % names(COMP_ICE))=="ICE_CICE") then
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=t_10, period = 24*60, name = 'T_10_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=q_10, period = 24*60, name = 'Q_10_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=u_10, period = 24*60, name = 'U_10_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=v_10, period = 24*60, name = 'V_10_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 24*60, name = 'LWDN_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=swdn, period = 24*60, name = 'SWDN_MOD', map_type = MAP_SRC_TYPE )
			call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=rain, period = 24*60, name = 'SNOW', map_type = MAP_SRC_TYPE )
		end if

		!-------------------------diagnostics---------------------------------
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=t_10, name='T_10_MOD',period=7200)
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=lwdn, name='LWDN_MOD',period=1440)
		!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=rain, name='RAIN',period=1440)
		
	end subroutine a_ini_reg_data	
	
	
!================================================================================================================================================================
	
	subroutine a_ini_allocate(this)
		class(cpl_atm_inmcm4) :: this
		allocate(t_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(q_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(u_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(v_10(cmp % i_loc_size, cmp % j_loc_size))
		allocate(lwdn(cmp % i_loc_size, cmp % j_loc_size))
		allocate(swdn(cmp % i_loc_size, cmp % j_loc_size))
		allocate(rain(cmp % i_loc_size, cmp % j_loc_size))
	end subroutine a_ini_allocate
!================================================================================================================================================================

	subroutine a_ini_main(this)
		class(cpl_atm_inmcm4) :: this
	end subroutine a_ini_main
				
!================================================================================================================================================================

	subroutine a_main_step(this)
		class(cpl_atm_inmcm4) :: this
	end subroutine a_main_step
		
	
end module cpl_atm_inmcm4_module
	
