	
	module off_atm_inmcm4_module
	
	use off_module
	use shared_utils_module
	use netcdf
	
	implicit none
	integer :: additional_ny=0	!0 - to make full grid, <0 - to make reduced grid (eg. for Arctic only)
	
	TYPE, EXTENDS(GridConstructor) :: off_atm_inmcm4
	
	CONTAINS
	
	PROCEDURE :: make_grid => a_make_grid
	PROCEDURE :: make_mask => a_make_mask
	PROCEDURE :: make_ic => a_make_ic
	
	END TYPE off_atm_inmcm4
	
	CONTAINS

!================================================================================================================================================================
	
	subroutine a_make_grid(this)
	
	class(off_atm_inmcm4) :: this

	! Input file
	character(*), parameter :: filename = "../data/INMCM4/tas_Amon_INM-CM4-8_lgm_r1i1p1f1_gr1_200001-209912.nc"

	! netCDF variables
	integer ncid
	integer nx_dimid, ny_dimid
	integer lon_id, lat_id
	integer nc_err
	
	! Work variables
	integer i, j, counter, il, jl

	! Temporary grid variables
	real(kind = 8), allocatable :: input_lat(:), input_lon(:), lat_t(:), lon_t(:)

!===================================
! Inquire grid from INMCM4 netCDF-file
!===================================

	! Open file
	nc_err = NF90_OPEN(TRIM(filename), NF90_NOWRITE, ncid)
	
	! Inquire dimension sizes
 	nc_err = NF90_INQ_DIMID(ncid, "lon", nx_dimid)
	nc_err = NF90_INQUIRE_DIMENSION(ncid, nx_dimid, len = il)
 	nc_err = NF90_INQ_DIMID(ncid, "lat", ny_dimid)
 	nc_err = NF90_INQUIRE_DIMENSION(ncid, ny_dimid, len = jl)

 	if(il /= this % nx .OR. jl /= this % ny) then
		write(*,*) 'Error in ATM_INMCM4 function: wrong grid sizes of input netCDF.'
		stop
	end if
			
	! Inquire coordinate variables
	allocate(input_lon(il))
	nc_err = NF90_INQ_VARID(ncid, "lon", lon_id)
	nc_err = NF90_GET_VAR(ncid, lon_id, input_lon, [1], [il]) 

	allocate(input_lat(jl))
	nc_err = NF90_INQ_VARID(ncid, "lat", lat_id)
	nc_err = NF90_GET_VAR(ncid, lat_id, input_lat, [1], [jl]) 
			
	nc_err = NF90_CLOSE(ncid)
	
!====================================================
! Make array of imaginary T-points, because we should
! provide corners of our cells.
!====================================================

	allocate(lon_t(il+1))
	allocate(lat_t(jl+1))	

	do i = 2, il
		lon_t(i) = (input_lon(i-1) + input_lon(i))/2.
	end do
	lon_t(1) = input_lon(1) - (lon_t(2) - input_lon(1))
	lon_t(il+1) = input_lon(il) + (input_lon(il) - lon_t(il))
		
	do j = 2, jl
		lat_t(j) = (input_lat(j-1) + input_lat(j))/2.
	end do
	lat_t(1) = input_lat(1) - (lat_t(2) - input_lat(1))
	lat_t(jl+1) = input_lat(jl) + (input_lat(jl) - lat_t(jl))		
		
!=====================================================
! Fill ouput 'GridType' structure with our information 
!=====================================================

 	do j = 1, this % ny
	do i = 1, this % nx
 	
 		this % center_lat(i,j  ) = input_lat(j) 
 		this % corner_lat(1,i,j) = lat_t(j)
 		this % corner_lat(2,i,j) = lat_t(j)
 		this % corner_lat(3,i,j) = lat_t(j+1)
 		this % corner_lat(4,i,j) = lat_t(j+1)
 
 		this % center_lon(i,j  ) = input_lon(i)
 		this % corner_lon(1,i,j) = lon_t(i)
 		this % corner_lon(2,i,j) = lon_t(i+1)
 		this % corner_lon(3,i,j) = lon_t(i+1)
 		this % corner_lon(4,i,j) = lon_t(i)
 		
		this % mask2D(i,j) = 1.
 		
 	end do
 	end do
 		
	this % z(:) = 0.	
	
      end subroutine a_make_grid
      
!================================================================================================================================================================
	
	subroutine a_make_mask(this)
	
	class(off_atm_inmcm4) :: this

	end subroutine a_make_mask
	
!================================================================================================================================================================
	
	subroutine a_make_ic(this)
	
	class(off_atm_inmcm4) :: this

	end subroutine a_make_ic
	
!================================================================================================================================================================

	end module off_atm_inmcm4_module
	
	