!========================================================================================
! This is automatically generated adapter for CMF-system.
! WARNING: for correct work don't change any names generated by generate_comp.sh script.
!
! Subroutines will be called by off_main program. 
! Fill them with appropriate actions, suitable for your model.
!========================================================================================

module atm_interim_off_adapter_module
use offline_module
implicit none
    
type, extends(Offline) :: atm_interim_off_adapter
 contains
    procedure, public :: make_grid
    procedure, public :: make_ic
end type
    
 CONTAINS

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by off_main program to get information about model grid.
! Here you should build your grid and then call this % register_grid(grid).
!----------------------------------------------------------------------------------------

subroutine make_grid(this, name, dims, grid_type)
    class(atm_interim_off_adapter) :: this
    
    ! These parameters are passed from main program - you can use it
    character(*), intent(in) :: name, grid_type
    integer, intent(in) :: dims(3)

    ! This object you should fill
    type(ModelGrid) :: grid
    
    character(*), parameter :: filename = "../data/Interim/interim_grid.nc"
    integer i, j, counter, il, jl
    real(kind = 8) :: input_lon(dims(1)), input_lat(dims(2)), lon_t(dims(1) + 1), lat_t(dims(2) + 1)
    type(FileHandler_NC) :: fh

!===================================
! Inquire grid from INTERIM netCDF-file
!===================================

    ! Open file and inquire dims
    call fh % open_file(filename)
    il = fh % get_dim_size("longitude")
    jl = fh % get_dim_size("latitude")

    if(il /= dims(1) .OR. jl /= dims(2)) then
        write(*,*) 'Error in ATM_INTERIM function: wrong grid sizes of input netCDF.'
        stop
    end if
            
    ! Inquire coordinate variables
    call fh % open_var("longitude")
    call fh % get(input_lon, [1], [il])
    
    call fh % open_var("latitude")
    call fh % get(input_lat, [1], [jl])
    
    call fh % close_file()

!====================================================
! Make array of imaginary T-points, because we should
! provide corners of our cells.
!====================================================

    do i = 2, il
        lon_t(i) = (input_lon(i-1) + input_lon(i))/2.
    end do
    lon_t(1) = input_lon(1) - (lon_t(2) - input_lon(1))
    lon_t(il+1) = input_lon(il) + (input_lon(il) - lon_t(il))
        
    do j = 2, jl
        lat_t(j) = (input_lat(j-1) + input_lat(j))/2.
    end do
    lat_t(1) = input_lat(1) - (lat_t(2) - input_lat(1))
    lat_t(jl+1) = input_lat(jl) + (input_lat(jl) - lat_t(jl))    
    
    
        
!=====================================================
! Fill ouput 'ModelGrid' structure with our information 
!=====================================================

    ! Initialize grid object and allocate arrays
    call grid % init(name, dims, grid_type) 

    do j = 1, jl
    do i = 1, il
    
        grid % center_lat(i,j  ) = input_lat(j) 
        grid % corner_lat(1,i,j) = lat_t(j)
        grid % corner_lat(2,i,j) = lat_t(j)
        grid % corner_lat(3,i,j) = lat_t(j+1)
        grid % corner_lat(4,i,j) = lat_t(j+1)
 
        grid % center_lon(i,j  ) = input_lon(i)
        grid % corner_lon(1,i,j) = lon_t(i)
        grid % corner_lon(2,i,j) = lon_t(i+1)
        grid % corner_lon(3,i,j) = lon_t(i+1)
        grid % corner_lon(4,i,j) = lon_t(i)
                
    end do
    end do
        
    grid % mask2D = 1
    grid % level(:) = 0.    
    
    ! Register grid if Offline block
    call this % register_model_grid(grid)
        
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main to add to IC-file all data you want.
! Here you should (probably) create FileHandler_NC object and put data.
!----------------------------------------------------------------------------------------

subroutine make_ic(this, name, dims, grid_type)
    class(atm_interim_off_adapter) :: this
    
    ! These parameters are passed from main program - you can use it
    character(*), intent(in) :: name, grid_type
    integer, intent(in) :: dims(3)
        
end subroutine

!========================================================================================

end module