	
	module cpl_atm_interim_module
	
	use comp_module
	use shared_module
	
	implicit none

!==================================================================================
! Define type 'cpl_atm', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_atm_interim
	
	CONTAINS
		PROCEDURE :: ini_params => a_ini_params
		PROCEDURE :: ini_reg_comp => a_ini_reg_comp
		PROCEDURE :: ini_reg_data => a_ini_reg_data
		PROCEDURE :: ini_allocate => a_ini_allocate
		PROCEDURE :: ini_main => a_ini_main
		PROCEDURE :: main_step => a_main_step
	END TYPE cpl_atm_interim
	
	
	integer atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day
	logical atm_read_cp
	
	real (kind=4), allocatable :: t2_interim(:,:), dew2_interim(:,:), u10_interim(:,:), v10_interim(:,:),temp(:,:)
	real (kind=4), allocatable :: swdn_interim(:,:), lwdn_interim(:,:), prec_interim(:,:), runoff_interim(:,:),sst_interim(:,:),aice_interim(:,:)

	CONTAINS
	
!================================================================================================================================================================
	subroutine a_ini_params(this)
		
	class(cpl_atm_interim) :: this
	namelist /ATM_MAIN/ atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day, atm_read_cp
	
	open(9, file= "./comp_atm_interim/atm_list.in")
	read(9, nml = ATM_MAIN)
	close(9)

	end subroutine a_ini_params
!================================================================================================================================================================

	subroutine a_ini_reg_comp(this)
		
	class(cpl_atm_interim) :: this

	call this % reg_model(	reg_np = run_np(COMP_ATM),		&
				reg_i_size = atm_i_size, 		&
				reg_j_size = atm_j_size,		&
				reg_k_size = atm_k_size,		&
				reg_m_size = atm_m_size,		&
				reg_decomp_type_2D = .FALSE.,		&
				reg_i_cycle = .TRUE.,			&
				reg_bsc_grid = .FALSE.,			&
				reg_time_steps_day = atm_steps_in_day,	&
				reg_time_save_it = 0,			&
				reg_read_cp = atm_read_cp,		&
				reg_halo_stencil = 0,			&
				reg_halo_wide = 0			)
	
	end subroutine a_ini_reg_comp

!================================================================================================================================================================

	subroutine a_ini_reg_data(this)
		
	class(cpl_atm_interim) :: this
	
	integer counter, func, i, j

 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t2_interim, 		name = 't2m', 	period = 6*60, 	file = "./data/Interim/T2.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = dew2_interim, 	name = 'd2m', 	period = 6*60, 	file = "./data/Interim/dew.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u10_interim, 	name = 'u10', 	period = 6*60, 	file = "./data/Interim/U.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v10_interim, 	name = 'v10', 	period = 6*60, 	file = "./data/Interim/V.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = sst_interim, 	name = 'sst', 	period = 6*60, 	file = "./data/Interim/sst.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn_interim, 	name = 'strd', 	period = 12*60,	file = "./data/Interim/lwdn-step12.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn_interim, 	name = 'ssrd', 	period = 12*60,	file = "./data/Interim/swdn-step12.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = prec_interim, 	name = 'tp', 	period = 12*60,	file = "./data/Interim/prec-step12.nc"  )
 	call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = aice_interim, 	name = 'ci', 	period = 6*60,	file = "./data/Interim/aice.nc"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff_interim, 	name = 'ro', 	period = 12*60,	file = "./data/Interim/runoff-step12.nc"  )
 	
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t2_interim, 	name = 'v2t', 	period = 6*60, 	file = "./data/Interim3/T22001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = dew2_interim, 	name = 'v2d', 	period = 6*60, 	file = "./data/Interim3/dew2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u10_interim, 	name = 'v10u', 	period = 6*60, 	file = "./data/Interim3/U2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v10_interim, 	name = 'v10v', 	period = 6*60, 	file = "./data/Interim3/V2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = sst_interim, 	name = 'sst', 	period = 6*60, 	file = "./data/Interim3/sst2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn_interim, 	name = 'strd', 	period = 12*60,	file = "./data/Interim3/lwdn2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn_interim, 	name = 'ssrd', 	period = 12*60,	file = "./data/Interim3/swdn2001-2013.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = prec_interim, 	name = 'tp', 	period = 12*60,	file = "./data/Interim3/prec2001-2013.cdf"  )
 	!!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff_interim, 	name = 'ro', 	period = 12*60,	file = "./data/Interim3/runoff2001-2013.cdf"  )
 	 	
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t2_interim, 		name = 'v2t', 	period = 6*60, 	file = "./data/Interim1/T21979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = dew2_interim, 	name = 'v2d', 	period = 6*60, 	file = "./data/Interim1/dew1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u10_interim, 	name = 'v10u', 	period = 6*60, 	file = "./data/Interim1/U1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v10_interim, 	name = 'v10v', 	period = 6*60, 	file = "./data/Interim1/V1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = sst_interim, 	name = 'sst', 	period = 6*60, 	file = "./data/Interim1/sst1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn_interim, 	name = 'strd', 	period = 12*60,	file = "./data/Interim1/lwdn1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn_interim, 	name = 'ssrd', 	period = 12*60,	file = "./data/Interim1/swdn1979-1990.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = prec_interim, 	name = 'tp', 	period = 12*60,	file = "./data/Interim1/prec1979-1990.cdf"  )
 	!!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff_interim, 	name = 'ro', 	period = 12*60,	file = "./data/Interim1/runoff1979-1990.cdf"  )
	
	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t2_interim, 		name = 'v2t', 	period = 6*60, 	file = "./data/Interim2/T21991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = dew2_interim, 	name = 'v2d', 	period = 6*60, 	file = "./data/Interim2/dew1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u10_interim, 	name = 'v10u', 	period = 6*60, 	file = "./data/Interim2/U1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v10_interim, 	name = 'v10v', 	period = 6*60, 	file = "./data/Interim2/V1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = sst_interim, 	name = 'sst', 	period = 6*60, 	file = "./data/Interim2/sst1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn_interim, 	name = 'strd', 	period = 12*60,	file = "./data/Interim2/str1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn_interim, 	name = 'ssrd', 	period = 12*60,	file = "./data/Interim2/ssr1991-2000.cdf"  )
 	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = prec_interim, 	name = 'tp', 	period = 12*60,	file = "./data/Interim2/prec1991-2000.cdf"  )
 	!!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff_interim, 	name = 'ro', 	period = 12*60,	file = "./data/Interim2/runoff1991-2000.cdf"  )
		
	
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=t2_interim, 		period = 6*60, name = 't2m', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=dew2_interim,		period = 6*60, name = 'd2m', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=u10_interim, 		period = 6*60, name = 'u10', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=v10_interim,		period = 6*60, name = 'v10', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=sst_interim,		period = 6*60, name = 'sst', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=lwdn_interim,		period = 6*60, name = 'strd', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=swdn_interim,		period = 6*60, name = 'ssrd', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=prec_interim,		period = 6*60, name = 'tp', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=aice_interim,		period = 6*60, name = 'ci', map_type = MAP_SRC_TYPE )
	!call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=runoff_interim,	period = 6*60, name = 'ro', map_type = MAP_SRC_TYPE )

	if(TRIM(cmp % names(COMP_ICE))=="ICE_CICE") then
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=t2_interim, 		period = 6*60, name = 't2m', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=dew2_interim,		period = 6*60, name = 'd2m', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=u10_interim, 		period = 6*60, name = 'u10', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=v10_interim,		period = 6*60, name = 'v10', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=lwdn_interim,		period = 6*60, name = 'strd', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=swdn_interim,		period = 6*60, name = 'ssrd', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=prec_interim,		period = 6*60, name = 'tp', map_type = MAP_SRC_TYPE )
	end if
	
	
	end subroutine a_ini_reg_data	
	
	
!================================================================================================================================================================
	
	subroutine a_ini_allocate(this)
		
	class(cpl_atm_interim) :: this
	
	allocate(t2_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(dew2_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(u10_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(v10_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(lwdn_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(swdn_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(prec_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(runoff_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(sst_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(aice_interim	(cmp % i_loc_size, cmp % j_loc_size))
	allocate(temp		(cmp % i_loc_size, cmp % j_loc_size))
	end subroutine a_ini_allocate
!================================================================================================================================================================

	subroutine a_ini_main(this)
		
	class(cpl_atm_interim) :: this

	
	end subroutine a_ini_main
				
!================================================================================================================================================================

	subroutine a_main_step(this)
		
	class(cpl_atm_interim) :: this
	integer i,j
	
	temp=sst_interim
	do j=1,cmp % j_loc_size
	do i=1,cmp % i_loc_size
		if(temp(i,j).lt.-32766.) temp(i,j)=interim_search_nearest(i,j,sst_interim,-32766.)
	end do
	end do
	sst_interim=temp

	temp=aice_interim
	do j=1,cmp % j_loc_size
	do i=1,cmp % i_loc_size
		if(temp(i,j).lt.-32766.) temp(i,j)=interim_search_nearest(i,j,aice_interim,-32766.)
	end do
	end do
	aice_interim=temp
	
	
	end subroutine a_main_step
		
!================================================================================================================================================================
	
	real(kind=4) function interim_search_nearest(i_c,j_c,field, misVal)
	
	! Input variables
	integer i_c, j_c
	real(kind=4) field(cmp % i_loc_size, cmp % j_loc_size)
	real(kind=4) misVal
	
	! Work variables
	integer rad
	integer s,n,w,e
	
	! Search with increasing radius
	do rad = 1, MAX(cmp % i_loc_size,cmp % j_loc_size)
		s = MAX(j_c-rad, 1)
		n = MIN(j_c+rad, cmp % j_loc_size)
		w = MAX(i_c-rad, 1)
		e = MIN(i_c+rad, cmp % i_loc_size)
		
		! Get mean value of search result
		if(ANY(field(w:e, s:n) .ge. misVal)) then
			interim_search_nearest = SUM(field(w:e, s:n), field(w:e, s:n) .ge. misVal)/COUNT(field(w:e, s:n) .ge. misVal)
			return
		end if
	end do
	
	! If we found nothing
	Stop 'Interim search nearest error'
	
	end function interim_search_nearest
		
	end module cpl_atm_interim_module
	