	
	module cpl_atm_ncar_module
	
	use comp_module
	use shared_module
	
	implicit none

!==================================================================================
! Define type 'cpl_atm', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_atm_ncar
	
	CONTAINS
		PROCEDURE :: ini_params => a_ini_params
		PROCEDURE :: ini_reg_comp => a_ini_reg_comp
		PROCEDURE :: ini_reg_data => a_ini_reg_data
		PROCEDURE :: ini_allocate => a_ini_allocate
		PROCEDURE :: ini_main => a_ini_main
		PROCEDURE :: main_step => a_main_step
	END TYPE cpl_atm_ncar
	
	
	integer atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day
	integer atm_forcing_type
	logical atm_read_cp
	
!	real (kind=4), allocatable :: ocn_t(:,:), ocn_aice(:,:), ocn_tice(:,:)
	real (kind=4), allocatable :: t_10(:,:), q_10(:,:), u_10(:,:), v_10(:,:)
	real (kind=4), allocatable :: swdn(:,:), lwdn(:,:), rain(:,:), snow(:,:)
	real (kind=4), allocatable :: probe(:,:)
	
	CONTAINS
	
!================================================================================================================================================================
	subroutine a_ini_params(this)
		
	class(cpl_atm_ncar) :: this
	namelist /ATM_MAIN/ atm_i_size, atm_j_size, atm_k_size, atm_m_size, atm_steps_in_day, atm_read_cp
	namelist /ATM_PHYSICS/ atm_forcing_type
	
	open(9, file= "./configure/atm_list.in")
	read(9, nml = ATM_MAIN)
	read(9, nml = ATM_PHYSICS)
	close(9)

	end subroutine a_ini_params
!================================================================================================================================================================

	subroutine a_ini_reg_comp(this)
		
	class(cpl_atm_ncar) :: this

	call this % reg_model(	reg_np = run_np(COMP_ATM),		&
				reg_i_size = atm_i_size, 		&
				reg_j_size = atm_j_size,		&
				reg_k_size = atm_k_size,		&
				reg_m_size = atm_m_size,		&
				reg_decomp_type_2D = .FALSE.,		&
				reg_i_cycle = .TRUE.,			&
				reg_bsc_grid = .FALSE.,			&
				reg_time_steps_day = atm_steps_in_day,	&
				reg_time_save_it = 0,			&
				reg_read_cp = atm_read_cp,		&
				reg_halo_stencil = 0,			&
				reg_halo_wide = 0			)
	
	end subroutine a_ini_reg_comp

!================================================================================================================================================================

	subroutine a_ini_reg_data(this)
		
	class(cpl_atm_ncar) :: this
	
	!------------corrected normal year forcing-------------------------
 	if(atm_forcing_type==1) then
		!a variable with small reading period to make reading frequent enough (see VITIM release notes)
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = probe , name = 'probe' , period = 120, file = "./data/CNYFv2/probe-atm_ncar.nc" )

		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'T_10_MOD', period = 6*60, file = "./data/CNYFv2/t_10.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'Q_10_MOD', period = 6*60, file = "./data/CNYFv2/q_10.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'U_10_MOD', period = 6*60, file = "./data/CNYFv2/u_10.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'V_10_MOD', period = 6*60, file = "./data/CNYFv2/v_10.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'LWDN_MOD', period = 24*60, file = "./data/CNYFv2/ncar_rad.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'SWDN_MOD', period = 24*60, file = "./data/CNYFv2/ncar_rad.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'RAIN', period = (365*1440)/12, file = "./data/CNYFv2/ncar_precip.15JUNE2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'SNOW', period = (365*1440)/12, file = "./data/CNYFv2/ncar_precip.15JUNE2009.nc"  )
	end if
	
	!------------------interannual forcing----------------------------
	if(atm_forcing_type==2) then
		!a variable with small reading period to make reading frequent enough (see VITIM release notes)
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = probe , name = 'probe' , period = 120, file = "./data/CNYFv2/probe-atm_ncar.nc" )
		
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'T_10_MOD', period = 6*60, file = "./data/IAFv2/t_10.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'Q_10_MOD', period = 6*60, file = "./data/IAFv2/q_10.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'U_10_MOD', period = 6*60, file = "./data/IAFv2/u_10.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'V_10_MOD', period = 6*60, file = "./data/IAFv2/v_10.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'LWDN_MOD', period = 24*60, file = "./data/IAFv2/ncar_rad.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'SWDN_MOD', period = 24*60, file = "./data/IAFv2/ncar_rad.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'RAIN', period = (365*1440)/12, file = "./data/IAFv2/ncar_precip.1948-2009.23OCT2012.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'SNOW', period = (365*1440)/12, file = "./data/IAFv2/ncar_precip.1948-2009.23OCT2012.nc"  )
		
		!tester for FD periods
		!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'for3', period = (365*1440)/12, file = "./data/IAFv2/timer123.nc"  )
	end if
	
	!------------------specific year forcing----------------------------
	if(atm_forcing_type==3) then
		!a variable with small reading period to make reading frequent enough (see VITIM release notes)
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = probe , name = 'probe' , period = 120, file = "./data/CNYFv2/probe-atm_ncar.nc" )
		
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'T_10_MOD', period = 6*60, file = "./data/IAFv2-1948/t_10.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'Q_10_MOD', period = 6*60, file = "./data/IAFv2-1948/q_10.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'U_10_MOD', period = 6*60, file = "./data/IAFv2-1948/u_10.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'V_10_MOD', period = 6*60, file = "./data/IAFv2-1948/v_10.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'LWDN_MOD', period = 24*60, file = "./data/IAFv2-1948/ncar_rad.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'SWDN_MOD', period = 24*60, file = "./data/IAFv2-1948/ncar_rad.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'RAIN', period = (365*1440)/12, file = "./data/IAFv2-1948/ncar_precip.1948.05APR2010.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'SNOW', period = (365*1440)/12, file = "./data/IAFv2-1948/ncar_precip.1948.05APR2010.nc"  )
	end if
	
	!------------------CNYFv2 monthly mean forcing----------------------------
	if(atm_forcing_type==4) then
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'T_10_MOD', period = 24*60, file = "./data/CNYFv2_mm/t_10_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'Q_10_MOD', period = 24*60, file = "./data/CNYFv2_mm/q_10_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'U_10_MOD', period = 24*60, file = "./data/CNYFv2_mm/u_10_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'V_10_MOD', period = 24*60, file = "./data/CNYFv2_mm/v_10_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'LWDN_MOD', period = 24*60, file = "./data/CNYFv2_mm/ncar_rad_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'SWDN_MOD', period = 24*60, file = "./data/CNYFv2_mm/ncar_rad_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'RAIN', period = 24*60, file = "./data/CNYFv2_mm/ncar_precip_mm_january.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'SNOW', period = 24*60, file = "./data/CNYFv2_mm/ncar_precip_mm_january.nc"  )
	end if
	
	!------------------interannual forcing 1982-2009----------------------------
	if(atm_forcing_type==5) then
		!a variable with small reading period to make reading frequent enough (see VITIM release notes)
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = probe , name = 'probe' , period = 120, file = "./data/CNYFv2/probe-atm_ncar.nc" )
		
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = t_10, name = 'T_10_MOD', period = 6*60, file = "./data/IAFv2_1982-2009/t_10.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = q_10, name = 'Q_10_MOD', period = 6*60, file = "./data/IAFv2_1982-2009/q_10.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = u_10, name = 'U_10_MOD', period = 6*60, file = "./data/IAFv2_1982-2009/u_10.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = v_10, name = 'V_10_MOD', period = 6*60, file = "./data/IAFv2_1982-2009/v_10.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = lwdn, name = 'LWDN_MOD', period = 24*60, file = "./data/IAFv2_1982-2009/ncar_rad.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = swdn, name = 'SWDN_MOD', period = 24*60, file = "./data/IAFv2_1982-2009/ncar_rad.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = rain, name = 'RAIN', period = (365*1440)/12, file = "./data/IAFv2_1982-2009/ncar_precip.1982-2009.nc"  )
		call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = snow, name = 'SNOW', period = (365*1440)/12, file = "./data/IAFv2_1982-2009/ncar_precip.1982-2009.nc"  )
	
		call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=U_10, name='U_10_MOD',period=14400)
	end if
	
	
	
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=t_10, period = 6*60, name = 'T_10_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=q_10, period = 6*60, name = 'Q_10_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=u_10, period = 6*60, name = 'U_10_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=v_10, period = 6*60, name = 'V_10_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 6*60, name = 'LWDN_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=swdn, period = 6*60, name = 'SWDN_MOD', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=rain, period = 6*60, name = 'RAIN', map_type = MAP_SRC_TYPE )
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=snow, period = 6*60, name = 'SNOW', map_type = MAP_SRC_TYPE )

	!--------------functions to be transfered from ATM to ICE-------------
	if(TRIM(cmp % names(COMP_ICE))=="ICE_CICE") then
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=t_10, period = 6*60, name = 'T_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=q_10, period = 6*60, name = 'Q_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=u_10, period = 6*60, name = 'U_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=v_10, period = 6*60, name = 'V_10_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=lwdn, period = 6*60, name = 'LWDN_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=swdn, period = 6*60, name = 'SWDN_MOD', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=rain, period = 6*60, name = 'RAIN', map_type = MAP_SRC_TYPE )
		call this % reg_array(send_to=COMP_ICE, action=ACTION_MAPPING, DIM2_arr=snow, period = 6*60, name = 'SNOW', map_type = MAP_SRC_TYPE )
	end if

	!-------------------------diagnostics---------------------------------
	call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=t_10, name='T_10_MOD',period=7200)
	!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=lwdn, name='LWDN_MOD',period=1440)
	!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=rain, name='RAIN',period=1440)
	!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=snow, name='snow_for3',period=360)
		
	end subroutine a_ini_reg_data	
	
	
!================================================================================================================================================================
	
	subroutine a_ini_allocate(this)
		
	class(cpl_atm_ncar) :: this
	
!	allocate(ocn_t   (cmp % i_loc_size, cmp % j_loc_size))
!	allocate(ocn_aice(cmp % i_loc_size, cmp % j_loc_size))
!	allocate(ocn_tice(cmp % i_loc_size, cmp % j_loc_size))
	
	allocate(t_10(cmp % i_loc_size, cmp % j_loc_size))
	allocate(q_10(cmp % i_loc_size, cmp % j_loc_size))
	allocate(u_10(cmp % i_loc_size, cmp % j_loc_size))
	allocate(v_10(cmp % i_loc_size, cmp % j_loc_size))
	allocate(lwdn(cmp % i_loc_size, cmp % j_loc_size))
	allocate(swdn(cmp % i_loc_size, cmp % j_loc_size))
	allocate(rain(cmp % i_loc_size, cmp % j_loc_size))
	allocate(snow(cmp % i_loc_size, cmp % j_loc_size))
	allocate(probe(cmp % i_loc_size, cmp % j_loc_size))
	
	end subroutine a_ini_allocate
!================================================================================================================================================================

	subroutine a_ini_main(this)
		
	class(cpl_atm_ncar) :: this
!-------
!	artificial atmosphere. FOR UNITS SEE o_tf_par_read_core.f90
!-------
	if(atm_forcing_type==0) then
		t_10 = 273.-20.	!temperature [deg.K] 
		q_10 = 0.002	!specific humidity (kg/kg)

		u_10 = 12.	!wind speed components [m/sec]
		v_10 = 0.

		lwdn = 50.	!total longwave radiation [w/m**2] !!WITHOUT Stephan_Boltzmann thermal radiation
		swdn = 5.	!shortwave solar radiation [w/m**2] 
		rain = 0.	!SEE o_tf_par_read_core.f90
		snow = 0.
	end if
	
	end subroutine a_ini_main
				
!================================================================================================================================================================

	subroutine a_main_step(this)
		
	class(cpl_atm_ncar) :: this
	
	!initialize array for calendar checking

!	if(rank_local==0) then
!		open(file='iafv2-1948-series.dat',unit=1) 
!		if(mod(time_l,atm_steps_in_day)==1) write(1,*) lwdn(50,50),time_l
!	end if		
!	close(1)	
	end subroutine a_main_step
		
!================================================================================================================================================================
	
	end module cpl_atm_ncar_module
	
