#include "ice_2Ddomain"

!-----------------------------------------
!Subroutines for conversion of ocean data to CICE format and vice versa
!-----------------------------------------

!~~~~~~~
	module ocn_general_module
!~~~~~~~

	use ice_par_module
	use non_ga_ice_array_module
	implicit none
	save

!------
!	fields to be received from OCN (see some comments in allocation section)
!------
	real (kind=4), allocatable, dimension(:,:) ::	sst_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	sss_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	frzmlt_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	uocn_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	vocn_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	ss_tltx_interlayer
  	real (kind=4), allocatable, dimension(:,:) ::	ss_tlty_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	ocn_ice_test_interlayer

!------
!	fields to be sent to OCN
!------
	real (kind=4), allocatable, dimension(:,:) ::	aice_interlayer 		! used to be send to DAS too
	real (kind=4), allocatable, dimension(:,:) ::	strocnxT_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	strocnyT_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	fswthru_ai_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	fresh_ai_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	fhocn_ai_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	fsalt_ai_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	vice_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	vsno_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	ice_ocn_test_interlayer
!------
!	fields for DAS corrections
!------
	real (kind=4), allocatable, dimension(:,:) ::	aice_a_interlayer
	real (kind=4), allocatable, dimension(:,:) ::	das_coef

!------
!	CICE fields used in masking
!------
										!see manual p.71
	real (kind=4), allocatable, dimension(:,:) ::	hm_interlayer		!(kind=dble_kind) hm mask of CICE. T-cells. =1-ocean; =0-land
	real (kind=4), allocatable, dimension(:,:) ::	uvm_interlayer		!(kind=dble_kind) uvm mask of CICE. V-cells. =1-ocean; =0-land

!------
!	same arrays + mask. Use for saving at DG file
!------
	real (kind=4), allocatable, dimension(:,:) ::	sst_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	sss_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	frzm_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	uocn_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	vocn_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	tltx_interl_dg
  	real (kind=4), allocatable, dimension(:,:) ::	tlty_interl_dg
!	real (kind=4), allocatable, dimension(:,:) ::	ocn_ice_test_interl_dg

	real (kind=4), allocatable, dimension(:,:) ::	aice_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	cnxT_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	cnyT_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	fswt_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	fres_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	fhoc_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	fsal_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	vice_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	vsno_interl_dg
!	real (kind=4), allocatable, dimension(:,:) ::	ice_ocn_test_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	hm_interl_dg
	real (kind=4), allocatable, dimension(:,:) ::	uvm_interl_dg

	contains


!~~~~~~~
	subroutine ocn_interlayer_allocate
!~~~~~~~

!------
!	fields to be received from OCN (see some comments in allocation section)
!------
	allocate(	sst_interlayer		(idom))
	allocate(	sss_interlayer		(idom))
	allocate(	frzmlt_interlayer	(idom))	!Freeze/melt potential. Revise the division by (which?) dt
	allocate(	uocn_interlayer		(idom))
	allocate(	vocn_interlayer		(idom))
	allocate(	ss_tltx_interlayer	(idom))	!sea surface slope, m/m
	allocate(	ss_tlty_interlayer	(idom))
	allocate(	ocn_ice_test_interlayer	(idom))
	
!------
!	fields to be sent to OCN
!------
	allocate(	aice_interlayer		(idom))
	allocate(	aice_a_interlayer	(idom))	 !Analysis array (after assimilation) received from DAS
	allocate(	das_coef	(idom))	 		 ! array of corrected coef for aicen array 
	allocate(	strocnxT_interlayer	(idom))  !ice-ocn stress, kg/m/s2
	allocate(	strocnyT_interlayer	(idom))
	allocate(	fswthru_ai_interlayer	(idom))	!penetrating shortwave
	allocate(	fresh_ai_interlayer	(idom))	!freshwater flux, kg/m2/s (in ocn must be divided by density). Not including flux to ponds (?)
	allocate(	fhocn_ai_interlayer	(idom))	!net heatflux to ocn
	allocate(	fsalt_ai_interlayer	(idom))	!salt flux to ocn, kg/m2/s
	allocate(	vice_interlayer		(idom))
	allocate(	vsno_interlayer		(idom))
	allocate(	ice_ocn_test_interlayer	(idom))
!------
!	CICE fields used in masking
!------
	allocate(	hm_interlayer		(idom))
	allocate(	uvm_interlayer		(idom))
!------
!	same arrays + mask. Use for saving at DG file
!------
	allocate(	sst_interl_dg		(idom))
	allocate(	sss_interl_dg		(idom))
	allocate(	frzm_interl_dg		(idom))	!Freeze/melt potential. Revise the division by (which?) dt
	allocate(	uocn_interl_dg		(idom))
	allocate(	vocn_interl_dg		(idom))
	allocate(	tltx_interl_dg		(idom))	!sea surface slope, m/m
	allocate(	tlty_interl_dg		(idom))
!	allocate(	ocn_ice_test_interl_dg	(idom))

	allocate(	aice_interl_dg		(idom))
	allocate(	cnxT_interl_dg		(idom))  !ice-ocn stress, kg/m/s2
	allocate(	cnyT_interl_dg		(idom))
	allocate(	fswt_interl_dg		(idom))	!penetrating shortwave
	allocate(	fres_interl_dg		(idom))	!freshwater flux, kg/m2/s (in ocn must be divided by density). Not including flux to ponds (?)
	allocate(	fhoc_interl_dg		(idom))	!net heatflux to ocn
	allocate(	fsal_interl_dg		(idom))	!salt flux to ocn, kg/m2/s
	allocate(	vice_interl_dg		(idom))
	allocate(	vsno_interl_dg		(idom))
!	allocate(	ice_ocn_test_interl_dg	(idom))
	allocate(	hm_interl_dg		(idom))
	allocate(	uvm_interl_dg		(idom))

	sst_interl_dg		= 0.
	sss_interl_dg		= 0.
	frzm_interl_dg		= 0.	!Freeze/melt potential. Revise the division by (which?) dt
	uocn_interl_dg		= 0.
	vocn_interl_dg		= 0.
	tltx_interl_dg		= 0.	!sea surface slope, m/m
	tlty_interl_dg		= 0.
!	ocn_ice_test_interl_dg	= 0.

	aice_interl_dg		= 0.
	cnxT_interl_dg		= 0.	!ice-ocn stress, kg/m/s2
	cnyT_interl_dg		= 0.
	fswt_interl_dg		= 0.	!penetrating shortwave
	fres_interl_dg		= 0.	!freshwater flux, kg/m2/s (in ocn must be divided by density). Not including flux to ponds (?)
	fhoc_interl_dg		= 0.	!net heatflux to ocn
	fsal_interl_dg		= 0.	!salt flux to ocn, kg/m2/s
	vice_interl_dg		= 0.
	vsno_interl_dg		= 0.
!	ice_ocn_test_interl_dg	= 0.
	
	end subroutine ocn_interlayer_allocate


!~~~~~~~
	subroutine get_ocn_fields
!~~~~~~~

	use ice_flux, only: sst,sss,frzmlt,uocn,vocn,ss_tltx,ss_tlty
	call rearrange_CPL_to_CICE(sst_interlayer(idom),	sst)
	call rearrange_CPL_to_CICE(sss_interlayer(idom),	sss)
	call rearrange_CPL_to_CICE(frzmlt_interlayer(idom),	frzmlt)
	call rearrange_CPL_to_CICE(uocn_interlayer(idom),	uocn)
	call rearrange_CPL_to_CICE(vocn_interlayer(idom),	vocn)
	call rearrange_CPL_to_CICE(ss_tltx_interlayer(idom),	ss_tltx)
	call rearrange_CPL_to_CICE(ss_tlty_interlayer(idom),	ss_tlty)
	end subroutine get_ocn_fields



!~~~~~~~
	subroutine put_ocn_fields
!~~~~~~~

	use ice_flux, only: strocnxT,strocnyT,fswthru,fresh,fhocn,fsalt,Tair,fswthru_ai,fresh_ai,fhocn_ai,fsalt_ai
	use ice_state, only: aice,vice,vsno
	use ice_blocks, only: nx_block, ny_block
	use ice_grid, only: tmask,hm,uvm
	use ice_domain_size, only:nx_global,ny_global
	use o_par_module
	integer i,j

	call rearrange_CICE_to_CPL(aice,	aice_interlayer(idom) )
	call rearrange_CICE_to_CPL(strocnxT,	strocnxT_interlayer(idom) )
	call rearrange_CICE_to_CPL(strocnyT,	strocnyT_interlayer(idom) )
	if(ice_remapping_mode==0) then
		call rearrange_CICE_to_CPL(fswthru,	fswthru_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fresh,	fresh_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fhocn,	fhocn_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fsalt,	fsalt_ai_interlayer(idom) )
	else 
		call rearrange_CICE_to_CPL(fswthru_ai,	fswthru_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fresh_ai,	fresh_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fhocn_ai,	fhocn_ai_interlayer(idom) )
		call rearrange_CICE_to_CPL(fsalt_ai,	fsalt_ai_interlayer(idom) )
	end if
	call rearrange_CICE_to_CPL(vice,	vice_interlayer(idom) )
	call rearrange_CICE_to_CPL(vsno,	vsno_interlayer(idom) )
	
	call rearrange_CICE_to_CPL(hm,		hm_interlayer(idom) )
	call rearrange_CICE_to_CPL(uvm,		uvm_interlayer(idom) )

	strocnxT_interlayer=-strocnxT_interlayer	!making flux direction = from ice to ocn
	strocnyT_interlayer=-strocnyT_interlayer
	
	!In case of reduced ICE grid make zero line to avoid incorrect remapping to OCN grid
	if(cice_and_schrum .and. ice_cpl_south==1) then
		aice_interlayer(:,1)=0.
		strocnxT_interlayer(:,1)=0.
		strocnyT_interlayer(:,1)=0.
		fswthru_ai_interlayer(:,1)=0.
		fresh_ai_interlayer(:,1)=0.
		fhocn_ai_interlayer(:,1)=0.
		fsalt_ai_interlayer(:,1)=0.
		!print *,'Zero interlayer line',rank_local
	end if	
		
		
		!-------
!start==>	test mapping with mask
!-------
!	if(nx_global==40 .and. ny_global==60) then
!		do j = cmp % south, cmp % north
!		do i = cmp % west, cmp % east
!			if(hm_interlayer(i,j) == 1.) aice_interlayer(i,j) = real (i+j)
!		end do
!		end do
!	end if

!-------
!<==end	test mapping with mask 
!-------

	end subroutine put_ocn_fields

!!~~~~~~~
	subroutine das_to_cice_aicen_correction ()
!!~~~~~~~
!!--------------------------------------
!!	The following procedures used for field AICE correction 
!!	using results from DAS
!!	2019.09.30
!!	author: Kaurkin, Maxim
!!-------
	use ice_state, only: aice,aicen,aice_a, das_coef_cice
	use ice_blocks, only: nx_block, ny_block, nghost
	use ice_domain_size, only: ncat
	use ice_blocks, only: nx_block, ny_block
	integer i,j,n
	real*8 ice_das_sens_r 
	ice_das_sens_r  = dble(1.) /ice_das_sens
	if(ice_rank_local == 0) then
		write(*,*) , "DAS CICE AICE correction"
		write(*,*) , "aice_a_interlayer before",maxval(aice_a_interlayer(1:nx_block, 1:ny_block))
		write(*,*) , "aice_interlayer 	before",maxval(aice_interlayer  (1:nx_block, 1:ny_block))
		write(*,*) , "aice_dif 			before",maxval(aice_interlayer  (1:nx_block, 1:ny_block) - aice_a_interlayer(1:nx_block, 1:ny_block) )
		write(*,*) , "aicen before            ",maxval(aicen 			(1:nx_block, 1:ny_block,2,1))
	endif
	call rearrange_CPL_to_CICE(aice_a_interlayer(idom),	aice_a)
	
   do j = 1, ny_block
   do i = 1, nx_block
   	das_coef_cice(i,j,1) =   MERGE( 1d0 , (aice_a(i,j,1) /  aice(i,j,1)), aice(i,j,1) <  0.0001 )
   	if ( das_coef_cice(i,j,1) < ice_das_sens ) 		das_coef_cice(i,j,1) 	= ice_das_sens
   	if ( das_coef_cice(i,j,1) > ice_das_sens_r ) 	das_coef_cice(i,j,1) 	= ice_das_sens_r
   	do n =1, ncat
   		aicen(i,j,n,1) = MERGE( dble(1d0) , das_coef_cice(i,j,1) * aicen(i,j,n,1)  , das_coef_cice(i,j,1) * aicen(i,j,n,1) > dble(1))
   	end do

	end do
    end do
    
	call rearrange_CICE_to_CPL(das_coef_cice,	das_coef(idom) )
	if(ice_rank_local == 0) write(*,*) , "aicen after              ",maxval(aicen 			(1:nx_block, 1:ny_block,2,1))
end subroutine das_to_cice_aicen_correction



!~~~~~~~
	subroutine cice_mask_land_for_dg (land_value)
!~~~~~~~

!--------------------------------------
!	The following procedures used for masking arrays in DG files
!	2016.10.30
!	author: Ibrayev, Rashit
!-------

!-------
!	this subroutine is used for masking solution domain when writing to DG file
!	put 'land_value' to land points 
!	in case of writing data to DG file 'land_value = missval = -9e33' ==> that is convenient for plotting with 'ncview', 'PyNGL'
!-------
!	ATTENTION
!	masked array should be the same arrays as in subroutine cice_ini_reg_comp(this) for ACTION_SAVE_DG
!-------
	implicit none
	integer i,j
	real (kind=4) land_value

	sst_interl_dg  = sst_interlayer
	sss_interl_dg  = sss_interlayer
	frzm_interl_dg = frzmlt_interlayer
	uocn_interl_dg = uocn_interlayer
	vocn_interl_dg = vocn_interlayer
	tltx_interl_dg = ss_tltx_interlayer
	tlty_interl_dg = ss_tlty_interlayer
!	ocn_ice_test_interl_dg	= ocn_ice_test_interlayer

	aice_interl_dg = aice_interlayer
	cnxT_interl_dg = strocnxT_interlayer
	cnyT_interl_dg = strocnyT_interlayer
	fswt_interl_dg = fswthru_ai_interlayer
	fres_interl_dg = fresh_ai_interlayer
	fhoc_interl_dg = fhocn_ai_interlayer
	fsal_interl_dg = fsalt_ai_interlayer
	vice_interl_dg = vice_interlayer
	vsno_interl_dg = vsno_interlayer
!	ice_ocn_test_interl_dg	= ice_ocn_test_interlayer
	hm_interl_dg   = hm_interlayer


	do j = ice_cpl_south, ice_cpl_north
	do i = ice_cpl_west,  ice_cpl_east
	if(hm_interlayer(i,j) == 0.) then
	sst_interl_dg  (i,j)	= land_value
	sss_interl_dg  (i,j)	= land_value
	frzm_interl_dg (i,j)	= land_value
	uocn_interl_dg (i,j)	= land_value
	vocn_interl_dg (i,j)	= land_value
	tltx_interl_dg (i,j)	= land_value
	tlty_interl_dg (i,j)	= land_value
!	ocn_ice_test_interl_dg	= land_value

	aice_interl_dg (i,j)	= land_value
	cnxT_interl_dg (i,j)	= land_value
	cnyT_interl_dg (i,j)	= land_value
	fswt_interl_dg (i,j)	= land_value
	fres_interl_dg (i,j)	= land_value
	fhoc_interl_dg (i,j)	= land_value
	fsal_interl_dg (i,j)	= land_value
	vice_interl_dg (i,j)	= land_value
	vsno_interl_dg (i,j)	= land_value
!	ice_ocn_test_interl_dg	= land_value
	hm_interl_dg (i,j)	= land_value
	end if
	end do
	end do

	!if(rank_local == 0) write(*,*) 'ICE_CICE: DG files have been filled with land_value=',land_value

	return
	end subroutine cice_mask_land_for_dg

end module ocn_general_module
