!========================================================================================
! This is automatically generated adapter for CMF-system.
! WARNING: for correct work don't change any names generated by generate_comp.sh script.
!========================================================================================
#include "ice_2Ddomain"

module ice_cice_cpl_adapter_module

use ice_par_module
use component_module
use component_splitter_module
use halo_updater_module
use decomposition_module
use model_info_module
use experiment_info_module
use point_saver_module
use cice_module
use atm_general_module
use ocn_general_module
use ice_cice_driver_module

use ice_flux
use ice_state
use ice_blocks

implicit none

type, extends(Component) :: ice_cice_cpl_adapter
 contains
    procedure :: ini_reg_comp
    procedure :: ini_allocate
    procedure :: ini_reg_data    
    procedure :: ini_main
    procedure :: make_step
    procedure :: finalize 
end type

type(Decomposition) :: ice_decomp

 CONTAINS

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register your model in the system.
! Here you should (probably) read or define necessary information and 
! must call this % register_model()
!----------------------------------------------------------------------------------------

subroutine ini_reg_comp(this)
    class(ice_cice_cpl_adapter) :: this
 
    !integer :: ierr
    logical :: read_proc_decomp = .false.
    character(NAME_LEN) :: land_file        
           
    ! If we want read predefine processor decomposition, set file name to read from
    if (read_proc_decomp) then
        land_file = this % file_prefix()//"_IC.nc"
    else
        land_file = ""
    end if

    call ice_cice_driver_ini_params
    
    ! Build decomposition of grid (or read it from IC-file)
    ice_decomp = Decomposition(ice_i_size, ice_j_size, 5, "2D", CompSplitter % comm_local_size(), CompSplitter % rank_local(), &
                                proc_decomp_file = land_file, is_icycle = ice_i_cycle_flag == 1, is_bsc = ice_tripole_flag == 1)

    ! Call registration routine of CMF (it needs only size of grid and timestep in whole(!) seconds)
    call this % register_model(decomp = ice_decomp, timestep_sec = 24*60*60/ice_steps_in_day) 
    
    !Initiate ice_par_module
    call ice_par_init
             
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to allocate all your data, which will interact with system.
! This is necessary, because you ini_reg_data() use adresses of your arrays and 
! they should be allocated at time of registration.
! Here you should do all array allocations.
!----------------------------------------------------------------------------------------

subroutine ini_allocate(this)
    class(ice_cice_cpl_adapter) :: this

    call ice_par_after_sync(ice_decomp)
    call atm_interlayer_allocate
    !if (atm_model_name == "ATM_INM" ) call ini_allocate_atm_inm(this)
    call ocn_interlayer_allocate

end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program to register all data you want to interact with system.
! Here you should call this % register_array(), this % register_*_event() or
! this % raise_event() to get result immediately.
!----------------------------------------------------------------------------------------

subroutine ini_reg_data(this)
    class(ice_cice_cpl_adapter) :: this

    !call ini_reg_data_cpdg(this)
    if (ModelsInfo(CompSplitter % comp_id("ATM")) % model_name() == "ATM_NCAR" ) call ini_reg_data_atm_ncar(this)
    if (ModelsInfo(CompSplitter % comp_id("ATM")) % model_name() == "ATM_INTERIM" ) call ini_reg_data_atm_interim(this)
    if (ModelsInfo(CompSplitter % comp_id("ATM")) % model_name() == "ATM_INM" ) call ini_reg_data_atm_inm(this)
    if (ModelsInfo(CompSplitter % comp_id("ATM")) % model_name() == "ATM_GFS" ) call ini_reg_data_atm_gfs(this)
    call ini_reg_data_ocn(this)
    
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program before main time cycle.
! Here you should do all model preparation actions.
!----------------------------------------------------------------------------------------

subroutine ini_main(this)
    class(ice_cice_cpl_adapter) :: this
    
    call ice_cice_driver_ini_main
        
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at every step of time cycle. 
! Here you should represent one time step of your model with all physics and internal 
! exhanges (like halo MPI-updates).
!----------------------------------------------------------------------------------------

subroutine make_step(this)
    class(ice_cice_cpl_adapter) :: this
    
    ice_dt = this % model_time()
    
    call ice_cice_driver_main_step

end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Will be called by main program at the end.
! Here you should do all postprocesing actions and memory deallocations.
!----------------------------------------------------------------------------------------

subroutine finalize(this)        
    class(ice_cice_cpl_adapter) :: this        

end subroutine

!========================================================================================

subroutine ini_reg_data_cpdg(obj)
	class(Component), intent(in) :: obj
	
	call obj % register_array(arr_name = "aice_cp",    arr = aice_interlayer)
	if(ExpInfo % search_cp()) then
		call obj % register_periodic_event(arr_name = "aice_cp", act = "READ_CP")
	end if
	
	call obj % register_array(arr_name = "ssu_from_atm",   arr = ustress_atm_interlayer)
	call obj % register_array(arr_name = "ssv_from_atm",   arr = vstress_atm_interlayer)
	call obj % register_array(arr_name = "tr_from_atm",    arr = flw_interlayer)
	call obj % register_array(arr_name = "sr_from_atm",    arr = sw_interlayer)	
	call obj % register_array(arr_name = "sh_from_atm",    arr = sns_atm_interlayer)
	call obj % register_array(arr_name = "lh_from_atm",    arr = ltn_atm_interlayer)
	call obj % register_array(arr_name = "p_from_atm",     arr = fsnow_interlayer)
	call obj % register_array(arr_name = "tice_to_atm",    arr = tice_atm_interlayer)
	
	call obj % register_periodic_event(arr_name = "ssu_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "ssv_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "tr_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "sr_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "sh_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "lh_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "p_from_atm", act = "SAVE_DG", dm = ice_diag_period)
	call obj % register_periodic_event(arr_name = "tice_to_atm", act = "SAVE_DG", dm = ice_diag_period)
	
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_ncar(obj)
    class(Component), intent(in) :: obj

    call obj % register_array(arr_name = "T_10MOD_AI",    arr = Tair_interlayer) 
    call obj % register_array(arr_name = "Q_10MOD_AI",    arr = Qa_interlayer)
    call obj % register_array(arr_name = "U_10MOD_AI",    arr = uatm_interlayer) 
    call obj % register_array(arr_name = "V_10MOD_AI",    arr = vatm_interlayer) 
    call obj % register_array(arr_name = "LWDNMOD_AI",    arr = flw_interlayer)
    call obj % register_array(arr_name = "SWDNMOD_AI",    arr = sw_interlayer)
    call obj % register_array(arr_name = "RAIN_AI",         arr = frain_interlayer)
    call obj % register_array(arr_name = "SNOW_AI",         arr = fsnow_interlayer)
        
    call obj % register_periodic_event(arr_name = "T_10MOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)   !6h is the period of the CORE data 
    call obj % register_periodic_event(arr_name = "Q_10MOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)   !60*60*3 seconds is the time of the first record in CNYFv2 and IAFv2 
    call obj % register_periodic_event(arr_name = "U_10MOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
    call obj % register_periodic_event(arr_name = "V_10MOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
    call obj % register_periodic_event(arr_name = "LWDNMOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
    call obj % register_periodic_event(arr_name = "SWDNMOD_AI",   act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)    
    call obj % register_periodic_event(arr_name = "RAIN_AI",       act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)
    call obj % register_periodic_event(arr_name = "SNOW_AI",       act = "RECV_MP", src = "ATM", dh = 6, delay = 60*60*3)       
    
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_interim(obj)
    class(Component), intent(in) :: obj

    call obj % register_array(arr_name = "t2m_AI",    arr = Tair_interlayer) 
    call obj % register_array(arr_name = "d2m_AI",    arr = Qa_interlayer)
    call obj % register_array(arr_name = "u10_AI",    arr = uatm_interlayer) 
    call obj % register_array(arr_name = "v10_AI",    arr = vatm_interlayer) 
    call obj % register_array(arr_name = "strd_AI",    arr = flw_interlayer)
    call obj % register_array(arr_name = "ssrd_AI",    arr = sw_interlayer)
    call obj % register_array(arr_name = "tp_AI",      arr = fsnow_interlayer)
        
    call obj % register_periodic_event(arr_name = "t2m_AI",   act = "RECV_MP", src = "ATM", dh = 6)
    call obj % register_periodic_event(arr_name = "d2m_AI",   act = "RECV_MP", src = "ATM", dh = 6)
    call obj % register_periodic_event(arr_name = "u10_AI",   act = "RECV_MP", src = "ATM", dh = 6)    
    call obj % register_periodic_event(arr_name = "v10_AI",   act = "RECV_MP", src = "ATM", dh = 6)    
    call obj % register_periodic_event(arr_name = "strd_AI",   act = "RECV_MP", src = "ATM", dh = 6)    
    call obj % register_periodic_event(arr_name = "ssrd_AI",   act = "RECV_MP", src = "ATM", dh = 6)    
    call obj % register_periodic_event(arr_name = "tp_AI",     act = "RECV_MP", src = "ATM", dh = 6)
    
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_gfs(obj)
    class(Component), intent(in) :: obj

    call obj % register_array(arr_name = "t2m_AI",    arr = Tair_interlayer) 
    call obj % register_array(arr_name = "q2m_AI",    arr = Qa_interlayer)
    call obj % register_array(arr_name = "u10_AI",    arr = uatm_interlayer) 
    call obj % register_array(arr_name = "v10_AI",    arr = vatm_interlayer) 
    call obj % register_array(arr_name = "trdi_AI",   arr = flw_interlayer)
    call obj % register_array(arr_name = "srdi_AI",   arr = sw_interlayer)
    call obj % register_array(arr_name = "tprr_AI",   arr = frain_interlayer)
    call obj % register_array(arr_name = "tprs_AI",   arr = fsnow_interlayer)    
        
    call obj % register_periodic_event(arr_name = "t2m_AI",   act = "RECV_MP", src = "ATM", dh = 3)
    call obj % register_periodic_event(arr_name = "q2m_AI",   act = "RECV_MP", src = "ATM", dh = 3)
    call obj % register_periodic_event(arr_name = "u10_AI",   act = "RECV_MP", src = "ATM", dh = 3)    
    call obj % register_periodic_event(arr_name = "v10_AI",   act = "RECV_MP", src = "ATM", dh = 3)    
    call obj % register_periodic_event(arr_name = "trdi_AI",  act = "RECV_MP", src = "ATM", dh = 3)    
    call obj % register_periodic_event(arr_name = "srdi_AI",  act = "RECV_MP", src = "ATM", dh = 3)    
    call obj % register_periodic_event(arr_name = "tprr_AI",  act = "RECV_MP", src = "ATM", dh = 3)
    call obj % register_periodic_event(arr_name = "tprs_AI",  act = "RECV_MP", src = "ATM", dh = 3)    
    
end subroutine

!========================================================================================

subroutine ini_reg_data_atm_inm(obj)
    class(Component), intent(in) :: obj
    
    !call obj % register_array(arr_name = "v_atm_AI"    ,   arr = vatm_interlayer(idom)       )
    !call obj % register_array(arr_name = "u_atm_AI"    ,   arr = uatm_interlayer(idom)       )
    !call obj % register_array(arr_name = "wind_atm_AI" ,   arr = wind_interlayer(idom)       )
    !call obj % register_array(arr_name = "hum_atm_AI"  ,   arr = Qa_interlayer(idom)         )
    !call obj % register_array(arr_name = "dens_atm_AI" ,   arr = rhoa_interlayer(idom)       )
    !call obj % register_array(arr_name = "ptemp_atm_AI",   arr = potT_interlayer(idom)       )
    !call obj % register_array(arr_name = "temp_atm_AI" ,   arr = Tair_interlayer(idom)       )
    !call obj % register_array(arr_name = "sr_atm_AI"   ,   arr = sw_interlayer(idom)         )
    !call obj % register_array(arr_name = "tr_atm_AI"   ,   arr = flw_interlayer(idom)        )
    !call obj % register_array(arr_name = "rain_atm_AI" ,   arr = frain_interlayer(idom)      )
    !call obj % register_array(arr_name = "snow_atm_AI" ,   arr = fsnow_interlayer(idom)      )
        
    !call obj % register_periodic_event(arr_name = "v_atm_AI"    ,   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "u_atm_AI"    ,   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "wind_atm_AI" ,   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "hum_atm_AI"  ,   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "dens_atm_AI" ,   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "ptemp_atm_AI",   act = "RECV_MP", src = "ATM", dh = 6)    
    !call obj % register_periodic_event(arr_name = "temp_atm_AI" ,   act = "RECV_MP", src = "ATM", dh = 6)
    !call obj % register_periodic_event(arr_name = "sr_atm_AI"   ,   act = "RECV_MP", src = "ATM", dh = 6)
    !call obj % register_periodic_event(arr_name = "tr_atm_AI"   ,   act = "RECV_MP", src = "ATM", dh = 6) 
    !call obj % register_periodic_event(arr_name = "rain_atm_AI" ,   act = "RECV_MP", src = "ATM", dh = 6) 
    !call obj % register_periodic_event(arr_name = "snow_atm_AI" ,   act = "RECV_MP", src = "ATM", dh = 6)       
    
end subroutine
 
!========================================================================================

subroutine ini_reg_data_ocn(obj)
    class(Component), intent(in) :: obj
	
    call obj % register_array(arr_name = "sst_OI",         arr = sst_interlayer(idom)) 
    call obj % register_array(arr_name = "sss_OI",         arr = sss_interlayer(idom))
    call obj % register_array(arr_name = "frzmlt_OI",      arr = frzmlt_interlayer(idom))
    call obj % register_array(arr_name = "uocn_OI",        arr = uocn_interlayer(idom))
    call obj % register_array(arr_name = "vocn_OI",        arr = vocn_interlayer(idom))
    call obj % register_array(arr_name = "ss_tltx_OI",     arr = ss_tltx_interlayer(idom))
    call obj % register_array(arr_name = "ss_tlty_OI",     arr = ss_tlty_interlayer(idom))
    !call obj % register_array(arr_name = "test_OI",       arr = ocn_ice_test_interlayer(idom))

    call obj % register_array(arr_name = "aice_IO",        arr = aice_interlayer(idom)) 
    call obj % register_array(arr_name = "strocnxt_IO",    arr = strocnxT_interlayer(idom)) 
    call obj % register_array(arr_name = "strocnyt_IO",    arr = strocnyT_interlayer(idom)) 
    call obj % register_array(arr_name = "fswthru_IO",     arr = fswthru_ai_interlayer(idom)) 
    call obj % register_array(arr_name = "fresh_IO",       arr = fresh_ai_interlayer(idom)) 
    call obj % register_array(arr_name = "fhocn_IO",       arr = fhocn_ai_interlayer(idom)) 
    call obj % register_array(arr_name = "fsalt_IO",       arr = fsalt_ai_interlayer(idom))
    call obj % register_array(arr_name = "vice_IO",        arr = vice_interlayer(idom))
    call obj % register_array(arr_name = "tice_IO",        arr = tice_atm_interlayer(idom))
    !call obj % register_array(arr_name = "test_IO",       arr = ice_ocn_test_interlayer(idom))
        
    call obj % register_periodic_event(arr_name = "sst_OI",      act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "sss_OI",      act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "frzmlt_OI",   act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "uocn_OI",     act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "vocn_OI",     act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "ss_tltx_OI",  act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)    
    call obj % register_periodic_event(arr_name = "ss_tlty_OI",  act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)
    !call obj % register_periodic_event(arr_name = "test_OI",     act = "RECV_MP", src = "OCN", dm = ocn_ice_mapping)

    call obj % register_periodic_event(arr_name = "aice_IO",     act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "strocnxt_IO", act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "strocnyt_IO", act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "fswthru_IO",  act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "fresh_IO",    act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "fhocn_IO",    act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)    
    call obj % register_periodic_event(arr_name = "fsalt_IO",    act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)
    call obj % register_periodic_event(arr_name = "vice_IO",     act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)       
    call obj % register_periodic_event(arr_name = "tice_IO",     act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)       
    !call obj % register_periodic_event(arr_name = "test_IO",    act = "SEND_MP", dst = "OCN", dm = ice_ocn_mapping)       

    !DAS registranion in ICE
    if ( ice_das_on ) then    
        call obj % register_array(arr_name = "aice",                arr = aice_interlayer(idom)) 
        call obj % register_array(arr_name = "aice_dif",            arr = aice_a_interlayer(idom)) 
        call obj % register_periodic_event( arr_name = "aice",      act = "SEND_DAS" ,  dh = 24, delay = 60*10)
        call obj % register_periodic_event( arr_name = "aice_dif",  act = "RECV_ENKF" , dh = 24, delay = 60*10)
    endif

    !-----Diagnostic arrays-----
    !call obj % register_array(arr_name = "sst_OI_dg",         arr = sst_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "sss_OI_dg",         arr = sss_interl_dg	 (idom))
    !call obj % register_array(arr_name = "frzmlt_OI_dg",      arr = frzm_interl_dg	 (idom))
    !call obj % register_array(arr_name = "uocn_OI_dg",        arr = uocn_interl_dg	 (idom))
    !call obj % register_array(arr_name = "vocn_OI_dg",        arr = vocn_interl_dg	 (idom))
    !call obj % register_array(arr_name = "ss_tltx_OI_dg",     arr = tltx_interl_dg 	 (idom))
    !call obj % register_array(arr_name = "ss_tlty_OI_dg",     arr = tlty_interl_dg 	 (idom))
    
    !call obj % register_array(arr_name = "aice_IO_dg",        arr = aice_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "strxt_IO_dg",       arr = cnxt_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "stryt_IO_dg",       arr = cnyt_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "fswthru_IO_dg",     arr = fswt_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "fresh_IO_dg",       arr = fres_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "fhocn_IO_dg",       arr = fhoc_interl_dg	 (idom)) 
    !call obj % register_array(arr_name = "fsalt_IO_dg",       arr = fsal_interl_dg	 (idom))
    !call obj % register_array(arr_name = "vice_IO_dg",        arr = vice_interl_dg	 (idom))
    !call obj % register_array(arr_name = "vsno_IO_dg",        arr = vsno_interl_dg	 (idom))
    !call obj % register_array(arr_name = "hm_dg",             arr = hm_interl_dg	 (idom))
    !call obj % register_array(arr_name = "uvm_dg",            arr = uvm_interl_dg	 (idom))
    
    !call obj % register_periodic_event(arr_name = "sst_OI_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "sss_OI_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "frzmlt_OI_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "uocn_OI_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "vocn_OI_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "ss_tltx_OI_dg",  	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "ss_tlty_OI_dg",  	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "aice_IO_dg",      	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "strxt_IO_dg", 	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "stryt_IO_dg", 	act = "SAVE_DG", dm = ice_diag_period)     
    !call obj % register_periodic_event(arr_name = "fswthru_IO_dg",  	act = "SAVE_DG", dm = ice_diag_period)         
    !call obj % register_periodic_event(arr_name = "fresh_IO_dg",      	act = "SAVE_DG", dm = ice_diag_period)         
    !call obj % register_periodic_event(arr_name = "fhocn_IO_dg",      	act = "SAVE_DG", dm = ice_diag_period)   
    !call obj % register_periodic_event(arr_name = "fsalt_IO_dg",     	act = "SAVE_DG", dm = ice_diag_period)       
    !call obj % register_periodic_event(arr_name = "vice_IO_dg",     	act = "SAVE_DG", dm = ice_diag_period)
    !call obj % register_periodic_event(arr_name = "vsno_IO_dg",     	act = "SAVE_DG", dm = ice_diag_period)
    !call obj % register_periodic_event(arr_name = "hm_dg",     		act = "SAVE_DG", dm = ice_diag_period)
    !call obj % register_periodic_event(arr_name = "uvm_dg",     	act = "SAVE_DG", dm = ice_diag_period)
    !call obj % register_periodic_event(arr_name = "test_IO",     	act = "SAVE_DG", dm = ice_diag_period)
    !call obj % register_periodic_event(arr_name = "test_OI",     	act = "SAVE_DG", dm = ice_diag_period)

    if ( ice_das_on ) then
        call obj % register_array(arr_name = "aice_A_dg",   arr =aice_a_interlayer(idom)) 
        call obj % register_array(arr_name = "das_coef",    arr = das_coef(idom))
        call obj % register_periodic_event(arr_name = "aice_A_dg",          act = "SAVE_DG", dm = ice_diag_period, delay = 60*10) 
        call obj % register_periodic_event(arr_name = "das_coef",           act = "SAVE_DG", dm = ice_diag_period, delay = 60*10) 
    endif

end subroutine

end module
    
