#include "ice_2Ddomain" 	
module cpl_ice_cice_module

	use cice_module
	use comp_module
	use shared_module
	use atm_general_module
	use ocn_general_module
	use non_ga_ice_array_module
	use ice_cice_driver_module
	use ice_par_module
	implicit none

!==============================================================================
! Define type 'cpl_atm', which extends class component and define is's 
! abstract methods
!==============================================================================

	TYPE, EXTENDS(comp) :: cpl_ice_cice
	
	CONTAINS
		PROCEDURE :: ini_params => cice_ini_params
		PROCEDURE :: ini_reg_comp => cice_ini_reg_comp
		PROCEDURE :: ini_reg_data => cice_ini_reg_data
		PROCEDURE :: ini_allocate => cice_ini_allocate
		PROCEDURE :: ini_main => cice_ini_main
		PROCEDURE :: main_step => cice_main_step
	END TYPE cpl_ice_cice
		
	CONTAINS
	
!================================================================================================================================================================
subroutine cice_ini_params(this)
	
	class(cpl_ice_cice) :: this
	call ice_cice_driver_ini_params
	
end subroutine cice_ini_params
!================================================================================================================================================================

subroutine cice_ini_reg_comp(this)
	class(cpl_ice_cice) :: this
	call this % reg_model(			&
		reg_np = run_np(COMP_ICE),	&
		reg_i_size = ice_i_size,	&
		reg_j_size = ice_j_size,        &
		reg_k_size = 5,                 &
		reg_m_size = 1,                 &
		reg_decomp_type_2D = .TRUE.,    &
		reg_i_cycle = ice_i_cycle_flag, &
		reg_bsc_grid =ice_tripole_flag,	&
		reg_time_steps_day = ice_steps_in_day, &
		reg_time_save_it = 0,           &
		reg_read_cp = ice_read_cp_flag, &
		reg_halo_stencil = 0,           &
		reg_halo_wide = 0)
		
	!Initiate ice_par_module
	call ice_par_init
end subroutine cice_ini_reg_comp

!================================================================================================================================================================
subroutine cice_ini_reg_data(this)
		
	class(cpl_ice_cice) :: this

	!Reading compactness from some spinup data
	call this % reg_array(recv_from=COUPLER,action=ACTION_READ_CP, DIM2_arr=aice_interlayer( idom ),name='aice_cp_save')
!------
!	ACTION_MAPPING. recv_from=COMP_OCN
!------
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= sst_interlayer		(idom),period=ocn_ice_mapping,name='sst')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= sss_interlayer		(idom),period=ocn_ice_mapping,name='sss')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= frzmlt_interlayer	(idom),period=ocn_ice_mapping,name='frzmlt')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= uocn_interlayer	(idom),period=ocn_ice_mapping,name='uocn')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= vocn_interlayer	(idom),period=ocn_ice_mapping,name='vocn')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= ss_tltx_interlayer	(idom),period=ocn_ice_mapping,name='ss_tltx')
	call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= ss_tlty_interlayer	(idom),period=ocn_ice_mapping,name='ss_tlty')
	!call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= ocn_ice_test_interlayer(idom),period=ocn_ice_mapping,name='ocn_ice_test')
!------
!	ACTION_MAPPING. send_to=COMP_OCN
!	NOTE: strocnxT, strocnx - force imposed on ice by ocn
!	NOTE: strocnxT is per unit of ice, strocnx is per unit of grid
!------
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=aice_interlayer	(idom),period=ice_ocn_mapping, name='aice')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=strocnxt_interlayer	(idom),period=ice_ocn_mapping, name='strocnxt')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=strocnyt_interlayer	(idom),period=ice_ocn_mapping, name='strocnyt')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=fswthru_ai_interlayer	(idom),period=ice_ocn_mapping, name='fswthru_ai')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=fresh_ai_interlayer	(idom),period=ice_ocn_mapping, name='fresh_ai')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=fhocn_ai_interlayer	(idom),period=ice_ocn_mapping, name='fhocn_ai')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=fsalt_ai_interlayer	(idom),period=ice_ocn_mapping, name='fsalt_ai')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=vice_interlayer	(idom),period=ice_ocn_mapping, name='vice')
	!call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=ice_ocn_test_interlayer(idom),period=ice_ocn_mapping, name='ice_ocn_test')
	call this % reg_array(send_to=COMP_OCN,action=ACTION_MAPPING,map_type=MAP_SRC_TYPE,DIM2_arr=tice_atm_interlayer	(idom),period=ice_ocn_mapping, name='tice')
	
!------
!	ACTION_MAPPING. recv_from = COMP_ATM
!------
	if(TRIM(cmp % names(COMP_ATM)) == 'ATM_NCAR') then
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Tair_interlayer	(idom),period=atm_ice_mapping,name ='T_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Qa_interlayer		(idom),period=atm_ice_mapping,name ='Q_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= uatm_interlayer	(idom),period=atm_ice_mapping,name ='U_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= vatm_interlayer	(idom),period=atm_ice_mapping,name ='V_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= flw_interlayer	(idom),period=atm_ice_mapping,name ='LWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= sw_interlayer		(idom),period=atm_ice_mapping,name ='SWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= frain_interlayer	(idom),period=atm_ice_mapping,name ='RAIN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= fsnow_interlayer	(idom),period=atm_ice_mapping,name ='SNOW')
	else if(TRIM(cmp % names(COMP_ATM)) == 'ATM_INM') then
		!---these fluxes are transferred from ATM_INM through OCN to ICE_CICE----
		!---temporarily all atm prec is treated by ICE_CICE as snow---  
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=ustress_atm_interlayer	(idom),period=ocn_ice_mapping,name='ssu_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=vstress_atm_interlayer	(idom),period=ocn_ice_mapping,name='ssv_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=flw_interlayer		(idom),period=ocn_ice_mapping,name='tr_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=sw_interlayer		(idom),period=ocn_ice_mapping,name='sr_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=sns_atm_interlayer	(idom),period=ocn_ice_mapping,name='sh_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=ltn_atm_interlayer	(idom),period=ocn_ice_mapping,name='lh_a2i')
		!call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=evap_atm_interlayer	(idom),period=ocn_ice_mapping,name='e_a2i')
		call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=fsnow_interlayer	(idom),period=ocn_ice_mapping, name='p_a2i' )
		!call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=Tair_interlayer	(idom),period=ocn_ice_mapping, name ='t2_a2i' )
		!call this % reg_array(recv_from=COMP_OCN,action=ACTION_MAPPING, DIM2_arr=from_atm1(odom),period=ocn_ice_mapping,name='tester_back')
		
		!additional parameters from atm-inm to ice
		call this % reg_array( recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= uatm_interlayer	(idom),period=ocn_ice_mapping,name ='u10m_a2i')
		call this % reg_array( recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= vatm_interlayer	(idom),period=ocn_ice_mapping,name ='v10m_a2i')
		call this % reg_array( recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= Tair_interlayer	(idom),period=ocn_ice_mapping,name ='t2m_a2i')
		call this % reg_array( recv_from=COMP_OCN,action=ACTION_MAPPING,DIM2_arr= Qa_interlayer		(idom),period=ocn_ice_mapping,name ='sq2m_a2i')
		
	else if(TRIM(cmp % names(COMP_ATM)) == 'ATM_INTERIM') then
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Tair_interlayer	(idom),period=atm_ice_mapping,name ='t2m')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Qa_interlayer		(idom),period=atm_ice_mapping,name ='d2m')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= uatm_interlayer	(idom),period=atm_ice_mapping,name ='u10')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= vatm_interlayer	(idom),period=atm_ice_mapping,name ='v10')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= flw_interlayer	(idom),period=atm_ice_mapping,name ='strd')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= sw_interlayer		(idom),period=atm_ice_mapping,name ='ssrd')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= fsnow_interlayer	(idom),period=atm_ice_mapping,name ='tp')
	else if(TRIM(cmp % names(COMP_ATM)) == 'ATM_GFS') then
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Tair_interlayer	(idom),period=3*60,name ='T')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Qa_interlayer		(idom),period=3*60,name ='Q')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= uatm_interlayer	(idom),period=3*60,name ='U')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= vatm_interlayer	(idom),period=3*60,name ='V')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= flw_interlayer	(idom),period=3*60,name ='LWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= sw_interlayer		(idom),period=3*60,name ='SWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= frain_interlayer	(idom),period=3*60,name ='RAIN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= fsnow_interlayer	(idom),period=3*60,name ='SNOW')
	else if(TRIM(cmp % names(COMP_ATM)) == 'ATM_INMCM4') then
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Tair_interlayer	(idom),period=24*60,name ='T_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= Qa_interlayer		(idom),period=24*60,name ='Q_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= uatm_interlayer	(idom),period=24*60,name ='U_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= vatm_interlayer	(idom),period=24*60,name ='V_10_MOD')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= flw_interlayer	(idom),period=24*60,name ='LWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= sw_interlayer		(idom),period=24*60,name ='SWDN')
		call this % reg_array( recv_from=COMP_ATM,action=ACTION_MAPPING,DIM2_arr= fsnow_interlayer	(idom),period=24*60,name ='SNOW')
	
	else
		stop 'Atmosphere specification error in ICE_CICE'
	end if

!------
!	SAVE_DG testers
!------
	!call this % reg_array(send_to=coupler,action=action_save_dg,dim2_arr=tair_interlayer	 (idom),period=ice_diag_period,name='T_10_MOD' )

!------
!	ACTION_SAVE_DG. save arrays with masking
!------
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= ustress_atm_interlayer	 (idom),period=ice_diag_period,name='ssu_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= vstress_atm_interlayer	 (idom),period=ice_diag_period,name='ssv_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= flw_interlayer	 (idom),period=ice_diag_period,name='tr_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= sw_interlayer	 (idom),period=ice_diag_period,name='sr_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= sns_atm_interlayer	 (idom),period=ice_diag_period,name='sh_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= ltn_atm_interlayer	 (idom),period=ice_diag_period,name='lh_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= fsnow_interlayer	 (idom),period=ice_diag_period,name='prec_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= tice_atm_interlayer	 (idom),period=ice_diag_period,name='tice_to_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= tair_interlayer	 (idom),period=ice_diag_period,name='tair_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= qa_interlayer	 (idom),period=ice_diag_period,name='humid_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= uatm_interlayer	 (idom),period=ice_diag_period,name='uwind_from_atm')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= vatm_interlayer	 (idom),period=ice_diag_period,name='vwind_from_atm')
	
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= sst_interl_dg	 (idom),period=ice_diag_period,name='SST')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= sss_interl_dg	 (idom),period=ice_diag_period,name='SSS')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= frzm_interl_dg	 (idom),period=ice_diag_period,name='frzmlt')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= uocn_interl_dg	 (idom),period=ice_diag_period,name='Uocn')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= vocn_interl_dg	 (idom),period=ice_diag_period,name='Vocn')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= tltx_interl_dg 	 (idom),period=ice_diag_period,name='ss_tltx')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= tlty_interl_dg 	 (idom),period=ice_diag_period,name='ss_tlty')
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr=ocn_ice_test_interlayer(idom),period=ice_diag_period,name='ocn_ice_test')
	
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr =aice_interl_dg	 (idom),period=ice_diag_period,name='Aice')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr =cnxt_interl_dg	 (idom),period=ice_diag_period,name='strocnxt')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr =cnyt_interl_dg 	 (idom),period=ice_diag_period,name='strocnyt')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= fswt_interl_dg	 (idom),period=ice_diag_period,name='penetr_ssr')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= fres_interl_dg	 (idom),period=ice_diag_period,name='Fresh')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= fhoc_interl_dg	 (idom),period=ice_diag_period,name='Fhocn')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr= fsal_interl_dg	 (idom),period=ice_diag_period,name='Fsalt')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr =vice_interl_dg	 (idom),period=ice_diag_period,name='Vice')
	call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr =vsno_interl_dg	 (idom),period=ice_diag_period,name='vsno')
	!call this % reg_array(send_to=coupler,action=action_save_dg,dim2_arr= hm_interl_dg	 (idom),period=ice_diag_period,name='hm' )
	!call this % reg_array(send_to=coupler,action=action_save_dg,dim2_arr= uvm_interlayer	 (idom),period=ice_diag_period,name='uvm' )
	!call this % reg_array(send_to=COUPLER,action=ACTION_SAVE_DG,DIM2_arr=ice_ocn_test_interlayer(idom),period=ice_diag_period,name='ice_ocn_test')
	
	end subroutine cice_ini_reg_data	
!====================================================================================================================================================================

subroutine cice_ini_allocate(this)
	
	class(cpl_ice_cice) :: this
	!allocate ( test_array ( cmp%west:cmp%east, cmp%south:cmp%north ) )
	call atm_interlayer_allocate
	call ocn_interlayer_allocate

end subroutine cice_ini_allocate

!===================================================================================================================================================================

subroutine cice_ini_main(this)
	
	class(cpl_ice_cice) :: this
	call ice_cice_driver_ini_main

end subroutine cice_ini_main

!==================================================================================================================================================================				

subroutine cice_main_step(this)
	
	class(cpl_ice_cice) :: this
	call ice_cice_driver_main_step

end subroutine cice_main_step
end module cpl_ice_cice_module
	
