
!--------------------------------------
!		
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!	Shirshov Institute of Oceanology, Russian Academy of Sciences (IO RAS) 
!		Ocean General Circulation Model	
!			INMIO
!
!	Institute of Numerical Mathematics, Russian Academy of Sciences (INM RAS)
!		Compact Modelling Framework	
!			CMF2.0, 3.0
!
! authors: 
!	2013.08.07 / Kalmykov, Vladimir, IO RAS, Russia, vvk88@mail.ru
!	2015.07 / Ushakov, Konstantin, IO RAS, Russia, ushakovkv@mail.ru
!
! reference:
!	K. V. Ushakov, R. A. Ibraev, and V. V. Kalmykov, 2015.
!		Simulation of the world ocean climate with a massively parallel numerical model, 
!		Izv., Atmos. Ocean. Phys. 51 (4), 362–380.
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Vychisl. Metody Programm., 2013, v. 14, iss. 3, 88-95. 
!	Kalmykov V.V., Ibrayev R.A., 2013. 
!		A framework for the ocean-ice-atmosphere-land coupled modeling on massively-parallel architectures. 
!		Certificate of state registration of the computer progam number 2013619320. 
!		Date of state registration in the Registry of the computer programs October 1, 2013
!	R. A. Ibraev, K. V. Ushakov, and R. N. Khabeev, 2012.
!		Eddy-resolving 1/10° model of the World Ocean,
!		Izv., Atmos. Ocean. Phys. 48 (1), 37–46.
!	Ibrayev R.A., 2001: Model of enclosed and semi-enclosed sea hydrodynamics. 
!		Russ. J. Numer. Anal. Math. Modelling, 2001, vol. 16, No 4, p. 291-304.
!--------------------------------------

module ice_par_module

#if CMF_VERSION == 2
    use shared_module
    use comp_module
#elif CMF_VERSION == 3
    use component_module
#endif
    
    implicit none
    
    character(NAME_LEN) :: atm_model_name
    character(NAME_LEN) :: lnd_model_name
    character(NAME_LEN) :: ice_model_name
    integer ice_cpl_east,ice_cpl_west,ice_cpl_south,ice_cpl_north
    integer :: ice_rank_local,ice_comm_local
    
    !input namelist parameters
	integer ice_i_size, ice_j_size, ice_steps_in_day, ice_np
	logical ice_read_cp_flag, ice_tripole_flag, ice_i_cycle_flag
    !input namelist parameters DAS
    logical ice_das_on  !ON / OFF data assimilation over DAS
    real*8 ice_das_sens ! 0.25 how to sharply change the value aicen array over assimilation  
	character(len=50) ice_resolution_mode
	real(kind=4),dimension (:,:), allocatable :: test_array
	

#if CMF_VERSION == 2
#elif CMF_VERSION == 3
    integer :: time_l, time_l_in_run
    type(DateTime) :: ice_dt
    character(NAME_LEN) :: run_experiment='aaa'
    integer :: time_l_save_cp, time_l_save_dg
    integer time_year,time_month,time_day,time_hour,time_min,time_sec
#endif

 contains

#if CMF_VERSION == 2

subroutine ice_par_init()
    atm_model_name = cmp % names(COMP_ATM)
    lnd_model_name = cmp % names(COMP_LND)
    ice_model_name = cmp % names(COMP_ICE)

    ice_cpl_east = cmp % east
    ice_cpl_west = cmp % west
    ice_cpl_south = cmp % south
    ice_cpl_north = cmp % north
    
    ice_rank_local = rank_local
    ice_comm_local = comm_local
end subroutine

#elif CMF_VERSION == 3

subroutine ice_par_init
            
    run_experiment=trim(ExpInfo % exp_name())
    ice_rank_local = CompSplitter % rank_local()
    ice_comm_local = CompSplitter % comm_local()
    
end subroutine

subroutine ice_par_after_sync(obj)
    type(Decomposition) :: obj
    
    atm_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("ATM")) % model_name())
    lnd_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("LND")) % model_name())
    ice_model_name = TRIM(ModelsInfo(CompSplitter % comp_id("ICE")) % model_name())
    
    ice_cpl_east = obj % east()
    ice_cpl_west = obj % west()
    ice_cpl_south = obj % south()
    ice_cpl_north = obj % north()

end subroutine

subroutine shared_timer
	time_year = ice_dt % year
	time_month = ice_dt % month
	time_day = ice_dt % day
	time_hour = ice_dt % hour
	time_min = ice_dt % minute
	time_sec = ice_dt % second
end subroutine
	
#endif
 
!----------------------------------------------------------------------------------------
! Decription: Print message and stop program
! Parameters:
!    msg      - string message
!    mode     - 'LCL' - all ocean cores, 'IND' - only one core
!----------------------------------------------------------------------------------------

subroutine ice_error(msg, mode)
    character(*), intent(in) :: msg
    character(3), intent(in) :: mode

#if CMF_VERSION == 2
    call shared_error(msg)
#elif CMF_VERSION == 3
    call CompSplitter % sync_error(msg, mode)
#endif
    
end subroutine    

end module

