!~~~~~~~
	module off_ice_cice_module
!~~~~~~~

!-------
!	Generate grid for CICE model
!	Generating sea-land mask for CICE
!	author Bayburin, Ruslan 2015
!	2016.05.10	editted by Ibrayev, Rashit
!-------
	use off_module
	use shared_utils_module
	use cpl_ice_cice_module, only :cpl_ice_cice, ice_tripole_flag, ice_resolution_mode

	implicit none
	
	integer il, jl		! Sizes of grid

	real(kind=8), parameter :: rad_to_deg = 180d0 / 3.1415926535d0
	real(kind=8), parameter :: deg_to_rad = 3.1415926535d0 / 180d0
	real(kind=8), parameter :: radius = 0.64d9, m_to_cm = 1d2

	integer :: additional_ny= 0	!0 - to make full grid, <0 - to make reduced grid (eg. for Arctic only)
					!-54 for global20, -480 for WOM025_ice, -240 for WOM05_ice
!===============================================================================
! Define type 'off_ice_cice', which extends class gridconstructor and define
! is's abstract methods
!===============================================================================
	type, extends(GridConstructor) :: off_ice_cice
	contains
	procedure :: make_grid => cice_make_grid
	procedure :: make_mask => cice_make_mask
	procedure :: make_ic => cice_make_ic
	end type off_ice_cice

	CONTAINS

!~~~~~~~
	subroutine cice_make_grid(this)
!~~~~~~~

!-------
!	Generate grid for CICE model
!	author Bayburin, Ruslan 2015
!	2016.04.07	editted by Ibrayev, Rashit
!-------

	class(off_ice_cice) :: this

	integer i, j, il, jl, nx, ny
	type(cpl_ice_cice) :: comp_ice_cice
    	character(len=256) :: filename

!-------
!	space grid and coordinate parameters
!-------
	real (kind=8), allocatable:: x1_v	(:,:)
	real (kind=8), allocatable:: x2_v	(:,:)
	real (kind=8), allocatable:: x1_t	(:,:)
	real (kind=8), allocatable:: x2_t	(:,:)
	real (kind=8), allocatable:: dx1_nw_v	(:,:)
	real (kind=8), allocatable:: dx1_ne_v	(:,:)
	real (kind=8), allocatable:: dx2_en_v	(:,:)
	real (kind=8), allocatable:: dx2_es_v	(:,:)
	real (kind=8), allocatable:: dx1_nw_t	(:,:)
	real (kind=8), allocatable:: dx1_ne_t	(:,:)
	real (kind=8), allocatable:: dx2_en_t	(:,:)
	real (kind=8), allocatable:: dx2_es_t	(:,:)
	real (kind=8), allocatable:: work	(:,:)
	real (kind=8), allocatable:: ulat	(:,:)
	real (kind=8), allocatable:: ulon	(:,:)
	real (kind=8), allocatable:: htn	(:,:)
	real (kind=8), allocatable:: hte	(:,:)
!	real (kind=8), allocatable:: hus	(:,:)
!	real (kind=8), allocatable:: huw	(:,:)
	real (kind=8), allocatable:: angle	(:,:)
	logical exists

!-------
!	Set grid size. here (il,jl) (inmio) = (nx,ny) + (2,2)
!	example
!		inmio il=42, jl=42
!		cice  nx=40, ny=40
!-------
	il = this % nx + 2
	jl = this % ny + 2
	write(*, *) "/comp_ice_cice/cice_make_grid: il,jl,this % nx, this % ny",il,jl,this % nx, this % ny
!-------
!	Initiate parameters by calling ini_params of cpl_module.f90
!	Reading namelist for grid type (tripole or spherical rectangular)
!-------
	call comp_ice_cice % ini_params

! Allocate grid arrays
	allocate (x1_v		(1:il, 1:jl))
	allocate (x2_v		(1:il, 1:jl))
	allocate (x1_t		(1:il, 1:jl))
	allocate (x2_t		(1:il, 1:jl))
	allocate (dx1_nw_v	(1:il, 1:jl) )
	allocate (dx1_ne_v	(1:il, 1:jl) )
	allocate (dx2_en_v	(1:il, 1:jl) )
	allocate (dx2_es_v	(1:il, 1:jl) )
	allocate (dx1_nw_t	(1:il, 1:jl) )
	allocate (dx1_ne_t	(1:il, 1:jl) )
	allocate (dx2_en_t	(1:il, 1:jl) )
	allocate (dx2_es_t	(1:il, 1:jl) )
	allocate (work		(1:il, 1:jl) )
	x1_v	 = 0.
	x2_v	 = 0.
	x1_t	 = 0.
	x2_t	 = 0.
	dx1_ne_v = 0
	dx1_nw_v = 0
	dx2_en_v = 0
	dx2_es_v = 0
	dx1_ne_t = 0
	dx1_nw_t = 0
	dx2_en_t = 0
	dx2_es_t = 0
	work	 = 0
	allocate( ulat		(this % nx, this % ny) )
	allocate( ulon		(this % nx, this % ny) )
	allocate( htn		(this % nx, this % ny) )
	allocate( hte		(this % nx, this % ny) )
!	allocate( hus		(this % nx, this % ny) )
!	allocate( huw		(this % nx, this % ny) )
	allocate( angle		(this % nx, this % ny) )
	ulat	= 0
	ulon	= 0
	htn	= 0
	hte	= 0
!	hus	= 0
!	huw	= 0
	angle	= 0

! Check for file with additional grid info
    inquire(file = add_grid_info_filename(this), exist = exists)
    if(.not. exists) then
       write(*, '(A)') "Error: no additional grid info from " // &
            "ocean grid generation file is present"
       write(*, '(A)') add_grid_info_filename(this) // " does not exist"
       stop 91
    endif
!-------
!	read grid from file made by inmio. all 'inmio' grid arrays are equal to inmio
!-------
	call this % get_var(trim(add_grid_info_filename(this)),"x1_v", arr_2D_8 = x1_v(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"x2_v", arr_2D_8 = x2_v(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"x1_t", arr_2D_8 = x1_t(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"x2_t", arr_2D_8 = x2_t(1:il, 1:jl))

	call this % get_var(trim(add_grid_info_filename(this)),"dx1_nw_v", arr_2D_8 = dx1_nw_v(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx1_ne_v", arr_2D_8 = dx1_ne_v(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx2_en_v", arr_2D_8 = dx2_en_v(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx2_es_v", arr_2D_8 = dx2_es_v(1:il, 1:jl))

	call this % get_var(trim(add_grid_info_filename(this)),"dx1_nw_t", arr_2D_8 = dx1_nw_t(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx1_ne_t", arr_2D_8 = dx1_ne_t(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx2_en_t", arr_2D_8 = dx2_en_t(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"dx2_es_t", arr_2D_8 = dx2_es_t(1:il, 1:jl))

	call this % get_var(trim(add_grid_info_filename(this)),"angle", arr_2D_8 = work(1:il, 1:jl))

!-------
!	make T-cell of inmio coincide with T-cell of cice
!	T-cell(i+1,j+1)(inmio) => T-cell(i,j)(cice) for i = 1,nx
!	variable distribution in T-cell of CICE, POP
!	ref. The Parallel Ocean Program (POP) Reference Manual LAUR-10-01853 23 March 2010
!	by definition of CICE & POP the T-point is placed at center of the T-cell
!-------
!-------
!
!
!		   INMIO				  CICE
!				FOR inmio (i,j)
!
!      V(i,j+1)                V(i+1,j+1)      V(i-1,j)               V(i,j)
!       *-----------+-----------*		*-----------+-----------*
!       !           !           !		!           !           !
!       !                       !		!                       !
!       !           !           !		!           !           !
!       !         T(i+1,j+1)    !		!         T(i,j)        !
!       + - - - - - + - - - - - +	==>	+ - - - - - + - - - - - +	
!       !                       !		!                       !
!       !           !           !		!           !           !
!       !                       !		!                       !
!       !           !           !		!           !           !
!       *-----------+-----------*		*-----------+-----------*
!      V(i,j)                 V(i+1,j)        V(i-1,j-1)              V(i,j-1)
!


!--------------------------INSERTION: shifting the grid halfstep right-----------------------------
	!Interpolating ANGLE from INMIO T-points to shifted CICE V-points
	do j = 1, this % ny
	do i = 1, this % nx
		angle(i,j) = (work(i+2,j+2)+work(i+2,j+1))*0.5d0
	end do
	end do

	do i=1,il-1
		work(i,:)=(x1_t(i,:)+x1_t(i+1,:))*0.5d0
	end do
	x1_t=work
	x1_t(1 ,:)=x1_t(il-1,:)
	x1_t(il,:)=x1_t(2   ,:)

	do i=1,il-1
		work(i,:)=(x1_v(i,:)+x1_v(i+1,:))*0.5d0
	end do
	x1_v=work
	x1_v(1 ,:)=x1_v(il-1,:)
	x1_v(il,:)=x1_v(2   ,:)
!----------------------------------------------------------------------------------------------------



!------------------------------------------------------------------------------
!	compute CICE grid parameters for interpolating weights
!------------------------------------------------------------------------------
!-------
!	define (lon,lat) coordinates of corner points of T-cell
!-------
!	corner(1 - ne
!	corner(2 - nw
!	corner(3 - sw
!	corner(4 - se
!-------
	
	select case (this % gType)
	
	case("T")
	
	do j = 1, this % ny
	do i = 1, this % nx
		this % center_lon(  i,j) = x1_t(i+1,j+1)
		this % corner_lon(1,i,j) = x1_v(i+1,j+1)
		this % corner_lon(2,i,j) = x1_v(i  ,j+1)
		this % corner_lon(3,i,j) = x1_v(i  ,j  )
		this % corner_lon(4,i,j) = x1_v(i+1,j  )

		this % center_lat(  i,j) = x2_t(i+1,j+1)
		this % corner_lat(1,i,j) = x2_v(i+1,j+1)
		this % corner_lat(2,i,j) = x2_v(i  ,j+1)
		this % corner_lat(3,i,j) = x2_v(i  ,j  )
		this % corner_lat(4,i,j) = x2_v(i+1,j  )
	end do
	end do

	case("V")
		x1_t(il,1:jl-1)=x1_t(2   ,1:jl-1)+360.
		do j = 1, this % ny
		do i = 1, this % nx
			
			this % center_lon(  i,j) = x1_v(i+1,j+1)
			
			this % corner_lon(1,i,j) = x1_t(i+2,j+2)
			this % corner_lon(2,i,j) = x1_t(i+1,j+2)
			this % corner_lon(3,i,j) = x1_t(i+1,j+1)
			this % corner_lon(4,i,j) = x1_t(i+2,j+1)

			this % center_lat(  i,j) = x2_v(i+1,j+1)
			
			this % corner_lat(1,i,j) = x2_t(i+2,j+2)
			this % corner_lat(2,i,j) = x2_t(i+1,j+2)
			this % corner_lat(3,i,j) = x2_t(i+1,j+1)
			this % corner_lat(4,i,j) = x2_t(i+2,j+1)
			
		end do
		end do

	case default
		write(*,*) "Error: no grid type defined for "//this % gType// " case."
		stop
	
	end select
	
	
	

!------------------------------------------------------------------------------
!	compile grid for CICE
!------------------------------------------------------------------------------
!-------
!	define (lon,lat) coordinates of V-cell
!-------
	do j = 1, this % ny
	do i = 1, this % nx
		ulon		 (  i,j) = x1_v(i+1,j+1)
		ulat		 (  i,j) = x2_v(i+1,j+1)
	end do
	end do

!-------
!	define grid spacings [m] (rightshifted grid)
!-------
	do j = 1, this % ny
	do i = 1, this % nx
		htn(i,j) = dx1_nw_t(i+2,j+1) + dx1_ne_t(i+1,j+1)	!dx1_nw_t(i+1,j+1) + dx1_ne_t(i+1,j+1)
		hte(i,j) = dx2_en_v(i+1,j  ) + dx2_es_v(i+1,j+1)	!dx2_en_t(i+1,j+1) + dx2_es_t(i+1,j+1)
	end do
	end do
	
	!HUS and HUW are not used. These are not shifted!
	!do j = 1, this % ny
	!do i = 1, this % nx
	!	hus(i,j) = dx1_nw_v(i+1,j  ) + dx1_ne_v(i+1,j  )
	!	huw(i,j) = dx2_en_v(i  ,j+1) + dx2_es_v(i  ,j+1)
	!end do
	!end do

!-------
!	transfer to units used in CICE [cm], [rad]
!-------
		ulat = ulat * deg_to_rad
		ulon = ulon * deg_to_rad
		htn = htn * m_to_cm
		hte = hte * m_to_cm
!		hus = hus * m_to_cm
!		huw = huw * m_to_cm


!          end do
!          htn(il,:) = htn(il-1,:)
!          htn(:,jl) = htn(:,jl-1)
!          hte(il,:) = hte(il-1,:)
!          hte(:,jl) = hte(:,jl-1)

	filename = trim(data_folder) // "cice.grid.nc"
	call this % put_var("ulat", "U-node latitude", "rad", 			arr_2D_8 = ulat	(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)
	call this % put_var("ulon", "U-node longitude", "rad", 			arr_2D_8 = ulon	(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)
	call this % put_var("htn", "Height of north edge of T-node", "cm", 	arr_2D_8 = htn	(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)
	call this % put_var("hte", "Height of east edge of T-node", "cm", 	arr_2D_8 = hte	(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)
	call this % put_var("angle", "angle FROM true east TO grid east", "rad",arr_2D_8 = angle(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)

	this % z(:) = 0.

	end subroutine cice_make_grid




!~~~~~~~
	subroutine cice_make_mask(this)
!~~~~~~~

!-------
!	Generating sea-land mask for CICE
!	author Bayburin, Ruslan 2015
!	2016.04.07	editted by Ibrayev, Rashit
!-------
	class(off_ice_cice) :: this
	integer i, j, il, jl, nx, ny
	character(100) :: filename
!-------
!	botom topography arrays
!-------
	real (kind=8), allocatable:: kv (:,:)
	real (kind=8), allocatable:: kt (:,:)
	real (kind=8), allocatable:: kmt	(:,:)

! Set grid size
	il = this % nx +2
	jl = this % ny +2

	allocate (kv		(1:il, 1:jl))
	allocate (kt		(1:il, 1:jl))
	kv =0
	kt =0
	allocate( kmt		(this % nx, this % ny) )
	kmt	= 0

!-------
!	read mask of the region from file made by inmio. all 'inmio' arrays are equal to inmio
!-------
	call this % get_var(trim(add_grid_info_filename(this)),"kv", arr_2D_8 = kv(1:il, 1:jl))
	call this % get_var(trim(add_grid_info_filename(this)),"kt", arr_2D_8 = kt(1:il, 1:jl))

!-------
!start==>	test mapping with mask
!-------
!-------
!	make island in CICE region (only in 160x80 configuration)
!-------
	!if(this % nx == 160 .and. this % ny == 80) then
	!	do j = 1, this % ny
	!	do i = 1, this % nx
	!		!if(i >= 20.and.i<=60.and.j>=30.and.j<=60) kv (i,j) = 0
	!		if((i-40)**2+1.5*(j-45)**2<400 ) kv (i,j) = 0
	!	end do
	!	end do
	!end if
!-------
!<==end	test mapping with mask 
!-------


!-------
!	make kmt - '~mask' for CICE
!-------
	do j = 1, this % ny
	do i = 1, this % nx
	if (kv(i+1, j+1) > 0 .or. kv(i+1, j) > 0) kmt (i,j) = max(kv(i+1,j+1),kv(i+1,j))
	end do
	end do

!-------
!	make mask2D - mask for interpolating weights
!-------
!	this % mask2D = misValInt
!	this % mask2D = missval
	this % mask2D = 0

	do j = 1, this % ny
	do i = 1, this % nx
	if (kv(i+1, j+1) > 0 .or. kv(i+1, j) > 0) this % mask2D (i,j) = 1
	end do
	end do


	filename = TRIM(data_folder) // "cice.kmt.nc"
	call this % put_var("kmt", "kmt", "metres", arr_2D_8 = kmt(:,1-additional_ny:this % ny), external_filename = filename, add_ny=additional_ny)

        write(*,*), "MASK for WTS Remaping"
	filename = TRIM(data_folder) // "ICE.mask.nc"
        call this % put_var(var="center_lat", long_name="latitude", units="degrees_north",axis = "Y", external_filename = filename, arr_2D=REAL(this % center_lat, kind = 4))
        call this % put_var(var="center_lon", long_name="longitude",units="degrees_east",axis = "X", external_filename = filename ,arr_2D=REAL(this % center_lon, kind = 4))
        call this % put_var(var="mask2D", long_name = "mask", units = "unitless", external_filename = filename, arr_2D = this % mask2D)

	end subroutine cice_make_mask




!~~~~~~~
	subroutine cice_make_ic(this)
!~~~~~~~
	class(off_ice_cice) :: this
	end subroutine cice_make_ic




!~~~~~~~
	function add_grid_info_filename(this)
!~~~~~~~
	class(off_ice_cice) :: this
	character(len=256) add_grid_info_filename

	add_grid_info_filename = trim(data_folder) // "ADD_GRID_INFO_OCN_" //     &
	trim(I2S(this % nx)) // 'x' // trim(I2S(this % ny)) // ".nc"

	end function add_grid_info_filename

	end module off_ice_cice_module

