	
	module cpl_lnd_core_module
	
	use comp_module
	use shared_module
	
	implicit none

!==================================================================================
! Define type 'cpl_lnd', which extends class component and define is's abstract methods
!==================================================================================

	TYPE, EXTENDS(comp) :: cpl_lnd_core
	
	CONTAINS
		PROCEDURE :: ini_params => l_ini_params
		PROCEDURE :: ini_reg_comp => l_ini_reg_comp
		PROCEDURE :: ini_reg_data => l_ini_reg_data
		PROCEDURE :: ini_allocate => l_ini_allocate
		PROCEDURE :: ini_main => l_ini_main
		PROCEDURE :: main_step => l_main_step
	END TYPE cpl_lnd_core
	
	
	integer lnd_i_size, lnd_j_size, lnd_k_size, lnd_m_size, lnd_steps_in_day
	integer lnd_rivers_type
	logical lnd_read_cp
	
	real (kind=4), allocatable :: runoff(:,:),salt_shifted(:,:)

	CONTAINS
	
!================================================================================================================================================================
	subroutine l_ini_params(this)
		
	class(cpl_lnd_core) :: this
	namelist /LND_MAIN/ lnd_i_size, lnd_j_size, lnd_k_size, lnd_m_size, lnd_steps_in_day, lnd_read_cp
	namelist /LND_PHYSICS/ lnd_rivers_type
	
	open(9, file= "./configure/lnd_list.in")
	read(9, nml = LND_MAIN)
	read(9, nml = LND_PHYSICS)
	close(9)

	end subroutine l_ini_params
!================================================================================================================================================================

	subroutine l_ini_reg_comp(this)
		
	class(cpl_lnd_core) :: this

	call this % reg_model(	reg_np = run_np(COMP_LND),		&
				reg_i_size = lnd_i_size, 		&
				reg_j_size = lnd_j_size,		&
				reg_k_size = lnd_k_size,		&
				reg_m_size = lnd_m_size,		&
				reg_decomp_type_2D = .FALSE.,		&
				reg_i_cycle = .TRUE.,			&
				reg_bsc_grid = .FALSE.,			&
				reg_time_steps_day = lnd_steps_in_day,	&
				reg_time_save_it = 0,			&
				reg_read_cp = lnd_read_cp,		&
				reg_halo_stencil = 0,			&
				reg_halo_wide = 0			)
	
	end subroutine l_ini_reg_comp

!================================================================================================================================================================

	subroutine l_ini_reg_data(this)
		
	class(cpl_lnd_core) :: this
	
 	if(lnd_rivers_type==1) call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff, name = 'runoff', period = (365*1440)/12, file = "./data/CNYFv2/runoff.daitren.clim.10FEB2011.nc"  )
	if(lnd_rivers_type==2) call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff, name = 'runoff', period = (365*1440)/12, file = "./data/IAFv2/runoff.daitren.iaf.10FEB2011.nc"  )
	if(lnd_rivers_type==4) call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = runoff, name = 'runoff', period = (365*1440)/12, file = "./data/CNYFv2_mm/runoff_mm_january.nc"  )
	
	!call this % reg_array(recv_from=COUPLER, action=ACTION_READ_FD, DIM2_arr = salt_shifted  , name = 'salt_shifted'  , period = (365*1440)/12, file = "./data/CNYFv2/salt_shifted.nc"  ) 		
	
	call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=runoff, 	period = (365*1440)/12, name = 'runoff', map_type = MAP_SRC_TYPE )
	!call this % reg_array(send_to=COMP_OCN, action=ACTION_MAPPING, DIM2_arr=salt_shifted,	period = (365*1440)/12, name = 'salt'  , map_type = MAP_SRC_TYPE )
	
	!call this%reg_array (send_to=coupler,action=action_save_dg,dim2_arr=runoff, name='runoff',period=7200)
	
	end subroutine l_ini_reg_data	
	
!================================================================================================================================================================
	
	subroutine l_ini_allocate(this)
		
	class(cpl_lnd_core) :: this
	
	allocate(runoff(cmp % i_loc_size, cmp % j_loc_size))
	allocate(salt_shifted(cmp % i_loc_size, cmp % j_loc_size))
	
	end subroutine l_ini_allocate
!================================================================================================================================================================

	subroutine l_ini_main(this)
		
	class(cpl_lnd_core) :: this
	
	end subroutine l_ini_main
				
!================================================================================================================================================================

	subroutine l_main_step(this)
		
	class(cpl_lnd_core) :: this
	
	end subroutine l_main_step
	
	
!================================================================================================================================================================
	
	end module cpl_lnd_core_module