	
	module off_lnd_das_module
	
	use off_module
	use shared_utils_module
	use netcdf
	
	implicit none
	integer :: additional_ny=0	!0 - to make full grid, <0 - to make reduced grid (eg. for Arctic only)
	
	TYPE, EXTENDS(GridConstructor) :: off_lnd_das
	
	CONTAINS
	
	PROCEDURE :: make_grid => l_make_grid
	PROCEDURE :: make_mask => l_make_mask
	PROCEDURE :: make_ic => l_make_ic
	
	END TYPE off_lnd_das
	
	CONTAINS

!================================================================================================================================================================
	
	subroutine l_make_grid(this)
	
	class(off_lnd_das) :: this

	! Input file
	!!character(*), parameter :: filename = "../data/CNYFv2/runoff.daitren.clim.10FEB2011.nc"

	! netCDF variables
	integer ncid
	integer nx_dimid, ny_dimid
	integer lon_id, lat_id
	integer nc_err
	
	! Work variables
	integer i, j, counter, il, jl

	! Temporary grid variables
	real(kind = 8) :: input_lon(this % nx),lon_t(this % nx + 1)
	real(kind = 8) :: input_lat(this % ny),lat_t(this % ny + 1)
	
	real(kind = 8), dimension(this % nx,this % ny,4) :: input_corner_lat,input_corner_lon
	real,parameter :: xstep = 1.0 !0.25
	real,parameter :: ystep = 1.0 !0.25
	
	il=this % nx
	jl=this % ny
	
!===================================
! Inquire grid from CORE netCDF-file
!===================================

	! Open file
	!!nc_err = NF90_OPEN(TRIM(filename), NF90_NOWRITE, ncid)
	
	! Inquire dimension sizes
 	!!nc_err = NF90_INQ_DIMID(ncid, "ni", nx_dimid)
	!!nc_err = NF90_INQUIRE_DIMENSION(ncid, nx_dimid, len = il)
 	!!nc_err = NF90_INQ_DIMID(ncid, "nj", ny_dimid)
 	!!nc_err = NF90_INQUIRE_DIMENSION(ncid, ny_dimid, len = jl)

 	!!if(il /= this % nx .OR. jl /= this % ny) then
	!!	write(*,*) 'Error in LND_CORE function: wrong grid sizes of input netCDF.'
	!!	stop
	!!end if
			
	! Inquire coordinate variables
	!allocate(input_lon(il))
	!!nc_err = NF90_INQ_VARID(ncid, "xc", lon_id)
	!!nc_err = NF90_GET_VAR(ncid, lon_id, input_lon) 
	do i=1,this % nx
		input_lon(i)=-180.+xstep/2.+(i-1)*xstep
	end do
	do j=1,this % ny
		input_lat(j)=-90.+ystep/2.+(j-1)*ystep
	end do
		
	!allocate(input_lat(jl))
	!!nc_err = NF90_INQ_VARID(ncid, "yc", lat_id)
	!!nc_err = NF90_GET_VAR(ncid, lat_id, input_lat) 
			
	!allocate(input_corner_lon(4,il))
	!!nc_err = NF90_INQ_VARID(ncid, "xv", lon_id)
	!!nc_err = NF90_GET_VAR(ncid, lon_id, input_corner_lon) 
			
	!allocate(input_corner_lat(4,jl))
	!!nc_err = NF90_INQ_VARID(ncid, "yv", lat_id)
	!!nc_err = NF90_GET_VAR(ncid, lat_id, input_corner_lat) 			
			
	!!nc_err = NF90_CLOSE(ncid)
	
!====================================================
! Make array of imaginary T-points, because we should
! provide corners of our cells.
!====================================================

	do i = 2, il
		lon_t(i) = (input_lon(i-1) + input_lon(i))/2.
	end do
	lon_t(1) = input_lon(1) - (lon_t(2) - input_lon(1))
	lon_t(il+1) = input_lon(il) + (input_lon(il) - lon_t(il))
		
	do j = 2, jl
		lat_t(j) = (input_lat(j-1) + input_lat(j))/2.
	end do
	lat_t(1) = input_lat(1) - (lat_t(2) - input_lat(1))
	lat_t(jl+1) = input_lat(jl) + (input_lat(jl) - lat_t(jl))	
		
!=====================================================
! Fill ouput 'GridType' structure with our information 
!=====================================================

	!print *,input_lat(5,5),input_center_lat(5,5) 

 	do j = 1, this % ny
	do i = 1, this % nx
 	
 		this % center_lat(i,j  ) = input_lat(j) 
 		this % corner_lat(1,i,j) = lat_t(j)
 		this % corner_lat(2,i,j) = lat_t(j)
 		this % corner_lat(3,i,j) = lat_t(j+1)
 		this % corner_lat(4,i,j) = lat_t(j+1)
 
 		this % center_lon(i,j  ) = input_lon(i)
 		this % corner_lon(1,i,j) = lon_t(i)
 		this % corner_lon(2,i,j) = lon_t(i+1)
 		this % corner_lon(3,i,j) = lon_t(i+1)
 		this % corner_lon(4,i,j) = lon_t(i)
 		
		this % mask2D(i,j) = 1.
 		
 	end do
 	end do
 		
	this % z(:) = 0.	
	
      end subroutine l_make_grid
      
!================================================================================================================================================================
	
	subroutine l_make_mask(this)
	class(off_lnd_das) :: this
	character(100) :: filename
	logical exists
	
	!filename = "../data/NOAA/lsmask.oisst.v2.nc"
	!INQUIRE(file = filename, exist = exists)
	!if(.NOT. exists) then
	!	Stop '***Cannot find LND_DAS mask***'
	!else
	!	call this % get_var(filename, "lsmask", arr_2D = this % mask2D)
	!end if	
	this % mask2D = 1.
	
	end subroutine l_make_mask
	
!================================================================================================================================================================
	
	subroutine l_make_ic(this)
	
	class(off_lnd_das) :: this

	end subroutine l_make_ic
	
!================================================================================================================================================================

	end module off_lnd_das_module
	
	