!=======================================================================================================
! Define grid for vertical direction: fill the array "z" with depths of model horizons (in meters)
!=======================================================================================================

subroutine o_user_grid_z
	
	use o_grid_module, only: z,k,kl
	implicit none
	real z_axis(100)
	
	! Define constant z_axis() and make array z() from it
   	z_axis(1) = 3.
	z_axis(2) = 9.
	z_axis(3) = 15.
	do k =  4,  9
		z_axis( k)=   15.  +  10.*(k-3)
	end do
	do k =  10,  15
		z_axis( k)=  90. +  15.*(k-10)
	end do
	z_axis(16)=  185.
	z_axis(17)=  205.
	z_axis(18)=  230.
	z_axis(19)=  255.
	z_axis(20)=  280.
	z_axis(21)=  310.
	z_axis(22)=  350.
	z_axis(23)=  400.
	z_axis(24)=  475.
	z_axis(25)=  550.
	z_axis(26)=  650.
	z_axis(27)=  775.
	z_axis(28)=  900.
	z_axis(29)= 1050.
	z_axis(30)= 1250.
	do k = 31,  50
		z_axis( k)= 1250. + 250.*(k-30)
	end do

	z(1:kl+1) = z_axis(1:kl+1)

end                                                               

!==========================================================================================================================================================
! Here you may define atmospheric parameters (used when there is no ATM component run and atm_model=1). By default they are referenced to the 10m height  
!==========================================================================================================================================================

subroutine atm_user_parameters

	use o_grid_module, only: i,j,jsouth, jnorth, iwest, ieast, myrank
	use o_tf_module, only: wind_speed_x_t, wind_speed_y_t, tair02, shumid, prec, ssr, str
	implicit none

	do j = jsouth(myrank)-1, jnorth(myrank)+1
	do i = iwest (myrank)-1, ieast (myrank)+1
		wind_speed_x_t (i,j) = 5.	!wind speed components [m/sec]
		wind_speed_y_t (i,j) = 0.
		tair02 (i,j) = 30.		!temperature [deg.C] 
		shumid (i,j) = 0.002 		!specific humidity (kg/kg)
		prec (i,j)   = 0. 		!precipitation [m/sec]
		ssr (i,j)    = 175. 		!shortwave solar radiation [w/m**2] 
		str (i,j)    = -50.		!total longwave radiation [w/m**2]
	end do
	end do
	
end

!======================================================================================================================
! Here you may define atmospheric surface fluxes (used when there is no ATM component run and atm_model=2). 
!======================================================================================================================

subroutine atm_user_fluxes
	
	use o_grid_module, only: i,j,jsouth, jnorth, iwest, ieast, myrank
	use o_tf_module, only: wind_stress_x_v, wind_stress_y_v, shumid, prec, ssr, str, sns, ltn, prec, evap
	implicit none

	do j = jsouth(myrank)-1, jnorth(myrank)+1
	do i = iwest (myrank)-1, ieast (myrank)+1
		wind_stress_x_v(i,j)	=0.002		!Wind surface stress components
		wind_stress_y_v(i,j)	=0.002
		ssr(i,j)	= 0.			!shortwave solar radiation [w/m**2] 
		str(i,j)	= 0.			!total longwave radiation [w/m**2]
		sns(i,j)	= 0.			!sensible heat flux [w/m**2]
		ltn(i,j)	= 0.			!latent heat flux [w/m**2]
		prec(i,j)	= 0.			!precipitation [m/s]
		evap(i,j)	= 0.			!evaporation [kg/m**2/s](!)
	end do
	end do
	
end
