!===============================================================
!	The hv array contains raw topography data (meters). 
!	By default it is interpolated to model grid from ETOPO5 database.   
!	You may redefine it here.
!===============================================================

	subroutine o_user_redefine_topo
!-------
!	make topography for test pool
!-------
	use o_grid_module, only: hv,i,j,il,jl,i_cycle,j_cycle
	implicit none
	
!-------
!	test pool
!-------
	hv=1000.		!	flat bottom, depth=1000 meters

	do j = jl/4, jl/4*3	!	add underwater moutntain
	do i = il/4, il/4*3
	hv(i,j)=100.
	end do
	end do

!-------
!	make 'zero' frame
!-------
	if(i_cycle.eq.0) then
		do j=1,jl 
		hv(1   ,j)=0.
		hv(2   ,j)=0.	!	this 'zero' frame (land frame) made for coupling with cice. 
		end do

		do j=1,jl 
		hv(il-1,j)=0.
		hv(il  ,j)=0.
		end do
	end if

	
	if(j_cycle.eq.0) then
		do i=1,il
		hv(i,1   )=0.
		hv(i,2   )=0.	!	this 'zero' frame (land frame) made for coupling with cice. 
		end do

		do i=1,il
		hv(i,jl-1)=0.
		hv(i,jl  )=0.
		end do
	end if


end


!===============================================================
!	You may finally tune bottom topography array manually 
!===============================================================

subroutine o_user_bottom_tuning

	use o_grid_module, only: kv
	implicit none
	
	!kv(91:93,46:48)=2  !Making Kerch strait
! wom01
!	kv(1580:1780,1100:1350)=0  !Caspy
!	kv(1420:1580,1175:1275)=0  !Black+Marmara
!	kv(3241,1:4)=0  	!an Antarctic bay
!	kv(3245,1:8)=0
!	kv(3258,1:13)=0

end

!===============================================================
!	By default initial T,S fields are interpolated to model grid from WOA2009 database.   
!	You may redefine them here. 
!===============================================================

subroutine o_user_redefine_T (initial_T)

	use o_grid_module, only: i,j,il, jl, kl, z, k, kt
	implicit none
	real(kind=4), dimension(2:il-1, 2:jl-1, kl) :: initial_T
	
!--- stable test initial field ---
	call o_user_grid_z
	initial_T=0.
	do j = 2, jl-1
	do i = 2, il-1
	do k = 1, kt(i,j)
!		initial_T(i,j,k) = 4. + 20. * exp(-z(k)/500.)
		initial_T(i,j,k) = -1.
	end do
	end do
	end do

end

subroutine o_user_redefine_S (initial_S)

	use o_grid_module, only: i,j,il, jl, kl, z, k, kt
	implicit none
	real(kind=4), dimension(2:il-1, 2:jl-1, kl) :: initial_S
	
!--- stable test initial field ---
	call o_user_grid_z
	initial_S=0.
	do j = 2, jl-1
	do i = 2, il-1
	do k = 1, kt(i,j)
		initial_S(i,j,k) = 35. -  5. * exp(-z(k)/500.)
!		initial_S(i,j,k) = 30.
	end do
	end do
	end do

end
