!-------
!	correct model bottom topography and initial conditions for T, S
!	call from /comp_ocn/off_ocn_module.f90
!	last edition:
!	2016.05.12	by Ibrayev, Rashit
!-------



!~~~~~~~
	subroutine o_user_redefine_topo
!~~~~~~~

!-------
!	make topography for test pool
!-------
	use o_grid_module, only: hv,i,j,il,jl,i_cycle,j_cycle
	implicit none
!-------
!	make 'zero' frame
!-------
	if(i_cycle.eq.0) then
		do j=1,jl 
		hv(1   ,j)=0.
		hv(2   ,j)=0.	!	this 'zero' frame (land frame) made for coupling with cice. 
		end do

		do j=1,jl 
		hv(il-1,j)=0.
		hv(il  ,j)=0.
		end do
	end if

	
	if(j_cycle.eq.0) then
		do i=1,il
		hv(i,1   )=0.
		hv(i,2   )=0.	!	this 'zero' frame (land frame) made for coupling with cice. 
		end do

		do i=1,il
		hv(i,jl-1)=0.
		hv(i,jl  )=0.
		end do
	end if


end



!~~~~~~~
	subroutine o_user_bottom_tuning
!~~~~~~~

!-------
!	if necessary tune bottom topography array manually
!	at this stage (x1,x2) coordinates are available
!-------

	use o_grid_module
!	use o_grid_module, only: kv
	implicit none
	
!	kv(1:11,46:48)=2  !Making Kerch strait
! wom01
!	kv(1580:1780,1100:1350)=0  !Caspy
!	kv(1420:1580,1175:1275)=0  !Black+Marmara
!	kv(3241,1:4)=0  	!an Antarctic bay
!	kv(3245,1:8)=0
!	kv(3258,1:13)=0

!	archard2: correct at bsc Greenwich
!	kv(70:71,18:20)=0

!-------
!	inciarc_g: delete some Seas using geographical coordinates
!-------
!-------
!	black sea
!-------
	do j = 2,jl-1
	do i = 2,il-1
	if (	(x1_v(i,j) > 27.00) .and.	&
		(x1_v(i,j) < 43.00) .and.	&
		(x2_v(i,j) > 40.00) .and.	&
		(x2_v(i,j) < 47.00)		) kv(i,j) = 0
	end do
	end do
!-------
!	caspian sea
!-------
	do j = 2,jl-1
	do i = 2,il-1
	if (	(x1_v(i,j) > 45.00) .and.	&
		(x1_v(i,j) < 55.00) .and.	&
		(x2_v(i,j) > 35.00) .and.	&
		(x2_v(i,j) < 50.00)		) kv(i,j) = 0
	end do
	end do
!-------
!	Mediterranean sea
!-------
	do j = 2,jl-1
	do i = 2,il-1
	if (	(x1_v(i,j) > -5.50) .and.	&
		(x1_v(i,j) < 37.00) .and.	&
		(x2_v(i,j) > 30.00) .and.	&
		(x2_v(i,j) < 43.00)		) kv(i,j) = 0
	if (	(x1_v(i,j) >  0.00) .and.	&
		(x1_v(i,j) < 30.00) .and.	&
		(x2_v(i,j) > 43.00) .and.	&
		(x2_v(i,j) < 49.00)		) kv(i,j) = 0
	end do
	end do


end

!===============================================================
!	By default initial T,S fields are interpolated to model grid from WOA2009 database.   
!	You may redefine them here. 
!===============================================================

!~~~~~~~
	subroutine o_user_redefine_T (initial_T)
!~~~~~~~

	use o_grid_module, only: i,j,il, jl, kl, z, k, kt
	implicit none
	real(kind=4), dimension(2:il-1, 2:jl-1, kl) :: initial_T
end

!~~~~~~~
	subroutine o_user_redefine_S (initial_S)
!~~~~~~~

	use o_grid_module, only: i,j,il, jl, kl, z, k, kt
	implicit none
	real(kind=4), dimension(2:il-1, 2:jl-1, kl) :: initial_S
end

