#!/bin/bash

# Catch all errors
ulimit -s unlimited

if [ $# != 4 ]; then
    echo "Usage: bash generate_ocean_all.sh 180 90 49 exp_name"
    exit 1
fi

SIZE_X=$1
SIZE_Y=$2
SIZE_K=$3
EXP=$4

OCN=ocn
ATM=atm_interim
ICE=ice_cice

rm data/OCN*.nc
rm data/ATM*.nc
rm data/LND*.nc
rm data/ICE*.nc

rm ../data/OCN*.nc
rm ../data/ATM*.nc
rm ../data/LND*.nc
rm ../data/ICE*.nc

set -e

# Make OCN-V with no mask (because no topo yet!) grid in CDO format 
bash off_launcher.sh grd $OCN $SIZE_X $SIZE_Y $SIZE_K V $EXP

# Interpolate ETOPO5 on OCN-V grid
bash off_launcher.sh int bil ~/House/ETOPO/ETOPO5.nc topo ./data/OCN_${SIZE_X}x${SIZE_Y}_V_cdo.nc 

# Make OCN-T grid with mask (because now we know topo)
bash off_launcher.sh grd $OCN $SIZE_X $SIZE_Y $SIZE_K T $EXP

# Make ATM_INTERIM grid
bash off_launcher.sh grd $ATM 480 241 4 T $EXP

# Make ICE_CICE grid
#bash off_launcher.sh grd $ICE $SIZE_X $SIZE_Y 5 T $EXP

# Make interpolation weights from ATM component to OCN
bash off_launcher.sh wts bil ./data/ATM_INTERIM_480x241_T_cdo.nc  ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc 

# Make interpolation weights to and from ICE_CICE
#bash off_launcher.sh wts bil ./data/ATM_INTERIM_480x241_T_cdo.nc ./data/ICE_CICE_${SIZE_X}x${SIZE_Y}_T_cdo.nc
#bash off_launcher.sh wsc ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc ./data/ICE_CICE_${SIZE_X}x${SIZE_Y}_T_cdo.nc 90
#bash off_launcher.sh wsc ./data/ICE_CICE_${SIZE_X}x${SIZE_Y}_T_cdo.nc ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc 90

# Make initial conditions for OCN: inteerpolate them to OCN grid and then put it to IC-file
#bash off_launcher.sh int bil ~/House/WOA2009/temperature_annual_1deg.nc t_an ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc $SIZE_K
#bash off_launcher.sh int bil ~/House/WOA2009/salinity_annual_1deg.nc    s_an ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc $SIZE_K
bash off_launcher.sh int bil ../../data_external/WOA13/woa13_decav_t01+t13_04v2_tan.nc t_an ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc $SIZE_K
bash off_launcher.sh int bil ../../data_external/WOA13/woa13_decav_s01+s13_04v2_san.nc s_an ./data/OCN_${SIZE_X}x${SIZE_Y}_T_cdo.nc $SIZE_K

bash off_launcher.sh ics $OCN $SIZE_X $SIZE_Y $SIZE_K T $EXP
