#!/bin/bash

ROOT_DIR="../"
set -e

function main() {

echo
echo "Running script: " $@ 
echo 

case "$1" in

	#====================
	"grd" | "ics") 
	
	if [ $# != 7 ]; then
		echo "Usage: bash make $1 ocn 360 180 90 T exp_name"
		exit 1
	fi
	
	#Compile
	cd $ROOT_DIR
	bash make --off $2 --test
	cd -
	
    # Run program
    comp_upper=`echo $2 | tr '[:lower:]' '[:upper:]'`
    echo '===Running ./off.exe ' $1 ${comp_upper:0:3} ${@:3:$#}
    ./off.exe $1 ${comp_upper:0:3} ${@:3:$#}
    ;;
	
    #====================
	"wts")

	if [ $# != 4 ]; then
		echo "Usage: bash make wts [bil,con,nn,bic,wav] src.nc dst.nc"
		exit
	fi
	
	src=$3
	dst=$4
	echo cdo gen$2,$4 -const,1,$3 ../data/${src:7:${#src}-16}_to_${dst:7:${#dst}-16}.nc 
	cdo gen$2,$4 -const,1,$3 ../data/${src:7:${#src}-16}_to_${dst:7:${#dst}-16}.nc
	;;
	
    #==================
	"wsc")

	if [ $# != 4 ]; then
		echo "Usage: bash make wsc ./data/src.nc ./data/dst.nc num_search_bins"
		exit
	fi
	
	src=$2
	dst=$3
	
	cd ./SCRIP/source
	make
	cd ..
	
	printf "&remap_inputs\n" > scrip_in
	printf "    num_maps = 1\n" >> scrip_in
	printf "    grid1_file = '../"$2"'\n" >> scrip_in
	printf "    grid2_file = '../"$3"'\n" >> scrip_in
	printf "    interp_file1 = '"../../data/${src:7:${#src}-16}_to_${dst:7:${#dst}-16}.nc"'\n" >> scrip_in
	printf "    interp_file2 = '"../../data/${dst:7:${#dst}-16}_to_${src:7:${#src}-16}.nc"'\n" >> scrip_in
	printf "    map1_name = 'forward mapping'\n" >> scrip_in
	printf "    map2_name = 'backward mapping'\n" >> scrip_in
	printf "    map_method = 'distwgt'\n" >> scrip_in
	printf "    normalize_opt = 'fracarea'\n" >> scrip_in
	printf "    output_opt = 'scrip'\n" >> scrip_in
	printf "    restrict_type = 'latitude'\n" >> scrip_in
	printf "    num_srch_bins = "$4"\n" >> scrip_in
	printf "    luse_grid1_area = .false.\n" >> scrip_in
	printf "    luse_grid2_area = .false.\n" >> scrip_in
	printf "/" >> scrip_in
	./scrip	
	;;	
		
    #====================
	"int")
        interpolate "${@:2:$#}"
	;;

    #====================
    "lnd")
    if [ $# != 9 ]; then
        echo "Usage: bash make $1 ocn 360 180 90 T exp_name 25 2D"
        exit 1
    fi
    
    #Compile
    cd $ROOT_DIR
    bash make --exe off $2  
    cd -
    
    # Run program
    comp_upper=`echo $2 | tr '[:lower:]' '[:upper:]'`
    ./off.exe $1 ${comp_upper:0:3} ${@:3:$#}        
    ;;
    
    #====================    
	*)
	echo "Error [off_launcher] : wrong argument $1"
    usage
	;;
esac
	
}

#========================================================================================

function usage()  { 
    echo ""
    echo "Usage: bash make [option] [arguments]"

    echo "Options:"
    echo " grd -- build CDO-representation of model grid"
    echo " ics -- add initial conditions to IC-file"
    echo " wts -- generate SCRIP-file of interpolation weights"
    echo " int -- make CDO-interpolation of netCDF file"
    echo " lnd -- analyze land mask and pre-build processor decomposition"

    echo "Run some option to get detailed instructions."
}

#========================================================================================

function interpolate() {

    if [ $# != 4 ] && [ $# != 5 ]; then
        echo "Usage: bash make int bil input.nc varName model_grid_cdo.nc [num_of_levels]"
        exit 0
    fi
            
    local INT_TYPE=$1
    local IN_FILE=$2
    local VAR_NAME=$3
    local GRID=$4
    local LEVEL_NUM=$5
                
    # Build output filename
    str=$GRID
    ofile=${str:0:${#str}-7}_$VAR_NAME.nc
    rm -f $ofile

    # Map level per level
    MAX_REMAPS=1 cdo remap$INT_TYPE,$GRID -selvar,$VAR_NAME $IN_FILE $ofile

    if [ -n "$LEVEL_NUM" ]; then    
    
        # Interpolate vertical level
        # Get z data from ncdump
        levels=$(ncdump -v z $GRID | sed -e '1,/data:/d' -e '$d')
        # Delete "z = ; ' '"
        levels=$(echo "$levels" | sed "s@z = @@")
        levels=$(echo "$levels" | sed "s@;@@")
        levels=$(echo ${levels//[[:space:]]})
        echo $levels    

        # Interpolate along z
        zfile=${ofile:0:${#ofile}-3}_z.nc
        rm -f $zfile
        cdo intlevel,$levels $ofile $zfile
    fi
}

#========================================================================================

main "$@"
	