#!/bin/bash

start_date_anchor="2020-03-08"
end_date_anchor="2020-03-29"

# start_date in seconds since Unix epoch
start_date_zero=$(date -d "$start_date_anchor" +%s)
end_date_zero=$(date -d "$end_date_anchor" +%s)
# start_date in "YYYY-MM-DD HH:MM:SS" format
start_date=$(date -d @"$((start_date_zero ))" +"%F %T")
start_adate_d=$(date -d @"$((start_date_zero ))" +"%F")


# end_date in "YYYY-MM-DD HH:MM:SS" format
cur_date_d=$(date -d @"$((start_date_zero))" +"%F")

#Find files with Obs Data. If exist then should assimilate
aviso_f=data/AVISO/SSL_$cur_date_d\_aviso.nc
argo_f=data/ARGO/T_$cur_date_d\_merge.nc
aice_f=data/AICE/AICE_$cur_date_d\_aice.nc

day_offset=86400
while [ -f $aviso_f ] && [ -f $argo_f ] && [ -f $aice_f ];
do
  echo Exist Obs Data for $cur_date_d 
  cur_date_zero=$(date -d "$cur_date_d" +%s)
  cur_date_d=$(date -d @"$((cur_date_zero + day_offset ))" +"%F")
  aviso_f=data/AVISO/SSL_$cur_date_d\_aviso.nc
  argo_f=data/ARGO/T_$cur_date_d\_merge.nc
  aice_f=data/AICE/AICE_$cur_date_d\_aice.nc
done

end_adate_d=$(date -d @"$((cur_date_zero ))" +"%F")
start_fdate_d=$end_adate_d
end_fdate_d=$(date -d @"$((end_date_zero))" +"%F")

adays=$(( ($(date --date="$end_adate_d" +%s) - $(date --date="$start_adate_d" +%s) )/($day_offset) ))
fdays=$(( ($(date --date="$end_fdate_d" +%s) - $(date --date="$start_fdate_d" +%s) )/($day_offset) ))

echo "Assimilation: $start_adate_d - $end_adate_d" this is $adays
echo "Pure forecast: $start_fdate_d - $end_fdate_d" this is $fdays

#make conf file
cat configure/exp_a01.in.sed \
	| sed "s/START_SED/$start_adate_d/g" \
    	| sed "s/DAYS_SED/$adays/g"  > configure/exp_a01.in

sed "s/SED_DATE/$start_adate_d/g" ice_in.sed > ice_in


#ON assimilation
cd configure
sed 's/DAS_SED/.TRUE./g' ocn_list.in.sed > ocn_list.in
sed 's/DAS_SED/.TRUE./g' ice_list.in.sed > ice_list.in
grep das *_list.in
cd ..
./run_test.sh a01

#OFF assimilation and start from the end of a01 exp
cd data
ln -s OCN_a01_CP_$end_adate_d\.nc OCN_f01_CP_$start_fdate_d\.nc
cd ..

cat configure/exp_a01.in.sed \
	| sed "s/START_SED/$start_fdate_d/g" \
	| sed "s/DAYS_SED/$fdays/g"  > configure/exp_f01.in
sed "s/SED_DATE/$start_fdate_d/g" ice_in.sed > ice_in

cd configure
sed 's/DAS_SED/.FALSE./g' ocn_list.in.sed > ocn_list.in
sed 's/DAS_SED/.FALSE./g' ice_list.in.sed > ice_list.in
grep das *_list.in
cd ..
./run_test.sh f01
