#define DEBUG
#define ARGO_T
#define ARGO_S
#undef SAT_SST
#define SAT_SSL
#define SAT_AICE
#include "../../comps/ocn/inmio4.1/source/o_defines.f90"
!========================================================================================
! AOI_module
!========================================================================================
!   There are 2 domain decompositions over MPIproc (1D) in this Obs Assim methods.
!   Corralation between points in different wide stips = 0
!   main decomposition are over np
!   each proc get approximately equal number of AS points
!   and backgound domain correspondent to this points
!   --------------------------------------
!   |       ij_hi
!   |j
!   |       ij_lo
!   ---------------------------------------
!   (strip) ~ ( ij_lo : ij_hi) ~ ( np_lo : np_hi)
!========================================================================================


module aoi_module
use datetime_module
use decomposition_module
use event_module
implicit none
#define _AS_DECOMP( _VAR_ ) \
    integer :: np\_VAR_ ,nl\_VAR_ ; \
    integer :: np_lo\_VAR_ , np_hi\_VAR_  ; \
!=============================================================
!   Common AS Array decomposition
!=============================================================
_AS_DECOMP()

!=============================================================
!  AS Array decomposition for specific vars (T,S,SST, SSL, AICE)
!=============================================================
_AS_DECOMP ( _T )

_AS_DECOMP ( _S )

_AS_DECOMP ( _SST )

_AS_DECOMP ( _SSL )

_AS_DECOMP ( _AICE )
integer :: var_id                       ! ID of current VAR (T,S,SSL,AICE,SST) ~(1,2,3,4,5)
integer :: ocn_il, ocn_jl, ocn_kl               ! OCN parameters
integer :: aoi_service_id               ! ID parameters
integer :: aoi_rnk, aoi_size, aoi_comm, ierr    ! For MPI communication

character(3) :: var_name
character(NAME_LEN)  x1_arr_name, x2_arr_name
character(NAME_LEN)  x12_o_name, var_o_name, ij_o_name
character(NAME_LEN)  var_b_in_o_name, var_a_in_o_name, var_o_dif_name
character(NAME_LEN) fname_obs, fname_dif,fname_DG        ! file names for obs data and markers
character(NAME_LEN) logfile                     ! file for liging L2 error

character(NAME_LEN)  var_b_name
character(NAME_LEN)  var_cp_name                ! name of var in CP file for EnKF
character(10) :: yyyymmdd
#define V_N 5
real*4  :: das_r_coef (V_N) = (/0.02,0.02,0.02,0.02,0.01/)  ! Diag coef`s of corr matrix of Obs
real*4  :: das_dif_tol(V_N) = (/2.5, 1.5, 0.3, 0.3, 6.00/)   ! If abs(var_b - var_o) > LAND_TOL then don`t assimilate this point

logical,parameter   :: used_enkf    (V_N) = (/1,1,1,1,0/)   ! logical parameter shows which vars used for Assimilation (T,S,SSL,AICE,SST)
logical             :: sended_enkf  (V_N) = (/0,0,0,0,0/)   ! logical var that shows what vars were sended to DAS (T,S,SSL,AICE,SST)
logical             :: recv_enkf    (V_N) = (/0,0,0,0,0/)   ! logical var that shows what vars need to recv  (T,S,SSL,AICE,SST)
type(Event)         :: recv_enkf_ev (V_N)                   ! Save events for Recv
type(Event)         :: send_das_ev  (V_N)                   ! Save events for Send



!========================================================================================
! EnOI_module
!========================================================================================
!=== Taken from namelist das_list.in === start
integer :: nrens = 0                        ! Number of elementes of Ensemble
integer :: ens_list_size                    ! Number of CP for getting Ens

real*4  :: das_mask_val                     ! Remove strange val from CP files
character(NAME_LEN) :: fname_ens_prefix     ! Prefix for ENS files i.e. 'data/EnKF_g00/OCN_g00_CP_'
!=== Taken from namelist das_list.in === stop

integer :: ndim_ens                         ! = ndim_ens_T + ndim_ens_S + ...
integer :: nrobs_ens                        ! = nrobs_ens_T + nrobs_ens_S + ...

integer :: ndim_ens_T                       ! size of 1d vector in (line) ~ ocn_il * ( ij_hi - ij_lo + 1)
integer :: nrobs_ens_T                      ! size of 1d-obs vector 

integer :: ndim_ens_S                       ! size of 1d vector in (line) ~ ocn_il * ( ij_hi - ij_lo + 1)
integer :: nrobs_ens_S                      ! size of 1d-obs vector 

integer :: ndim_ens_SSL                     ! size of 1d vector in (line) ~ ocn_il * ( ij_hi - ij_lo + 1)
integer :: nrobs_ens_SSL                    ! size of 1d-obs vector 
real*4  :: mean_b_SSL                       ! mean of background SSL 1. SSL_b = SSL_b - mean_b_SSL 2. assimilate 3. SSL_a = SSL_a + mean_b_SSL

integer :: ndim_ens_AICE                    ! size of 1d vector in (line) ~ ocn_il * ( ij_hi - ij_lo + 1)
integer :: nrobs_ens_AICE                   ! size of 1d-obs vector 


real*4, allocatable :: psi_ens(:)           ! state vector (forecast -> vector) (ndim_ens)
real*4, allocatable :: A_ens(:,:)           ! Ensemble matrix (ndim_ens, nrens)
real*4, allocatable :: D_ens(:)             ! matrix  holding observation innovations (nrobs_ens)
real*4, allocatable :: S_ens(:,:)           ! matrix holding HA' (nrobs_ens, nrens)
real*4, allocatable :: R_ens(:,:)           ! Error covariance matrix for observations ( nrobs_ens, nrobs_ens) and then for covariance


real*4, allocatable :: z_axis (:)           ! Z leves for Argo data analyse, get from obs (nl)
integer,allocatable :: nl_axis(:)           ! Z leves(from 49) for Argo data analyse, get from obs (nl)

!Time Calc Measured
real*4 das_time_total, das_time_strt, das_time_end
type(DateTime) :: time_get                  ! DateTime for current and prev times

type(DateTime), allocatable :: time_ens(:)      ! Arrays of ensamble elements dates
end module

!========================================================================================
! Derived class for AOI service.
!========================================================================================

module service_aoi_module

use utils_module
use actions_module
use service_module

use communicator_ga_module
use component_splitter_module
use model_info_module
use file_handler_nc_module
use cpl_comp_clients_module
use decomposition_module

#ifdef DEBUG
use debug_module
#endif

implicit none
type, extends(Service) :: service_aoi
private

 contains  
    !=======================
    ! ===== PUBLIC API =====
    procedure, public :: init => init_aoi
    procedure, public :: destroy => destroy_aoi
    procedure, public :: handle_request => handle_aoi
    procedure, public :: after_init => after_init_aoi
    procedure, private :: handle_send_das                   ! Get all arrays for data assimilation, common for all methods and data types
    procedure, private :: common_switcher                   ! Common var(T,S,SSL,AICE) np, nl size switcher, used in reg_recv_enkf, send_switcher
    procedure, private :: reg_recv_enkf                     ! Assimilate Data EnKF(EnOI) method
    procedure, private :: send_switcher                     ! Switch vars name for differnt data
    procedure, private :: process_enkf                      ! Get Ens, Obs, Assimilate Data EnKF(EnOI) method (make_enkf)

    procedure, private :: find_error_norm                   ! Find norm of error
    procedure, private :: get_background                    ! Get Background array of model
    
    procedure, private :: get_obs_data                      ! Read obs data from file ( after preproc)
    procedure, private :: save_dif_data                     ! Save file with markers for nice pictures
    procedure, private :: das_init_sizes                    ! Set Sizes for DAS, get NAMELIST, init log file
    
    procedure, private :: get_one_ens                       ! EnOI: read one element of enf from file
    procedure, private :: read_ens                          ! EnOI: Initial Read A_ens matrix
    procedure, private :: get_ens_date_from_list            ! EnOI: generate date and filename for ens element
    procedure, private :: make_ens_vars                     ! EnOI: make matrix D,S,psi of ensemble
    procedure, private :: send_ens_vars                     ! EnOI: send psi to model
    procedure, private :: make_enkf                         ! EnOI: make EnKF data ass using external call
    procedure, private :: allocate_enoi_arr_nrobs           ! EnOI: allocate obs arrays for EnOI
    procedure, private :: allocate_enoi_arr_ndim            ! EnOI: allocate ndim arrays for EnOI
    procedure, private :: deallocate_enoi_arr               ! EnOI: deallocate work arrays for EnOI
    procedure, private :: make_1Dvar_from_3dvar             ! Copy 1d Var to 3d Var
    procedure, private :: make_3dvar_from_1Dvar             ! Copy 3d Var to 1d Var
    ! ===== PUBLIC API =====
    !=======================    
end type

 CONTAINS

subroutine init_aoi(this)
    class(service_aoi) :: this
    write(*,*), "DAS INIT"
end subroutine

subroutine destroy_aoi(this)
    class(service_aoi) :: this
    
end subroutine

subroutine after_init_aoi(this)
    use aoi_module
    class(service_aoi) :: this
    if( ALL(sended_enkf .EQV. used_enkf)) then
        call this % allocate_enoi_arr_ndim
        if(ANY(recv_enkf)) call this % read_ens
        write(*,'(a,<nrens>f8.2)')," A_ens(T)    = ",A_ens(1, 1:nrens)        
        write(*,'(a,<nrens>f8.2)')," A_ens(AICE) = ",A_ens(ndim_ens, 1:nrens)        
    endif       
    
end subroutine
 
!========================================================================================

subroutine handle_aoi(this, ev)
    use aoi_module
    use datetime_module
    implicit none
    class(service_aoi) :: this
    type(Event), intent(in) :: ev
#define _VAR_SWITCHER_( _VAR_NAME_ ) \
                            var_o_name      = TRIM(_VAR_NAME_//"_o");     \
                            x12_o_name      = TRIM(_VAR_NAME_//"_x12_o"); \
                            ij_o_name       = TRIM(_VAR_NAME_//"_ij_o");  \
                            var_b_in_o_name = TRIM(_VAR_NAME_//"_b_in_o");\
                            var_a_in_o_name = TRIM(_VAR_NAME_//"_a_in_o");\
                            var_o_dif_name  = TRIM(_VAR_NAME_//"_o_dif"); \
                            var_b_name      = TRIM(_VAR_NAME_//"_b")

    select case (ev % action())
        case( "SEND_DAS")
            if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
                call this % try_register_service_ga(ev)
                !Take size from OCN model (TBD from ALL) and namelist
                if (nrens == 0) call this % das_init_sizes(ev) 
                call this % send_switcher(ev)
#ifdef DEBUG
                write(*,*),"DAS: Register array ",TRIM(ev % ga_name()), TRIM(ev % time())
#endif
            elseif (ev % ga_name()  == "OCN_t_f_SEND_DAS" .or. &
                    ev % ga_name()  == "OCN_s_f_SEND_DAS" .or. &
                    ev % ga_name()  == "OCN_aice_SEND_DAS" .or. &
                    ev % ga_name()  == "ICE_aice_SEND_DAS" .or. &
                    ev % ga_name()  == "OCN_ssl_f_SEND_DAS" ) then
!===============================================================================
!           Select which var we try to assimilate
!===============================================================================
                write(*,*),"DAS: SEND_DAS ",TRIM(ev % time_rec()), TRIM(ev % time())
                if (  time_get == DateTime(MERGE(ev % time_rec(), ev % time(), ev % time_rec() /= "")) ) return
                time_get = DateTime(MERGE(ev % time_rec(), ev % time(), ev % time_rec() /= ""))
                !if ( time_get == ExpInfo % start_date()) return
                write(*,*),TRIM(ev % ga_name()), " SEND_DAS ", TRIM(send_das_ev (1) % ga_name())
                do while (TRIM(this % comm_ % get_info(send_das_ev (1) % ga_name(), COMM_GA_STATUS)) /= "full"); end do
                write(*,*),TRIM(ev % ga_name()), " SEND_DAS ", TRIM(send_das_ev (2) % ga_name())
                do while (TRIM(this % comm_ % get_info(send_das_ev (2) % ga_name(), COMM_GA_STATUS)) /= "full"); end do
                write(*,*),TRIM(ev % ga_name()), " SEND_DAS ", TRIM(send_das_ev (3) % ga_name())
                do while (TRIM(this % comm_ % get_info(send_das_ev (3) % ga_name(), COMM_GA_STATUS)) /= "full"); end do
                write(*,*),TRIM(ev % ga_name()), " SEND_DAS ", TRIM(send_das_ev (4) % ga_name())
                do while (TRIM(this % comm_ % get_info(send_das_ev (4) % ga_name(), COMM_GA_STATUS)) /= "full"); end do
                call this % handle_send_das(ev)
            else
                write(*,*),"DAS: SEND_DAS wrong name",ev % ga_name()
                stop(3)
            endif

        case( "RECV_ENKF")
            if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
                call this % try_register_service_ga(ev)
                call list_of_ensemble_dates
                call this % reg_recv_enkf(ev)
            else
                write(*,*),"DAS: RECV_ENKF ", ev % ga_name()
                if(TRIM(this % comm_ % get_info(ev % ga_name(), COMM_GA_STATUS)) == "full") then
                    write(*,*),"DAS: RECV_ENKF full ", ev % ga_name()
                    return
                endif
                write(*,*),"DAS: RECV_ENKF ",sended_enkf, recv_enkf
                if ( all(sended_enkf (1:4)) ) then
                    call this % process_enkf()
                else
                    return
                endif
                
                write(*,*),"AOI: RECV_ENKF free", TRIM(ev % ga_name())
                call this % comm_ % sync( aoi_service_id )
            endif
        case("ERROR")
            call this % report_handle(ev)
!             call sleep(1)
            call exit(1)

    end select

end subroutine

!======================================
!   Handle send_das
!======================================
subroutine handle_send_das(this, ev )
    use aoi_module
    implicit none
    class(service_aoi) :: this

    type(Event),            intent(in) :: ev
    integer i
#ifdef DEBUG
    write(*,*),"DAS: handle_send_das",TRIM(ev % ga_name())
#endif

    call this % comm_ % sync(aoi_service_id)
    
    call cpu_time(das_time_strt)
    do i =1, size(used_enkf)
        if( .NOT. used_enkf(i) ) cycle
        call this % comm_ % sync(aoi_service_id)
#ifdef DEBUG
        write(*,*),"DAS: handle_send_das ", TRIM(send_das_ev(i) % ga_name())
#endif  
        call this % send_switcher (send_das_ev(i)) 
!==========================
!   Get Background
!   size (1:ocn_il,ij_lo:ij_hi,1:nl)
!==========================
#ifdef DEBUG
        write(*,*), "DAS: get background"
#endif
        call this % get_background(send_das_ev(i))
#ifdef DEBUG
        write(*,*), "DAS: get background: DONE!"
        write(*,*), "DAS: get obs", TRIM(fname_obs)
#endif
        call this % get_obs_data    (send_das_ev(i))
#ifdef DEBUG
        write(*,*), "DAS: get obs and projection: DONE!"
#endif
        call cpu_time(das_time_end)
        das_time_total = das_time_end - das_time_strt
#ifdef DEBUG
        write(*,*), "DAS: save_dif_dataa: DONE!"
#endif
#define _DEC_SAVE( _VAR_ ) \
    np\_VAR_    = np ;              nl\_VAR_        = nl ; \
    np_lo\_VAR_ = np_lo;            np_hi\_VAR_     = np_hi ; \
    

!===============================================================================
!           Save domain decomposition for particular var
!===============================================================================
        select case ( send_das_ev(i) % ga_name() )
            case("OCN_t_f_SEND_DAS")
#ifdef ARGO_T
            _DEC_SAVE (_T)
#endif
            case("OCN_s_f_SEND_DAS")
#ifdef ARGO_S
            _DEC_SAVE (_S)
#endif
            case("OCN_ssl_f_SEND_DAS")
#ifdef SAT_SSL
            _DEC_SAVE (_SSL)
#endif
            case("OCN_aice_SEND_DAS","ICE_aice_SEND_DAS")
#ifdef SAT_AICE
                _DEC_SAVE (_AICE)
#endif
        
#ifdef SAT_SST
            _DEC_SAVE (_SST)
#endif
        end select
#ifdef DEBUG
        write(*,*), "DAS: DEC_SAVE: DONE!"
#endif
    enddo
end subroutine


!============================================================================
!   Common Switcher
!============================================================================
subroutine common_switcher( this, ev )
    use aoi_module
    class(service_aoi)      :: this
    type(Event),intent(in)  :: ev
#ifdef DEBUG
    write(*,*),aoi_rnk,"DAS: commin_switcher ",TRIM(ev % ga_name())
#endif
    yyyymmdd = time_get % to_string()
    select case ( ev % ga_name() )
        case("OCN_t_f_SEND_DAS","OCN_t_dif_RECV_ENKF","OCN_t_f_RECV_ENKF")
            fname_obs           = TRIM('data/ARGO/T_'//yyyymmdd//"_merge.nc")
            fname_dif           = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/T_'//yyyymmdd//"_dif.nc")
            fname_DG            = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/T_'//yyyymmdd//"_DG.nc")
             _VAR_SWITCHER_( "T" )
            np = np_T ; nl = nl_T ;
            var_id = 1;
        case("OCN_s_f_SEND_DAS","OCN_s_dif_RECV_ENKF","OCN_s_f_RECV_ENKF")
            fname_obs           = TRIM('data/ARGO/S_'//yyyymmdd//"_merge.nc")
            fname_dif           = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/S_'//yyyymmdd//"_dif.nc")
            fname_DG            = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/S_'//yyyymmdd//"_DG.nc")
            _VAR_SWITCHER_( "S" )
            np = np_S ; nl = nl_S ;
            var_id = 2;
        case("OCN_ssl_f_SEND_DAS","OCN_ssl_dif_RECV_ENKF","OCN_ssl_f_RECV_ENKF")
            fname_obs           = TRIM('data/AVISO/SSL_'//yyyymmdd//"_aviso.nc")
            fname_dif           = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/SSL_'//yyyymmdd//"_dif.nc")
            fname_DG            = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/SSL_'//yyyymmdd//"_DG.nc")
            _VAR_SWITCHER_( "SSL" )
            np = np_SSL ; nl = nl_SSL ;
            var_id = 3;
        case("OCN_aice_SEND_DAS","ICE_aice_SEND_DAS","OCN_aice_dif_RECV_ENKF","OCN_aice_RECV_ENKF", "ICE_aice_dif_RECV_ENKF","ICE_aice_RECV_ENKF")
            fname_obs           = TRIM('data/AICE/AICE_'//yyyymmdd//"_aice.nc")
            fname_dif           = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/AICE_'//yyyymmdd//"_dif.nc")
            fname_DG            = TRIM('data/'//TRIM(ExpInfo % exp_name())//'/AICE_'//yyyymmdd//"_DG.nc")
            _VAR_SWITCHER_( "AICE" )
            np = np_AICE ; nl = nl_AICE ;
            var_id = 4;
    end select

end subroutine
!============================================================================
!   Send Switcher
!============================================================================
subroutine send_switcher( this, ev )
    use aoi_module
    class(service_aoi)      :: this
    type(Event),intent(in)  :: ev
#ifdef DEBUG
    write(*,*),aoi_rnk,"DAS: send_switcher ",TRIM(ev % ga_name())
#endif
    call this % common_switcher(ev)
    select case ( ev % ga_name() )
        case("OCN_t_f_SEND_DAS")
            sended_enkf (1) = 1 
            send_das_ev (1) = ev
             _VAR_SWITCHER_( "T" )
        case("OCN_s_f_SEND_DAS")
            sended_enkf (2) = 1
            send_das_ev (2) = ev
            _VAR_SWITCHER_( "S" )
        case("OCN_ssl_f_SEND_DAS")
            sended_enkf (3) = 1
            send_das_ev (3) = ev
            _VAR_SWITCHER_( "SSL" )
        case("OCN_aice_SEND_DAS","ICE_aice_SEND_DAS")
            sended_enkf (4) = 1
            send_das_ev (4) = ev
            _VAR_SWITCHER_( "AICE" )
    end select

end subroutine
!============================================================================
!   Registration recv_enkf
!============================================================================

subroutine reg_recv_enkf( this, ev )
    use aoi_module
    class(service_aoi)      :: this
    type(Event),intent(in)  :: ev
#ifdef DEBUG
    write(*,*),"DAS: reg_recv_enkf ",TRIM(ev % ga_name())
#endif 
    call this % common_switcher(ev)
    call this % comm_ % put_info( ev % ga_name() , COMM_GA_STATUS, "free")
    call this % comm_ % sync( aoi_service_id )
    select case ( ev % ga_name() )
        case("OCN_t_dif_RECV_ENKF","OCN_t_f_RECV_ENKF")
            recv_enkf   (1) = 1
            recv_enkf_ev(1) = ev
        case("OCN_s_dif_RECV_ENKF","OCN_s_f_RECV_ENKF")
            recv_enkf   (2) = 1
            recv_enkf_ev(2) = ev
        case("OCN_ssl_dif_RECV_ENKF","OCN_ssl_f_RECV_ENKF")
            recv_enkf   (3) = 1
            recv_enkf_ev(3) = ev
        case("OCN_aice_dif_RECV_ENKF","OCN_aice_RECV_ENKF", "ICE_aice_dif_RECV_ENKF","ICE_aice_RECV_ENKF")
            recv_enkf   (4) = 1
            recv_enkf_ev(4) = ev
        case default
            write(*,*),"ENKF: Incorrect var name ", ev % ga_name()
            stop(4)
    end select
#ifdef DEBUG
    write(*,'(a,a,10l)')," EnOI reg_recv_enkf ",TRIM(ev % ga_name()), recv_enkf, used_enkf
    write(*,'(l,a,5l)'), ALL(recv_enkf .EQV. used_enkf), " EnOI reg_recv_enkf ", (recv_enkf .EQV. used_enkf)
#endif


end subroutine

subroutine read_ens(this)
    use aoi_module
    use datetime_module
    implicit none
    class(service_aoi)      :: this
    type(DateTime) :: time_ens_get
    character(NAME_LEN) :: fname_ens = ""
    integer i
    do i=1, nrens
        call this % get_ens_date_from_list(time_ens_get,fname_ens, i )
        write(*,*), "Warning: slow read from file", fname_ens

        if ( used_enkf(1) ) then
            _VAR_SWITCHER_( "T" )
            var_id = 1
            var_cp_name =TRIM("t_cp")
            call this % get_one_ens( fname_ens, time_ens_get, i, var_cp_name, &
                1, &
                ndim_ens_T)
            write(*,*),"DAS: process_enkf T OK!"
        endif

        if ( used_enkf(2) ) then
            _VAR_SWITCHER_( "S" )
            var_id = 2
            var_cp_name =TRIM("s_cp")
            call this % get_one_ens( fname_ens, time_ens_get, i, var_cp_name, &
                ndim_ens_T + 1, &
                ndim_ens_T + ndim_ens_S )
            sended_enkf (2) = 0
            write(*,*),"DAS: process_enkf T OK!"
        endif
        if ( used_enkf(3) ) then
            _VAR_SWITCHER_( "SSL" )
            var_id = 3
            var_cp_name =TRIM("ssl_cp")
            call this % get_one_ens( fname_ens, time_ens_get, i, var_cp_name, &
                ndim_ens_T + ndim_ens_S + 1, &
                ndim_ens_T + ndim_ens_S + ndim_ens_SSL)
        endif
        if ( used_enkf(4) ) then
            _VAR_SWITCHER_( "AICE" )
            var_id = 4
            var_cp_name =TRIM("aice_cp")
            call this % get_one_ens( fname_ens, time_ens_get, i, var_cp_name, &
                ndim_ens_T + ndim_ens_S + ndim_ens_SSL + 1, &
                ndim_ens_T + ndim_ens_S + ndim_ens_SSL + ndim_ens_AICE)
            
        endif
    enddo

end subroutine


!============================================================================
!   process_enkf
!============================================================================

subroutine process_enkf( this )
    use aoi_module
    class(service_aoi) :: this
#ifdef DEBUG
    write(*,*),"DAS: process_enkf ", sended_enkf, recv_enkf
#endif

#define _GET_ENS( _CP_NAME_, _LO_D, _HI_D, _LO_O, _HI_O, _VAR_) \
    var_cp_name = TRIM( _CP_NAME_ ); \
    call this % make_ens_vars(  A_ens  ( ( _LO_D ) : ( _HI_D ), 1:nrens), \
                                S_ens  ( ( _LO_O ) : ( _HI_O ), 1:nrens), \
                                D_ens  ( ( _LO_O ) : ( _HI_O ) ), \
                                psi_ens( ( _LO_D ) : ( _HI_D ) ), \
                                ( _LO_D ) , ( _HI_D ), ( _LO_O ), ( _HI_O ), \
                                (np_lo\_VAR_), (np_hi\_VAR_), \
                                (nl\_VAR_) )

    call cpu_time(das_time_strt)
    call this % allocate_enoi_arr_nrobs

    if ( sended_enkf(1) ) then
        _VAR_SWITCHER_( "T" )
        var_id = 1
        _GET_ENS("t_cp",1,ndim_ens_T,1,nrobs_ens_T, _T)
        sended_enkf (1) = 0
        write(*,*),"DAS: process_enkf T OK!"
    endif

    if ( sended_enkf(2) ) then
        _VAR_SWITCHER_( "S" )
        var_id = 2
        _GET_ENS("s_cp",ndim_ens_T+1,ndim_ens_T+ndim_ens_S,nrobs_ens_T+1,nrobs_ens_T+nrobs_ens_S, _S)
        sended_enkf (2) = 0
        write(*,*),"DAS: process_enkf T OK!"
    endif
    if ( sended_enkf(3) ) then
        _VAR_SWITCHER_( "SSL" )
        var_id = 3
        _GET_ENS("ssl_cp",ndim_ens_T+ndim_ens_S+1,ndim_ens_T + ndim_ens_S + ndim_ens_SSL, nrobs_ens_T+nrobs_ens_S+1, nrobs_ens_T+nrobs_ens_S+nrobs_ens_SSL , _SSL)
        sended_enkf (3) = 0
    endif
    if ( sended_enkf(4) ) then
        _VAR_SWITCHER_( "AICE" )
        var_id = 4
        _GET_ENS("aice_cp",ndim_ens_T+ndim_ens_S+ndim_ens_SSL+1,ndim_ens_T + ndim_ens_S + ndim_ens_SSL + ndim_ens_AICE, &
        nrobs_ens_T+nrobs_ens_S+nrobs_ens_SSL+1, nrobs_ens_T+nrobs_ens_S+nrobs_ens_SSL+nrobs_ens_AICE , _AICE)
        sended_enkf (4) = 0
    endif

#ifdef DEBUG
        write(*,'(a,3i7,l5,3i7)') , "EnKF: make_enkf ",ndim_ens, nrobs_ens, nrens, sended_enkf, nrobs_ens_T, nrobs_ens_SSL, nrobs_ens_AICE
#endif
    call this % make_enkf()


#define _SEND_ENS( _LO_D, _HI_D, _VAR_, _KL_ , _EV_ ) \
    call this % send_ens_vars( psi_ens( ( _LO_D ) : ( _HI_D )), ( _LO_D ) , ( _HI_D ), (np\_VAR_), (nl\_VAR_), ( _KL_ ), _EV_ )

    if ( recv_enkf(1) ) then
        _VAR_SWITCHER_( "T" )
        _SEND_ENS(1,ndim_ens_T,_T, ocn_kl, recv_enkf_ev(1))
    endif
    
    if ( recv_enkf(2) ) then
        _VAR_SWITCHER_( "S" )
        _SEND_ENS(ndim_ens_T+1,ndim_ens_T+ndim_ens_S,_S,  ocn_kl, recv_enkf_ev(2))
    endif
    if ( recv_enkf(3) ) then
        _VAR_SWITCHER_( "SSL" )
        _SEND_ENS(ndim_ens_T+ndim_ens_S+1,ndim_ens_T+ndim_ens_S+ndim_ens_SSL,_SSL, 1, recv_enkf_ev(3))
    endif
    if ( recv_enkf(4) ) then
        _VAR_SWITCHER_( "AICE" )
        _SEND_ENS(ndim_ens_T+ndim_ens_S+ndim_ens_SSL+1,ndim_ens_T+ndim_ens_S+ndim_ens_SSL+ndim_ens_AICE,_AICE, 1, recv_enkf_ev(4))
    endif
    
    call this % deallocate_enoi_arr
    write(*,'(a,3f9.1)')," DAS: EnOI time",das_time_strt, das_time_end, das_time_total

end subroutine

!======================================
!   Get var_b_name
!======================================

#define _GET_BACKGROUND_( _ARR_ , _GA_NAME_ , _NL_)                                       \
        dec_bg   = Decomposition(ocn_il, ocn_jl, _NL_ , "1D", CompSplitter % comm_local_size(CompSplitter % i_am()), CompSplitter % rank_local() ); \
        lo = dec_bg % lower_bound_3D();  hi = dec_bg % upper_bound_3D(); \
        call this % comm_ % init_array(arr_name = var_b_name, datatype = "real4", dimnum = "3D", holder_id = aoi_service_id, holder_decomp = dec_bg); \
        if (.NOT. allocated( _ARR_ )) allocate( _ARR_( lo(1):hi(1), lo(2):hi(2),lo(3):hi(3) ) ); \
        call this % comm_ % get( _GA_NAME_ , lo, hi, _ARR_); \
        call this % comm_ % sync(aoi_service_id); \
        if(CompSplitter % is_first_rank()) call this % comm_ % put_info( _GA_NAME_ , COMM_GA_STATUS, "free")

subroutine get_background(this, ev )
    use decomposition_module
    use datetime_module
    use const_module
    use aoi_module
    class(service_aoi) :: this
    type(Event),            intent(in) :: ev
    type(Decomposition) :: dec_bg
    real*4, allocatable:: var_tmp4(:,:,:)                                       ! For BG and CP
    real*8, allocatable:: var_tmp8(:,:,:)                                       ! For BG and CP
    integer :: lo(3), hi(3)
   
#ifdef DEBUG
    write(*,*),"DAS: Start get_background ", TRIM(ev % ga_name()), TRIM(var_b_name), nl
#endif
    select case ( ev % ga_name() )
        case("OCN_t_f_SEND_DAS","OCN_s_f_SEND_DAS")        
            if( main_kind == 8) then
                _GET_BACKGROUND_( var_tmp8 , ev % ga_name(), ocn_kl )
                write(*,*),"DAS: Get_background ", ev % ga_name(), ocn_kl
                allocate(var_tmp4 ( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) )
                         var_tmp4 ( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) = &
                real(    var_tmp8 ( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) )
                call this % comm_ % put(var_b_name, lo , hi , var_tmp4)
                deallocate( var_tmp4, var_tmp8 )
            else
                write(*,*),"ENKF: TS use real8", TRIM(ev % ga_name())
                stop(10)
            endif
        case("OCN_ssl_f_SEND_DAS")
            _GET_BACKGROUND_( var_tmp8 , ev % ga_name(), 1 )
            allocate(var_tmp4( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) )
                     var_tmp4( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) = &
            real(    var_tmp8( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) )
            !mean_b_SSL = 0d0; k = 0
            !do i=1, ocn_il
            !  do j=1, ocn_jl ! HAKEL NAGEL
            !    if( abs( var_tmp8 (i, j, 1) ) < das_mask_val ) then
            !        mean_b_SSL =  var_tmp8 (i, j, 1) + mean_b_SSL
            !        k = k + 1
            !    endif
            !  enddo
            !enddo
            !mean_b_SSL = mean_b_SSL / dble(k)
            !call MPI_ALLREDUCE (mean_b_SSL_local, mean_b_SSL, 1, MPI_DOUBLE_PRECISION, MPI_SUM, aoi_comm, ierr)
            !mean_b_SSL = mean_b_SSL / dble(ocn_il*ocn_jl)

            !write(*,*),"DAS: Mean SSL ", mean_b_SSL
            !var_tmp8 = var_tmp8 - mean_b_SSL
            call this % comm_ % put(var_b_name, lo , hi , var_tmp4)
            deallocate( var_tmp4, var_tmp8 )
        case("OCN_aice_SEND_DAS","ICE_aice_SEND_DAS")
            _GET_BACKGROUND_( var_tmp4 , ev % ga_name(), 1 )
            call this % comm_ % put(var_b_name, lo , hi , var_tmp4)
            deallocate( var_tmp4 )
        case default
            write(*,*),"SEND_DAS: Incorrect var name", TRIM(ev % ga_name())
            stop(4)
    end select
    call this % comm_ % sync(aoi_service_id);

#ifdef DEBUG
    write(*,*),aoi_rnk, "Get_background OK ", TRIM(ev % ga_name()) ,TRIM(var_b_name)
#endif

end subroutine



!==================
!   Allocate work arrays for limits
!   And get some parallel parameters
!==================

subroutine das_init_sizes(this, ev)
    use decomposition_module
    use datetime_module
    use aoi_module
    implicit none
    class(service_aoi) :: this
    type(Event), intent(in) :: ev
    type(Decomposition) :: ocn_dec
    integer i
    logical file_exists

    namelist /DAS_MAIN/     nrens, ens_list_size, &
                            das_mask_val, &
                            fname_ens_prefix
    write(*,*), "DAS Init SIZES",nrens
    logfile = TRIM('data/das_err.log.'//ExpInfo % exp_name())
    inquire(file = "./configure/das_list.in", exist = file_exists)
    if (file_exists) then
        open(101, file= "./configure/das_list.in")
        read(101, nml = DAS_MAIN)
        close(91)
    else
        stop 'ERROR: das_list.in not found'
    end if
    write(*,*), "DAS Init SIZES",nrens, ens_list_size,TRIM(fname_ens_prefix)
    aoi_size = CompSplitter %  comm_local_size()
    aoi_rnk  = CompSplitter %  rank_local () +  1
    aoi_comm = CompSplitter %  comm_local ()
    aoi_service_id = CompSplitter % i_am_id()

    ocn_dec = ModelsInfo(CompSplitter % comp_id( ev % src() )) % decomp()
    ocn_il = ocn_dec % i_size()
    ocn_jl = ocn_dec % j_size()
    ocn_kl = 49 !ocn_dec % k_size()
    open (337,file=logfile, POSITION='APPEND')
    write(337,2002), "yyyy-mm-dd", "L2er", " var" , " L2 ","   np", "   nl", (i, i=1, 5), " aoi", "das_time"
2002        format (a10,x,3(a4,x),2(a5,x),<5>i6,x,a4,x,a8)
    close(337)
end subroutine

!==================
!   Allocate work arrays for EnOI
!   and calculate sizes
!==================

subroutine allocate_enoi_arr_ndim(this)
    use aoi_module
    implicit none
    class(service_aoi) :: this

    ndim_ens_T     =   0
    ndim_ens_S     =   0
    ndim_ens_SSL   =   0
    ndim_ens_AICE  =   0
#ifdef DEBUG
    write(*,*), " allocate_enoi_arr_ndim "
#endif
    if ( used_enkf(1) ) ndim_ens_T     =   ocn_jl * ocn_il * ocn_kl
    if ( used_enkf(2) ) ndim_ens_S     =   ocn_jl * ocn_il * ocn_kl
    if ( used_enkf(3) ) ndim_ens_SSL   =   ocn_jl * ocn_il
    if ( used_enkf(4) ) ndim_ens_AICE  =   ocn_jl * ocn_il
    ndim_ens       =   ndim_ens_S  + ndim_ens_T  + ndim_ens_SSL  + ndim_ens_AICE

#ifdef DEBUG
    write(*,'(a,i7)')," ndim_ens        = ", ndim_ens
    write(*,'(a,i7)')," ndim_ens_T      = ", ndim_ens_T
    write(*,'(a,i7)')," ndim_ens_S      = ", ndim_ens_S
    write(*,'(a,i7)')," ndim_ens_AICE   = ", ndim_ens_AICE
    write(*,'(a,i7)')," ndim_ens_SSL    = ", ndim_ens_SSL
    write(*,'(a,i7)')," nrens =    ", nrens
#endif
    allocate(   psi_ens ( ndim_ens) )                   ! state vector (forecast -> vector) (ndim_ens)
    allocate(   A_ens   ( ndim_ens, nrens) )            ! Ensemble matrix (ndim_ens, nrens)
end subroutine

!==================
!   Allocate work arrays for EnOI
!   and calculate sizes
!==================

subroutine allocate_enoi_arr_nrobs(this)
    use aoi_module
    implicit none
    class(service_aoi) :: this

#ifdef DEBUG
    write(*,*), " EnOI: allocate_enoi_arr_nrobs"
#endif
    nrobs_ens_SSL   =   np_SSL  
    nrobs_ens_T     =   np_T * nl_T
    nrobs_ens_S     =   np_S * nl_S
    nrobs_ens_AICE  =   np_AICE 
    nrobs_ens = nrobs_ens_S + nrobs_ens_T + nrobs_ens_SSL + nrobs_ens_AICE 

#ifdef DEBUG
    write(*,'(a,i7)'), " nrobs_ens       = ", nrobs_ens
    write(*,'(a,i7)'), " nrobs_ens_T     = ", nrobs_ens_T
    write(*,'(a,i7)'), " nrobs_ens_S     = ", nrobs_ens_S
    write(*,'(a,i7)'), " nrobs_ens_AICE  = ", nrobs_ens_AICE
    write(*,'(a,i7)'), " nrobs_ens_SSL   = ", nrobs_ens_SSL
    write(*,'(a,i7)'), " nrens           = ", nrens
#endif

    allocate(   D_ens   ( nrobs_ens) )                  ! matrix  holding observation innovations
    allocate(   S_ens   ( nrobs_ens, nrens) )           ! matrix holding HA' (nrobs_ens, nrens)
    allocate(   R_ens   ( nrobs_ens, nrobs_ens) )       ! Error covariance matrix for observations ( nrobs_ens, nrobs_ens) and then for covariance
    R_ens = 0
end subroutine
!==================
!   Deallocate work arrays for EnOI
!==================

subroutine deallocate_enoi_arr(this)
    use aoi_module
    class(service_aoi) :: this

    !deallocate(   psi_ens )         ! state vector (forecast -> vector) (ndim_ens)
    !deallocate(   A_ens   )         ! Ensemble matrix (ndim_ens, nrens)
    deallocate(   D_ens   )         ! matrix  holding observation innovations
    deallocate(   S_ens   )         ! matrix holding HA' (nrobs_ens, nrens)
    deallocate(   R_ens   )         ! Error covariance matrix for observations ( nrobs_ens, nrobs_ens) and then for covariance
end subroutine

!==================
!   Read obs data in 2D GA
!   x12_o(1:2,1:np) and var_o(1:nl,1:np)
!==================

subroutine get_obs_data(this, ev)
    use aoi_module
    use decomposition_module
    use datetime_module
    implicit none
    class(service_aoi) :: this
    type(Event), intent(in) :: ev
    type(Decomposition) :: dec_var, dec_x12
    type(FileHandler_NC) :: hdr_file
    real*4, allocatable:: var_tmp4(:,:,:)           ! For BG and CP
    real*4, allocatable:: x12_o(:,:), var_o(:,:)    ! For OBS data
    integer*8, allocatable:: ij_o(:,:)              ! For OBS data
    real*4, allocatable:: var_b_in_o(:,:)           ! For B proj in OBS
    integer :: lo(3), hi(3), n_ok, n, k
    real*4 mean_o_SSL
#ifdef DEBUG
    write(*,*),"Start Get_obs data ", TRIM(ev % ga_name()),  TRIM(fname_obs)
#endif
    call hdr_file % open_file(fname_obs, CompSplitter % comm_local())
    np = hdr_file % get_dim_size("np")   !222 !
    nl = hdr_file % get_dim_size("nl")
    
#ifdef DEBUG
    write(*,*),"DAS: obs size np(init) nl ",np, nl
#endif
!   Read var_o
    allocate( var_o(1:nl, 1:np), var_b_in_o(1:nl,1:np))
    call hdr_file % open_var("var_o")
    call hdr_file % get(var_o, (/1, 1 /), (/nl, np /), time_get) 
    
#ifdef DEBUG
    write(*,*), "DAS: get", var_o_name
#endif

!x12_o is 2D array
!x1_o and x2_o is 1D array
    allocate( x12_o(1:2, 1:np), ij_o( 1:2, 1:np) ) 
    call hdr_file % open_var("x1_o")
    call hdr_file % get(x12_o(1,1:np), (/1 /), (/np /) )
    call hdr_file % open_var("x2_o")
    call hdr_file % get(x12_o(2,1:np), (/1 /), (/np /))

    call hdr_file % open_var("i_o")
    call hdr_file % get(ij_o(1,1:np), (/1 /), (/np /) )
    call hdr_file % open_var("j_o")
    call hdr_file % get(ij_o(2,1:np), (/1 /), (/np /))

!Z AXIS
    if ( nl > 1 ) then
        if(.NOT. allocated(z_axis)) then
            allocate ( z_axis(1:nl), nl_axis(1:nl) )
            call hdr_file % open_var("z_o")
            call hdr_file % get(z_axis(1:nl), (/1/), (/ nl /))        
            call hdr_file % open_var("nl_o")
            call hdr_file % get(nl_axis(1:nl), (/1/),(/ nl /))        
        endif
    endif    
#ifdef DEBUG
    write(*,*),'get_projection ',TRIM(var_b_name), das_dif_tol(var_id),' start', MERGE(ocn_kl, 1, nl > 1) , nl
#endif
    allocate(   var_tmp4 ( 1:ocn_il, 1:ocn_jl, 1 : MERGE(ocn_kl, 1, nl > 1)  ) )
    call this % comm_ % get(var_b_name, (/ 1, 1, 1 /), (/ ocn_il, ocn_jl, MERGE(ocn_kl, 1, nl > 1) /), var_tmp4)

    if( var_id == 3) then
        n_ok = 1
        mean_b_SSL = 0
        mean_o_SSL = 0
        do n = 1, np
            if (    var_tmp4 ( ij_o(1, n), ij_o(2,n), 1) == 0.0 .or. & 
                abs(var_tmp4 ( ij_o(1, n), ij_o(2,n), 1) > 999 ) ) cycle
            mean_b_SSL = mean_b_SSL + var_tmp4 ( ij_o(1, n), ij_o(2,n), 1)
            mean_o_SSL = mean_o_SSL + var_o( 1,n )
            n_ok = n_ok + 1
        enddo
        mean_b_SSL = mean_b_SSL /real (n_ok -1)
        mean_o_SSL = mean_o_SSL /real (n_ok -1)
        write(*,"(a, 2i5,a, 2f9.4)"),'Mean_SSL',n_ok -1, np, ' from', mean_b_SSL, mean_o_SSL
    endif
    n_ok = 1
    do n = 1, np
        do k = 1, nl 
            var_b_in_o(k,  n_ok) = var_tmp4 ( ij_o(1, n), ij_o(2,n), MERGE ( 1, nl_axis (k) , nl == 1))
            var_o     (k,  n_ok) = var_o     (k,  n)
            ij_o      (1:2,n_ok) = ij_o      (1:2,n)
            x12_o     (1:2,n_ok) = x12_o     (1:2,n)
            if (var_b_in_o(k,  n_ok) == 0.0 .or. abs(var_b_in_o(k,  n_ok)> 999 ) ) exit
            if( var_id == 3) then
                var_b_in_o(k,  n_ok) = var_b_in_o(k,  n_ok) - mean_b_SSL
                !var_o(k,  n_ok) = var_o(k,  n_ok)  - mean_o_SSL
            endif
            !If point is good, lets make var_b_in_o == var_o => var_o_dif = 0
            if ( abs( var_b_in_o( k,n_ok ) - var_o( k, n_ok ) ) < das_dif_tol(var_id)   ) then
                    !write(*,"(3i5, a,3f9.4)"), n_ok, n, k, " var_o ", var_o(k, n_ok), var_b_in_o( k,n_ok )
            elseif  ( (var_b_in_o( k,n_ok ) - var_o( k, n_ok ) ) >  das_dif_tol(var_id) ) then
                write(*,"(3i5, a, 4f8.2)"), n_ok, n, k, " LOW  ", var_b_in_o( k,n_ok ), var_b_in_o( k,n_ok ) - das_dif_tol(var_id), var_o( k,n_ok ), das_dif_tol(var_id)
                var_o( k,n_ok ) = var_b_in_o( k,n_ok ) - das_dif_tol(var_id)
            elseif  ( (var_b_in_o( k,n_ok ) - var_o( k,n_ok ) ) < -das_dif_tol(var_id) ) then
                write(*,"(3i5, a, 4f8.2)"), n_ok, n, k, " HIGH ", var_b_in_o( k,n_ok ), var_b_in_o( k,n_ok ) + das_dif_tol(var_id), var_o( k, n_ok ),das_dif_tol(var_id)
                var_o( k,n_ok ) = var_b_in_o( k,n_ok ) + das_dif_tol(var_id)
            endif
            if( k == nl) n_ok = n_ok + 1
        enddo
    enddo
#ifdef DEBUG
    write(*,*),'Bad point removed. np ',np, ' from', n_ok-1
#endif
!!!!!!!!!!!!!!!!!
!!! Change np !!!
    np = n_ok - 1

!Register array var_o and var_in_o ( with common decomposition )
    dec_var = Decomposition(nl, np, 1 , "1D", CompSplitter % comm_local_size(CompSplitter % i_am()), CompSplitter % rank_local() )
    lo = dec_var % lower_bound_3D()
    hi = dec_var % upper_bound_3D()

    call this % comm_ % init_array(arr_name = var_o_name,       datatype = "real4", dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_var)
    call this % comm_ % init_array(arr_name = var_a_in_o_name,  datatype = "real4", dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_var)
    call this % comm_ % init_array(arr_name = var_b_in_o_name,  datatype = "real4", dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_var)
    call this % comm_ % init_array(arr_name = var_o_dif_name,   datatype = "real4", dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_var)

    call this % comm_ % put(var_o_name,      lo, hi, var_o)
    call this % comm_ % put(var_b_in_o_name, lo, hi, var_b_in_o)
    call this % comm_ % sum_arrays (1.0, var_o_name, -1.0, var_b_in_o_name, var_o_dif_name )
#ifdef DEBUG    
    write(*,*),"var_b_in_o_name, var_o_name, ij_o_name: DONE!",np,nl
#endif
    call this % find_error_norm( var_b_in_o_name, var_o_name, np, nl , 'B')

    dec_x12 = Decomposition(2,  np, 1 , "1D", CompSplitter % comm_local_size(CompSplitter % i_am()), CompSplitter % rank_local() )
    lo = dec_x12 % lower_bound_3D()
    hi = dec_x12 % upper_bound_3D()
    call this % comm_ % init_array(arr_name = x12_o_name, datatype = "real4", dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_x12)
    call this % comm_ % init_array(arr_name = ij_o_name,  datatype = "int8" , dimnum = "2D", holder_id = aoi_service_id, holder_decomp = dec_x12)
    call this % comm_ % put(x12_o_name, lo, hi, x12_o)
    call this % comm_ % put(ij_o_name,  lo, hi, ij_o)

    !call this % comm_ % print_arr (var_o_name)
    !call this % comm_ % print_arr (var_o_dif_name)
    !call this % comm_ % print_arr (var_b_in_o_name)
    !call this % comm_ % print_arr (x12_o_name)
    !call this % comm_ % print_arr (ij_o_name)
    call this % comm_ % sync(aoi_service_id)
    
    call hdr_file % close_file()

    deallocate( var_o, var_b_in_o, x12_o, ij_o, var_tmp4 )
!!!!
    np_lo = 1; np_hi = np;

end subroutine

!==================
!   Save to NCfile var_obs_dif data from 2D GAs
!   x12_o(1:2,1:np) and var_dif_o(1:nl,1:np)
!==================
#define _SAVE_DG_( _ARR_ , _NAME_ , _NL_) \
        var_DG = 0; \
        do n=1,np; \
            var_DG( ij_o(1, n), ij_o(2,n)) = _ARR_ ( _NL_ , n); \
        enddo; \
        call DG_file % create_var( _NAME_ , "real4", dim1_name = "i", dim2_name = "j"); \
        call DG_file % put(var_DG, (/1,1/), (/ocn_il,ocn_jl/))
subroutine save_dif_data(this,flag_a)
    use aoi_module
    use decomposition_module
    use datetime_module
    implicit none
    logical, intent(in) :: flag_a
    class(service_aoi) :: this
    type(Decomposition) :: dec_var
    type(FileHandler_NC) :: hdr_file,DG_file
    real*4, allocatable:: x12_o(:,:)        ! For OBS data
    integer*8, allocatable:: ij_o(:,:)          ! For OBS data
    real*4 , allocatable:: var_dif(:,:), var_b_tmp(:,:), var_a_tmp(:,:), var_o_tmp(:,:)
    real*4, allocatable:: var_DG(:,:)
    integer :: lo(3), hi(3),n
#ifdef DEBUG
    write(*,*), " DAS:save_dif_data", TRIM(fname_dif), nl, np
#endif
    ! Open file
    call hdr_file % create_file(fname_dif)
    call hdr_file % create_dim("nl", nl)
    call hdr_file % create_dim("np", np)
    call DG_file  % create_file(fname_DG)
    call DG_file % create_dim("i", ocn_il)
    call DG_file % create_dim("j", ocn_jl)
!Register array
    dec_var = Decomposition(nl, np, 1 , "1D", CompSplitter % comm_local_size(CompSplitter % i_am()), CompSplitter % rank_local() )
    lo = dec_var % lower_bound_3D()
    hi = dec_var % upper_bound_3D()
#ifdef DEBUG
    write(*,'(a,6(i4,x),a)'), " DAS: Save dif lohi ", lo, hi, TRIM(var_o_dif_name)
#endif
!   Get var_dif and x12_o from GA
    allocate( var_dif   ( lo(1)     :hi(1)     ,   lo(2)    :hi(2)))
    allocate( var_b_tmp ( lo(1)     :hi(1)     ,   lo(2)    :hi(2)))
    if (flag_a)  allocate( var_a_tmp ( lo(1)     :hi(1)     ,   lo(2)    :hi(2)))
    allocate( var_o_tmp ( lo(1)     :hi(1)     ,   lo(2)    :hi(2)))
    allocate( x12_o     ( 1         :2         ,   lo(2)    :hi(2)))
    allocate( ij_o      ( 1         :2         ,   lo(2)    :hi(2)))
    allocate(var_DG(ocn_il,ocn_jl))
#ifdef DEBUG
    write(*,*)," DAS: Save dif: allocate ",lo(2), hi(2)
#endif
    call this % comm_ % get( var_o_dif_name,  lo, hi, var_dif )
    call this % comm_ % get( var_b_in_o_name, lo, hi, var_b_tmp )
    if (flag_a) call this % comm_ % get( var_a_in_o_name, lo, hi, var_a_tmp )
    call this % comm_ % get( var_o_name,      lo, hi, var_o_tmp )
    call this % comm_ % get( x12_o_name,    (/1,lo(2)/), (/2,hi(2)/), x12_o     )
    call this % comm_ % get( ij_o_name,     (/1,lo(2)/), (/2,hi(2)/), ij_o      )
    call this % comm_ % sync(aoi_service_id)
#ifdef DEBUG
    write(*,*)," DAS: Save dif: get ",lo(1),hi(1),lo(2),hi(2)
#endif
!   Put to file

    call hdr_file % create_var( "var_dif", "real4", "nl","np" )
    call hdr_file % put (var_dif  (lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
    _SAVE_DG_ ( var_dif , "var_dif" , 1 )
#ifdef DEBUG
    write(*,*)," DAS: Save dif: put var_dif "
#endif
    call hdr_file % create_var( "var_b",   "real4", "nl","np" )
    call hdr_file % put (var_b_tmp(lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
    _SAVE_DG_ ( var_b_tmp , "var_b" , 1 )
#ifdef DEBUG
    write(*,*)," DAS: Save dif: put var_b_tmp "
#endif
    call hdr_file % create_var( "var_o",   "real4", "nl","np" )
    call hdr_file % put (var_o_tmp(lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
    _SAVE_DG_ ( var_o_tmp , "var_o" , 1 )
#ifdef DEBUG
    write(*,*), " DAS: Save dif: put var_o_tmp "
#endif
    if(flag_a) then
        call hdr_file % create_var( "var_a",   "real4", "nl","np" )
        call hdr_file % put (var_a_tmp(lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
        _SAVE_DG_ ( var_a_tmp , "var_a" , 1 )
        var_dif = var_a_tmp - var_b_tmp
#ifdef DEBUG
        write(*,*), " DAS: Save dif: put var_a_dif "
#endif
        call hdr_file % create_var( "var_ab_dif",   "real4", "nl","np" )
        call hdr_file % put (var_dif(lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
        _SAVE_DG_ ( var_dif , "var_ab_dif" , 1 )

        var_dif = var_o_tmp - var_a_tmp
        call hdr_file % create_var( "var_oa_dif",   "real4", "nl","np" )
        call hdr_file % put (var_dif(lo(1):hi(1),lo(2):hi(2)), (/lo(1),lo(2)/), (/hi(1),hi(2)/))
        _SAVE_DG_ ( var_dif , "var_oa_dif" , 1 )

#ifdef DEBUG
        write(*,*)," DAS: Save dif: put var_a_dif "
#endif
    endif
    call hdr_file % create_var     ("x1_o", "real4", "np" )
    call hdr_file % put( x12_o(1,lo(2):hi(2)), (/lo(2)/) , (/hi(2)/) )
#ifdef DEBUG
    write(*,*)," DAS: Save dif: put x1_o "
#endif
    call hdr_file % create_var     ("x2_o", "real4", "np" )
    call hdr_file % put( x12_o(2,lo(2):hi(2)), (/lo(2)/) , (/hi(2)/) )
#ifdef DEBUG
    write(*,*), " DAS: Save dif: put x2_o "
#endif
    call hdr_file % close_file()
    call DG_file % close_file()
#ifdef DEBUG
    write(*,*), " DAS: Save dif: hdr_file % close_file "
#endif
    deallocate( var_dif, var_b_tmp, var_o_tmp, x12_o , ij_o, var_DG, var_a_tmp)
end subroutine
!==================
!   Get data for EnKF and put in A_ens, S_ens, D_ens, psi_ens
!
!==================

subroutine make_ens_vars(this, A_ens_, S_ens_, D_ens_, psi_ens_, ndim_ens_lo, ndim_ens_hi, nrobs_ens_lo, nrobs_ens_hi, np_lo_, np_hi_, nl_)

    use aoi_module,only: aoi_service_id, ocn_il,ocn_jl, ocn_kl, nl_axis, &
                        nrens, var_id, &
                        ij_o_name, var_o_name, var_o_dif_name, var_b_name, das_mask_val, &
                        R_ens, das_r_coef, das_dif_tol
    use datetime_module
    implicit none
    class(service_aoi) :: this
    integer,intent(in) :: ndim_ens_lo, ndim_ens_hi
    integer,intent(in) :: nrobs_ens_lo, nrobs_ens_hi
    integer,intent(in) :: nl_
    integer,intent(in) :: np_lo_, np_hi_

    real*4,intent(inout) :: A_ens_  ( ndim_ens_lo : ndim_ens_hi, 1:nrens)
    real*4,intent(inout) :: psi_ens_( ndim_ens_lo : ndim_ens_hi)
    real*4,intent(inout) :: S_ens_  ( nrobs_ens_lo: nrobs_ens_hi,1:nrens)
    real*4,intent(inout) :: D_ens_  ( nrobs_ens_lo: nrobs_ens_hi)
    real*4, allocatable:: var_tmp4(:,:,:)       ! For BG and CP
    real*4, allocatable:: var_o(:,:)            ! For OBS data
    integer*8, allocatable:: ij_o(:,:)          ! For OBS data
    real*4, allocatable::  var_tmp_in_o(:,:)    ! For B proj in OBS
    integer lo(3),hi(3)
    integer i, n, k
#ifdef DEBUG
    write(*,*), " EnOI: make_ens_vars ",TRIM(ij_o_name), TRIM(var_o_name)
    write(*,*) ," EnOI: MERGE nl axis",  ocn_kl , nl_, np_lo_, np_hi_
    write(*,*) ," EnOI: D_ens nrobs_ens_lohi)",  nrobs_ens_lo, nrobs_ens_hi
#endif
    allocate(ij_o           (1:2 , np_lo_:np_hi_ ) )
    allocate(var_tmp_in_o   (1:nl_, np_lo_:np_hi_ ) )
    allocate(var_o          (1:nl_, np_lo_:np_hi_ ) )
    write(*,*) ," EnOI: ij_o allocated",ij_o_name
    call this % comm_ % get(ij_o_name,  (/1,np_lo_/) , (/2,np_hi_/)  , ij_o (1:2, np_lo_:np_hi_) )
    call this % comm_ % get(var_o_name, (/1,np_lo_/) , (/nl_,np_hi_/), var_o(1:nl_, np_lo_:np_hi_) )
    call this % comm_ % sync(aoi_service_id)
    write(*,*) ," EnOI: ij_o_name get",ij_o_name
    lo = (/1,       1, 1 /)
    if ( TRIM(var_o_name) == "T_o" .or. TRIM(var_o_name) == "S_o") then
        hi = (/ocn_il,  ocn_jl, ocn_kl /) 
    else
        hi = (/ocn_il,  ocn_jl, 1 /) 
    endif
    write(*,*) ," EnOI: var_tmp allocate ",lo(1),hi(1), lo(2),hi(2), lo(3),hi(3)
    allocate( var_tmp4( lo(1):hi(1), lo(2):hi(2),       lo(3):hi(3) ) )
    write(*,*) ," EnOI: var_tmp allocated"
    
!==================================
!Make A_ens ( ndims_ens, nrens, nl_)
!==================================
    do i=1, nrens
        call this % make_3Dvar_from_1Dvar(  A_ens_(ndim_ens_lo:ndim_ens_hi, i),lo, hi, ndim_ens_lo, ndim_ens_hi, &
                                            var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) )  )
#ifdef DEBUG
        write(*,'(a,f8.1)')," A_ens maxval    ",maxval(A_ens_(ndim_ens_lo:ndim_ens_hi, i))
        write(*,'(a,f8.1)')," var_tmp4 maxval ",maxval(var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ))
        write(*,'(a,2i8)')," nrobs_lohi = ", nrobs_ens_lo, nrobs_ens_hi
#endif
        if ( nrobs_ens_lo < nrobs_ens_hi ) then
!==================================
!Make S_ens ( nrobs_ens, nrens, nl_)
!==================================
#ifdef DEBUG
    write(*,'(a,2i8,a)')," np_lohi = ", np_lo_, np_hi_, TRIM(ij_o_name)
#endif
        do n = np_lo_, np_hi_
            do k = 1, nl_
                var_tmp_in_o(k, n) = var_tmp4( ij_o(1,n), ij_o(2,n), MERGE(1, nl_axis(k), nl_==1) )
                !If point is bad, lets make var_b_in_o == var_o => var_o_dif = 0
                !!!if ( TRIM(var_o_name) == "SSL_o") var_tmp_in_o(k, n) = var_tmp_in_o(k, n) - mean_b_SSL
                if ( abs( var_tmp_in_o( k,n ))  > das_mask_val ) then
                    write(*,"(a,2i4,a,2f8.1)")," BAD ENS var_b (", ij_o(1,n), ij_o(2,n),") = ",var_tmp_in_o(k, n), var_o( k,n )   
                    var_tmp_in_o(k, n) = var_o( k,n )

                endif
                !write(*,"(a,2i4,a,2f7.1)"),"var_b (", ij_o(1,n), ij_o(2,n),") = ",var_tmp_in_o(k, n), var_tmp_in_o(k, n) - var_o( k,n )
            enddo
        enddo

        call this % make_1Dvar_from_3Dvar(  var_tmp_in_o(1:nl_,np_lo_:np_hi_), (/1,np_lo_,1/), (/nl_,np_hi_,1/),nrobs_ens_lo,nrobs_ens_hi, S_ens_(nrobs_ens_lo:nrobs_ens_hi, i))
        S_ens_(nrobs_ens_lo:nrobs_ens_hi, i) = S_ens_(nrobs_ens_lo:nrobs_ens_hi, i) / das_dif_tol(var_id) ! NORM To 1
#ifdef DEBUG
    !if(var_id==1  ) write(*,'(a,i3,a,<nrobs_ens_hi-nrobs_ens_lo+1>f6.1)')," S_ens(",i,")=",S_ens_ ( nrobs_ens_lo:nrobs_ens_hi, i )
    write(*,'(a,i3,a,2f8.1)')," S_ens(",i,")=",das_dif_tol(var_id), maxval(S_ens_ ( nrobs_ens_lo:nrobs_ens_hi, i ))
#endif
        endif
    enddo
    if ( nrobs_ens_lo < nrobs_ens_hi ) then
!==================================
!Get D_ens_matrix
!==================================
        call this % comm_ % get(var_o_dif_name,    (/1, np_lo_/) , (/nl_, np_hi_/), var_tmp_in_o (1:nl_, np_lo_:np_hi_ ) )
        call this % comm_ % sync(aoi_service_id)
        call this % make_1Dvar_from_3Dvar(  var_tmp_in_o(1:nl_,np_lo_:np_hi_), (/1,np_lo_,1/), (/nl_,np_hi_,1/),nrobs_ens_lo,nrobs_ens_hi, D_ens_(nrobs_ens_lo:nrobs_ens_hi))
        D_ens_(nrobs_ens_lo:nrobs_ens_hi) = D_ens_(nrobs_ens_lo:nrobs_ens_hi) / das_dif_tol(var_id) ! NORM To 1
#ifdef DEBUG
    !write(*,'(a,a,<nrobs_ens_hi-nrobs_ens_lo+1>f6.1)'),TRIM(var_o_dif_name)," D_ens = ",D_ens_ ( nrobs_ens_lo:nrobs_ens_hi )
    write(*,'(a,a,2f8.2)'),TRIM(var_o_dif_name)," D_ens = ",das_dif_tol(var_id), maxval(D_ens_ ( nrobs_ens_lo:nrobs_ens_hi ))
#endif
    endif

!==================================
!Get psi_ens_matrix
!==================================
    write(*,*) ,"EnOI: try to get psi matrix ",TRIM(var_b_name), hi
    
    call this % comm_ % get( var_b_name, lo, hi, var_tmp4( lo(1):hi(1),lo(2):hi(2),lo(3):hi(3)) )
    
    call this % comm_ % sync(aoi_service_id)
    call this % make_1Dvar_from_3Dvar(  var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ), lo, hi, ndim_ens_lo, ndim_ens_hi, &
                                                psi_ens_(ndim_ens_lo:ndim_ens_hi) )
#ifdef DEBUG
    write(*,'(2(a,x),3f8.1)'),TRIM(var_b_name),"get psi_ens maxval ", &
        minval(psi_ens_(ndim_ens_lo:ndim_ens_hi)),maxval(psi_ens_(ndim_ens_lo:ndim_ens_hi))
#endif
    deallocate(var_tmp_in_o,var_o,var_tmp4,ij_o)

!==================================
!Make R_ens_matrix
!==================================
    do i = nrobs_ens_lo, nrobs_ens_hi
        R_ens(i,i) = das_r_coef(var_id)
    enddo

end subroutine

!==================
!   Read one element from file fname to column of A-matrix (iA-index)
!   fname   - name of file (reading from)
!   varname - name of var  (reading to)
!   time_ens_get - DateTime of reading data
!   var_cp_name - t_cp, s_cp, ssl_cp ....
!   iA - indx Ens of A matrix
!==================

subroutine get_one_ens( this, fname_ens,  time_ens_get, iA, var_cp_name ,ndim_ens_lo, ndim_ens_hi )
    use aoi_module,only: ocn_il, ocn_jl, ocn_kl, nl, das_mask_val, var_id, A_ens
    use decomposition_module
    use datetime_module
    implicit none
    class(service_aoi) :: this
    character(NAME_LEN),intent(in) :: fname_ens
    type(DateTime),intent(in) :: time_ens_get
    integer,intent(in) :: iA
    character(NAME_LEN),intent(in) :: var_cp_name
    integer,intent(in) :: ndim_ens_lo, ndim_ens_hi
    type(FileHandler_NC) :: hdr_file
    real*8, allocatable:: var_tmp8(:,:,:)
    real*4, allocatable:: var_tmp4(:,:,:)
    integer :: lo(3), hi(3)
    integer :: il, jl, kl
    integer i,j,k
    write(*,*) , "EnOI: get_ens ", TRIM(var_cp_name), TRIM(fname_ens),  iA
    call hdr_file % open_file(fname_ens, CompSplitter % comm_local())
    call hdr_file % open_var(TRIM(var_cp_name))
    il = hdr_file % get_dim_size("i")
    jl = hdr_file % get_dim_size("j")
    kl = hdr_file % get_dim_size("k")
    
    if ( il /= ocn_il .or. jl /= ocn_jl .or. kl /= ocn_kl ) then
        write(*,*) , "EnOI: wrong dim", var_cp_name, il,jl, kl, nl, ocn_kl
        stop(10)
    endif
    lo=(/1, 1, 1/) 
!Register array varname and its dec !TBD ocn_kl!!!!
!   Read t_f ( 1: kl), but put in GA only 1:nl !
    write(*,*) , "EnOI: get_ens ", fname_ens, var_id
    
    select case ( var_id )
    case(1,2)
        write(*,*) , "EnOI: get_ens case12", var_cp_name, var_id
        hi=(/ocn_il, ocn_jl, ocn_kl/)
        allocate(var_tmp8(lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)))
        allocate(var_tmp4(lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)))
        call hdr_file % get( var_tmp8,      lo,     hi, time_ens_get)
                var_tmp4    (lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)) = &
        real(   var_tmp8    (lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)) )
        write(*,*) , "EnOI: get_ens case12 got", var_cp_name, var_id
    case(3)
        write(*,*) , "EnOI: get_ens case3", var_cp_name, var_id
        hi=(/ocn_il, ocn_jl, 1/)
        allocate(var_tmp8(lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)))
        allocate(var_tmp4(lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)))
        call hdr_file % get( var_tmp8,      lo,     hi, time_ens_get)
                var_tmp4    (lo(1):hi(1), lo(2):hi(2), 1:1) = &
        real(   var_tmp8    (lo(1):hi(1), lo(2):hi(2), 1:1) )
    case(4)
        write(*,*) , "EnOI: get_ens case4", var_cp_name, var_id
        hi=(/ocn_il, ocn_jl, 1/)
        allocate(var_tmp4(lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)))
        call hdr_file % get( var_tmp4,   lo,     hi, time_ens_get)
    case default
            write(*,*),"RECV_DAS: Incorrect var name", var_id
            stop(4)
    end select
    call hdr_file % close_file()
#ifdef DEBUG
    write(*,*)," var_ens_get(maxval) = ",maxval(var_tmp4( lo(1):hi(1),     lo(2):hi(2),    lo(3):hi(3) ) )
#endif
    
    do i = lo(1),hi(1)
      do j = lo(2),hi(2)
        do k = lo(3),hi(3)
          if ( var_tmp4(i,j,k) > das_mask_val  ) then
            var_tmp4(i,j,k) = 0
          endif
        enddo
      enddo
    enddo
    call this % make_1Dvar_from_3Dvar( var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3)), lo, hi, ndim_ens_lo, ndim_ens_hi, &
                                                A_ens(ndim_ens_lo:ndim_ens_hi, iA) )
#ifdef DEBUG
    write(*,'(a,a,f8.1)'),TRIM(var_cp_name)," var_ens_get  = ",maxval(A_ens( ndim_ens_lo:ndim_ens_hi , iA ) )
#endif
    deallocate(var_tmp4)
    if(allocated(var_tmp8)) deallocate(var_tmp8)
end subroutine

!==================
!   Make enkf assimiltaion
!==================
subroutine make_enkf(this)
    use decomposition_module
    use datetime_module
    use aoi_module
    implicit none
    class(service_aoi) :: this
    type(FileHandler_NC) :: hdr_file
    character(NAME_LEN) :: fname_psi      ! file names for obs data and markers
    integer :: i

#ifdef DEBUG
    !write(*,'(i2,a,<ndim_ens>f6.1)'),aoi_rnk," psi_ens = ",psi_ens(1:ndim_ens)
#endif
    write(*,'(a,2i10,a)'), " analysis2_EnOI ", nrobs_ens, ndim_ens, TRIM(time_get % to_string())
    
    

    ! Open file
    fname_psi= "data/enoi_D_"//yyyymmdd//".nc"
    call hdr_file % create_file(fname_psi)
    call hdr_file % create_dim("nrobs_ens", nrobs_ens)
    call hdr_file % create_var( "D_ens",   "real4", "nrobs_ens" )
    call hdr_file % put (D_ens(1:nrobs_ens), (/1 /), (/nrobs_ens /))    
    call hdr_file % close_file()


    call analysis2_EnOI(    A_ens(1:ndim_ens,1:nrens), psi_ens(1:ndim_ens), D_ens(1:nrobs_ens),&
                            R_ens(1:nrobs_ens,1:nrobs_ens), S_ens(1:nrobs_ens,1:nrens), &
                            int( ndim_ens, 8), int(nrens, 8), int(nrobs_ens, 8), .TRUE.)
    

#ifdef DEBUG
    write(*,*) , "EnOI: make_enoi ok"
    !write(*,'(i2,a,<ndim_ens>f6.1)'),aoi_rnk," psi_ens = ",psi_ens(1:ndim_ens)
#endif

end subroutine

!==================
!   Send psi from EnOI back to the model
!==================

subroutine send_ens_vars(this, psi_ens_, ndim_ens_lo, ndim_ens_hi,np_, nl_, kl_,  ev )
    use aoi_module
    use datetime_module
    implicit none
    class(service_aoi) :: this
    integer,intent(in) :: ndim_ens_lo, ndim_ens_hi
    real*4,intent(in) :: psi_ens_( ndim_ens_lo : ndim_ens_hi)
    integer,intent(in) :: nl_ ,np_, kl_
    type(Event), intent(in) :: ev
    real*4, allocatable:: var_tmp4(:,:,:)       ! For BG and CP
    real*8, allocatable:: var_tmp8(:,:,:)       ! For BG and CP
    integer*8, allocatable:: ij_o(:,:)          ! For OBS data
    real*4, allocatable:: var_a_in_o(:,:)       ! For B proj in OBS
    integer lo(3),hi(3), i,j ,n,k
    call this % common_switcher(ev)

    lo = (/1,       1,      1   /)
    hi = (/ocn_il,  ocn_jl, kl_ /)
    allocate( var_tmp4    ( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ) )
    allocate( var_tmp8    ( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ) )
    allocate( var_a_in_o (1:nl_, 1:np_ ) )
    allocate( ij_o       (1:2  , 1:np_ ) )
    call this % comm_ % get(ij_o_name,  (/1,1/) , (/2,np_/)  , ij_o (1:2, 1:np_) )
#ifdef DEBUG
    write(*,'(a,a,x,f8.1)'),TRIM( ev % ga_name() )," send_ens_vars maxval ",maxval(psi_ens_(ndim_ens_lo:ndim_ens_hi))
    write(*,'(a,x,8(i8,x))')," send psi_ens maxval ", ndim_ens_lo, ndim_ens_hi, lo, hi
#endif
    call this % make_3Dvar_from_1Dvar(  psi_ens_(ndim_ens_lo:ndim_ens_hi),lo, hi, ndim_ens_lo, ndim_ens_hi, &
                                            var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) )  )
#ifdef DEBUG
    write(*,*)," send_ens_vars ", TRIM(var_a_in_o_name), np_, nl_ !, nl_axis (5)
#endif
    do n = 1, np_
        do k = 1, nl_
            var_a_in_o(k,  n) = var_tmp4( ij_o(1, n), ij_o(2,n), MERGE ( 1, nl_axis (k) , nl_ == 1) )
        enddo
    enddo
    call this % comm_ % put(var_a_in_o_name, (/1, 1 /), (/nl_, np_ /), var_a_in_o)
! Put back and send to OCN
    call this % comm_ % put ( var_b_name, lo, hi , var_tmp4 )
    select case ( ev % ga_name() )
        case("OCN_t_dif_RECV_ENKF","OCN_t_f_RECV_ENKF","OCN_s_dif_RECV_ENKF","OCN_s_f_RECV_ENKF")
            !var_dif( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ) = 29 ! DEBG
            do i=lo(1),hi(1)
            do j=lo(2),hi(2)
            do k=lo(3),hi(3)                  
                if( var_tmp4(i,j,k) < -2. ) var_tmp4(i,j,k) = -2.
                var_tmp8(i,j,k) = dble(var_tmp4(i,j,k))
            enddo
            enddo
            enddo
            if( main_kind == 8) then
                call this % comm_ % put ( ev % ga_name(), lo, hi , var_tmp8 )
            else
                write(*,*),"ENKF: TS use real8", ev % ga_name()
            endif
        case("OCN_ssl_dif_RECV_ENKF","OCN_ssl_f_RECV_ENKF")
            !var_tmp4( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) ) = 0.777 ! DEBG
                    var_tmp8 (  lo(1):hi(1),lo(2):hi(2),lo(3):hi(3) ) = &
            dble(   var_tmp4 (  lo(1):hi(1),lo(2):hi(2),lo(3):hi(3) ))
            call this % comm_ % put ( ev % ga_name(), lo, hi , var_tmp8 )
        case("OCN_aice_dif_RECV_ENKF","OCN_aice_RECV_ENKF","ICE_aice_dif_RECV_ENKF","ICE_aice_RECV_ENKF")
            
            do i=lo(1),hi(1)
                do j=lo(2),hi(2)
                    !var_tmp(i,j,1) = 0.222 !DBG
                    if( var_tmp4(i,j,1) > 1. ) var_tmp4(i,j,1) = 1.
                    if( var_tmp4(i,j,1) < 0. ) var_tmp4(i,j,1) = 0.
                enddo
            enddo
            call this % comm_ % put ( ev % ga_name(), lo, hi , var_tmp4 )
        case default
            write(*,*),"EnOI: Incorrect var name", ev % ga_name()
            stop(4)
    end select
#ifdef DEBUG
    write(*,'(a,x,a,f8.1)'), TRIM( ev % ga_name() ), " send var_dif maxval ", maxval(var_tmp4(lo(1):hi(1), lo(2):hi(2), 1:1))
#endif


    call this % comm_ % sync( aoi_service_id )
    if(CompSplitter % is_first_rank()) call this % comm_ % put_info( ev % ga_name(), COMM_GA_STATUS, "full")
    if(CompSplitter % is_first_rank()) call this % comm_ % put_info( ev % ga_name(), COMM_GA_OPT, "new")
#ifdef DEBUG
    write(*,*),"EnOI: send_ens_vars full", TRIM(ev % ga_name())
#endif    
    deallocate( var_tmp4, var_tmp8,var_a_in_o, ij_o)
    !call this % get_projection( var_b_name )
    call cpu_time(das_time_end)
#ifdef DEBUG
    write(*,*),"EnOI: save_dif and fine error norm", TRIM(ev % ga_name()),TRIM(var_a_in_o_name), nl_, kl_
#endif 
    das_time_total = das_time_total + das_time_end - das_time_strt

    !call this % find_error_norm( var_b_in_o_name, var_o_name, np_, nl_ , 'B')
    call this % find_error_norm( var_a_in_o_name, var_o_name, np_, nl_, 'A')
    call this % save_dif_data(.TRUE.)
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      calculate L2,max,min norm error between two arrays
! Parameters:
!   Input
!   b_name      - first  array
!   a_name      - second array
!   _np,_nl     - size of target arrays
!   er_name     - name of norm ( A - analize error, B - background error)
!----------------------------------------------------------------------------------------

subroutine find_error_norm(this, a_name, b_name, np_, nl_, er_name)
    use aoi_module, only:   yyyymmdd, time_get, logfile, das_time_total, &
                            var_b_name, aoi_service_id, aoi_rnk
    use decomposition_module
    implicit none
    class(service_aoi) :: this
    character(*),   intent(in) :: a_name, b_name
    integer,        intent(in) :: np_, nl_
    character,   intent(in) :: er_name*1
    type(Decomposition) :: dec
    integer :: lo(3), hi(3), i, j
    real*4, allocatable :: a_arr(:,:), b_arr(:,:)
    real*4, allocatable :: L2_norm(:), L2_norm_res(:)
    real*4, allocatable :: max_dif(:), min_dif(:), max_dif_res(:), min_dif_res(:)

    dec = Decomposition( nl_, np_, 1 , "1D", CompSplitter % comm_local_size(CompSplitter % i_am()), CompSplitter % rank_local() )
    lo = dec % lower_bound_3D()
    hi = dec % upper_bound_3D()
    allocate( a_arr      ( lo(1):hi(1), lo(2):hi(2) ),&
              b_arr      ( lo(1):hi(1), lo(2):hi(2) ),&
              L2_norm    ( lo(1):hi(1) ),&
              L2_norm_res( lo(1):hi(1) ),&
              max_dif    ( lo(1):hi(1) ),&
              max_dif_res( lo(1):hi(1) ),&
              min_dif    ( lo(1):hi(1) ),&
              min_dif_res( lo(1):hi(1) ) )

    call this % comm_ % get(a_name, lo , hi , a_arr)
    call this % comm_ % get(b_name, lo , hi , b_arr)
    call this % comm_ % sync(aoi_service_id)
    write(*,'(4a)'),"DAS: ",TRIM(a_name), " DateTime: ", TRIM(time_get % to_string())
    L2_norm(1:nl_) = 0.
    do i = 1, nl_
      max_dif(i) = abs (  b_arr (i,lo(2)) - a_arr (i,lo(2)) )
      min_dif(i) = max_dif(i)
    enddo

    do i = 1, nl_
      do j = lo(2), hi(2)
        L2_norm( i ) = L2_norm( i ) + ( b_arr (i,j) - a_arr (i,j)) ** 2
#ifdef DEBUG
        !if( i == 1 ) write(*,'(i4,a,i3,a,3f7.2)'),aoi_rnk," L2 ( ", j ," )= ", L2_norm( i ) , b_arr (i,j) , a_arr (i,j)
#endif
        if (  max_dif(i) < abs( b_arr (i,j) - a_arr (i,j) ) ) max_dif(i) = abs( b_arr (i,j) - a_arr (i,j) )
      enddo
    enddo

    L2_norm_res = L2_norm; max_dif_res = max_dif; min_dif_res= min_dif; 
    if( aoi_rnk == 1 ) then
        L2_norm_res = sqrt( L2_norm_res / np_ )
        write(*,'(i3,a,a,a,f7.2,a,f7.2)'),nl_, "DAS:",er_name," L2 norm = ", L2_norm_res( 1 ), " max dif = ", max_dif_res( 1 )
        open (336,file=logfile, POSITION='APPEND')
        write(*,*), TRIM(yyyymmdd), TRIM(er_name), TRIM(var_b_name), "L2 ",&
                         np_, nl_, L2_norm_res(1:nl_),  das_time_total
        write(336,2001), yyyymmdd, er_name, var_b_name, "L2 ",&
                         np_, nl_, L2_norm_res(1:nl_),  das_time_total


2001        format (a10,x,3(a4,x),2(i5,x),<nl_>f6.2,x,f8.3)
        close(336)

        
    endif
deallocate( a_arr, b_arr, L2_norm, L2_norm_res, max_dif, max_dif_res, min_dif, min_dif_res )

end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      Copy 1dvar(1:nl) to 3dvar(lo(1):hi(1), lo(2):hi(2))
!----------------------------------------------------------------------------------------
subroutine make_3Dvar_from_1Dvar(this, var1d, lo, hi, n_lo, n_hi, var3d )
    class(service_aoi) :: this
    integer,intent(in)  :: n_lo, n_hi               ! 1d size
    real*4, intent(in)  :: var1D (n_lo:n_hi)
    integer,intent(in)  :: lo(3), hi(3)     ! dim size (from:to) 3dim for future use
    real*4, intent(inout) :: var3d ( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) )
    integer i,j,k,n
#ifdef DEBUG
    write(*,'(2f8.1, a,2i8,a,6i5)'), var1D ( n_hi),var3d ( hi(1),hi(2),hi(3) ), " make_3dvar_from_1Dvar ",n_lo,n_hi,' lohi= ',lo,hi
#endif
    n = n_lo
    do i = lo(1), hi(1)
      do j = lo(2), hi(2)
        do k = lo(3), hi(3)
            var3d(i,j,k) = var1d(n)
            !write(*,'(i7,3i5, f8.1)'), n, i, j, k, var3d(i,j,k)
            n = n + 1
        enddo
      enddo
    enddo
    if ( n_hi /= (n-1) ) then
        write(*,'(a,3i8,a,6i5)'),"EnOI:copy 1d to 3d err",n,n_lo,n_hi,' lohi=',lo,hi
        stop(8)
    endif
end subroutine

!========================================================================================
!----------------------------------------------------------------------------------------
! Description:      Copy 3Dvar(1:il,jl_lo:jl_hi) to 1Dvar(1:nl)
!----------------------------------------------------------------------------------------
subroutine make_1dvar_from_3dvar(this, var3d, lo, hi, n_lo, n_hi, var1d)
    use aoi_module,only:  das_mask_val
    class(service_aoi) :: this
    integer,intent(in) :: n_lo, n_hi
    integer,intent(in) :: lo(3), hi(3)     ! dim size (from:to) 3dim for future use
    real*4, intent(in) :: var3d ( lo(1):hi(1), lo(2):hi(2), lo(3):hi(3) )
    real*4, intent(out):: var1d ( n_lo:n_hi )

    integer i,j,k,n
    !write(*,'(2f8.1, a,2i8,a,6i5)'), var1D ( n_lo),var3d ( lo(1),lo(2) ), " make_1Dvar_from_3dvar ",n_lo,n_hi,' lohi= ',lo,hi
    n = n_lo
    do i = lo(1), hi(1)
      do j = lo(2), hi(2)
        do k = lo(3), hi(3)
            var1d(n) = MERGE ( var3d(i,j,k), 0., abs( var3d(i,j,k) ) < das_mask_val )
            !var1d(n) =        var3d(i,j,k)
            !write(*,'(2f8.1, a,i8,a,6i5)'), var1D ( n),var3d ( i,j ), " 1Dvar_from_3dvar ",n,' lohi= ',i,j
            n = n + 1
        enddo
      enddo
    enddo
    if ( n_hi /= (n -1) ) then
        write(*,'(a,3i8,a,6i5)'),"EnOI:copy 3d to 1d err",n,n_lo,n_hi,' lohi=',lo,hi
        stop(9)
    endif
end subroutine

!================================
!   Get one ENS date and Fname
!   en_n  -         index of ens element
!   time_ens_get -  date  for en_n - element of ens
!   fname_ens -     file name where will be taken en_n - element
!================================
subroutine get_ens_date_from_list(this,time_ens_get,fname_ens, en_n)
    use datetime_module
    use aoi_module
    implicit none
    integer,intent(in) :: en_n
    type(DateTime),intent(out) :: time_ens_get
    character(NAME_LEN),intent(out) :: fname_ens
    character(10) :: yyyymmdd_tmp
    class(service_aoi) :: this
    integer i, n
    n = 0
    if ( en_n > nrens) then
        write(*,*)," EnOI: Nrens ", n, en_n, time_get % to_string()
        stop(5)
    endif
    time_get = ExpInfo % start_date()
    write(*,*)," EnOI: time_get ", time_get % to_string()
    do i =1, ens_list_size
        !if( time_ens(i) % month == time_get % month .or. mod(time_ens(i) % month, 12) == mod(time_get % month + 1, 12) .or. mod(time_ens(i) % month, 12) == mod(time_get % month - 1, 12) ) then
        if( time_ens(i) % month == time_get % month .or. mod(time_ens(i) % month, 12) == mod(time_get % month + 1, 12) ) then
        !if( time_ens(i) % month == time_get % month ) then 
            n = n + 1
            if ( en_n == n ) then
              time_ens_get = time_ens (i)
              yyyymmdd_tmp = TRIM( time_ens_get % to_string() )
              !fname_ens_prefix = 'data/EnOI_g00/OCN_g00_CP_'
              fname_ens    = TRIM(fname_ens_prefix)//yyyymmdd_tmp//".nc"
#ifdef DEBUG
              write(*,*),"get_ens ", n, TRIM( yyyymmdd_tmp ), TRIM(fname_ens)
#endif
              return
            endif
        endif
    enddo
    write(*,*)," EnOI: Cannot find such many ens members ", n, en_n,fname_ens ,time_ens(i) % to_string()
    stop(6)
end subroutine

end module

