#define DEBG
program aice_preproc
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none

    type(FileHandler_NC) :: handler
    type(DateTime) :: dat_st
    character(10) :: yyyymmdd

!for write Data
    real*4 , allocatable :: x1_o1(:),x2_o1(:)
    real*4 , allocatable :: x1_o2(:),x2_o2(:)
    real*4 , allocatable :: x1_o(:),x2_o(:)
    real*4 , allocatable:: var_o1(:,:),var_o2(:,:)
    real*4 , allocatable:: var_o(:,:)
    integer*4, allocatable:: i_o(:),j_o(:)
    integer, allocatable:: sort_indx(:)
    integer np,np1,np2
!working variables    character(10) :: yyyymmdd

!Input and output files
    character(100)            :: in_fname1, in_fname2, out_fname, fname



    real*4, external :: x1_shift
    call get_args(  dat_st, in_fname1, in_fname2)
    call get_aice_data(in_fname1, x1_o1, x2_o1, var_o1, np1, dat_st, 12000)  !NORTH
    call get_aice_data(in_fname2, x1_o2, x2_o2, var_o2, np2, dat_st, 12000)  !SOUTH
    np = np1 + np2
    write(*,*), 'Merge np = ', np
    allocate(x1_o (np),x2_o (np), i_o (np), j_o (np), var_o(1,np), sort_indx(np))
    x1_o(1:np1)     = x1_o1(1:np1)
    x2_o(1:np1)     = x2_o1(1:np1)
    var_o(1,1:np1)   = var_o1(1,1:np1)

    x1_o(       np1+1:np1+np2)  = x1_o2(1:np2)
    x2_o(       np1+1:np1+np2)  = x2_o2(1:np2)
    var_o(1,   np1+1:np1+np2)   = var_o2(1,1:np2)
    
    call sort_x12_var( x1_o, x2_o, var_o, 1, np, sort_indx )
    call x12_o_to_ij_o(x1_o, x2_o, i_o, j_o, var_o, np, 1 )

#ifdef DEBG
    write(*,'(a,i9,a,10f8.2)'),'x1_o(',np,')=',x1_o(1:10)
    write(*,'(a,i9,a,10f8.2)'),'x2_o(',np,')=',x2_o(1:10)
    write(*,'(a,i9,a,10f8.2)'),'var_o(',np,')=',var_o(1,1:10)
    write(*,'(a,i9,a,10f8.2)'),'var_o(',np,')=',maxval(var_o),minval(var_o)
#endif
    yyyymmdd = dat_st % to_string()
    out_fname = 'aice.nc'
    fname='AICE_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "aice", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), var_o(1,1:np), 1, np  )
    deallocate( var_o, x1_o ,x2_o)


contains



subroutine get_aice_data(in_fname, x1_o,x2_o, var_o, np, dat_st, MAX_PNTS)
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none
    type(FileHandler_NC)    :: handler
    character(100), intent(IN) :: in_fname
    real*4, allocatable, intent(INOUT) ::  x1_o(:),x2_o(:)
    real*4, allocatable, intent(INOUT) ::  var_o(:,:)
    integer, intent(INOUT) ::np
    integer, intent(IN) :: MAX_PNTS !for delete some data
    type(DateTime), intent(INOUT) :: dat_st
!for read Data
    real*4, allocatable:: lon(:,:),lat(:,:)
    real*4, allocatable:: ice_conc(:,:)
    integer xc,yc


    integer,parameter:: FillValue = -999
    integer i,j,n,id
    integer PNT_STEP,PNT_STEP_X, PNT_STEP_Y
    real*4 x1,x2

    call handler % open_file(in_fname)
    write(*,*),'open_file ',TRIM(in_fname)
    xc = handler % get_dim_size("xc")
    yc = handler % get_dim_size("yc")
    write(*,'(a,2i9)'),'xc,yc = ',xc,yc
    allocate(lat(xc,yc),lon(xc,yc),ice_conc(xc,yc))
      
    call handler % open_var("lat")
    call handler % get (  lat, (/1, 1/) , (/xc, yc/)  )
    call handler % open_var("lon")
    call handler % get (  lon, (/1, 1/) , (/xc, yc/)  )
    call handler % open_var("ice_conc")
    call handler % get (  ice_conc,(/1, 1/) , (/xc, yc/)  , dt = dat_st)
    call handler % close_file()

#ifdef DEBG
    write(*,'(a,2i9,a,10f8.2)'),'LAT(',xc,yc,')=', lat(1:5,1:2)
    write(*,'(a,2i9,a,10f8.2)'),'LON(',xc,yc,')=', lon(1:5,1:2)
    write(*,'(a,2i9,a,10f8.2)'),'AICE(',xc,yc,')=',   ice_conc(1:5,1:2)
    write(*,*),"[",min_x1,max_x1,"]x[",min_x2,max_x2,"]"
    PNT_STEP =  sqrt (dble((xc * yc ) / MAX_PNTS) )
    PNT_STEP_X = PNT_STEP / 6 
    PNT_STEP_Y = PNT_STEP
    write(*,*),"PNT_STEP = ",PNT_STEP_X, PNT_STEP_Y 

#endif

    id = 0
    do i=1,xc,PNT_STEP_X
      do j=1,yc,PNT_STEP_Y
        x1 = lon(i,j);  x1 = x1_shift(x1); x2 = lat(i,j)
        if ( id == MAX_PNTS ) exit
        if( x1 > min_x1 .and. x1 < max_x1 .and. x2 > min_x2 .and. x2 < max_x2 .and. &
            ice_conc(i,j)  /= FillValue .and. ice_conc(i,j) > 100. .and. ice_conc(i,j) < 9000. ) then
            id = id + 1
        endif
      enddo
    enddo
    np = id

#ifdef DEBG
    write(*,*),"np =", np
#endif
    allocate(x1_o(np),x2_o(np), var_o(1,np))
    x1_o=0; x2_o = 0; var_o = 0;
    id=0
    do i=1,xc,PNT_STEP_X
      do j=1,yc,PNT_STEP_Y
        x1 = lon(i,j);  x1 = x1_shift(x1); x2 = lat(i,j)
        if ( id == MAX_PNTS ) exit
        if( x1 > min_x1 .and. x1 < max_x1 .and. x2 > min_x2 .and. x2 < max_x2 .and. &
            ice_conc(i,j)  /= FillValue .and. ice_conc(i,j) > 100. .and. ice_conc(i,j) < 9000. ) then
            id = id + 1
            x1_o(id) = x1
            x2_o(id) = x2
            var_o(1,id) = ice_conc(i,j) * 10e-5
        endif
      enddo
    enddo
    deallocate(ice_conc, lon, lat)


end subroutine 
end program
