!=======================================================================================
! Define set GA interface i4 or i8 ( recomend i8 for x64 OS)
!========================================================================================
#define GA_IS 8     ! GA Integer Size 8 or 4
#define DEBUG

subroutine analysis2_EnOI(A,psi,D, R, S, ndim, nrens, nrobs, verbose)
! Computes the analysed vector psi for the static ensemble A 
   use m_multa
   implicit none
   integer*8, intent(in) :: ndim             ! dimension of model state
   integer*8, intent(in) :: nrens            ! number of ensemble members
   integer*8, intent(in) :: nrobs            ! number of observations
   
   real*4, intent(inout) :: A(ndim,nrens)   ! ensemble matrix
   real*4, intent(inout) :: psi(ndim)       ! state vector (forecast -> vector)
   real*4, intent(in)    :: D(nrobs)  ! matrix  holding observation innovations
   real*4, intent(inout)    :: S(nrobs,nrens)  ! matrix holding HA(input) and then HA'
   real*4, intent(inout) :: R(nrobs,nrobs)  ! Error covariance matrix for observations
   logical, intent(in) :: verbose

   real*4, allocatable, dimension(:,:) :: X1,X2,U,Reps,V,I_N
   real*4, allocatable, dimension(:) :: X4,X5
!   real X3(nrobs,1)
   real*4, allocatable, dimension(:,:) :: X3
   real*4, allocatable, dimension(:)   :: sig,work
   real(kind=8) :: tarray(2)
        
   real*4 sigsum,sigsum1,oneobs(1,1)
   integer*4 ierr,nrsigma,i,j,lwork,m
   integer*4 iblkmax, iens
   integer*4, parameter :: target= 3*22+1  ! Case dependent: test one value
   character(len=2) tag2
#ifdef DEBUG
    write(*,*), "EnOI start", ndim,nrens,nrobs
    write(*,*)," psi_ens maxval ",minval(psi), maxval(psi)
    write(*,'((a,x),2f9.2)')," A_ens minmax ",minval(A), maxval(A)
    write(*,'((a,x),2f9.2)')," D_ens minmax ",minval(D), maxval(D)
    write(*,'((a,x),2f9.2)')," R_ens minmax ",minval(R), maxval(R)
    write(*,'((a,x),2f9.2)')," S_ens minmax ",minval(S), maxval(S)
#endif
    allocate (I_N(nrens,nrens))
    I_N=real(-1.0)/real(nrens)
    
    do i=1,nrens
      I_N(i,i)=I_N(i,i)+1.0
    enddo
    write(*,'((a,x),2f8.4)')," I_N minmax ",minval(I_N), maxval(I_N)
    write(*,'(a,<nrens>f6.1)')," I_N = ",I_N(1:nrens,2)
    write(*,'(a,<nrens>f6.1)')," S_N = ",S(2, 1:nrens)

    S=matmul(S,I_N)
    !call sgemm('n','n', nrobs, nrens, nrobs, real(1.0), S, nrobs, I_N, nrobs, real(1.0), R, nrobs)
    write(*,'(a,<nrens>f6.1)')," I_N = ",I_N(1:nrens,2)
    write(*,'(a,<nrens>f6.1)')," S_N = ",S(2, 1:nrens)
    !A=matmul(A,I_N)
    deallocate(I_N)
#ifdef DEBUG
    write(*,'((a,x),2f8.2)')," S_ens minmax ",minval(S), maxval(S)
    write(*,'((a,x),2f8.2)')," A_ens minmax ",minval(A), maxval(A)
#endif

   if(verbose) then
            do i=1,nrobs
            !print *, i,R(i,i)*nrens
           enddo
      endif
   if (nrobs > 1) then
!      R=float(nrens)*R+matmul(S,transpose(S))
      call CPU_TIME (tarray(1))
      write(*,'(a40,f10.1,2i8,a11,i10)'),      "1st dgemm(nrobs, nrobs, nrens): start", tarray(1), nrobs, nrens, ' MFLOPs ~ ', nrobs* nrobs* nrens /1000000
      call sgemm('n','t', nrobs, nrobs, nrens, real(1.0), S, nrobs, S, nrobs, real(nrens), R, nrobs)
      call CPU_TIME (tarray(2))
      write(*,'(a40,2f10.1)'), "1st dgemm(nrobs, nrobs, nrens): done ", tarray(2), tarray(2) - tarray(1)
      
!EnKF call dgemm('n','t', nrobs, nrobs, nrens, 1.0, S, nrobs, S, nrobs, float(nrens), R, nrobs)

      allocate (U(nrobs,nrobs) , stat=ierr )
      allocate (V(nrobs,nrobs) , stat=ierr )
      allocate (sig(nrobs) , stat=ierr )
      lwork=2*max(3*nrobs+nrobs,5*nrobs)
      allocate(work(lwork), stat=ierr)
      sig=0.d0
      !R=U*sigma*V
      !note that V=transpose(V) ....... Nasty

      call CPU_TIME (tarray(1))
      write(*,'(a40,f10.1,i8,a11,i8)'),          "2nd dgesvd(nrobs, nrobs, nrobs): start", tarray(1), nrobs, ' MFLOPs ~ ', nrobs*nrobs*nrobs/1000000
!$OMP CRITICAL
      call sgesvd('A', 'A', nrobs, nrobs, R, nrobs, sig, U, nrobs, V, nrobs, work, lwork, ierr)
!$OMP END CRITICAL
      call CPU_TIME (tarray(2))
      write(*,'(a40,2f10.1)'), "2nd dgesvd(nrobs, nrobs, nrobs): done ", tarray(2), tarray(2) - tarray(1)
      if (ierr /= 0) then
         print *,'ierr from call dgesvd= ',ierr
         stop
      endif

      if(verbose) then
         open(10,file='sigma.dat')
            do i=1,nrobs
               write(10,'(i5,g12.3)')i,sig(i)
            enddo
         close(10)
      endif

      sigsum=sum( sig(1:nrobs) )
      sigsum1=real(0.0)
   ! Significant eigenvalues. digma=sigma^-1
      nrsigma=0
      do i=1,nrobs                 ! singular values are in descending order
         if (sigsum1/sigsum < real(0.999)) then !using all the singular value until we
                                          !0.999 %
            nrsigma=nrsigma+1
            sigsum1=sigsum1+sig(i)
            sig(i) = 1.0/sig(i)
         else
            sig(i:nrobs)=0.d0
            exit
         endif
      enddo

      !if (verbose) then
      !   write(*,'(a,i5,g13.5)') ' dominant sing. values and share ',nrsigma,sigsum1/sigsum
      !   write(*,'(8g12.3)')dble(1.)/sig(1:nrsigma), sig(nrsigma+1:nrobs)
      !endif

      allocate (X1(nrobs,nrobs))
      do i=1,nrobs
      do j=1,nrobs
         X1(i,j)=sig(i)*U(j,i)
      enddo
      enddo
      deallocate(sig)

      allocate (X2(nrobs,1))
!     X2=X1*D : (nrobs,nrobs)*(nrobs,1)
      call CPU_TIME (tarray(1))
      write(*,'(a40,f10.1,i8,a11,i8)'),      "3ed dgemm(nrobs, nrobs): start", tarray(1), nrobs, 'MFLOPs ~ ', nrobs*nrobs*nrobs/1000000
      call sgemm('n','n',nrobs,int(1,8),nrobs,real(1.0),X1,nrobs,D ,nrobs,real(0.0),X2,nrobs)
      call CPU_TIME (tarray(2))
      write(*,'(a40,2f10.1)'), "3ed dgemm(nrobs, nrobs): done ", tarray(2), tarray(2) - tarray(1)
      deallocate(X1) 

!     X3=transpose(transpose(V))*X2  : (nrobs,nnrobs)*(nrobs,1)
       allocate (X3(nrobs,1), stat=ierr)
       call CPU_TIME (tarray(1))
       write(*,'(a40,f10.1,i8,a11,i8)'),          "4th dgemm(nrobs, nrobs, nrobs): start", tarray(1), nrobs,  'MFLOPs ~ ',nrobs*nrobs*nrobs/1000000
       call sgemm('t','n',nrobs,int(1,8)    ,nrobs,real(1.0),V ,nrobs,X2,nrobs,real(0.0),X3,nrobs)
       call CPU_TIME (tarray(2))
       write(*,'(a40,2f10.1)'), "4th dgemm(nrobs, nrobs, nrobs): done ", tarray(2), tarray(2) - tarray(1)
!ENKF  call dgemm('t','n',nrobs,nrens,nrobs,1.0,V ,nrobs,X2,nrobs,0.0,X3,nrobs)
      deallocate(V)
      deallocate(X2)

   else
!      print *,'BEWARE ; ONLY 1 OBS FOUND !!!!!!!'
       allocate (X3(nrobs,1), stat=ierr)
      oneobs=matmul(S,transpose(S))+R*real(nrens)
      print *,'oneobs: ',oneobs(1,1)
      X3(:,1)=D/oneobs(1,1)
   endif
!
   if (2_8*ndim*nrobs < 1_8*nrens*(nrobs+ndim)) then
!    Code for few observations ( m<nN/(2n-N) )
!    Representer option 
      if (verbose) print * ,'analysis: Representer approach is used'
      allocate (Reps(ndim,nrobs))

!    Reps=matmul(A,transpose(S))  :  (ndim,nrens)*(nrobs,nrens)^T = (ndim,nrobs)
      call sgemm('n','t',ndim,nrobs,nrens,real(1.0),A,ndim,S,nrobs,real(0.0),Reps,ndim)
!
! Remove mean of Representers
    allocate (I_N(nrobs,nrobs))
    I_N=-1./real(nrobs)
    do i=1, nrobs
       I_N(i,i)=I_N(i,i)+1.
    end do
!
!   Reps = matmul(Reps,I_N)
!!!!!    call dgemm('n','n',ndim,nrobs,nrobs,1.0,Reps,ndim,I_N,nrobs,0.0,Reps,ndim)
    deallocate(I_N)
!
!    psi=psi+matmul(Reps,X3)
! EnKF      call dgemm('n','n',ndim,nrens,nrobs,1.0,Reps,ndim,X3,nrobs,1.0,A,ndim)
      allocate (X3(nrobs,1), stat=ierr)
      call sgemv('n',ndim,nrobs,real(1.0),Reps,ndim,X3,int(1,8),real(1.0),psi,int(1,8)) ! single step operation
      deallocate(Reps)

      tag2(1:2)='X3'
      open(10,file='X.uf',form='unformatted')
         write(10)tag2,nrens,nrobs,X3,S
      close(10)

   else
      allocate(X4(nrens))
!      X4=matmul(transpose(S),X3)    : (nrens,nrobs)*(nrobs,1)
       call CPU_TIME (tarray(1))
       write(*,'(a40,f10.1,2i8,a11,i8)'),           "5th dgemm(nrens, nrobs, nrobs): start", tarray(1), nrens, nrobs, 'MFLOPs ~ ',nrobs*nrobs*nrens/1000000
       call sgemm('t','n',nrens,int(1,8)    ,nrobs,real(1.0),S,nrobs,X3,nrobs,real(0.0),X4,nrens)
       call CPU_TIME (tarray(2))
       write(*,'(a40,2f10.1)'), "5th dgemm(nrens, nrobs, nrobs): done ", tarray(2), tarray(2) - tarray(1)
!EnKF  call dgemm('t','n',nrens,nrens,nrobs,1.0,S,nrobs,X3,nrobs,0.0,X4,nrens)
     allocate (I_N(nrens,nrens))
      I_N=-1.0/real(nrens)
      do i=1,nrens
         I_N(i,i)=I_N(i,i)+1.0
      enddo

      allocate(X5(nrens))
      !in order to have A' we center X4: A'*X4=A*I_N*X4=A*X5
      X5=matmul(I_N,X4)
      deallocate(I_N)
      deallocate(X4)
      deallocate(X3)
      
!psi=psi+A*X5    :  (ndim,nrens)*(nrens)=ndim
!      call dgemm('n','n',ndim,1,nrens,1.0,A,ndim,X5,nrens,1.0,psi,ndim)
      call CPU_TIME (tarray(1))
      write(*,'(a40,f10.1,2i8,a11,i8)'),      "6th dgemv(ndim,nrens): start", tarray(1), ndim, nrens, 'MFLOPs ~ ',ndim*nrens/1000000
      call sgemv('n',ndim,nrens,real(1.0),A,ndim,X5,int(1,8),real(1.0),psi,int(1,8)) ! single step operation
      call CPU_TIME (tarray(2))
      write(*,'(a40,2f10.1)'), "6th dgemv(ndim,nrens): done ", tarray(2), tarray(2) - tarray(1)

   deallocate(X5)
   if(verbose) print *,'Analysis, TEM(1) =', psi(target)
!
   endif 
#ifdef DEBUG
    write(*,*), "EnOI on", ndim,nrens,nrobs
    write(*,'((a,x),2f9.2)')," psi_ens maxval ",minval(psi), maxval(psi)
    write(*,'((a,x),2f9.2)')," A_ens minmax ",minval(A), maxval(A)
    write(*,'((a,x),2f9.2)')," D_ens minmax ",minval(D), maxval(D)
    write(*,'((a,x),2f9.1)')," R_ens minmax ",minval(R), maxval(R)
    write(*,'((a,x),2f9.2)')," S_ens minmax ",minval(S), maxval(S)
#endif

end subroutine analysis2_EnOI
!=====================
!     Auxiliary routine: printing a matrix.
!=======================
      SUBROUTINE PRINT_MATRIX( DESC, M, N, A, LDA )
      CHARACTER*(*)    DESC
      INTEGER*8          M, N, LDA
      REAL*8 A( LDA, * )

      INTEGER          I, J
      WRITE(*,*)
      WRITE(*,*) DESC
      DO I = 1, M
         WRITE(*,9998) ( A( I, J ), J = 1, N )
      END DO

 9998 FORMAT( 11(:,1X,F6.2) )
      RETURN
      END
