#define DEBG
program argo_preproc
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none

    type(FileHandler_NC) :: handler
    type(DateTime) :: dat_st
    character(10) :: yyyymmdd


!for write Data
    real*4 , allocatable :: x1_o1(:),x2_o1(:)
    real*4 , allocatable :: t_o1(:,:), s_o1(:,:)

    real*4 , allocatable :: x1_o2(:),x2_o2(:)
    real*4 , allocatable :: t_o2(:,:), s_o2(:,:)

    real*4 , allocatable :: x1_o3(:),x2_o3(:)
    real*4 , allocatable :: t_o3(:,:), s_o3(:,:)

    real*4 , allocatable :: x1_o(:),x2_o(:)
    integer*4,allocatable:: i_o(:),j_o(:)
    real*4 , allocatable :: t_o(:,:), s_o(:,:)

    integer, allocatable :: sort_indx(:)

! Z grid    
    real*4, allocatable  ::   z(:)
    integer,allocatable  ::   nl_axis(:)  

    integer np1,np2,np3, np
!working variables
    integer i,j,k,n,id

    character(100)            :: in_fname1, in_fname2, in_fname3, fname, out_fname
    

    call get_args(  dat_st, in_fname1, in_fname2, in_fname3 )
    call get_argo_data(in_fname1, x1_o1, x2_o1, t_o1, s_o1, z, nl_axis, np1)    
    call get_argo_data(in_fname2, x1_o2, x2_o2, t_o2, s_o2, z, nl_axis, np2)
    call get_argo_data(in_fname3, x1_o3, x2_o3, t_o3, s_o3, z, nl_axis, np3)
    
    np = np1 + np2 + np3
    write(*,*), 'Merge np = ', np

    allocate(x1_o (np),x2_o (np), i_o (np), j_o (np), t_o(nl,np), s_o(nl,np), sort_indx(np))
    x1_o(1:np1)     = x1_o1(1:np1)
    x2_o(1:np1)     = x2_o1(1:np1)
    t_o(1:nl,1:np1)   = t_o1(1:nl,1:np1)
    s_o(1:nl,1:np1)   = s_o1(1:nl,1:np1)

    x1_o(       np1+1:np1+np2)     = x1_o2(1:np2)
    x2_o(       np1+1:np1+np2)     = x2_o2(1:np2)
    t_o(1:nl,   np1+1:np1+np2)   = t_o2(1:nl,1:np2)
    s_o(1:nl,   np1+1:np1+np2)   = s_o2(1:nl,1:np2)

    x1_o(       np1+np2+1:np1+np2+np3)  = x1_o3(    1:np3)
    x2_o(       np1+np2+1:np1+np2+np3)  = x2_o3(    1:np3)
    t_o(1:nl,   np1+np2+1:np1+np2+np3)  = t_o3(1:nl,1:np3)
    s_o(1:nl,   np1+np2+1:np1+np2+np3)  = s_o3(1:nl,1:np3)

    call sort_x12_var( x1_o, x2_o, t_o, nl, np, sort_indx )
    call x12_o_to_ij_o(x1_o, x2_o, i_o, j_o,  t_o, np, nl)
    yyyymmdd = dat_st % to_string()

    out_fname = 'merge.nc'
    fname='T_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "tem", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), t_o(1:nl,1:np), nl, np, z(1:nl), nl_axis(1:nl)  )
#ifdef DEBG
    write(*,*), 'SORT'
    write(*,'(a,i4,a,2f6.1)'), 'x1_o (',np,')=',minval(x1_o(1:np)),      maxval(x1_o(1:np))
    write(*,'(a,i4,a,2f6.1)'), 'x2_o (',np,')=',minval(x2_o(1:np)),      maxval(x2_o(1:np))
    write(*,'(a,2i3,a,2f5.1)'),'t_o(',np,1,')=',minval(t_o(1,1:np)),     maxval(t_o(1,1:np))
    write(*,'(a,2i3,a,2f5.1)'),'t_o(',np,1,')=',minval(s_o(1,1:np)),     maxval(s_o(1,1:np))
#endif
    do i=1,np
        t_o(1:nl, i) = s_o ( 1:nl, sort_indx(i)) ! use t_o as tmp array for sorting s_o
    enddo
    s_o = t_o
#ifdef DEBG
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',np,1,')=',s_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',np,10,')=',s_o(10,1:np)
#endif
    deallocate( t_o)
    fname='S_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "sal", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), s_o(1:nl,1:np), nl, np, z(1:nl), nl_axis(1:nl)  )
    deallocate( s_o, x1_o ,x2_o)


contains

subroutine get_argo_data(in_fname, x1_o,x2_o, t_o, s_o, z, nl_axis, np)
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none
    type(FileHandler_NC)    :: handler
    character(100), intent(IN) :: in_fname
    real*4, allocatable, intent(INOUT) ::  x1_o(:),x2_o(:)
    real*4, allocatable, intent(INOUT) ::  t_o(:,:), s_o(:,:)
    real*4, allocatable, intent(INOUT) ::  z(:)
    integer,allocatable, intent(INOUT) ::  nl_axis(:)
    integer, intent(INOUT) ::np

!for read Data
    real*4 , allocatable:: lon(:),lat(:)
    real*4 , allocatable:: TEMP(:,:),PSAL(:,:),Z_m(:,:)
    integer :: aNP,aNL
    real*4, parameter :: BAD_P = 99999.
!working vars    
    integer i,j,k,n,id
    real*4, external :: x1_shift
    if(.NOT. allocated(z)) then
        allocate ( z(1:nl), nl_axis(1:nl) )
        call o_ini_set_grid_z(z, nl_axis)
    endif
    call handler % open_file(in_fname)
    aNP =handler % get_dim_size("N_PROF")
    aNL =handler % get_dim_size("N_LEVELS")
    allocate(lat(aNP),lon(aNP),TEMP(aNL,aNP),PSAL(aNL,aNP),Z_m(aNL,aNP))
    lat(1:aNP)=0d0; lon(1:aNP)=0d0
    TEMP(1:aNL,1:aNP) = 0d0
    PSAL(1:aNL,1:aNP) = 0d0
    Z_m (1:aNL,1:aNP) = 0d0

    call handler % open_var("LATITUDE")
    call handler % get (  lat,(/1 /) , (/aNP /)  )
    call handler % open_var("LONGITUDE")
    call handler % get (  lon,(/1 /) , (/aNP /)  )
    call handler % open_var("TEMP")
    call handler % get (  TEMP, (/1, 1 /) , (/aNL, aNP /)  )
    call handler % open_var("PSAL")
    call handler % get (  PSAL, (/1, 1 /) , (/aNL, aNP /)  )
    call handler % open_var("PRES")
    call handler % get (  Z_m,  (/1, 1 /) , (/aNL, aNP /)  )
    call handler % close_file()
        
#ifdef DEBG
    write(*,'(a,i4,a,<anp>f5.1)'),'LAT(',aNP,')=',LAT(1:aNP)
    write(*,'(a,i3,a,<anp>f5.1)'),'LON(',aNP,')=',LON(1:aNP)
    write(*,'(a,2i3,a,5f7.1)'),'TEMP(',aNL,1,')=',TEMP(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'TEMP(',aNL,2,')=',TEMP(2,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PSAL(',aNL,1,')=',PSAL(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PSAL(',aNL,2,')=',PSAL(2,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PRES(',aNL,1,')=',Z_M(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PRES(',aNL,2,')=',Z_M(2,1:5)
    write(*,*),"BAD_P",BAD_P
#endif

! Convert PRES to Z 
    do i=1,aNL
        do j=1,aNP
            if(Z_m(i,j) /= BAD_P) Z_m(i,j) = Z_m(i,j)/(0.98*1.025)
        enddo
    enddo

!Check how many points in our region
    np=0
    do i=1,aNP
        if( x1_shift(real(Lon(i))) > min_x1 .and. x1_shift(real(Lon(i))) < max_x1 .and. &
                   Lat(i) > min_x2 .and.  Lat(i) < max_x2          .and. &
                   TEMP(1,i) /= BAD_P .and.  PSAL(1,i) /= BAD_P    .and. &
                   TEMP(1,i) <  MAX_T .and.  TEMP(1,i)  > MIN_T    .and. &
                   PSAL(1,i) <  MAX_S .and.  PSAL(1,i)  > MIN_S    .and. &
                   TEMP(55,i) <  MAX_T .and.  TEMP(55,i)  > MIN_T    .and. &
                   PSAL(55,i) <  MAX_S .and.  PSAL(55,i)  > MIN_S ) np = np + 1
    enddo
#ifdef DEBG
    write(*,'(a,i4,a,i4,a,i4)'),"np ",np,"from ", aNP, ", nl =", nl
#endif

!allocate assimilated data arrys
    allocate (x1_o (np), x2_o (np)) 
    allocate (t_o(nl,np), s_o(nl,np))

!Copy pnts in new arrays and vertical extrapolation
    n=1
    do i=1,aNP
        if( x1_shift(real(Lon(i))) > min_x1 .and. x1_shift(real(Lon(i))) < max_x1   .and. &
                   Lat(i) > min_x2 .and.  Lat(i) < max_x2                           .and. &
                   TEMP(1,i) /= BAD_P .and.  PSAL(1,i) /= BAD_P                     .and. & 
                   TEMP(1,i) <  MAX_T .and.  TEMP(1,i)  > MIN_T                     .and. &
                   PSAL(1,i) <  MAX_S .and.  PSAL(1,i)  > MIN_S                     .and. &
                   TEMP(nl_argo,i) <  MAX_T .and.  TEMP(nl_argo,i)  > MIN_T                   .and. &
                   PSAL(nl_argo,i) <  MAX_S .and.  PSAL(nl_argo,i)  > MIN_S )  then
            x1_o(n)=x1_shift(real(Lon(i)))
            x2_o(n)=Lat(i)
            t_o(1:nl,n) = BAD_P
            s_o(1:nl,n) = BAD_P
            do k=1,nl
                j = minloc(abs(Z_m(1:aNL,i)-z(k)), DIM = 1)
                !write(*,'(a,2i4,a,2f8.1)'),'Z(',i,k,')=' ,Z_m(j,i),z(k)
                if(       IsNan(TEMP(j,i))  .or. TEMP(j,i) >  MAX_T .or.    TEMP(j,i)  < MIN_T .or. &
                          IsNan(PSAL(j,i))  .or. PSAL(j,i) >  MAX_S .or.    PSAL(j,i)  < MIN_S ) then 
                    write(*,'(a,2i3,a,2f7.1)'),'BAD TS_o(',j,i,')=',      TEMP(j,i), PSAL(j,i)
                    exit
                endif
                t_o(k,n)=TEMP(j,i)
                s_o(k,n)=PSAL(j,i) - S_ref
            enddo !k=1,nl
            n=n+1
        endif
    enddo ! i=1,NP
#ifdef DEBG
    write(*,'(a,i3,a,2f7.1)'),'x1_o(',NP,')=',      minval(x1_o(1:np)),      maxval(x1_o(1:np))
    write(*,'(a,i4,a,2f7.1)'),'x2_o(',NP,')=',      minval(x2_o(1:np)),      maxval(x2_o(1:np))
    write(*,'(a,2i3,a,2f7.1)'),'T_o(',1,NL,')=',    minval(t_o(1,1:np)),     maxval(t_o(1,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'T_o(',7,NL,')=',    minval(t_o(7,1:np)),     maxval(t_o(7,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'S_o(',1,NL,')=',    minval(s_o(1,1:np)),     maxval(s_o(1,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'S_o(',7,NL,')=',    minval(s_o(7,1:np)),     maxval(s_o(7,1:np))
#endif

#ifdef DEBG
    write(*,'(a,i4,a,<np>f5.1)'), 'x1_o (',np,')=',x1_o(1:np)
    write(*,'(a,i4,a,<np>f5.1)'), 'x2_o (',np,')=',x2_o(1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',1,np,')=',t_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',7,np,')=',t_o(10,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',1,np,')=',s_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',7,np,')=',s_o(10,1:np)
#endif

end subroutine 
end program
