#define DEBG
program argo_preproc
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none

    type(FileHandler_NC) :: handler
    type(DateTime) :: dat_st
    character(10) :: yyyymmdd
!Z grid array
    real*4 Z(50)
!for read Data
    real*4 , allocatable:: lon(:),lat(:)
    real*4 , allocatable:: TEMP(:,:),PSAL(:,:),Z_m(:,:)
    integer :: aNP,aNL
    real*4,parameter :: BAD_P = 99999.
!Input and output files
    integer,parameter :: nl = 30
!for write Data
    real*8 , allocatable:: x1_o(:),x2_o(:)
    integer*8, allocatable:: i_o(:),j_o(:)
    real*8 , allocatable:: t_o(:,:), s_o(:,:)
    integer , allocatable:: sort_indx(:)
    integer np
!working variables
    integer i,j,k,n,id

    character(100)            :: in_fname, out_fname,fname
    real*8, external :: x1_shift

    call get_args(  in_fname, out_fname,   dat_st)
    call o_ini_set_grid_z(z)
    call handler % open_file(in_fname)
    aNP =handler % get_dim_size("N_PROF")
    aNL =handler % get_dim_size("N_LEVELS")
    allocate(lat(aNP),lon(aNP),TEMP(aNL,aNP),PSAL(aNL,aNP),Z_m(aNL,aNP))
    lat(1:aNP)=0d0; lon(1:aNP)=0d0
    TEMP(1:aNL,1:aNP) = 0d0
    PSAL(1:aNL,1:aNP) = 0d0
    Z_m (1:aNL,1:aNP) = 0d0

    call handler % open_var("LATITUDE")
    call handler % get (  lat,(/1 /) , (/aNP /)  )
    call handler % open_var("LONGITUDE")
    call handler % get (  lon,(/1 /) , (/aNP /)  )
    call handler % open_var("TEMP")
    call handler % get (  TEMP, (/1, 1 /) , (/aNL, aNP /)  )
    call handler % open_var("PSAL")
    call handler % get (  PSAL, (/1, 1 /) , (/aNL, aNP /)  )
    call handler % open_var("PRES")
    call handler % get (  Z_m,  (/1, 1 /) , (/aNL, aNP /)  )
    call handler % close_file()
        
#ifdef DEBG
    write(*,'(a,i4,a,<anp>f5.1)'),'LAT(',aNP,')=',LAT(1:aNP)
    write(*,'(a,i3,a,<anp>f5.1)'),'LON(',aNP,')=',LON(1:aNP)
    write(*,'(a,2i3,a,5f7.1)'),'TEMP(',aNL,1,')=',TEMP(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'TEMP(',aNL,2,')=',TEMP(2,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PSAL(',aNL,1,')=',PSAL(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PSAL(',aNL,2,')=',PSAL(2,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PRES(',aNL,1,')=',Z_M(1,1:5)
    write(*,'(a,2i3,a,5f7.1)'),'PRES(',aNL,2,')=',Z_M(2,1:5)
    write(*,*),"BAD_P",BAD_P
#endif

! Convert PRES to Z 
    do i=1,aNL
        do j=1,aNP
            if(Z_m(i,j) /= BAD_P) Z_m(i,j) = Z_m(i,j)/(0.98*1.025)
        enddo
    enddo

!Check how many points in our region
    np=0
    do i=1,aNP
        if( x1_shift(dble(Lon(i))) > min_x1 .and. x1_shift(dble(Lon(i))) < max_x1 .and. &
                      Lat(i) > min_x2 .and.  Lat(i) < max_x2          .and. &
                   TEMP(1,i) /= BAD_P .and.  PSAL(1,i) /= BAD_P ) np = np + 1
    enddo
#ifdef DEBG
    write(*,'(a,i4,a,i4)'),"np ",np,"from ", aNP
#endif

!allocate assimilated data arrys
    allocate(x1_o (np),x2_o (np), i_o (np), j_o (np), t_o(nl,np), s_o(nl,np), sort_indx(np))

!Copy pnts in new arrays and vertical extrapolation
    n=1
    do i=1,aNP
        if( x1_shift(dble(Lon(i))) > min_x1 .and. x1_shift(dble(Lon(i))) < max_x1 .and. &
                      Lat(i) > min_x2 .and.  Lat(i) < max_x2          .and. &
                   TEMP(1,i) /= BAD_P .and.  PSAL(1,i) /= BAD_P ) then
            x1_o(n)=x1_shift(dble(Lon(i)))
            x2_o(n)=Lat(i)
            t_o(1:nl,n) = BAD_P
            s_o(1:nl,n) = BAD_P
            do k=1,nl
                j = minloc(abs(Z_m(1:aNL,i)-z(k)), DIM = 1)
                !write(*,'(a,2i4,a,2f8.1)'),'Z(',i,k,')=' ,Z_m(j,i),z(k)
                if(       IsNan(TEMP(j,i))  .or. TEMP(j,i) >  MAX_T .or.    TEMP(j,i)  < MIN_T .or. &
                          IsNan(PSAL(j,i))  .or. PSAL(j,i) >  MAX_S .or.    PSAL(j,i)  < MIN_S ) exit
                t_o(k,n)=TEMP(j,i)
                s_o(k,n)=PSAL(j,i) - S_ref
            enddo !k=1,nl
            n=n+1
        endif
    enddo ! i=1,NP
#ifdef DEBG
    write(*,'(a,i3,a,2f7.1)'),'x1_o(',NP,')=',      minval(x1_o(1:np)),      maxval(x1_o(1:np))
    write(*,'(a,i4,a,2f7.1)'),'x2_o(',NP,')=',       minval(x2_o(1:np)),      maxval(x2_o(1:np))
    write(*,'(a,2i3,a,2f7.1)'),'T_o(',1,NL,')=',    minval(t_o(1,1:np)),     maxval(t_o(10,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'T_o(',10,NL,')=',   minval(t_o(10,1:np)),    maxval(t_o(10,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'S_o(',1,NL,')=',    minval(s_o(1,1:np)),     maxval(s_o(1,1:np))
    write(*,'(a,2i3,a,2f7.1)'),'S_o(',10,NL,')=',   minval(s_o(10,1:np)),    maxval(s_o(10,1:np))
#endif

#ifdef DEBG
    write(*,'(a,i4,a,<np>f5.1)'), 'x1_o (',np,')=',x1_o(1:np)
    write(*,'(a,i4,a,<np>f5.1)'), 'x2_o (',np,')=',x2_o(1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',1,np,')=',t_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',10,np,')=',t_o(10,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',1,np,')=',s_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',10,np,')=',s_o(10,1:np)
#endif
    call sort_x12_var( x1_o, x2_o, t_o, nl, np, sort_indx )
    call x12_o_to_ij_o(x1_o, x2_o, i_o, j_o, np )
    yyyymmdd = dat_st % to_string()
    fname='T_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "tem", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), t_o(1:nl,1:np), nl, np  )
#ifdef DEBG
    write(*,*), 'SORT'
    write(*,'(a,i4,a,<np>f5.1)'), 'x1_o (',np,')=',x1_o(1:np)
    write(*,'(a,i4,a,<np>f5.1)'), 'x2_o (',np,')=',x2_o(1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',np,1,')=',t_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'t_o(',np,10,')=',t_o(10,1:np)
#endif
    do i=1,np
        t_o(1:nl, i) = s_o ( 1:nl, sort_indx(i)) ! use t_o as tmp array for sorting s_o
    enddo
    s_o = t_o
#ifdef DEBG
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',np,1,')=',s_o(1,1:np)
    write(*,'(a,2i3,a,<np>f5.1)'),'s_o(',np,10,')=',s_o(10,1:np)
#endif
    deallocate( t_o)
    fname='S_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "sal", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), s_o(1:nl,1:np), nl, np  )
    deallocate( s_o, x1_o ,x2_o)
end program
