#define DEBG
#define BAD_P -999
module data_preproc
!Domain settings
!#define OCN_CDO "OCN_180x84_T_cdo"
#define OCN_CDO "OCN_1440x720_T_cdo"
    real*4,parameter::min_x1=-115, max_x1=  245
    real*4,parameter::min_x2= -85, max_x2=  90
    real*4,parameter:: S_ref =0. ! t_c in mode is ( - S_ref + anomaly )
!for checking T and S limits
    real*4, parameter:: MAX_T = 60, MIN_T = -4
    real*4, parameter:: MAX_S = 60, MIN_S =  0
    real*4, parameter:: MAX_SSL = 3, MIN_SSL =  -3
!Input and output files
    integer, parameter :: nl = 11      ! Number of Z levels in Argo data for analyse  
    !integer, parameter :: nl_min = 29 ! Low Z level of INMIO model for assim
    integer, parameter :: nl_min = 25 ! Low Z level of INMIO model for assim
    integer, parameter :: nl_argo = 55 ! Low Z level of INMIO model for assim
end module

module mod_x1_indx
        type x1_indx
            real*4 x1
            integer indx
        end type
        interface assignment (=)
            module procedure assign_x1_indx
        end interface
    contains
    subroutine assign_x1_indx(this, from)
        type (x1_indx), intent(out) :: this
        type (x1_indx), target, intent(in) :: from
        this%x1 = from%x1
        this%indx = from%indx
    end subroutine
end module mod_x1_indx

!============================
!       model gird is [x1_ssc_west_south : x1_west_south +x1_ssc_size]=M_G usualy
!       [-115 : 245 ]
!       if x1 is an element of Model_Grid, then x1_shift=x1
!       else x1_shifted = x1 - x1_ssc_size
!=============================
        real*4 function  x1_shift(x1)
            implicit none
            real*4, parameter:: x1_ssc_west_south = -115.
            real*4, INTENT(IN) :: x1

            if ( (x1.ge.x1_ssc_west_south) .and. x1 .le. (x1_ssc_west_south+360.0) )then
                x1_shift = x1 ! x1 if [-115: 245 ]
            else if ( x1 < x1_ssc_west_south .and. x1 .ge. -180.0 ) then 
                x1_shift = x1 +  360.0 ! x1+360 if [ -180 ; -115 )
            else if ( x1 > (x1_ssc_west_south+360.0) .and. x1 .le. 360.0) then
                x1_shift = x1 -360.0 ! x1-360 if [ 245; 360 ]
            else
                write(*,*), 'Somthing wrong with your input X1',x1,x1_shift,x1_ssc_west_south,x1_ssc_west_south+360.0
                pause 
            endif

        end function

!============================
!       read comman line args
! 1st is NC filename (necessary)
! 2nd is date (optional)
! If date is not set will try to read from NC file
!=============================

subroutine  get_args(  dat_st, fname1, fname2, fname3 )
!                      argo.nc   argo_prep.nc       2014-10-17
use datetime_module
implicit none
! Read Command line args
    integer::narg,cptArg !#of arg & counter of arg
    character(len=19)::argname !Arg name
!Date and time
    type(DateTime) :: dat_st
!NC filename
    character(len=100),intent(out) ::fname1,fname2
    character(len=100), intent(out), optional :: fname3
!Check if any arguments are found
    narg=command_argument_count()
    if(narg <  2 ) then
        write(*,*),"Example: ./preproc 2014-10-17 infile.nc outfile.nc "
        write(*,*),"ARGO: ./preproc 2014-10-17 infile1.nc infile2.nc infile3.nc "
        stop
    else
        call get_command_argument( 1,   argname)
        write(*,*), argname
        call get_command_argument( 2,   fname1)
        write(*,*), fname1
        call get_command_argument( 3,   fname2)
        write(*,*), fname2
        if ( narg == 4 ) then
            call get_command_argument( 4,   fname3)
            write(*,*), fname3
        endif
    
        argname(11:19)=" 00:00:00"
        dat_st = DateTime(argname)
        write(*,*) , dat_st % to_string()
    endif
end subroutine

subroutine  save_obs_data( out_fname, var_name, dat_st, x1_o, x2_o, i_o, j_o, var_o, nl, np, z, nl_axis )
    use file_handler_nc_module
    use netcdf
    implicit none
    type(FileHandler_NC) :: handler
    character(100)          :: out_fname
    character(3)            ::var_name
    type(DateTime) :: dat_st
    !Coordiant
    real*4, intent(INOUT) ::    x1_o(1:np),x2_o(1:np)
    integer*4, intent(INOUT) :: i_o(1:np),j_o(1:np)
    real*4, intent(INOUT) ::    var_o(1:nl,1:np)
    integer, intent(IN) :: np, nl
    real*4,  intent(INOUT),optional ::  z(1:nl)
    integer, intent(INOUT),optional ::  nl_axis(1:nl)
    call handler % create_file(out_fname)
    call handler % create_dim("nl",nl)
    call handler % create_dim("np",np)
    call handler % put_att("date", dat_st % to_string(), is_global = .TRUE.)
    call handler % put_att("var_name", var_name, is_global = .TRUE.)

    call handler % create_var     ("x1_o", "real4", "np" )
    call handler % put( x1_o(1:np), lo =(/ 1/) , hi =(/ np/) )
    call handler % create_var     ("x2_o", "real4", "np"  )
    call handler % put( x2_o(1:np), lo = (/ 1/) , hi =(/ np/) )
    call handler % create_var     ("i_o", "int", "np" )
    call handler % put( i_o(1:np), lo =(/ 1/) , hi =(/ np/) )
    call handler % create_var     ("j_o", "int", "np" )
    call handler % put( j_o(1:np), lo =(/ 1/) , hi =(/ np/) )
    call handler % create_var     ("var_o", "real4", "nl","np"  )
    call handler % put( var_o(1:nl,1:np), lo = (/ 1, 1/) , hi =(/ nl, np/) )
    if ( present(nl_axis) .and. nl > 1) then
        call handler % create_var     ("nl_o", "int", "nl" )
        call handler % put(nl_axis(1:nl), lo =(/ 1/) , hi =(/ nl/) )
        call handler % create_var     ("z_o", "real4", "nl" )
        call handler % put( z(1:nl), lo =(/ 1/) , hi =(/ nl/) )
    endif
    call handler % close_file()

end subroutine


subroutine  x12_o_to_ij_o( x1_o, x2_o, i_o, j_o, var_o, np, nl )

    use file_handler_nc_module
    use utils_module
    implicit none
    real*4 , allocatable:: x1_b(:,:),x2_b(:,:),x1_b_ok(:)
    integer*4 , allocatable:: j_ok(:)
    real*4, intent(INOUT) ::  x1_o(1:np), x2_o(1:np)
    integer*4, intent(INOUT) :: i_o(1:np), j_o(1:np)
    real*4, intent(INOUT) :: var_o(1:nl,1:np)
    integer np,nl
    type(FileHandler_NC) :: handler
    !Model GRID FIle
    character(100) :: grid_fname = TRIM(OCN_CDO//".nc")
    integer il,jl,i,j,n, n_ok
    integer i_unk, j_unk

    write(*,*), "Take mesh form", grid_fname
    call handler % open_file(grid_fname)
    il = handler % get_dim_size("grid_xsize")
    jl = handler % get_dim_size("grid_ysize")
    allocate(x1_b(il,jl),x2_b(il,jl),j_ok(il),x1_b_ok(il))    
    call handler % open_var("grid_center_lat")
    call handler % get (  x2_b, (/1, 1/) , (/il, jl/)  )
    call handler % open_var("grid_center_lon")
    call handler % get (  x1_b, (/1, 1/) , (/il, jl/)  )
    call handler % close_file()

#ifdef DEBG
!    write(*,'(a,2i9,a,10f8.2)'),'x1_b(',il,jl,')=', x1_b(1:5,1:2)
!    write(*,'(a,2i9,a,10f8.2)'),'x2_b(',il,jl,')=', x2_b(1:5,1:2)
#endif
    do n=1,np
        !write(*,'(a,2f8.2)'),'x12  =', x1_o(n), x2_o(n)
        do i=1,il
            j_ok (i) = search_binary(real(x2_o(n)), x2_b(i, 1:jl), 1, jl)
            if (j_ok(i) /= 0 ) then
                x1_b_ok(i) = x1_b(i,j_ok(i))
            else
                x1_b_ok(i) = BAD_P
            endif          
        enddo
        !write(*,"(a,<il-1>f6.1)"),'x1_b_ok =', x1_b_ok(1:il)
        i_o(n) = int(search_linear (real(x1_o(n)), x1_b_ok(1:il),1, il),4)
        j_o(n) = int(j_ok(i_o(n)),4)
        !write(*,'(a,4f8.2,a,3i6)'),'x12  =', x1_o(n), x2_o(n), x1_b(i_o(n), j_o(n)), x2_b(i_o(n), j_o(n)),' i,j',i_o(n), j_o(n), n
    enddo

    do n=1,np-1
        if ( i_o (n)  == - 999) cycle
        i_unk = i_o (n) 
        j_unk = j_o (n)
        do i = n+1, np
            if( (abs(i_unk - i_o (i)) + abs(j_unk - j_o (i)) ).le. 1 ) then
                i_o (i) = BAD_P
                j_o (i) = BAD_P
                x1_o (i) = BAD_P.
                x2_o (i) = BAD_P.
                var_o(1:nl,i) = BAD_P
            endif
        enddo
    enddo
    do n=1,np-1
        if ( i_o(n) == BAD_P ) then
            do i = n+1, np
                if( i_o(i) /= BAD_P ) then
                    i_o  (n) = i_o  (i)
                    j_o  (n) = j_o  (i)
                    x1_o (n) = x1_o (i)
                    x2_o (n) = x2_o (i)
                    var_o(1:nl,n) = var_o(1:nl,i)
                    i_o  (i) = BAD_P
                    j_o  (i) = BAD_P   
                    x1_o (i) = BAD_P
                    x2_o (i) = BAD_P
                    var_o(1:nl,i) = BAD_P
                    n_ok = n
                    exit
                endif
            enddo
        endif
        
    enddo
    write(*,*  ),n_ok, 'n_ok ',np, nl
    np = n_ok
    deallocate(x1_b,x2_b,j_ok,x1_b_ok)
end subroutine




!===================================
! Define grid for vertical direction
! Define constant z_axis() and work array z() from it
!===================================
subroutine o_ini_set_grid_z(z, nl_axis)
    use data_preproc
    implicit none
    integer k
    real z_axis(50)
    real z(nl)
    integer nl_axis(nl)
    z_axis(1) = 3.
    z_axis(2) = 9.
    z_axis(3) = 15.
    do k =  4,  9
        z_axis( k)=   15.  +  10.*(k-3)
    end do
    do k =  10,  11
        z_axis( k)=  90. +  15.*(k-10)
    end do
    do k =  10,  15
        z_axis( k)=  90. +  15.*(k-10)
    end do
    z_axis(16)=  185.
    z_axis(17)=  205.
    z_axis(18)=  230.
    z_axis(19)=  255.
    z_axis(20)=  280.
    z_axis(21)=  310.
    z_axis(22)=  350.
    z_axis(23)=  400.
    z_axis(24)=  475.
    z_axis(25)=  550.
    z_axis(26)=  650.
    z_axis(27)=  775.
    z_axis(28)=  900.
    z_axis(29)= 1050.
    z_axis(30)= 1250.
    do k = 31,  50
        z_axis( k)= 1250. + 250.*(k-30)
    end do

    do k = 1, nl
        nl_axis(k) = (1 + nl_min * (k-1) / (nl-1)  )
        !nl_axis(k) = k
        z(k) = z_axis ( nl_axis(k) )
        write(*,'(a,f6.1,2i4)') , 'Z = ', z_axis ( nl_axis(k) ), nl_axis(k), k
    enddo

end subroutine




!===================================
! Sort Arrays x1_o, x2_o, var_o
! according x1_o increas order
!===================================
subroutine sort_x12_var( x1_o, x2_o, var_o, nl, np, sort_indx )
    use mod_x1_indx
    implicit none

    integer, parameter :: QSORT_THRESHOLD = 1000000
    type( x1_indx ) :: HOLD
    type( x1_indx ),allocatable :: array(:)
    !Coordiant
    real*4 x1_o(1:np),x2_o(1:np)
    real*4 var_o(1:nl,1:np)
    integer sort_indx(1:np)
    real*4,allocatable :: x_tmp(:)
    real*4,allocatable :: var_tmp(:,:)
    integer np, nl
    integer i
    allocate( array(1:np), x_tmp(1:np) ,var_tmp(1:nl,1:np))
    do i=1,np
        array(i) % x1 = x2_o(i)
        array(i) % indx = i
        !write(*,*),"before sort", x1_o(i), sort_indx (i)
    enddo
    HOLD=array(1)
    call QSORT_DATA()
    do i=1,np
        x2_o (i) = array(i) % x1
        sort_indx (i) = array(i) % indx
        !write(*,*),"sort", x1_o(i), sort_indx (i)
        x_tmp(i) = x1_o ( array(i) % indx )
        var_tmp(1:nl,i) = var_o ( 1:nl, array(i) % indx )
    enddo

    x1_o = x_tmp
    var_o = var_tmp

    deallocate( array, x_tmp, var_tmp )
     CONTAINS

    subroutine QSORT_DATA
        include "qsort_inline.f90"
    end subroutine QSORT_DATA

    logical function QSORT_COMPARE(a,b)
        integer, intent(in) :: a,b
        QSORT_COMPARE = ( array(a)%x1 < array(b)%x1 )
    end function QSORT_COMPARE

end subroutine



