#define DEBG
program argo_profile
    use file_handler_nc_module
    use netcdf
    use data_preproc
    use IFPORT
    implicit none

    type(FileHandler_NC) :: handler
    type(DateTime) :: dat_st
    character(10) :: yyyymmdd
!Z grid array
    real*4 Z(50)

    real*4,parameter :: BAD_P = 99999.
!Input and output files
    integer,parameter :: nl = 30
!for write Data
    real*8 , allocatable:: x1_o(:),x2_o(:)
    real*8 , allocatable:: t_o(:,:) ! argo profile
    real*8 , allocatable:: t_b(:,:) ! background with assimilation
    real*8 , allocatable:: t_m(:,:) ! only model 
    real*8 , allocatable:: wts(:)   ! wts for finding best Drifter
    integer np
!working variables
    integer i,j,k,n,id,n_best(1)

    character(100)            :: in_fname, in2_fname,fname
    call get_args(  in_fname, in2_fname,   dat_st)
    call o_ini_set_grid_z(z)

    call handler % open_file(in_fname)
    np = handler % get_dim_size("np")

    allocate(x1_o(np),x2_o(np),t_o(1:nl,1:np),t_b(1:nl,1:np),t_m(1:nl,1:np),wts(1:np))
    call handler % open_var("x2_o")
    call handler % get (  x2_o,(/1/) , (/np/)  )
    call handler % open_var("x1_o")
    call handler % get (  x1_o,(/1/) , (/np/)  )
    call handler % open_var("var_o")
    call handler % get (  t_o,(/1,1/) , (/nl,np/)  )
    call handler % open_var("var_b")
    call handler % get (  t_b,(/1,1/) , (/nl,np/)  )
    call handler % close_file()
!get T_b from second file
    call handler % open_file(in2_fname)
    call handler % open_var("var_b")
    call handler % get (  t_m,(/1,1/) , (/nl,np/)  )
    call handler % close_file()
    wts = 0 
    do n=1,np
      do k=1,nl
        wts(n) = wts(n) + abs( ( t_o(k,n) - t_b(k,n)) / t_o(k,n) ) 
      enddo
      if ( wts(n) < 0.01 ) wts(n) = 99
    enddo
!find best argo drifter
    do n=1,np
      write(*,'(i8,x,3(f8.3,x))'), n, wts(n),x1_o(n),x2_o(n)
    enddo  
!    n_best = 57
    n_best = minloc(wts(1:np))
    
!Copy pnts in new arrays and vertical extrapolation
    write(*,'(a,i3,a,f7.1)'),'x1_o(',n_best(1),')=',      x1_o(n_best(1))
    write(*,'(a,i3,a,f7.1)'),'x2_o(',n_best(1),')=',      x2_o(n_best(1))
    write(*,'(a,2i3,a,2f7.1)'),'T_o(',1,NL,')=',    minval(t_o(1:nl,n_best(1))),     maxval(t_o(1:nl,n_best(1)))
    write(*,'(a,2i3,a,2f7.1)'),'T_b(',1,NL,')=',    minval(t_b(1:nl,n_best(1))),     maxval(t_b(1:nl,n_best(1)))
    write(*,'(a,2i3,a,2f7.1)'),'T_m(',1,NL,')=',    minval(t_m(1:nl,n_best(1))),     maxval(t_m(1:nl,n_best(1)))

    yyyymmdd = dat_st % to_string()
    write(fname, '(a,a,a,i2)'),'T_',yyyymmdd,"_",n_best(1)
    write(*,'(a8,x,3(a8,x))'), "z","t_o","t_b","t_m"
    do k=1,nl
      write(*,'(f8.1,x,3(f8.2,x))'), z(k),t_o(k,n_best(1)),t_b(k,n_best(1)),t_m(k,n_best(1))
    enddo 

    open (unit = 1, file = fname) 
    do k=1,nl
      write(1,'(f8.1,x,3(f8.2,x))'), z(k),t_o(k,n_best(1)),t_b(k,n_best(1)),t_m(k,n_best(1))
    enddo  
    close(1)
    deallocate( t_o,t_b,t_m, x1_o ,x2_o)
end program
