#define DEBG
program sst_preproc
    use file_handler_nc_module
    use netcdf
    use data_preproc
    implicit none

    type(FileHandler_NC) :: handler
    type(DateTime) :: dat_st
    character(10) :: yyyymmdd
!for read Data
    integer*4, allocatable:: lon(:),lat(:)
    integer*2, allocatable:: ssl(:)
    integer dimx
!for delete some data
    integer,parameter:: MAX_PNTS = -1, PNT_STEP = 1
    real*4, parameter:: scale_latlon = 1.e-06, scale_ssl =1.e-03

!for write Data
    real*4 , allocatable:: x1_o(:),x2_o(:)
    integer*4, allocatable:: i_o(:),j_o(:)
    real*4 , allocatable:: var_o(:,:)
    integer, allocatable:: sort_indx(:)
    integer np
!working variables    character(10) :: yyyymmdd
    integer i,n,id
    real*4 x1,x2, mean_ssl_obs
!Input and output files
    character(100)            :: in_fname, out_fname,fname

    real*4, external :: x1_shift
   call get_args(  dat_st, in_fname, out_fname )

    call handler % open_file(in_fname)
    dimx = handler % get_dim_size("time")
    allocate(lat(dimx),lon(dimx),ssl(dimx))
    write(*,*),'open_file ',TRIM(in_fname)
    call handler % open_var("latitude")
    call handler % get (  lat,(/1 /) , (/dimx /)    )
    write(*,*),'get lat '
    call handler % open_var("longitude")
    call handler % get (  lon,(/1 /) , (/dimx /)    )
    call handler % open_var("adt_filtered")
    call handler % get (  ssl,(/1 /) , (/dimx /)    )
    call handler % close_file()
#ifdef DEBG
    write(*,'(a,i9,a,10f8.2)'),'LAT(',dimx,')=',scale_latlon * lat(1:10)
    write(*,'(a,i9,a,10f8.2)'),'LON(',dimx,')=',scale_latlon * lon(1:10)
    write(*,'(a,i9,a,10f8.3)'),'SSL(',dimx,')=',scale_ssl    * ssl(1:10)
    write(*,*),"[",min_x1,max_x1,"]x[",min_x2,max_x2,"]"
#endif
    !ssl =0 ! DEBG 
    id = 0
    do i=1,dimx,PNT_STEP
        x1 = scale_latlon *lon(i);  x1 = x1_shift(x1); x2 = scale_latlon * lat(i)
        if ( id == MAX_PNTS ) exit
        if( x1 > min_x1 .and. x1 < max_x1 .and. x2 > min_x2 .and. x2 < max_x2 .and. &
            scale_ssl *ssl(i) > MIN_SSL .and. scale_ssl * ssl(i) < MAX_SSL ) then
            id = id + 1
        endif
    enddo
    np = id

#ifdef DEBG
    write(*,'(a,i9,a,10f8.2)'),'LAT(',dimx,')=',scale_latlon * lat(1:10)
    write(*,'(a,i9,a,10f8.2)'),'LON(',dimx,')=',scale_latlon * lon(1:10)
    write(*,'(a,i9,a,10f8.2)'),'SSL(',dimx,')=',scale_ssl    * ssl(1:10)

    write(*,*),"[",min_x1,max_x1,"]x[",min_x2,max_x2,"]"
#endif
    allocate(x1_o(np),x2_o(np),i_o(np),j_o(np),var_o(1,np), sort_indx(np))
    x1_o=0; x2_o = 0; var_o = 0;
    id=0
    do i=1,dimx,PNT_STEP
        x1 = scale_latlon *lon(i);  x1 = x1_shift(x1); x2 = scale_latlon * lat(i)
        if ( id == MAX_PNTS ) exit
        if( x1 > min_x1 .and. x1 < max_x1 .and. x2 > min_x2 .and. x2 < max_x2 .and. &
            scale_ssl *ssl(i) > MIN_SSL .and. scale_ssl * ssl(i) < MAX_SSL ) then
            id = id + 1
            x1_o(id) = x1
            x2_o(id) = x2
            var_o(1,id) = scale_ssl *ssl(i) 
        endif
    enddo

	call sort_x12_var( x1_o, x2_o, var_o, 1, np, sort_indx )
    call x12_o_to_ij_o(x1_o, x2_o, i_o, j_o,  var_o, np, 1)

    mean_SSL_obs = 0.
    do i=1,np
        mean_SSL_obs = mean_SSL_obs + var_o(1,i)
    enddo
    mean_SSL_obs = mean_SSL_obs / dble(np)
    var_o = var_o - mean_SSL_obs
#ifdef DEBG
    write(*,'(a,i9,a,10f8.2)'),'x1_o (',np,')=',x1_o(1:10)
    write(*,'(a,i9,a,10i8)'  ),'i_o  (',np,')=',i_o(1:10)
    write(*,'(a,i9,a,10f8.2)'),'x2_o (',np,')=',x2_o(1:10)
    write(*,'(a,i9,a,10i8)'  ),'j_o  (',np,')=',j_o(1:10)
    write(*,'(a,i9,a,10f8.3)'),'var_o(',np,')=',var_o(1,1:10)
    write(*,'(a,i9,a,f8.2)'),'MEAN_SSL( ',np,' ) = ', mean_SSL_obs
#endif
    yyyymmdd = dat_st % to_string()
    fname='SSL_'//yyyymmdd//"_"//out_fname
    call save_obs_data( fname, "ssl", dat_st, x1_o(1:np), x2_o(1:np), i_o(1:np), j_o(1:np), var_o(1,1:np), 1, np  )

	end program
