
#include "../comp_ocn/o_defines.f90"

#define DEBUG
#define _SEND_FREE( _ARRAY_NAME_ ) \
    call this % comm_ % sync(nst_service_id); \
    if(CompSplitter % is_first_rank()) call this % comm_ % put_info( ( _ARRAY_NAME_ ), COMM_GA_STATUS, "free");

#define _SEND_FULL( _ARRAY_NAME_ ) \
    call this % comm_ % sync( nst_service_id ); \
    if(CompSplitter % is_first_rank()) call this % comm_ % put_info( ( _ARRAY_NAME_ ), COMM_GA_STATUS, "full");

module service_nst_module
use datetime_module
use utils_module
use actions_module
use service_module
use event_module
use communicator_ga_module
use component_splitter_module
use model_info_module
use file_handler_nc_module
use cpl_comp_clients_module
use decomposition_module
#ifdef DEBUG
use debug_module
#endif
implicit none

type, extends(Service) :: service_nst
private

 contains
    !=======================
    ! ===== PUBLIC API =====
    procedure, public :: init => init_nst
    procedure, public :: destroy => destroy_nst
    procedure, public :: handle_request => handle_nst
    procedure, private :: set_nst_sizes               ! Get sizes for service nst from SEA and OCN
    procedure, private :: get_mesh_and_make_wts       ! Get x1(2)_ocn mesh arrays from OCN, make wts and deallocate
    procedure, private :: calc_wts_matrix             ! Prepare info for wts vector calculation
    procedure, private :: helper_for_calc_wts         ! Vector calcucaltion for wts
    procedure, private :: handler_get_ocn_data        ! Handler get data arrays from ocn
    procedure, private :: make_var_to_sea             ! Make boundary data arrys for sea
    procedure, private :: make_var_with_wts_sea       ! Calculate data for sea
    procedure, private :: handle_recv_nst             ! Handler send data arrays to sea
    procedure, private :: send_var_to_sea             ! Send boundary data arrys to sea
    ! ===== PUBLIC API =====
    !=======================
end type

!=============================================================
!   Common NST Array decomposition
!=============================================================

type(Decomposition) :: ocn_dec,sea_dec
type(Decomposition) :: nst_dec, nst_dec_vert,nst_dec_horz  ! SEA DECOMPOSITION in service_nst

integer :: ocn_il, ocn_jl, ocn_kl               ! OCN size
integer :: sea_il, sea_jl, sea_kl               ! SEA size
integer :: lo_ocn(3,2), hi_ocn(3,2)             ! LIMITS ARRAYS
integer :: lo_sea(3), hi_sea(3)                 ! LIMITS FOR SEA
integer :: sea_id, ocn_id, nst_service_id       ! ID parameters
integer :: nst_rnk, nst_size, nst_comm, ierr    ! For MPI communication

character(NAME_LEN)  x1_ocn_name, x2_ocn_name,var_ocn_name  !TMP GA Sended from OCN
character(NAME_LEN)  x1_sea_name, x2_sea_name,var_sea_name  !TMP GA Sended from and to SEA

!Arrays ocn (data from glodal domain) and sea (data fo local domain)
real(kind=main_kind), allocatable :: var_ocn_west(:,:,:,:) ,var_sea_west(:,:,:,:)
real(kind=main_kind), allocatable :: var_ocn_east(:,:,:,:) ,var_sea_east(:,:,:,:)
real(kind=main_kind), allocatable :: var_ocn_south(:,:,:,:),var_sea_south(:,:,:,:)
real(kind=main_kind), allocatable :: var_sea_north(:,:,:,:),var_ocn_north(:,:,:,:)
!matrix for interpolation
real(kind=main_kind), allocatable :: wts_matrix_west(:,:,:,:), wts_matrix_east(:,:,:,:)
real(kind=main_kind), allocatable :: wts_matrix_south(:,:,:,:), wts_matrix_north(:,:,:,:)
!model grids
real*8 , allocatable :: x1_ocn(:,:), x2_ocn(:,:)     !global domain
real*8 , allocatable :: x1_sea(:,:), x2_sea(:,:)     !local domain
real*8 , allocatable :: x1_wts_sea(:,:), x2_wts_sea(:,:) !useful info from sea
real*8 , allocatable :: x1_wts_ocn(:), x2_wts_ocn(:) !info for v from ocn
integer, allocatable :: x1_sea_ind(:,:), x2_sea_ind(:,:) !index for wts calc

!Time Calc Measured
type(DateTime) :: time_get
real*4 nst_time_total, nst_time_strt, nst_time_end

character(10)    :: inter_mode  !interpolation method name
integer :: wts_vector_size      !wts matrix vector point count, default = 4
logical :: wts_ready            !state of wts matrix
integer :: iiwest, iieast, jjsouth, jjnorth             !data split in NST service
integer, allocatable  :: start_point_for_sea_calc(:,:)  !list of points uses for wts matrix


integer :: test_1

 CONTAINS

subroutine init_nst(this)
    class(service_nst) :: this

    wts_ready = .False.

    !TODO get this info from other souce
    inter_mode = "blinx2y2z1"
    wts_vector_size = 4
    allocate(start_point_for_sea_calc(1:wts_vector_size,1:3))
    start_point_for_sea_calc = reshape((/0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0/),(/4,3/))
end subroutine

subroutine destroy_nst(this)
    class(service_nst) :: this
    deallocate(var_ocn_west)
    deallocate(var_sea_west)
    deallocate(var_ocn_east)
    deallocate(var_sea_east)
    deallocate(var_ocn_south)
    deallocate(var_sea_south)
    deallocate(var_ocn_north)
    deallocate(var_sea_north)
    deallocate(x1_sea_ind)
    deallocate(x2_sea_ind)
    deallocate(wts_matrix_west)
    deallocate(wts_matrix_east)
    deallocate(wts_matrix_south)
    deallocate(wts_matrix_north)
end subroutine

!========================================================================================
subroutine handle_nst(this, ev)
    class(service_nst) :: this
    type(Event), intent(in) :: ev

    !GA arrays for sending from and to comps
    x1_ocn_name  = TRIM("OCN_x1_t_SEND_NST")
    x2_ocn_name  = TRIM("OCN_x2_t_SEND_NST")
    x1_sea_name  = TRIM("SEA_x1_t_SEND_NST")
    x2_sea_name  = TRIM("SEA_x2_t_SEND_NST")

    select case (ev % action())
        case( "SEND_NST")
            if (.NOT. this % comm_ % is_registered(ev % ga_name())) then
                call this % try_register_service_ga(ev)
                call this % set_nst_sizes(ev)
                write(*,*),nst_rnk,"NST: Register array ",TRIM(ev % ga_name())
                _SEND_FREE ( ev % ga_name() )
            else if (   ev % ga_name()  == x1_ocn_name .or. ev % ga_name()  == x2_ocn_name .or. &
              ev % ga_name()  == x1_sea_name .or. ev % ga_name()  == x2_sea_name ) then
                if (wts_ready == .False.) then
                    call this % get_mesh_and_make_wts (ev)
                endif
            else if (   ev % ga_name()  == "OCN_t_f_SEND_NST" ) then
                call this % handler_get_ocn_data(ev, (/1,1/))
            endif

        case( "RECV_NST")
            if (.NOT. this % comm_ % is_registered(ev % ga_name() )) then
                call this % try_register_service_ga(ev)
            else if (   ev % ga_name()  == "SEA_t_f_nst_RECV_NST ") then
                call this % handle_recv_nst(ev, 1)
            endif
        case("ERROR")
            call this % report_handle(ev)
            call exit(1)
    end select
end subroutine

!======================================
!  get decomposition, size and MPI info for ocean and sea
!  create data decomposition for future calculation
!======================================
subroutine set_nst_sizes(this, ev)
    use decomposition_module
    use datetime_module
    class(service_nst) :: this
    type(Event), intent(in) :: ev

    !#ifdef DEBUG
    !write(*,"(i2,a)"),nst_rnk,"NST: set_nst_sizes"
    !#endif

    if (   ev % ga_name()  == x1_ocn_name ) then
        nst_size = CompSplitter %  comm_local_size()
        nst_rnk  = CompSplitter %  rank_local ()
        nst_comm = CompSplitter %  comm_local ()
        nst_service_id = CompSplitter % i_am_id()

        ocn_dec = ModelsInfo(CompSplitter % comp_id( ev % src() )) % decomp()
        ocn_il = ocn_dec % i_size()
        ocn_jl = ocn_dec % j_size()
        ocn_kl = ocn_dec % k_size()
        ocn_id = CompSplitter % comp_id(ev % src() )
    endif
    if (   ev % ga_name()  == x1_sea_name ) then
        sea_dec = ModelsInfo(CompSplitter % comp_id( ev % src() )) % decomp()
        sea_il = sea_dec % i_size()
        sea_jl = sea_dec % j_size()
        sea_kl = sea_dec % k_size()
        sea_id = CompSplitter % comp_id(ev % src() )

        nst_dec      = Decomposition(sea_il, sea_jl, sea_kl , "1D", nst_size, nst_rnk )
        nst_dec_vert = Decomposition(wts_vector_size, sea_jl, sea_kl , "1D", nst_size, nst_rnk )  ! 4 points for wts
        nst_dec_horz = Decomposition(wts_vector_size, sea_il, sea_kl , "1D", nst_size, nst_rnk )  ! 4 points for wts
        call nst_dec  % show()
        lo_sea = nst_dec % lower_bound_3D()
        hi_sea = nst_dec % upper_bound_3D()
        iiwest = nst_dec_vert % south()
        iieast = nst_dec_vert % north()
        jjsouth= nst_dec_horz % south()
        jjnorth= nst_dec_horz % north()
    endif
end subroutine

!======================================
!  get model grid for ocn and sea
!  create simple sea grid for future calculation
!  find sea position in ocean
!  create simple ocn grid for future calculation
!  create WTS matrix for all side
!  deallocate models grid
!======================================
subroutine get_mesh_and_make_wts(this, ev)
    use decomposition_module
    use datetime_module
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    real*8  :: x1_min,x2_min,x1_max,x2_max          ! Limits mesh
    integer :: i,j

    ! Wait unteel ocn and sea send all mesh array
    ! TODO wait kv and kt here too
    ! TODO Separate subs

    write(*,*),nst_rnk,"NST: get_mesh_and_make_wts wait ", TRIM (ev % ga_name()),ev % action()
    do while (TRIM(this % comm_ % get_info(x1_sea_name, COMM_GA_STATUS)) /= "full"); end do
    do while (TRIM(this % comm_ % get_info(x2_sea_name, COMM_GA_STATUS)) /= "full"); end do
    do while (TRIM(this % comm_ % get_info(x1_ocn_name, COMM_GA_STATUS)) /= "full"); end do
    do while (TRIM(this % comm_ % get_info(x2_ocn_name, COMM_GA_STATUS)) /= "full"); end do
    write(*,*),nst_rnk,"NST: get_mesh_and_make_wts allocate ", TRIM (ev % ga_name()),ev % action()

    allocate(x1_ocn(1:ocn_il,1:ocn_jl))
    allocate(x2_ocn(1:ocn_il,1:ocn_jl))
    allocate(x1_sea(1:ocn_il,1:ocn_jl))
    allocate(x2_sea(1:ocn_il,1:ocn_jl))
    allocate(x1_wts_sea(1:ocn_il,1:2))
    allocate(x2_wts_sea(1:ocn_jl,1:2))

    call this % comm_ % get( x1_ocn_name ,(/1,1/),(/ocn_il,ocn_jl/), x1_ocn )
    call this % comm_ % get( x2_ocn_name ,(/1,1/),(/ocn_il,ocn_jl/), x2_ocn )
    call this % comm_ % get( x1_sea_name ,(/1,1/),(/sea_il,sea_jl/), x1_sea )
    call this % comm_ % get( x2_sea_name ,(/1,1/),(/sea_il,sea_jl/), x2_sea )

    x1_min = x1_sea(sea_il,1);
    x2_min = x1_sea(sea_il,sea_jl);
    x1_max = x1_sea(sea_il-1,sea_jl-1);
    x2_max = x2_sea(sea_il-1,sea_jl-1);
    x1_wts_sea(1,1)=x1_min
    x2_wts_sea(1,1)=x2_min
    do i = 1,sea_il-1
        x1_wts_sea(i+1,1)=x1_sea(i,1)
        x1_wts_sea(i,2)=(x1_wts_sea(i,1)+x1_wts_sea(i+1,1))*0.5
    enddo
    do j = 1,sea_jl-1
        x2_wts_sea(j+1,1)=x2_sea(1,j)
        x2_wts_sea(j,2)=(x2_wts_sea(j,1)+x2_wts_sea(j+1,1))*0.5
    enddo

    !Find sea position in ocean (with west and south side)
    lo_ocn = reshape((/1,1,1,1,1,1/),(/3,2/))
    hi_ocn = reshape((/2,2,ocn_kl,2,2,ocn_kl/),(/3,2/))
    do i=1,ocn_il-1
        if ( x1_min > x1_ocn(i,1) ) lo_ocn(1,1) = i
        if ( x1_max > x1_ocn(i,1) ) hi_ocn(1,1) = i+1
    enddo
    do j=1,ocn_jl-1
        if ( x2_min > x2_ocn(1,j) ) lo_ocn(2,1) = j
        if ( x2_max > x2_ocn(1,j) ) hi_ocn(2,1) = j+1
    enddo


    lo_ocn(1,2)=lo_ocn(1,1)
    lo_ocn(2,2)=lo_ocn(2,1)
    hi_ocn(1,2)=hi_ocn(1,1)
    hi_ocn(2,2)=hi_ocn(2,1)

    if (x1_wts_sea(1,2)*2 < (x1_ocn(lo_ocn(1,1),1)+x1_ocn(lo_ocn(1,1)+1,1))) lo_ocn(1,2) = lo_ocn(1,1) - 1
    if (x2_wts_sea(1,2)*2 < (x2_ocn(1,lo_ocn(2,1))+x2_ocn(1,lo_ocn(2,1)+1))) lo_ocn(2,2) = lo_ocn(2,1) - 1
    if (x1_wts_sea(sea_il-1,2)*2 < (x1_ocn(hi_ocn(1,1),1)+x1_ocn(hi_ocn(1,1)-1,1))) hi_ocn(1,2) = hi_ocn(1,1) - 1
    if (x2_wts_sea(sea_jl-1,2)*2 < (x2_ocn(1,hi_ocn(2,1))+x2_ocn(1,hi_ocn(2,1)-1))) hi_ocn(2,2) = hi_ocn(2,1) - 1


    allocate(x1_wts_ocn(lo_ocn(1,2):hi_ocn(1,2)))
    allocate(x2_wts_ocn(lo_ocn(2,2):hi_ocn(2,2)))
    do i = lo_ocn(1,2), hi_ocn(1,2)
        x1_wts_ocn(i) = (x1_ocn(i,1)+x1_ocn(i+1,1))*0.5
    enddo

    do j = lo_ocn(2,2), hi_ocn(2,2)
        x2_wts_ocn(j) = (x2_ocn(1,j)+x2_ocn(1,j+1))*0.5
    enddo

    allocate(x1_sea_ind(jjsouth:jjnorth,1:2))
    allocate(x2_sea_ind(iiwest:iieast,1:2))

    i = lo_ocn(1,1)
    do j=1,jjsouth
        if (x1_ocn(i+1,lo_ocn(2,1)) < x1_wts_sea(j,1)) i=i+1
    enddo
    do j = jjsouth,jjnorth
        if (x1_ocn(i+1,lo_ocn(2,1)) < x1_wts_sea(j,1)) i=i+1
        x1_sea_ind(j,1) = i
    enddo

    j = lo_ocn(2,1)
    do i=1,iiwest
        if (x2_ocn(lo_ocn(1,1),j+1) < x2_wts_sea(i,1)) j=j+1
    enddo
    do i = iiwest,iieast
        if (x2_ocn(lo_ocn(1,1),j+1) < x2_wts_sea(i,1)) j=j+1
        x2_sea_ind(i,1) = j
    enddo

    i = lo_ocn(1,2)
    do j=1,jjsouth
        if (x1_wts_ocn(i+1) < x1_wts_sea(j,2)) i=i+1
    enddo
    do j = jjsouth,jjnorth
        if (x1_wts_ocn(i+1) < x1_wts_sea(j,2)) i=i+1
        x1_sea_ind(j,2) = i
    enddo

    j = lo_ocn(2,2)
    do i=1,iiwest
        if (x2_wts_ocn(j+1) < x2_wts_sea(i,2)) j=j+1
    enddo
    do i = iiwest,iieast
        if (x2_wts_ocn(j+1) < x2_wts_sea(i,2)) j=j+1
        x2_sea_ind(i,2) = j
    enddo

    if (CompSplitter % is_first_rank()) then
        write(*,'(a,4f7.2)'),"NST:get ocn limits ", x1_ocn(1,1),x2_ocn(1,1), x1_ocn(ocn_il-1,ocn_jl-1), x2_ocn(ocn_il-1,ocn_jl-1)
        write(*,'(a,4f7.2)'),"NST:get mesh limits ", x1_min, x2_min, x1_max, x2_max
        write(*,'(a,4f7.2)'),"NST:ocn [sea mesh] ",x1_ocn(lo_ocn(1,1),lo_ocn(2,1)),x2_ocn(lo_ocn(1,1),lo_ocn(2,1)), \
                                       x1_ocn(hi_ocn(1,1),hi_ocn(2,1)),x2_ocn(hi_ocn(1,1),hi_ocn(2,1))
        write(*,*),"NST:get mesh limits indexes lo", lo_ocn
        write(*,*),"NST:get mesh limits indexes hi", hi_ocn
    endif

    !allocate and calculate wts matrix
    allocate(wts_matrix_west(1:wts_vector_size,iiwest:iieast,1:sea_kl,1:2))
    call this % calc_wts_matrix(ev, "w")

    allocate(wts_matrix_east(1:wts_vector_size,iiwest:iieast,1:sea_kl,1:2))
    call this % calc_wts_matrix(ev, "e")

    allocate(wts_matrix_south(1:wts_vector_size,jjsouth:jjnorth,1:sea_kl,1:2))
    call this % calc_wts_matrix(ev, "s")

    allocate(wts_matrix_north(1:wts_vector_size,jjsouth:jjnorth,1:sea_kl,1:2))
    call this % calc_wts_matrix(ev, "n")

    wts_ready = .TRUE.
    deallocate(x1_ocn)
    deallocate(x2_ocn)
    deallocate(x1_sea)
    deallocate(x2_sea)
    deallocate(x1_wts_sea)
    deallocate(x2_wts_sea)
    deallocate(x1_wts_ocn)
    deallocate(x2_wts_ocn)
    _SEND_FREE ( x1_ocn_name )
    _SEND_FREE ( x2_ocn_name )
    _SEND_FREE ( x1_sea_name )
    _SEND_FREE ( x1_sea_name )
end subroutine

!======================================
!  prepare data to calculate vector* of WTS matrix
!  calculate all vectors*
!======================================
subroutine calc_wts_matrix(this, ev, side_mode)
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    character, intent(in)   :: side_mode
    real*8                  :: const_a(2), const_b(2)
    integer                 :: ind, k

    if (TRIM(inter_mode) .eq. TRIM("blinx2y2z1")) then
        if (side_mode .eq. 'w') then
            const_a(1)=(x1_wts_sea(1,1)-x1_ocn(lo_ocn(1,1),lo_ocn(2,1)))/(x1_ocn(lo_ocn(1,1)+1,lo_ocn(2,1))-x1_ocn(lo_ocn(1,1),lo_ocn(2,1)))
            const_a(2)=(x1_wts_sea(1,2)-x1_wts_ocn(lo_ocn(1,2)))/(x1_wts_ocn(lo_ocn(1,2)+1)-x1_wts_ocn(lo_ocn(1,2)))
            do ind=iiwest,iieast
                const_b(1)=(x2_wts_sea(ind,1)-x2_ocn(lo_ocn(1,1),x2_sea_ind(ind,1)))/(x2_ocn(lo_ocn(1,1),x2_sea_ind(ind,1)+1)-x2_ocn(lo_ocn(1,1),x2_sea_ind(ind,1)))
                const_b(2)=(x2_wts_sea(ind,2)-x2_wts_ocn(x2_sea_ind(ind,2)))/(x2_wts_ocn(x2_sea_ind(ind,2)+1)-x2_wts_ocn(x2_sea_ind(ind,2)))
                do k=1,sea_kl
                    call this % helper_for_calc_wts(ev, "w", const_info=(/const_a(1),const_b(1)/), pos00=wts_matrix_west(1,ind,k,1), pos01=wts_matrix_west(2,ind,k,1), pos10=wts_matrix_west(3,ind,k,1), pos11=wts_matrix_west(4,ind,k,1))
                    if (ind < sea_jl) call this % helper_for_calc_wts(ev, "w", const_info=(/const_a(2),const_b(2)/), pos00=wts_matrix_west(1,ind,k,2), pos01=wts_matrix_west(2,ind,k,2), pos10=wts_matrix_west(3,ind,k,2), pos11=wts_matrix_west(4,ind,k,2))
                enddo
                !write(*,*), nst_rnk, 'w', 1,ind,const_a, const_b
            enddo

        else if (side_mode .eq. 'e') then
            const_a(1)=(x1_wts_sea(sea_il,1)-x1_ocn(hi_ocn(1,1)-1,lo_ocn(2,1)))/(x1_ocn(hi_ocn(1,1),lo_ocn(2,1))-x1_ocn(hi_ocn(1,1)-1,lo_ocn(2,1)))
            const_a(2)=(x1_wts_sea(sea_il-1,2)-x1_wts_ocn(hi_ocn(1,2)-1))/(x1_wts_ocn(hi_ocn(1,2))-x1_wts_ocn(hi_ocn(1,2)-1))
            do ind=iiwest,iieast
                const_b(1)=(x2_wts_sea(ind,1)-x2_ocn(hi_ocn(1,1)-1,x2_sea_ind(ind,1)))/(x2_ocn(hi_ocn(1,1),x2_sea_ind(ind,1)+1)-x2_ocn(hi_ocn(1,1)-1,x2_sea_ind(ind,1)))
                const_b(2)=(x2_wts_sea(ind,2)-x2_wts_ocn(x2_sea_ind(ind,2)))/(x2_wts_ocn(x2_sea_ind(ind,2)+1)-x2_wts_ocn(x2_sea_ind(ind,2)))
                do k=1,sea_kl
                    call this % helper_for_calc_wts(ev, "e", const_info=(/const_a(1),const_b(1)/), pos00=wts_matrix_east(1,ind,k,1), pos01=wts_matrix_east(2,ind,k,1), pos10=wts_matrix_east(3,ind,k,1), pos11=wts_matrix_east(4,ind,k,1))
                    if (ind < sea_jl) call this % helper_for_calc_wts(ev, "e", const_info=(/const_a(2),const_b(2)/), pos00=wts_matrix_east(1,ind,k,2), pos01=wts_matrix_east(2,ind,k,2), pos10=wts_matrix_east(3,ind,k,2), pos11=wts_matrix_east(4,ind,k,2))
                enddo
                !write(*,*), nst_rnk, 'e', sea_il,ind,const_a, const_b
            enddo

        else if (side_mode .eq. 's') then
            const_b(1)=(x2_wts_sea(1,1)-x2_ocn(lo_ocn(1,1),lo_ocn(2,1)))/(x2_ocn(lo_ocn(1,1),lo_ocn(2,1)+1)-x2_ocn(lo_ocn(1,1),lo_ocn(2,1)))
            const_b(2)=(x2_wts_sea(1,2)-x2_wts_ocn(lo_ocn(2,2)))/(x2_wts_ocn(lo_ocn(2,2)+1)-x2_wts_ocn(lo_ocn(2,2)))
            do ind=jjsouth,jjnorth
                const_a(1)=(x1_wts_sea(ind,1)-x1_ocn(x1_sea_ind(ind,1),lo_ocn(2,1)))/(x1_ocn(x1_sea_ind(ind,1)+1,lo_ocn(2,1))-x1_ocn(x1_sea_ind(ind,1),lo_ocn(2,1)))
                const_a(2)=(x1_wts_sea(ind,2)-x1_wts_ocn(x1_sea_ind(ind,2)))/(x1_wts_ocn(x1_sea_ind(ind,2)+1)-x1_wts_ocn(x1_sea_ind(ind,2)))
                do k=1,sea_kl
                    call this % helper_for_calc_wts(ev, "s", const_info=(/const_a(1),const_b(1)/), pos00=wts_matrix_south(1,ind,k,1), pos01=wts_matrix_south(2,ind,k,1), pos10=wts_matrix_south(3,ind,k,1), pos11=wts_matrix_south(4,ind,k,1))
                    if (ind < sea_il) call this % helper_for_calc_wts(ev, "s", const_info=(/const_a(2),const_b(2)/), pos00=wts_matrix_south(1,ind,k,2), pos01=wts_matrix_south(2,ind,k,2), pos10=wts_matrix_south(3,ind,k,2), pos11=wts_matrix_south(4,ind,k,2))
                enddo
                !write(*,*), nst_rnk, 's', ind,1,const_a, const_b
            enddo

        else if (side_mode .eq. 'n') then
            const_b(1)=(x2_wts_sea(sea_jl,1)-x2_ocn(lo_ocn(1,1),hi_ocn(2,1)-1))/(x2_ocn(lo_ocn(1,1),hi_ocn(2,1))-x2_ocn(lo_ocn(1,1),hi_ocn(2,1)-1))
            const_b(2)=(x2_wts_sea(sea_jl,1)-x2_wts_ocn(hi_ocn(2,2)-1))/(x2_wts_ocn(hi_ocn(2,2))-x2_wts_ocn(hi_ocn(2,2)-1))
            do ind=jjsouth,jjnorth
                const_a(1)=(x1_wts_sea(ind,1)-x1_ocn(x1_sea_ind(ind,1),hi_ocn(2,1)))/(x1_ocn(x1_sea_ind(ind,1)+1,hi_ocn(2,1))-x1_ocn(x1_sea_ind(ind,1),hi_ocn(2,1)))
                const_a(2)=(x1_wts_sea(ind,2)-x1_wts_ocn(x1_sea_ind(ind,2)))/(x1_wts_ocn(x1_sea_ind(ind,2)+1)-x1_wts_ocn(x1_sea_ind(ind,2)))
                do k=1,sea_kl
                    call this % helper_for_calc_wts(ev, "n", const_info=(/const_a(1),const_b(1)/), pos00=wts_matrix_north(1,ind,k,1), pos01=wts_matrix_north(2,ind,k,1), pos10=wts_matrix_north(3,ind,k,1), pos11=wts_matrix_north(4,ind,k,1))
                    if (ind < sea_il) call this % helper_for_calc_wts(ev, "n", const_info=(/const_a(2),const_b(2)/), pos00=wts_matrix_north(1,ind,k,2), pos01=wts_matrix_north(2,ind,k,2), pos10=wts_matrix_north(3,ind,k,2), pos11=wts_matrix_north(4,ind,k,2))
                enddo
                !write(*,*), nst_rnk, 'n', ind,sea_jl,const_a, const_b
            enddo
        endif

      else
          write(*,*), "Error: Unknown method for iterpolation [WTS]"
      endif
end subroutine calc_wts_matrix

!======================================
!  calculate vector* of WTS matrix
!======================================
subroutine helper_for_calc_wts(this, ev, side_mode, const_info, pos00, pos01, pos10, pos11)
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    character, intent(in)   :: side_mode
    real*8, intent(in)      :: const_info(2)
    real(kind=main_kind), intent(inout) :: pos00, pos01, pos10, pos11

    if (TRIM(inter_mode) .eq. TRIM("blinx2y2z1")) then
        pos00=(1-const_info(1))*(1-const_info(2))
        pos01=const_info(1)*(1-const_info(2))
        pos10=(1-const_info(1))*const_info(2)
        pos11=const_info(1)*const_info(2)
    endif
end subroutine

!======================================
!   get data
!   send free
!   make boundary arrays
!======================================
subroutine handler_get_ocn_data(this, ev, q_id) !q2_id
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    integer, intent(in) ::q_id(2)

    !#ifdef DEBUG
    !write(*,*),nst_rnk,"NST: Handler GET OCN DATA",TRIM(ev % ga_name())
    !#endif

    if(.NOT. allocated(var_ocn_west)) allocate(var_ocn_west(lo_ocn(1,q_id(2)):lo_ocn(1,q_id(2))+1, lo_ocn(2,q_id(2)):hi_ocn(2,q_id(2)),lo_ocn(3,q_id(2)):hi_ocn(3,q_id(2)),1:1))
    if(.NOT. allocated(var_ocn_east)) allocate(var_ocn_east(hi_ocn(1,q_id(2))-1:hi_ocn(1,q_id(2)), lo_ocn(2,q_id(2)):hi_ocn(2,q_id(2)),lo_ocn(3,q_id(2)):hi_ocn(3,q_id(2)),1:1))
    if(.NOT. allocated(var_ocn_south))allocate(var_ocn_south(lo_ocn(1,q_id(2)):hi_ocn(1,q_id(2)),lo_ocn(2,q_id(2)):lo_ocn(2,q_id(2))+1,lo_ocn(3,q_id(2)):hi_ocn(3,q_id(2)),1:1))
    if(.NOT. allocated(var_ocn_north))allocate(var_ocn_north(lo_ocn(1,q_id(2)):hi_ocn(1,q_id(2)),hi_ocn(2,q_id(2))-1:hi_ocn(2,q_id(2)),lo_ocn(3,q_id(2)):hi_ocn(3,q_id(2)),1:1))

    call cpu_time(nst_time_strt)
    call this % comm_ % get(ev % ga_name(), (/lo_ocn(1,1),  lo_ocn(2,1),lo_ocn(3,1)/),(/lo_ocn(1,1)+1,hi_ocn(2,1),hi_ocn(3,1)/), var_ocn_west(:,:,:,q_id(1)))
    call this % comm_ % get(ev % ga_name(), (/hi_ocn(1,1)-1,lo_ocn(2,1),lo_ocn(3,1)/),(/hi_ocn(1,1),hi_ocn(2,1),  hi_ocn(3,1)/), var_ocn_east(:,:,:,q_id(1)))
    call this % comm_ % get(ev % ga_name(), (/lo_ocn(1,1),  lo_ocn(2,1),lo_ocn(3,1)/),(/hi_ocn(1,1),lo_ocn(2,1)+1,hi_ocn(3,1)/), var_ocn_south(:,:,:,q_id(1)))
    call this % comm_ % get(ev % ga_name(), (/lo_ocn(1,1),hi_ocn(2,1)-1,lo_ocn(3,1)/),(/hi_ocn(1,1),hi_ocn(2,1),  hi_ocn(3,1)/), var_ocn_north(:,:,:,q_id(1)))
    _SEND_FREE ( ev % ga_name() )

    write(*,'(i3,a)'),nst_size," NST get var ocn "
    call this % make_var_to_sea(ev, q_id)

    write(*,'(i3,a)'),nst_size," NST make_var sea "
    call cpu_time(nst_time_end)

    nst_time_total = nst_time_end - nst_time_strt
    if( nst_rnk == 0 ) write(*,'(i3,a,3f7.2)'),nst_size," NST time make_var_to_sea",nst_time_strt,nst_time_end, nst_time_total
end subroutine

!=======================
!   Make boundary arrays for sending to sea
!=======================
subroutine make_var_to_sea(this,ev,q_id)
    !use decomposition_module
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    integer, intent(in) ::q_id(2)

    if (.NOT. allocated (var_sea_west)) allocate(var_sea_west(1:1,             iiwest:iieast,1:sea_kl,1:1))
    if (.NOT. allocated (var_sea_east)) allocate(var_sea_east(sea_il:sea_il,   iiwest:iieast,1:sea_kl,1:1))
    if (.NOT. allocated (var_sea_south))allocate(var_sea_south(jjsouth:jjnorth,     1:1,     1:sea_kl,1:1))
    if (.NOT. allocated (var_sea_north))allocate(var_sea_north(jjsouth:jjnorth,sea_jl:sea_jl,1:sea_kl,1:1))

    call this % make_var_with_wts_sea(ev,"w",q_id)
    !write(*,*), var_sea_west
    call this % make_var_with_wts_sea(ev,"e",q_id)
    !write(*,*), var_sea_east
    call this % make_var_with_wts_sea(ev,"s",q_id)
    !write(*,*), var_sea_south
    call this % make_var_with_wts_sea(ev,"n",q_id)
    !write(*,*), var_sea_north

    call cpu_time(nst_time_end)
    nst_time_total = nst_time_total + nst_time_end - nst_time_strt
end subroutine

!======================================
!  loop calculation of data for sea
!======================================
subroutine make_var_with_wts_sea(this, ev, side_mode, q_id)
    class(service_nst)      :: this
    type(Event), intent(in) :: ev
    character, intent(in) :: side_mode
    integer, intent(in) ::q_id(2)
    integer   :: i, j, k

    if (TRIM(inter_mode) .eq. TRIM("blinx2y2z1")) then
        if (side_mode .eq. 'w') then
            do j=iiwest,iieast
                do k=1,sea_kl
                    var_sea_west(1,j,k,q_id(1)) = calc_point_wts("w",(/x2_sea_ind(j,1),k/),(/j,k/),q_id)
                end do
            enddo

        else if (side_mode .eq. 'e') then
            do j=iiwest,iieast
                do k=1,sea_kl
                    var_sea_east(sea_il,j,k,q_id(1)) = calc_point_wts("e",(/x2_sea_ind(j,1),k/),(/j,k/),q_id)
                end do
            enddo
        else if (side_mode .eq. 's') then
            do i=jjsouth,jjnorth
                do k=1,sea_kl
                    var_sea_south(i,1,k,q_id(1)) = calc_point_wts("s",(/x1_sea_ind(i,1),k/),(/i,k/),q_id)
                end do
            enddo
        else if (side_mode .eq. 'n') then
            do i=jjsouth,jjnorth
                do k=1,sea_kl
                    var_sea_north(i,sea_jl,k,q_id(1)) = calc_point_wts("n",(/x1_sea_ind(i,1),k/),(/i,k/),q_id)
                end do
            enddo
        endif
    else
        write(*,*), "Error: Unknown mode for ocn to sea interpolation with WTS"
    endif
end subroutine

!======================================
!  calculate value for sea point
!======================================
function calc_point_wts (side_mode, ocn_point, wts_point, q_id) result(var_sea_inter)
    character, intent(in) :: side_mode
    integer, intent(in)   :: ocn_point(2), wts_point(2), q_id(2)
    real(kind=main_kind)  :: var_sea_inter
    integer               :: calc_pos(3), i

    if (TRIM(inter_mode) .eq. TRIM("blinx2y2z1")) then
        var_sea_inter = 0
        if (side_mode .eq. 'w') then
            calc_pos=(/lo_ocn(1,1),ocn_point(1),ocn_point(2)/)
            do i = 1,wts_vector_size
              var_sea_inter=var_sea_inter+var_ocn_west(calc_pos(1)+start_point_for_sea_calc(i,1), \
                                                   calc_pos(2)+start_point_for_sea_calc(i,2), \
                                                   calc_pos(3)+start_point_for_sea_calc(i,3),q_id(1)) * \
                                      wts_matrix_west(i, wts_point(1), wts_point(2),q_id(2))
            enddo

        else if (side_mode .eq. 'e') then
            calc_pos=(/hi_ocn(1,1)-1,ocn_point(1),ocn_point(2)/)
            do i = 1,wts_vector_size
                var_sea_inter=var_sea_inter+var_ocn_east(calc_pos(1)+start_point_for_sea_calc(i,1), \
                                                   calc_pos(2)+start_point_for_sea_calc(i,2), \
                                                   calc_pos(3)+start_point_for_sea_calc(i,3),q_id(1)) * \
                                      wts_matrix_east(i, wts_point(1), wts_point(2),q_id(2))
            enddo

        else if (side_mode .eq. 's') then
            calc_pos=(/ocn_point(1),lo_ocn(2,1),ocn_point(2)/)
            do i = 1,wts_vector_size
                var_sea_inter=var_sea_inter+var_ocn_south(calc_pos(1)+start_point_for_sea_calc(i,1), \
                                                   calc_pos(2)+start_point_for_sea_calc(i,2), \
                                                   calc_pos(3)+start_point_for_sea_calc(i,3),q_id(1)) * \
                                      wts_matrix_south(i, wts_point(1), wts_point(2),q_id(2))
            enddo

        else if (side_mode .eq. 'n') then
            calc_pos=(/ocn_point(1),hi_ocn(2,1)-1,ocn_point(2)/)
            do i = 1,wts_vector_size
                var_sea_inter=var_sea_inter+var_ocn_north(calc_pos(1)+start_point_for_sea_calc(i,1), \
                                                   calc_pos(2)+start_point_for_sea_calc(i,2), \
                                                   calc_pos(3)+start_point_for_sea_calc(i,3),q_id(1)) * \
                                      wts_matrix_north(i, wts_point(1), wts_point(2),q_id(2))
            enddo
        else
            write(*,*), "Error: non-existed side"
        endif
    else
        write(*,*), "Error: unknow mode for interpolation from ocn to sea"
    endif
end function calc_point_wts

!============================================================================
!   Handler for send data to sea
!============================================================================
subroutine  handle_recv_nst ( this, ev, q_id)
    class(service_nst)      :: this
    type(Event), intent(in) :: ev
    integer, intent(in) ::q_id

    !#ifdef DEBUG
    !write(*,*),nst_rnk,"NST: Handle start RECV_NST ",TRIM(ev % ga_name())
    !#endif

    call cpu_time(nst_time_strt)
    call this % send_var_to_sea(ev,q_id)
    call cpu_time(nst_time_end)
    nst_time_total = nst_time_total + nst_time_end - nst_time_strt
    if( nst_rnk == 0 ) write(*,'(i3,a,3f7.2)'),nst_size," NST time send2sea",nst_time_strt,nst_time_end, nst_time_total
end subroutine

!======================================
!  Send data to sea border
!======================================
subroutine send_var_to_sea(this,ev,q_id)
    use decomposition_module
    class(service_nst) :: this
    type(Event), intent(in) :: ev
    integer, intent(in) ::q_id

    if (allocated (var_sea_west)) call this % comm_ % put ( ev % ga_name(),(/1,iiwest,1/),(/1,iieast,sea_kl/),var_sea_west(:,:,:,q_id))
    if (allocated (var_sea_east)) call this % comm_ % put ( ev % ga_name(),(/sea_il,iiwest,1/),(/sea_il,iieast,sea_kl/),var_sea_east(:,:,:,q_id))
    if (allocated (var_sea_south)) call this % comm_ % put ( ev % ga_name(),(/jjsouth,1,1/),(/jjnorth,1,sea_kl/), var_sea_south(:,:,:,q_id))
    if (allocated (var_sea_north)) call this % comm_ % put ( ev % ga_name(),(/jjsouth,sea_jl,1/),(/jjnorth,sea_jl,sea_kl/), var_sea_north(:,:,:,q_id))

    _SEND_FULL ( ev % ga_name() )
end subroutine


end module
