
!===============================================
! Main program and entry point of overall system
!===============================================

    program main

    use component_module
    use communicator_ga_module
    use event_module
    use event_scheduler_module
    use cpl_comp_clients_module
    use actions_module
    use statistics_module
    use service_module
    use service_dtr_module
    use service_cpl_module
    use service_iod_module
    use debug_module
#include "cpp_modules.inc"

    implicit none

    class(Component), pointer :: comp_p
    class(Service), pointer :: service_p
    type(Communicator) :: comm
    character(NAME_LEN) :: compile_time_name
    
    integer :: comp
    
!========================================================================================
! Section 1 : Shared initialization
!========================================================================================
        
    ! Read command line arguments and split MPI-communicator
    call CompSplitter % init(COMMAND_ARGUMENT_COUNT()/2)
!     if (CompSplitter % rank_world() == 0) call CompSplitter % show_split()
    call Debugger % init()
    call Stats % init()

    call Stats % init_time_mark("overall", "GLB")  
    call Stats % init_time_mark("shared_ini", "GLB")    
    
    ! Get information about experiment
    call ExpInfo % init(COMMAND_ARGUMENT_COUNT())
    if (CompSplitter % rank_world() == 0) call ExpInfo % show()
    
    ! Compile time choice of what components actually used #TODO beautify it
    select case (CompSplitter % i_am())
#include "cpp_comp_alloc.inc"
        case default
            if(search_linear(CompSplitter % i_am(), SERVICE_COMPS, 1, SIZE(SERVICE_COMPS)) == 0) &
                call CompSplitter % sync_error("No <"//CompSplitter % i_am()//"> component is currently compiled.", "LCL")
    end select

    ! Set model time defined on compile time
    call ModelsInfo(CompSplitter % i_am_id()) % set_model_name( compile_time_name)
    
    call Stats % print_time_dif("shared_ini", msg = "Shared initializations...")
    
!========================================================================================
! Section 2 : Personal initialization
!========================================================================================
    
    ! Register Comm groups using Comm client

    call Stats % init_time_mark("init", "GLB")
    
    call Stats % init_time_mark("ga_groups", "GLB")    
    call comm % init(CompSplitter % comp_defined(), CompSplitter % comm_local_size())    
    call Stats % print_time_dif("shared_ini", msg = "Comm init...")
    
    call comm_register_groups(comm)
    call Stats % print_time_dif("ga_groups", msg = "Registration of GA groups...")
        
    ! Register models
    call Stats % init_time_mark("comp_reg", "GLB")    
    if (CompSplitter % is_model()) then
        call comp_p % init(comm)
        call comp_p % ini_reg_comp()
    end if
    
    ! Synchronize models info array using ModelInfo client 
    call model_info_synchronize()
    call show_run_models()

    if (CompSplitter % is_model()) then
        call comp_p % ini_allocate()
        call comp_p % ini_reg_data()
    end if        
    
    call Stats % print_time_dif("comp_reg", msg = "Component registration...")
   

    call Stats % print_time_dif("init", "Initialization complete...")
   
    call Stats % init_time_mark("main_cycle", "GLB", msg = "Start modeling cycle...")
   
!========================================================================================
! Section 3 : Main event loops of components and services
!========================================================================================

!     if(run_comps <= 2) call this % sync_error("ComponentSplitter: at least one component should be run with CPL and IOD.", "GLB")

    ! Start process I/O requests from component
    if(.NOT. CompSplitter % is_launched("IOD")) &
        call CompSplitter % sync_error("ComponentSplitter: IOD component is not set.", "GLB")    

    if(.NOT. CompSplitter % is_launched("CPL")) call CompSplitter % sync_error("ComponentSplitter: CPL component is not set.", "GLB")    
      
!      if (CompSplitter % comm_local_size("DTR") /= 2 ) &
!          call CompSplitter % sync_error("ComponentSplitter: <DTR> service must be present exactly on 2 cores. THIS IS BUG AND IT WILL BE FIXED ", "GLB")
!        
      
    select case(CompSplitter % i_am())
        case("DTR")
            allocate(ServiceDTR :: service_p)
        case("CPL")
            allocate(ServiceCPL :: service_p)
        case("IOD")
            allocate(ServiceIOD :: service_p)
#include "cpp_service_alloc.inc"
    end select
    
    ! Start model cycle
    if (CompSplitter % is_model()) then
        call comp_p % model_cycle()
    else
            call service_p % init_base(comm)
            call service_p % init()
            
        ! TODO Ugly quick fix, because strange bug:
        ! System doesn't work with one of np = 1
        if (CompSplitter % i_am() /= "DTR" .OR. CompSplitter % rank_local() == 0) then
            call service_p % request_cycle()
        else
            call CompSplitter % barrier()
        end if
    end if
        
!========================================================================================
! Section 4 : Finalize
!========================================================================================

    call Stats % print_time_dif("main_cycle", msg = "Finish modeling cycle...")
    call Stats % print_time_dif("service_cpl_without_init", msg = "ServiceCPL without init...")
        
    call Stats % init_time_mark("finalize", "GLB")       
    
    ! Destroy models and services
    if (CompSplitter % is_model()) then
        call comp_p % finalize()
        call comp_p % destroy()
    else
        call service_p % destroy()
        call service_p % destroy_base()
    end if
    
    call Stats % print_time_dif("finalize", msg = "Finalizing procedures...")
    call Stats % print_time_dif("overall", msg = "Finishing program...")
    
    call Debugger % destroy()
    call Stats % destroy()
    
    call comm % destroy()
    
    call CompSplitter % destroy()
  
    stop
    
    end

